C;;;;;;;;;;;;;;;;;;;;;;;;;;;;; -*- Mode: Fortran -*- ;;;;;;;;;;;;;;;;;;;;;;;;;;;
C;; RGQCD4.f -- 
C;; Copyright (C) F. Jegerlehner 1998. Email: <fjeger@ifh.de>
C;; 
C;; Author          : Fred Jegerlehner
C;; Created On      : Fri May 15 10:47:36 1998
C;; Last Modified By: Friedrich Jegerlehner
C;; Last Modified On: Thu Dec 12 18:54:36 2019
C;; Update Count    : 85
C;; Status          : Unknown, Use with caution!
C;; 
C;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
C#	$Id: RGQCD4.f,v 1.1 1998/05/15 08:22:37 fjeger Exp fjeger $
*##################################################################*
C This version is based on the papers:
*-------------------------------------------------------------------
*         Vermaseren, Larin and Ritbergen   (hep-ph/9703284)       *
*         Rodrigo, Pich and Santamaria      (hep-ph/9707474)       *
*         Chetyrkin, Kniehl and Steinhauser (hep-ph/9708255)       *
*-------------------------------------------------------------------
*##################################################################*
C Subroutines: 
C
C           - RGQCD4(S,ALinp,Einp,ALout,MQout,NFout,FTQ,IOR)
C
C INPUT PARAMETERS: 
C
C S=E**2 (E=enegy scale for which the QCD coupling (ALout) 
C (Alpha QCD) and the current quark masses (MQout) are wanted; 
C ALinp is alpha_s determined at scale Einp in the MS-bar scheme 
C to order IOR in perturbation theory ("experimental input"); 
C FTQ=1: matching condition using MSbar-mass
C FTQ=2: matching condition using pole-mass
C Use the function rpm1 to calculate the pole mass: 
C       MPOL(iq,ior)=mmsb(iq,mu=mpol,ior)*rpm1(iq,alfas(mpol),ior)
C IOR=1,2,3,4 order in perturbation theory
C Input values of quark masses are taken from the PDG 1998
C They must be provided via the COMMON /QUAMA/MQ,MP,TH from the calling 
C program.
C
C OUTPUT PARAMETERS: 
C
C ALOUT(Alhpa_QCD), MQOUT(I) I=1,...,6(=u,d,s,c,b,t) 
C
C the running alpha_s and the quark masses at scale E=SQRT(S) in 
C MS-bar scheme to order IOR in perturbation theory for NFout 
C (number of flavors with thresholds below E). Matching of n_f light 
C flavor effective theory to n_f+1 "full QCD"                       

*-------------------------------------------------------------------
* RG convention; MSbar scheme
*-------------------------------------------------------------------
* da/d(ln mu^2)=beta(a)
*            =-beta_0 a^2-beta_1 a^3-beta_2 a^4-beta_3 a^5+O(a^6)
*-------------------------------------------------------------------
* d(ln m_q)/d(ln mu^2)=gamma_m(a)
*          =-gamma_0 a-gamma_1 a^2-gamma_2 a^3-gamma_3 a^4+O(a^5)
*-------------------------------------------------------------------
*-- mu renormalization point in MSbar
*-- g=g(mu^2) renormalized strong coupling constant
*-- adpmu=alpha_s/pi=g^2/4pi^2
*-- m_q=m_q(mu^2) renormalized running quark mass
*-------------------------------------------------------------------
      subroutine rgqcd4(s,alinp,einp,alout,mqout,nfout,ftq,ior)
      implicit none
      integer nfinp,nfout,nf,nl,ftq,ior,iq,i,j,k,nlight
      real *8 s,alinp,einp,alout,mu,alfas,f,beta0,msbarmass
      real *8 pi,zeta2,zeta3,zeta4,B4
      real *8 mq(6),mp(6),th(6),mqout(6)
      real *8 la(6),mh(6,6),Mhos(6)
      common /const/pi,zeta2,zeta3,zeta4,B4
C Input: Quark masses MQ and physical thresholds TH for switching NF
      common /quama/mq,mp,th
      common /param/la,mh,Mhos
      data pi /3.141592653589793d0/,zeta3 /1.20205690315959d0/
      zeta2 = pi**2/6d0
      zeta4 = pi**4/90d0
      B4    =-1.762800d0
C
C IOR=0 switches off QCD correction i.e. alphas=0.0
C
      mu=sqrt(s)
      if (mu.lt.1.d0) then 
        write (*,10) '   mu=',mu,' out of perturbative regime'
        stop
      endif
      if (einp.lt.1.d0) then 
        write (*,10) ' einp=',einp,' out of perturbative regime'
        stop
      endif
* count number of light flavors: mq(i) < 1 GeV
      nlight=3
      if (mq(4).le.1.d0) nlight=4
      if (mq(5).le.1.d0) nlight=5
      if (mq(6).le.1.d0) nlight=6
      if (ior.eq.0) then
C          do i=1,6
C             as(i)=0.d0
C          enddo
      else
C initialize RG coefficients up to 4th order for NF=3,4,5,6
C Fill common PARAM with values at different flavor thresholds
         call inicoe1()
         call getnf1(einp,nfinp)
*-------------------------------------------------------------------
* Provisional (lowest order) fill of RG integration constants 
*                common arrays la,mh,Mhos
*-------------------------------------------------------------------
        beta0=11.d0-2.d0/3.d0*nfinp 
        f=(4.d0*pi)/(beta0*alinp)
        do i=1,6
            la(i)=einp*exp(-0.5d0*f)
            do k=1,6 
              mh(i,k)=mq(i)
            enddo
            Mhos(i)=mq(i)
        enddo
*-------------------------------------------------------------------
*          Final fill of RG integration constants 
*                common arrays la,mh,Mhos
*-------------------------------------------------------------------
        call getlambda1(alinp,einp,nfinp,ftq,ior)
C        call getmasses(alinp,einp,nfinp,ftq,ior)
      endif
C
C### CALCULATION OF RUNNING PARAMETERS
C
      call getnf1(mu,nfout)
      alout=alfas(mu,nfout,ior)
      do iq=1,6 
        mqout(iq)=msbarmass(iq,mu,alout,nfout,ior)
      enddo
 10   format(1x,a6,e13.8,a60)
      return
      end

      subroutine inicoe1() 
*-------------------------------------------------------------------
* initialize RG coeffitients; here up to four loop
*-------------------------------------------------------------------
      implicit none
      integer ini,nf,nf2,nf3
      real *8 pi,zeta2,zeta3,zeta4,B4
      real *8 beta0,beta1,beta2,beta3,gamma0,gamma1,gamma2,gamma3
      real *8 be0(6),b1(6),b2(6),b3(6),c0(6),c1(6),c2(6),c3(6)
      common /const/pi,zeta2,zeta3,zeta4,B4
      common /rgcoe/be0,b1,b2,b3,c0,c1,c2,c3
      save
      data ini/0/
      if (ini.eq.0) goto 2
      return
 2    do 20 nf=1,6
      nf2=nf*nf
      nf3=nf2*nf
*-------------------------------------------------------------------
* RG coefficients eq. (6) & eq. (15) of VLR
*-------------------------------------------------------------------
C      beta0 =   2.75d0   - 0.166667d0 * nf
C      beta1 =   6.375d0  - 0.791667d0 * nf
C      beta2 =  22.3203d0 - 4.36892d0  * nf + 0.0940394d0 * nf2
      beta0 =  (11.d0-2.d0/3.d0*nf)/4.d0
      beta1 =  (102.d0-38.d0/3.d0*nf)/16.d0
      beta2 =  (2857.d0/2.d0-5033.d0/18.d0*nf+325.d0/54.d0*nf2)/64.d0
      beta3 = 114.23033d0-27.133944d0 * nf + 1.5823791d0 * nf2 
     &      + 0.0058566958d0 * nf3
      gamma0 = 1d0
      gamma1 = 4.20833d0 - 0.138889d0 * nf
      gamma2 = 19.5156d0 - 2.28412d0  * nf - 0.0270062d0 * nf2
      gamma3 = 98.9434d0 - 19.1075d0  * nf + 0.276163d0  * nf2
     &       + 0.00579322d0 * nf3
*-------------------------------------------------------------------
* auxiliary coefficients for analytic solution of RG equations 
*-------------------------------------------------------------------
      be0(nf)=beta0
      b1(nf)=beta1/beta0
      b2(nf)=beta2/beta0
      b3(nf)=beta3/beta0
      c0(nf)=gamma0/beta0
      c1(nf)=gamma1/beta0
      c2(nf)=gamma2/beta0
      c3(nf)=gamma3/beta0
C      write (7,*) ' RGQCD4: nf,beta0,beta1/beta0^2,beta2/beta0^3' 
C      write (7,*) nf,beta0,b1(nf)/beta0,b2(nf)/beta0**2
*-------------------------------------------------------------------
  20  continue
      ini=1
      return
      end

      subroutine getnf1(e,nf)
C find n_f effective for given energy scale e
      implicit none
      integer nf,i,j
      real *8 e
      real *8 mq(6),mp(6),th(6)
      common /quama/mq,mp,th
      if(e.lt.1.d0) stop ' energy scale out of perturbative range! '
      nf=6
      do i=1,3
        if(e.lt.th(nf)) nf=nf-1
      enddo
      return
      end

      subroutine getcoe1(nf)
      implicit none
      integer nf
      real *8 be0(6),b1(6),b2(6),b3(6),c0(6),c1(6),c2(6),c3(6)
      real *8 be0x,b1x,b2x,b3x,c0x,c1x,c2x,c3x
      common /rgcoe/be0,b1,b2,b3,c0,c1,c2,c3
      common /coeff/be0x,b1x,b2x,b3x,c0x,c1x,c2x,c3x
      be0x=be0(nf)
      b1x=b1(nf)
      b2x=b2(nf)
      b3x=b3(nf)
      c0x=c0(nf)
      c1x=c1(nf)
      c2x=c2(nf)
      c3x=c3(nf)
      return
      end

      subroutine getlambda1(alinp,einp,nfinp,ftq,ior)
*-------------------------------------------------------------------
* compute Lambda_QCD in MSbar scheme for given alinp(einp)
* result in array la(6) in common /param/la,mh,Mhos
*-------------------------------------------------------------------
      implicit none
      logical down
      integer i,ior,nfinp,nf,ftq
      real *8 alinp,einp,alsmu,adpmu,mu,mu2,la20,la2,la2old,alfas
      real *8 L,LL1,LL2,LL3,bLi,zetag20,zetag2OS0
      real *8 pi,f,lfun1
      real flag,test,be0
      real *8 be0x,b1x,b2x,b3x,c0x,c1x,c2x,c3x
      real *8 mq(6),mp(6),th(6)
      real *8 la(6),mh(6,6),Mhos(6)
      common /coeff/be0x,b1x,b2x,b3x,c0x,c1x,c2x,c3x
C Input: Quark masses MQ and physical thresholds TH for switching NF
      common /quama/mq,mp,th
      common /param/la,mh,Mhos
      data pi /3.141592653589793d0/
*
      flag=1e-5
      nf   =nfinp
      alsmu=alinp
      mu   =einp
      down=.TRUE.      
C f= pi/(beta0*alphas(mu))
 2      call getcoe1(nf)
        mu2=mu**2
        adpmu=alsmu/pi
C        f    =1d0/(be0*adpmu)
C        if (ior.le.3) then
CC Lambda=mu*lfun1(f,ior)
C          la2=mu2*lfun1(f,ior)**2
C        write (*,*) ior,nf,mu,sqrt(la2),alsmu
C        else if (ior.eq.4) then
        la20 =mu2*dexp(-1d0/(be0x*adpmu))
        la2  =la20          ! start value of Lambda_QCD ** 2
        la(nf)=sqrt(la2)
        do i=1,9
          L  =log(mu2/la2)
          if (l.lt.0.0d0) write(*,*) ' RGQCD4:nf,la,mu',nf,sqrt(la2),mu
          LL1=log(L)
          LL2=LL1*LL1
          LL3=LL2*LL1
          bLi=1d0/(be0x*L)
          la2old=la2
          if (ior.eq.2) then
            la2=mu2*dexp(-(1d0-b1x*LL1*bLi**1)/(be0x*adpmu))
          else if (ior.eq.3) then
            la2=mu2*dexp(-(1d0-b1x*LL1*bLi**1
     &          +(b1x**2*(LL2-LL1-1d0)+b2x)*bLi**2)/(be0x*adpmu))
          else if (ior.eq.4) then
            la2=mu2*dexp(-(1d0-b1x*LL1*bLi**1
     &          +(b1x**2*(LL2-LL1-1d0)+b2x)*bLi**2
     &          +(b1x**3*(-LL3+2.5d0*LL2+2d0*LL1-0.5d0)
     &          -3d0*b1x*b2x*LL1+0.5d0*b3x)*bLi**3)/(be0x*adpmu))
          endif
        enddo
      test=abs(sqrt(la2)-sqrt(la2old))
      if (test.gt.flag) write (*,*) 
     &  ' Insufficient convergence of Lambda_QCD: ',test,' > ',flag 
C        endif
      la(nf)=sqrt(la2)
C     write (*,*) ' la(nf)=',la(nf)
      if (down) then
        if (nf.gt.3) then
          mu   =th(nf)
          alsmu=alfas(mu,nf,ior)
C          write (*,*) ' *',ior,nf,mu,alsmu
          if (ftq.eq.0) then
          else if (ftq.eq.1) then
C           alsmu=alsmu*zetag20(2.d0*mh(nf,nf),nf,ior)
            alsmu=alsmu*zetag20(th(nf),nf,ior)
          else if (ftq.eq.2) then
            alsmu=alsmu*zetag2OS0(2.d0*Mhos(nf),nf,ior)
          else
            write (*,*) ' Matching convention not defined! (ftq?)'
          endif
          nf   =nf-1
          goto 2
        else
          down=.FALSE.
          nf   =nfinp
          alsmu=alinp
          mu   =einp
          goto 3
        endif      
      else
 3       continue
         if (nf.lt.6) then
            mu   =th(nf+1)
            alsmu=alfas(mu,nf,ior)
c     write (*,*) ' *',ior,nf,mu,alsmu
            if (ftq.eq.0) then
            else if (ftq.eq.1) then
C           alsmu=alsmu/zetag20(2.d0*mh(nf,nf),nf,ior)
               alsmu=alsmu/zetag20(th(nf),nf,ior)
            else if (ftq.eq.2) then
               alsmu=alsmu/zetag2OS0(2.d0*Mhos(nf),nf,ior)
            else
               write (*,*) ' Matching convention not defined! (ftq?)'
            endif
            nf   =nf+1
            goto 2
         else
            goto 9
         endif      
      endif
 9    continue
      return
      end

      subroutine getmasses(alinp,einp,nfinp,ftq,ior)
*-------------------------------------------------------------------
* compute mq(mu) in MSbar scheme for given alinp(einp) 
*        m_q (mu^2) q=s,c,b,t eq. (20) of VLR
* result in arrays mh(6 flavor,6 threshold) and 
*                Mhos(6 flavor) in common /param/la,mh,Mhos
* !!!!!! only do it for heavy quarks with masses > 1 GeV
*-------------------------------------------------------------------
      implicit none
      integer i,j,k,jj,ior,nfinp,nf,ftq,nf0,nf1
      real *8 alinp,einp,alsmu,adpmu,adp0,adp1,mu,mu2,la2,mqx,mos,alfas
      real *8 rpm1,rm,als,zetam0,zetamOS0
      real *8 pi
      real *8 mq(6),mp(6),th(6),rxx(6)
      real *8 la(6),mh(6,6),Mhos(6)
C Input: Quark masses MQ and physical thresholds TH for switching NF
      common /quama/mq,mp,th
      common /param/la,mh,Mhos
      data pi /3.141592653589793d0/
*
C 2      call getcoe1(nf)
C        alsmu =alfas(mu,nf,ior)
C        adpmu =alsmu/pi
C        ms = mshat * adpmu**(4/9)*(1d0 + 0.895062d0 * adpmu 
C     &      + 1.37143d0 * adpmu**2 + 1.95168d0 * adpmu**3)
C        mc = mchat * adpmu**(12/25)*(1d0 + 1.01413d0 * adpmu
C     &      + 1.38921d0 * adpmu**2 + 1.09054d0 * adpmu**3)
C        mb = mbhat * adpmu**(12/23)*(1d0 + 1.17549d0 * adpmu
C     &      + 1.50071d0 * adpmu**2 + 0.172478d0 * adpmu**3)
C        mt = mthat * adpmu**(4/7)*(1d0 + 1.39796d0 * adpmu
C     &      + 1.79348d0 * adpmu**2 - 0.683433d0 * adpmu**3)
C 9      continue
* Calculate pole masses for c,b and t (meaningless for u,d and s)
        Mhos(6)=mq(6)
        do j=1,2
          jj=6-j
          mqx=mq(jj)
          mos=mqx
          do k=1,4
*
* if mass < 1 GeV skip calculation of Mhos (set it Mhos=mq instead)
*
            if (mos.le.1.d0) goto 10 
            als =alfas(mos,jj,ior)
            mos =mqx*rpm1(jj,als,mos,ior)
* Get MSbar mass at mu=M-pole
            adp0=alfas(mqx,jj,ior)/pi
            adp1=alfas(mos,jj,ior)/pi
            mqx =mqx*rm(adp1,jj,ior)/rm(adp0,jj,ior)
 10         Mhos(jj)=mos
          enddo
        enddo
* Calculate MSbar masses at scale mu for u,d,s,c,b and t
* t: full QCD; MSbar top mass
        if (mq(6).gt.1.d0) then
          jj=6
          mos =Mhos(jj)
          mqx =mos/rpm1(jj,alfas(mos,jj,ior),mos,ior)
          adp0=alfas(mos,jj,ior)/pi
          do k=1,4 
            adp1=alfas(mqx,jj,ior)/pi
            mqx =mqx*rm(adp1,jj,ior)/rm(adp0,jj,ior)
            mh(jj,jj)=mqx
          enddo
* go to nf=5 effective theory
          if (ftq.eq.0) then
            rxx(jj)=1d0
          else if (ftq.eq.1) then
            rxx(jj)=zetam0(mqx,jj,ior)
          else if (ftq.eq.2) then
            rxx(jj)=zetamOS0(Mhos(jj),jj,ior)
          else
            write (*,*) ' Matching convention not defined! (ftq?)'
          endif
        else
          mh(jj,jj)  =mq(jj)
        endif
* Run b mass from m_b(m_b) to m_b(m_t):
        if (mq(5).gt.1.d0) then
          jj=5
          mh(jj,jj)=mq(jj)
          adp0=alfas(mqx,jj,ior)/pi
          adp1=alfas(mq(jj),jj,ior)/pi
          mqx =mq(jj)*rm(adp0,jj,ior)/rm(adp1,jj,ior)
* and apply matching factor to get m_b(m_t) in nf=6 theory
          mqx =mqx/rxx(jj+1)
          mh(jj,jj+1)=mqx
* go to nf=4 effective theory
          if (ftq.eq.0) then
            rxx(jj)=1d0
          else if (ftq.eq.1) then
            rxx(jj)=zetam0(mqx,jj,ior)
          else if (ftq.eq.2) then
            rxx(jj)=zetamOS0(Mhos(jj),jj,ior)
          else
            write (*,*) ' Matching convention not defined! (ftq?)'
          endif
        else
          mh(jj,jj)  =mq(jj)
          mh(jj,jj+1)=mq(jj)
        endif
* Run c mass from m_c(m_c) to m_c(m_b):
        if (mq(4).gt.1.d0) then
          jj=4
          mh(jj,jj)=mq(jj)
          adp0=alfas(mq(jj+1),jj,ior)/pi
          adp1=alfas(mq(jj),jj,ior)/pi
          mqx =mq(jj)*rm(adp0,jj,ior)/rm(adp1,jj,ior)
* and apply matching factor to get m_c(m_b) in nf=5 theory
          mqx =mqx/rxx(jj+1)
          mh(jj,jj+1)=mqx
* Run c mass from m_c(m_b) to m_c(m_t) in nf=5 effective theory:
          adp0=alfas(mh(jj+2,jj+2),jj+1,ior)/pi
          adp1=alfas(mqx,jj+1,ior)/pi
          mqx =mqx*rm(adp0,jj,ior)/rm(adp1,jj,ior)
* and apply matching factor to get m_c(m_t) in nf=6 theory
          mqx =mqx/rxx(jj+2)
          mh(jj,jj+2)=mqx
        else
          mh(jj,jj)  =mq(jj)
          mh(jj,jj+1)=mq(jj)
          mh(jj,jj+2)=mq(jj)
        endif
* the light quarks we treat as massless
      return
      end

      function alfas(mu,nf,ior)
*-------------------------------------------------------------------
* compute alpha_s(mu) in MSbar scheme in terms of known 
*                     Lambda_QCD(nf,ior)  
*-------------------------------------------------------------------
      implicit none
      integer nf,ior
      real *8 alfas,mu,adpmu,mu2,la2
      real *8 L,LL1,LL2,LL3,bLi
      real *8 pi
      real *8 be0x,b1x,b2x,b3x,c0x,c1x,c2x,c3x
      real *8 mq(6),mp(6),th(6)
      real *8 la(6),mh(6,6),Mhos(6)
      common /coeff/be0x,b1x,b2x,b3x,c0x,c1x,c2x,c3x
C Input: Quark masses MQ and physical thresholds TH for switching NF
      common /quama/mq,mp,th
      common /param/la,mh,Mhos
      data pi /3.141592653589793d0/
      call getcoe1(nf)
*
      mu2=mu**2
      la2=la(nf)**2
      L  =log(mu2/la2)
      if (l.lt.0.0d0) write(*,*) ' RGQCD4:nf,la,mu',nf,sqrt(la2),mu
      LL1=log(L)
      LL2=LL1*LL1
      LL3=LL2*LL1
      bLi=1d0/(be0x*L)
*-------------------------------------------------------------------
* alpha_s (mu2)/pi eq. (57) of CKS
*-------------------------------------------------------------------
      if (ior.eq.1) then
        adpmu=bLi
      else if (ior.eq.2) then
        adpmu=bLi-b1x*LL1*bLi**2
      else if (ior.eq.3) then
        adpmu=bLi-b1x*LL1*bLi**2+(b1x**2*(LL2-LL1-1d0)+b2x)*bLi**3
      else if (ior.eq.4) then
        adpmu=bLi-b1x*LL1*bLi**2+(b1x**2*(LL2-LL1-1d0)+b2x)*bLi**3
     &        +(b1x**3*(-LL3+2.5d0*LL2+2d0*LL1-0.5d0)
     &        -3d0*b1x*b2x*LL1+0.5d0*b3x)*bLi**4
      endif
      alfas=adpmu*pi
      return
      end

      function msbarmass(iq,mu,als,nf,ior)
*-------------------------------------------------------------------
* compute mass m_i(mu) in MSbar scheme in terms of known 
*                     mh(i,nf)  
*-------------------------------------------------------------------
      implicit none
      integer iq,nf,ior
      real *8 mu,als,alfas,mqx,adp0,adp1,rm,msbarmass
      real *8 pi
      real *8 mq(6),mp(6),th(6)
      real *8 la(6),mh(6,6),Mhos(6)
C Input: Quark masses MQ and physical thresholds TH for switching NF
      common /quama/mq,mp,th
      common /param/la,mh,Mhos
      data pi /3.141592653589793d0/
* Get MSbar mass for flavor i at scale mu
      mqx=2.d0*th(iq)  ! iq=1,2,3 mq at scale 2 GeV
      if (iq.gt.3) mqx =mh(iq,nf)
      adp0=alfas(mqx,nf,ior)/pi
      mqx =mqx*rm(als,nf,ior)/rm(adp0,nf,ior)
      msbarmass=mqx
      return
      end

      function rm(adp,nf,ior)
*-------------------------------------------------------------------
* calculate ratio of masses at two scales
* m_q (mu^2) / m_q (mu_0^2) = rm(x)/rm(x_0) eq. (59) of CKS
*-------------------------------------------------------------------
      implicit none
      integer nf,ior
      real *8 adp,rm,x
      real *8 be0x,b1x,b2x,b3x,c0x,c1x,c2x,c3x
      common /coeff/be0x,b1x,b2x,b3x,c0x,c1x,c2x,c3x
      call getcoe1(nf)

      x=adp  ! alpha/pi

      if (ior.eq.1) then
        rm= x**c0x
      else if (ior.eq.2) then
        rm= x**c0x*(1d0+(c1x-b1x*c0x)*x)
      else if (ior.eq.3) then
        rm= x**c0x*(1d0+(c1x-b1x*c0x)*x
     &    +0.5d0*((c1x-b1x*c0x)**2+c2x-b1x*c1x
     &            +b1x**2*c0x-b2x*c0x)*x**2)
      else if (ior.eq.4) then
        rm= x**c0x*(1d0+(c1x-b1x*c0x)*x
     &    +0.5d0*((c1x-b1x*c0x)**2+c2x-b1x*c1x
     &            +b1x**2*c0x-b2x*c0x)*x**2
     &    +((c1x-b1x*c0x)**3/6d0+(c1x-b1x*c0x)*
     &    *(c2x-b1x*c1x+b1x**2*c0x-b2x*c0x)*0.5d0
     &    +(c3x-b1x*c2x+b1x**2*c1x-b2x*c1x
     &     -b1x**3*c0x+2d0*b1x*b2x*c0x-b3x*c0x)/3d0
     &     )*x**3)
      endif
      return
      end

*-------------------------------------------------------------------
* The following renormalization factors relate between the full n_f  
* flavor QCD quantities to the effective n_l=n_f-1 massless flavors 
* QCD one's, which are used as the m_h-decoupled theory at scales 
*                         m_l < mu < m_h
*-------------------------------------------------------------------

      function zetag2(mu,mh,nf,ior)
*-------------------------------------------------------------------
* alpha_s^prime(mu) =zeta_g^2    alpha_s(mu)   eq. (23) of CKS
* (effective alpha) =zeta_g^2 (full theory alpha)  
*        mh MSbar mass in n_f flavour theory 
*-------------------------------------------------------------------
      implicit none
      integer ior,nf,nl
      real *8 mu,mh,alfas,alsmu,adpmu,logmu,logmu2,logmu3,zetag2
      real *8 e1,e2,e3,e3p
      real *8 pi,zeta2,zeta3,zeta4,B4
      common /const/pi,zeta2,zeta3,zeta4,B4
*
      alsmu = alfas(mu,nf,ior)    ! alpha_s at mu
      adpmu = alsmu/pi        
      logmu =log(mu**2/mh**2)
      logmu2=logmu*logmu
      logmu3=logmu*logmu2
      nl    =nf-1
      e1    =-1d0/6d0*logmu
      e2    =11d0/72d0-11d0/24d0*logmu+1d0/36d0*logmu2
      e3    =564731d0/124416d0-82043d0/27648d0*zeta3-955d0/576d0*logmu
     &       +53d0/576d0*logmu2-1d0/216d0*logmu3
      e3p   =-2633d0/31104d0+67d0/576d0*logmu-1d0/36d0*logmu2
      if (ior.eq.4) then
        zetag2   =1d0+e1*adpmu+e2*adpmu**2+(e3+e3p*nl)*adpmu**3
      else if (ior.eq.3) then
        zetag2   =1d0+e1*adpmu+e2*adpmu**2
      else if (ior.eq.2) then
        zetag2   =1d0+e1*adpmu
      else
        zetag2   =1d0
      endif
      return
      end

      function zetag20(muh,nf,ior)
*-------------------------------------------------------------------
* alpha_s^prime(mu) =zeta_g^2    alpha_s(mu)   eq. (23) of CKS
* (effective alpha) =zeta_g^2 (full theory alpha)  
*        mh MSbar mass in n_f flavour theory 
*  her at mu=muh where muh=m_h^(nf)(muh) referenz value      
*-------------------------------------------------------------------
      implicit none
      integer ior,nf,nl
      real *8 muh,alfas,alsmh,adpmh,zetag20
      real *8 pi,zeta2,zeta3,zeta4,B4
      common /const/pi,zeta2,zeta3,zeta4,B4
*
      alsmh = alfas(muh,nf,ior)      ! alpha_s at muh 
      adpmh = alsmh/pi
      nl    =nf-1
      if (ior.eq.4) then
        zetag20  =1d0+0.1528d0*adpmh**2+(0.9721d0-0.0847d0*nl)*adpmh**3
      else if (ior.eq.3) then
        zetag20  =1d0+0.1528d0*adpmh**2
      else
        zetag20  =1d0
      endif
      return
      end

      function zetag2OS(mu,Mhos,nf,ior)
*-------------------------------------------------------------------
* alpha_s^prime(mu) =zeta_g^2    alpha_s(mu)   eq. (24) of CKS
* (effective alpha) =zeta_g^2 (full theory alpha)  
*               Mhos pole mass 
*-------------------------------------------------------------------
      implicit none
      integer ior,nf,nl
      real *8 mu,Mhos,alfas,alsmu,adpmu,logmo,logmo2,logmo3,zetag2OS
      real *8 f1,f2,f3,f3p
      real *8 pi,zeta2,zeta3,zeta4,B4
      common /const/pi,zeta2,zeta3,zeta4,B4
*
      alsmu = alfas(mu,nf,ior)    ! alpha_s at Mhos = pole mass
      adpmu = alsmu/pi
      logmo =log(mu**2/Mhos**2)
      logmo2=logmo*logmo
      logmo3=logmo*logmo2
      nl    =nf-1
      f1    =-1d0/6d0*logmo
      f2    =-7d0/24d0-19d0/24d0*logmo+1d0/36d0*logmo2
      f3    =-58933d0/124416d0-2d0/3d0*zeta2*(1d0+1d0/3d0*log(2.d0))
     &       -80507d0/27648d0*zeta3-8521d0/1728d0*logmo
     &       -131d0/576d0*logmo2-1d0/216d0*logmo3
      f3p   =2479d0/31104d0+1d0/9d0*zeta2+409d0/1728d0*logmo
      if (ior.eq.4) then
        zetag2OS =1d0+f1*adpmu+f2*adpmu**2+(f3+f3p*nl)*adpmu**3
      else if (ior.eq.3) then
        zetag2OS =1d0+f1*adpmu+f2*adpmu**2
      else if (ior.eq.2) then
        zetag2OS =1d0+f1*adpmu
      else
        zetag2OS =1d0
      endif
      return
      end

      function zetag2OS0(Mhos,nf,ior)
*-------------------------------------------------------------------
* alpha_s^prime(mu) =zeta_g^2    alpha_s(mu)   eq. (24) of CKS
* (effective alpha) =zeta_g^2 (full theory alpha)  
*             mu = Mhos pole mass 
*-------------------------------------------------------------------
      implicit none
      integer ior,nf,nl
      real *8 Mhos,alfas,alsmo,adpmo,zetag2OS0
      real *8 pi,zeta2,zeta3,zeta4,B4
      common /const/pi,zeta2,zeta3,zeta4,B4
*
      alsmo = alfas(Mhos,nf,ior)    ! alpha_s at Mhos = pole mass
      adpmo = alsmo/pi
      nl    =nf-1      
      if (ior.eq.4) then
        zetag2OS0=1d0-0.2917d0*adpmo**2-(5.3239d0-0.2625d0*nl)*adpmo**3
      else if (ior.eq.3) then
        zetag2OS0=1d0-0.2917d0*adpmo**2
      else
        zetag2OS0=1d0
      endif
      return
      end

      function zetam(mu,mh,nf,ior)
*-------------------------------------------------------------------
*   m_q^prime(mu) =zeta_m       m_q(mu)       eq. (20) of CKS
* (effective mass)=zeta_m (full theory mass)  
*        mh MSbar mass in n_f flavour theory 
*-------------------------------------------------------------------
      implicit none
      integer ior,nf,nl
      real *8 mu,mh,alfas,alsmu,adpmu,logmu,logmu2,logmu3,zetam
      real *8 d2,d3,d3p
      real *8 pi,zeta2,zeta3,zeta4,B4
      common /const/pi,zeta2,zeta3,zeta4,B4
*
      alsmu = alfas(mu,nf,ior)    ! alpha_s at mu
      adpmu = alsmu/pi        
      logmu =log(mu**2/mh**2)
      logmu2=logmu*logmu
      logmu3=logmu*logmu2
      nl    =nf-1
      B4    =-1.762800d0
      d2    =89d0/432d0-5d0/36d0*logmu+1d0/12d0*logmu2
      d3    =2951d0/2916d0-407d0/864d0*zeta3+5d0/4d0*zeta4-1d0/36d0*B4
     &       -(311d0/2592d0+5d0/6d0*zeta3)*logmu+175d0/432d0*logmu2
     &       +29d0/216d0*logmu3
      d3p   =1327d0/11664d0-2d0/27d0*zeta3
     &       -53d0/432d0*logmu-1d0/108d0*logmu3
      if (ior.eq.4) then
        zetam    =1d0+d2*adpmu**2+(d3+d3p*nl)*adpmu**3
      else if (ior.eq.3) then
        zetam    =1d0+d2*adpmu**2
      else
        zetam    =1d0
      endif
      return
      end

      function zetam0(muh,nf,ior)
*-------------------------------------------------------------------
*   m_q^prime(mu) =zeta_m       m_q(mu)       eq. (20) of CKS
* (effective mass)=zeta_m (full theory mass)  
*        mh MSbar mass in n_f flavour theory 
*  her at mu=muh where muh=m_h^(nf)(muh) referenz value      
*-------------------------------------------------------------------
      implicit none
      integer ior,nf,nl
      real *8 muh,alfas,alsmh,adpmh,zetam0
      real *8 pi,zeta2,zeta3,zeta4,B4
      common /const/pi,zeta2,zeta3,zeta4,B4
*
      alsmh = alfas(muh,nf,ior)      ! alpha_s at muh 
      adpmh = alsmh/pi
      nl    =nf-1
      if (ior.eq.4) then
        zetam0   =1d0+0.2060d0*adpmh**2+(1.8476d0+0.0247d0*nl)*adpmh**3
      else if (ior.eq.3) then
        zetam0   =1d0+0.2060d0*adpmh**2
      else
        zetam0   =1d0
      endif
      return
      end


      function zetamOS(mu,Mhos,nf,ior)
*-------------------------------------------------------------------
*   m_q^prime(mu) =zeta_m       m_q(mu)       eq. (22) of CKS
* (effective mass)=zeta_m (full theory mass)  
*               Mhos pole mass 
*-------------------------------------------------------------------
      implicit none
      integer ior,nf,nl
      real *8 mu,Mhos,alfas,alsmu,adpmu,logmo,logmo2,logmo3,zetamOS
      real *8 h2,h3,h3p
      real *8 pi,zeta2,zeta3,zeta4,B4
      common /const/pi,zeta2,zeta3,zeta4,B4
*
      alsmu = alfas(mu,nf,ior)    ! alpha_s at Mhos = pole mass
      adpmu = alsmu/pi
      logmo =log(mu**2/Mhos**2)
      logmo2=logmo*logmo
      logmo3=logmo*logmo2
      nl    =nf-1
      h2    =89d0/432d0-5d0/36d0*logmo+1d0/12d0*logmo2
      h3    =1871d0/2916d0-407d0/864d0*zeta3+5d0/4d0*zeta4-1d0/36d0*B4
     &      +(121d0/2592d0-5d0/6d0*zeta3)*logmo+319d0/432d0*logmo2
     &      +29d0/216d0*logmo3
      h3p   =1327d0/11664d0-2d0/27d0*zeta3
     &      -53d0/432d0*logmo-1d0/108d0*logmo3
      if (ior.eq.4) then
        zetamOS  =1d0+h2*adpmu**2+(h3+h3p*nl)*adpmu**3
      else if (ior.eq.3) then
        zetamOS  =1d0+h2*adpmu**2
      else
        zetamOS  =1d0
      endif
      return
      end

      function zetamOS0(Mhos,nf,ior)
*-------------------------------------------------------------------
*   m_q^prime(mu) =zeta_m       m_q(mu)       eq. (22) of CKS
* (effective mass)=zeta_m (full theory mass)  
*             mu = Mhos pole mass 
*-------------------------------------------------------------------
      implicit none
      integer ior,nf,nl
      real *8 Mhos,alfas,alsmo,adpmo,zetamOS0
      real *8 pi,zeta2,zeta3,zeta4,B4
      common /const/pi,zeta2,zeta3,zeta4,B4
*
      alsmo = alfas(Mhos,nf,ior)    ! alpha_s at Mhos = pole mass
      adpmo = alsmo/pi
      nl    =nf-1      
      if (ior.eq.4) then
        zetamOS0 =1d0+0.2060d0*adpmo**2+(1.4773d0+0.0247d0*nl)*adpmo**3
      else if (ior.eq.3) then
        zetamOS0 =1d0+0.2060d0*adpmo**2
      else
        zetamOS0 =1d0
      endif
      return
      end

       function rpm1(nf1,als,mos,ior)
* Calculate pole mass to MSbar mass ratio m/m_q(mu=m)
* Gray et al. 1990; input nf,alpha_s(m) and m
* Note: this relationship is only known to two loop
       implicit none
       integer nf1,ior,i 
       real *8 pi,rpm1,als,mos,adp,c
       real *8 mq(6),mp(6),th(6)
       common /quama/mq,mp,th
       pi =3.141592653589793d0
       adp=als/pi
       rpm1=1.d0
       if (ior.eq.1) then  
         rpm1=1.d0+4.d0/3.d0*adp
       else if (ior.ge.2) then
         c=0.d0
         do i=1,nf1-1
           c=c+1.d0-mq(i)/mos
         enddo
         rpm1=1.d0+(4.d0/3.d0*adp+(16.11d0-1.04d0*c)*adp**2)
       else
       endif      
       return
       end

      DOUBLE PRECISION function lfun1(f,ior)
C Lambda_QCD from alphas at scale mu in MSbar scheme to three-loop
C calculate ln(Lambda**2/mu**2)=-(f(mu)+...); f=(4pi)/(beta0*alphas(mu))
C -> Lambda=mu*lfun1(f,ior)
      implicit none
      integer ior
      real *8 alphas,f,lnx,t1,t8,t13,t14
      real *8 vb2
      real *8 b1,b2,b3
      real *8 be0x,b1x,b2x,b3x,c0x,c1x,c2x,c3x
      common /coeff/be0x,b1x,b2x,b3x,c0x,c1x,c2x,c3x
      b1 =b1x/be0x
      b2 =b2x/be0x**2
C      write (*,*) ' b1,b2:',b1,b2
      if (ior.eq.3) then
         t1 = f**2
         t8 = b1**2
         vb2=4.d0*b2-t8
         t13 = sqrt(abs(vb2))
         t14 = 1.d0/t13
         if (vb2.gt.0.d0) then
            lnx = f-b1*log(t1+b1*f+b2)/2.d0+(t8-2.d0*b2)*t14
     #           *atan((2.d0*f+b1)*t14)
         else if (vb2.eq.0.d0) then
            lnx = f-b1*log(t1+b1*f+b2)/2.d0-(t8-2.d0*b2)/
     #             (2.d0*f+b1)
         else
            lnx = f-b1*log(t1+b1*f+b2)/2.d0-(2.d0*b2-t8)*t14
     #           *log((2.d0*f+b1-t13)/(2.d0*f+b1+t13))/2.d0
         endif
         lfun1=exp(-0.5d0*lnx)
      else if (ior.eq.2) then
         lfun1=EXP(-0.5d0*F*(1.D0-B1/F*LOG(F+B1)))
      else
         lfun1=EXP(-0.5d0*F)
      endif
      return
      end

      subroutine gcn(x,y,f)
*
* external for NAG D02BDF integration routine
* input x, f(x), output f=dy/dx(x)
*
      implicit none
      integer n
      parameter(n=1)
      real *8 x,y(n),f(n)
      integer ior,nf,nf2,nf3
      real *8 a,eex
      real *8 beta0,beta1,beta2,beta3
      common /case/ior

*   implemented for Euclidean region only 
* (i.e. space like momentum transfer assumed)
      eex =abs(x)
* MSbar RG
      beta0=0.d0
      beta1=0.d0
      beta2=0.d0
      beta3=0.d0
      call getnf1(sqrt(eex),nf)
      nf2=nf*nf
      nf3=nf2*nf
      if (ior.ge.1) then
        beta0 =  (11.d0-2.d0/3.d0*nf)/4.d0
      else
      endif
      if (ior.ge.2) then
        beta1 =  (102.d0-38.d0/3.d0*nf)/16.d0
      else
      endif
      if (ior.ge.3) then
        beta2 =  (2857.d0/2.d0-5033.d0/18.d0*nf+325.d0/54.d0*nf2)/64.d0
      else
      endif
      if (ior.ge.4) then
        beta3 = 114.23033d0-27.133944d0 * nf + 1.5823791d0 * nf2 
     &      + 0.0058566958d0 * nf3
      else
      endif
* MSbar RG 
      a=y(1)
      f(1)=-(beta0+beta1*a+beta2*a**2+beta3*a**3)*a**2/x
C      write (7,*) ior,beta0,beta1,beta2,beta3
      return
      end

