
c;;;;;;;;;;;;;;;;;;;;;;;;;;;;; -*- Mode: Fortran -*- ;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; hadr5n15.f --- 
c;; Author          : Friedrich Jegerlehner
c;; Created On      : Sun Dec 14 13:39:30 2014
c;; Last Modified By: Friedrich Jegerlehner
c;; Last Modified On: Wed Dec 11 11:21:50 2019
c;; RCS: $Id$
c;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; Copyright (C) 2016 Friedrich Jegerlehner
c;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; 
       subroutine hadr5n(e,st2,der,errder,deg,errdeg)
c Provides real part of 5 flavor hadronic contribution to photon vacuum polarization
c single precision HADR5 ; double precision DHADR5
c ******************************************************************
c *                                                                *
c *      subroutine for the evaluation of the light hadron         *
c *           contributions to Delta_r  and  Delta_g               *
c *                    using fits to the                           *
c *          QED vacuum polarization from e^+ e^- data             *
c *                                                                *
c *    F. Jegerlehner, Institut fr Physik                         *
c *                    Humboldt-Universitt zu Berlin              *
c *                    Newtonstrae 15, D-12489 Berlin, Germany    *
c *    E-mail: fjeger@physik.hu-berlin.de                          *
c *    Phone :   +49-3375 500 277                                  *
c *                                                                *
c *    Reference: F. Jegerlehner, Z. Phys. C32 (1986) 195          *
c *               H. Burkhardt et al., Z. Phys. C42 (1989) 497     *
c *               S. Eidelman, F. Jegerlehner, Z. Phys. C (1995)   *
c *               F. Jegerlehner,                                  *
c *                    Nucl.Phys.Proc.Suppl.131:213-222,2004       *
c *                    Nucl.Phys.Proc.Suppl.162:22-32,2006         *
c *                    Nucl.Phys.Proc.Suppl.181-182:135-140,2008   *
c *                                                                *
c ******************************************************************
c       VERSION: 20/12/2016
c
C  Notation: E energy ( momentum transfer ): E>0 timelike , E<0 spacelike
c      sin2ell=0.23153 ! pm 0.00016 LEPEEWG Phys Rep 427 (2006) 257
C            st2 is sin^2(Theta); st2=0.23153 is the reference value
C  the routine returns the hadronic contribution of 5 flavors (u,d,s,c,b)
C                 to   DER=Delta_r with hadronic error ERRDER
C                and   DEG=Delta_g with hadronic error ERRDEG
C  The effective value of the fine structure constant alphaQED at energy
C  E is alphaQED(E)=alphaQED(0)/(1-Delta_r) ,similarly for the SU(2)
C  coupling alphaSU2(E)=alphaSU2(0)/(1-Delta_g), where Delta_r(g) is the
C  sum of leptonic, hadronic contributions (top to be added).
C
      IMPLICIT NONE
      INTEGER NA,NB,NC,IJ,I,ini
      PARAMETER(NA=979,NB=2174,NC=200)
      real e,st2,der,errder,deg,errdeg,fac
      real dal(2),dg2(2)
      REAL ETA(NA),DAT(NA,2),DGT(NA,2)
      REAL ESA(NB),DAS(NB,2),DGS(NB,2)
      REAL EMA(NC),DAM(NC,2),DGM(NC,2)
      REAL XXX(NC),YGG(NC,2),Y3G(NC,2)
      COMMON /DATM16/EMA,DAM,DGM
      COMMON /DATT16/ETA,DAT,DGT
      COMMON /DATS16/ESA,DAS,DGS
      COMMON /DATH16/XXX,YGG,Y3G
      data ini/0/
C initialize data

      call dalhad_spacelike16

      call dalhad_timelike16

      call dalhad_timelike1h16

c boundaries of data sets
c space-like and unphysical region  -1.0000E+03 - 0.27599999D0 (ETA,DAT)
c time-like R data range 0.28207018D0 - 38.29000000D0          (ESA,DAS)
c time-like1 pQCD range 1.3005E+01 - 2.0000E+04                (EMA,DAM)
c 2 pion threshold at 0.27914....
      fac=0.23153d0/st2
      if (e.eq.0.0) then
        der=0.0
        errder=0.0
        deg=0.0
        errdeg=0.0
        return
c first bridge cracks between regions
      else if ((e.ge.XXX(NC)).and.(e.lt.ETA(1))) then
        DO I=1,2
        dal(i)=YGG(NC,I)
     &    +(DAT(1,I)-YGG(NC,I))/(ETA(1)-XXX(NC))*(E-XXX(NC))
        dg2(i)=Y3G(NC,I)
     &    +(DAT(1,I)-Y3G(NC,I))/(ETA(1)-XXX(NC))*(E-XXX(NC))
        ENDDO
        der=dal(1)
        errder=dal(2)
        deg=dg2(1)*fac
        errdeg=dg2(2)*fac
        return
      else if ((e.ge.ETA(NA)).and.(e.lt.ESA(1))) then
        DO I=1,2
        dal(i)=DAT(NA,I)
     &    +(DAS(1,I)-DAT(NA,I))/(ESA(1)-ETA(NA))*(E-ETA(NA))
        dg2(i)=DGT(NA,I)
     &    +(DGS(1,I)-DGT(NA,I))/(ESA(1)-ETA(NA))*(E-ETA(NA))
        ENDDO
        der=dal(1)
        errder=dal(2)
        deg=dg2(1)*fac
        errdeg=dg2(2)*fac
        return
      else if ((e.lt.ETA(NA)).and.(e.ge.ETA(1))) then
        ij=NA
        do while (ETA(ij).gt.e)
          ij=ij-1
        enddo
        DO I=1,2
        dal(i)=DAT(IJ,I)
     &    +(DAT(IJ+1,I)-DAT(IJ,I))/(ETA(IJ+1)-ETA(IJ))*(E-ETA(IJ))
        dg2(i)=DGT(IJ,I)
     &    +(DGT(IJ+1,I)-DGT(IJ,I))/(ETA(IJ+1)-ETA(IJ))*(E-ETA(IJ))
        ENDDO
        der=dal(1)
        errder=dal(2)
        deg=dg2(1)*fac
        errdeg=dg2(2)*fac
        return
      else if ((e.lt.EMA(NC)).and.(e.ge.EMA(1))) then
        ij=NC
        do while (EMA(ij).gt.e)
          ij=ij-1
        enddo
        DO I=1,2
        dal(i)=DAM(IJ,I)
     &    +(DAM(IJ+1,I)-DAM(IJ,I))/(EMA(IJ+1)-EMA(IJ))*(E-EMA(IJ))
        dg2(i)=DGM(IJ,I)
     &    +(DGM(IJ+1,I)-DGM(IJ,I))/(EMA(IJ+1)-EMA(IJ))*(E-EMA(IJ))
        ENDDO
        der=dal(1)
        errder=dal(2)
        deg=dg2(1)*fac
        errdeg=dg2(2)*fac
        return
      else if ((e.lt.ESA(NB)).and.(e.ge.ESA(1))) then
        if ((e.gt.3.96).and.(e.lt.4.52).and.(ini.eq.0)) then
c           write (*,*) ' ******************************************'
c           write (*,*) ' * Note: fluctuations in the Psi(4,5,6)   *'
c           write (*,*) ' * region are because we are using the    *'
c           write (*,*) ' * CB and BES data without any smoothing  *'
c           write (*,*) ' *********** FJ@HU Berlin *****************'
           ini=1
        endif
        ij=NB
        do while (ESA(ij).gt.e)
          ij=ij-1
        enddo
        DO I=1,2
        dal(i)=DAS(IJ,I)
     &    +(DAS(IJ+1,I)-DAS(IJ,I))/(ESA(IJ+1)-ESA(IJ))*(E-ESA(IJ))
        dg2(i)=DGS(IJ,I)
     &    +(DGS(IJ+1,I)-DGS(IJ,I))/(ESA(IJ+1)-ESA(IJ))*(E-ESA(IJ))
        ENDDO
        der=dal(1)
        errder=dal(2)
        deg=dg2(1)*fac
        errdeg=dg2(2)*fac
        return
      else if ((e.lt.XXX(NC)).and.(e.ge.XXX(1))) then
        ij=NC
        do while (XXX(ij).gt.e)
          ij=ij-1
        enddo
        DO I=1,2
        dal(i)=YGG(IJ,I)
     &    +(YGG(IJ+1,I)-YGG(IJ,I))/(XXX(IJ+1)-XXX(IJ))*(E-XXX(IJ))
        dg2(i)=Y3G(IJ,I)
     &    +(Y3G(IJ+1,I)-Y3G(IJ,I))/(XXX(IJ+1)-XXX(IJ))*(E-XXX(IJ))
        ENDDO
        der=dal(1)
        errder=dal(2)
        deg=dg2(1)*fac
        errdeg=dg2(2)*fac
        return
      else 
         if (ini.le.10) then
            write(*,*) ' out of range! e=',e
            ini=ini+1
            if (ini.eq.10) write(*,*) ' Warning: continuing anyway ...'
         endif
      endif
      return
      end

       subroutine dhadr5n(de,dst2,dder,derrder,ddeg,derrdeg)
c **********************************************************************
c *    F. Jegerlehner, University of Silesia, PL-40007 Katowice, Poland *
c **********************************************************************
c Converts hadr5n15 to double precision variables in dhadr5n15
c
       implicit none
       real *8 de,dst2,dder,ddeg,derrder,derrdeg
       real    se,sst2,sder,sdeg,serrder,serrdeg
       se  =sngl(de)
       sst2=sngl(dst2)
       call hadr5n(se,sst2,sder,serrder,sdeg,serrdeg)
       dder   =dble(sder)
       ddeg   =dble(sdeg)
       derrder=dble(serrder)
       derrdeg=dble(serrdeg)
       return
       end


c      ++++ dalhadslow15 ++++  

c update may 2016 
       subroutine dalhad_timelike16
       IMPLICIT NONE
       INTEGER NB,I,J
       PARAMETER(NB=2174)
       REAL ESA(NB),DAS(NB,2),DGS(NB,2)
       COMMON /DATS16/ESA,DAS,DGS
c 
c******************************************************************************
c*                                                                            *
c*    --- intRdat1 ---                              Sun May 15 22:49:43 2016  *
c*                                                                            *
c***************************# FJ@HU Berlin #***********************************
c 
c  "Delta alpha"               Update version (29/09/2015) 
c 
c ISCAN:  1, Ismooth: 0, IGS: 0, DSET:     , iplot: 0
c IKE,Emin,Emax,NP,N,EPSREL:  1, 6.0000E-01, 1.4000E+00, 100, 10, 1.00E-02, 
c ALS(sta)(sys):  0.1184( 0.0007)( 0.0000), MZINP:   91.1876, MTOP:    173.20
c NF,IOR,ICHK: 5,4,2,ISU2: 0,NFL: 5,INAG: 1,inotail:  0, sin2W:   0.23153
c CHPTCUT,EC,ECUT: 0.3180,     5.20,    11.50
c 
  
       DATA  (ESA  (I), I=   1, 100) /
     &   0.28207018D0, 0.28800000D0, 0.29400000D0, 0.30000000D0,
     &   0.30600000D0, 0.30900000D0, 0.31500000D0, 0.32600000D0,
     &   0.33100000D0, 0.33600000D0, 0.34100000D0, 0.34600000D0,
     &   0.35200000D0, 0.35600000D0, 0.36100000D0, 0.36600000D0,
     &   0.37100000D0, 0.37600000D0, 0.38000000D0, 0.38500000D0,
     &   0.38900000D0, 0.39400000D0, 0.39800000D0, 0.40300000D0,
     &   0.40700000D0, 0.41100000D0, 0.41600000D0, 0.41871100D0,
     &   0.42000000D0, 0.42400000D0, 0.42800000D0, 0.43200000D0,
     &   0.43600000D0, 0.44000000D0, 0.44400000D0, 0.44800000D0,
     &   0.45200000D0, 0.45600000D0, 0.46000000D0, 0.46400000D0,
     &   0.46700000D0, 0.47100000D0, 0.47500000D0, 0.47800000D0,
     &   0.48200000D0, 0.48600000D0, 0.48900000D0, 0.49300000D0,
     &   0.49600000D0, 0.50000000D0, 0.52000000D0, 0.54000000D0,
     &   0.56600000D0, 0.59200000D0, 0.60000000D0, 0.60200000D0,
     &   0.60800000D0, 0.61600000D0, 0.62400000D0, 0.63200000D0,
     &   0.64000000D0, 0.64300000D0, 0.64800000D0, 0.65600000D0,
     &   0.66300000D0, 0.67100000D0, 0.67800000D0, 0.68600000D0,
     &   0.68800000D0, 0.69300000D0, 0.69600000D0, 0.70000000D0,
     &   0.70700000D0, 0.71400000D0, 0.72100000D0, 0.72800000D0,
     &   0.73500000D0, 0.74200000D0, 0.74800000D0, 0.75000000D0,
     &   0.75500000D0, 0.75744000D0, 0.76007300D0, 0.76207300D0,
     &   0.76507300D0, 0.76851700D0, 0.77060500D0, 0.77205400D0,
     &   0.77314300D0, 0.77400300D0, 0.77470900D0, 0.77530500D0,
     &   0.77582000D0, 0.77627100D0, 0.77667300D0, 0.77726600D0,
     &   0.77746600D0, 0.77766900D0, 0.77795100D0, 0.77821500D0/
       DATA  (ESA  (I), I= 101, 200) /
     &   0.77846300D0, 0.77869800D0, 0.77892200D0, 0.77913700D0,
     &   0.77934400D0, 0.77954500D0, 0.77974200D0, 0.77993500D0,
     &   0.78012700D0, 0.78031800D0, 0.78051000D0, 0.78070600D0,
     &   0.78090800D0, 0.78112100D0, 0.78135100D0, 0.78161200D0,
     &   0.78193600D0, 0.78268000D0, 0.78341500D0, 0.78373400D0,
     &   0.78399100D0, 0.78421700D0, 0.78442600D0, 0.78462500D0,
     &   0.78481700D0, 0.78500500D0, 0.78519100D0, 0.78537800D0,
     &   0.78556600D0, 0.78575600D0, 0.78595100D0, 0.78615100D0,
     &   0.78635800D0, 0.78657300D0, 0.78679800D0, 0.78703400D0,
     &   0.78728400D0, 0.78755000D0, 0.78773500D0, 0.78793500D0,
     &   0.78814300D0, 0.78847900D0, 0.78884800D0, 0.78925800D0,
     &   0.78971900D0, 0.79024600D0, 0.79085900D0, 0.79158800D0,
     &   0.79248000D0, 0.79361500D0, 0.79513900D0, 0.79736300D0,
     &   0.79800000D0, 0.80000000D0, 0.80200000D0, 0.80400000D0,
     &   0.80600000D0, 0.80800000D0, 0.81200000D0, 0.81400000D0,
     &   0.81600000D0, 0.82000000D0, 0.82200000D0, 0.82400000D0,
     &   0.82600000D0, 0.82800000D0, 0.83000000D0, 0.83100000D0,
     &   0.83700000D0, 0.84300000D0, 0.84900000D0, 0.85400000D0,
     &   0.86000000D0, 0.86600000D0, 0.87200000D0, 0.87700000D0,
     &   0.88300000D0, 0.88900000D0, 0.89400000D0, 0.89700000D0,
     &   0.90250000D0, 0.90500000D0, 0.91100000D0, 0.91600000D0,
     &   0.92200000D0, 0.92700000D0, 0.93300000D0, 0.93800000D0,
     &   0.94300000D0, 0.94900000D0, 0.95200000D0, 0.95400000D0,
     &   0.95800000D0, 0.96000000D0, 0.96200000D0, 0.96400000D0,
     &   0.96600000D0, 0.96800000D0, 0.97000000D0, 0.97400000D0/
       DATA  (ESA  (I), I= 201, 300) /
     &   0.97800000D0, 0.98000000D0, 0.98070000D0, 0.98140000D0,
     &   0.98210000D0, 0.98280000D0, 0.98350000D0, 0.98420000D0,
     &   0.98490000D0, 0.98560000D0, 0.98630000D0, 0.98700000D0,
     &   0.98770000D0, 0.98840000D0, 0.98910000D0, 0.98980000D0,
     &   0.99050000D0, 0.99120000D0, 0.99190000D0, 0.99260000D0,
     &   0.99330000D0, 0.99400000D0, 0.99470000D0, 0.99540000D0,
     &   0.99610000D0, 0.99680000D0, 0.99750000D0, 0.99820000D0,
     &   0.99890000D0, 0.99960000D0, 0.99990000D0, 1.00001000D0,
     &   1.00030000D0, 1.00100000D0, 1.00170000D0, 1.00240000D0,
     &   1.00310000D0, 1.00380000D0, 1.00450000D0, 1.00520000D0,
     &   1.00590000D0, 1.00660000D0, 1.00730000D0, 1.00800000D0,
     &   1.00870000D0, 1.00940000D0, 1.01010000D0, 1.01080000D0,
     &   1.01150000D0, 1.01220000D0, 1.01290000D0, 1.01360000D0,
     &   1.01430000D0, 1.01500000D0, 1.01520000D0, 1.01540000D0,
     &   1.01560000D0, 1.01580000D0, 1.01600000D0, 1.01620000D0,
     &   1.01640000D0, 1.01660000D0, 1.01680000D0, 1.01700000D0,
     &   1.01720000D0, 1.01740000D0, 1.01770000D0, 1.01790000D0,
     &   1.01800000D0, 1.01820000D0, 1.01840000D0, 1.01860000D0,
     &   1.01880000D0, 1.01900000D0, 1.01920000D0, 1.01940000D0,
     &   1.01960000D0, 1.01980000D0, 1.02000000D0, 1.02020000D0,
     &   1.02040000D0, 1.02060000D0, 1.02080000D0, 1.02100000D0,
     &   1.02120000D0, 1.02140000D0, 1.02160000D0, 1.02180000D0,
     &   1.02200000D0, 1.02220000D0, 1.02240000D0, 1.02270000D0,
     &   1.02290000D0, 1.02300000D0, 1.02320000D0, 1.02340000D0,
     &   1.02360000D0, 1.02380000D0, 1.02400000D0, 1.02420000D0/
       DATA  (ESA  (I), I= 301, 400) /
     &   1.02440000D0, 1.02460000D0, 1.02480000D0, 1.02500000D0,
     &   1.02570000D0, 1.02640000D0, 1.02710000D0, 1.02780000D0,
     &   1.02850000D0, 1.02920000D0, 1.02990000D0, 1.03060000D0,
     &   1.03130000D0, 1.03200000D0, 1.03270000D0, 1.03340000D0,
     &   1.03410000D0, 1.03480000D0, 1.03550000D0, 1.03620000D0,
     &   1.03690000D0, 1.03760000D0, 1.03830000D0, 1.03900000D0,
     &   1.03970000D0, 1.03999000D0, 1.04001000D0, 1.04040000D0,
     &   1.04110000D0, 1.04180000D0, 1.04250000D0, 1.04320000D0,
     &   1.04390000D0, 1.04460000D0, 1.04530000D0, 1.04600000D0,
     &   1.04670000D0, 1.04740000D0, 1.04810000D0, 1.04880000D0,
     &   1.04950000D0, 1.05020000D0, 1.05090000D0, 1.05160000D0,
     &   1.05230000D0, 1.05300000D0, 1.05370000D0, 1.05440000D0,
     &   1.05510000D0, 1.05580000D0, 1.05650000D0, 1.05720000D0,
     &   1.05790000D0, 1.05860000D0, 1.05930000D0, 1.06000000D0,
     &   1.06200000D0, 1.06700000D0, 1.07700000D0, 1.08700000D0,
     &   1.09700000D0, 1.10700000D0, 1.11700000D0, 1.12700000D0,
     &   1.13700000D0, 1.13800000D0, 1.15200000D0, 1.16300000D0,
     &   1.16700000D0, 1.17700000D0, 1.18700000D0, 1.18800000D0,
     &   1.19700000D0, 1.20700000D0, 1.21200000D0, 1.21700000D0,
     &   1.22700000D0, 1.23700000D0, 1.24700000D0, 1.25700000D0,
     &   1.26300000D0, 1.26700000D0, 1.27700000D0, 1.28700000D0,
     &   1.28800000D0, 1.29700000D0, 1.30700000D0, 1.31200000D0,
     &   1.31700000D0, 1.32700000D0, 1.33700000D0, 1.34700000D0,
     &   1.35000000D0, 1.35700000D0, 1.36200000D0, 1.36700000D0,
     &   1.37700000D0, 1.38700000D0, 1.38800000D0, 1.39500000D0/
       DATA  (ESA  (I), I= 401, 500) /
     &   1.41000000D0, 1.41900000D0, 1.42700000D0, 1.43500000D0,
     &   1.43800000D0, 1.45200000D0, 1.45700000D0, 1.46000000D0,
     &   1.46400000D0, 1.47200000D0, 1.48100000D0, 1.49000000D0,
     &   1.49100000D0, 1.50000000D0, 1.50100000D0, 1.50600000D0,
     &   1.51000000D0, 1.52300000D0, 1.52500000D0, 1.53800000D0,
     &   1.53900000D0, 1.54000000D0, 1.55000000D0, 1.56000000D0,
     &   1.56200000D0, 1.57400000D0, 1.57500000D0, 1.58000000D0,
     &   1.58100000D0, 1.58700000D0, 1.59000000D0, 1.59900000D0,
     &   1.60000000D0, 1.61000000D0, 1.61900000D0, 1.62400000D0,
     &   1.62500000D0, 1.63800000D0, 1.64000000D0, 1.65000000D0,
     &   1.65700000D0, 1.66000000D0, 1.66200000D0, 1.66300000D0,
     &   1.67300000D0, 1.67500000D0, 1.68100000D0, 1.68700000D0,
     &   1.69000000D0, 1.70000000D0, 1.71000000D0, 1.71200000D0,
     &   1.71300000D0, 1.71900000D0, 1.72200000D0, 1.72500000D0,
     &   1.73800000D0, 1.74000000D0, 1.74600000D0, 1.75000000D0,
     &   1.75600000D0, 1.75700000D0, 1.76000000D0, 1.76200000D0,
     &   1.76400000D0, 1.76900000D0, 1.77500000D0, 1.78100000D0,
     &   1.79000000D0, 1.79900000D0, 1.80000000D0, 1.81200000D0,
     &   1.81900000D0, 1.82500000D0, 1.82600000D0, 1.83400000D0,
     &   1.83500000D0, 1.83800000D0, 1.84500000D0, 1.85700000D0,
     &   1.85900000D0, 1.87400000D0, 1.88100000D0, 1.88600000D0,
     &   1.90000000D0, 1.91900000D0, 1.92500000D0, 1.93000000D0,
     &   1.93800000D0, 1.94000000D0, 1.94500000D0, 1.94600000D0,
     &   1.95000000D0, 1.95500000D0, 1.96000000D0, 1.96200000D0,
     &   1.98100000D0, 1.99050000D0, 2.00500000D0, 2.05000000D0/
       DATA  (ESA  (I), I= 501, 600) /
     &   2.10000000D0, 2.15010000D0, 2.22000000D0, 2.26000000D0,
     &   2.30000000D0, 2.40000000D0, 2.45000000D0, 2.50000000D0,
     &   2.55000000D0, 2.60000000D0, 2.65000000D0, 2.70000000D0,
     &   2.75000000D0, 2.80000000D0, 2.85000000D0, 2.90000000D0,
     &   2.91000000D0, 2.92000000D0, 2.93000000D0, 2.94000000D0,
     &   2.95000000D0, 2.96000000D0, 2.97000000D0, 2.97200000D0,
     &   2.97400000D0, 2.97600000D0, 2.97800000D0, 2.98000000D0,
     &   2.98200000D0, 2.98400000D0, 2.98600000D0, 2.98800000D0,
     &   2.99000000D0, 2.99900000D0, 3.00500000D0, 3.00800000D0,
     &   3.01000000D0, 3.01200000D0, 3.01400000D0, 3.01600000D0,
     &   3.01800000D0, 3.02000000D0, 3.02100000D0, 3.02200000D0,
     &   3.02300000D0, 3.02400000D0, 3.02500000D0, 3.02600000D0,
     &   3.02700000D0, 3.02800000D0, 3.02900000D0, 3.03000000D0,
     &   3.03100000D0, 3.03200000D0, 3.03300000D0, 3.03400000D0,
     &   3.03500000D0, 3.03600000D0, 3.03700000D0, 3.03800000D0,
     &   3.03900000D0, 3.04000000D0, 3.04100000D0, 3.04200000D0,
     &   3.04300000D0, 3.04400000D0, 3.04500000D0, 3.04600000D0,
     &   3.04700000D0, 3.04800000D0, 3.04900000D0, 3.05000000D0,
     &   3.05100000D0, 3.05200000D0, 3.05300000D0, 3.05400000D0,
     &   3.05500000D0, 3.05600000D0, 3.05700000D0, 3.05800000D0,
     &   3.05900000D0, 3.06000000D0, 3.06050000D0, 3.06100000D0,
     &   3.06150000D0, 3.06200000D0, 3.06250000D0, 3.06300000D0,
     &   3.06350000D0, 3.06400000D0, 3.06450000D0, 3.06500000D0,
     &   3.06550000D0, 3.06600000D0, 3.06650000D0, 3.06700000D0,
     &   3.06750000D0, 3.06800000D0, 3.06850000D0, 3.06900000D0/
       DATA  (ESA  (I), I= 601, 700) /
     &   3.06950000D0, 3.07000000D0, 3.07050000D0, 3.07100000D0,
     &   3.07150000D0, 3.07200000D0, 3.07250000D0, 3.07300000D0,
     &   3.07350000D0, 3.07400000D0, 3.07450000D0, 3.07500000D0,
     &   3.07550000D0, 3.07600000D0, 3.07650000D0, 3.07700000D0,
     &   3.07750000D0, 3.07800000D0, 3.07850000D0, 3.07900000D0,
     &   3.07970000D0, 3.08000000D0, 3.08050000D0, 3.08100000D0,
     &   3.08150000D0, 3.08200000D0, 3.08250000D0, 3.08300000D0,
     &   3.08350000D0, 3.08400000D0, 3.08450000D0, 3.08500000D0,
     &   3.08550000D0, 3.08587000D0, 3.08688300D0, 3.08789600D0,
     &   3.08890900D0, 3.08992200D0, 3.09093500D0, 3.09194800D0,
     &   3.09296100D0, 3.09397400D0, 3.09498700D0, 3.09600000D0,
     &   3.09605000D0, 3.09610000D0, 3.09615000D0, 3.09620000D0,
     &   3.09625000D0, 3.09630000D0, 3.09635000D0, 3.09640000D0,
     &   3.09645000D0, 3.09650000D0, 3.09655000D0, 3.09660000D0,
     &   3.09665000D0, 3.09665500D0, 3.09673500D0, 3.09677100D0,
     &   3.09679300D0, 3.09680900D0, 3.09682000D0, 3.09682900D0,
     &   3.09683600D0, 3.09684300D0, 3.09684800D0, 3.09685300D0,
     &   3.09685700D0, 3.09686100D0, 3.09686400D0, 3.09686800D0,
     &   3.09687000D0, 3.09687300D0, 3.09687600D0, 3.09687800D0,
     &   3.09688100D0, 3.09688300D0, 3.09688500D0, 3.09688700D0,
     &   3.09688900D0, 3.09689100D0, 3.09689300D0, 3.09689500D0,
     &   3.09689700D0, 3.09689900D0, 3.09690100D0, 3.09690400D0,
     &   3.09690600D0, 3.09690900D0, 3.09691200D0, 3.09692000D0,
     &   3.09692800D0, 3.09693100D0, 3.09693400D0, 3.09693600D0,
     &   3.09693900D0, 3.09694100D0, 3.09694300D0, 3.09694500D0/
       DATA  (ESA  (I), I= 701, 800) /
     &   3.09694700D0, 3.09694900D0, 3.09695100D0, 3.09695300D0,
     &   3.09695500D0, 3.09695700D0, 3.09695900D0, 3.09696200D0,
     &   3.09696400D0, 3.09696700D0, 3.09697000D0, 3.09697200D0,
     &   3.09697600D0, 3.09697900D0, 3.09698300D0, 3.09698700D0,
     &   3.09699200D0, 3.09699700D0, 3.09700400D0, 3.09701100D0,
     &   3.09702000D0, 3.09703100D0, 3.09704700D0, 3.09706900D0,
     &   3.09710500D0, 3.09715000D0, 3.09720000D0, 3.09725000D0,
     &   3.09730000D0, 3.09735000D0, 3.09740000D0, 3.09745000D0,
     &   3.09750000D0, 3.09755000D0, 3.09760000D0, 3.09765000D0,
     &   3.09770000D0, 3.09775000D0, 3.09780000D0, 3.09785000D0,
     &   3.09790000D0, 3.09795000D0, 3.09800000D0, 3.09898700D0,
     &   3.09997400D0, 3.10096100D0, 3.10194800D0, 3.10293500D0,
     &   3.10392200D0, 3.10490900D0, 3.10589600D0, 3.10688300D0,
     &   3.10787000D0, 3.10800000D0, 3.10900000D0, 3.11000000D0,
     &   3.11100000D0, 3.11200000D0, 3.11300000D0, 3.11400000D0,
     &   3.11500000D0, 3.11600000D0, 3.11700000D0, 3.11800000D0,
     &   3.11900000D0, 3.12000000D0, 3.12100000D0, 3.12200000D0,
     &   3.12300000D0, 3.12400000D0, 3.12500000D0, 3.12600000D0,
     &   3.12700000D0, 3.12800000D0, 3.12900000D0, 3.13000000D0,
     &   3.13100000D0, 3.13200000D0, 3.13300000D0, 3.13400000D0,
     &   3.13500000D0, 3.13600000D0, 3.13700000D0, 3.13800000D0,
     &   3.13900000D0, 3.14000000D0, 3.14100000D0, 3.14200000D0,
     &   3.14300000D0, 3.14400000D0, 3.14500000D0, 3.14600000D0,
     &   3.14700000D0, 3.14800000D0, 3.14900000D0, 3.15000000D0,
     &   3.15100000D0, 3.15200000D0, 3.15300000D0, 3.15400000D0/
       DATA  (ESA  (I), I= 801, 900) /
     &   3.15500000D0, 3.15600000D0, 3.15700000D0, 3.15800000D0,
     &   3.15900000D0, 3.16000000D0, 3.16100000D0, 3.16200000D0,
     &   3.16300000D0, 3.16400000D0, 3.16500000D0, 3.16600000D0,
     &   3.16700000D0, 3.16800000D0, 3.16900000D0, 3.17000000D0,
     &   3.17100000D0, 3.17200000D0, 3.17300000D0, 3.17400000D0,
     &   3.17500000D0, 3.17600000D0, 3.17700000D0, 3.17800000D0,
     &   3.17900000D0, 3.18000000D0, 3.18100000D0, 3.18200000D0,
     &   3.18300000D0, 3.18400000D0, 3.18500000D0, 3.18600000D0,
     &   3.18700000D0, 3.18800000D0, 3.18900000D0, 3.19000000D0,
     &   3.19100000D0, 3.19200000D0, 3.19300000D0, 3.19400000D0,
     &   3.19500000D0, 3.19600000D0, 3.19700000D0, 3.19800000D0,
     &   3.19900000D0, 3.19990000D0, 3.20100000D0, 3.20200000D0,
     &   3.20300000D0, 3.20400000D0, 3.20500000D0, 3.20600000D0,
     &   3.20700000D0, 3.20800000D0, 3.20900000D0, 3.21000000D0,
     &   3.25000000D0, 3.30000000D0, 3.37000000D0, 3.45000000D0,
     &   3.50000000D0, 3.52000000D0, 3.54000000D0, 3.56000000D0,
     &   3.57500000D0, 3.59000000D0, 3.59500000D0, 3.60500000D0,
     &   3.61000000D0, 3.61500000D0, 3.62000000D0, 3.62500000D0,
     &   3.63000000D0, 3.63500000D0, 3.64000000D0, 3.64500000D0,
     &   3.65000000D0, 3.65200000D0, 3.65400000D0, 3.65600000D0,
     &   3.65800000D0, 3.66000000D0, 3.66200000D0, 3.66300000D0,
     &   3.66400000D0, 3.66500000D0, 3.66600000D0, 3.66700000D0,
     &   3.66800000D0, 3.66900000D0, 3.67000000D0, 3.67100000D0,
     &   3.67200000D0, 3.67300000D0, 3.67400000D0, 3.67496000D0,
     &   3.67551000D0, 3.67606000D0, 3.67661000D0, 3.67716000D0/
       DATA  (ESA  (I), I= 901,1000) /
     &   3.67771000D0, 3.67826000D0, 3.67881000D0, 3.67936000D0,
     &   3.67991000D0, 3.68046000D0, 3.68101000D0, 3.68156000D0,
     &   3.68211000D0, 3.68266000D0, 3.68321000D0, 3.68376000D0,
     &   3.68431000D0, 3.68486000D0, 3.68527300D0, 3.68541000D0,
     &   3.68552000D0, 3.68569900D0, 3.68574600D0, 3.68578100D0,
     &   3.68580900D0, 3.68583200D0, 3.68585100D0, 3.68586800D0,
     &   3.68588300D0, 3.68589500D0, 3.68590700D0, 3.68591800D0,
     &   3.68592800D0, 3.68593700D0, 3.68594600D0, 3.68595300D0,
     &   3.68596100D0, 3.68596800D0, 3.68597500D0, 3.68598200D0,
     &   3.68598800D0, 3.68599500D0, 3.68600100D0, 3.68600700D0,
     &   3.68601300D0, 3.68602000D0, 3.68602600D0, 3.68603300D0,
     &   3.68603900D0, 3.68604700D0, 3.68605500D0, 3.68606600D0,
     &   3.68607500D0, 3.68608500D0, 3.68609000D0, 3.68609500D0,
     &   3.68610000D0, 3.68611400D0, 3.68612500D0, 3.68613300D0,
     &   3.68614100D0, 3.68614700D0, 3.68615400D0, 3.68616000D0,
     &   3.68616700D0, 3.68617300D0, 3.68617900D0, 3.68618500D0,
     &   3.68619200D0, 3.68619800D0, 3.68620500D0, 3.68621200D0,
     &   3.68621900D0, 3.68622700D0, 3.68623400D0, 3.68624300D0,
     &   3.68625200D0, 3.68626200D0, 3.68627300D0, 3.68628500D0,
     &   3.68629700D0, 3.68631200D0, 3.68632900D0, 3.68634800D0,
     &   3.68637100D0, 3.68639900D0, 3.68643400D0, 3.68648100D0,
     &   3.68654800D0, 3.68651000D0, 3.68660000D0, 3.68667000D0,
     &   3.68672000D0, 3.68685000D0, 3.68690700D0, 3.68706000D0,
     &   3.68761000D0, 3.68816000D0, 3.68871000D0, 3.68926000D0,
     &   3.68981000D0, 3.69036000D0, 3.69091000D0, 3.69146000D0/
       DATA  (ESA  (I), I=1001,1100) /
     &   3.69201000D0, 3.69256000D0, 3.69311000D0, 3.69366000D0,
     &   3.69421000D0, 3.69476000D0, 3.69531000D0, 3.69586000D0,
     &   3.69641000D0, 3.69696000D0, 3.69796000D0, 3.69950000D0,
     &   3.70050000D0, 3.70250000D0, 3.70450000D0, 3.70700000D0,
     &   3.71200000D0, 3.72128100D0, 3.73082800D0, 3.73660200D0,
     &   3.74060200D0, 3.74360300D0, 3.74597300D0, 3.74791700D0,
     &   3.74955600D0, 3.75096800D0, 3.75220600D0, 3.75330900D0,
     &   3.75430100D0, 3.75520500D0, 3.75603600D0, 3.75680500D0,
     &   3.75752300D0, 3.75819800D0, 3.75883700D0, 3.75944400D0,
     &   3.76002600D0, 3.76058600D0, 3.76112800D0, 3.76165500D0,
     &   3.76217200D0, 3.76268100D0, 3.76318700D0, 3.76369200D0,
     &   3.76420200D0, 3.76472200D0, 3.76526100D0, 3.76582800D0,
     &   3.76644200D0, 3.76713800D0, 3.76800500D0, 3.77000000D0,
     &   3.77199500D0, 3.77286200D0, 3.77355800D0, 3.77417200D0,
     &   3.77474000D0, 3.77527800D0, 3.77579800D0, 3.77630800D0,
     &   3.77681400D0, 3.77731900D0, 3.77782800D0, 3.77834500D0,
     &   3.77887200D0, 3.77941400D0, 3.77997400D0, 3.78055600D0,
     &   3.78116300D0, 3.78180200D0, 3.78247700D0, 3.78319500D0,
     &   3.78396400D0, 3.78479500D0, 3.78569900D0, 3.78669100D0,
     &   3.78779400D0, 3.78903200D0, 3.79044400D0, 3.79208300D0,
     &   3.79402700D0, 3.79639700D0, 3.79939800D0, 3.80339800D0,
     &   3.80917200D0, 3.81871900D0, 3.82200000D0, 3.83200000D0,
     &   3.85000000D0, 3.85200000D0, 3.86700000D0, 3.87300000D0,
     &   3.87800000D0, 3.88400000D0, 3.89000000D0, 3.89600000D0,
     &   3.90000000D0, 3.90200000D0, 3.90800000D0, 3.91400000D0/
       DATA  (ESA  (I), I=1101,1200) /
     &   3.92000000D0, 3.92600000D0, 3.93200000D0, 3.93800000D0,
     &   3.94400000D0, 3.95000000D0, 3.95600000D0, 3.96200000D0,
     &   3.96800000D0, 3.97400000D0, 3.98000000D0, 3.98600000D0,
     &   3.99200000D0, 3.99800000D0, 4.00000000D0, 4.00400000D0,
     &   4.01000000D0, 4.01600000D0, 4.02000000D0, 4.02200000D0,
     &   4.02800000D0, 4.03000000D0, 4.03400000D0, 4.04000000D0,
     &   4.04600000D0, 4.05000000D0, 4.05200000D0, 4.05800000D0,
     &   4.06400000D0, 4.07000000D0, 4.07600000D0, 4.08200000D0,
     &   4.08800000D0, 4.09000000D0, 4.09400000D0, 4.10000000D0,
     &   4.10500000D0, 4.10900000D0, 4.11000000D0, 4.11300000D0,
     &   4.11800000D0, 4.12100000D0, 4.12500000D0, 4.12900000D0,
     &   4.13000000D0, 4.13300000D0, 4.13600000D0, 4.14100000D0,
     &   4.14700000D0, 4.15000000D0, 4.15300000D0, 4.15900000D0,
     &   4.16500000D0, 4.17100000D0, 4.17700000D0, 4.18000000D0,
     &   4.18200000D0, 4.18500000D0, 4.19000000D0, 4.19500000D0,
     &   4.19900000D0, 4.20100000D0, 4.20800000D0, 4.21200000D0,
     &   4.21400000D0, 4.22000000D0, 4.22600000D0, 4.23200000D0,
     &   4.23800000D0, 4.24000000D0, 4.24400000D0, 4.25000000D0,
     &   4.25600000D0, 4.26200000D0, 4.26800000D0, 4.27400000D0,
     &   4.28000000D0, 4.29200000D0, 4.30000000D0, 4.30400000D0,
     &   4.31000000D0, 4.31600000D0, 4.32800000D0, 4.33000000D0,
     &   4.34000000D0, 4.35000000D0, 4.35200000D0, 4.36400000D0,
     &   4.37600000D0, 4.38000000D0, 4.38200000D0, 4.38800000D0,
     &   4.39000000D0, 4.39400000D0, 4.40000000D0, 4.40600000D0,
     &   4.41000000D0, 4.41200000D0, 4.41800000D0, 4.42400000D0/
       DATA  (ESA  (I), I=1201,1300) /
     &   4.43000000D0, 4.43600000D0, 4.44000000D0, 4.44200000D0,
     &   4.44800000D0, 4.45000000D0, 4.46000000D0, 4.47200000D0,
     &   4.48400000D0, 4.49600000D0, 4.50450000D0, 4.54000000D0,
     &   4.55000000D0, 4.57000000D0, 4.60000000D0, 4.62000000D0,
     &   4.65000000D0, 4.66000000D0, 4.70000000D0, 4.71000000D0,
     &   4.80000000D0, 4.90000000D0, 4.98000000D0, 5.06500000D0,
     &   5.15000000D0, 5.22500000D0, 5.25000000D0, 5.30000000D0,
     &   5.40000000D0, 5.45000000D0, 5.50000000D0, 5.60000000D0,
     &   5.70000000D0, 5.75000000D0, 5.80000000D0, 5.85000000D0,
     &   5.90000000D0, 5.95000000D0, 6.00000000D0, 6.05000000D0,
     &   6.10000000D0, 6.15000000D0, 6.20000000D0, 6.25000000D0,
     &   6.30000000D0, 6.35000000D0, 6.40000000D0, 6.45000000D0,
     &   6.50000000D0, 6.55000000D0, 6.60000000D0, 6.65000000D0,
     &   6.70000000D0, 6.75000000D0, 6.80000000D0, 6.85000000D0,
     &   6.90000000D0, 6.95000000D0, 7.00000000D0, 7.05000000D0,
     &   7.10000000D0, 7.15000000D0, 7.20000000D0, 7.25000000D0,
     &   7.30000000D0, 7.35000000D0, 7.40000000D0, 7.44000000D0,
     &   7.50000000D0, 7.60000000D0, 7.70000000D0, 7.80000000D0,
     &   7.90000000D0, 8.00000000D0, 8.10000000D0, 8.20000000D0,
     &   8.30000000D0, 8.40000000D0, 8.50000000D0, 8.60000000D0,
     &   8.70000000D0, 8.80000000D0, 8.90000000D0, 8.91000000D0,
     &   9.00000000D0, 9.10000000D0, 9.20000000D0, 9.28000000D0,
     &   9.30000000D0, 9.35000000D0, 9.39000000D0, 9.40000000D0,
     &   9.41000000D0, 9.41500000D0, 9.42000000D0, 9.42400000D0,
     &   9.42800000D0, 9.43000000D0, 9.43200000D0, 9.43400000D0/
       DATA  (ESA  (I), I=1301,1400) /
     &   9.43600000D0, 9.43800000D0, 9.44000000D0, 9.44100000D0,
     &   9.44200000D0, 9.44300000D0, 9.44400000D0, 9.44500000D0,
     &   9.44600000D0, 9.44700000D0, 9.44800000D0, 9.44900000D0,
     &   9.44937000D0, 9.44959000D0, 9.44981000D0, 9.45003000D0,
     &   9.45025000D0, 9.45047000D0, 9.45069000D0, 9.45091000D0,
     &   9.45113000D0, 9.45135000D0, 9.45157000D0, 9.45179000D0,
     &   9.45201000D0, 9.45223000D0, 9.45245000D0, 9.45267000D0,
     &   9.45289000D0, 9.45311000D0, 9.45333000D0, 9.45355000D0,
     &   9.45377000D0, 9.45399000D0, 9.45421000D0, 9.45443000D0,
     &   9.45465000D0, 9.45487000D0, 9.45509000D0, 9.45531000D0,
     &   9.45553000D0, 9.45575000D0, 9.45597000D0, 9.45619000D0,
     &   9.45641000D0, 9.45663000D0, 9.45685000D0, 9.45707000D0,
     &   9.45729000D0, 9.45751000D0, 9.45773000D0, 9.45795000D0,
     &   9.45817000D0, 9.45839000D0, 9.45861000D0, 9.45883000D0,
     &   9.45905000D0, 9.45927000D0, 9.45949000D0, 9.45971000D0,
     &   9.45993000D0, 9.46000000D0, 9.46014595D0, 9.46019268D0,
     &   9.46021366D0, 9.46022606D0, 9.46023464D0, 9.46024132D0,
     &   9.46024704D0, 9.46025085D0, 9.46025467D0, 9.46025848D0,
     &   9.46026039D0, 9.46026325D0, 9.46026516D0, 9.46026802D0,
     &   9.46026897D0, 9.46027088D0, 9.46027279D0, 9.46027470D0,
     &   9.46027565D0, 9.46027660D0, 9.46027851D0, 9.46027946D0,
     &   9.46028042D0, 9.46028233D0, 9.46028328D0, 9.46028423D0,
     &   9.46028519D0, 9.46028709D0, 9.46028805D0, 9.46028900D0,
     &   9.46029091D0, 9.46029186D0, 9.46029377D0, 9.46029568D0,
     &   9.46030045D0, 9.46030426D0, 9.46030617D0, 9.46030807D0/
       DATA  (ESA  (I), I=1401,1500) /
     &   9.46030998D0, 9.46031094D0, 9.46031189D0, 9.46031284D0,
     &   9.46031475D0, 9.46031570D0, 9.46031666D0, 9.46031761D0,
     &   9.46031952D0, 9.46032047D0, 9.46032143D0, 9.46032333D0,
     &   9.46032429D0, 9.46032619D0, 9.46032715D0, 9.46032906D0,
     &   9.46033096D0, 9.46033287D0, 9.46033478D0, 9.46033669D0,
     &   9.46033955D0, 9.46034145D0, 9.46034527D0, 9.46034908D0,
     &   9.46035290D0, 9.46035862D0, 9.46036530D0, 9.46037388D0,
     &   9.46038628D0, 9.46040726D0, 9.46045494D0, 9.46059000D0,
     &   9.46081000D0, 9.46103000D0, 9.46125000D0, 9.46147000D0,
     &   9.46169000D0, 9.46191000D0, 9.46213000D0, 9.46235000D0,
     &   9.46257000D0, 9.46279000D0, 9.46301000D0, 9.46323000D0,
     &   9.46345000D0, 9.46367000D0, 9.46389000D0, 9.46411000D0,
     &   9.46433000D0, 9.46455000D0, 9.46477000D0, 9.46499000D0,
     &   9.46521000D0, 9.46543000D0, 9.46565000D0, 9.46587000D0,
     &   9.46609000D0, 9.46631000D0, 9.46653000D0, 9.46675000D0,
     &   9.46697000D0, 9.46719000D0, 9.46741000D0, 9.46763000D0,
     &   9.46785000D0, 9.46807000D0, 9.46829000D0, 9.46851000D0,
     &   9.46873000D0, 9.46895000D0, 9.46917000D0, 9.46939000D0,
     &   9.46961000D0, 9.46983000D0, 9.47005000D0, 9.47027000D0,
     &   9.47049000D0, 9.47071000D0, 9.47093000D0, 9.47115000D0,
     &   9.47137000D0, 9.47337000D0, 9.47500000D0, 9.47700000D0,
     &   9.48000000D0, 9.48400000D0, 9.49000000D0, 9.49500000D0,
     &   9.50500000D0, 9.51000000D0, 9.51500000D0, 9.52000000D0,
     &   9.52500000D0, 9.53000000D0, 9.54000000D0, 9.55000000D0,
     &   9.60000000D0, 9.70000000D0, 9.80000000D0, 9.90000000D0/
       DATA  (ESA  (I), I=1501,1600) /
     &   9.95000000D0, 9.97500000D0, 9.99000000D0,10.00000000D0,
     &  10.01000000D0,10.01100000D0,10.01200000D0,10.01226000D0,
     &  10.01281000D0,10.01336000D0,10.01391000D0,10.01446000D0,
     &  10.01501000D0,10.01556000D0,10.01611000D0,10.01666000D0,
     &  10.01721000D0,10.01776000D0,10.01831000D0,10.01886000D0,
     &  10.01941000D0,10.01996000D0,10.02051000D0,10.02106000D0,
     &  10.02161000D0,10.02216000D0,10.02250000D0,10.02253750D0,
     &  10.02257500D0,10.02261250D0,10.02265000D0,10.02268750D0,
     &  10.02272500D0,10.02276250D0,10.02280000D0,10.02283750D0,
     &  10.02287500D0,10.02291250D0,10.02295000D0,10.02298750D0,
     &  10.02302500D0,10.02306250D0,10.02310000D0,10.02313750D0,
     &  10.02317500D0,10.02321250D0,10.02322000D0,10.02322500D0,
     &  10.02323000D0,10.02323400D0,10.02323700D0,10.02324000D0,
     &  10.02324200D0,10.02324400D0,10.02324600D0,10.02324800D0,
     &  10.02324930D0,10.02324950D0,10.02324960D0,10.02324980D0,
     &  10.02324988D0,10.02324994D0,10.02325000D0,10.02325060D0,
     &  10.02325150D0,10.02325250D0,10.02325340D0,10.02325440D0,
     &  10.02325540D0,10.02325630D0,10.02325680D0,10.02325730D0,
     &  10.02325780D0,10.02325850D0,10.02325900D0,10.02325950D0,
     &  10.02325980D0,10.02325990D0,10.02325995D0,10.02326000D0,
     &  10.02326005D0,10.02326010D0,10.02326050D0,10.02326100D0,
     &  10.02326150D0,10.02326390D0,10.02326490D0,10.02326680D0,
     &  10.02326770D0,10.02326870D0,10.02326970D0,10.02327060D0,
     &  10.02327160D0,10.02327250D0,10.02327350D0,10.02327440D0,
     &  10.02327540D0,10.02327630D0,10.02327730D0,10.02327820D0/
       DATA  (ESA  (I), I=1601,1700) /
     &  10.02328010D0,10.02328110D0,10.02328210D0,10.02328300D0,
     &  10.02328490D0,10.02328590D0,10.02328750D0,10.02329000D0,
     &  10.02330000D0,10.02331000D0,10.02332000D0,10.02333000D0,
     &  10.02334000D0,10.02335000D0,10.02336250D0,10.02339000D0,
     &  10.02340000D0,10.02341000D0,10.02342000D0,10.02343750D0,
     &  10.02347500D0,10.02351250D0,10.02355000D0,10.02358750D0,
     &  10.02362500D0,10.02366250D0,10.02370000D0,10.02373750D0,
     &  10.02377500D0,10.02381250D0,10.02385000D0,10.02388750D0,
     &  10.02392500D0,10.02396250D0,10.02400000D0,10.02436000D0,
     &  10.02491000D0,10.02546000D0,10.02656000D0,10.02766000D0,
     &  10.02876000D0,10.02986000D0,10.03096000D0,10.03206000D0,
     &  10.03316000D0,10.03426000D0,10.03500000D0,10.03600000D0,
     &  10.03700000D0,10.03800000D0,10.03900000D0,10.04000000D0,
     &  10.04500000D0,10.05000000D0,10.05500000D0,10.06000000D0,
     &  10.06500000D0,10.07000000D0,10.07500000D0,10.08000000D0,
     &  10.20000000D0,10.25000000D0,10.26000000D0,10.27000000D0,
     &  10.28000000D0,10.29000000D0,10.29500000D0,10.30000000D0,
     &  10.31000000D0,10.32000000D0,10.33000000D0,10.34000000D0,
     &  10.34420000D0,10.34493333D0,10.34566667D0,10.34640000D0,
     &  10.34713333D0,10.34786667D0,10.34860000D0,10.34933333D0,
     &  10.35006667D0,10.35080000D0,10.35153333D0,10.35226667D0,
     &  10.35300000D0,10.35309167D0,10.35318333D0,10.35327500D0,
     &  10.35336667D0,10.35345833D0,10.35355000D0,10.35364167D0,
     &  10.35373333D0,10.35382500D0,10.35391667D0,10.35400833D0,
     &  10.35410000D0,10.35419167D0,10.35428333D0,10.35437500D0/
       DATA  (ESA  (I), I=1701,1800) /
     &  10.35440000D0,10.35446667D0,10.35455833D0,10.35465000D0,
     &  10.35474167D0,10.35483334D0,10.35492500D0,10.35501667D0,
     &  10.35510834D0,10.35512330D0,10.35514660D0,10.35515710D0,
     &  10.35516340D0,10.35516780D0,10.35517110D0,10.35517370D0,
     &  10.35517580D0,10.35517760D0,10.35517920D0,10.35518060D0,
     &  10.35518180D0,10.35518290D0,10.35518390D0,10.35518480D0,
     &  10.35518570D0,10.35518650D0,10.35518720D0,10.35518790D0,
     &  10.35518860D0,10.35518930D0,10.35518990D0,10.35519050D0,
     &  10.35519110D0,10.35519170D0,10.35519230D0,10.35519280D0,
     &  10.35519340D0,10.35519400D0,10.35519460D0,10.35519530D0,
     &  10.35519600D0,10.35519680D0,10.35519770D0,10.35519970D0,
     &  10.35520000D0,10.35520100D0,10.35520230D0,10.35520320D0,
     &  10.35520400D0,10.35520470D0,10.35520540D0,10.35520600D0,
     &  10.35520660D0,10.35520720D0,10.35520770D0,10.35520830D0,
     &  10.35520890D0,10.35520950D0,10.35521010D0,10.35521070D0,
     &  10.35521140D0,10.35521210D0,10.35521280D0,10.35521350D0,
     &  10.35521430D0,10.35521520D0,10.35521610D0,10.35521710D0,
     &  10.35521820D0,10.35521940D0,10.35522080D0,10.35522240D0,
     &  10.35522420D0,10.35522630D0,10.35522890D0,10.35523220D0,
     &  10.35523660D0,10.35524290D0,10.35525340D0,10.35527670D0,
     &  10.35529000D0,10.35529300D0,10.35529700D0,10.35531000D0,
     &  10.35532500D0,10.35538334D0,10.35547500D0,10.35556667D0,
     &  10.35565834D0,10.35575000D0,10.35584167D0,10.35593334D0,
     &  10.35602500D0,10.35611667D0,10.35620834D0,10.35630000D0,
     &  10.35639167D0,10.35648334D0,10.35657500D0,10.35666667D0/
       DATA  (ESA  (I), I=1801,1900) /
     &  10.35740000D0,10.35813333D0,10.35886667D0,10.35960000D0,
     &  10.36033333D0,10.36106667D0,10.36180000D0,10.36326667D0,
     &  10.36473333D0,10.36546667D0,10.36620000D0,10.37000000D0,
     &  10.37500000D0,10.38000000D0,10.38500000D0,10.39000000D0,
     &  10.39500000D0,10.40000000D0,10.40500000D0,10.41000000D0,
     &  10.41500000D0,10.42000000D0,10.42500000D0,10.43000000D0,
     &  10.43500000D0,10.44000000D0,10.44500000D0,10.45000000D0,
     &  10.45500000D0,10.46000000D0,10.46500000D0,10.47000000D0,
     &  10.47300000D0,10.52897000D0,10.54217000D0,10.54889000D0,
     &  10.55317000D0,10.55622000D0,10.55855000D0,10.56042000D0,
     &  10.56197000D0,10.56328000D0,10.56442000D0,10.56543000D0,
     &  10.56633000D0,10.56714000D0,10.56788000D0,10.56857000D0,
     &  10.56921000D0,10.56980000D0,10.57036000D0,10.57090000D0,
     &  10.57141000D0,10.57189000D0,10.57237000D0,10.57282000D0,
     &  10.57327000D0,10.57371000D0,10.57415000D0,10.57459000D0,
     &  10.57503000D0,10.57547000D0,10.57594000D0,10.57643000D0,
     &  10.57695000D0,10.57755000D0,10.57829000D0,10.58000000D0,
     &  10.58171000D0,10.58245000D0,10.58305000D0,10.58357000D0,
     &  10.58406000D0,10.58453000D0,10.58497000D0,10.58541000D0,
     &  10.58585000D0,10.58629000D0,10.58673000D0,10.58718000D0,
     &  10.58763000D0,10.58811000D0,10.58859000D0,10.58910000D0,
     &  10.58964000D0,10.59020000D0,10.59079000D0,10.59143000D0,
     &  10.59212000D0,10.59286000D0,10.59367000D0,10.59457000D0,
     &  10.59558000D0,10.59671000D0,10.59803000D0,10.59958000D0,
     &  10.60145000D0,10.60378000D0,10.60683000D0,10.61111000D0/
       DATA  (ESA  (I), I=1901,2000) /
     &  10.61783000D0,10.63103000D0,10.68700000D0,10.69000000D0,
     &  10.71309000D0,10.72975000D0,10.74256000D0,10.75286000D0,
     &  10.76141000D0,10.76869000D0,10.77500000D0,10.78057000D0,
     &  10.78554000D0,10.79004000D0,10.79414000D0,10.79791000D0,
     &  10.80141000D0,10.80468000D0,10.80775000D0,10.81065000D0,
     &  10.81341000D0,10.81604000D0,10.81858000D0,10.82102000D0,
     &  10.82339000D0,10.82570000D0,10.82796000D0,10.83019000D0,
     &  10.83240000D0,10.83460000D0,10.83681000D0,10.83906000D0,
     &  10.84135000D0,10.84374000D0,10.84626000D0,10.84900000D0,
     &  10.85212000D0,10.85601000D0,10.86500000D0,10.87288000D0,
     &  10.87627000D0,10.87895000D0,10.88128000D0,10.88340000D0,
     &  10.88539000D0,10.88728000D0,10.88910000D0,10.89088000D0,
     &  10.89262000D0,10.89434000D0,10.89605000D0,10.89775000D0,
     &  10.89946000D0,10.90119000D0,10.90292000D0,10.90468000D0,
     &  10.90648000D0,10.90830000D0,10.91017000D0,10.91208000D0,
     &  10.91406000D0,10.91609000D0,10.91820000D0,10.92038000D0,
     &  10.92266000D0,10.92504000D0,10.92754000D0,10.93017000D0,
     &  10.93295000D0,10.93590000D0,10.93905000D0,10.94242000D0,
     &  10.94606000D0,10.95000000D0,10.97500000D0,10.97638000D0,
     &  10.97771000D0,10.97900000D0,10.98025000D0,10.98146000D0,
     &  10.98263000D0,10.98377000D0,10.98489000D0,10.98598000D0,
     &  10.98705000D0,10.98810000D0,10.98913000D0,10.99014000D0,
     &  10.99114000D0,10.99213000D0,10.99310000D0,10.99407000D0,
     &  10.99504000D0,10.99600000D0,10.99697000D0,10.99793000D0,
     &  10.99890000D0,10.99988000D0,11.00087000D0,11.00188000D0/
       DATA  (ESA  (I), I=2001,2100) /
     &  11.00290000D0,11.00396000D0,11.00506000D0,11.00621000D0,
     &  11.00742000D0,11.00873000D0,11.01018000D0,11.01186000D0,
     &  11.01399000D0,11.01900000D0,11.02401000D0,11.02614000D0,
     &  11.02782000D0,11.02927000D0,11.03058000D0,11.03179000D0,
     &  11.03294000D0,11.03404000D0,11.03510000D0,11.03612000D0,
     &  11.03713000D0,11.03812000D0,11.03910000D0,11.04007000D0,
     &  11.04103000D0,11.04200000D0,11.04296000D0,11.04392000D0,
     &  11.04490000D0,11.04587000D0,11.04686000D0,11.04786000D0,
     &  11.04887000D0,11.04990000D0,11.05095000D0,11.05202000D0,
     &  11.05311000D0,11.05423000D0,11.05537000D0,11.05654000D0,
     &  11.05775000D0,11.05900000D0,11.06029000D0,11.06162000D0,
     &  11.06300000D0,11.07000000D0,11.09200000D0,11.11500000D0,
     &  11.13700000D0,11.16000000D0,11.18200000D0,11.20500000D0,
     &  11.22700000D0,11.25000000D0,11.27300000D0,11.29500000D0,
     &  11.31800000D0,11.34000000D0,11.50000000D0,11.65500000D0,
     &  11.67700000D0,11.70000000D0,11.72200000D0,11.74500000D0,
     &  11.76700000D0,11.79000000D0,11.81200000D0,11.83500000D0,
     &  11.85800000D0,11.88000000D0,11.90300000D0,11.92500000D0,
     &  11.94800000D0,11.97000000D0,11.99300000D0,12.01500000D0,
     &  12.03800000D0,12.06000000D0,12.08300000D0,12.10500000D0,
     &  12.12700000D0,12.15000000D0,12.17200000D0,12.19500000D0,
     &  12.21700000D0,12.24000000D0,12.26200000D0,12.28500000D0,
     &  12.30700000D0,12.33000000D0,12.35200000D0,12.37500000D0,
     &  12.39800000D0,12.42000000D0,12.44300000D0,12.46500000D0,
     &  12.48800000D0,12.51000000D0,12.53300000D0,12.55500000D0/
       DATA  (ESA  (I), I=2101,2174) /
     &  12.57800000D0,12.60000000D0,12.62300000D0,12.64500000D0,
     &  12.66700000D0,12.69000000D0,12.71200000D0,12.73500000D0,
     &  12.75700000D0,12.78000000D0,12.80200000D0,12.82500000D0,
     &  12.84700000D0,12.87000000D0,12.89200000D0,12.91500000D0,
     &  12.93800000D0,12.96000000D0,12.98300000D0,13.00500000D0,
     &  13.02800000D0,13.05000000D0,13.07300000D0,13.09500000D0,
     &  13.11800000D0,13.14000000D0,13.16300000D0,13.18500000D0,
     &  13.20800000D0,13.23000000D0,13.25200000D0,13.27500000D0,
     &  13.29700000D0,13.32000000D0,13.34200000D0,13.36500000D0,
     &  13.38700000D0,13.41000000D0,13.43200000D0,13.45500000D0,
     &  13.47700000D0,13.50000000D0,13.75000000D0,14.00000000D0,
     &  14.04000000D0,16.00000000D0,17.00000000D0,20.00000000D0,
     &  21.95000000D0,22.05000000D0,25.00000000D0,25.01000000D0,
     &  27.50000000D0,27.60000000D0,27.66000000D0,29.00000000D0,
     &  29.93000000D0,30.10000000D0,30.38000000D0,30.80000000D0,
     &  31.10000000D0,31.29000000D0,33.20000000D0,33.30000000D0,
     &  33.89000000D0,34.00000000D0,34.50000000D0,34.70000000D0,
     &  34.86000000D0,35.01000000D0,35.45000000D0,36.10000000D0,
     &  36.38000000D0,38.29000000D0/
  
         DATA ((DAS  (I,J), I=   1, 100), J=1,1) /
     &  -9.22863404E-04,-9.81443967E-04,-1.03794066E-03,-1.09530550E-03,
     &  -1.15482288E-03,-1.18434784E-03,-1.23545802E-03,-1.32891616E-03,
     &  -1.36655666E-03,-1.40423520E-03,-1.46201107E-03,-1.51764858E-03,
     &  -1.58109680E-03,-1.62271585E-03,-1.67439886E-03,-1.72644869E-03,
     &  -1.77850529E-03,-1.83153893E-03,-1.87470978E-03,-1.92980541E-03,
     &  -1.97505465E-03,-2.03133426E-03,-2.07598313E-03,-2.13464304E-03,
     &  -2.18300716E-03,-2.23091648E-03,-2.29054443E-03,-2.32379358E-03,
     &  -2.33954820E-03,-2.38992788E-03,-2.44155075E-03,-2.49330213E-03,
     &  -2.54576743E-03,-2.59901396E-03,-2.65519939E-03,-2.71364232E-03,
     &  -2.77206334E-03,-2.83007410E-03,-2.89477752E-03,-2.95956674E-03,
     &  -3.00875539E-03,-3.07408936E-03,-3.14040122E-03,-3.19054025E-03,
     &  -3.25910494E-03,-3.32915949E-03,-3.38343650E-03,-3.45935289E-03,
     &  -3.51665964E-03,-3.59717005E-03,-4.04172340E-03,-4.52048354E-03,
     &  -5.20289413E-03,-6.07698536E-03,-6.37128328E-03,-6.45845303E-03,
     &  -6.68802141E-03,-6.99995726E-03,-7.33058902E-03,-7.67256963E-03,
     &  -8.02750939E-03,-8.14603586E-03,-8.36314757E-03,-8.71764332E-03,
     &  -9.01735248E-03,-9.32842895E-03,-9.58018218E-03,-9.80939911E-03,
     &  -9.84320530E-03,-9.91733308E-03,-9.94565544E-03,-9.96606193E-03,
     &  -9.85890804E-03,-9.61164642E-03,-9.16413147E-03,-8.60744110E-03,
     &  -7.82011410E-03,-6.89743585E-03,-6.08426585E-03,-5.86650848E-03,
     &  -5.45752052E-03,-5.00737162E-03,-4.70265913E-03,-4.53646337E-03,
     &  -4.43805913E-03,-4.69461094E-03,-5.09784385E-03,-5.44370818E-03,
     &  -5.80528733E-03,-6.02496428E-03,-6.27458064E-03,-6.44671127E-03,
     &  -6.59544832E-03,-6.61484580E-03,-6.57040064E-03,-6.50483849E-03,
     &  -6.41672869E-03,-6.32729725E-03,-6.20306243E-03,-6.03876674E-03/
  
         DATA ((DAS  (I,J), I= 101, 200), J=1,1) /
     &  -5.73215120E-03,-5.44733195E-03,-5.13122628E-03,-4.76935845E-03,
     &  -4.39962357E-03,-3.91085188E-03,-3.43180698E-03,-2.96248889E-03,
     &  -2.37747774E-03,-1.72430515E-03,-9.99683527E-04,-1.05304385E-04,
     &   8.16453710E-04, 1.78840655E-03, 3.02956580E-03, 4.52367409E-03,
     &   6.54721558E-03, 1.15417902E-02, 1.62613060E-02, 1.80354044E-02,
     &   1.93049450E-02, 2.02966031E-02, 2.11033149E-02, 2.17415135E-02,
     &   2.22910746E-02, 2.27408429E-02, 2.31090602E-02, 2.34116755E-02,
     &   2.36279570E-02, 2.37553627E-02, 2.38956718E-02, 2.46191847E-02,
     &   2.40159787E-02, 2.40313258E-02, 2.39826659E-02, 2.39161214E-02,
     &   2.38368378E-02, 2.36751517E-02, 2.35711857E-02, 2.33098909E-02,
     &   2.31514734E-02, 2.28514630E-02, 2.25846211E-02, 2.22001902E-02,
     &   2.17829653E-02, 2.12710371E-02, 2.06790600E-02, 2.00244068E-02,
     &   1.92642000E-02, 1.84177681E-02, 1.75197802E-02, 1.65190965E-02,
     &   1.63107199E-02, 1.57556940E-02, 1.52006680E-02, 1.47155080E-02,
     &   1.44187272E-02, 1.41047758E-02, 1.36666095E-02, 1.35704787E-02,
     &   1.34031095E-02, 1.32508356E-02, 1.31491362E-02, 1.30378293E-02,
     &   1.29169320E-02, 1.27987126E-02, 1.26918624E-02, 1.26210460E-02,
     &   1.22841593E-02, 1.19489562E-02, 1.16079207E-02, 1.13208640E-02,
     &   1.09722525E-02, 1.06174056E-02, 1.02415569E-02, 9.92513343E-03,
     &   9.53500426E-03, 9.14709893E-03, 8.82193520E-03, 8.63572341E-03,
     &   8.28899221E-03, 8.12837354E-03, 7.75443904E-03, 7.43565523E-03,
     &   7.04147458E-03, 6.69432863E-03, 6.26584918E-03, 5.89396992E-03,
     &   5.49096427E-03, 4.98834211E-03, 4.72755138E-03, 4.54232535E-03,
     &   4.17136380E-03, 3.96300048E-03, 3.74615859E-03, 3.51803650E-03,
     &   3.26463628E-03, 3.01423036E-03, 2.75195280E-03, 2.19803074E-03/
  
         DATA ((DAS  (I,J), I= 201, 300), J=1,1) /
     &   1.58156251E-03, 1.21601155E-03, 1.08002597E-03, 9.37189798E-04,
     &   7.95143161E-04, 6.46396184E-04, 5.06770582E-04, 3.32789317E-04,
     &   1.49004322E-04,-3.93871667E-06,-1.85243139E-04,-3.70439072E-04,
     &  -5.63004255E-04,-7.54374785E-04,-9.62438233E-04,-1.17438578E-03,
     &  -1.40143486E-03,-1.63030703E-03,-1.86124238E-03,-2.10896757E-03,
     &  -2.36417090E-03,-2.55369323E-03,-2.92066173E-03,-3.22415933E-03,
     &  -3.54093864E-03,-3.88326655E-03,-4.24590588E-03,-4.56176923E-03,
     &  -5.03170855E-03,-5.46510964E-03,-5.63704807E-03,-5.89263314E-03,
     &  -5.94580879E-03,-6.56236187E-03,-7.13846726E-03,-7.73873850E-03,
     &  -8.37128051E-03,-9.09729446E-03,-9.89978134E-03,-1.06691979E-02,
     &  -1.15402559E-02,-1.25540297E-02,-1.34054075E-02,-1.47308816E-02,
     &  -1.60563558E-02,-1.74557568E-02,-1.91390371E-02,-2.10110827E-02,
     &  -2.31392127E-02,-2.55616803E-02,-2.83608976E-02,-3.16051903E-02,
     &  -3.53619802E-02,-3.89848708E-02,-4.09658270E-02,-4.23068843E-02,
     &  -4.41796959E-02,-4.50868404E-02,-4.45442029E-02,-4.78459349E-02,
     &  -4.91793952E-02,-5.03825484E-02,-5.14496717E-02,-5.22939181E-02,
     &  -5.28427939E-02,-5.29467526E-02,-5.20157240E-02,-5.03314030E-02,
     &  -4.91630153E-02,-4.59222773E-02,-4.13708028E-02,-3.53647829E-02,
     &  -2.55248180E-02,-1.89411048E-02,-8.89546210E-03, 1.89284253E-03,
     &   1.28687460E-02, 2.34796783E-02, 3.32125976E-02, 4.16833222E-02,
     &   4.87198229E-02, 5.42772082E-02, 5.84157497E-02, 6.13045710E-02,
     &   6.31349123E-02, 6.41221551E-02, 6.44275777E-02, 6.41944405E-02,
     &   6.35954661E-02, 6.27051298E-02, 6.10534374E-02, 6.05798533E-02,
     &   5.82410969E-02, 5.76511354E-02, 5.63977078E-02, 5.50282455E-02,
     &   5.36823338E-02, 5.23702519E-02, 5.10849853E-02, 4.98164229E-02/
  
         DATA ((DAS  (I,J), I= 301, 400), J=1,1) /
     &   4.96409414E-02, 4.74426223E-02, 4.63321689E-02, 4.52589281E-02,
     &   4.17846754E-02, 3.88111404E-02, 3.62298630E-02, 3.39793706E-02,
     &   3.20258896E-02, 2.99514239E-02, 2.87790575E-02, 2.74323258E-02,
     &   2.61745047E-02, 2.51350513E-02, 2.41558425E-02, 2.32443047E-02,
     &   2.24837134E-02, 2.16989407E-02, 2.10330761E-02, 2.03999908E-02,
     &   1.98983524E-02, 1.93603384E-02, 1.93425222E-02, 1.83926322E-02,
     &   1.79649053E-02, 1.78245409E-02, 1.80476971E-02, 1.76653759E-02,
     &   1.71377311E-02, 1.67682837E-02, 1.64302486E-02, 1.61223375E-02,
     &   1.58300368E-02, 1.55665775E-02, 1.53176430E-02, 1.50702415E-02,
     &   1.48417074E-02, 1.46139933E-02, 1.44280279E-02, 1.42086944E-02,
     &   1.40265426E-02, 1.38494606E-02, 1.36682916E-02, 1.35010979E-02,
     &   1.33452191E-02, 1.31796596E-02, 1.30380965E-02, 1.28895176E-02,
     &   1.27408331E-02, 1.26195059E-02, 1.24875368E-02, 1.23596814E-02,
     &   1.22370557E-02, 1.20974004E-02, 1.19980603E-02, 1.18833756E-02,
     &   1.15689528E-02, 1.08885006E-02, 9.83259283E-03, 9.04678443E-03,
     &   8.42873489E-03, 7.91976826E-03, 7.47404472E-03, 7.08335576E-03,
     &   6.74438703E-03, 6.70782189E-03, 6.31048161E-03, 6.04364240E-03,
     &   5.95349116E-03, 5.73239927E-03, 5.52581283E-03, 5.50336853E-03,
     &   5.32289575E-03, 5.13637162E-03, 5.04242063E-03, 4.94962355E-03,
     &   4.69475820E-03, 4.63253518E-03, 4.49474901E-03, 4.37911446E-03,
     &   4.31145108E-03, 4.26699955E-03, 4.15054516E-03, 4.02831686E-03,
     &   4.01749611E-03, 3.93028715E-03, 3.82347277E-03, 3.78319182E-03,
     &   3.73835308E-03, 3.64848773E-03, 3.56374716E-03, 3.46899630E-03,
     &   3.44552342E-03, 3.38978657E-03, 3.35457302E-03, 3.32163146E-03,
     &   3.27297711E-03, 3.24152746E-03, 3.23647501E-03, 3.22482256E-03/
  
         DATA ((DAS  (I,J), I= 401, 500), J=1,1) /
     &   3.17805900E-03, 3.15179392E-03, 3.11125397E-03, 2.95090381E-03,
     &   2.94209797E-03, 2.87792640E-03, 2.88764478E-03, 2.89690627E-03,
     &   2.91167388E-03, 2.95433192E-03, 3.01559031E-03, 3.08278205E-03,
     &   3.08978457E-03, 3.15934332E-03, 3.16775557E-03, 3.21329412E-03,
     &   3.25503212E-03, 3.42523770E-03, 3.45467733E-03, 3.66213386E-03,
     &   3.67729475E-03, 3.69207929E-03, 3.83162728E-03, 3.94262892E-03,
     &   3.95938496E-03, 4.03370667E-03, 4.03623836E-03, 4.03985400E-03,
     &   4.03279292E-03, 4.02996027E-03, 4.02733730E-03, 4.01452990E-03,
     &   4.01145735E-03, 4.00827268E-03, 4.00417246E-03, 4.00243378E-03,
     &   4.00200707E-03, 3.99280431E-03, 3.99712217E-03, 4.05319785E-03,
     &   4.12670640E-03, 4.15990811E-03, 4.18222868E-03, 4.19330465E-03,
     &   4.31292340E-03, 4.33109576E-03, 4.38434108E-03, 4.42893627E-03,
     &   4.45096507E-03, 4.53619905E-03, 4.63743238E-03, 4.65897696E-03,
     &   4.67351701E-03, 4.74163365E-03, 4.77843158E-03, 4.81626508E-03,
     &   4.96779181E-03, 4.98569583E-03, 5.02525613E-03, 5.03451073E-03,
     &   5.02948802E-03, 5.02620963E-03, 5.01710810E-03, 5.01113478E-03,
     &   5.00480189E-03, 4.99016260E-03, 4.97622131E-03, 4.99651497E-03,
     &   5.05709605E-03, 5.11748943E-03, 5.12380473E-03, 5.14970312E-03,
     &   5.13868688E-03, 5.14770688E-03, 5.15238890E-03, 5.17632033E-03,
     &   5.17779570E-03, 5.18445478E-03, 5.20305633E-03, 5.14748749E-03,
     &   5.12318720E-03, 4.87033447E-03, 4.74235168E-03, 4.64754395E-03,
     &   4.42832856E-03, 4.30983880E-03, 4.30271792E-03, 4.29769318E-03,
     &   4.28504198E-03, 4.28194029E-03, 4.26424191E-03, 4.25852938E-03,
     &   4.23073669E-03, 4.19415041E-03, 4.15940470E-03, 4.14553959E-03,
     &   3.91389103E-03, 3.72245840E-03, 3.76086950E-03, 3.89365033E-03/
  
         DATA ((DAS  (I,J), I= 501, 600), J=1,1) /
     &   4.09127751E-03, 4.18515003E-03, 4.56948942E-03, 4.69433765E-03,
     &   4.73303981E-03, 4.70353953E-03, 4.78736566E-03, 4.64237646E-03,
     &   4.42601361E-03, 4.23666747E-03, 4.01403058E-03, 3.72475901E-03,
     &   3.33406727E-03, 2.77752628E-03, 1.99758321E-03, 8.55013609E-04,
     &   5.56354109E-04, 2.24289268E-04,-1.46646973E-04,-5.64914310E-04,
     &  -1.03901939E-03,-1.58110042E-03,-2.20764371E-03,-2.34514606E-03,
     &  -2.48673563E-03,-2.63295006E-03,-2.78396716E-03,-2.94014886E-03,
     &  -3.10177187E-03,-3.26914645E-03,-3.44232716E-03,-3.62183602E-03,
     &  -3.80802663E-03,-4.74186786E-03,-5.46216240E-03,-5.85867456E-03,
     &  -6.13833303E-03,-6.43129143E-03,-6.73830609E-03,-7.06010636E-03,
     &  -7.39925528E-03,-7.75498289E-03,-7.93997237E-03,-8.12906335E-03,
     &  -8.32408715E-03,-8.52442142E-03,-8.73031407E-03,-8.94198140E-03,
     &  -9.15967826E-03,-9.38366348E-03,-9.61413034E-03,-9.85154389E-03,
     &  -1.00961246E-02,-1.03482010E-02,-1.06080997E-02,-1.08761628E-02,
     &  -1.11529109E-02,-1.14387903E-02,-1.17341575E-02,-1.20394914E-02,
     &  -1.23553469E-02,-1.26821556E-02,-1.30206936E-02,-1.33715142E-02,
     &  -1.37352903E-02,-1.41127543E-02,-1.45047357E-02,-1.49120677E-02,
     &  -1.53357339E-02,-1.57766056E-02,-1.62357927E-02,-1.67144512E-02,
     &  -1.72139762E-02,-1.77357780E-02,-1.82811028E-02,-1.88517649E-02,
     &  -1.94495465E-02,-2.00765102E-02,-2.07349515E-02,-2.14270919E-02,
     &  -2.21556496E-02,-2.29235842E-02,-2.33233316E-02,-2.37341856E-02,
     &  -2.41566251E-02,-2.45912548E-02,-2.50386174E-02,-2.54988477E-02,
     &  -2.59727474E-02,-2.64610972E-02,-2.69645726E-02,-2.74836239E-02,
     &  -2.80192646E-02,-2.85723011E-02,-2.91434297E-02,-2.97339004E-02,
     &  -3.03443069E-02,-3.09757887E-02,-3.16294588E-02,-3.23065100E-02/
  
         DATA ((DAS  (I,J), I= 601, 700), J=1,1) /
     &  -3.30082174E-02,-3.37359605E-02,-3.44912189E-02,-3.52754229E-02,
     &  -3.60904399E-02,-3.69381912E-02,-3.78208094E-02,-3.87402984E-02,
     &  -3.96990218E-02,-4.06995517E-02,-4.17445260E-02,-4.28373187E-02,
     &  -4.39811166E-02,-4.51795990E-02,-4.64369490E-02,-4.77574011E-02,
     &  -4.91458256E-02,-5.06074409E-02,-5.21484261E-02,-5.37755823E-02,
     &  -5.62124200E-02,-5.73145408E-02,-5.92479853E-02,-6.13030681E-02,
     &  -6.34913854E-02,-6.58265328E-02,-6.83238582E-02,-7.10007400E-02,
     &  -7.38775545E-02,-7.69773161E-02,-8.03271893E-02,-8.39587677E-02,
     &  -8.79092128E-02,-9.10633980E-02,-1.00893091E-01,-1.12939818E-01,
     &  -1.28052320E-01,-1.47578863E-01,-1.73806838E-01,-2.11002735E-01,
     &  -2.70238041E-01,-3.61642945E-01,-5.53634660E-01,-1.16945623E+00,
     &  -1.23691511E+00,-1.31257427E+00,-1.39814972E+00,-1.49522747E+00,
     &  -1.60685198E+00,-1.73631257E+00,-1.88822224E+00,-2.06892029E+00,
     &  -2.28735101E+00,-2.55653491E+00,-2.89629395E+00,-3.32352820E+00,
     &  -3.93134154E+00,-4.00217681E+00,-5.58829053E+00,-6.74431228E+00,
     &  -7.67315289E+00,-4.78552980E+00,-9.10579868E+00,-9.30340678E+00,
     &  -1.00787630E+01,-1.05157810E+01,-1.08192606E+01,-1.10994705E+01,
     &  -1.12941569E+01,-1.14508396E+01,-1.15363709E+01,-1.15972730E+01,
     &  -1.16006385E+01,-1.15653082E+01,-1.14729941E+01,-1.13742381E+01,
     &  -1.11606513E+01,-1.09684537E+01,-1.07312503E+01,-1.02381435E+01,
     &  -9.75546601E+00,-9.22489155E+00,-8.64949718E+00,-8.72768332E+00,
     &  -8.14301011E+00,-7.49467514E+00,-6.78421029E+00,-5.60891548E+00,
     &  -4.75926159E+00,-3.43558913E+00,-1.96829282E+00, 1.98116321E+00,
     &   5.61798933E+00, 6.79160809E+00, 7.83277676E+00, 8.44879791E+00,
     &   9.25376253E+00, 9.25159278E+00, 9.77599346E+00, 1.02515893E+01/
  
         DATA ((DAS  (I,J), I= 701, 800), J=1,1) /
     &   1.06758453E+01, 1.09785938E+01, 1.12082033E+01, 1.13253714E+01,
     &   1.14419123E+01, 1.15258481E+01, 1.15807672E+01, 1.16157339E+01,
     &   9.00422058E+00, 9.52689967E+00, 1.14928245E+01, 1.14245338E+01,
     &   1.12553232E+01, 1.11059217E+01, 1.08855282E+01, 1.06493207E+01,
     &   1.03424290E+01, 1.00945659E+01, 9.35951061E+00, 9.17427339E+00,
     &   8.65686830E+00, 8.07379841E+00, 7.32387697E+00, 6.39783160E+00,
     &   5.39063494E+00, 4.44297507E+00, 3.70686030E+00, 3.17570141E+00,
     &   2.77576619E+00, 2.46436028E+00, 2.21529493E+00, 2.01169577E+00,
     &   1.84228352E+00, 1.69903052E+00, 1.57645154E+00, 1.47035652E+00,
     &   1.37764337E+00, 1.29593694E+00, 1.22339255E+00, 1.15855483E+00,
     &   1.10026033E+00, 1.04756834E+00, 9.99709657E-01, 5.26920315E-01,
     &   3.59061239E-01, 2.73256120E-01, 2.22465708E-01, 1.85275842E-01,
     &   1.59679011E-01, 1.40552356E-01, 1.25681116E-01, 1.13780292E-01,
     &   1.04038165E-01, 1.02884903E-01, 9.48459021E-02, 8.80411703E-02,
     &   8.22062458E-02, 7.71452052E-02, 7.27162434E-02, 6.88062451E-02,
     &   6.53289320E-02, 6.22164912E-02, 5.94143159E-02, 5.68780485E-02,
     &   5.45716007E-02, 5.24652166E-02, 5.05338301E-02, 4.87564808E-02,
     &   4.71149423E-02, 4.55948552E-02, 4.41833077E-02, 4.28687301E-02,
     &   4.16408310E-02, 4.04922670E-02, 3.94162315E-02, 3.84033617E-02,
     &   3.74505332E-02, 3.65535029E-02, 3.57049373E-02, 3.49022252E-02,
     &   3.41415975E-02, 3.34196944E-02, 3.27316625E-02, 3.20788483E-02,
     &   3.14569992E-02, 3.08665953E-02, 3.03006298E-02, 2.97596485E-02,
     &   2.92394374E-02, 2.87466362E-02, 2.82690975E-02, 2.78135252E-02,
     &   2.73761133E-02, 2.69584125E-02, 2.65536721E-02, 2.61614488E-02,
     &   2.57863483E-02, 2.54250713E-02, 2.50768700E-02, 2.47408532E-02/
  
         DATA ((DAS  (I,J), I= 801, 900), J=1,1) /
     &   2.44186043E-02, 2.41030056E-02, 2.37997739E-02, 2.35064630E-02,
     &   2.32242079E-02, 2.29478835E-02, 2.26828678E-02, 2.24236034E-02,
     &   2.21733181E-02, 2.19305217E-02, 2.16948869E-02, 2.14661096E-02,
     &   2.12439086E-02, 2.10285918E-02, 2.08183055E-02, 2.06145549E-02,
     &   2.04160648E-02, 2.02229860E-02, 2.00351540E-02, 1.98523393E-02,
     &   1.96743745E-02, 1.95016101E-02, 1.93330696E-02, 1.91689497E-02,
     &   1.90088199E-02, 1.88528024E-02, 1.87008985E-02, 1.85530391E-02,
     &   1.84091757E-02, 1.82692841E-02, 1.81333706E-02, 1.80016024E-02,
     &   1.78741830E-02, 1.77510408E-02, 1.76316975E-02, 1.75190387E-02,
     &   1.72353428E-02, 1.71128348E-02, 1.69927259E-02, 1.68749653E-02,
     &   1.67595400E-02, 1.66464345E-02, 1.65356187E-02, 1.64268952E-02,
     &   1.63201004E-02, 1.62256841E-02, 1.61124653E-02, 1.60115324E-02,
     &   1.59124386E-02, 1.58151821E-02, 1.57197222E-02, 1.56259524E-02,
     &   1.55337467E-02, 1.54430767E-02, 1.53539593E-02, 1.52663551E-02,
     &   1.26372956E-02, 1.07642148E-02, 9.09807449E-03, 7.61987729E-03,
     &   6.64792989E-03, 6.19239656E-03, 5.66099862E-03, 5.01416904E-03,
     &   4.41508263E-03, 3.66433746E-03, 3.37683089E-03, 2.70019251E-03,
     &   2.31001563E-03, 1.87539764E-03, 1.37346332E-03, 7.92014516E-04,
     &   1.13107184E-04,-6.90358126E-04,-1.65744588E-03,-2.84674432E-03,
     &  -4.34996628E-03,-5.07028257E-03,-5.87768661E-03,-6.78943915E-03,
     &  -7.82798120E-03,-9.02249238E-03,-1.04118534E-02,-1.11954363E-02,
     &  -1.20490227E-02,-1.29825734E-02,-1.40080340E-02,-1.51398552E-02,
     &  -1.63956866E-02,-1.77973151E-02,-1.93719536E-02,-2.11540405E-02,
     &  -2.31878180E-02,-2.55310954E-02,-2.82608804E-02,-3.13421320E-02,
     &  -3.33591858E-02,-3.55974765E-02,-3.80957145E-02,-4.09022218E-02/
  
         DATA ((DAS  (I,J), I= 901,1000), J=1,1) /
     &  -4.40781187E-02,-4.77018736E-02,-5.18759490E-02,-5.67367674E-02,
     &  -6.24701210E-02,-6.93358631E-02,-7.77091907E-02,-8.81533233E-02,
     &  -1.01556007E-01,-1.19408364E-01,-1.44445205E-01,-1.82414689E-01,
     &  -2.49576669E-01,-3.68369427E-01,-5.38458907E-01,-6.37096806E-01,
     &  -7.45058648E-01,-1.01132972E+00,-1.10757494E+00,-1.13828381E+00,
     &  -1.25370265E+00,-1.29800514E+00,-1.35720181E+00,-1.39719753E+00,
     &  -1.40619379E+00,-1.43220931E+00,-1.47356024E+00,-1.48774525E+00,
     &  -1.49613637E+00,-1.49912542E+00,-1.49689419E+00,-1.49093549E+00,
     &  -1.47890939E+00,-1.46322374E+00,-1.44399629E+00,-1.41765728E+00,
     &  -1.38653836E+00,-1.34665164E+00,-1.30626674E+00,-1.25988356E+00,
     &  -1.20717910E+00,-1.13738098E+00,-1.07024282E+00,-9.83289829E-01,
     &  -9.01420230E-01,-7.82060473E-01,-6.51800503E-01,-4.57254150E-01,
     &  -2.87676711E-01,-9.23483858E-02, 6.58956482E-03, 1.05528717E-01,
     &   2.03829633E-01, 4.70460203E-01, 6.65035031E-01, 7.95320662E-01,
     &   9.14709373E-01, 9.96602332E-01, 1.08358386E+00, 1.15074720E+00,
     &   1.22057513E+00, 1.27330521E+00, 1.31971383E+00, 1.36012728E+00,
     &   1.40004223E+00, 1.43115208E+00, 1.45751763E+00, 1.47680219E+00,
     &   1.49251164E+00, 1.50456349E+00, 1.51054346E+00, 1.51280044E+00,
     &   1.50983540E+00, 1.50146908E+00, 1.48730920E+00, 1.44579935E+00,
     &   1.41981776E+00, 1.41101924E+00, 1.37104808E+00, 1.31174416E+00,
     &   1.26760051E+00, 1.15269464E+00, 1.12151204E+00, 1.02528847E+00,
     &   9.08953115E-01, 9.72111999E-01, 8.33272833E-01, 7.47869881E-01,
     &   6.96287075E-01, 5.89665604E-01, 5.52486385E-01, 4.75473684E-01,
     &   5.77435839E-01, 2.22518559E-01, 1.74219613E-01, 1.44097839E-01,
     &   1.23301651E-01, 1.08027993E-01, 9.63156727E-02, 8.70400289E-02/
  
         DATA ((DAS  (I,J), I=1001,1100), J=1,1) /
     &   7.95067650E-02, 7.32625521E-02, 6.79969381E-02, 6.34822204E-02,
     &   5.95920630E-02, 5.62531640E-02, 5.32936736E-02, 5.06631312E-02,
     &   4.83133391E-02, 4.62029422E-02, 4.28501944E-02, 3.86657868E-02,
     &   3.64240935E-02, 3.27576497E-02, 2.98845161E-02, 2.70672536E-02,
     &   2.30500937E-02, 1.84657752E-02, 1.56271345E-02, 1.42973994E-02,
     &   1.35987977E-02, 1.30767642E-02, 1.26884316E-02, 1.23720357E-02,
     &   1.21368023E-02, 1.19302729E-02, 1.17558436E-02, 1.16056580E-02,
     &   1.14753222E-02, 1.13883909E-02, 1.12602708E-02, 1.11712998E-02,
     &   1.10924624E-02, 1.10225836E-02, 1.09608960E-02, 1.09063521E-02,
     &   1.08583929E-02, 1.08166817E-02, 1.07808680E-02, 1.07507474E-02,
     &   1.06915285E-02, 1.07069377E-02, 1.06932803E-02, 1.06854254E-02,
     &   1.06824996E-02, 1.06880019E-02, 1.07027651E-02, 1.07257218E-02,
     &   1.07613863E-02, 1.08158343E-02, 1.09048142E-02, 1.11949682E-02,
     &   1.15779881E-02, 1.17607843E-02, 1.19092593E-02, 1.20388914E-02,
     &   1.21559250E-02, 1.22626969E-02, 1.23620323E-02, 1.24528144E-02,
     &   1.25375216E-02, 1.26149996E-02, 1.26887687E-02, 1.27526610E-02,
     &   1.27714946E-02, 1.28262772E-02, 1.28554224E-02, 1.29330790E-02,
     &   1.29816867E-02, 1.30261138E-02, 1.30450348E-02, 1.30539815E-02,
     &   1.30547666E-02, 1.30391975E-02, 1.30133975E-02, 1.29736117E-02,
     &   1.29111215E-02, 1.30707134E-02, 1.27480479E-02, 1.26344999E-02,
     &   1.24941293E-02, 1.23192331E-02, 1.21067521E-02, 1.18416495E-02,
     &   1.14601228E-02, 1.08952664E-02, 1.07264000E-02, 1.02725851E-02,
     &   9.56878094E-03, 9.49364830E-03, 8.99082296E-03, 8.79279769E-03,
     &   8.59779593E-03, 8.44247094E-03, 8.26212439E-03, 7.98682625E-03,
     &   7.87532234E-03, 7.88888403E-03, 7.71130312E-03, 7.69840773E-03/
  
         DATA ((DAS  (I,J), I=1101,1200), J=1,1) /
     &   7.92612424E-03, 7.80424360E-03, 7.93995433E-03, 7.78920729E-03,
     &   7.55897941E-03, 7.78716234E-03, 7.49193022E-03, 6.91064179E-03,
     &   6.74749608E-03, 6.94055203E-03, 6.95294229E-03, 6.91370861E-03,
     &   6.77980289E-03, 6.38289082E-03, 6.19608092E-03, 5.76035687E-03,
     &   5.40392434E-03, 5.24347495E-03, 5.50784922E-03, 5.95533453E-03,
     &   6.89561212E-03, 7.23126592E-03, 7.64724226E-03, 7.91038725E-03,
     &   8.20103623E-03, 8.28102900E-03, 8.26906452E-03, 8.67256994E-03,
     &   9.04386985E-03, 8.95696569E-03, 8.91542964E-03, 8.97458243E-03,
     &   9.02154973E-03, 9.05887713E-03, 9.04271831E-03, 9.06517260E-03,
     &   9.05779320E-03, 8.94342799E-03, 8.89187089E-03, 8.91544611E-03,
     &   8.99145900E-03, 9.08837469E-03, 9.33650834E-03, 9.45164660E-03,
     &   9.45848316E-03, 9.45401879E-03, 9.36264789E-03, 9.11939892E-03,
     &   9.10549558E-03, 9.33221129E-03, 9.59594918E-03, 9.83178425E-03,
     &   9.84631780E-03, 9.94299163E-03, 1.00903246E-02, 1.02405855E-02,
     &   1.05074942E-02, 1.07868915E-02, 1.09043786E-02, 1.10209834E-02,
     &   1.12113200E-02, 1.13348544E-02, 1.12302029E-02, 1.10077424E-02,
     &   1.09331111E-02, 1.10627690E-02, 1.10152437E-02, 1.06317897E-02,
     &   1.05037304E-02, 1.05834575E-02, 1.05830029E-02, 1.01244908E-02,
     &   9.66403899E-03, 9.51615918E-03, 9.44661634E-03, 9.38954557E-03,
     &   9.33790704E-03, 9.06782659E-03, 8.93633073E-03, 8.98134431E-03,
     &   8.96531443E-03, 8.77126123E-03, 8.39075624E-03, 8.34008710E-03,
     &   8.23646112E-03, 8.30650329E-03, 8.33555108E-03, 8.39447864E-03,
     &   8.44985743E-03, 8.46875606E-03, 8.48364356E-03, 8.41499950E-03,
     &   8.31000783E-03, 8.22978532E-03, 8.59279914E-03, 8.88703760E-03,
     &   8.87447419E-03, 8.90519381E-03, 8.91456053E-03, 8.88923006E-03/
  
         DATA ((DAS  (I,J), I=1201,1300), J=1,1) /
     &   9.19320255E-03, 9.62097899E-03, 9.73769657E-03, 9.72757607E-03,
     &   9.77398030E-03, 9.80841979E-03, 9.86792337E-03, 9.92952985E-03,
     &   9.89592742E-03, 9.85616472E-03, 9.83825017E-03, 9.56951444E-03,
     &   9.51530712E-03, 9.60869687E-03, 9.63759804E-03, 9.52190813E-03,
     &   9.44951570E-03, 9.44070078E-03, 9.44473683E-03, 9.45607391E-03,
     &   9.68253958E-03, 9.92327225E-03, 9.92772969E-03, 9.93246571E-03,
     &   9.91189687E-03, 9.72103871E-03, 9.95679285E-03, 1.01294566E-02,
     &   1.03646199E-02, 1.04579168E-02, 1.05422942E-02, 1.06930990E-02,
     &   1.08276961E-02, 1.08905922E-02, 1.09511086E-02, 1.10095434E-02,
     &   1.10661322E-02, 1.11210648E-02, 1.11744968E-02, 1.12265569E-02,
     &   1.12773534E-02, 1.13269779E-02, 1.13755088E-02, 1.14230137E-02,
     &   1.14695509E-02, 1.15151715E-02, 1.15599200E-02, 1.16038360E-02,
     &   1.16469536E-02, 1.16893035E-02, 1.17309127E-02, 1.17718048E-02,
     &   1.18120010E-02, 1.18515199E-02, 1.18903776E-02, 1.19285884E-02,
     &   1.19661646E-02, 1.20031166E-02, 1.20394531E-02, 1.20751815E-02,
     &   1.21103073E-02, 1.21448346E-02, 1.21787661E-02, 1.22121029E-02,
     &   1.22448446E-02, 1.22769894E-02, 1.23085340E-02, 1.23333339E-02,
     &   1.23697994E-02, 1.24285782E-02, 1.24847757E-02, 1.25382583E-02,
     &   1.25888450E-02, 1.26362951E-02, 1.26802878E-02, 1.27203958E-02,
     &   1.27560435E-02, 1.27864424E-02, 1.28104868E-02, 1.28265769E-02,
     &   1.28323155E-02, 1.28239047E-02, 1.27949475E-02, 1.27905407E-02,
     &   1.27336593E-02, 1.26154667E-02, 1.23785299E-02, 1.19746259E-02,
     &   1.18071068E-02, 1.11135092E-02, 9.83100561E-03, 9.24054259E-03,
     &   8.41254289E-03, 7.86003393E-03, 7.16912718E-03, 6.47814714E-03,
     &   5.61459790E-03, 5.09666756E-03, 4.50500256E-03, 3.82271993E-03/
  
         DATA ((DAS  (I,J), I=1301,1400), J=1,1) /
     &   3.02736284E-03, 2.08839265E-03, 9.63190764E-04, 3.12582904E-04,
     &  -4.09581188E-04,-1.21577697E-03,-2.12156803E-03,-3.14663417E-03,
     &  -4.31625199E-03,-5.66350279E-03,-7.23276989E-03,-9.08624540E-03,
     &  -9.86196147E-03,-1.03511556E-02,-1.08653584E-02,-1.14200359E-02,
     &  -1.19529236E-02,-1.25058548E-02,-1.12188810E-02,-1.37123463E-02,
     &  -1.43624573E-02,-1.50456214E-02,-1.57639806E-02,-1.65200534E-02,
     &  -1.73167083E-02,-1.81571722E-02,-1.90449648E-02,-1.99844304E-02,
     &  -2.09798265E-02,-2.20363631E-02,-2.31598059E-02,-2.43566776E-02,
     &  -2.56343836E-02,-2.70013652E-02,-2.84672859E-02,-3.00432452E-02,
     &  -3.17421339E-02,-3.35788376E-02,-3.55708235E-02,-3.77386232E-02,
     &  -4.01065555E-02,-4.27036385E-02,-4.55647704E-02,-4.87322220E-02,
     &  -5.22575188E-02,-5.62075496E-02,-6.06603129E-02,-6.57198650E-02,
     &  -7.15190829E-02,-7.82329279E-02,-8.60961935E-02,-9.54315517E-02,
     &  -1.06694883E-01,-1.20551944E-01,-1.38014737E-01,-1.36979242E-05,
     &  -1.91362686E-01,-2.35101353E-01,-3.02535240E-01,-4.20020568E-01,
     &  -6.75674241E-01,-8.34182862E-01,-1.60142397E+00,-2.23331817E+00,
     &  -2.69215868E+00,-3.04601765E+00,-3.26660671E-03,-3.59209862E+00,
     &  -3.82888779E+00,-3.99368353E+00,-4.16098076E+00,-4.32581645E+00,
     &  -4.40528061E+00,-4.51664602E+00,-4.58409660E+00,-4.66769864E+00,
     &  -4.68785229E+00,-4.72486021E+00,-4.73307606E+00,-4.72305354E+00,
     &  -4.70777610E+00,-4.68471861E+00,-5.23640033E-02,-5.10993419E-02,
     &  -4.49767362E+00,-4.33567719E+00,-4.23510922E+00,-4.12174014E+00,
     &  -3.99103322E+00,-3.68025605E+00,-3.49905972E+00,-3.30269690E+00,
     &  -2.85634812E+00,-2.60927202E+00,-7.92770966E-01,-1.46828594E+00,
     &   1.71723081E-01, 1.47458700E+00, 2.07471965E+00, 2.61764416E+00/
  
         DATA ((DAS  (I,J), I=1401,1500), J=1,1) /
     &   3.10009534E+00, 3.31697172E+00, 3.51441024E+00, 3.69480917E+00,
     &   4.00710047E+00, 4.13832588E+00, 4.25552641E+00, 4.36346908E+00,
     &   4.52192516E+00, 4.58369127E+00, 4.63613713E+00, 4.71000657E+00,
     &   4.73385823E+00, 4.75821863E+00, 4.76041041E+00, 4.74780157E+00,
     &   4.71660354E+00, 4.73692686E+00, 4.69524664E+00, 4.54727478E+00,
     &   4.43858020E+00, 4.35878628E+00, 4.19094692E+00, 4.02377854E+00,
     &   3.85887990E+00, 3.62198211E+00, 3.36709897E+00, 3.07529474E+00,
     &   2.72042195E+00, 2.26180083E+00, 1.61986505E+00, 8.90286765E-01,
     &   5.15081076E-01, 3.64483440E-01, 2.83402359E-01, 2.32751829E-01,
     &   1.98113169E-01, 1.72976984E-01, 1.53801801E-01, 1.38775296E-01,
     &   1.26660644E-01, 1.16686345E-01, 1.08331368E-01, 1.01231112E-01,
     &   9.51227695E-02, 8.98121583E-02, 3.82164250E-02, 8.10334785E-02,
     &   7.73591969E-02, 7.40698931E-02, 7.11035051E-02, 6.84156280E-02,
     &   6.59689062E-02, 6.37323198E-02, 6.16799598E-02, 5.97900103E-02,
     &   5.80439729E-02, 1.75568349E-02, 5.49227125E-02, 5.35222285E-02,
     &   5.22144500E-02, 5.09905207E-02, 4.98426857E-02, 4.87641277E-02,
     &   4.77488327E-02, 4.67914808E-02, 4.58873563E-02, 4.50322763E-02,
     &   4.42225401E-02, 4.34548997E-02, 4.27265796E-02, 4.20354009E-02,
     &   4.13803117E-02, 4.07645303E-02, 4.01850632E-02, 3.95654459E-02,
     &   3.89976805E-02, 3.84592721E-02, 3.79456561E-02, 3.74543744E-02,
     &   3.69836547E-02, 3.34525515E-02, 3.13014651E-02, 2.92510609E-02,
     &   2.69870622E-02, 2.49284328E-02, 2.31179124E-02, 2.26743605E-02,
     &   2.10195369E-02, 1.85318810E-02, 1.69420028E-02, 1.68680321E-02,
     &   1.67095394E-02, 1.65339647E-02, 1.61998598E-02, 1.59126345E-02,
     &   1.49980867E-02, 1.41906389E-02, 1.37165668E-02, 1.30843517E-02/
  
         DATA ((DAS  (I,J), I=1501,1600), J=1,1) /
     &   1.23370079E-02, 1.14764771E-02, 1.03710980E-02, 8.85716983E-03,
     &   5.07566400E-03, 4.35909830E-03, 3.51537751E-03, 3.27090174E-03,
     &   2.71368025E-03, 2.09457677E-03, 1.40266341E-03, 6.24276775E-04,
     &  -2.57895502E-04,-1.26612064E-03,-2.42953112E-03,-3.78699374E-03,
     &  -5.39155032E-03,-7.31753787E-03,-9.67249015E-03,-1.26180563E-02,
     &  -1.64091238E-02,-2.14730908E-02,-2.85898008E-02,-3.94212066E-02,
     &  -5.70876591E-02,-9.23913292E-02,-1.39817079E-01,-1.47778957E-01,
     &  -1.56611473E-01,-1.66465542E-01,-1.77529056E-01,-1.90038745E-01,
     &  -2.04298858E-01,-2.20701911E-01,-2.39770947E-01,-2.62211896E-01,
     &  -2.89004059E-01,-3.21545691E-01,-3.61904226E-01,-4.13269315E-01,
     &  -4.80830668E-01,-5.73616603E-01,-7.08823121E-01,-9.23485313E-01,
     &  -1.31304722E+00,-2.19436738E+00,-2.50320340E+00,-2.63758518E+00,
     &  -3.01580858E+00,-3.24600923E+00,-3.41272570E+00,-2.96104998E+00,
     &  -2.59843979E+00,-3.91667461E+00,-3.78500682E+00,-2.56050921E-01,
     &  -1.35728479E+00,-1.32564086E+00,-1.30985901E+00,-3.13559803E+00,
     &  -3.11685030E+00,-3.10267022E+00,-3.08838885E+00,-2.94019450E+00,
     &  -2.70100267E+00,-2.41475970E+00,-2.14201682E+00,-2.25755959E+00,
     &  -1.91434569E+00,-1.57519602E+00,-1.37598715E+00,-1.18516249E+00,
     &  -9.72322525E-01,-6.65319625E-01,-4.41168517E-01, 4.02950358E-02,
     &   4.03982758E-02, 4.04326645E-02, 4.04498543E-02, 4.04670409E-02,
     &   4.04842245E-02, 4.05014049E-02, 4.06387373E-02, 4.08101235E-02,
     &   4.09811984E-02, 1.69492310E+00, 2.04676952E+00, 2.64353948E+00,
     &   2.87447922E+00, 3.09097548E+00, 3.25211802E+00, 3.16674833E+00,
     &   3.36945661E+00, 3.55720908E+00, 3.61344663E+00, 3.64524873E+00,
     &   3.65526623E+00, 3.65460789E+00, 3.64310125E+00, 3.62494372E+00/
  
         DATA ((DAS  (I,J), I=1601,1700), J=1,1) /
     &   2.63601336E+00, 3.52911000E+00, 3.48548962E+00, 3.44236732E+00,
     &   3.33415552E+00, 3.27813563E+00, 3.18715153E+00, 3.04168987E+00,
     &   2.53085652E+00, 2.13281973E+00, 1.83088114E+00, 1.59922415E+00,
     &   1.41736695E+00, 1.24513024E+00, 1.08110388E+00, 8.98650950E-01,
     &   8.37137465E-01, 7.83495198E-01, 7.36235066E-01, 6.66368004E-01,
     &   5.53932469E-01, 4.74411703E-01, 4.15238133E-01, 3.69506751E-01,
     &   3.33108921E-01, 3.03458968E-01, 2.78841132E-01, 2.58075556E-01,
     &   2.40325409E-01, 2.24978066E-01, 2.11577028E-01, 1.99774715E-01,
     &   1.89301017E-01, 1.79943116E-01, 1.71533539E-01, 1.19949156E-01,
     &   8.45926868E-02, 6.69215865E-02, 4.92790311E-02, 4.03160822E-02,
     &   3.49965358E-02, 3.14580010E-02, 2.89327645E-02, 2.70395928E-02,
     &   2.55673256E-02, 2.43894794E-02, 2.37212227E-02, 2.29413805E-02,
     &   2.22748859E-02, 2.16986485E-02, 2.11954511E-02, 2.07521938E-02,
     &   1.91453229E-02, 1.81360472E-02, 1.74416897E-02, 1.69333608E-02,
     &   1.65434178E-02, 1.62300221E-02, 1.59803238E-02, 1.57725802E-02,
     &   1.40006799E-02, 1.35083646E-02, 1.33843818E-02, 1.32408988E-02,
     &   1.30701697E-02, 1.28583201E-02, 1.27306117E-02, 1.25835976E-02,
     &   1.22056692E-02, 1.16371327E-02, 1.06492178E-02, 8.40920803E-03,
     &   6.27237160E-03, 5.72110896E-03, 5.08535645E-03, 4.34398778E-03,
     &   3.46814993E-03, 2.41741911E-03, 1.13340444E-03,-4.71653118E-04,
     &  -2.53598038E-03,-5.29086092E-03,-9.15797421E-03,-1.49824425E-02,
     &  -2.45678350E-02,-2.62390367E-02,-2.80622367E-02,-3.00595429E-02,
     &  -3.22573243E-02,-3.46940544E-02,-3.73837695E-02,-4.04000134E-02,
     &  -4.37929905E-02,-4.76388256E-02,-5.20340992E-02,-5.71049672E-02,
     &  -6.30215041E-02,-7.00135615E-02,-7.84027831E-02,-8.86558148E-02/
  
         DATA ((DAS  (I,J), I=1701,1800), J=1,1) /
     &  -9.18570903E-02,-1.01472800E-01,-1.17946483E-01,-1.39908883E-01,
     &  -1.70645707E-01,-2.16719573E-01,-2.93410602E-01,-4.46294268E-01,
     &  -8.98166437E-01,-1.07042885E+00,-1.51552096E+00,-1.73385046E+00,
     &  -2.13295608E+00,-2.37591459E+00,-2.59249355E+00,-2.78878755E+00,
     &  -2.95763310E+00,-3.19572809E+00,-3.26682243E+00, 3.18141502E-02,
     &  -3.54226720E+00,-3.65702827E+00,-3.75453314E+00,-3.83526103E+00,
     &  -2.66023491E+00,-3.97023487E+00,-4.01971072E+00,-4.06502796E+00,
     &  -4.10449129E+00,-4.13505638E+00,-4.20425974E+00,-4.14152480E+00,
     &  -4.11463919E+00,-3.85478532E+00,-3.63523755E+00, 8.67813755E-02,
     &  -3.82401715E+00,-3.65586548E+00,-3.44621053E+00,-3.14821100E+00,
     &  -2.79404472E+00,-2.32562384E+00,-1.77966197E+00, 8.19882232E-01,
     &   1.38797472E-02, 8.25600519E-01, 1.80752209E+00, 2.35545203E+00,
     &   2.82938490E+00, 3.18704374E+00, 2.69140984E+00, 2.98589144E+00,
     &   3.26986640E+00, 3.53960647E+00, 3.75055375E+00, 3.98334432E+00,
     &   4.18986205E+00, 1.58915079E+00, 1.72960714E+00, 1.87586869E+00,
     &   2.98545357E-01, 2.24182727E+00, 2.43852424E+00, 2.64447277E+00,
     &   2.89065726E+00, 3.17965940E+00, 3.47780356E+00, 3.81251742E+00,
     &   3.61246692E+00, 3.33976303E+00, 3.06515102E+00, 2.82079713E+00,
     &   2.63772386E+00, 1.03264281E+00, 2.62856715E+00, 2.39988339E+00,
     &   2.16233472E+00, 1.88130591E+00, 1.54333179E+00, 1.09824450E+00,
     &   9.42386191E-01, 9.13153564E-01, 6.14190146E-01, 7.76729855E-01,
     &   6.86488403E-01, 4.74085280E-01, 3.21226847E-01, 2.44532655E-01,
     &   1.98457769E-01, 1.67724897E-01, 1.45762310E-01, 1.29286898E-01,
     &   1.16472114E-01, 1.06218050E-01, 9.78277798E-02, 9.08362323E-02,
     &   8.49199689E-02, 7.98472933E-02, 7.54524635E-02, 7.16061925E-02/
  
         DATA ((DAS  (I,J), I=1801,1900), J=1,1) /
     &   5.23745452E-02, 4.27585405E-02, 3.69899746E-02, 3.31499849E-02,
     &   3.03948755E-02, 2.83177674E-02, 2.67082119E-02, 2.43696612E-02,
     &   2.27508066E-02, 2.21145518E-02, 2.15629086E-02, 1.95766640E-02,
     &   1.81176586E-02, 1.72401338E-02, 1.66511718E-02, 1.62261905E-02,
     &   1.59031326E-02, 1.56475252E-02, 1.54385486E-02, 1.52623371E-02,
     &   1.49727134E-02, 1.49855670E-02, 1.48748212E-02, 1.47744440E-02,
     &   1.46825679E-02, 1.45977380E-02, 1.45181069E-02, 1.44423843E-02,
     &   1.43692019E-02, 1.42969430E-02, 1.42232210E-02, 1.41416818E-02,
     &   1.40872819E-02, 1.32440387E-02, 1.28330016E-02, 1.25350084E-02,
     &   1.22985520E-02, 1.21032293E-02, 1.20810398E-02, 1.18097737E-02,
     &   1.16830302E-02, 1.15839715E-02, 1.15007069E-02, 1.14316889E-02,
     &   1.14062146E-02, 1.13343420E-02, 1.13044287E-02, 1.12861091E-02,
     &   1.12793848E-02, 1.12837331E-02, 1.12988733E-02, 1.13252168E-02,
     &   1.13552313E-02, 1.14089862E-02, 1.14687883E-02, 1.15376371E-02,
     &   1.16203578E-02, 1.17153270E-02, 1.18250300E-02, 1.19501236E-02,
     &   1.20910750E-02, 1.22481116E-02, 1.24335461E-02, 1.26457889E-02,
     &   1.28908958E-02, 1.31963981E-02, 1.36001920E-02, 1.45870619E-02,
     &   1.55199489E-02, 1.58719370E-02, 1.61269938E-02, 1.63244399E-02,
     &   1.64899965E-02, 1.66302241E-02, 1.67454320E-02, 1.68457069E-02,
     &   1.69318018E-02, 1.70048259E-02, 1.70701259E-02, 1.71149877E-02,
     &   1.71540486E-02, 1.71849418E-02, 1.72057806E-02, 1.72182693E-02,
     &   1.72220133E-02, 1.72170508E-02, 1.72039449E-02, 1.71813124E-02,
     &   1.71493920E-02, 1.71084414E-02, 1.70577411E-02, 1.69795285E-02,
     &   1.68917567E-02, 1.67935556E-02, 1.67400314E-02, 1.66257138E-02,
     &   1.64934580E-02, 1.63363289E-02, 1.61547826E-02, 1.59338423E-02/
  
         DATA ((DAS  (I,J), I=1901,2000), J=1,1) /
     &   1.56561209E-02, 1.52803503E-02, 1.46245828E-02, 1.45894335E-02,
     &   1.44003060E-02, 1.43478190E-02, 1.42330265E-02, 1.41806224E-02,
     &   1.41386432E-02, 1.41038266E-02, 1.40745029E-02, 1.40494847E-02,
     &   1.40155791E-02, 1.40096570E-02, 1.39939403E-02, 1.39806798E-02,
     &   1.39679063E-02, 1.39603091E-02, 1.39531462E-02, 1.39477647E-02,
     &   1.39440820E-02, 1.39420664E-02, 1.39194467E-02, 1.39224177E-02,
     &   1.39459429E-02, 1.39503075E-02, 1.39573374E-02, 1.39659963E-02,
     &   1.39765847E-02, 1.39893300E-02, 1.40044615E-02, 1.40223372E-02,
     &   1.40432281E-02, 1.40679538E-02, 1.40972531E-02, 1.41298571E-02,
     &   1.41776044E-02, 1.42391216E-02, 1.43974943E-02, 1.45170616E-02,
     &   1.45947359E-02, 1.46363818E-02, 1.46701567E-02, 1.46986948E-02,
     &   1.47234737E-02, 1.47451492E-02, 1.47642864E-02, 1.47813542E-02,
     &   1.47964767E-02, 1.48099377E-02, 1.48218928E-02, 1.48324098E-02,
     &   1.48416638E-02, 1.48497320E-02, 1.48565611E-02, 1.48623049E-02,
     &   1.48669968E-02, 1.48706018E-02, 1.48760805E-02, 1.48750220E-02,
     &   1.48754698E-02, 1.48749118E-02, 1.48734536E-02, 1.48709754E-02,
     &   1.48674411E-02, 1.48628361E-02, 1.48570297E-02, 1.48501978E-02,
     &   1.48421676E-02, 1.48435849E-02, 1.48345561E-02, 1.48237155E-02,
     &   1.48105978E-02, 1.47816668E-02, 1.46646944E-02, 1.46903381E-02,
     &   1.46874692E-02, 1.46850092E-02, 1.46829578E-02, 1.46813119E-02,
     &   1.46800651E-02, 1.46792023E-02, 1.46787180E-02, 1.46786064E-02,
     &   1.46766222E-02, 1.46770263E-02, 1.46784317E-02, 1.46343850E-02,
     &   1.46363420E-02, 1.46388241E-02, 1.46861319E-02, 1.46891365E-02,
     &   1.46926008E-02, 1.46965460E-02, 1.47010449E-02, 1.47060200E-02,
     &   1.47115883E-02, 1.47177786E-02, 1.47246184E-02, 1.47322112E-02/
  
         DATA ((DAS  (I,J), I=2001,2100), J=1,1) /
     &   1.47405141E-02, 1.47498195E-02, 1.47602013E-02, 1.47718337E-02,
     &   1.47849107E-02, 1.47999995E-02, 1.48177634E-02, 1.48396170E-02,
     &   1.48690076E-02, 1.49433622E-02, 1.50178210E-02, 1.50480960E-02,
     &   1.50826105E-02, 1.51042826E-02, 1.51229848E-02, 1.51134340E-02,
     &   1.51262889E-02, 1.51379932E-02, 1.51487161E-02, 1.51585172E-02,
     &   1.51677246E-02, 1.51762755E-02, 1.51842866E-02, 1.51917838E-02,
     &   1.51987633E-02, 1.52053591E-02, 1.52114832E-02, 1.52171421E-02,
     &   1.52228419E-02, 1.52276922E-02, 1.52319981E-02, 1.52362591E-02,
     &   1.52401853E-02, 1.52438182E-02, 1.52471560E-02, 1.52501983E-02,
     &   1.52529440E-02, 1.52554176E-02, 1.52575965E-02, 1.52595011E-02,
     &   1.52623070E-02, 1.52633312E-02, 1.52640896E-02, 1.52645444E-02,
     &   1.52646289E-02, 1.52606662E-02, 1.52197873E-02, 1.51633707E-02,
     &   1.51603289E-02, 1.51352901E-02, 1.51162542E-02, 1.50973330E-02,
     &   1.50820245E-02, 1.50669146E-02, 1.50527970E-02, 1.50371474E-02,
     &   1.50205605E-02, 1.50034916E-02, 1.49820643E-02, 1.49606369E-02,
     &   1.49844300E-02, 1.50073137E-02, 1.50277562E-02, 1.50479512E-02,
     &   1.50663677E-02, 1.50848559E-02, 1.51019330E-02, 1.51192534E-02,
     &   1.51361103E-02, 1.51518612E-02, 1.51679887E-02, 1.51831295E-02,
     &   1.51986944E-02, 1.52133565E-02, 1.52284726E-02, 1.52427474E-02,
     &   1.52574956E-02, 1.52714488E-02, 1.52858881E-02, 1.52995683E-02,
     &   1.53131290E-02, 1.53271868E-02, 1.53405262E-02, 1.53543665E-02,
     &   1.53675099E-02, 1.53811563E-02, 1.53941236E-02, 1.54075949E-02,
     &   1.54204024E-02, 1.54337138E-02, 1.54463747E-02, 1.54595390E-02,
     &   1.54726322E-02, 1.54850920E-02, 1.54980533E-02, 1.55103911E-02,
     &   1.55232287E-02, 1.55354517E-02, 1.55481728E-02, 1.55602874E-02/
  
         DATA ((DAS  (I,J), I=2101,2174), J=1,1) /
     &   1.55728982E-02, 1.55849099E-02, 1.55974158E-02, 1.56093295E-02,
     &   1.56211970E-02, 1.56335555E-02, 1.56453314E-02, 1.56575962E-02,
     &   1.56692842E-02, 1.56791431E-02, 1.56908013E-02, 1.57029508E-02,
     &   1.57145357E-02, 1.57266099E-02, 1.57381241E-02, 1.57501255E-02,
     &   1.57620906E-02, 1.57735020E-02, 1.57853976E-02, 1.57967435E-02,
     &   1.58085715E-02, 1.58198536E-02, 1.58316159E-02, 1.58429341E-02,
     &   1.58547258E-02, 1.58658868E-02, 1.58775235E-02, 1.58886245E-02,
     &   1.59001991E-02, 1.59112414E-02, 1.59222554E-02, 1.59337402E-02,
     &   1.59446973E-02, 1.59561231E-02, 1.59670245E-02, 1.59783926E-02,
     &   1.59892392E-02, 1.60367937E-02, 1.60475735E-02, 1.60587943E-02,
     &   1.60694807E-02, 1.60806044E-02, 1.61645568E-02, 1.62815779E-02,
     &   1.63000385E-02, 1.71305189E-02, 1.75084497E-02, 1.85079936E-02,
     &   1.91816611E-02, 1.91002973E-02, 1.98583157E-02, 1.98607228E-02,
     &   2.04306823E-02, 2.04524781E-02, 2.04655365E-02, 2.07489667E-02,
     &   2.09378269E-02, 2.09717275E-02, 2.10271393E-02, 2.11092799E-02,
     &   2.11672445E-02, 2.12036547E-02, 2.15571208E-02, 2.15750622E-02,
     &   2.16830487E-02, 2.17023666E-02, 2.19991587E-02, 2.18206589E-02,
     &   2.18802599E-02, 2.19062850E-02, 2.19481021E-02, 2.20563364E-02,
     &   2.21023502E-02, 2.24069326E-02/
  
         DATA ((DAS  (I,J), I=   1, 100), J=2,2) /
     &   5.39299266E-06, 5.72162351E-06, 6.06383577E-06, 6.43386830E-06,
     &   6.85034397E-06, 7.01934626E-06, 8.04745494E-06, 8.23689127E-06,
     &   8.69799636E-06, 9.20636787E-06, 9.09095103E-06, 9.05066250E-06,
     &   9.03675271E-06, 9.18907119E-06, 9.41841656E-06, 9.70395757E-06,
     &   9.97157831E-06, 1.02039508E-05, 1.03928510E-05, 1.06885425E-05,
     &   1.09490963E-05, 1.12433507E-05, 1.14660674E-05, 1.17550252E-05,
     &   1.20173386E-05, 1.23244824E-05, 1.27393495E-05, 1.29552691E-05,
     &   1.30570163E-05, 1.33660926E-05, 1.36882701E-05, 1.39748246E-05,
     &   1.43207140E-05, 1.46709223E-05, 1.50334434E-05, 1.54120937E-05,
     &   1.58032809E-05, 1.62434471E-05, 1.66789535E-05, 1.71276496E-05,
     &   1.75134354E-05, 1.80226442E-05, 1.85740778E-05, 1.90412133E-05,
     &   1.97493871E-05, 2.06751123E-05, 2.14205831E-05, 2.19503371E-05,
     &   2.22169143E-05, 2.24786294E-05, 2.59304640E-05, 2.99653333E-05,
     &   3.55726965E-05, 4.21553056E-05, 4.65465925E-05, 4.75507330E-05,
     &   4.87493128E-05, 4.96164177E-05, 5.13265786E-05, 5.17856063E-05,
     &   5.28009983E-05, 5.32049264E-05, 5.42045570E-05, 5.59413867E-05,
     &   5.80205252E-05, 6.03018059E-05, 6.20216864E-05, 6.39565441E-05,
     &   6.47850885E-05, 6.63160469E-05, 6.77972454E-05, 6.74261339E-05,
     &   6.94491556E-05, 7.23322384E-05, 7.58133795E-05, 8.08262229E-05,
     &   8.69053058E-05, 9.75953954E-05, 1.12106509E-04, 1.32647534E-04,
     &   1.52297095E-04, 1.58982792E-04, 1.70317010E-04, 1.81931302E-04,
     &   2.04804319E-04, 2.44071588E-04, 2.75744838E-04, 3.01279897E-04,
     &   3.23749168E-04, 3.43319657E-04, 3.60206934E-04, 3.75300104E-04,
     &   3.88464574E-04, 3.98179503E-04, 4.07557752E-04, 4.20937951E-04,
     &   4.26548256E-04, 4.29298866E-04, 4.31599195E-04, 4.32759557E-04/
  
         DATA ((DAS  (I,J), I= 101, 200), J=2,2) /
     &   4.32602868E-04, 4.31293623E-04, 4.28859688E-04, 4.25240620E-04,
     &   4.20733140E-04, 4.14310238E-04, 4.06350372E-04, 3.96753805E-04,
     &   3.85291346E-04, 3.71882462E-04, 3.56264152E-04, 3.37802379E-04,
     &   3.16190295E-04, 2.90582166E-04, 2.59805758E-04, 2.21255866E-04,
     &   1.69487541E-04, 7.49741313E-05, 1.55922734E-04, 2.07563295E-04,
     &   2.46881271E-04, 2.78676412E-04, 3.05395570E-04, 3.28274738E-04,
     &   3.47936024E-04, 3.64767535E-04, 3.79285268E-04, 3.91864689E-04,
     &   4.02578194E-04, 4.11587795E-04, 4.19035837E-04, 4.43825777E-04,
     &   4.29780970E-04, 4.33856444E-04, 4.36264200E-04, 4.37249745E-04,
     &   4.36199313E-04, 4.33413584E-04, 4.31060767E-04, 4.24343650E-04,
     &   4.20527948E-04, 4.13982918E-04, 4.06470449E-04, 3.97796117E-04,
     &   3.88114695E-04, 3.75196693E-04, 3.60346018E-04, 3.43439032E-04,
     &   3.23869943E-04, 3.01293928E-04, 2.74805678E-04, 2.43832288E-04,
     &   2.36543157E-04, 2.15542114E-04, 1.96544688E-04, 1.83078840E-04,
     &   1.72650338E-04, 1.65153218E-04, 1.44062163E-04, 1.31393119E-04,
     &   1.22814770E-04, 1.11457791E-04, 1.07173058E-04, 1.02881685E-04,
     &   9.90654318E-05, 9.54947733E-05, 9.23612943E-05, 9.08565995E-05,
     &   8.37232735E-05, 7.88937487E-05, 7.55614794E-05, 7.35165401E-05,
     &   7.22534689E-05, 7.12549380E-05, 7.06427898E-05, 7.03382567E-05,
     &   7.01436726E-05, 7.03525679E-05, 7.10015542E-05, 7.15123505E-05,
     &   7.27659143E-05, 7.34501799E-05, 7.53007898E-05, 7.71089130E-05,
     &   7.98741341E-05, 8.25944832E-05, 8.65175538E-05, 9.01713957E-05,
     &   9.45434492E-05, 1.00567475E-04, 1.04104845E-04, 1.06617030E-04,
     &   1.12283963E-04, 1.15228536E-04, 1.18178694E-04, 1.21764391E-04,
     &   1.25910722E-04, 1.30769579E-04, 1.35872047E-04, 1.46298547E-04/
  
         DATA ((DAS  (I,J), I= 201, 300), J=2,2) /
     &   1.58121379E-04, 1.65393501E-04, 1.68196870E-04, 1.71138816E-04,
     &   1.74222671E-04, 1.77454565E-04, 1.80841942E-04, 1.84381357E-04,
     &   1.88069536E-04, 1.91937194E-04, 1.95993372E-04, 2.00249127E-04,
     &   2.04714870E-04, 2.09380681E-04, 2.14277626E-04, 2.19448913E-04,
     &   2.24906076E-04, 2.30674480E-04, 2.36795275E-04, 2.43312046E-04,
     &   2.50276472E-04, 2.57750715E-04, 2.65816031E-04, 2.74575888E-04,
     &   2.84161607E-04, 2.94773465E-04, 3.06678475E-04, 3.20399521E-04,
     &   3.37084759E-04, 3.61449899E-04, 3.85227035E-04, 4.17968966E-04,
     &   3.78417813E-04, 3.78897993E-04, 3.87750320E-04, 3.99361347E-04,
     &   4.13745629E-04, 4.29665684E-04, 4.48819219E-04, 4.67166757E-04,
     &   4.89037712E-04, 5.13302500E-04, 5.35534020E-04, 6.01226436E-04,
     &   6.04100309E-04, 6.42042832E-04, 6.84964926E-04, 7.33793441E-04,
     &   7.89667764E-04, 8.53980014E-04, 9.28334091E-04, 1.01465695E-03,
     &   1.11467646E-03, 1.21162105E-03, 1.26464877E-03, 1.30072861E-03,
     &   1.35060511E-03, 1.37466156E-03, 1.36005590E-03, 1.44884287E-03,
     &   1.48428697E-03, 1.51653170E-03, 1.54508556E-03, 1.56773942E-03,
     &   1.58199087E-03, 1.58470358E-03, 1.55951518E-03, 1.51449914E-03,
     &   1.48246807E-03, 1.39515999E-03, 1.27271649E-03, 1.11129784E-03,
     &   8.47331691E-04, 6.71511181E-04, 4.04719301E-04, 1.38470413E-04,
     &   2.13336710E-04, 4.87048083E-04, 7.46136520E-04, 9.73393760E-04,
     &   1.16233093E-03, 1.31186067E-03, 1.42308378E-03, 1.50101183E-03,
     &   1.55071407E-03, 1.57729140E-03, 1.58566938E-03, 1.58007603E-03,
     &   1.56404804E-03, 1.54046660E-03, 1.49625581E-03, 1.48385347E-03,
     &   1.42120511E-03, 1.40566327E-03, 1.37213017E-03, 1.33573943E-03,
     &   1.29990796E-03, 1.26475553E-03, 1.23043436E-03, 1.19704656E-03/
  
         DATA ((DAS  (I,J), I= 301, 400), J=2,2) /
     &   1.19258203E-03, 1.13389865E-03, 1.10405077E-03, 1.07537708E-03,
     &   9.83975242E-04, 9.05185374E-04, 8.37311415E-04, 7.78440215E-04,
     &   7.27136530E-04, 6.72042385E-04, 6.42589410E-04, 6.07532363E-04,
     &   5.76351705E-04, 5.48508209E-04, 5.23564196E-04, 5.01167190E-04,
     &   4.81039057E-04, 4.62942320E-04, 4.46703072E-04, 4.32220621E-04,
     &   4.18201553E-04, 4.07431335E-04, 4.11570843E-04, 3.95466153E-04,
     &   4.07438863E-04, 4.92939394E-04, 4.80514836E-04, 3.80499224E-04,
     &   3.51214494E-04, 3.32620536E-04, 3.18685338E-04, 3.07159832E-04,
     &   2.97731119E-04, 2.88683177E-04, 2.80673479E-04, 2.73448192E-04,
     &   2.66756547E-04, 2.60487088E-04, 2.54817385E-04, 2.49068516E-04,
     &   2.43788784E-04, 2.38707871E-04, 2.33763257E-04, 2.29097239E-04,
     &   2.24665178E-04, 2.20463096E-04, 2.16402942E-04, 2.12598841E-04,
     &   2.08938870E-04, 2.05489521E-04, 2.02157171E-04, 1.98958538E-04,
     &   1.95871436E-04, 1.92910718E-04, 1.90093506E-04, 1.87415171E-04,
     &   1.80289528E-04, 1.65361461E-04, 1.45161814E-04, 1.33337703E-04,
     &   1.25861444E-04, 1.21178051E-04, 1.18580489E-04, 1.17447274E-04,
     &   1.17047458E-04, 1.16997537E-04, 1.18113794E-04, 1.20522792E-04,
     &   1.21409143E-04, 1.23746062E-04, 1.26775664E-04, 1.26990561E-04,
     &   1.30053702E-04, 1.34013140E-04, 1.35917806E-04, 1.38458069E-04,
     &   1.43114502E-04, 1.48533590E-04, 1.54543822E-04, 1.61277709E-04,
     &   1.65275894E-04, 1.68467543E-04, 1.76188083E-04, 1.85199343E-04,
     &   1.86045360E-04, 1.94887770E-04, 2.05883913E-04, 2.11559623E-04,
     &   2.17960601E-04, 2.31362412E-04, 2.47252018E-04, 2.66836886E-04,
     &   2.73269210E-04, 2.90174761E-04, 3.03096191E-04, 3.17708259E-04,
     &   3.52479306E-04, 4.02141089E-04, 4.08268184E-04, 4.65347662E-04/
  
         DATA ((DAS  (I,J), I= 401, 500), J=2,2) /
     &   5.76062927E-04, 5.84845796E-04, 4.96639758E-04, 3.98004080E-04,
     &   4.05285594E-04, 3.45658721E-04, 3.38282168E-04, 3.34098385E-04,
     &   3.25597403E-04, 3.08985698E-04, 2.92572037E-04, 2.74262149E-04,
     &   2.72693774E-04, 2.55909341E-04, 2.52737740E-04, 2.38552294E-04,
     &   2.29182459E-04, 1.97366571E-04, 1.93667318E-04, 1.76408104E-04,
     &   1.75723989E-04, 1.75434385E-04, 1.65660229E-04, 1.51444304E-04,
     &   1.47582275E-04, 1.37198611E-04, 1.36454935E-04, 1.38657708E-04,
     &   1.38330300E-04, 1.34132809E-04, 1.35486919E-04, 1.28091703E-04,
     &   1.28767993E-04, 1.26202843E-04, 1.26824836E-04, 1.22077680E-04,
     &   1.22048986E-04, 1.18032378E-04, 1.19995705E-04, 1.24909646E-04,
     &   1.26151624E-04, 1.29212857E-04, 1.29066321E-04, 1.30367230E-04,
     &   1.41790434E-04, 1.44053004E-04, 1.56581264E-04, 1.54730504E-04,
     &   1.58560753E-04, 1.58025611E-04, 1.75209223E-04, 1.77799482E-04,
     &   1.79491215E-04, 1.90697014E-04, 1.92501532E-04, 1.94074043E-04,
     &   2.10495014E-04, 2.13974201E-04, 2.21818913E-04, 2.24614184E-04,
     &   2.27248063E-04, 2.26894317E-04, 2.27562949E-04, 2.26576252E-04,
     &   2.27013010E-04, 2.26023974E-04, 2.22996033E-04, 2.21862285E-04,
     &   2.22995914E-04, 2.21829682E-04, 2.23082434E-04, 2.34085102E-04,
     &   2.47497844E-04, 2.59625572E-04, 2.61741236E-04, 2.80320712E-04,
     &   2.84287945E-04, 2.86279178E-04, 3.00212617E-04, 3.12757283E-04,
     &   3.13665333E-04, 3.26420549E-04, 3.32720577E-04, 3.34271223E-04,
     &   3.35582298E-04, 3.59156535E-04, 3.70024239E-04, 3.79042131E-04,
     &   3.90452753E-04, 3.95699676E-04, 4.07910710E-04, 4.10404431E-04,
     &   4.22820633E-04, 4.40409900E-04, 4.62026278E-04, 4.72995389E-04,
     &   6.73563350E-04, 7.50429932E-04, 6.49533103E-04, 4.61004748E-04/
  
         DATA ((DAS  (I,J), I= 501, 600), J=2,2) /
     &   3.76636643E-04, 3.32235287E-04, 3.01280254E-04, 2.88497026E-04,
     &   2.82749847E-04, 2.64945766E-04, 2.56037422E-04, 2.44472090E-04,
     &   2.39170775E-04, 2.41220705E-04, 2.48220505E-04, 2.60398406E-04,
     &   2.78887967E-04, 3.06897147E-04, 3.50484775E-04, 4.20841006E-04,
     &   4.40369920E-04, 4.61626966E-04, 4.86119295E-04, 5.14164969E-04,
     &   5.46396654E-04, 5.83656540E-04, 6.27304382E-04, 6.36888535E-04,
     &   6.46826139E-04, 6.57124381E-04, 6.67796178E-04, 6.78858728E-04,
     &   6.90232241E-04, 7.02041106E-04, 7.14307168E-04, 7.27056842E-04,
     &   7.40301101E-04, 8.06403501E-04, 8.58448169E-04, 8.87176898E-04,
     &   9.07405539E-04, 9.28581805E-04, 9.50812021E-04, 9.74156453E-04,
     &   9.98706392E-04, 1.02455368E-03, 1.03799670E-03, 1.05180816E-03,
     &   1.06599715E-03, 1.08058492E-03, 1.09558233E-03, 1.11101243E-03,
     &   1.12688807E-03, 1.14323432E-03, 1.16006633E-03, 1.17741152E-03,
     &   1.19528765E-03, 1.21372240E-03, 1.23274194E-03, 1.25237413E-03,
     &   1.27264865E-03, 1.29359714E-03, 1.31525335E-03, 1.33765336E-03,
     &   1.36083571E-03, 1.38484167E-03, 1.40971546E-03, 1.43550452E-03,
     &   1.46225983E-03, 1.49003621E-03, 1.51889269E-03, 1.54889297E-03,
     &   1.58010583E-03, 1.61260567E-03, 1.64647313E-03, 1.68179568E-03,
     &   1.71866916E-03, 1.75719655E-03, 1.79749089E-03, 1.83967575E-03,
     &   1.88388651E-03, 1.93027175E-03, 1.97899498E-03, 2.03023648E-03,
     &   2.08419556E-03, 2.14109316E-03, 2.17071916E-03, 2.20117470E-03,
     &   2.23249356E-03, 2.26471391E-03, 2.29787411E-03, 2.33201666E-03,
     &   2.36718474E-03, 2.40342602E-03, 2.44078925E-03, 2.47932807E-03,
     &   2.51909773E-03, 2.56015883E-03, 2.60257421E-03, 2.64641262E-03,
     &   2.69174582E-03, 2.73865215E-03, 2.78721388E-03, 2.83752074E-03/
  
         DATA ((DAS  (I,J), I= 601, 700), J=2,2) /
     &   2.88966728E-03, 2.94375724E-03, 2.99989984E-03, 3.05821442E-03,
     &   3.11882917E-03, 3.18188256E-03, 3.24752441E-03, 3.31591709E-03,
     &   3.38723681E-03, 3.46167517E-03, 3.53944086E-03, 3.62076162E-03,
     &   3.70588647E-03, 3.79508813E-03, 3.88866606E-03, 3.98695021E-03,
     &   4.09030382E-03, 4.19912924E-03, 4.31387180E-03, 4.43502726E-03,
     &   4.61648192E-03, 4.69885076E-03, 4.84282915E-03, 4.99586293E-03,
     &   5.15883282E-03, 5.33273845E-03, 5.51871728E-03, 5.71807109E-03,
     &   5.93229486E-03, 6.16311588E-03, 6.41253884E-03, 6.68290570E-03,
     &   6.97696785E-03, 7.21172105E-03, 7.94307078E-03, 8.83874950E-03,
     &   9.96112022E-03, 1.14086707E-02, 1.33465957E-02, 1.60747828E-02,
     &   2.02002127E-02, 2.71651801E-02, 4.14366113E-02, 8.71296009E-02,
     &   9.21343064E-02, 9.77467149E-02, 1.04084224E-01, 1.11296409E-01,
     &   1.19576797E-01, 1.29180180E-01, 1.40448749E-01, 1.53852706E-01,
     &   1.70055548E-01, 1.90023140E-01, 2.15214283E-01, 2.47931170E-01,
     &   2.92002031E-01, 2.97256427E-01, 4.14909073E-01, 5.00662523E-01,
     &   5.69560470E-01, 6.29480561E-01, 6.75653785E-01, 7.15997493E-01,
     &   7.48330164E-01, 7.80540664E-01, 8.02667368E-01, 8.23154867E-01,
     &   8.37611583E-01, 8.49529518E-01, 8.56226808E-01, 8.61225894E-01,
     &   8.61650989E-01, 8.59135055E-01, 8.52157971E-01, 8.44617389E-01,
     &   8.28307012E-01, 8.13693047E-01, 7.95754544E-01, 7.74209337E-01,
     &   7.48786748E-01, 7.19237849E-01, 6.85347406E-01, 6.46947247E-01,
     &   6.03930575E-01, 5.56266499E-01, 5.04013811E-01, 4.17411836E-01,
     &   3.54631349E-01, 2.53995231E-01, 1.47366134E-01, 1.47264450E-01,
     &   4.17322795E-01, 5.03931925E-01, 5.80604140E-01, 6.25951578E-01,
     &   6.85287815E-01, 7.19184100E-01, 7.48738809E-01, 7.74167123E-01/
  
         DATA ((DAS  (I,J), I= 701, 800), J=2,2) /
     &   7.95717928E-01, 8.13661870E-01, 8.28281088E-01, 8.39860736E-01,
     &   8.48682218E-01, 8.55018031E-01, 8.59128011E-01, 8.61650108E-01,
     &   8.61228849E-01, 8.57946713E-01, 8.52016796E-01, 8.46853598E-01,
     &   8.34224084E-01, 8.23180487E-01, 8.06957517E-01, 7.89563415E-01,
     &   7.66869639E-01, 7.43734601E-01, 7.11473907E-01, 6.80086842E-01,
     &   6.41739408E-01, 5.98489714E-01, 5.42860275E-01, 4.79088721E-01,
     &   3.99460128E-01, 3.29166182E-01, 2.74563563E-01, 2.35164103E-01,
     &   2.05498108E-01, 1.82398854E-01, 1.63923771E-01, 1.48821189E-01,
     &   1.36250693E-01, 1.25628107E-01, 1.16535265E-01, 1.08665209E-01,
     &   1.01787667E-01, 9.57265878E-02, 9.03451207E-02, 8.55353040E-02,
     &   8.12108428E-02, 7.73019447E-02, 7.37515622E-02, 3.86733499E-02,
     &   2.62113261E-02, 1.98276188E-02, 1.59474681E-02, 1.33396841E-02,
     &   1.14666416E-02, 1.00562029E-02, 8.95585229E-03, 8.07347802E-03,
     &   7.35016341E-03, 7.26449894E-03, 6.66718760E-03, 6.16122644E-03,
     &   5.72715849E-03, 5.35068625E-03, 5.02106701E-03, 4.73007406E-03,
     &   4.47129977E-03, 4.23968025E-03, 4.03115854E-03, 3.84244871E-03,
     &   3.67085963E-03, 3.51416642E-03, 3.37051375E-03, 3.23834215E-03,
     &   3.11633129E-03, 3.00335584E-03, 2.89845077E-03, 2.80078375E-03,
     &   2.70963315E-03, 2.62437017E-03, 2.54444445E-03, 2.46937214E-03,
     &   2.39872603E-03, 2.33212812E-03, 2.26924192E-03, 2.20976701E-03,
     &   2.15343509E-03, 2.10000495E-03, 2.04925958E-03, 2.00100308E-03,
     &   1.95505816E-03, 1.91126374E-03, 1.86947410E-03, 1.82955578E-03,
     &   1.79138713E-03, 1.75485682E-03, 1.71986280E-03, 1.68631132E-03,
     &   1.65411617E-03, 1.62319785E-03, 1.59348299E-03, 1.56490377E-03,
     &   1.53739737E-03, 1.51090557E-03, 1.48537427E-03, 1.46075321E-03/
  
         DATA ((DAS  (I,J), I= 801, 900), J=2,2) /
     &   1.43699558E-03, 1.41405776E-03, 1.39189902E-03, 1.37048135E-03,
     &   1.34976917E-03, 1.32972917E-03, 1.31033015E-03, 1.29154284E-03,
     &   1.27333975E-03, 1.25569504E-03, 1.23858444E-03, 1.22198506E-03,
     &   1.20587539E-03, 1.19023511E-03, 1.17504509E-03, 1.16028725E-03,
     &   1.14594456E-03, 1.13200090E-03, 1.11844105E-03, 1.10525063E-03,
     &   1.09241607E-03, 1.07992452E-03, 1.06776387E-03, 1.05592267E-03,
     &   1.04439014E-03, 1.03315613E-03, 1.02221109E-03, 1.01154611E-03,
     &   1.00115285E-03, 9.91023616E-04, 9.81151317E-04, 9.71529549E-04,
     &   9.62152633E-04, 9.53015705E-04, 9.44114853E-04, 9.35447322E-04,
     &   9.27011850E-04, 9.18530470E-04, 9.10240825E-04, 9.02137823E-04,
     &   8.94216761E-04, 8.86473353E-04, 8.78903784E-04, 8.71504798E-04,
     &   8.64273819E-04, 8.57907482E-04, 8.52193262E-04, 8.45288025E-04,
     &   8.38520967E-04, 8.31888267E-04, 8.25386248E-04, 8.19011371E-04,
     &   8.12760235E-04, 8.06629562E-04, 8.00616201E-04, 7.94716368E-04,
     &   6.23668384E-04, 5.19880939E-04, 4.66597288E-04, 4.88244928E-04,
     &   5.55610270E-04, 6.01693217E-04, 6.64328159E-04, 7.50858226E-04,
     &   8.38968860E-04, 9.57777045E-04, 1.00733572E-03, 1.12031796E-03,
     &   1.18971834E-03, 1.26893950E-03, 1.35967279E-03, 1.46654263E-03,
     &   1.59334912E-03, 1.74569380E-03, 1.93177166E-03, 2.16388280E-03,
     &   2.46117159E-03, 2.60476688E-03, 2.76643545E-03, 2.94977359E-03,
     &   3.15941508E-03, 3.40141409E-03, 3.68382069E-03, 3.84346048E-03,
     &   4.01761776E-03, 4.20835513E-03, 4.41814996E-03, 4.64999418E-03,
     &   4.90754507E-03, 5.19531197E-03, 5.51892818E-03, 5.88551636E-03,
     &   6.30422321E-03, 6.78698601E-03, 7.34969108E-03, 7.98508598E-03,
     &   8.40109371E-03, 8.86273781E-03, 9.37795357E-03, 9.95662908E-03/
  
         DATA ((DAS  (I,J), I= 901,1000), J=2,2) /
     &   1.06112443E-02, 1.13577802E-02, 1.22170377E-02, 1.32166025E-02,
     &   1.43938550E-02, 1.58007375E-02, 1.75116026E-02, 1.96367460E-02,
     &   2.23470732E-02, 2.59222677E-02, 3.08534152E-02, 3.80881949E-02,
     &   4.97169446E-02, 7.13977159E-02, 1.05497570E-01, 1.24905874E-01,
     &   1.46075441E-01, 1.98162497E-01, 2.16928227E-01, 2.32415997E-01,
     &   2.45568705E-01, 2.56644035E-01, 2.65730775E-01, 2.73554940E-01,
     &   2.79961409E-01, 2.84546526E-01, 2.88441777E-01, 2.91199268E-01,
     &   2.92834790E-01, 2.93425491E-01, 2.93006015E-01, 2.91859804E-01,
     &   2.89533593E-01, 2.86489633E-01, 2.82386543E-01, 2.77107402E-01,
     &   2.71556346E-01, 2.63778941E-01, 2.55915818E-01, 2.46878284E-01,
     &   2.36608338E-01, 2.23006743E-01, 2.09922951E-01, 1.92977187E-01,
     &   1.77021771E-01, 1.53759549E-01, 1.28372463E-01, 9.04552690E-02,
     &   5.74041913E-02, 1.93357775E-02, 3.78192455E-04, 1.92392374E-02,
     &   3.83969653E-02, 9.03637452E-02, 1.28286513E-01, 1.53678629E-01,
     &   1.76946520E-01, 1.92906510E-01, 2.09857858E-01, 2.22946577E-01,
     &   2.36554004E-01, 2.46828963E-01, 2.55871476E-01, 2.63739508E-01,
     &   2.71522512E-01, 2.77078229E-01, 2.82362622E-01, 2.86470745E-01,
     &   2.89519506E-01, 2.91850912E-01, 2.93001411E-01, 2.93426053E-01,
     &   2.92840137E-01, 2.91209509E-01, 2.88456919E-01, 2.84566486E-01,
     &   2.79985689E-01, 2.73583997E-01, 2.65764521E-01, 2.56682252E-01,
     &   2.45611442E-01, 2.32463193E-01, 2.16979778E-01, 1.98218386E-01,
     &   1.75473644E-01, 1.87821962E-01, 1.60670878E-01, 1.43947485E-01,
     &   1.33837077E-01, 1.12892970E-01, 1.05566084E-01, 8.97937465E-02,
     &   5.81422491E-02, 4.28999193E-02, 3.39710803E-02, 2.81140513E-02,
     &   2.39786819E-02, 2.09040363E-02, 1.85287981E-02, 1.66388999E-02/
  
         DATA ((DAS  (I,J), I=1001,1100), J=2,2) /
     &   1.50994940E-02, 1.38214291E-02, 1.27434139E-02, 1.18219264E-02,
     &   1.10252130E-02, 1.03295631E-02, 9.71691420E-03, 9.17326828E-03,
     &   8.68760269E-03, 8.25113426E-03, 7.56130565E-03, 6.70054958E-03,
     &   6.24035915E-03, 5.48862330E-03, 4.90068456E-03, 4.32388197E-03,
     &   3.50451446E-03, 2.61054531E-03, 2.08468977E-03, 1.86729802E-03,
     &   1.74607491E-03, 1.66752199E-03, 1.61207488E-03, 1.57038611E-03,
     &   1.53768764E-03, 1.51115560E-03, 1.48900332E-03, 1.47008884E-03,
     &   1.45363660E-03, 1.43905892E-03, 1.42595669E-03, 1.41401411E-03,
     &   1.40299873E-03, 1.39270774E-03, 1.38297404E-03, 1.37372610E-03,
     &   1.36481097E-03, 1.35614592E-03, 1.34766658E-03, 1.33930461E-03,
     &   1.33097046E-03, 1.32260401E-03, 1.31411545E-03, 1.30545558E-03,
     &   1.29650624E-03, 1.28715963E-03, 1.27723878E-03, 1.26654994E-03,
     &   1.25471391E-03, 1.24103527E-03, 1.22377943E-03, 1.18497880E-03,
     &   1.14937369E-03, 1.13676498E-03, 1.12811582E-03, 1.12156506E-03,
     &   1.11634926E-03, 1.11206255E-03, 1.10841815E-03, 1.10521579E-03,
     &   1.10225719E-03, 1.09863891E-03, 1.09639512E-03, 1.09299153E-03,
     &   1.08874208E-03, 1.08248846E-03, 1.06423122E-03, 1.08028708E-03,
     &   1.08421993E-03, 1.08604246E-03, 1.08635366E-03, 1.08549013E-03,
     &   1.08351645E-03, 1.08037763E-03, 1.07603727E-03, 1.07037208E-03,
     &   1.06316092E-03, 1.05429418E-03, 1.04286782E-03, 1.02785597E-03,
     &   1.00992662E-03, 9.88087771E-04, 9.61101827E-04, 9.26139734E-04,
     &   8.83473512E-04, 8.25154596E-04, 8.08279484E-04, 7.60263862E-04,
     &   6.93881969E-04, 6.87742312E-04, 6.45777647E-04, 6.31693427E-04,
     &   6.21691644E-04, 6.08561216E-04, 5.95853016E-04, 5.86287223E-04,
     &   5.81049790E-04, 5.76433427E-04, 5.69308647E-04, 5.58198455E-04/
  
         DATA ((DAS  (I,J), I=1101,1200), J=2,2) /
     &   5.53590898E-04, 5.34110763E-04, 5.12562159E-04, 5.11398290E-04,
     &   4.95545585E-04, 4.80664392E-04, 4.79411642E-04, 4.76008275E-04,
     &   4.79990744E-04, 4.94839716E-04, 5.39315297E-04, 4.61376688E-04,
     &   4.18020255E-04, 4.29579736E-04, 4.39362622E-04, 4.56115362E-04,
     &   4.53253320E-04, 4.40891896E-04, 4.40124223E-04, 4.35897608E-04,
     &   4.19179209E-04, 4.10363576E-04, 3.88458319E-04, 3.84048782E-04,
     &   3.84443718E-04, 3.75058027E-04, 3.68780448E-04, 3.64224740E-04,
     &   3.54934384E-04, 3.47185995E-04, 3.44092076E-04, 3.42913051E-04,
     &   3.41889663E-04, 3.41958410E-04, 3.40103973E-04, 3.37547843E-04,
     &   3.34703527E-04, 3.34689270E-04, 3.36339198E-04, 3.34534208E-04,
     &   3.24561061E-04, 3.17919472E-04, 3.13767844E-04, 3.11680791E-04,
     &   3.11118565E-04, 3.10881371E-04, 3.11004603E-04, 3.09571589E-04,
     &   3.05670393E-04, 3.03350842E-04, 3.01004061E-04, 2.99503329E-04,
     &   2.97900071E-04, 2.95856140E-04, 2.93863907E-04, 2.92937362E-04,
     &   2.92049477E-04, 2.93244410E-04, 2.90156139E-04, 2.88244755E-04,
     &   2.87876357E-04, 2.87758420E-04, 2.85141065E-04, 2.83218579E-04,
     &   2.84102424E-04, 2.81109755E-04, 2.82025311E-04, 2.88627839E-04,
     &   2.78296060E-04, 2.77818264E-04, 2.75760976E-04, 2.73863797E-04,
     &   2.75995728E-04, 2.71187174E-04, 2.69928844E-04, 2.68307310E-04,
     &   2.66888386E-04, 2.64423656E-04, 2.66380473E-04, 2.66628720E-04,
     &   2.66356328E-04, 2.64982826E-04, 2.66214748E-04, 2.66792221E-04,
     &   2.61839074E-04, 2.61326558E-04, 2.61686303E-04, 2.57576896E-04,
     &   2.56705472E-04, 2.57428115E-04, 2.56512550E-04, 2.57674170E-04,
     &   2.58904765E-04, 2.53501683E-04, 2.52572267E-04, 2.51797386E-04,
     &   2.57221736E-04, 2.54224627E-04, 2.53391895E-04, 2.51931121E-04/
  
         DATA ((DAS  (I,J), I=1201,1300), J=2,2) /
     &   2.55783309E-04, 2.63245475E-04, 2.68200809E-04, 2.63489092E-04,
     &   2.63502999E-04, 2.67213929E-04, 2.64632625E-04, 2.67312477E-04,
     &   2.77163112E-04, 3.11149504E-04, 2.91867596E-04, 2.92869566E-04,
     &   3.04265573E-04, 2.82461096E-04, 2.78934684E-04, 2.77710473E-04,
     &   2.93909491E-04, 2.96007021E-04, 3.27114625E-04, 3.30086011E-04,
     &   3.80719581E-04, 4.46621794E-04, 4.78256710E-04, 5.94616303E-04,
     &   6.84806989E-04, 6.78305307E-04, 5.73197921E-04, 4.71427742E-04,
     &   3.82239218E-04, 3.56585836E-04, 3.36822014E-04, 3.07907969E-04,
     &   2.87446606E-04, 2.79238626E-04, 2.72024736E-04, 2.65624505E-04,
     &   2.59900446E-04, 2.54745649E-04, 2.50071829E-04, 2.45818251E-04,
     &   2.41926161E-04, 2.38350063E-04, 2.35052124E-04, 2.32000594E-04,
     &   2.29168610E-04, 2.26533276E-04, 2.24074947E-04, 2.21776666E-04,
     &   2.19623714E-04, 2.17603256E-04, 2.15704046E-04, 2.13916195E-04,
     &   2.12230972E-04, 2.10640645E-04, 2.09138352E-04, 2.07717982E-04,
     &   2.06374090E-04, 2.05101810E-04, 2.03896797E-04, 2.02755164E-04,
     &   2.01673439E-04, 2.00648521E-04, 1.99677648E-04, 1.98758368E-04,
     &   1.97888514E-04, 1.97066184E-04, 1.96289725E-04, 1.95700625E-04,
     &   1.94868979E-04, 1.93617610E-04, 1.92530411E-04, 1.91605644E-04,
     &   1.90845244E-04, 1.90255281E-04, 1.89846718E-04, 1.89636632E-04,
     &   1.89650102E-04, 1.89923161E-04, 1.90507541E-04, 1.91479351E-04,
     &   1.92949413E-04, 1.95095314E-04, 1.98211686E-04, 1.98594154E-04,
     &   2.02838685E-04, 2.10116452E-04, 2.23054226E-04, 2.44387494E-04,
     &   2.53417391E-04, 2.92908205E-04, 3.74376024E-04, 4.14644288E-04,
     &   4.73120688E-04, 5.13135320E-04, 5.64024021E-04, 6.15661122E-04,
     &   6.80996241E-04, 7.20520879E-04, 7.65924057E-04, 8.18559296E-04/
  
         DATA ((DAS  (I,J), I=1301,1400), J=2,2) /
     &   8.80226051E-04, 9.53369671E-04, 1.04139956E-03, 1.09244883E-03,
     &   1.14921598E-03, 1.21269519E-03, 1.28412465E-03, 1.36506626E-03,
     &   1.45751888E-03, 1.56408251E-03, 1.68820275E-03, 1.83454485E-03,
     &   1.89554567E-03, 1.93383229E-03, 1.97373841E-03, 2.01536816E-03,
     &   2.05883476E-03, 2.10426162E-03, 2.15178340E-03, 2.20154741E-03,
     &   2.25371507E-03, 2.30846371E-03, 2.36598849E-03, 2.42650478E-03,
     &   2.49025080E-03, 2.55749075E-03, 2.62851844E-03, 2.70366157E-03,
     &   2.78328676E-03, 2.86780548E-03, 2.95768117E-03, 3.05343766E-03,
     &   3.15566940E-03, 3.26505370E-03, 3.38236575E-03, 3.50849694E-03,
     &   3.64447749E-03, 3.79150455E-03, 3.95097745E-03, 4.12454226E-03,
     &   4.31414870E-03, 4.52212351E-03, 4.75126621E-03, 5.00497562E-03,
     &   5.28741927E-03, 5.60376376E-03, 5.96049319E-03, 6.36585765E-03,
     &   6.83051840E-03, 7.36849800E-03, 7.99861821E-03, 8.74674499E-03,
     &   9.64942348E-03, 1.07600224E-02, 1.21596714E-02, 1.39780025E-02,
     &   1.64357457E-02, 1.99417387E-02, 2.53472777E-02, 3.47649406E-02,
     &   5.52586341E-02, 6.79650631E-02, 1.29452187E-01, 1.80123489E-01,
     &   2.16852462E-01, 2.45271743E-01, 2.68621660E-01, 2.89055401E-01,
     &   3.08015067E-01, 3.21214056E-01, 3.34649609E-01, 3.47863357E-01,
     &   3.54234325E-01, 3.63163111E-01, 3.68513137E-01, 3.75153933E-01,
     &   3.76884557E-01, 3.79444375E-01, 3.80508183E-01, 3.79706426E-01,
     &   3.78483068E-01, 3.76636139E-01, 3.70787255E-01, 3.66685676E-01,
     &   3.61644639E-01, 3.48656348E-01, 3.40592806E-01, 3.31377083E-01,
     &   3.20832947E-01, 2.96109027E-01, 2.81588989E-01, 2.65847962E-01,
     &   2.30067297E-01, 2.10260957E-01, 1.66674129E-01, 1.18689157E-01,
     &   1.26742528E-02, 1.17113131E-01, 1.65221227E-01, 2.08743388E-01/
  
         DATA ((DAS  (I,J), I=1401,1500), J=2,2) /
     &   2.47415766E-01, 2.64801734E-01, 2.80629708E-01, 2.95091783E-01,
     &   3.20127117E-01, 3.30646752E-01, 3.40041869E-01, 3.48176702E-01,
     &   3.61295560E-01, 3.66347263E-01, 3.70551218E-01, 3.76472613E-01,
     &   3.78384673E-01, 3.80337686E-01, 3.80513500E-01, 3.79502891E-01,
     &   3.77001997E-01, 3.73299937E-01, 3.68672767E-01, 3.63342045E-01,
     &   3.54432764E-01, 3.48103820E-01, 3.34863791E-01, 3.21463109E-01,
     &   3.08224108E-01, 2.89252176E-01, 2.68801236E-01, 2.45429164E-01,
     &   2.16981870E-01, 1.80218091E-01, 1.28758760E-01, 7.02755501E-02,
     &   4.01965287E-02, 2.81244173E-02, 2.16249463E-02, 1.75648724E-02,
     &   1.47883547E-02, 1.27699905E-02, 1.12366467E-02, 1.00323029E-02,
     &   9.06136732E-03, 8.26200710E-03, 7.59245572E-03, 7.02348450E-03,
     &   6.53402669E-03, 6.10851460E-03, 5.73519535E-03, 5.40502984E-03,
     &   5.11095228E-03, 4.84735956E-03, 4.60975145E-03, 4.39447195E-03,
     &   4.19852018E-03, 4.01940999E-03, 3.85506408E-03, 3.70373331E-03,
     &   3.56393437E-03, 3.43440110E-03, 3.31404617E-03, 3.20193059E-03,
     &   3.09723926E-03, 2.99926115E-03, 2.90737325E-03, 2.82102733E-03,
     &   2.73973906E-03, 2.66307896E-03, 2.59066486E-03, 2.52215553E-03,
     &   2.45724537E-03, 2.39565989E-03, 2.33715180E-03, 2.28149777E-03,
     &   2.22849558E-03, 2.17796166E-03, 2.12972904E-03, 2.08364548E-03,
     &   2.03957187E-03, 1.99738087E-03, 1.95695570E-03, 1.91818902E-03,
     &   1.88098206E-03, 1.60093535E-03, 1.42996899E-03, 1.26682321E-03,
     &   1.08613835E-03, 9.19682178E-04, 7.60962816E-04, 6.78363966E-04,
     &   5.65582609E-04, 5.28388787E-04, 4.99187722E-04, 4.65745784E-04,
     &   4.39074501E-04, 4.17531842E-04, 3.84746175E-04, 3.59494036E-04,
     &   3.00343434E-04, 2.69992851E-04, 2.71407727E-04, 3.22893747E-04/
  
         DATA ((DAS  (I,J), I=1501,1600), J=2,2) /
     &   4.41690770E-04, 6.19901558E-04, 8.70271574E-04, 1.22528833E-03,
     &   2.12879530E-03, 2.30096826E-03, 2.50388661E-03, 2.56271564E-03,
     &   2.69684571E-03, 2.84593288E-03, 3.01261597E-03, 3.20019338E-03,
     &   3.41284391E-03, 3.65593999E-03, 3.93650682E-03, 4.26391297E-03,
     &   4.65093120E-03, 5.11544564E-03, 5.68328644E-03, 6.39320018E-03,
     &   7.30607106E-03, 8.52337081E-03, 1.02277333E-02, 1.27844055E-02,
     &   1.70454678E-02, 2.55661184E-02, 3.69962004E-02, 3.89147368E-02,
     &   4.10430094E-02, 4.34173767E-02, 4.60831105E-02, 4.90973026E-02,
     &   5.25329888E-02, 5.64851257E-02, 6.10794715E-02, 6.64861566E-02,
     &   7.29411100E-02, 8.07811967E-02, 9.05044945E-02, 1.02879447E-01,
     &   1.19156079E-01, 1.41510218E-01, 1.74083911E-01, 2.25799807E-01,
     &   3.19651802E-01, 5.31970889E-01, 6.06400046E-01, 6.64981442E-01,
     &   7.30275517E-01, 7.85072924E-01, 8.24591696E-01, 8.58108250E-01,
     &   8.73551361E-01, 8.79682735E-01, 8.71970962E-01, 8.44644717E-01,
     &   8.13182667E-01, 8.07229252E-01, 8.04134197E-01, 7.97703098E-01,
     &   7.95039484E-01, 7.93007184E-01, 7.90945035E-01, 7.68643308E-01,
     &   7.29195587E-01, 6.76411220E-01, 6.20485241E-01, 5.48849744E-01,
     &   4.67452855E-01, 3.86422676E-01, 3.38540076E-01, 2.88848447E-01,
     &   2.37572445E-01, 1.63609620E-01, 1.09606252E-01, 5.49668668E-02,
     &   2.20090972E-02, 1.10102988E-02, 5.51141874E-03, 2.20323513E-04,
     &   5.50020089E-03, 1.09990749E-02, 5.49556645E-02, 1.09595123E-01,
     &   1.63598613E-01, 4.05014778E-01, 4.92855515E-01, 6.33605511E-01,
     &   6.87739413E-01, 7.38599251E-01, 7.80175530E-01, 8.10241082E-01,
     &   8.36237971E-01, 8.53670899E-01, 8.67227746E-01, 8.74879509E-01,
     &   8.79061392E-01, 8.79520859E-01, 8.76963102E-01, 8.72363183E-01/
  
         DATA ((DAS  (I,J), I=1601,1700), J=2,2) /
     &   8.57158719E-01, 8.46911051E-01, 8.35549389E-01, 8.24594948E-01,
     &   7.99934758E-01, 7.86438446E-01, 7.64518644E-01, 7.30280558E-01,
     &   6.06406426E-01, 5.10444881E-01, 4.37782719E-01, 3.81962858E-01,
     &   3.38148368E-01, 3.03019894E-01, 2.67944593E-01, 2.13181055E-01,
     &   1.98361396E-01, 1.85445381E-01, 1.74092192E-01, 1.57220065E-01,
     &   1.30132307E-01, 1.10974266E-01, 9.67181993E-02, 8.57002189E-02,
     &   7.69315575E-02, 6.97882232E-02, 6.38572042E-02, 5.88543465E-02,
     &   5.45777833E-02, 5.08801830E-02, 4.76515113E-02, 4.48079390E-02,
     &   4.22844830E-02, 4.00299953E-02, 3.80036685E-02, 2.55745343E-02,
     &   1.70538840E-02, 1.27928201E-02, 8.53177985E-03, 6.40161230E-03,
     &   5.12386426E-03, 4.27234169E-03, 3.66437237E-03, 3.20862940E-03,
     &   2.85437367E-03, 2.57116267E-03, 2.41060473E-03, 2.22337215E-03,
     &   2.06352140E-03, 1.92548070E-03, 1.80509601E-03, 1.69920714E-03,
     &   1.31706130E-03, 1.07951015E-03, 9.18363327E-04, 8.02466808E-04,
     &   7.15768749E-04, 6.49362505E-04, 5.95337173E-04, 5.51447682E-04,
     &   3.01734651E-04, 2.97750300E-04, 3.04362388E-04, 3.10086970E-04,
     &   3.21242524E-04, 3.39875050E-04, 3.51624155E-04, 3.67710501E-04,
     &   4.11564035E-04, 4.87521997E-04, 6.34157804E-04, 9.96944127E-04,
     &   1.35654791E-03, 1.45026646E-03, 1.55864486E-03, 1.68534496E-03,
     &   1.83536487E-03, 2.01570600E-03, 2.23647144E-03, 2.51282517E-03,
     &   2.86859621E-03, 3.34347126E-03, 4.00895336E-03, 5.00800458E-03,
     &   6.67413291E-03, 6.96398180E-03, 7.28016235E-03, 7.62650973E-03,
     &   8.00751055E-03, 8.42858775E-03, 8.89651933E-03, 9.41952139E-03,
     &   1.00078501E-02, 1.06747092E-02, 1.14368504E-02, 1.23161535E-02,
     &   1.33421160E-02, 1.45546096E-02, 1.60093972E-02, 1.77875717E-02/
  
         DATA ((DAS  (I,J), I=1701,1800), J=2,2) /
     &   1.83432089E-02, 2.00101302E-02, 2.28670156E-02, 2.66758161E-02,
     &   3.20063791E-02, 3.99974266E-02, 5.32973580E-02, 7.98120390E-02,
     &   1.58181089E-01, 1.88056739E-01, 2.65249889E-01, 3.23953734E-01,
     &   3.72323298E-01, 4.14540755E-01, 4.51996202E-01, 4.85597364E-01,
     &   5.15611511E-01, 5.43431546E-01, 5.69704225E-01, 5.93714542E-01,
     &   6.14834429E-01, 6.34370568E-01, 6.51984682E-01, 6.67412523E-01,
     &   6.82070173E-01, 6.94076580E-01, 7.03452391E-01, 7.11412715E-01,
     &   7.17539777E-01, 7.21335289E-01, 7.22290186E-01, 7.20676783E-01,
     &   7.16017516E-01, 7.07784744E-01, 6.95407121E-01, 6.81491764E-01,
     &   6.59939643E-01, 6.32522628E-01, 5.98683292E-01, 5.50474431E-01,
     &   4.92419566E-01, 4.13934124E-01, 3.11087339E-01, 4.26221238E-02,
     &   2.57127214E-04, 1.40818841E-01, 3.11082284E-01, 4.13929582E-01,
     &   4.92415557E-01, 5.50470922E-01, 5.98680295E-01, 6.32520067E-01,
     &   6.59937509E-01, 6.81490042E-01, 6.95405728E-01, 7.07783726E-01,
     &   7.16016850E-01, 7.20676447E-01, 7.22290157E-01, 7.21335544E-01,
     &   7.17540337E-01, 7.11413553E-01, 7.03453481E-01, 6.94077899E-01,
     &   6.82071729E-01, 6.67414316E-01, 6.51986686E-01, 6.34372778E-01,
     &   6.14836838E-01, 5.93717137E-01, 5.69707006E-01, 5.43434507E-01,
     &   5.15614638E-01, 4.85600648E-01, 4.51999641E-01, 4.14544342E-01,
     &   3.72327029E-01, 3.23957602E-01, 2.65253890E-01, 1.88060869E-01,
     &   1.61030084E-01, 1.55960220E-01, 1.49671562E-01, 1.32302711E-01,
     &   1.16649308E-01, 7.98119428E-02, 5.33016005E-02, 4.00005842E-02,
     &   3.20099301E-02, 2.66800658E-02, 2.28712660E-02, 2.00141083E-02,
     &   1.77918228E-02, 1.60136485E-02, 1.45587168E-02, 1.33463676E-02,
     &   1.23204053E-02, 1.14410132E-02, 1.06789612E-02, 1.00121021E-02/
  
         DATA ((DAS  (I,J), I=1801,1900), J=2,2) /
     &   6.67838575E-03, 5.01225853E-03, 4.01320822E-03, 3.34772768E-03,
     &   2.87284852E-03, 2.51708727E-03, 2.24074924E-03, 1.83968715E-03,
     &   1.56303218E-03, 1.45468338E-03, 1.36098855E-03, 1.02673014E-03,
     &   7.87025403E-04, 6.48032693E-04, 5.58859533E-04, 4.97809805E-04,
     &   4.54121911E-04, 4.21828646E-04, 3.97381447E-04, 3.78541104E-04,
     &   3.63813480E-04, 3.52178952E-04, 3.42938993E-04, 3.35596781E-04,
     &   3.29796019E-04, 3.25282852E-04, 3.21883412E-04, 3.19494567E-04,
     &   3.18091892E-04, 3.17775916E-04, 3.18028476E-04, 3.17611368E-04,
     &   3.21366230E-04, 3.91419423E-04, 4.58013686E-04, 5.12900176E-04,
     &   5.59533164E-04, 5.99260617E-04, 6.33509455E-04, 6.62972281E-04,
     &   6.88281226E-04, 7.09707313E-04, 7.27728033E-04, 7.42645858E-04,
     &   7.54518440E-04, 7.63490421E-04, 7.69774023E-04, 7.73490533E-04,
     &   7.74609974E-04, 7.73258981E-04, 7.69464787E-04, 7.63110413E-04,
     &   7.54353014E-04, 7.43368473E-04, 7.29438563E-04, 7.13476463E-04,
     &   6.94503849E-04, 6.72871938E-04, 6.48084588E-04, 6.20082887E-04,
     &   5.88890525E-04, 5.54645755E-04, 5.15006249E-04, 4.70947861E-04,
     &   4.22324602E-04, 3.66467847E-04, 3.05269788E-04, 2.73217270E-04,
     &   4.04975668E-04, 4.74088231E-04, 5.27325168E-04, 5.69845081E-04,
     &   6.06198006E-04, 6.37419122E-04, 6.63331273E-04, 6.86071039E-04,
     &   7.05731030E-04, 7.22456288E-04, 7.36425050E-04, 7.48056802E-04,
     &   7.57230725E-04, 7.64556499E-04, 7.69597619E-04, 7.72746499E-04,
     &   7.73916896E-04, 7.73096317E-04, 7.70347659E-04, 7.65568009E-04,
     &   7.58701465E-04, 7.49818257E-04, 7.38763083E-04, 7.25321029E-04,
     &   7.05491830E-04, 6.90996600E-04, 6.69421455E-04, 6.44646737E-04,
     &   6.16224500E-04, 5.83480177E-04, 5.45682795E-04, 5.01258645E-04/
  
         DATA ((DAS  (I,J), I=1901,2000), J=2,2) /
     &   4.48575517E-04, 3.82886174E-04, 2.73495009E-04, 2.93883517E-04,
     &   2.81705780E-04, 2.77359321E-04, 2.75858749E-04, 2.75371582E-04,
     &   2.75091435E-04, 2.75264395E-04, 2.75753596E-04, 2.76319220E-04,
     &   2.76873363E-04, 2.77316653E-04, 2.77621671E-04, 2.77834684E-04,
     &   2.77887150E-04, 2.77810758E-04, 2.77560089E-04, 2.77076039E-04,
     &   2.76415207E-04, 2.75538089E-04, 2.74456226E-04, 2.73169245E-04,
     &   2.71656705E-04, 2.69876299E-04, 2.67860371E-04, 2.65593329E-04,
     &   2.63065624E-04, 2.60274373E-04, 2.57207583E-04, 2.53846615E-04,
     &   2.50133929E-04, 2.46077216E-04, 2.41772702E-04, 2.36490974E-04,
     &   2.30790249E-04, 2.24759652E-04, 2.16422821E-04, 2.19692438E-04,
     &   2.23498267E-04, 2.27057348E-04, 2.30679804E-04, 2.34061735E-04,
     &   2.37036730E-04, 2.39885028E-04, 2.42605727E-04, 2.45137202E-04,
     &   2.47505160E-04, 2.49726335E-04, 2.51796314E-04, 2.53706092E-04,
     &   2.55469745E-04, 2.57092084E-04, 2.58551093E-04, 2.59872887E-04,
     &   2.61060835E-04, 2.62103779E-04, 2.62998725E-04, 2.63749634E-04,
     &   2.64382120E-04, 2.64893519E-04, 2.65298100E-04, 2.65584157E-04,
     &   2.65735047E-04, 2.65779270E-04, 2.65763334E-04, 2.65718430E-04,
     &   2.65661049E-04, 2.65648254E-04, 2.65785422E-04, 2.65088490E-04,
     &   2.63705416E-04, 2.62165693E-04, 2.57019833E-04, 2.56859757E-04,
     &   2.56671704E-04, 2.56450821E-04, 2.56188437E-04, 2.55882020E-04,
     &   2.55537424E-04, 2.55150333E-04, 2.54715656E-04, 2.53617073E-04,
     &   2.52485254E-04, 2.51372619E-04, 2.50265720E-04, 2.49146806E-04,
     &   2.47997016E-04, 2.46819355E-04, 2.45622183E-04, 2.44378592E-04,
     &   2.43085394E-04, 2.41746365E-04, 2.40336621E-04, 2.38891556E-04,
     &   2.37389238E-04, 2.35812908E-04, 2.34126620E-04, 2.32439311E-04/
  
         DATA ((DAS  (I,J), I=2001,2100), J=2,2) /
     &   2.30644510E-04, 2.28824895E-04, 2.26706859E-04, 2.24512759E-04,
     &   2.22285653E-04, 2.20032196E-04, 2.17443388E-04, 2.14659228E-04,
     &   2.11768897E-04, 2.07792898E-04, 2.08620504E-04, 2.10257858E-04,
     &   2.12110268E-04, 2.13936590E-04, 2.15527514E-04, 2.17172725E-04,
     &   2.18839092E-04, 2.20472933E-04, 2.21832036E-04, 2.23201074E-04,
     &   2.24475507E-04, 2.25726431E-04, 2.26923566E-04, 2.28057509E-04,
     &   2.29144320E-04, 2.30196252E-04, 2.31183390E-04, 2.32111937E-04,
     &   2.33010861E-04, 2.33852379E-04, 2.34664408E-04, 2.35440503E-04,
     &   2.36174017E-04, 2.36876923E-04, 2.37564245E-04, 2.38243820E-04,
     &   2.38894018E-04, 2.39227727E-04, 2.39573188E-04, 2.39955281E-04,
     &   2.40412010E-04, 2.40987004E-04, 2.41512591E-04, 2.41244855E-04,
     &   2.40929100E-04, 2.38956964E-04, 2.36925117E-04, 2.37243018E-04,
     &   2.40175112E-04, 2.45887891E-04, 2.53364304E-04, 2.62922984E-04,
     &   2.73549325E-04, 2.86002616E-04, 2.99901247E-04, 3.14601251E-04,
     &   3.31772286E-04, 3.50204858E-04, 3.57330053E-04, 3.64455249E-04,
     &   3.49951799E-04, 3.36989881E-04, 3.26232190E-04, 3.16352305E-04,
     &   3.07968916E-04, 3.00127594E-04, 2.93370364E-04, 2.86966131E-04,
     &   2.81141349E-04, 2.76039534E-04, 2.71134665E-04, 2.66804466E-04,
     &   2.62611781E-04, 2.58886443E-04, 2.55258283E-04, 2.52017248E-04,
     &   2.48845226E-04, 2.45998775E-04, 2.43201221E-04, 2.40680979E-04,
     &   2.38300101E-04, 2.35947811E-04, 2.33818241E-04, 2.31707909E-04,
     &   2.29791964E-04, 2.27888241E-04, 2.26155488E-04, 2.24429652E-04,
     &   2.22855220E-04, 2.21283667E-04, 2.19847012E-04, 2.18410154E-04,
     &   2.17035631E-04, 2.15775584E-04, 2.14511990E-04, 2.13351712E-04,
     &   2.12186317E-04, 2.11114570E-04, 2.10036511E-04, 2.09043677E-04/
  
         DATA ((DAS  (I,J), I=2101,2174), J=2,2) /
     &   2.08043633E-04, 2.07121431E-04, 2.06191350E-04, 2.05332612E-04,
     &   2.04502577E-04, 2.03663994E-04, 2.02888428E-04, 2.02104030E-04,
     &   2.01377819E-04, 2.00642742E-04, 1.99961375E-04, 1.99270891E-04,
     &   1.98630404E-04, 1.97980760E-04, 1.97377628E-04, 1.96765353E-04,
     &   1.96170960E-04, 1.95618439E-04, 1.95056866E-04, 1.94518021E-04,
     &   1.93986570E-04, 1.93491824E-04, 1.92988243E-04, 1.92518952E-04,
     &   1.92041276E-04, 1.91595845E-04, 1.91141861E-04, 1.90718389E-04,
     &   1.90286518E-04, 1.89883431E-04, 1.89489781E-04, 1.89087974E-04,
     &   1.88712629E-04, 1.88329295E-04, 1.87971012E-04, 1.87604910E-04,
     &   1.87262554E-04, 1.86912704E-04, 1.86585268E-04, 1.86250353E-04,
     &   1.85936856E-04, 1.85616049E-04, 1.82529160E-04, 1.80029639E-04,
     &   1.79673092E-04, 1.69687785E-04, 1.67219638E-04, 1.63140082E-04,
     &   1.61698209E-04, 1.61629787E-04, 1.60241840E-04, 1.60238156E-04,
     &   1.59465513E-04, 1.59440309E-04, 1.59424225E-04, 1.59108822E-04,
     &   1.58918054E-04, 1.58885378E-04, 1.58832921E-04, 1.58757268E-04,
     &   1.58705345E-04, 1.58673330E-04, 1.58384674E-04, 1.58371079E-04,
     &   1.58294553E-04, 1.58280593E-04, 1.58237238E-04, 1.58194235E-04,
     &   1.58180426E-04, 1.58163193E-04, 1.58108857E-04, 1.58039528E-04,
     &   1.58010911E-04, 1.57833568E-04/


c      ++++ deghadslow15 ++++  

c update may 2016 
c 
c******************************************************************************
c*                                                                            *
c*    --- intRdat1 ---                              Tue May 17 04:39:19 2016  *
c*                                                                            *
c***************************# FJ@HU Berlin #***********************************
c 
c  "Delta alpha"               Update version (29/09/2015) 
c 
c ISCAN:  1, Ismooth: 0, IGS: 0, DSET: WAVE, iplot: 0
c IKE,Emin,Emax,NP,N,EPSREL:  1, 6.0000E-01, 1.4000E+00, 100, 10, 1.00E-02, 
c ALS(sta)(sys):  0.1184( 0.0007)( 0.0000), MZINP:   91.1876, MTOP:    173.20
c NF,IOR,ICHK: 5,4,2,ISU2: 2,NFL: 5,INAG: 1,inotail:  0, sin2W:   0.23153
c CHPTCUT,EC,ECUT: 0.3180,     5.20,    11.50
c 
  
       DATA  (ESA  (I), I=   1, 100) /
     &   0.28207018D0, 0.28800000D0, 0.29400000D0, 0.30000000D0,
     &   0.30600000D0, 0.30900000D0, 0.31500000D0, 0.32600000D0,
     &   0.33100000D0, 0.33600000D0, 0.34100000D0, 0.34600000D0,
     &   0.35200000D0, 0.35600000D0, 0.36100000D0, 0.36600000D0,
     &   0.37100000D0, 0.37600000D0, 0.38000000D0, 0.38500000D0,
     &   0.38900000D0, 0.39400000D0, 0.39800000D0, 0.40300000D0,
     &   0.40700000D0, 0.41100000D0, 0.41600000D0, 0.41871100D0,
     &   0.42000000D0, 0.42400000D0, 0.42800000D0, 0.43200000D0,
     &   0.43600000D0, 0.44000000D0, 0.44400000D0, 0.44800000D0,
     &   0.45200000D0, 0.45600000D0, 0.46000000D0, 0.46400000D0,
     &   0.46700000D0, 0.47100000D0, 0.47500000D0, 0.47800000D0,
     &   0.48200000D0, 0.48600000D0, 0.48900000D0, 0.49300000D0,
     &   0.49600000D0, 0.50000000D0, 0.52000000D0, 0.54000000D0,
     &   0.56600000D0, 0.59200000D0, 0.60000000D0, 0.60200000D0,
     &   0.60800000D0, 0.61600000D0, 0.62400000D0, 0.63200000D0,
     &   0.64000000D0, 0.64300000D0, 0.64800000D0, 0.65600000D0,
     &   0.66300000D0, 0.67100000D0, 0.67800000D0, 0.68600000D0,
     &   0.68800000D0, 0.69300000D0, 0.69600000D0, 0.70000000D0,
     &   0.70700000D0, 0.71400000D0, 0.72100000D0, 0.72800000D0,
     &   0.73500000D0, 0.74200000D0, 0.74800000D0, 0.75000000D0,
     &   0.75500000D0, 0.75744000D0, 0.76007300D0, 0.76207300D0,
     &   0.76507300D0, 0.76851700D0, 0.77060500D0, 0.77205400D0,
     &   0.77314300D0, 0.77400300D0, 0.77470900D0, 0.77530500D0,
     &   0.77582000D0, 0.77627100D0, 0.77667300D0, 0.77726600D0,
     &   0.77746600D0, 0.77766900D0, 0.77795100D0, 0.77821500D0/
       DATA  (ESA  (I), I= 101, 200) /
     &   0.77846300D0, 0.77869800D0, 0.77892200D0, 0.77913700D0,
     &   0.77934400D0, 0.77954500D0, 0.77974200D0, 0.77993500D0,
     &   0.78012700D0, 0.78031800D0, 0.78051000D0, 0.78070600D0,
     &   0.78090800D0, 0.78112100D0, 0.78135100D0, 0.78161200D0,
     &   0.78193600D0, 0.78268000D0, 0.78341500D0, 0.78373400D0,
     &   0.78399100D0, 0.78421700D0, 0.78442600D0, 0.78462500D0,
     &   0.78481700D0, 0.78500500D0, 0.78519100D0, 0.78537800D0,
     &   0.78556600D0, 0.78575600D0, 0.78595100D0, 0.78615100D0,
     &   0.78635800D0, 0.78657300D0, 0.78679800D0, 0.78703400D0,
     &   0.78728400D0, 0.78755000D0, 0.78773500D0, 0.78793500D0,
     &   0.78814300D0, 0.78847900D0, 0.78884800D0, 0.78925800D0,
     &   0.78971900D0, 0.79024600D0, 0.79085900D0, 0.79158800D0,
     &   0.79248000D0, 0.79361500D0, 0.79513900D0, 0.79736300D0,
     &   0.79800000D0, 0.80000000D0, 0.80200000D0, 0.80400000D0,
     &   0.80600000D0, 0.80800000D0, 0.81200000D0, 0.81400000D0,
     &   0.81600000D0, 0.82000000D0, 0.82200000D0, 0.82400000D0,
     &   0.82600000D0, 0.82800000D0, 0.83000000D0, 0.83100000D0,
     &   0.83700000D0, 0.84300000D0, 0.84900000D0, 0.85400000D0,
     &   0.86000000D0, 0.86600000D0, 0.87200000D0, 0.87700000D0,
     &   0.88300000D0, 0.88900000D0, 0.89400000D0, 0.89700000D0,
     &   0.90250000D0, 0.90500000D0, 0.91100000D0, 0.91600000D0,
     &   0.92200000D0, 0.92700000D0, 0.93300000D0, 0.93800000D0,
     &   0.94300000D0, 0.94900000D0, 0.95200000D0, 0.95400000D0,
     &   0.95800000D0, 0.96000000D0, 0.96200000D0, 0.96400000D0,
     &   0.96600000D0, 0.96800000D0, 0.97000000D0, 0.97400000D0/
       DATA  (ESA  (I), I= 201, 300) /
     &   0.97800000D0, 0.98000000D0, 0.98070000D0, 0.98140000D0,
     &   0.98210000D0, 0.98280000D0, 0.98350000D0, 0.98420000D0,
     &   0.98490000D0, 0.98560000D0, 0.98630000D0, 0.98700000D0,
     &   0.98770000D0, 0.98840000D0, 0.98910000D0, 0.98980000D0,
     &   0.99050000D0, 0.99120000D0, 0.99190000D0, 0.99260000D0,
     &   0.99330000D0, 0.99400000D0, 0.99470000D0, 0.99540000D0,
     &   0.99610000D0, 0.99680000D0, 0.99750000D0, 0.99820000D0,
     &   0.99890000D0, 0.99960000D0, 0.99990000D0, 1.00001000D0,
     &   1.00030000D0, 1.00100000D0, 1.00170000D0, 1.00240000D0,
     &   1.00310000D0, 1.00380000D0, 1.00450000D0, 1.00520000D0,
     &   1.00590000D0, 1.00660000D0, 1.00730000D0, 1.00800000D0,
     &   1.00870000D0, 1.00940000D0, 1.01010000D0, 1.01080000D0,
     &   1.01150000D0, 1.01220000D0, 1.01290000D0, 1.01360000D0,
     &   1.01430000D0, 1.01500000D0, 1.01520000D0, 1.01540000D0,
     &   1.01560000D0, 1.01580000D0, 1.01600000D0, 1.01620000D0,
     &   1.01640000D0, 1.01660000D0, 1.01680000D0, 1.01700000D0,
     &   1.01720000D0, 1.01740000D0, 1.01770000D0, 1.01790000D0,
     &   1.01800000D0, 1.01820000D0, 1.01840000D0, 1.01860000D0,
     &   1.01880000D0, 1.01900000D0, 1.01920000D0, 1.01940000D0,
     &   1.01960000D0, 1.01980000D0, 1.02000000D0, 1.02020000D0,
     &   1.02040000D0, 1.02060000D0, 1.02080000D0, 1.02100000D0,
     &   1.02120000D0, 1.02140000D0, 1.02160000D0, 1.02180000D0,
     &   1.02200000D0, 1.02220000D0, 1.02240000D0, 1.02270000D0,
     &   1.02290000D0, 1.02300000D0, 1.02320000D0, 1.02340000D0,
     &   1.02360000D0, 1.02380000D0, 1.02400000D0, 1.02420000D0/
       DATA  (ESA  (I), I= 301, 400) /
     &   1.02440000D0, 1.02460000D0, 1.02480000D0, 1.02500000D0,
     &   1.02570000D0, 1.02640000D0, 1.02710000D0, 1.02780000D0,
     &   1.02850000D0, 1.02920000D0, 1.02990000D0, 1.03060000D0,
     &   1.03130000D0, 1.03200000D0, 1.03270000D0, 1.03340000D0,
     &   1.03410000D0, 1.03480000D0, 1.03550000D0, 1.03620000D0,
     &   1.03690000D0, 1.03760000D0, 1.03830000D0, 1.03900000D0,
     &   1.03970000D0, 1.03999000D0, 1.04001000D0, 1.04040000D0,
     &   1.04110000D0, 1.04180000D0, 1.04250000D0, 1.04320000D0,
     &   1.04390000D0, 1.04460000D0, 1.04530000D0, 1.04600000D0,
     &   1.04670000D0, 1.04740000D0, 1.04810000D0, 1.04880000D0,
     &   1.04950000D0, 1.05020000D0, 1.05090000D0, 1.05160000D0,
     &   1.05230000D0, 1.05300000D0, 1.05370000D0, 1.05440000D0,
     &   1.05510000D0, 1.05580000D0, 1.05650000D0, 1.05720000D0,
     &   1.05790000D0, 1.05860000D0, 1.05930000D0, 1.06000000D0,
     &   1.06200000D0, 1.06700000D0, 1.07700000D0, 1.08700000D0,
     &   1.09700000D0, 1.10700000D0, 1.11700000D0, 1.12700000D0,
     &   1.13700000D0, 1.13800000D0, 1.15200000D0, 1.16300000D0,
     &   1.16700000D0, 1.17700000D0, 1.18700000D0, 1.18800000D0,
     &   1.19700000D0, 1.20700000D0, 1.21200000D0, 1.21700000D0,
     &   1.22700000D0, 1.23700000D0, 1.24700000D0, 1.25700000D0,
     &   1.26300000D0, 1.26700000D0, 1.27700000D0, 1.28700000D0,
     &   1.28800000D0, 1.29700000D0, 1.30700000D0, 1.31200000D0,
     &   1.31700000D0, 1.32700000D0, 1.33700000D0, 1.34700000D0,
     &   1.35000000D0, 1.35700000D0, 1.36200000D0, 1.36700000D0,
     &   1.37700000D0, 1.38700000D0, 1.38800000D0, 1.39500000D0/
       DATA  (ESA  (I), I= 401, 500) /
     &   1.41000000D0, 1.41900000D0, 1.42700000D0, 1.43500000D0,
     &   1.43800000D0, 1.45200000D0, 1.45700000D0, 1.46000000D0,
     &   1.46400000D0, 1.47200000D0, 1.48100000D0, 1.49000000D0,
     &   1.49100000D0, 1.50000000D0, 1.50100000D0, 1.50600000D0,
     &   1.51000000D0, 1.52300000D0, 1.52500000D0, 1.53800000D0,
     &   1.53900000D0, 1.54000000D0, 1.55000000D0, 1.56000000D0,
     &   1.56200000D0, 1.57400000D0, 1.57500000D0, 1.58000000D0,
     &   1.58100000D0, 1.58700000D0, 1.59000000D0, 1.59900000D0,
     &   1.60000000D0, 1.61000000D0, 1.61900000D0, 1.62400000D0,
     &   1.62500000D0, 1.63800000D0, 1.64000000D0, 1.65000000D0,
     &   1.65700000D0, 1.66000000D0, 1.66200000D0, 1.66300000D0,
     &   1.67300000D0, 1.67500000D0, 1.68100000D0, 1.68700000D0,
     &   1.69000000D0, 1.70000000D0, 1.71000000D0, 1.71200000D0,
     &   1.71300000D0, 1.71900000D0, 1.72200000D0, 1.72500000D0,
     &   1.73800000D0, 1.74000000D0, 1.74600000D0, 1.75000000D0,
     &   1.75600000D0, 1.75700000D0, 1.76000000D0, 1.76200000D0,
     &   1.76400000D0, 1.76900000D0, 1.77500000D0, 1.78100000D0,
     &   1.79000000D0, 1.79900000D0, 1.80000000D0, 1.81200000D0,
     &   1.81900000D0, 1.82500000D0, 1.82600000D0, 1.83400000D0,
     &   1.83500000D0, 1.83800000D0, 1.84500000D0, 1.85700000D0,
     &   1.85900000D0, 1.87400000D0, 1.88100000D0, 1.88600000D0,
     &   1.90000000D0, 1.91900000D0, 1.92500000D0, 1.93000000D0,
     &   1.93800000D0, 1.94000000D0, 1.94500000D0, 1.94600000D0,
     &   1.95000000D0, 1.95500000D0, 1.96000000D0, 1.96200000D0,
     &   1.98100000D0, 1.99050000D0, 2.00500000D0, 2.05000000D0/
       DATA  (ESA  (I), I= 501, 600) /
     &   2.10000000D0, 2.15010000D0, 2.22000000D0, 2.26000000D0,
     &   2.30000000D0, 2.40000000D0, 2.45000000D0, 2.50000000D0,
     &   2.55000000D0, 2.60000000D0, 2.65000000D0, 2.70000000D0,
     &   2.75000000D0, 2.80000000D0, 2.85000000D0, 2.90000000D0,
     &   2.91000000D0, 2.92000000D0, 2.93000000D0, 2.94000000D0,
     &   2.95000000D0, 2.96000000D0, 2.97000000D0, 2.97200000D0,
     &   2.97400000D0, 2.97600000D0, 2.97800000D0, 2.98000000D0,
     &   2.98200000D0, 2.98400000D0, 2.98600000D0, 2.98800000D0,
     &   2.99000000D0, 2.99900000D0, 3.00500000D0, 3.00800000D0,
     &   3.01000000D0, 3.01200000D0, 3.01400000D0, 3.01600000D0,
     &   3.01800000D0, 3.02000000D0, 3.02100000D0, 3.02200000D0,
     &   3.02300000D0, 3.02400000D0, 3.02500000D0, 3.02600000D0,
     &   3.02700000D0, 3.02800000D0, 3.02900000D0, 3.03000000D0,
     &   3.03100000D0, 3.03200000D0, 3.03300000D0, 3.03400000D0,
     &   3.03500000D0, 3.03600000D0, 3.03700000D0, 3.03800000D0,
     &   3.03900000D0, 3.04000000D0, 3.04100000D0, 3.04200000D0,
     &   3.04300000D0, 3.04400000D0, 3.04500000D0, 3.04600000D0,
     &   3.04700000D0, 3.04800000D0, 3.04900000D0, 3.05000000D0,
     &   3.05100000D0, 3.05200000D0, 3.05300000D0, 3.05400000D0,
     &   3.05500000D0, 3.05600000D0, 3.05700000D0, 3.05800000D0,
     &   3.05900000D0, 3.06000000D0, 3.06050000D0, 3.06100000D0,
     &   3.06150000D0, 3.06200000D0, 3.06250000D0, 3.06300000D0,
     &   3.06350000D0, 3.06400000D0, 3.06450000D0, 3.06500000D0,
     &   3.06550000D0, 3.06600000D0, 3.06650000D0, 3.06700000D0,
     &   3.06750000D0, 3.06800000D0, 3.06850000D0, 3.06900000D0/
       DATA  (ESA  (I), I= 601, 700) /
     &   3.06950000D0, 3.07000000D0, 3.07050000D0, 3.07100000D0,
     &   3.07150000D0, 3.07200000D0, 3.07250000D0, 3.07300000D0,
     &   3.07350000D0, 3.07400000D0, 3.07450000D0, 3.07500000D0,
     &   3.07550000D0, 3.07600000D0, 3.07650000D0, 3.07700000D0,
     &   3.07750000D0, 3.07800000D0, 3.07850000D0, 3.07900000D0,
     &   3.07970000D0, 3.08000000D0, 3.08050000D0, 3.08100000D0,
     &   3.08150000D0, 3.08200000D0, 3.08250000D0, 3.08300000D0,
     &   3.08350000D0, 3.08400000D0, 3.08450000D0, 3.08500000D0,
     &   3.08550000D0, 3.08587000D0, 3.08688300D0, 3.08789600D0,
     &   3.08890900D0, 3.08992200D0, 3.09093500D0, 3.09194800D0,
     &   3.09296100D0, 3.09397400D0, 3.09498700D0, 3.09600000D0,
     &   3.09605000D0, 3.09610000D0, 3.09615000D0, 3.09620000D0,
     &   3.09625000D0, 3.09630000D0, 3.09635000D0, 3.09640000D0,
     &   3.09645000D0, 3.09650000D0, 3.09655000D0, 3.09660000D0,
     &   3.09665000D0, 3.09665500D0, 3.09673500D0, 3.09677100D0,
     &   3.09679300D0, 3.09680900D0, 3.09682000D0, 3.09682900D0,
     &   3.09683600D0, 3.09684300D0, 3.09684800D0, 3.09685300D0,
     &   3.09685700D0, 3.09686100D0, 3.09686400D0, 3.09686800D0,
     &   3.09687000D0, 3.09687300D0, 3.09687600D0, 3.09687800D0,
     &   3.09688100D0, 3.09688300D0, 3.09688500D0, 3.09688700D0,
     &   3.09688900D0, 3.09689100D0, 3.09689300D0, 3.09689500D0,
     &   3.09689700D0, 3.09689900D0, 3.09690100D0, 3.09690400D0,
     &   3.09690600D0, 3.09690900D0, 3.09691200D0, 3.09692000D0,
     &   3.09692800D0, 3.09693100D0, 3.09693400D0, 3.09693600D0,
     &   3.09693900D0, 3.09694100D0, 3.09694300D0, 3.09694500D0/
       DATA  (ESA  (I), I= 701, 800) /
     &   3.09694700D0, 3.09694900D0, 3.09695100D0, 3.09695300D0,
     &   3.09695500D0, 3.09695700D0, 3.09695900D0, 3.09696200D0,
     &   3.09696400D0, 3.09696700D0, 3.09697000D0, 3.09697200D0,
     &   3.09697600D0, 3.09697900D0, 3.09698300D0, 3.09698700D0,
     &   3.09699200D0, 3.09699700D0, 3.09700400D0, 3.09701100D0,
     &   3.09702000D0, 3.09703100D0, 3.09704700D0, 3.09706900D0,
     &   3.09710500D0, 3.09715000D0, 3.09720000D0, 3.09725000D0,
     &   3.09730000D0, 3.09735000D0, 3.09740000D0, 3.09745000D0,
     &   3.09750000D0, 3.09755000D0, 3.09760000D0, 3.09765000D0,
     &   3.09770000D0, 3.09775000D0, 3.09780000D0, 3.09785000D0,
     &   3.09790000D0, 3.09795000D0, 3.09800000D0, 3.09898700D0,
     &   3.09997400D0, 3.10096100D0, 3.10194800D0, 3.10293500D0,
     &   3.10392200D0, 3.10490900D0, 3.10589600D0, 3.10688300D0,
     &   3.10787000D0, 3.10800000D0, 3.10900000D0, 3.11000000D0,
     &   3.11100000D0, 3.11200000D0, 3.11300000D0, 3.11400000D0,
     &   3.11500000D0, 3.11600000D0, 3.11700000D0, 3.11800000D0,
     &   3.11900000D0, 3.12000000D0, 3.12100000D0, 3.12200000D0,
     &   3.12300000D0, 3.12400000D0, 3.12500000D0, 3.12600000D0,
     &   3.12700000D0, 3.12800000D0, 3.12900000D0, 3.13000000D0,
     &   3.13100000D0, 3.13200000D0, 3.13300000D0, 3.13400000D0,
     &   3.13500000D0, 3.13600000D0, 3.13700000D0, 3.13800000D0,
     &   3.13900000D0, 3.14000000D0, 3.14100000D0, 3.14200000D0,
     &   3.14300000D0, 3.14400000D0, 3.14500000D0, 3.14600000D0,
     &   3.14700000D0, 3.14800000D0, 3.14900000D0, 3.15000000D0,
     &   3.15100000D0, 3.15200000D0, 3.15300000D0, 3.15400000D0/
       DATA  (ESA  (I), I= 801, 900) /
     &   3.15500000D0, 3.15600000D0, 3.15700000D0, 3.15800000D0,
     &   3.15900000D0, 3.16000000D0, 3.16100000D0, 3.16200000D0,
     &   3.16300000D0, 3.16400000D0, 3.16500000D0, 3.16600000D0,
     &   3.16700000D0, 3.16800000D0, 3.16900000D0, 3.17000000D0,
     &   3.17100000D0, 3.17200000D0, 3.17300000D0, 3.17400000D0,
     &   3.17500000D0, 3.17600000D0, 3.17700000D0, 3.17800000D0,
     &   3.17900000D0, 3.18000000D0, 3.18100000D0, 3.18200000D0,
     &   3.18300000D0, 3.18400000D0, 3.18500000D0, 3.18600000D0,
     &   3.18700000D0, 3.18800000D0, 3.18900000D0, 3.19000000D0,
     &   3.19100000D0, 3.19200000D0, 3.19300000D0, 3.19400000D0,
     &   3.19500000D0, 3.19600000D0, 3.19700000D0, 3.19800000D0,
     &   3.19900000D0, 3.19990000D0, 3.20100000D0, 3.20200000D0,
     &   3.20300000D0, 3.20400000D0, 3.20500000D0, 3.20600000D0,
     &   3.20700000D0, 3.20800000D0, 3.20900000D0, 3.21000000D0,
     &   3.25000000D0, 3.30000000D0, 3.37000000D0, 3.45000000D0,
     &   3.50000000D0, 3.52000000D0, 3.54000000D0, 3.56000000D0,
     &   3.57500000D0, 3.59000000D0, 3.59500000D0, 3.60500000D0,
     &   3.61000000D0, 3.61500000D0, 3.62000000D0, 3.62500000D0,
     &   3.63000000D0, 3.63500000D0, 3.64000000D0, 3.64500000D0,
     &   3.65000000D0, 3.65200000D0, 3.65400000D0, 3.65600000D0,
     &   3.65800000D0, 3.66000000D0, 3.66200000D0, 3.66300000D0,
     &   3.66400000D0, 3.66500000D0, 3.66600000D0, 3.66700000D0,
     &   3.66800000D0, 3.66900000D0, 3.67000000D0, 3.67100000D0,
     &   3.67200000D0, 3.67300000D0, 3.67400000D0, 3.67496000D0,
     &   3.67551000D0, 3.67606000D0, 3.67661000D0, 3.67716000D0/
       DATA  (ESA  (I), I= 901,1000) /
     &   3.67771000D0, 3.67826000D0, 3.67881000D0, 3.67936000D0,
     &   3.67991000D0, 3.68046000D0, 3.68101000D0, 3.68156000D0,
     &   3.68211000D0, 3.68266000D0, 3.68321000D0, 3.68376000D0,
     &   3.68431000D0, 3.68486000D0, 3.68527300D0, 3.68541000D0,
     &   3.68552000D0, 3.68569900D0, 3.68574600D0, 3.68578100D0,
     &   3.68580900D0, 3.68583200D0, 3.68585100D0, 3.68586800D0,
     &   3.68588300D0, 3.68589500D0, 3.68590700D0, 3.68591800D0,
     &   3.68592800D0, 3.68593700D0, 3.68594600D0, 3.68595300D0,
     &   3.68596100D0, 3.68596800D0, 3.68597500D0, 3.68598200D0,
     &   3.68598800D0, 3.68599500D0, 3.68600100D0, 3.68600700D0,
     &   3.68601300D0, 3.68602000D0, 3.68602600D0, 3.68603300D0,
     &   3.68603900D0, 3.68604700D0, 3.68605500D0, 3.68606600D0,
     &   3.68607500D0, 3.68608500D0, 3.68609000D0, 3.68609500D0,
     &   3.68610000D0, 3.68611400D0, 3.68612500D0, 3.68613300D0,
     &   3.68614100D0, 3.68614700D0, 3.68615400D0, 3.68616000D0,
     &   3.68616700D0, 3.68617300D0, 3.68617900D0, 3.68618500D0,
     &   3.68619200D0, 3.68619800D0, 3.68620500D0, 3.68621200D0,
     &   3.68621900D0, 3.68622700D0, 3.68623400D0, 3.68624300D0,
     &   3.68625200D0, 3.68626200D0, 3.68627300D0, 3.68628500D0,
     &   3.68629700D0, 3.68631200D0, 3.68632900D0, 3.68634800D0,
     &   3.68637100D0, 3.68639900D0, 3.68643400D0, 3.68648100D0,
     &   3.68654800D0, 3.68651000D0, 3.68660000D0, 3.68667000D0,
     &   3.68672000D0, 3.68685000D0, 3.68690700D0, 3.68706000D0,
     &   3.68761000D0, 3.68816000D0, 3.68871000D0, 3.68926000D0,
     &   3.68981000D0, 3.69036000D0, 3.69091000D0, 3.69146000D0/
       DATA  (ESA  (I), I=1001,1100) /
     &   3.69201000D0, 3.69256000D0, 3.69311000D0, 3.69366000D0,
     &   3.69421000D0, 3.69476000D0, 3.69531000D0, 3.69586000D0,
     &   3.69641000D0, 3.69696000D0, 3.69796000D0, 3.69950000D0,
     &   3.70050000D0, 3.70250000D0, 3.70450000D0, 3.70700000D0,
     &   3.71200000D0, 3.72128100D0, 3.73082800D0, 3.73660200D0,
     &   3.74060200D0, 3.74360300D0, 3.74597300D0, 3.74791700D0,
     &   3.74955600D0, 3.75096800D0, 3.75220600D0, 3.75330900D0,
     &   3.75430100D0, 3.75520500D0, 3.75603600D0, 3.75680500D0,
     &   3.75752300D0, 3.75819800D0, 3.75883700D0, 3.75944400D0,
     &   3.76002600D0, 3.76058600D0, 3.76112800D0, 3.76165500D0,
     &   3.76217200D0, 3.76268100D0, 3.76318700D0, 3.76369200D0,
     &   3.76420200D0, 3.76472200D0, 3.76526100D0, 3.76582800D0,
     &   3.76644200D0, 3.76713800D0, 3.76800500D0, 3.77000000D0,
     &   3.77199500D0, 3.77286200D0, 3.77355800D0, 3.77417200D0,
     &   3.77474000D0, 3.77527800D0, 3.77579800D0, 3.77630800D0,
     &   3.77681400D0, 3.77731900D0, 3.77782800D0, 3.77834500D0,
     &   3.77887200D0, 3.77941400D0, 3.77997400D0, 3.78055600D0,
     &   3.78116300D0, 3.78180200D0, 3.78247700D0, 3.78319500D0,
     &   3.78396400D0, 3.78479500D0, 3.78569900D0, 3.78669100D0,
     &   3.78779400D0, 3.78903200D0, 3.79044400D0, 3.79208300D0,
     &   3.79402700D0, 3.79639700D0, 3.79939800D0, 3.80339800D0,
     &   3.80917200D0, 3.81871900D0, 3.82200000D0, 3.83200000D0,
     &   3.85000000D0, 3.85200000D0, 3.86700000D0, 3.87300000D0,
     &   3.87800000D0, 3.88400000D0, 3.89000000D0, 3.89600000D0,
     &   3.90000000D0, 3.90200000D0, 3.90800000D0, 3.91400000D0/
       DATA  (ESA  (I), I=1101,1200) /
     &   3.92000000D0, 3.92600000D0, 3.93200000D0, 3.93800000D0,
     &   3.94400000D0, 3.95000000D0, 3.95600000D0, 3.96200000D0,
     &   3.96800000D0, 3.97400000D0, 3.98000000D0, 3.98600000D0,
     &   3.99200000D0, 3.99800000D0, 4.00000000D0, 4.00400000D0,
     &   4.01000000D0, 4.01600000D0, 4.02000000D0, 4.02200000D0,
     &   4.02800000D0, 4.03000000D0, 4.03400000D0, 4.04000000D0,
     &   4.04600000D0, 4.05000000D0, 4.05200000D0, 4.05800000D0,
     &   4.06400000D0, 4.07000000D0, 4.07600000D0, 4.08200000D0,
     &   4.08800000D0, 4.09000000D0, 4.09400000D0, 4.10000000D0,
     &   4.10500000D0, 4.10900000D0, 4.11000000D0, 4.11300000D0,
     &   4.11800000D0, 4.12100000D0, 4.12500000D0, 4.12900000D0,
     &   4.13000000D0, 4.13300000D0, 4.13600000D0, 4.14100000D0,
     &   4.14700000D0, 4.15000000D0, 4.15300000D0, 4.15900000D0,
     &   4.16500000D0, 4.17100000D0, 4.17700000D0, 4.18000000D0,
     &   4.18200000D0, 4.18500000D0, 4.19000000D0, 4.19500000D0,
     &   4.19900000D0, 4.20100000D0, 4.20800000D0, 4.21200000D0,
     &   4.21400000D0, 4.22000000D0, 4.22600000D0, 4.23200000D0,
     &   4.23800000D0, 4.24000000D0, 4.24400000D0, 4.25000000D0,
     &   4.25600000D0, 4.26200000D0, 4.26800000D0, 4.27400000D0,
     &   4.28000000D0, 4.29200000D0, 4.30000000D0, 4.30400000D0,
     &   4.31000000D0, 4.31600000D0, 4.32800000D0, 4.33000000D0,
     &   4.34000000D0, 4.35000000D0, 4.35200000D0, 4.36400000D0,
     &   4.37600000D0, 4.38000000D0, 4.38200000D0, 4.38800000D0,
     &   4.39000000D0, 4.39400000D0, 4.40000000D0, 4.40600000D0,
     &   4.41000000D0, 4.41200000D0, 4.41800000D0, 4.42400000D0/
       DATA  (ESA  (I), I=1201,1300) /
     &   4.43000000D0, 4.43600000D0, 4.44000000D0, 4.44200000D0,
     &   4.44800000D0, 4.45000000D0, 4.46000000D0, 4.47200000D0,
     &   4.48400000D0, 4.49600000D0, 4.50450000D0, 4.54000000D0,
     &   4.55000000D0, 4.57000000D0, 4.60000000D0, 4.62000000D0,
     &   4.65000000D0, 4.66000000D0, 4.70000000D0, 4.71000000D0,
     &   4.80000000D0, 4.90000000D0, 4.98000000D0, 5.06500000D0,
     &   5.15000000D0, 5.22500000D0, 5.25000000D0, 5.30000000D0,
     &   5.40000000D0, 5.45000000D0, 5.50000000D0, 5.60000000D0,
     &   5.70000000D0, 5.75000000D0, 5.80000000D0, 5.85000000D0,
     &   5.90000000D0, 5.95000000D0, 6.00000000D0, 6.05000000D0,
     &   6.10000000D0, 6.15000000D0, 6.20000000D0, 6.25000000D0,
     &   6.30000000D0, 6.35000000D0, 6.40000000D0, 6.45000000D0,
     &   6.50000000D0, 6.55000000D0, 6.60000000D0, 6.65000000D0,
     &   6.70000000D0, 6.75000000D0, 6.80000000D0, 6.85000000D0,
     &   6.90000000D0, 6.95000000D0, 7.00000000D0, 7.05000000D0,
     &   7.10000000D0, 7.15000000D0, 7.20000000D0, 7.25000000D0,
     &   7.30000000D0, 7.35000000D0, 7.40000000D0, 7.44000000D0,
     &   7.50000000D0, 7.60000000D0, 7.70000000D0, 7.80000000D0,
     &   7.90000000D0, 8.00000000D0, 8.10000000D0, 8.20000000D0,
     &   8.30000000D0, 8.40000000D0, 8.50000000D0, 8.60000000D0,
     &   8.70000000D0, 8.80000000D0, 8.90000000D0, 8.91000000D0,
     &   9.00000000D0, 9.10000000D0, 9.20000000D0, 9.28000000D0,
     &   9.30000000D0, 9.35000000D0, 9.39000000D0, 9.40000000D0,
     &   9.41000000D0, 9.41500000D0, 9.42000000D0, 9.42400000D0,
     &   9.42800000D0, 9.43000000D0, 9.43200000D0, 9.43400000D0/
       DATA  (ESA  (I), I=1301,1400) /
     &   9.43600000D0, 9.43800000D0, 9.44000000D0, 9.44100000D0,
     &   9.44200000D0, 9.44300000D0, 9.44400000D0, 9.44500000D0,
     &   9.44600000D0, 9.44700000D0, 9.44800000D0, 9.44900000D0,
     &   9.44937000D0, 9.44959000D0, 9.44981000D0, 9.45003000D0,
     &   9.45025000D0, 9.45047000D0, 9.45069000D0, 9.45091000D0,
     &   9.45113000D0, 9.45135000D0, 9.45157000D0, 9.45179000D0,
     &   9.45201000D0, 9.45223000D0, 9.45245000D0, 9.45267000D0,
     &   9.45289000D0, 9.45311000D0, 9.45333000D0, 9.45355000D0,
     &   9.45377000D0, 9.45399000D0, 9.45421000D0, 9.45443000D0,
     &   9.45465000D0, 9.45487000D0, 9.45509000D0, 9.45531000D0,
     &   9.45553000D0, 9.45575000D0, 9.45597000D0, 9.45619000D0,
     &   9.45641000D0, 9.45663000D0, 9.45685000D0, 9.45707000D0,
     &   9.45729000D0, 9.45751000D0, 9.45773000D0, 9.45795000D0,
     &   9.45817000D0, 9.45839000D0, 9.45861000D0, 9.45883000D0,
     &   9.45905000D0, 9.45927000D0, 9.45949000D0, 9.45971000D0,
     &   9.45993000D0, 9.46000000D0, 9.46014595D0, 9.46019268D0,
     &   9.46021366D0, 9.46022606D0, 9.46023464D0, 9.46024132D0,
     &   9.46024704D0, 9.46025085D0, 9.46025467D0, 9.46025848D0,
     &   9.46026039D0, 9.46026325D0, 9.46026516D0, 9.46026802D0,
     &   9.46026897D0, 9.46027088D0, 9.46027279D0, 9.46027470D0,
     &   9.46027565D0, 9.46027660D0, 9.46027851D0, 9.46027946D0,
     &   9.46028042D0, 9.46028233D0, 9.46028328D0, 9.46028423D0,
     &   9.46028519D0, 9.46028709D0, 9.46028805D0, 9.46028900D0,
     &   9.46029091D0, 9.46029186D0, 9.46029377D0, 9.46029568D0,
     &   9.46030045D0, 9.46030426D0, 9.46030617D0, 9.46030807D0/
       DATA  (ESA  (I), I=1401,1500) /
     &   9.46030998D0, 9.46031094D0, 9.46031189D0, 9.46031284D0,
     &   9.46031475D0, 9.46031570D0, 9.46031666D0, 9.46031761D0,
     &   9.46031952D0, 9.46032047D0, 9.46032143D0, 9.46032333D0,
     &   9.46032429D0, 9.46032619D0, 9.46032715D0, 9.46032906D0,
     &   9.46033096D0, 9.46033287D0, 9.46033478D0, 9.46033669D0,
     &   9.46033955D0, 9.46034145D0, 9.46034527D0, 9.46034908D0,
     &   9.46035290D0, 9.46035862D0, 9.46036530D0, 9.46037388D0,
     &   9.46038628D0, 9.46040726D0, 9.46045494D0, 9.46059000D0,
     &   9.46081000D0, 9.46103000D0, 9.46125000D0, 9.46147000D0,
     &   9.46169000D0, 9.46191000D0, 9.46213000D0, 9.46235000D0,
     &   9.46257000D0, 9.46279000D0, 9.46301000D0, 9.46323000D0,
     &   9.46345000D0, 9.46367000D0, 9.46389000D0, 9.46411000D0,
     &   9.46433000D0, 9.46455000D0, 9.46477000D0, 9.46499000D0,
     &   9.46521000D0, 9.46543000D0, 9.46565000D0, 9.46587000D0,
     &   9.46609000D0, 9.46631000D0, 9.46653000D0, 9.46675000D0,
     &   9.46697000D0, 9.46719000D0, 9.46741000D0, 9.46763000D0,
     &   9.46785000D0, 9.46807000D0, 9.46829000D0, 9.46851000D0,
     &   9.46873000D0, 9.46895000D0, 9.46917000D0, 9.46939000D0,
     &   9.46961000D0, 9.46983000D0, 9.47005000D0, 9.47027000D0,
     &   9.47049000D0, 9.47071000D0, 9.47093000D0, 9.47115000D0,
     &   9.47137000D0, 9.47337000D0, 9.47500000D0, 9.47700000D0,
     &   9.48000000D0, 9.48400000D0, 9.49000000D0, 9.49500000D0,
     &   9.50500000D0, 9.51000000D0, 9.51500000D0, 9.52000000D0,
     &   9.52500000D0, 9.53000000D0, 9.54000000D0, 9.55000000D0,
     &   9.60000000D0, 9.70000000D0, 9.80000000D0, 9.90000000D0/
       DATA  (ESA  (I), I=1501,1600) /
     &   9.95000000D0, 9.97500000D0, 9.99000000D0,10.00000000D0,
     &  10.01000000D0,10.01100000D0,10.01200000D0,10.01226000D0,
     &  10.01281000D0,10.01336000D0,10.01391000D0,10.01446000D0,
     &  10.01501000D0,10.01556000D0,10.01611000D0,10.01666000D0,
     &  10.01721000D0,10.01776000D0,10.01831000D0,10.01886000D0,
     &  10.01941000D0,10.01996000D0,10.02051000D0,10.02106000D0,
     &  10.02161000D0,10.02216000D0,10.02250000D0,10.02253750D0,
     &  10.02257500D0,10.02261250D0,10.02265000D0,10.02268750D0,
     &  10.02272500D0,10.02276250D0,10.02280000D0,10.02283750D0,
     &  10.02287500D0,10.02291250D0,10.02295000D0,10.02298750D0,
     &  10.02302500D0,10.02306250D0,10.02310000D0,10.02313750D0,
     &  10.02317500D0,10.02321250D0,10.02322000D0,10.02322500D0,
     &  10.02323000D0,10.02323400D0,10.02323700D0,10.02324000D0,
     &  10.02324200D0,10.02324400D0,10.02324600D0,10.02324800D0,
     &  10.02324930D0,10.02324950D0,10.02324960D0,10.02324980D0,
     &  10.02324988D0,10.02324994D0,10.02325000D0,10.02325060D0,
     &  10.02325150D0,10.02325250D0,10.02325340D0,10.02325440D0,
     &  10.02325540D0,10.02325630D0,10.02325680D0,10.02325730D0,
     &  10.02325780D0,10.02325850D0,10.02325900D0,10.02325950D0,
     &  10.02325980D0,10.02325990D0,10.02325995D0,10.02326000D0,
     &  10.02326005D0,10.02326010D0,10.02326050D0,10.02326100D0,
     &  10.02326150D0,10.02326390D0,10.02326490D0,10.02326680D0,
     &  10.02326770D0,10.02326870D0,10.02326970D0,10.02327060D0,
     &  10.02327160D0,10.02327250D0,10.02327350D0,10.02327440D0,
     &  10.02327540D0,10.02327630D0,10.02327730D0,10.02327820D0/
       DATA  (ESA  (I), I=1601,1700) /
     &  10.02328010D0,10.02328110D0,10.02328210D0,10.02328300D0,
     &  10.02328490D0,10.02328590D0,10.02328750D0,10.02329000D0,
     &  10.02330000D0,10.02331000D0,10.02332000D0,10.02333000D0,
     &  10.02334000D0,10.02335000D0,10.02336250D0,10.02339000D0,
     &  10.02340000D0,10.02341000D0,10.02342000D0,10.02343750D0,
     &  10.02347500D0,10.02351250D0,10.02355000D0,10.02358750D0,
     &  10.02362500D0,10.02366250D0,10.02370000D0,10.02373750D0,
     &  10.02377500D0,10.02381250D0,10.02385000D0,10.02388750D0,
     &  10.02392500D0,10.02396250D0,10.02400000D0,10.02436000D0,
     &  10.02491000D0,10.02546000D0,10.02656000D0,10.02766000D0,
     &  10.02876000D0,10.02986000D0,10.03096000D0,10.03206000D0,
     &  10.03316000D0,10.03426000D0,10.03500000D0,10.03600000D0,
     &  10.03700000D0,10.03800000D0,10.03900000D0,10.04000000D0,
     &  10.04500000D0,10.05000000D0,10.05500000D0,10.06000000D0,
     &  10.06500000D0,10.07000000D0,10.07500000D0,10.08000000D0,
     &  10.20000000D0,10.25000000D0,10.26000000D0,10.27000000D0,
     &  10.28000000D0,10.29000000D0,10.29500000D0,10.30000000D0,
     &  10.31000000D0,10.32000000D0,10.33000000D0,10.34000000D0,
     &  10.34420000D0,10.34493333D0,10.34566667D0,10.34640000D0,
     &  10.34713333D0,10.34786667D0,10.34860000D0,10.34933333D0,
     &  10.35006667D0,10.35080000D0,10.35153333D0,10.35226667D0,
     &  10.35300000D0,10.35309167D0,10.35318333D0,10.35327500D0,
     &  10.35336667D0,10.35345833D0,10.35355000D0,10.35364167D0,
     &  10.35373333D0,10.35382500D0,10.35391667D0,10.35400833D0,
     &  10.35410000D0,10.35419167D0,10.35428333D0,10.35437500D0/
       DATA  (ESA  (I), I=1701,1800) /
     &  10.35440000D0,10.35446667D0,10.35455833D0,10.35465000D0,
     &  10.35474167D0,10.35483334D0,10.35492500D0,10.35501667D0,
     &  10.35510834D0,10.35512330D0,10.35514660D0,10.35515710D0,
     &  10.35516340D0,10.35516780D0,10.35517110D0,10.35517370D0,
     &  10.35517580D0,10.35517760D0,10.35517920D0,10.35518060D0,
     &  10.35518180D0,10.35518290D0,10.35518390D0,10.35518480D0,
     &  10.35518570D0,10.35518650D0,10.35518720D0,10.35518790D0,
     &  10.35518860D0,10.35518930D0,10.35518990D0,10.35519050D0,
     &  10.35519110D0,10.35519170D0,10.35519230D0,10.35519280D0,
     &  10.35519340D0,10.35519400D0,10.35519460D0,10.35519530D0,
     &  10.35519600D0,10.35519680D0,10.35519770D0,10.35519970D0,
     &  10.35520000D0,10.35520100D0,10.35520230D0,10.35520320D0,
     &  10.35520400D0,10.35520470D0,10.35520540D0,10.35520600D0,
     &  10.35520660D0,10.35520720D0,10.35520770D0,10.35520830D0,
     &  10.35520890D0,10.35520950D0,10.35521010D0,10.35521070D0,
     &  10.35521140D0,10.35521210D0,10.35521280D0,10.35521350D0,
     &  10.35521430D0,10.35521520D0,10.35521610D0,10.35521710D0,
     &  10.35521820D0,10.35521940D0,10.35522080D0,10.35522240D0,
     &  10.35522420D0,10.35522630D0,10.35522890D0,10.35523220D0,
     &  10.35523660D0,10.35524290D0,10.35525340D0,10.35527670D0,
     &  10.35529000D0,10.35529300D0,10.35529700D0,10.35531000D0,
     &  10.35532500D0,10.35538334D0,10.35547500D0,10.35556667D0,
     &  10.35565834D0,10.35575000D0,10.35584167D0,10.35593334D0,
     &  10.35602500D0,10.35611667D0,10.35620834D0,10.35630000D0,
     &  10.35639167D0,10.35648334D0,10.35657500D0,10.35666667D0/
       DATA  (ESA  (I), I=1801,1900) /
     &  10.35740000D0,10.35813333D0,10.35886667D0,10.35960000D0,
     &  10.36033333D0,10.36106667D0,10.36180000D0,10.36326667D0,
     &  10.36473333D0,10.36546667D0,10.36620000D0,10.37000000D0,
     &  10.37500000D0,10.38000000D0,10.38500000D0,10.39000000D0,
     &  10.39500000D0,10.40000000D0,10.40500000D0,10.41000000D0,
     &  10.41500000D0,10.42000000D0,10.42500000D0,10.43000000D0,
     &  10.43500000D0,10.44000000D0,10.44500000D0,10.45000000D0,
     &  10.45500000D0,10.46000000D0,10.46500000D0,10.47000000D0,
     &  10.47300000D0,10.52897000D0,10.54217000D0,10.54889000D0,
     &  10.55317000D0,10.55622000D0,10.55855000D0,10.56042000D0,
     &  10.56197000D0,10.56328000D0,10.56442000D0,10.56543000D0,
     &  10.56633000D0,10.56714000D0,10.56788000D0,10.56857000D0,
     &  10.56921000D0,10.56980000D0,10.57036000D0,10.57090000D0,
     &  10.57141000D0,10.57189000D0,10.57237000D0,10.57282000D0,
     &  10.57327000D0,10.57371000D0,10.57415000D0,10.57459000D0,
     &  10.57503000D0,10.57547000D0,10.57594000D0,10.57643000D0,
     &  10.57695000D0,10.57755000D0,10.57829000D0,10.58000000D0,
     &  10.58171000D0,10.58245000D0,10.58305000D0,10.58357000D0,
     &  10.58406000D0,10.58453000D0,10.58497000D0,10.58541000D0,
     &  10.58585000D0,10.58629000D0,10.58673000D0,10.58718000D0,
     &  10.58763000D0,10.58811000D0,10.58859000D0,10.58910000D0,
     &  10.58964000D0,10.59020000D0,10.59079000D0,10.59143000D0,
     &  10.59212000D0,10.59286000D0,10.59367000D0,10.59457000D0,
     &  10.59558000D0,10.59671000D0,10.59803000D0,10.59958000D0,
     &  10.60145000D0,10.60378000D0,10.60683000D0,10.61111000D0/
       DATA  (ESA  (I), I=1901,2000) /
     &  10.61783000D0,10.63103000D0,10.68700000D0,10.69000000D0,
     &  10.71309000D0,10.72975000D0,10.74256000D0,10.75286000D0,
     &  10.76141000D0,10.76869000D0,10.77500000D0,10.78057000D0,
     &  10.78554000D0,10.79004000D0,10.79414000D0,10.79791000D0,
     &  10.80141000D0,10.80468000D0,10.80775000D0,10.81065000D0,
     &  10.81341000D0,10.81604000D0,10.81858000D0,10.82102000D0,
     &  10.82339000D0,10.82570000D0,10.82796000D0,10.83019000D0,
     &  10.83240000D0,10.83460000D0,10.83681000D0,10.83906000D0,
     &  10.84135000D0,10.84374000D0,10.84626000D0,10.84900000D0,
     &  10.85212000D0,10.85601000D0,10.86500000D0,10.87288000D0,
     &  10.87627000D0,10.87895000D0,10.88128000D0,10.88340000D0,
     &  10.88539000D0,10.88728000D0,10.88910000D0,10.89088000D0,
     &  10.89262000D0,10.89434000D0,10.89605000D0,10.89775000D0,
     &  10.89946000D0,10.90119000D0,10.90292000D0,10.90468000D0,
     &  10.90648000D0,10.90830000D0,10.91017000D0,10.91208000D0,
     &  10.91406000D0,10.91609000D0,10.91820000D0,10.92038000D0,
     &  10.92266000D0,10.92504000D0,10.92754000D0,10.93017000D0,
     &  10.93295000D0,10.93590000D0,10.93905000D0,10.94242000D0,
     &  10.94606000D0,10.95000000D0,10.97500000D0,10.97638000D0,
     &  10.97771000D0,10.97900000D0,10.98025000D0,10.98146000D0,
     &  10.98263000D0,10.98377000D0,10.98489000D0,10.98598000D0,
     &  10.98705000D0,10.98810000D0,10.98913000D0,10.99014000D0,
     &  10.99114000D0,10.99213000D0,10.99310000D0,10.99407000D0,
     &  10.99504000D0,10.99600000D0,10.99697000D0,10.99793000D0,
     &  10.99890000D0,10.99988000D0,11.00087000D0,11.00188000D0/
       DATA  (ESA  (I), I=2001,2100) /
     &  11.00290000D0,11.00396000D0,11.00506000D0,11.00621000D0,
     &  11.00742000D0,11.00873000D0,11.01018000D0,11.01186000D0,
     &  11.01399000D0,11.01900000D0,11.02401000D0,11.02614000D0,
     &  11.02782000D0,11.02927000D0,11.03058000D0,11.03179000D0,
     &  11.03294000D0,11.03404000D0,11.03510000D0,11.03612000D0,
     &  11.03713000D0,11.03812000D0,11.03910000D0,11.04007000D0,
     &  11.04103000D0,11.04200000D0,11.04296000D0,11.04392000D0,
     &  11.04490000D0,11.04587000D0,11.04686000D0,11.04786000D0,
     &  11.04887000D0,11.04990000D0,11.05095000D0,11.05202000D0,
     &  11.05311000D0,11.05423000D0,11.05537000D0,11.05654000D0,
     &  11.05775000D0,11.05900000D0,11.06029000D0,11.06162000D0,
     &  11.06300000D0,11.07000000D0,11.09200000D0,11.11500000D0,
     &  11.13700000D0,11.16000000D0,11.18200000D0,11.20500000D0,
     &  11.22700000D0,11.25000000D0,11.27300000D0,11.29500000D0,
     &  11.31800000D0,11.34000000D0,11.50000000D0,11.65500000D0,
     &  11.67700000D0,11.70000000D0,11.72200000D0,11.74500000D0,
     &  11.76700000D0,11.79000000D0,11.81200000D0,11.83500000D0,
     &  11.85800000D0,11.88000000D0,11.90300000D0,11.92500000D0,
     &  11.94800000D0,11.97000000D0,11.99300000D0,12.01500000D0,
     &  12.03800000D0,12.06000000D0,12.08300000D0,12.10500000D0,
     &  12.12700000D0,12.15000000D0,12.17200000D0,12.19500000D0,
     &  12.21700000D0,12.24000000D0,12.26200000D0,12.28500000D0,
     &  12.30700000D0,12.33000000D0,12.35200000D0,12.37500000D0,
     &  12.39800000D0,12.42000000D0,12.44300000D0,12.46500000D0,
     &  12.48800000D0,12.51000000D0,12.53300000D0,12.55500000D0/
       DATA  (ESA  (I), I=2101,2174) /
     &  12.57800000D0,12.60000000D0,12.62300000D0,12.64500000D0,
     &  12.66700000D0,12.69000000D0,12.71200000D0,12.73500000D0,
     &  12.75700000D0,12.78000000D0,12.80200000D0,12.82500000D0,
     &  12.84700000D0,12.87000000D0,12.89200000D0,12.91500000D0,
     &  12.93800000D0,12.96000000D0,12.98300000D0,13.00500000D0,
     &  13.02800000D0,13.05000000D0,13.07300000D0,13.09500000D0,
     &  13.11800000D0,13.14000000D0,13.16300000D0,13.18500000D0,
     &  13.20800000D0,13.23000000D0,13.25200000D0,13.27500000D0,
     &  13.29700000D0,13.32000000D0,13.34200000D0,13.36500000D0,
     &  13.38700000D0,13.41000000D0,13.43200000D0,13.45500000D0,
     &  13.47700000D0,13.50000000D0,13.75000000D0,14.00000000D0,
     &  14.04000000D0,16.00000000D0,17.00000000D0,20.00000000D0,
     &  21.95000000D0,22.05000000D0,25.00000000D0,25.01000000D0,
     &  27.50000000D0,27.60000000D0,27.66000000D0,29.00000000D0,
     &  29.93000000D0,30.10000000D0,30.38000000D0,30.80000000D0,
     &  31.10000000D0,31.29000000D0,33.20000000D0,33.30000000D0,
     &  33.89000000D0,34.00000000D0,34.50000000D0,34.70000000D0,
     &  34.86000000D0,35.01000000D0,35.45000000D0,36.10000000D0,
     &  36.38000000D0,38.29000000D0/
  
         DATA ((DGS  (I,J), I=   1, 100), J=1,1) /
     &  -1.90463081E-03,-2.02675799E-03,-2.14418306E-03,-2.26331980E-03,
     &  -2.38693542E-03,-2.44823717E-03,-2.55337279E-03,-2.74552427E-03,
     &  -2.82275883E-03,-2.89874091E-03,-3.01791236E-03,-3.13253505E-03,
     &  -3.26311147E-03,-3.34867512E-03,-3.45483398E-03,-3.56173710E-03,
     &  -3.66867816E-03,-3.77682576E-03,-3.86493723E-03,-3.97729154E-03,
     &  -4.06950368E-03,-4.18366321E-03,-4.27420638E-03,-4.39333192E-03,
     &  -4.49155176E-03,-4.58880517E-03,-4.71012696E-03,-4.77693522E-03,
     &  -4.80883697E-03,-4.91094670E-03,-5.01564088E-03,-5.12034021E-03,
     &  -5.22643879E-03,-5.33446046E-03,-5.44768701E-03,-5.56605149E-03,
     &  -5.68409857E-03,-5.80102231E-03,-5.93199644E-03,-6.06392437E-03,
     &  -6.16220839E-03,-6.29373530E-03,-6.42662309E-03,-6.52844988E-03,
     &  -6.66488100E-03,-6.80545085E-03,-6.91500056E-03,-7.06700613E-03,
     &  -7.18206261E-03,-7.34402330E-03,-8.23857719E-03,-9.19454777E-03,
     &  -1.05366053E-02,-1.22588600E-02,-1.28156671E-02,-1.29807073E-02,
     &  -1.34004037E-02,-1.40358401E-02,-1.46786904E-02,-1.53174930E-02,
     &  -1.59615461E-02,-1.61994003E-02,-1.65895950E-02,-1.72367303E-02,
     &  -1.77476494E-02,-1.82505434E-02,-1.86366098E-02,-1.89245272E-02,
     &  -1.89006416E-02,-1.89022513E-02,-1.88570403E-02,-1.87431706E-02,
     &  -1.82072872E-02,-1.73045417E-02,-1.60468511E-02,-1.41723632E-02,
     &  -1.17650146E-02,-8.81573231E-03,-5.82704686E-03,-4.76545524E-03,
     &  -1.86850872E-03,-4.62663971E-04, 1.07065383E-03, 2.26314105E-03,
     &   4.05305862E-03, 6.00859642E-03, 7.09572127E-03, 7.89372989E-03,
     &   8.50536463E-03, 8.97773653E-03, 9.42825280E-03, 9.77029341E-03,
     &   1.00658486E-02, 1.03679383E-02, 1.06888052E-02, 1.11621238E-02,
     &   1.13330491E-02, 1.15065382E-02, 1.17475428E-02, 1.19501866E-02/
  
         DATA ((DGS  (I,J), I= 101, 200), J=1,1) /
     &   1.21032790E-02, 1.22406311E-02, 1.23707093E-02, 1.24950429E-02,
     &   1.26103634E-02, 1.26911038E-02, 1.27702374E-02, 1.28477643E-02,
     &   1.29152817E-02, 1.29753501E-02, 1.30379913E-02, 1.30700818E-02,
     &   1.31031547E-02, 1.31380285E-02, 1.31547513E-02, 1.31698583E-02,
     &   1.31747228E-02, 1.31635830E-02, 1.31984394E-02, 1.32513007E-02,
     &   1.33192293E-02, 1.33977928E-02, 1.34896680E-02, 1.35956429E-02,
     &   1.37084011E-02, 1.38287890E-02, 1.39639611E-02, 1.41149828E-02,
     &   1.42776094E-02, 1.44388734E-02, 1.46312213E-02, 1.48207602E-02,
     &   1.50168939E-02, 1.51975733E-02, 1.54200778E-02, 1.56066493E-02,
     &   1.57670032E-02, 1.58862743E-02, 1.58938440E-02, 1.59813997E-02,
     &   1.61268746E-02, 1.63223968E-02, 1.65194517E-02, 1.66755011E-02,
     &   1.68373853E-02, 1.70000997E-02, 1.71482269E-02, 1.72941697E-02,
     &   1.74746857E-02, 1.76293991E-02, 1.79003685E-02, 1.83754226E-02,
     &   1.85259462E-02, 1.89728080E-02, 1.94196698E-02, 1.97616125E-02,
     &   2.00705838E-02, 2.03150702E-02, 2.06590736E-02, 2.08116741E-02,
     &   2.09834304E-02, 2.11973071E-02, 2.12315427E-02, 2.12511649E-02,
     &   2.12323387E-02, 2.12230465E-02, 2.11399840E-02, 2.11204025E-02,
     &   2.09094893E-02, 2.05524903E-02, 2.00635365E-02, 1.97093120E-02,
     &   1.91491209E-02, 1.85245325E-02, 1.78289843E-02, 1.72305975E-02,
     &   1.64738336E-02, 1.56921472E-02, 1.50270507E-02, 1.46360588E-02,
     &   1.38854859E-02, 1.35518524E-02, 1.27080131E-02, 1.19513107E-02,
     &   1.10710797E-02, 1.02413231E-02, 9.22246465E-03, 8.29339643E-03,
     &   7.31896748E-03, 6.01838317E-03, 5.33543145E-03, 4.84139493E-03,
     &   3.80748537E-03, 3.24233741E-03, 2.68169856E-03, 2.05827885E-03,
     &   1.37476266E-03, 6.47331890E-04,-7.13486046E-05,-1.72432398E-03/
  
         DATA ((DGS  (I,J), I= 201, 300), J=1,1) /
     &  -3.52136425E-03,-4.58692849E-03,-4.98116189E-03,-5.41286857E-03,
     &  -5.82909517E-03,-6.25930005E-03,-6.67156108E-03,-7.20255217E-03,
     &  -7.75409827E-03,-8.22408819E-03,-8.77250458E-03,-9.32122848E-03,
     &  -9.93683668E-03,-1.05449091E-02,-1.11978739E-02,-1.18802589E-02,
     &  -1.26189184E-02,-1.33146771E-02,-1.40871346E-02,-1.49015210E-02,
     &  -1.57611876E-02,-1.64132609E-02,-1.75587401E-02,-1.86564696E-02,
     &  -1.97439375E-02,-2.09150667E-02,-2.21260513E-02,-2.34521889E-02,
     &  -2.48654460E-02,-2.63750812E-02,-2.69910779E-02,-2.78623604E-02,
     &  -2.81312904E-02,-3.02623918E-02,-3.22357496E-02,-3.42638047E-02,
     &  -3.65248541E-02,-3.88236445E-02,-4.14619279E-02,-4.39970020E-02,
     &  -4.68984867E-02,-5.00439387E-02,-5.29935047E-02,-5.73203980E-02,
     &  -6.16472913E-02,-6.61780184E-02,-7.16914412E-02,-7.77820743E-02,
     &  -8.46972679E-02,-9.25701262E-02,-1.01669743E-01,-1.12175969E-01,
     &  -1.24353233E-01,-1.36146147E-01,-1.42540400E-01,-1.47071938E-01,
     &  -1.52975480E-01,-1.55892282E-01,-1.54149889E-01,-1.64874480E-01,
     &  -1.69192089E-01,-1.73080501E-01,-1.76531697E-01,-1.79312151E-01,
     &  -1.81064103E-01,-1.81405304E-01,-1.78390639E-01,-1.72978596E-01,
     &  -1.69150381E-01,-1.58649118E-01,-1.43907824E-01,-1.24454148E-01,
     &  -9.25799150E-02,-7.12914604E-02,-3.87099341E-02,-3.76578607E-03,
     &   3.17991399E-02, 6.61587982E-02, 9.76866171E-02, 1.25139794E-01,
     &   1.47937327E-01, 1.65940089E-01, 1.79351051E-01, 1.88709270E-01,
     &   1.94695654E-01, 1.97845799E-01, 1.98838795E-01, 1.98133703E-01,
     &   1.96188131E-01, 1.93267635E-01, 1.87928484E-01, 1.86410009E-01,
     &   1.78827590E-01, 1.76926333E-01, 1.72863736E-01, 1.68437971E-01,
     &   1.64081799E-01, 1.59800779E-01, 1.55644101E-01, 1.51588370E-01/
  
         DATA ((DGS  (I,J), I= 301, 400), J=1,1) /
     &   1.51024394E-01, 1.43914419E-01, 1.40295081E-01, 1.36828525E-01,
     &   1.25630868E-01, 1.16038441E-01, 1.07700772E-01, 1.00468369E-01,
     &   9.41573070E-02, 8.74678480E-02, 8.37058853E-02, 7.93337287E-02,
     &   7.54840677E-02, 7.20112638E-02, 6.88432392E-02, 6.59928024E-02,
     &   6.34732030E-02, 6.10783848E-02, 5.89196395E-02, 5.69384892E-02,
     &   5.52903420E-02, 5.35906448E-02, 5.36328623E-02, 5.06237792E-02,
     &   4.93338660E-02, 4.88681217E-02, 4.95986235E-02, 4.85011672E-02,
     &   4.68605847E-02, 4.57462499E-02, 4.47172916E-02, 4.37864279E-02,
     &   4.29418603E-02, 4.21042348E-02, 4.13256302E-02, 4.05901027E-02,
     &   3.98947756E-02, 3.92300310E-02, 3.86006102E-02, 3.79909375E-02,
     &   3.74352871E-02, 3.68855243E-02, 3.63574228E-02, 3.58475491E-02,
     &   3.53192405E-02, 3.48967147E-02, 3.44552230E-02, 3.39872021E-02,
     &   3.36037750E-02, 3.32016086E-02, 3.28067191E-02, 3.24257264E-02,
     &   3.20867724E-02, 3.16578824E-02, 3.13026438E-02, 3.10150480E-02,
     &   3.00616973E-02, 2.80504663E-02, 2.49510970E-02, 2.26733115E-02,
     &   2.09187143E-02, 1.95152100E-02, 1.83095422E-02, 1.72579105E-02,
     &   1.63811928E-02, 1.62783508E-02, 1.52600128E-02, 1.45821529E-02,
     &   1.43667676E-02, 1.38282195E-02, 1.33177480E-02, 1.32622448E-02,
     &   1.28313645E-02, 1.23954553E-02, 1.21603583E-02, 1.19534659E-02,
     &   1.15391135E-02, 1.12058222E-02, 1.08997703E-02, 1.06762141E-02,
     &   1.05355176E-02, 1.04344563E-02, 1.02047543E-02, 1.00098767E-02,
     &   9.99286037E-03, 9.80947334E-03, 9.62653411E-03, 9.57320823E-03,
     &   9.45883355E-03, 9.28388867E-03, 9.12912275E-03, 8.95882909E-03,
     &   8.91004103E-03, 8.79478624E-03, 8.72230863E-03, 8.60140347E-03,
     &   8.45365601E-03, 8.35768294E-03, 8.35199095E-03, 8.27248614E-03/
  
         DATA ((DGS  (I,J), I= 401, 500), J=1,1) /
     &   8.05966020E-03, 7.84679709E-03, 7.64331813E-03, 7.52831598E-03,
     &   7.50295633E-03, 7.43782957E-03, 7.54381956E-03, 7.59877951E-03,
     &   7.65996140E-03, 7.78114760E-03, 7.85030020E-03, 7.85464470E-03,
     &   7.84879619E-03, 7.75001299E-03, 7.73513222E-03, 7.66057660E-03,
     &   7.59590990E-03, 7.36295521E-03, 7.32845302E-03, 7.13082222E-03,
     &   7.11925566E-03, 7.10618255E-03, 6.98342146E-03, 6.83978430E-03,
     &   6.80647126E-03, 6.55597309E-03, 6.53098024E-03, 6.40571320E-03,
     &   6.38001361E-03, 6.24613644E-03, 6.19392178E-03, 6.10210143E-03,
     &   6.10028039E-03, 6.19021223E-03, 6.36935750E-03, 6.51589507E-03,
     &   6.54523246E-03, 6.98885684E-03, 7.05486903E-03, 7.37844939E-03,
     &   7.53619669E-03, 7.58525534E-03, 7.61850259E-03, 7.63455347E-03,
     &   7.79105527E-03, 7.82217066E-03, 7.94298162E-03, 8.08882492E-03,
     &   8.16869777E-03, 8.46539560E-03, 8.81068241E-03, 8.88999682E-03,
     &   8.93069081E-03, 9.20037566E-03, 9.34522540E-03, 9.49748194E-03,
     &   1.01813750E-02, 1.02655766E-02, 1.04774884E-02, 1.05602517E-02,
     &   1.05605357E-02, 1.05502705E-02, 1.05248571E-02, 1.05070392E-02,
     &   1.04881084E-02, 1.04366319E-02, 1.03709131E-02, 1.03979839E-02,
     &   1.05355014E-02, 1.06846235E-02, 1.06988826E-02, 1.08263635E-02,
     &   1.08620848E-02, 1.08759629E-02, 1.08762377E-02, 1.08690457E-02,
     &   1.08651583E-02, 1.08508512E-02, 1.07917652E-02, 1.05955522E-02,
     &   1.05534132E-02, 1.01438066E-02, 9.92071734E-03, 9.76732612E-03,
     &   9.34826434E-03, 8.98087384E-03, 8.92004255E-03, 8.89094179E-03,
     &   8.87898497E-03, 8.87584797E-03, 8.86230510E-03, 8.85960266E-03,
     &   8.85414606E-03, 8.84211437E-03, 8.82001679E-03, 8.80582324E-03,
     &   8.41176463E-03, 7.73457398E-03, 7.56550042E-03, 7.88958650E-03/
  
         DATA ((DGS  (I,J), I= 501, 600), J=1,1) /
     &   7.94446024E-03, 8.93935604E-03, 9.61401547E-03, 9.97564190E-03,
     &   1.01424278E-02, 1.02821289E-02, 1.05695575E-02, 1.03762999E-02,
     &   1.00354455E-02, 9.77275518E-03, 9.46301234E-03, 9.04345082E-03,
     &   8.45122699E-03, 7.57640761E-03, 6.33200140E-03, 4.50972495E-03,
     &   4.03324877E-03, 3.50120963E-03, 2.90561242E-03, 2.23434310E-03,
     &   1.47399058E-03, 6.01916962E-04,-4.07159178E-04,-6.28924632E-04,
     &  -8.57156304E-04,-1.09283219E-03,-1.33626522E-03,-1.58807617E-03,
     &  -1.84873205E-03,-2.11867981E-03,-2.39792674E-03,-2.68747215E-03,
     &  -2.98785997E-03,-4.49455436E-03,-5.65705553E-03,-6.29729320E-03,
     &  -6.74901010E-03,-7.22235461E-03,-7.71842716E-03,-8.23829788E-03,
     &  -8.78525523E-03,-9.36024499E-03,-9.65997116E-03,-9.96702880E-03,
     &  -1.02823680E-02,-1.06063063E-02,-1.09390706E-02,-1.12814951E-02,
     &  -1.16336920E-02,-1.19960789E-02,-1.23689237E-02,-1.27530720E-02,
     &  -1.31475681E-02,-1.35554061E-02,-1.39759018E-02,-1.44095921E-02,
     &  -1.48573867E-02,-1.53200204E-02,-1.57980185E-02,-1.62921546E-02,
     &  -1.68033197E-02,-1.73322724E-02,-1.78801972E-02,-1.84479946E-02,
     &  -1.90368088E-02,-1.96477917E-02,-2.02823076E-02,-2.09416952E-02,
     &  -2.16276019E-02,-2.23413970E-02,-2.30848481E-02,-2.38598188E-02,
     &  -2.46686339E-02,-2.55135898E-02,-2.63965603E-02,-2.73205699E-02,
     &  -2.82884882E-02,-2.93036949E-02,-3.03699647E-02,-3.14907913E-02,
     &  -3.26706029E-02,-3.39141932E-02,-3.45615472E-02,-3.52268908E-02,
     &  -3.59110041E-02,-3.66149249E-02,-3.73395405E-02,-3.80848414E-02,
     &  -3.88522399E-02,-3.96430836E-02,-4.04584720E-02,-4.12989961E-02,
     &  -4.21664383E-02,-4.30621080E-02,-4.39870477E-02,-4.49434657E-02,
     &  -4.59321190E-02,-4.69549069E-02,-4.80136322E-02,-4.91102266E-02/
  
         DATA ((DGS  (I,J), I= 601, 700), J=1,1) /
     &  -5.02467532E-02,-5.14254484E-02,-5.26487090E-02,-5.39187634E-02,
     &  -5.52387164E-02,-5.66117145E-02,-5.80412785E-02,-5.95305600E-02,
     &  -6.10833874E-02,-6.27039276E-02,-6.43963659E-02,-6.61663390E-02,
     &  -6.80189247E-02,-6.99600922E-02,-7.19967048E-02,-7.41355184E-02,
     &  -7.63844151E-02,-7.87517506E-02,-8.12476373E-02,-8.38831698E-02,
     &  -8.78301501E-02,-8.96217999E-02,-9.27534836E-02,-9.60822316E-02,
     &  -9.96266889E-02,-1.03408959E-01,-1.07453844E-01,-1.11789400E-01,
     &  -1.16448847E-01,-1.21469321E-01,-1.26894887E-01,-1.32776714E-01,
     &  -1.39174995E-01,-1.44283630E-01,-1.60204033E-01,-1.79715147E-01,
     &  -2.04191721E-01,-2.35817325E-01,-2.78297077E-01,-3.38541144E-01,
     &  -4.34481571E-01,-5.82525825E-01,-8.93486568E-01,-1.89090780E+00,
     &  -2.00016826E+00,-2.12271038E+00,-2.26131352E+00,-2.41854650E+00,
     &  -2.59934028E+00,-2.80902246E+00,-3.05506452E+00,-3.34773399E+00,
     &  -3.70151758E+00,-4.13750409E+00,-4.68779837E+00,-5.37977280E+00,
     &  -6.36422404E+00,-6.47895314E+00,-9.04791891E+00,-1.09202817E+01,
     &  -1.24246882E+01,-7.74771898E+00,-1.47450880E+01,-1.50651461E+01,
     &  -1.63209600E+01,-1.70287807E+01,-1.75203147E+01,-1.79741597E+01,
     &  -1.82894855E+01,-1.85432582E+01,-1.86817899E+01,-1.87804306E+01,
     &  -1.87858816E+01,-1.87286586E+01,-1.85791410E+01,-1.84191898E+01,
     &  -1.80732516E+01,-1.77619569E+01,-1.73777678E+01,-1.65791022E+01,
     &  -1.57973285E+01,-1.49379780E+01,-1.40060344E+01,-1.41326694E+01,
     &  -1.31856973E+01,-1.21356147E+01,-1.09849028E+01,-9.08132422E+00,
     &  -7.70517333E+00,-5.56127325E+00,-3.18475131E+00, 3.21202682E+00,
     &   9.10245061E+00, 1.10033146E+01, 1.26896544E+01, 1.36873996E+01,
     &   1.49911690E+01, 1.49876548E+01, 1.58370059E+01, 1.66073097E+01/
  
         DATA ((DGS  (I,J), I= 701, 800), J=1,1) /
     &   1.72944604E+01, 1.77848102E+01, 1.81566996E+01, 1.83464723E+01,
     &   1.85352289E+01, 1.86711763E+01, 1.87601267E+01, 1.88167609E+01,
     &   1.45869958E+01, 1.54335585E+01, 1.86176893E+01, 1.85070816E+01,
     &   1.82330179E+01, 1.79910382E+01, 1.76340755E+01, 1.72514995E+01,
     &   1.67544393E+01, 1.63529852E+01, 1.51624451E+01, 1.48624237E+01,
     &   1.40244030E+01, 1.30800278E+01, 1.18654096E+01, 1.03655305E+01,
     &   8.73421378E+00, 7.19932651E+00, 6.00707050E+00, 5.14677343E+00,
     &   4.49901417E+00, 3.99464233E+00, 3.59124105E+00, 3.26147955E+00,
     &   2.98708924E+00, 2.75506805E+00, 2.55653175E+00, 2.38469386E+00,
     &   2.23453006E+00, 2.10219340E+00, 1.98469616E+00, 1.87968109E+00,
     &   1.78526383E+00, 1.69992072E+00, 1.62240593E+00, 8.56648340E-01,
     &   5.84774270E-01, 4.45807516E-01, 3.63544890E-01, 3.03310619E-01,
     &   2.61845977E-01, 2.30868337E-01, 2.06786038E-01, 1.87511890E-01,
     &   1.71732009E-01, 1.69864295E-01, 1.56847156E-01, 1.45825535E-01,
     &   1.36374752E-01, 1.28179920E-01, 1.21006566E-01, 1.14680802E-01,
     &   1.09049705E-01, 1.04009580E-01, 9.94719823E-02, 9.53597631E-02,
     &   9.16268529E-02, 8.82163632E-02, 8.50893254E-02, 8.22112032E-02,
     &   7.95565877E-02, 7.70929293E-02, 7.48068695E-02, 7.26786252E-02,
     &   7.06919045E-02, 6.88343610E-02, 6.70893535E-02, 6.54495979E-02,
     &   6.39101963E-02, 6.24548620E-02, 6.10796264E-02, 5.97804051E-02,
     &   5.85493366E-02, 5.73859384E-02, 5.62760945E-02, 5.52149974E-02,
     &   5.42084794E-02, 5.32535459E-02, 5.23383107E-02, 5.14577420E-02,
     &   5.06205582E-02, 4.98240451E-02, 4.90560838E-02, 4.83129250E-02,
     &   4.76116586E-02, 4.69300085E-02, 4.62761052E-02, 4.56466459E-02,
     &   4.50405385E-02, 4.44490088E-02, 4.38860124E-02, 4.33427667E-02/
  
         DATA ((DGS  (I,J), I= 801, 900), J=1,1) /
     &   4.28245443E-02, 4.23180010E-02, 4.18215607E-02, 4.13534355E-02,
     &   4.08947694E-02, 4.04505310E-02, 4.00201251E-02, 3.96024312E-02,
     &   3.91981451E-02, 3.88013894E-02, 3.84208881E-02, 3.80515307E-02,
     &   3.76962182E-02, 3.73485242E-02, 3.70105590E-02, 3.66814147E-02,
     &   3.63607253E-02, 3.60495719E-02, 3.57437326E-02, 3.54493001E-02,
     &   3.51663912E-02, 3.48869000E-02, 3.46142440E-02, 3.43506454E-02,
     &   3.40934960E-02, 3.38431381E-02, 3.35996496E-02, 3.33629526E-02,
     &   3.31330097E-02, 3.29098340E-02, 3.26935009E-02, 3.24844304E-02,
     &   3.22831183E-02, 3.20924296E-02, 3.19065652E-02, 3.17300779E-02,
     &   3.11836540E-02, 3.09855485E-02, 3.07912824E-02, 3.06007874E-02,
     &   3.04140756E-02, 3.02345809E-02, 3.00556455E-02, 2.98800792E-02,
     &   2.97034460E-02, 2.95507168E-02, 2.93675955E-02, 2.92043525E-02,
     &   2.90440890E-02, 2.88868287E-02, 2.87325110E-02, 2.85809316E-02,
     &   2.84318437E-02, 2.82852099E-02, 2.81410901E-02, 2.79994216E-02,
     &   2.37497192E-02, 2.07547192E-02, 1.81039279E-02, 1.57510296E-02,
     &   1.41971057E-02, 1.34632742E-02, 1.25979852E-02, 1.15277871E-02,
     &   1.05079576E-02, 9.10057220E-03, 8.38570125E-03, 7.28564998E-03,
     &   7.28427390E-03, 6.95437913E-03, 6.03329586E-03, 5.03297913E-03,
     &   3.89697854E-03, 2.57110672E-03, 9.87333613E-04,-9.51742699E-04,
     &  -3.39612710E-03,-4.56569061E-03,-5.87629136E-03,-7.35562049E-03,
     &  -9.04005501E-03,-1.09769129E-02,-1.32291284E-02,-1.44991584E-02,
     &  -1.58825268E-02,-1.73953691E-02,-1.90570348E-02,-2.08909309E-02,
     &  -2.29256457E-02,-2.51964682E-02,-2.77474754E-02,-3.06344472E-02,
     &  -3.39290425E-02,-3.77248938E-02,-4.21467289E-02,-4.71377754E-02,
     &  -5.04049736E-02,-5.40304928E-02,-5.80770314E-02,-6.26228543E-02/
  
         DATA ((DGS  (I,J), I= 901,1000), J=1,1) /
     &  -6.77669558E-02,-7.36364286E-02,-8.03972270E-02,-8.82703095E-02,
     &  -9.75565962E-02,-1.08676962E-01,-1.22239098E-01,-1.39155229E-01,
     &  -1.60863203E-01,-1.89778132E-01,-2.30329479E-01,-2.91827313E-01,
     &  -4.00607076E-01,-5.93011169E-01,-8.68498496E-01,-1.02825844E+00,
     &  -1.20312001E+00,-1.63438881E+00,-1.79027341E+00,-1.84001135E+00,
     &  -2.02695070E+00,-2.09870570E+00,-2.19458423E+00,-2.25936371E+00,
     &  -2.27393461E+00,-2.31607092E+00,-2.38304540E+00,-2.40602030E+00,
     &  -2.41961107E+00,-2.42445232E+00,-2.42083848E+00,-2.41118742E+00,
     &  -2.39170922E+00,-2.36630379E+00,-2.33516185E+00,-2.29250159E+00,
     &  -2.24209951E+00,-2.17749656E+00,-2.11208674E+00,-2.03696174E+00,
     &  -1.95159839E+00,-1.83854913E+00,-1.72980811E+00,-1.58897378E+00,
     &  -1.45637282E+00,-1.26305054E+00,-1.05207360E+00,-7.36974554E-01,
     &  -4.62316668E-01,-1.45951088E-01, 1.42948059E-02, 1.74542647E-01,
     &   3.33756760E-01, 7.65607772E-01, 1.08075294E+00, 1.29177144E+00,
     &   1.48514062E+00, 1.61777941E+00, 1.75865996E+00, 1.86744178E+00,
     &   1.98053931E+00, 2.06594415E+00, 2.14111035E+00, 2.20656642E+00,
     &   2.27121509E+00, 2.32160248E+00, 2.36430572E+00, 2.39554017E+00,
     &   2.42098414E+00, 2.44050404E+00, 2.45018956E+00, 2.45384510E+00,
     &   2.44904274E+00, 2.43549214E+00, 2.41255793E+00, 2.34532606E+00,
     &   2.30324471E+00, 2.28899409E+00, 2.22425437E+00, 2.12820215E+00,
     &   2.05670441E+00, 1.87059591E+00, 1.82009067E+00, 1.66424114E+00,
     &   1.47581734E+00, 1.57811330E+00, 1.35324096E+00, 1.21491715E+00,
     &   1.13137050E+00, 9.58679884E-01, 8.98462156E-01, 7.73727688E-01,
     &   9.38871645E-01, 3.64026010E-01, 2.85797999E-01, 2.37010841E-01,
     &   2.03327971E-01, 1.78589728E-01, 1.59619650E-01, 1.44596164E-01/
  
         DATA ((DGS  (I,J), I=1001,1100), J=1,1) /
     &   1.32394746E-01, 1.22281157E-01, 1.13752571E-01, 1.06440185E-01,
     &   1.00139379E-01, 9.47314279E-02, 8.99379939E-02, 8.56773476E-02,
     &   8.18714250E-02, 7.84532450E-02, 7.30228434E-02, 6.62454092E-02,
     &   6.26145662E-02, 5.66760807E-02, 5.20225271E-02, 4.74595034E-02,
     &   4.09531931E-02, 3.35288550E-02, 2.89321672E-02, 2.67794043E-02,
     &   2.56486478E-02, 2.48037250E-02, 2.41752488E-02, 2.36632109E-02,
     &   2.32825724E-02, 2.29483813E-02, 2.26661472E-02, 2.24231526E-02,
     &   2.22122848E-02, 2.20716995E-02, 2.18643870E-02, 2.17204694E-02,
     &   2.15929540E-02, 2.14799392E-02, 2.13801836E-02, 2.12919913E-02,
     &   2.12144587E-02, 2.11470411E-02, 2.10891715E-02, 2.10405197E-02,
     &   2.09447367E-02, 2.09698245E-02, 2.09478340E-02, 2.09352418E-02,
     &   2.09306351E-02, 2.09396821E-02, 2.09637342E-02, 2.10010652E-02,
     &   2.10589913E-02, 2.11473630E-02, 2.12917115E-02, 2.17622000E-02,
     &   2.23831079E-02, 2.26794151E-02, 2.29200873E-02, 2.31302186E-02,
     &   2.33199321E-02, 2.34930175E-02, 2.36540536E-02, 2.38012341E-02,
     &   2.39385745E-02, 2.40642061E-02, 2.41838324E-02, 2.42874640E-02,
     &   2.43181194E-02, 2.44070047E-02, 2.44543719E-02, 2.45803178E-02,
     &   2.46592223E-02, 2.47313655E-02, 2.47622088E-02, 2.47769103E-02,
     &   2.47784087E-02, 2.47534383E-02, 2.47119199E-02, 2.46477769E-02,
     &   2.45468954E-02, 2.44238799E-02, 2.42835744E-02, 2.41001666E-02,
     &   2.38734131E-02, 2.35907659E-02, 2.32475618E-02, 2.28194585E-02,
     &   2.22033999E-02, 2.12916819E-02, 2.10192789E-02, 2.02876657E-02,
     &   1.91540466E-02, 1.90330687E-02, 1.82240552E-02, 1.79055037E-02,
     &   1.75914950E-02, 1.73421251E-02, 1.70522389E-02, 1.66086543E-02,
     &   1.64295426E-02, 1.64522526E-02, 1.61668706E-02, 1.61482299E-02/
  
         DATA ((DGS  (I,J), I=1101,1200), J=1,1) /
     &   1.65193047E-02, 1.63241560E-02, 1.65462223E-02, 1.63043284E-02,
     &   1.59337058E-02, 1.63055560E-02, 1.58296434E-02, 1.48904509E-02,
     &   1.46284879E-02, 1.49434519E-02, 1.49658667E-02, 1.49045878E-02,
     &   1.46900180E-02, 1.40494021E-02, 1.37475820E-02, 1.30433764E-02,
     &   1.24683994E-02, 1.22108263E-02, 1.26405216E-02, 1.33660382E-02,
     &   1.48911994E-02, 1.54355975E-02, 1.61108641E-02, 1.65393935E-02,
     &   1.70124591E-02, 1.71435308E-02, 1.71248969E-02, 1.77806427E-02,
     &   1.83842268E-02, 1.82457258E-02, 1.81798393E-02, 1.82779698E-02,
     &   1.83563175E-02, 1.84175170E-02, 1.83928034E-02, 1.84313170E-02,
     &   1.84211541E-02, 1.82373798E-02, 1.81542448E-02, 1.81935527E-02,
     &   1.83185813E-02, 1.84767138E-02, 1.88801517E-02, 1.90681554E-02,
     &   1.90796013E-02, 1.90734700E-02, 1.89265483E-02, 1.85339744E-02,
     &   1.85137765E-02, 1.88819922E-02, 1.93101943E-02, 1.96943410E-02,
     &   1.97201843E-02, 1.98800037E-02, 2.01208276E-02, 2.03651926E-02,
     &   2.07981092E-02, 2.12514879E-02, 2.14430239E-02, 2.16330128E-02,
     &   2.19422206E-02, 2.21428088E-02, 2.19754639E-02, 2.16167282E-02,
     &   2.14967344E-02, 2.17096925E-02, 2.16358901E-02, 2.10178603E-02,
     &   2.08131073E-02, 2.09430341E-02, 2.09437867E-02, 2.02032788E-02,
     &   1.94588382E-02, 1.92217478E-02, 1.91116366E-02, 1.90215962E-02,
     &   1.89401543E-02, 1.85071907E-02, 1.82973681E-02, 1.83717367E-02,
     &   1.83479422E-02, 1.80359670E-02, 1.74241993E-02, 1.73428721E-02,
     &   1.71787903E-02, 1.72958999E-02, 1.73436785E-02, 1.74434943E-02,
     &   1.75375399E-02, 1.75695948E-02, 1.75944293E-02, 1.74854112E-02,
     &   1.73160798E-02, 1.71875838E-02, 1.77776945E-02, 1.82564073E-02,
     &   1.82374870E-02, 1.82879555E-02, 1.83052627E-02, 1.82663673E-02/
  
         DATA ((DGS  (I,J), I=1201,1300), J=1,1) /
     &   1.87608259E-02, 1.94557999E-02, 1.96462543E-02, 1.96305675E-02,
     &   1.97078383E-02, 1.97643215E-02, 1.98642032E-02, 1.99681747E-02,
     &   1.99179213E-02, 1.98576716E-02, 1.98315862E-02, 1.94084635E-02,
     &   1.93240566E-02, 1.94820599E-02, 1.95388928E-02, 1.93581352E-02,
     &   1.92507244E-02, 1.92397032E-02, 1.92591451E-02, 1.92807040E-02,
     &   1.96757444E-02, 2.00959379E-02, 2.01262006E-02, 2.01583548E-02,
     &   2.01483599E-02, 2.02227277E-02, 2.02475172E-02, 2.05399957E-02,
     &   2.09457345E-02, 2.11088782E-02, 2.12573152E-02, 2.15243318E-02,
     &   2.17640687E-02, 2.18764184E-02, 2.19846549E-02, 2.20892603E-02,
     &   2.21906153E-02, 2.22890265E-02, 2.23847446E-02, 2.24779765E-02,
     &   2.25688958E-02, 2.26576491E-02, 2.27443608E-02, 2.28291377E-02,
     &   2.29120764E-02, 2.29932465E-02, 2.30727213E-02, 2.31505613E-02,
     &   2.32268168E-02, 2.33015330E-02, 2.33747486E-02, 2.34464957E-02,
     &   2.35168029E-02, 2.35856932E-02, 2.36531857E-02, 2.37192953E-02,
     &   2.37840333E-02, 2.38474072E-02, 2.39094213E-02, 2.39700760E-02,
     &   2.40293684E-02, 2.40872924E-02, 2.41438379E-02, 2.41989914E-02,
     &   2.42527357E-02, 2.43050493E-02, 2.43559070E-02, 2.43955244E-02,
     &   2.44531278E-02, 2.45440959E-02, 2.46284143E-02, 2.47055700E-02,
     &   2.47749035E-02, 2.48355670E-02, 2.48864609E-02, 2.49261448E-02,
     &   2.49527028E-02, 2.49635323E-02, 2.49550069E-02, 2.49219060E-02,
     &   2.48564195E-02, 2.47462160E-02, 2.45705391E-02, 2.45481349E-02,
     &   2.42911847E-02, 2.38285286E-02, 2.29822311E-02, 2.16115531E-02,
     &   2.10534308E-02, 1.87681432E-02, 1.45830995E-02, 1.26627799E-02,
     &   9.97301608E-03, 8.17946400E-03, 5.93760078E-03, 3.69626183E-03,
     &   8.95918493E-04,-7.83338204E-04,-2.70144443E-03,-4.91308950E-03/
  
         DATA ((DGS  (I,J), I=1301,1400), J=1,1) /
     &  -7.49101834E-03,-1.05341553E-02,-1.41805558E-02,-1.62888383E-02,
     &  -1.86289143E-02,-2.12411957E-02,-2.41760979E-02,-2.74973702E-02,
     &  -3.12868916E-02,-3.56518230E-02,-4.07359362E-02,-4.67406915E-02,
     &  -4.92537635E-02,-5.08385858E-02,-5.25044195E-02,-5.43013636E-02,
     &  -5.60277235E-02,-5.78190108E-02,-5.36502551E-02,-6.17275554E-02,
     &  -6.38336391E-02,-6.60467924E-02,-6.83739540E-02,-7.08232821E-02,
     &  -7.34040685E-02,-7.61267663E-02,-7.90027770E-02,-8.20461734E-02,
     &  -8.52707463E-02,-8.86933729E-02,-9.23327305E-02,-9.62099474E-02,
     &  -1.00349013E-01,-1.04777271E-01,-1.09526024E-01,-1.14631227E-01,
     &  -1.20134639E-01,-1.26084477E-01,-1.32537324E-01,-1.39559688E-01,
     &  -1.47230347E-01,-1.55643298E-01,-1.64911588E-01,-1.75172146E-01,
     &  -1.86591879E-01,-1.99387463E-01,-2.13811558E-01,-2.30201237E-01,
     &  -2.48986931E-01,-2.70735396E-01,-2.96207203E-01,-3.26447591E-01,
     &  -3.62933301E-01,-4.07820937E-01,-4.64388695E-01,-5.50794438E-01,
     &  -6.37200164E-01,-7.78883914E-01,-9.97324092E-01,-1.37789696E+00,
     &  -2.20604146E+00,-2.71950183E+00,-5.20484224E+00,-7.25175050E+00,
     &  -8.73808225E+00,-9.88434510E+00,-2.78974532E-02,-1.16532767E+01,
     &  -1.24203129E+01,-1.29541392E+01,-1.34960687E+01,-1.40300244E+01,
     &  -1.42874343E+01,-1.46481826E+01,-1.48666767E+01,-1.51374906E+01,
     &  -1.52027747E+01,-1.53226553E+01,-1.53492690E+01,-1.53168029E+01,
     &  -1.52673143E+01,-1.51926237E+01,-1.86939719E-01,-1.82843075E-01,
     &  -1.45867249E+01,-1.40619665E+01,-1.37361946E+01,-1.33689557E+01,
     &  -1.29455540E+01,-1.19388470E+01,-1.13518939E+01,-1.07158117E+01,
     &  -9.26994465E+00,-8.46958580E+00,-2.58535563E+00,-4.77356554E+00,
     &   5.38950286E-01, 4.75934487E+00, 6.70336721E+00, 8.46207402E+00/
  
         DATA ((DGS  (I,J), I=1401,1500), J=1,1) /
     &   1.00248883E+01, 1.07274205E+01, 1.13669873E+01, 1.19513574E+01,
     &   1.29629692E+01, 1.33880504E+01, 1.37677006E+01, 1.41173615E+01,
     &   1.46306517E+01, 1.48307319E+01, 1.50006209E+01, 1.52399077E+01,
     &   1.53171709E+01, 1.53960820E+01, 1.54031819E+01, 1.53623379E+01,
     &   1.52612774E+01, 1.53271112E+01, 1.51920956E+01, 1.47127672E+01,
     &   1.43606705E+01, 1.41021924E+01, 1.35585068E+01, 1.30169948E+01,
     &   1.24828351E+01, 1.17154471E+01, 1.08897988E+01, 9.94455117E+00,
     &   8.79500424E+00, 7.30938315E+00, 5.22994700E+00, 2.86660857E+00,
     &   1.65119670E+00, 1.16336244E+00, 9.00714616E-01, 7.36641118E-01,
     &   6.24435214E-01, 5.43010875E-01, 4.80896134E-01, 4.32220293E-01,
     &   3.92976881E-01, 3.60666760E-01, 3.33602143E-01, 3.10601968E-01,
     &   2.90814921E-01, 2.73611982E-01, 2.59393342E-01, 2.45174690E-01,
     &   2.33272347E-01, 2.22617072E-01, 2.13007824E-01, 2.04300763E-01,
     &   1.96374881E-01, 1.89129694E-01, 1.82481277E-01, 1.76358959E-01,
     &   1.70702819E-01, 1.65647268E-01, 1.60591726E-01, 1.56054944E-01,
     &   1.51818465E-01, 1.47853601E-01, 1.44135231E-01, 1.40641271E-01,
     &   1.37352240E-01, 1.34250906E-01, 1.31321992E-01, 1.28551950E-01,
     &   1.25928790E-01, 1.23441992E-01, 1.21082565E-01, 1.18843451E-01,
     &   1.16721243E-01, 1.14726366E-01, 1.12849122E-01, 1.10841819E-01,
     &   1.09002480E-01, 1.07258239E-01, 1.05594308E-01, 1.04002725E-01,
     &   1.02477749E-01, 9.10378845E-02, 8.40686146E-02, 7.74252153E-02,
     &   7.00891808E-02, 6.34176818E-02, 5.75484796E-02, 5.61081842E-02,
     &   5.07401109E-02, 4.26777150E-02, 3.75236357E-02, 3.72801205E-02,
     &   3.67628415E-02, 3.61902481E-02, 3.51003139E-02, 3.41622742E-02,
     &   3.11619501E-02, 2.84716869E-02, 2.68623567E-02, 2.47415843E-02/
  
         DATA ((DGS  (I,J), I=1501,1600), J=1,1) /
     &   2.22845986E-02, 1.94790912E-02, 1.58876545E-02, 1.09763918E-02,
     &  -1.28027427E-03,-3.60217566E-03,-6.33597311E-03,-7.12809451E-03,
     &  -8.93350699E-03,-1.09393751E-02,-1.31810977E-02,-1.57029351E-02,
     &  -1.85609674E-02,-2.18273254E-02,-2.55963785E-02,-2.99940288E-02,
     &  -3.51920958E-02,-4.14313808E-02,-4.90602203E-02,-5.86022477E-02,
     &  -7.08831256E-02,-8.72873335E-02,-1.10341035E-01,-1.45427829E-01,
     &  -2.02655534E-01,-3.17015850E-01,-4.70643325E-01,-4.96434431E-01,
     &  -5.25045813E-01,-5.56966332E-01,-5.92804636E-01,-6.33327561E-01,
     &  -6.79520673E-01,-7.32655454E-01,-7.94426214E-01,-8.67119679E-01,
     &  -9.53908123E-01,-1.05932095E+00,-1.19005523E+00,-1.35644327E+00,
     &  -1.57529621E+00,-1.87585968E+00,-2.31383703E+00,-3.00919682E+00,
     &  -4.27111294E+00,-7.12599196E+00,-8.12641101E+00,-8.56171676E+00,
     &  -9.78690378E+00,-1.05325976E+01,-1.10726458E+01,-9.60952312E+00,
     &  -8.43491226E+00,-1.27050981E+01,-1.22785839E+01,-8.47162596E-01,
     &  -4.41441269E+00,-4.31190785E+00,-4.26078537E+00,-1.01749403E+01,
     &  -1.01142104E+01,-1.00682765E+01,-1.00220146E+01,-9.54196553E+00,
     &  -8.76714638E+00,-7.83991347E+00,-6.95641172E+00,-7.33069187E+00,
     &  -6.21891183E+00,-5.12029715E+00,-4.47499565E+00,-3.85685332E+00,
     &  -3.16739632E+00,-2.17291535E+00,-1.44681794E+00, 1.12797106E-01,
     &   1.13131533E-01, 1.13242929E-01, 1.13298612E-01, 1.13354285E-01,
     &   1.13409948E-01, 1.13465601E-01, 1.13910465E-01, 1.14465639E-01,
     &   1.15019805E-01, 5.47266869E+00, 6.61241223E+00, 8.54554175E+00,
     &   9.29362971E+00, 9.99493061E+00, 1.05169230E+01, 1.02403832E+01,
     &   1.08970204E+01, 1.15052109E+01, 1.16873824E+01, 1.17903996E+01,
     &   1.18228495E+01, 1.18207170E+01, 1.17834433E+01, 1.17246252E+01/
  
         DATA ((DGS  (I,J), I=1601,1700), J=1,1) /
     &   8.52116221E+00, 1.14141890E+01, 1.12728886E+01, 1.11332016E+01,
     &   1.07826689E+01, 1.06012024E+01, 1.03064757E+01, 9.83527870E+00,
     &   8.18052544E+00, 6.89115644E+00, 5.91308042E+00, 5.16266902E+00,
     &   4.57357514E+00, 4.01564514E+00, 3.48431108E+00, 2.89328744E+00,
     &   2.69402550E+00, 2.52026096E+00, 2.36717021E+00, 2.14084835E+00,
     &   1.77663353E+00, 1.51904020E+00, 1.32735798E+00, 1.17921933E+00,
     &   1.06131504E+00, 9.65269297E-01, 8.85524194E-01, 8.18257798E-01,
     &   7.60759345E-01, 7.11044346E-01, 6.67634056E-01, 6.29402546E-01,
     &   5.95474850E-01, 5.65161578E-01, 5.37920229E-01, 3.70821593E-01,
     &   2.56290243E-01, 1.99047482E-01, 1.41896795E-01, 1.12862141E-01,
     &   9.56296370E-02, 8.41664016E-02, 7.59855651E-02, 6.98521895E-02,
     &   6.50822587E-02, 6.12660521E-02, 5.91008267E-02, 5.65739543E-02,
     &   5.44142513E-02, 5.25469205E-02, 5.09161898E-02, 4.94796245E-02,
     &   4.42708915E-02, 4.09979631E-02, 3.87451583E-02, 3.70949616E-02,
     &   3.58282567E-02, 3.48095152E-02, 3.39971121E-02, 3.33206156E-02,
     &   2.74962767E-02, 2.58666018E-02, 2.54580234E-02, 2.49862853E-02,
     &   2.44262959E-02, 2.37331111E-02, 2.33159582E-02, 2.28362693E-02,
     &   2.16051177E-02, 1.97565328E-02, 1.65494543E-02, 9.28644609E-03,
     &   2.36165214E-03, 5.75430190E-04,-1.48448158E-03,-3.88651792E-03,
     &  -6.72414310E-03,-1.01283027E-02,-1.42881436E-02,-1.94879454E-02,
     &  -2.61754693E-02,-3.50999169E-02,-4.76272434E-02,-6.64950712E-02,
     &  -9.75457404E-02,-1.02959362E-01,-1.08865356E-01,-1.15335335E-01,
     &  -1.22454717E-01,-1.30348132E-01,-1.39061046E-01,-1.48831691E-01,
     &  -1.59822697E-01,-1.72280655E-01,-1.86518420E-01,-2.02944653E-01,
     &  -2.22110279E-01,-2.44759861E-01,-2.71935307E-01,-3.05148232E-01/
  
         DATA ((DGS  (I,J), I=1701,1800), J=1,1) /
     &  -3.15518208E-01,-3.46666626E-01,-4.00030238E-01,-4.71173567E-01,
     &  -5.70740071E-01,-7.19988182E-01,-9.68415113E-01,-1.46365446E+00,
     &  -2.92741357E+00,-3.48542684E+00,-4.92722312E+00,-5.63446251E+00,
     &  -6.92729375E+00,-7.71431438E+00,-8.41588318E+00,-9.05174246E+00,
     &  -9.59868744E+00,-1.03699536E+01,-1.06002509E+01, 8.50921130E-02,
     &  -1.14925050E+01,-1.18642530E+01,-1.21801026E+01,-1.24416061E+01,
     &  -8.63531921E+00,-1.28788297E+01,-1.30390979E+01,-1.31858950E+01,
     &  -1.33137294E+01,-1.34127395E+01,-1.36369114E+01,-1.34336928E+01,
     &  -1.33466017E+01,-1.25048515E+01,-1.17936658E+01, 2.63148597E-01,
     &  -1.24051836E+01,-1.18604863E+01,-1.11813466E+01,-1.02160305E+01,
     &  -9.06877216E+00,-7.55140650E+00,-5.78286070E+00, 2.63789766E+00,
     &   2.69968097E-02, 2.65642103E+00, 5.83718029E+00, 7.61210132E+00,
     &   9.14732203E+00, 1.03058939E+01, 8.70037652E+00, 9.65429698E+00,
     &   1.05741831E+01, 1.14479577E+01, 1.21312837E+01, 1.28853672E+01,
     &   1.35543444E+01, 5.12980552E+00, 5.58478875E+00, 6.05857685E+00,
     &   9.49120118E-01, 7.24403422E+00, 7.88119886E+00, 8.54833227E+00,
     &   9.34580289E+00, 1.02819736E+01, 1.12477583E+01, 1.23320039E+01,
     &   1.16839761E+01, 1.08006007E+01, 9.91104420E+00, 9.11950350E+00,
     &   8.52647038E+00, 3.32709749E+00, 8.49680884E+00, 7.75602873E+00,
     &   6.98653231E+00, 6.07618975E+00, 4.98138303E+00, 3.53960238E+00,
     &   3.03472726E+00, 2.94003339E+00, 1.97159488E+00, 2.49811318E+00,
     &   2.20579213E+00, 1.51775018E+00, 1.02259257E+00, 7.74155388E-01,
     &   6.24903975E-01, 5.25350271E-01, 4.54206336E-01, 4.00837123E-01,
     &   3.59325861E-01, 3.26109590E-01, 2.98930749E-01, 2.76282819E-01,
     &   2.57118078E-01, 2.40685990E-01, 2.26449662E-01, 2.13990292E-01/
  
         DATA ((DGS  (I,J), I=1801,1900), J=1,1) /
     &   1.51692307E-01, 1.20542476E-01, 1.01855734E-01, 8.94162701E-02,
     &   8.04910816E-02, 7.37621574E-02, 6.85477854E-02, 6.09714604E-02,
     &   5.57264623E-02, 5.36649243E-02, 5.18774709E-02, 4.54407754E-02,
     &   4.07111540E-02, 3.78651369E-02, 3.59538694E-02, 3.45737898E-02,
     &   3.35238747E-02, 3.26924554E-02, 3.20120900E-02, 3.14378631E-02,
     &   3.04962598E-02, 3.05344794E-02, 3.01723227E-02, 2.98437553E-02,
     &   2.95427276E-02, 2.92645260E-02, 2.90031678E-02, 2.87544721E-02,
     &   2.85140070E-02, 2.82765350E-02, 2.80343255E-02, 2.77667956E-02,
     &   2.75885468E-02, 2.48191061E-02, 2.34787224E-02, 2.25088996E-02,
     &   2.17400598E-02, 2.11052946E-02, 2.10318480E-02, 2.01518723E-02,
     &   1.97402673E-02, 1.94185035E-02, 1.91480164E-02, 1.89237660E-02,
     &   1.88406418E-02, 1.86072789E-02, 1.85098828E-02, 1.84500762E-02,
     &   1.84278640E-02, 1.84415532E-02, 1.84902209E-02, 1.85751934E-02,
     &   1.86720772E-02, 1.88458844E-02, 1.90392803E-02, 1.92620013E-02,
     &   1.95296580E-02, 1.98369986E-02, 2.01920661E-02, 2.05969890E-02,
     &   2.10532806E-02, 2.15616770E-02, 2.21620434E-02, 2.28492371E-02,
     &   2.36428681E-02, 2.46320856E-02, 2.59396068E-02, 2.91352480E-02,
     &   3.21560211E-02, 3.32957276E-02, 3.41215356E-02, 3.47607783E-02,
     &   3.52967406E-02, 3.57506669E-02, 3.61235678E-02, 3.64480954E-02,
     &   3.67266895E-02, 3.69629429E-02, 3.71741755E-02, 3.73191957E-02,
     &   3.74454247E-02, 3.75451759E-02, 3.76123576E-02, 3.76524710E-02,
     &   3.76642370E-02, 3.76477865E-02, 3.76049370E-02, 3.75311941E-02,
     &   3.74273315E-02, 3.72941834E-02, 3.71294063E-02, 3.68754464E-02,
     &   3.65904473E-02, 3.62715886E-02, 3.60973226E-02, 3.57259736E-02,
     &   3.52963028E-02, 3.47857521E-02, 3.41956248E-02, 3.34770665E-02/
  
         DATA ((DGS  (I,J), I=1901,2000), J=1,1) /
     &   3.25729471E-02, 3.13468954E-02, 2.91854648E-02, 2.90696111E-02,
     &   2.84417047E-02, 2.82607003E-02, 2.78804229E-02, 2.77039040E-02,
     &   2.75623110E-02, 2.74447587E-02, 2.73456389E-02, 2.72609530E-02,
     &   2.71478729E-02, 2.71257493E-02, 2.70721601E-02, 2.70267439E-02,
     &   2.69833711E-02, 2.69566455E-02, 2.69314587E-02, 2.69121539E-02,
     &   2.68984441E-02, 2.68902198E-02, 2.68153122E-02, 2.68233669E-02,
     &   2.68980496E-02, 2.69107050E-02, 2.69320278E-02, 2.69586480E-02,
     &   2.69915330E-02, 2.70314112E-02, 2.70790161E-02, 2.71354866E-02,
     &   2.72017007E-02, 2.72802753E-02, 2.73735858E-02, 2.74774648E-02,
     &   2.76301621E-02, 2.78269856E-02, 2.83343845E-02, 2.87169157E-02,
     &   2.89665839E-02, 2.90998150E-02, 2.92080227E-02, 2.92991221E-02,
     &   2.93781650E-02, 2.94476013E-02, 2.95086803E-02, 2.95627111E-02,
     &   2.96106739E-02, 2.96539950E-02, 2.96929320E-02, 2.97268646E-02,
     &   2.97556768E-02, 2.97796501E-02, 2.97993436E-02, 2.98159583E-02,
     &   2.98298086E-02, 2.98404131E-02, 2.98566066E-02, 2.98517640E-02,
     &   2.98515434E-02, 2.98483385E-02, 2.98422964E-02, 2.98326899E-02,
     &   2.98194524E-02, 2.98029017E-02, 2.97824866E-02, 2.97584196E-02,
     &   2.97304845E-02, 2.97331196E-02, 2.97016520E-02, 2.96642452E-02,
     &   2.96193734E-02, 2.95230123E-02, 2.91273241E-02, 2.92094728E-02,
     &   2.91992940E-02, 2.91904666E-02, 2.91829901E-02, 2.91768540E-02,
     &   2.91720380E-02, 2.91684857E-02, 2.91661736E-02, 2.91650888E-02,
     &   2.91579514E-02, 2.91585642E-02, 2.91624339E-02, 2.90190831E-02,
     &   2.90247602E-02, 2.90321446E-02, 2.91847476E-02, 2.91938386E-02,
     &   2.92044190E-02, 2.92165636E-02, 2.92304959E-02, 2.92459770E-02,
     &   2.92633735E-02, 2.92827787E-02, 2.93042811E-02, 2.93282098E-02/
  
         DATA ((DGS  (I,J), I=2001,2100), J=1,1) /
     &   2.93544322E-02, 2.93838758E-02, 2.94167804E-02, 2.94537028E-02,
     &   2.94952659E-02, 2.95432798E-02, 2.95998676E-02, 2.96695520E-02,
     &   2.97633555E-02, 3.00009189E-02, 3.02388235E-02, 3.03354958E-02,
     &   3.04461970E-02, 3.05154487E-02, 3.05751721E-02, 3.05434408E-02,
     &   3.05843283E-02, 3.06215214E-02, 3.06555618E-02, 3.06866424E-02,
     &   3.07158069E-02, 3.07428575E-02, 3.07681667E-02, 3.07918175E-02,
     &   3.08137981E-02, 3.08345296E-02, 3.08537393E-02, 3.08714424E-02,
     &   3.08892651E-02, 3.09043424E-02, 3.09176434E-02, 3.09307925E-02,
     &   3.09428508E-02, 3.09539458E-02, 3.09640721E-02, 3.09732283E-02,
     &   3.09814106E-02, 3.09886920E-02, 3.09950060E-02, 3.10004119E-02,
     &   3.10087116E-02, 3.10112137E-02, 3.10128289E-02, 3.10134349E-02,
     &   3.10128089E-02, 3.09954107E-02, 3.08486824E-02, 3.06510065E-02,
     &   3.06268880E-02, 3.05307971E-02, 3.04540478E-02, 3.03793344E-02,
     &   3.03156971E-02, 3.02521008E-02, 3.01917485E-02, 3.01270872E-02,
     &   3.00587261E-02, 2.99888121E-02, 2.98223780E-02, 2.96559438E-02,
     &   2.97194809E-02, 2.97794770E-02, 2.98322014E-02, 2.98835339E-02,
     &   2.99297419E-02, 2.99755969E-02, 3.00175167E-02, 3.00596431E-02,
     &   3.01002972E-02, 3.01380026E-02, 3.01763523E-02, 3.02121384E-02,
     &   3.02487254E-02, 3.02830187E-02, 3.03182134E-02, 3.03513110E-02,
     &   3.03853762E-02, 3.04174920E-02, 3.04506192E-02, 3.04819111E-02,
     &   3.05128457E-02, 3.05448314E-02, 3.05751097E-02, 3.06064546E-02,
     &   3.06361585E-02, 3.06669388E-02, 3.06961332E-02, 3.07264096E-02,
     &   3.07551469E-02, 3.07849686E-02, 3.08132914E-02, 3.08426991E-02,
     &   3.08719086E-02, 3.08996701E-02, 3.09285144E-02, 3.09559396E-02,
     &   3.09844449E-02, 3.10115569E-02, 3.10397456E-02, 3.10665644E-02/
  
         DATA ((DGS  (I,J), I=2101,2174), J=1,1) /
     &   3.10944559E-02, 3.11209987E-02, 3.11486099E-02, 3.11748921E-02,
     &   3.12010517E-02, 3.12282727E-02, 3.12541911E-02, 3.12811660E-02,
     &   3.13068545E-02, 3.13260927E-02, 3.13542426E-02, 3.13808873E-02,
     &   3.14062779E-02, 3.14327247E-02, 3.14579297E-02, 3.14841861E-02,
     &   3.15103478E-02, 3.15352851E-02, 3.15612665E-02, 3.15860342E-02,
     &   3.16118412E-02, 3.16364447E-02, 3.16620828E-02, 3.16868450E-02,
     &   3.17126221E-02, 3.17367124E-02, 3.17620686E-02, 3.17862094E-02,
     &   3.18113695E-02, 3.18353619E-02, 3.18592831E-02, 3.18842164E-02,
     &   3.19079944E-02, 3.19327799E-02, 3.19564182E-02, 3.19810592E-02,
     &   3.20045608E-02, 3.21044535E-02, 3.21277927E-02, 3.21520797E-02,
     &   3.21752035E-02, 3.21992670E-02, 3.23833126E-02, 3.26350795E-02,
     &   3.26747286E-02, 3.44435189E-02, 3.52411742E-02, 3.73368859E-02,
     &   3.87408947E-02, 3.85719797E-02, 4.01479781E-02, 4.01529760E-02,
     &   4.13353209E-02, 4.13805472E-02, 4.14076653E-02, 4.19949094E-02,
     &   4.23860480E-02, 4.24562514E-02, 4.25709918E-02, 4.27410562E-02,
     &   4.28610496E-02, 4.29364153E-02, 4.36676212E-02, 4.37056137E-02,
     &   4.39283501E-02, 4.39683024E-02, 4.45839011E-02, 4.42127948E-02,
     &   4.43364590E-02, 4.43902770E-02, 4.44763145E-02, 4.47000777E-02,
     &   4.47951979E-02, 4.54247138E-02/
  
         DATA ((DGS  (I,J), I=   1, 100), J=2,2) /
     &   1.19068029E-05, 1.26186028E-05, 1.33595559E-05, 1.41596559E-05,
     &   1.50585603E-05, 1.54249238E-05, 1.76254781E-05, 1.81185857E-05,
     &   1.90726156E-05, 2.01080651E-05, 1.98406474E-05, 1.97591556E-05,
     &   1.97545799E-05, 2.00972807E-05, 2.06060470E-05, 2.12280263E-05,
     &   2.18079697E-05, 2.23092918E-05, 2.27144291E-05, 2.33429898E-05,
     &   2.38908656E-05, 2.45037885E-05, 2.49643376E-05, 2.55680716E-05,
     &   2.61183698E-05, 2.67634817E-05, 2.76412167E-05, 2.80969254E-05,
     &   2.83099531E-05, 2.89581797E-05, 2.96343701E-05, 3.02333262E-05,
     &   3.09616114E-05, 3.16967426E-05, 3.24555482E-05, 3.32468923E-05,
     &   3.40656780E-05, 3.49871350E-05, 3.58973706E-05, 3.68334581E-05,
     &   3.76390698E-05, 3.86979663E-05, 3.98478028E-05, 4.08263236E-05,
     &   4.23148560E-05, 4.42722529E-05, 4.58506619E-05, 4.69457161E-05,
     &   4.74759419E-05, 4.79753197E-05, 5.51253423E-05, 6.34410151E-05,
     &   7.48508185E-05, 8.80905360E-05, 9.67120873E-05, 9.87498644E-05,
     &   1.01148654E-04, 1.02767515E-04, 1.05523504E-04, 1.05450301E-04,
     &   1.06704957E-04, 1.07229855E-04, 1.08769134E-04, 1.11416857E-04,
     &   1.14824877E-04, 1.18235123E-04, 1.20284648E-04, 1.22034394E-04,
     &   1.23172102E-04, 1.24453748E-04, 1.26448660E-04, 1.23293735E-04,
     &   1.23140506E-04, 1.23602176E-04, 1.23741546E-04, 1.24363511E-04,
     &   1.22411441E-04, 1.23429242E-04, 1.23783167E-04, 1.72646069E-04,
     &   1.55166279E-04, 1.58431030E-04, 1.55171783E-04, 1.53138152E-04,
     &   1.49403094E-04, 1.55833346E-04, 1.59465663E-04, 1.54093148E-04,
     &   1.52510768E-04, 1.51749224E-04, 1.49496450E-04, 1.51095166E-04,
     &   1.52999943E-04, 1.57368572E-04, 1.64219388E-04, 1.76650364E-04,
     &   1.71203826E-04, 1.69193189E-04, 1.60075365E-04, 1.51607264E-04/
  
         DATA ((DGS  (I,J), I= 101, 200), J=2,2) /
     &   1.42173382E-04, 1.34523290E-04, 1.27684500E-04, 1.22840407E-04,
     &   1.20778557E-04, 1.20049486E-04, 1.20790263E-04, 1.22377568E-04,
     &   1.24900389E-04, 1.28645745E-04, 1.33231992E-04, 1.37111654E-04,
     &   1.40797912E-04, 1.44344931E-04, 1.47622508E-04, 1.49608164E-04,
     &   1.49187857E-04, 1.40377331E-04, 1.29327337E-04, 1.26123391E-04,
     &   1.24598912E-04, 1.24082762E-04, 1.24454116E-04, 1.25474180E-04,
     &   1.26330878E-04, 1.26179032E-04, 1.26199487E-04, 1.26603049E-04,
     &   1.27723478E-04, 1.29087143E-04, 1.31988616E-04, 1.36576468E-04,
     &   1.43531416E-04, 1.52349010E-04, 1.62912122E-04, 1.73676495E-04,
     &   1.82866100E-04, 1.87058546E-04, 1.82828458E-04, 1.75669368E-04,
     &   1.71225016E-04, 1.61775364E-04, 1.53237491E-04, 1.45455520E-04,
     &   1.39502289E-04, 1.35483206E-04, 1.32039651E-04, 1.31303759E-04,
     &   1.29567607E-04, 1.29899222E-04, 1.29819029E-04, 1.36853400E-04,
     &   1.38290837E-04, 1.41236271E-04, 1.23834412E-04, 1.28811757E-04,
     &   1.35601575E-04, 1.34005187E-04, 1.19032747E-04, 1.20113792E-04,
     &   1.23112142E-04, 1.30282255E-04, 1.32570484E-04, 1.32068085E-04,
     &   1.31542648E-04, 1.30421079E-04, 1.29862314E-04, 1.29483084E-04,
     &   1.29642779E-04, 1.32182602E-04, 1.35837009E-04, 1.39151594E-04,
     &   1.44596019E-04, 1.49333381E-04, 1.54151639E-04, 1.58217738E-04,
     &   1.63102213E-04, 1.68578267E-04, 1.74006853E-04, 1.77524271E-04,
     &   1.84617034E-04, 1.88095657E-04, 1.97055859E-04, 2.05281870E-04,
     &   2.16609203E-04, 2.27235182E-04, 2.41831007E-04, 2.55456205E-04,
     &   2.71128699E-04, 2.92706189E-04, 3.05067929E-04, 3.13929380E-04,
     &   3.33546250E-04, 3.44648640E-04, 3.55290625E-04, 3.67543686E-04,
     &   3.81181620E-04, 3.96604070E-04, 4.13283656E-04, 4.49432578E-04/
  
         DATA ((DGS  (I,J), I= 201, 300), J=2,2) /
     &   4.91392874E-04, 5.16288744E-04, 5.25752287E-04, 5.35629704E-04,
     &   5.45939970E-04, 5.56704697E-04, 5.67954468E-04, 5.79705168E-04,
     &   5.91970514E-04, 6.04810316E-04, 6.18250911E-04, 6.32330626E-04,
     &   6.47089369E-04, 6.62554893E-04, 6.78795130E-04, 6.95909918E-04,
     &   7.13987597E-04, 7.33092145E-04, 7.53358621E-04, 7.74930289E-04,
     &   7.97981149E-04, 8.22725619E-04, 8.49343782E-04, 8.78431245E-04,
     &   9.10164167E-04, 9.45240288E-04, 9.84467001E-04, 1.02939402E-03,
     &   1.08311519E-03, 1.15939549E-03, 1.22868867E-03, 1.32178883E-03,
     &   1.21584292E-03, 1.21939307E-03, 1.24899283E-03, 1.28719819E-03,
     &   1.33301339E-03, 1.38490854E-03, 1.44724143E-03, 1.50689490E-03,
     &   1.57796748E-03, 1.65679192E-03, 1.72898356E-03, 1.94235119E-03,
     &   1.95162206E-03, 2.07478463E-03, 2.21408694E-03, 2.37253007E-03,
     &   2.55380361E-03, 2.76241552E-03, 3.00355983E-03, 3.28348224E-03,
     &   3.60772849E-03, 3.92195369E-03, 4.09382798E-03, 4.21075817E-03,
     &   4.37242579E-03, 4.45038457E-03, 4.40302050E-03, 4.69078869E-03,
     &   4.80566910E-03, 4.91017824E-03, 5.00271996E-03, 5.07613642E-03,
     &   5.12232566E-03, 5.13111811E-03, 5.04949223E-03, 4.90360701E-03,
     &   4.79980099E-03, 4.51683785E-03, 4.11996680E-03, 3.59668569E-03,
     &   2.74062788E-03, 2.17001999E-03, 1.30238082E-03, 4.22751565E-04,
     &   6.74676804E-04, 1.57062709E-03, 2.41238321E-03, 3.14963338E-03,
     &   3.76224637E-03, 4.24696765E-03, 4.60746901E-03, 4.86003775E-03,
     &   5.02112223E-03, 5.10726681E-03, 5.13443670E-03, 5.11632581E-03,
     &   5.06439616E-03, 4.98799108E-03, 4.84473124E-03, 4.80455697E-03,
     &   4.60153836E-03, 4.55117730E-03, 4.44250915E-03, 4.32457363E-03,
     &   4.20844227E-03, 4.09448587E-03, 3.98325594E-03, 3.87512222E-03/
  
         DATA ((DGS  (I,J), I= 301, 400), J=2,2) /
     &   3.86070014E-03, 3.67047565E-03, 3.57378950E-03, 3.48090695E-03,
     &   3.18475449E-03, 2.92942079E-03, 2.70934113E-03, 2.51847509E-03,
     &   2.35214070E-03, 2.17348178E-03, 2.07798554E-03, 1.96427806E-03,
     &   1.86311657E-03, 1.77274888E-03, 1.69175027E-03, 1.61896897E-03,
     &   1.55348971E-03, 1.49454136E-03, 1.44156003E-03, 1.39420081E-03,
     &   1.34878121E-03, 1.31315119E-03, 1.32574547E-03, 1.27145814E-03,
     &   1.30468102E-03, 1.55615430E-03, 1.52148223E-03, 1.21985687E-03,
     &   1.12618829E-03, 1.06603176E-03, 1.02042932E-03, 9.82471228E-04,
     &   9.51331926E-04, 9.21354888E-04, 8.94769252E-04, 8.70825293E-04,
     &   8.48651114E-04, 8.27922595E-04, 8.09224179E-04, 7.90215985E-04,
     &   7.72804159E-04, 7.56138742E-04, 7.39848757E-04, 7.24368444E-04,
     &   7.09816471E-04, 6.95852122E-04, 6.82707373E-04, 6.69979908E-04,
     &   6.57822544E-04, 6.46185582E-04, 6.35079143E-04, 6.24470073E-04,
     &   6.14139488E-04, 6.04270742E-04, 5.94778859E-04, 5.85883800E-04,
     &   5.61842905E-04, 5.10499779E-04, 4.38199926E-04, 3.93116588E-04,
     &   3.62866455E-04, 3.42224856E-04, 3.28501845E-04, 3.19460787E-04,
     &   3.13818088E-04, 3.13322418E-04, 3.11034793E-04, 3.13006598E-04,
     &   3.13986485E-04, 3.17083119E-04, 3.22148857E-04, 3.22503993E-04,
     &   3.27876123E-04, 3.34988897E-04, 3.38551259E-04, 3.43383443E-04,
     &   3.52680760E-04, 3.63993273E-04, 3.76900552E-04, 3.91480518E-04,
     &   4.00438377E-04, 4.07470541E-04, 4.25456960E-04, 4.46034412E-04,
     &   4.47893331E-04, 4.67281477E-04, 4.91601156E-04, 5.02867517E-04,
     &   5.16891799E-04, 5.43258427E-04, 5.74978768E-04, 6.13977107E-04,
     &   6.25784408E-04, 6.61409944E-04, 6.84125626E-04, 7.14551256E-04,
     &   7.74526846E-04, 8.61136674E-04, 8.69381849E-04, 9.52710376E-04/
  
         DATA ((DGS  (I,J), I= 401, 500), J=2,2) /
     &   1.13232247E-03, 1.17495064E-03, 1.10519834E-03, 1.04594632E-03,
     &   1.02824784E-03, 9.53941991E-04, 9.43659574E-04, 9.37149643E-04,
     &   9.16345997E-04, 8.62886745E-04, 8.15228522E-04, 7.70431118E-04,
     &   7.70734007E-04, 7.55610294E-04, 7.46898549E-04, 7.20749329E-04,
     &   7.18440896E-04, 6.82179150E-04, 6.80455639E-04, 6.68181187E-04,
     &   6.68752587E-04, 6.71825384E-04, 6.54805543E-04, 6.35162990E-04,
     &   6.30058461E-04, 6.87574156E-04, 6.92466349E-04, 7.41482400E-04,
     &   7.48060712E-04, 7.80374761E-04, 8.04797714E-04, 8.05972123E-04,
     &   8.10064917E-04, 8.05064038E-04, 8.01584368E-04, 7.75676802E-04,
     &   7.73164503E-04, 7.12966895E-04, 7.08951982E-04, 6.66857848E-04,
     &   6.21012949E-04, 6.08888497E-04, 5.92742881E-04, 5.89272132E-04,
     &   5.42270403E-04, 5.32419700E-04, 5.42537919E-04, 5.09694511E-04,
     &   5.16676706E-04, 4.54253282E-04, 4.72987844E-04, 4.71659298E-04,
     &   4.73329959E-04, 4.83292006E-04, 4.62648143E-04, 4.43948548E-04,
     &   4.87478467E-04, 5.08372478E-04, 5.57572323E-04, 5.74062042E-04,
     &   5.87624238E-04, 5.84903291E-04, 5.92334708E-04, 5.88626449E-04,
     &   5.97094012E-04, 6.06503055E-04, 6.08038987E-04, 6.31659352E-04,
     &   6.72814327E-04, 6.94483247E-04, 7.03538273E-04, 7.48205523E-04,
     &   7.82777272E-04, 8.08420657E-04, 8.12919209E-04, 8.52343906E-04,
     &   8.63855579E-04, 8.56356661E-04, 8.85476289E-04, 9.12599376E-04,
     &   9.15158314E-04, 9.64641733E-04, 9.91242803E-04, 1.00135868E-03,
     &   1.01807319E-03, 1.12644849E-03, 1.17627863E-03, 1.22287920E-03,
     &   1.28254711E-03, 1.30380766E-03, 1.35308029E-03, 1.36376276E-03,
     &   1.41264041E-03, 1.46664995E-03, 1.51122219E-03, 1.53123143E-03,
     &   1.91139799E-03, 2.23934778E-03, 2.51950497E-03, 1.69105857E-03/
  
         DATA ((DGS  (I,J), I= 501, 600), J=2,2) /
     &   1.48540251E-03, 1.44846630E-03, 9.29218212E-04, 8.64034373E-04,
     &   8.18860873E-04, 7.27799753E-04, 7.09557396E-04, 6.67366712E-04,
     &   6.33990868E-04, 6.19087587E-04, 6.14494434E-04, 6.18916278E-04,
     &   6.32842206E-04, 6.60421169E-04, 7.11106657E-04, 8.01695466E-04,
     &   8.28170547E-04, 8.56736878E-04, 8.90784101E-04, 9.30611818E-04,
     &   9.77177500E-04, 1.03177847E-03, 1.09678637E-03, 1.11113393E-03,
     &   1.12607796E-03, 1.14162095E-03, 1.15777931E-03, 1.17457962E-03,
     &   1.19178510E-03, 1.20970782E-03, 1.22838239E-03, 1.24785392E-03,
     &   1.26812354E-03, 1.36850898E-03, 1.44977708E-03, 1.49477152E-03,
     &   1.52639423E-03, 1.55947586E-03, 1.59427007E-03, 1.63085439E-03,
     &   1.66938341E-03, 1.71000297E-03, 1.73115075E-03, 1.75289741E-03,
     &   1.77525092E-03, 1.79825231E-03, 1.82191244E-03, 1.84627527E-03,
     &   1.87135499E-03, 1.89719879E-03, 1.92382462E-03, 1.95128344E-03,
     &   1.97959751E-03, 2.00881519E-03, 2.03897884E-03, 2.07013355E-03,
     &   2.10232737E-03, 2.13561156E-03, 2.17004078E-03, 2.20567347E-03,
     &   2.24257208E-03, 2.28080348E-03, 2.32043931E-03, 2.36155646E-03,
     &   2.40423750E-03, 2.44857126E-03, 2.49465341E-03, 2.54258711E-03,
     &   2.59248383E-03, 2.64446411E-03, 2.69865856E-03, 2.75520890E-03,
     &   2.81427123E-03, 2.87601166E-03, 2.94061332E-03, 3.00827641E-03,
     &   3.07922021E-03, 3.15368547E-03, 3.23193702E-03, 3.31426692E-03,
     &   3.40099804E-03, 3.49248834E-03, 3.54013932E-03, 3.58913514E-03,
     &   3.63952827E-03, 3.69138204E-03, 3.74475744E-03, 3.79972444E-03,
     &   3.85635187E-03, 3.91471803E-03, 3.97490082E-03, 4.03698818E-03,
     &   4.10106854E-03, 4.16724112E-03, 4.23560665E-03, 4.30627753E-03,
     &   4.37936895E-03, 4.45500883E-03, 4.53332941E-03, 4.61447703E-03/
  
         DATA ((DGS  (I,J), I= 601, 700), J=2,2) /
     &   4.69860364E-03, 4.78587874E-03, 4.87647789E-03, 4.97059497E-03,
     &   5.06843760E-03, 5.17023007E-03, 5.27621505E-03, 5.38665549E-03,
     &   5.50183681E-03, 5.62206934E-03, 5.74769109E-03, 5.87907093E-03,
     &   6.01661216E-03, 6.16075643E-03, 6.31198864E-03, 6.47084360E-03,
     &   6.63790922E-03, 6.81383760E-03, 6.99934958E-03, 7.19524860E-03,
     &   7.48867900E-03, 7.62188968E-03, 7.85475374E-03, 8.10228417E-03,
     &   8.36590698E-03, 8.64724172E-03, 8.94813025E-03, 9.27068149E-03,
     &   9.61731637E-03, 9.99083301E-03, 1.03944779E-02, 1.08320447E-02,
     &   1.13079892E-02, 1.16879608E-02, 1.28718173E-02, 1.43218302E-02,
     &   1.61390064E-02, 1.84828632E-02, 2.16209608E-02, 2.60390163E-02,
     &   3.27201319E-02, 4.40003358E-02, 6.71145324E-02, 1.41120989E-01,
     &   1.49226880E-01, 1.58317043E-01, 1.68581624E-01, 1.80262882E-01,
     &   1.93674266E-01, 2.09228454E-01, 2.27479678E-01, 2.49189505E-01,
     &   2.75432575E-01, 3.07773264E-01, 3.48574332E-01, 4.01564549E-01,
     &   4.72944353E-01, 4.81454688E-01, 6.72011986E-01, 8.10903443E-01,
     &   9.22494717E-01, 1.01954491E+00, 1.09432986E+00, 1.15967297E+00,
     &   1.21204091E+00, 1.26421099E+00, 1.30004874E+00, 1.33323153E+00,
     &   1.35664650E+00, 1.37594951E+00, 1.38679684E+00, 1.39489367E+00,
     &   1.39558217E+00, 1.39150722E+00, 1.38020671E+00, 1.36799353E+00,
     &   1.34157625E+00, 1.31790659E+00, 1.28885231E+00, 1.25395639E+00,
     &   1.21278043E+00, 1.16492124E+00, 1.11003023E+00, 1.04783501E+00,
     &   9.78162615E-01, 9.00962999E-01, 8.16331391E-01, 6.76065611E-01,
     &   5.74382556E-01, 4.11386265E-01, 2.38683475E-01, 2.38518782E-01,
     &   6.75921395E-01, 8.16198765E-01, 9.40381712E-01, 1.01382916E+00,
     &   1.10993372E+00, 1.16483419E+00, 1.21270278E+00, 1.25388802E+00/
  
         DATA ((DGS  (I,J), I= 701, 800), J=2,2) /
     &   1.28879300E+00, 1.31785610E+00, 1.34153427E+00, 1.36028936E+00,
     &   1.37457717E+00, 1.38483904E+00, 1.39149581E+00, 1.39558075E+00,
     &   1.39489845E+00, 1.38958251E+00, 1.37997805E+00, 1.37161543E+00,
     &   1.35115990E+00, 1.33327302E+00, 1.30699732E+00, 1.27882478E+00,
     &   1.24206857E+00, 1.20459766E+00, 1.15234629E+00, 1.10150991E+00,
     &   1.03940008E+00, 9.69350268E-01, 8.79249468E-01, 7.75961211E-01,
     &   6.46989944E-01, 5.33137641E-01, 4.44699966E-01, 3.80886247E-01,
     &   3.32837438E-01, 2.95424521E-01, 2.65501191E-01, 2.41040167E-01,
     &   2.20680263E-01, 2.03475313E-01, 1.88748029E-01, 1.76001243E-01,
     &   1.64861991E-01, 1.55045134E-01, 1.46329018E-01, 1.38538780E-01,
     &   1.31534654E-01, 1.25203599E-01, 1.19453218E-01, 6.26390762E-02,
     &   4.24554817E-02, 3.21166953E-02, 2.58328186E-02, 2.16097415E-02,
     &   1.85766938E-02, 1.62929042E-02, 1.45113490E-02, 1.30828377E-02,
     &   1.19119454E-02, 1.17732818E-02, 1.08064761E-02, 9.98762261E-03,
     &   9.28520866E-03, 8.67607840E-03, 8.14283102E-03, 7.67214345E-03,
     &   7.25363712E-03, 6.87911210E-03, 6.54199444E-03, 6.23696349E-03,
     &   5.95966065E-03, 5.70648238E-03, 5.47442374E-03, 5.26095896E-03,
     &   5.06394964E-03, 4.88157319E-03, 4.71226663E-03, 4.55468195E-03,
     &   4.40765040E-03, 4.27015362E-03, 4.14130025E-03, 4.02030671E-03,
     &   3.90648091E-03, 3.79921116E-03, 3.69795237E-03, 3.60221770E-03,
     &   3.51157304E-03, 3.42562763E-03, 3.34403008E-03, 3.26646316E-03,
     &   3.19263983E-03, 3.12229893E-03, 3.05520510E-03, 2.99114204E-03,
     &   2.92991279E-03, 2.87133712E-03, 2.81524984E-03, 2.76149923E-03,
     &   2.70994573E-03, 2.66046071E-03, 2.61292544E-03, 2.56723018E-03,
     &   2.52327333E-03, 2.48096067E-03, 2.44020477E-03, 2.40092432E-03/
  
         DATA ((DGS  (I,J), I= 801, 900), J=2,2) /
     &   2.36304367E-03, 2.32649231E-03, 2.29120446E-03, 2.25711870E-03,
     &   2.22417758E-03, 2.19232737E-03, 2.16151773E-03, 2.13170145E-03,
     &   2.10283424E-03, 2.07487453E-03, 2.04778324E-03, 2.02152361E-03,
     &   1.99606108E-03, 1.97136311E-03, 1.94739905E-03, 1.92414005E-03,
     &   1.90155891E-03, 1.87963003E-03, 1.85832927E-03, 1.83763394E-03,
     &   1.81752268E-03, 1.79797540E-03, 1.77897328E-03, 1.76049869E-03,
     &   1.74253518E-03, 1.72506747E-03, 1.70808145E-03, 1.69156418E-03,
     &   1.67550396E-03, 1.65989039E-03, 1.64471446E-03, 1.62996874E-03,
     &   1.61564758E-03, 1.60174742E-03, 1.58826732E-03, 1.57520961E-03,
     &   1.56258100E-03, 1.54941049E-03, 1.53655317E-03, 1.52400152E-03,
     &   1.51174880E-03, 1.49978918E-03, 1.48811785E-03, 1.47673132E-03,
     &   1.46562774E-03, 1.45587478E-03, 1.45088441E-03, 1.44015135E-03,
     &   1.42963802E-03, 1.41933826E-03, 1.40924617E-03, 1.39935606E-03,
     &   1.38966249E-03, 1.38016019E-03, 1.37084412E-03, 1.36170741E-03,
     &   1.09065962E-03, 9.30637806E-04, 8.47446261E-04, 8.72981508E-04,
     &   9.69997077E-04, 1.03904918E-03, 1.13435518E-03, 1.26774430E-03,
     &   1.40489773E-03, 1.59163049E-03, 1.67030886E-03, 1.84833972E-03,
     &   1.95867899E-03, 2.08497673E-03, 2.22958692E-03, 2.40045837E-03,
     &   2.60368096E-03, 2.84830185E-03, 3.14758189E-03, 3.52143631E-03,
     &   4.00086907E-03, 4.23261819E-03, 4.49364157E-03, 4.78976401E-03,
     &   5.12849067E-03, 5.51962727E-03, 5.97621231E-03, 6.23436638E-03,
     &   6.51603434E-03, 6.82455693E-03, 7.16394662E-03, 7.53904937E-03,
     &   7.95578872E-03, 8.42146830E-03, 8.94521216E-03, 9.53855659E-03,
     &   1.02163162E-02, 1.09978252E-02, 1.19088143E-02, 1.29375528E-02,
     &   1.36111236E-02, 1.43586099E-02, 1.51928644E-02, 1.61299024E-02/
  
         DATA ((DGS  (I,J), I= 901,1000), J=2,2) /
     &   1.71899371E-02, 1.83988523E-02, 1.97903384E-02, 2.14090750E-02,
     &   2.33156056E-02, 2.55940590E-02, 2.83648614E-02, 3.18066541E-02,
     &   3.61962465E-02, 4.19866304E-02, 4.99731983E-02, 6.16908690E-02,
     &   8.05252801E-02, 1.15640555E-01, 1.70870584E-01, 2.02305398E-01,
     &   2.36592869E-01, 3.20956156E-01, 3.51350253E-01, 3.76435172E-01,
     &   3.97738086E-01, 4.15676355E-01, 4.30393785E-01, 4.43066273E-01,
     &   4.53442574E-01, 4.60868905E-01, 4.67177889E-01, 4.71644087E-01,
     &   4.74293076E-01, 4.75249812E-01, 4.74570403E-01, 4.72713930E-01,
     &   4.68946259E-01, 4.64016079E-01, 4.57370467E-01, 4.48820054E-01,
     &   4.39829230E-01, 4.27232477E-01, 4.14496891E-01, 3.99859160E-01,
     &   3.83225339E-01, 3.61195381E-01, 3.40004089E-01, 3.12557718E-01,
     &   2.86715377E-01, 2.49038504E-01, 2.07920087E-01, 1.46507218E-01,
     &   9.29759149E-02, 3.13192099E-02, 6.99165677E-04, 3.11628569E-02,
     &   6.21909634E-02, 1.46358981E-01, 2.07780878E-01, 2.48907441E-01,
     &   2.86593496E-01, 3.12443245E-01, 3.39898660E-01, 3.61097933E-01,
     &   3.83137337E-01, 3.99779276E-01, 4.14425072E-01, 4.27168609E-01,
     &   4.39774430E-01, 4.48772803E-01, 4.57331723E-01, 4.63985487E-01,
     &   4.68923444E-01, 4.72699528E-01, 4.74562946E-01, 4.75250721E-01,
     &   4.74301737E-01, 4.71660673E-01, 4.67202414E-01, 4.60901234E-01,
     &   4.53481899E-01, 4.43113335E-01, 4.30448442E-01, 4.15738254E-01,
     &   3.97807306E-01, 3.76511613E-01, 3.51433748E-01, 3.21046677E-01,
     &   2.84207939E-01, 3.04208008E-01, 2.60232500E-01, 2.33146306E-01,
     &   2.16770895E-01, 1.82848600E-01, 1.70981554E-01, 1.45435779E-01,
     &   9.41713097E-02, 6.94841234E-02, 5.50226493E-02, 4.55364649E-02,
     &   3.88387844E-02, 3.38591163E-02, 3.00122506E-02, 2.69514706E-02/
  
         DATA ((DGS  (I,J), I=1001,1100), J=2,2) /
     &   2.44583691E-02, 2.23885509E-02, 2.06427452E-02, 1.91504607E-02,
     &   1.78602679E-02, 1.67337630E-02, 1.57416915E-02, 1.48613811E-02,
     &   1.40749788E-02, 1.33682588E-02, 1.22513529E-02, 1.08578068E-02,
     &   1.01128338E-02, 8.89603195E-03, 7.94452079E-03, 7.01122774E-03,
     &   5.68597178E-03, 4.24134144E-03, 3.39285216E-03, 3.04259106E-03,
     &   2.84746656E-03, 2.72111488E-03, 2.63197715E-03, 2.56498665E-03,
     &   2.51246158E-03, 2.46985477E-03, 2.43429053E-03, 2.40393132E-03,
     &   2.37752963E-03, 2.35414048E-03, 2.33312225E-03, 2.31396742E-03,
     &   2.29630244E-03, 2.27980159E-03, 2.26419658E-03, 2.24937241E-03,
     &   2.23508374E-03, 2.22119781E-03, 2.20761140E-03, 2.19421495E-03,
     &   2.18086498E-03, 2.16746523E-03, 2.15387200E-03, 2.14000663E-03,
     &   2.12568015E-03, 2.11072033E-03, 2.09484438E-03, 2.07774293E-03,
     &   2.05881037E-03, 2.03693612E-03, 2.00935035E-03, 1.94735960E-03,
     &   1.89052177E-03, 1.87040402E-03, 1.85660640E-03, 1.84615719E-03,
     &   1.83783764E-03, 1.83099996E-03, 1.82518660E-03, 1.82007810E-03,
     &   1.81535838E-03, 1.80958841E-03, 1.80600787E-03, 1.80058057E-03,
     &   1.79380685E-03, 1.78384323E-03, 1.75478014E-03, 1.78032304E-03,
     &   1.78657531E-03, 1.78946802E-03, 1.78995417E-03, 1.78856942E-03,
     &   1.78541663E-03, 1.78040839E-03, 1.77348714E-03, 1.76445721E-03,
     &   1.75296756E-03, 1.73884524E-03, 1.72065462E-03, 1.69676953E-03,
     &   1.66825901E-03, 1.63355710E-03, 1.59071640E-03, 1.53528492E-03,
     &   1.46774910E-03, 1.37566377E-03, 1.34907121E-03, 1.27356812E-03,
     &   1.16960441E-03, 1.16001606E-03, 1.09462873E-03, 1.07273895E-03,
     &   1.05720458E-03, 1.03685399E-03, 1.01719103E-03, 1.00238671E-03,
     &   9.94276692E-04, 9.87163643E-04, 9.76139730E-04, 9.59041575E-04/
  
         DATA ((DGS  (I,J), I=1101,1200), J=2,2) /
     &   9.51892067E-04, 9.22113236E-04, 8.89329306E-04, 8.87447537E-04,
     &   8.63407785E-04, 8.40922759E-04, 8.38919114E-04, 8.33694577E-04,
     &   8.39540126E-04, 8.61740074E-04, 9.29005460E-04, 8.11327708E-04,
     &   7.47016728E-04, 7.63897727E-04, 7.78315221E-04, 8.03118146E-04,
     &   7.98735251E-04, 7.80250899E-04, 7.79031830E-04, 7.72731845E-04,
     &   7.47947265E-04, 7.34968559E-04, 7.02958916E-04, 6.96441408E-04,
     &   6.96881532E-04, 6.83244276E-04, 6.74175765E-04, 6.67512615E-04,
     &   6.54116212E-04, 6.42977342E-04, 6.38462352E-04, 6.36659826E-04,
     &   6.35078880E-04, 6.35131069E-04, 6.32423769E-04, 6.28686446E-04,
     &   6.24572454E-04, 6.24463370E-04, 6.26762458E-04, 6.24156780E-04,
     &   6.10074331E-04, 6.00761895E-04, 5.94917140E-04, 5.91940813E-04,
     &   5.91141623E-04, 5.90745940E-04, 5.90848129E-04, 5.88757616E-04,
     &   5.83250047E-04, 5.79996567E-04, 5.76712199E-04, 5.74523167E-04,
     &   5.72197147E-04, 5.69273628E-04, 5.66425803E-04, 5.65098425E-04,
     &   5.63846964E-04, 5.65405095E-04, 5.61098816E-04, 5.58397596E-04,
     &   5.57810561E-04, 5.57606924E-04, 5.53914458E-04, 5.51233175E-04,
     &   5.52381365E-04, 5.48218880E-04, 5.49321067E-04, 5.58118128E-04,
     &   5.44049306E-04, 5.43365405E-04, 5.40523283E-04, 5.37858230E-04,
     &   5.40582264E-04, 5.34032216E-04, 5.32229988E-04, 5.29947924E-04,
     &   5.27938665E-04, 5.24423504E-04, 5.26850086E-04, 5.27097587E-04,
     &   5.26614106E-04, 5.24671594E-04, 5.26065010E-04, 5.26792274E-04,
     &   5.20032773E-04, 5.19158901E-04, 5.19594926E-04, 5.13944755E-04,
     &   5.12567120E-04, 5.13440303E-04, 5.12198517E-04, 5.13611739E-04,
     &   5.15194889E-04, 5.08021155E-04, 5.06691298E-04, 5.05565103E-04,
     &   5.12604340E-04, 5.08629689E-04, 5.07426945E-04, 5.05404094E-04/
  
         DATA ((DGS  (I,J), I=1201,1300), J=2,2) /
     &   5.10345337E-04, 5.20097838E-04, 5.26627692E-04, 5.20315570E-04,
     &   5.20228961E-04, 5.25136511E-04, 5.21523485E-04, 5.24893118E-04,
     &   5.37933051E-04, 5.84479226E-04, 5.57645422E-04, 5.58490683E-04,
     &   5.74108192E-04, 5.43800289E-04, 5.38569252E-04, 5.36623773E-04,
     &   5.58401054E-04, 5.61170850E-04, 6.04315480E-04, 6.08426098E-04,
     &   6.80748883E-04, 7.77985752E-04, 8.25315350E-04, 1.00388386E-03,
     &   1.14442357E-03, 1.13393774E-03, 9.69812525E-04, 8.13176435E-04,
     &   6.78818630E-04, 6.40886529E-04, 6.11933523E-04, 5.70055338E-04,
     &   5.40805375E-04, 5.29171168E-04, 5.18996543E-04, 5.10011383E-04,
     &   5.02010932E-04, 4.94836797E-04, 4.88359329E-04, 4.82488602E-04,
     &   4.77139270E-04, 4.72245297E-04, 4.67752057E-04, 4.63613956E-04,
     &   4.59792635E-04, 4.56255596E-04, 4.52975131E-04, 4.49927489E-04,
     &   4.47092212E-04, 4.44451609E-04, 4.41990328E-04, 4.39695013E-04,
     &   4.37554022E-04, 4.35557194E-04, 4.33695663E-04, 4.31961697E-04,
     &   4.30348570E-04, 4.28850452E-04, 4.27462323E-04, 4.26179895E-04,
     &   4.24999552E-04, 4.23918304E-04, 4.22933745E-04, 4.22044023E-04,
     &   4.21247819E-04, 4.20544333E-04, 4.19933273E-04, 4.19511101E-04,
     &   4.18989821E-04, 4.18425486E-04, 4.18257143E-04, 4.18511572E-04,
     &   4.19227152E-04, 4.20456375E-04, 4.22269532E-04, 4.24760092E-04,
     &   4.28052682E-04, 4.32315214E-04, 4.37777954E-04, 4.44768366E-04,
     &   4.53749388E-04, 4.65447526E-04, 4.81042876E-04, 4.82885727E-04,
     &   5.02657802E-04, 5.34685183E-04, 5.88455639E-04, 6.72390818E-04,
     &   7.06797850E-04, 8.52432021E-04, 1.13911783E-03, 1.27714783E-03,
     &   1.47511415E-03, 1.60938812E-03, 1.77914977E-03, 1.95054272E-03,
     &   2.16647534E-03, 2.29671543E-03, 2.44603797E-03, 2.61882642E-03/
  
         DATA ((DGS  (I,J), I=1301,1400), J=2,2) /
     &   2.82090853E-03, 3.06020413E-03, 3.34775565E-03, 3.51433018E-03,
     &   3.69943465E-03, 3.90628923E-03, 4.13890450E-03, 4.40233968E-03,
     &   4.70306932E-03, 5.04951538E-03, 5.45283882E-03, 5.92815156E-03,
     &   6.12622171E-03, 6.25052349E-03, 6.38007159E-03, 6.51520328E-03,
     &   6.65628541E-03, 6.80371762E-03, 6.95793619E-03, 7.11941827E-03,
     &   7.28868677E-03, 7.46631607E-03, 7.65293839E-03, 7.84925137E-03,
     &   8.05602668E-03, 8.27412013E-03, 8.50448347E-03, 8.74817822E-03,
     &   9.00639196E-03, 9.28045759E-03, 9.57187632E-03, 9.88234502E-03,
     &   1.02137893E-02, 1.05684031E-02, 1.09486976E-02, 1.13575603E-02,
     &   1.17983283E-02, 1.22748796E-02, 1.27917472E-02, 1.33542631E-02,
     &   1.39687430E-02, 1.46427241E-02, 1.53852750E-02, 1.62074053E-02,
     &   1.71226152E-02, 1.81476409E-02, 1.93034864E-02, 2.06168764E-02,
     &   2.21223463E-02, 2.38653195E-02, 2.59067658E-02, 2.83304733E-02,
     &   3.12548234E-02, 3.48526946E-02, 3.93868918E-02, 4.52773349E-02,
     &   5.32390387E-02, 6.45963607E-02, 8.21069217E-02, 1.12614040E-01,
     &   1.78999978E-01, 2.20160271E-01, 4.19336994E-01, 5.83477669E-01,
     &   7.02454631E-01, 7.94513823E-01, 8.70151710E-01, 9.36343168E-01,
     &   9.97759617E-01, 1.04051538E+00, 1.08403746E+00, 1.12684104E+00,
     &   1.14747865E+00, 1.17640186E+00, 1.19373231E+00, 1.21524398E+00,
     &   1.22085003E+00, 1.22914210E+00, 1.23258811E+00, 1.22999097E+00,
     &   1.22602812E+00, 1.22004532E+00, 1.20109890E+00, 1.18781257E+00,
     &   1.17148304E+00, 1.12940979E+00, 1.10328938E+00, 1.07343670E+00,
     &   1.03928085E+00, 9.59192129E-01, 9.12157057E-01, 8.61166810E-01,
     &   7.45261732E-01, 6.81102628E-01, 5.39910864E-01, 3.84472066E-01,
     &   4.10538753E-02, 3.79366810E-01, 5.35204445E-01, 6.76186734E-01/
  
         DATA ((DGS  (I,J), I=1401,1500), J=2,2) /
     &   8.01459002E-01, 8.57777743E-01, 9.09049639E-01, 9.55896948E-01,
     &   1.03699444E+00, 1.07107092E+00, 1.10150472E+00, 1.12785606E+00,
     &   1.17035226E+00, 1.18671634E+00, 1.20033431E+00, 1.21951561E+00,
     &   1.22570938E+00, 1.23203582E+00, 1.23260534E+00, 1.22933165E+00,
     &   1.22123045E+00, 1.20923829E+00, 1.19424940E+00, 1.17698148E+00,
     &   1.14812146E+00, 1.12761997E+00, 1.08473126E+00, 1.04132215E+00,
     &   9.98436770E-01, 9.36980584E-01, 8.70733415E-01, 7.95023762E-01,
     &   7.02873825E-01, 5.83784117E-01, 4.17090763E-01, 2.27644694E-01,
     &   1.30208801E-01, 9.11030724E-02, 7.00489144E-02, 5.68967471E-02,
     &   4.79024304E-02, 4.13640162E-02, 3.63967391E-02, 3.24951984E-02,
     &   2.93497422E-02, 2.67600737E-02, 2.45908978E-02, 2.27475334E-02,
     &   2.11617386E-02, 1.97830846E-02, 1.85735002E-02, 1.75037046E-02,
     &   1.65508097E-02, 1.56966651E-02, 1.49266931E-02, 1.42290509E-02,
     &   1.35940175E-02, 1.30135396E-02, 1.24808881E-02, 1.19903970E-02,
     &   1.15372614E-02, 1.11173798E-02, 1.07272300E-02, 1.03637703E-02,
     &   1.00243603E-02, 9.70669690E-03, 9.40876184E-03, 9.12877927E-03,
     &   8.86518031E-03, 8.61657374E-03, 8.38172157E-03, 8.15951848E-03,
     &   7.94897457E-03, 7.74920062E-03, 7.55939563E-03, 7.37883616E-03,
     &   7.20686713E-03, 7.04289397E-03, 6.88637578E-03, 6.73681938E-03,
     &   6.59377421E-03, 6.45682778E-03, 6.32560178E-03, 6.19974854E-03,
     &   6.07894809E-03, 5.16930363E-03, 4.61349192E-03, 4.08259872E-03,
     &   3.49377760E-03, 2.95008139E-03, 2.42977907E-03, 2.15780571E-03,
     &   1.78441391E-03, 1.66057612E-03, 1.56296648E-03, 1.45066271E-03,
     &   1.36063714E-03, 1.28756415E-03, 1.17560125E-03, 1.08859140E-03,
     &   8.81094618E-04, 7.72089235E-04, 7.78109358E-04, 9.63191634E-04/
  
         DATA ((DGS  (I,J), I=1501,1600), J=2,2) /
     &   1.37164832E-03, 1.96643905E-03, 2.78962518E-03, 3.94824397E-03,
     &   6.88387510E-03, 7.44249688E-03, 8.10071425E-03, 8.29151446E-03,
     &   8.72650032E-03, 9.20993736E-03, 9.75037351E-03, 1.03584934E-02,
     &   1.10478333E-02, 1.18357965E-02, 1.27451396E-02, 1.38062105E-02,
     &   1.50603842E-02, 1.65655932E-02, 1.84055095E-02, 2.07056458E-02,
     &   2.36632255E-02, 2.76069481E-02, 3.31284232E-02, 4.14108035E-02,
     &   5.52142500E-02, 8.28158713E-02, 1.19841885E-01, 1.26056674E-01,
     &   1.32950866E-01, 1.40642238E-01, 1.49277440E-01, 1.59041410E-01,
     &   1.70170734E-01, 1.82973009E-01, 1.97855604E-01, 2.15369627E-01,
     &   2.36279328E-01, 2.61675920E-01, 2.93172837E-01, 3.33259314E-01,
     &   3.85984586E-01, 4.58396856E-01, 5.63913541E-01, 7.31437965E-01,
     &   1.03545475E+00, 1.72322448E+00, 1.96432442E+00, 2.15408838E+00,
     &   2.36559686E+00, 2.54310325E+00, 2.67111725E+00, 2.77968813E+00,
     &   2.82971333E+00, 2.84957481E+00, 2.82459390E+00, 2.73607542E+00,
     &   2.63415974E+00, 2.61487471E+00, 2.60484884E+00, 2.58401644E+00,
     &   2.57538814E+00, 2.56880487E+00, 2.56212491E+00, 2.48988245E+00,
     &   2.36209861E+00, 2.19111310E+00, 2.00995089E+00, 1.77790051E+00,
     &   1.51422984E+00, 1.25174706E+00, 1.09663994E+00, 9.35672779E-01,
     &   7.69573325E-01, 5.29983938E-01, 3.55049611E-01, 1.78054868E-01,
     &   7.12933764E-02, 3.56635815E-02, 1.78486366E-02, 5.86924571E-04,
     &   1.78122890E-02, 3.56272215E-02, 1.78018580E-01, 3.55013562E-01,
     &   5.29948283E-01, 1.31197285E+00, 1.59651721E+00, 2.05245166E+00,
     &   2.22780875E+00, 2.39256010E+00, 2.52723902E+00, 2.62463100E+00,
     &   2.70884327E+00, 2.76531412E+00, 2.80922911E+00, 2.83401562E+00,
     &   2.84756208E+00, 2.84905044E+00, 2.84076505E+00, 2.82586443E+00/
  
         DATA ((DGS  (I,J), I=1601,1700), J=2,2) /
     &   2.77661229E+00, 2.74341680E+00, 2.70661273E+00, 2.67112778E+00,
     &   2.59124551E+00, 2.54752662E+00, 2.47652134E+00, 2.36561318E+00,
     &   1.96434509E+00, 1.65349482E+00, 1.41811874E+00, 1.23730028E+00,
     &   1.09537107E+00, 9.81578653E-01, 8.67958476E-01, 6.90561771E-01,
     &   6.42556126E-01, 6.00716997E-01, 5.63940364E-01, 5.09286112E-01,
     &   4.21540133E-01, 3.59481053E-01, 3.13301040E-01, 2.77610230E-01,
     &   2.49205687E-01, 2.26066099E-01, 2.06853590E-01, 1.90647694E-01,
     &   1.76794498E-01, 1.64816750E-01, 1.54358012E-01, 1.45146734E-01,
     &   1.36972416E-01, 1.29669373E-01, 1.23105415E-01, 8.28431335E-02,
     &   5.52415139E-02, 4.14380628E-02, 2.76341919E-02, 2.07329029E-02,
     &   1.65928756E-02, 1.38335308E-02, 1.18631348E-02, 1.03858506E-02,
     &   9.23731809E-03, 8.31892438E-03, 7.79816613E-03, 7.19077008E-03,
     &   6.67207473E-03, 6.22403144E-03, 5.83318405E-03, 5.48929591E-03,
     &   4.24704475E-03, 3.47326602E-03, 2.94708923E-03, 2.56761439E-03,
     &   2.28287051E-03, 2.06405158E-03, 1.88540288E-03, 1.73972867E-03,
     &   8.90123051E-04, 8.76216066E-04, 8.99793756E-04, 9.20128398E-04,
     &   9.59497597E-04, 1.02469070E-03, 1.06550053E-03, 1.12105369E-03,
     &   1.27100418E-03, 1.52714009E-03, 2.01448687E-03, 3.20429627E-03,
     &   4.37586664E-03, 4.68064722E-03, 5.03292247E-03, 5.44455384E-03,
     &   5.93173131E-03, 6.51713430E-03, 7.23348944E-03, 8.12991721E-03,
     &   9.28360706E-03, 1.08231163E-02, 1.29800659E-02, 1.62175553E-02,
     &   2.16159235E-02, 2.25549927E-02, 2.35793585E-02, 2.47014445E-02,
     &   2.59357841E-02, 2.72999439E-02, 2.88158802E-02, 3.05102074E-02,
     &   3.24161485E-02, 3.45764752E-02, 3.70454508E-02, 3.98939516E-02,
     &   4.32175291E-02, 4.71453410E-02, 5.18580209E-02, 5.76182547E-02/
  
         DATA ((DGS  (I,J), I=1701,1800), J=2,2) /
     &   5.94181843E-02, 6.48179911E-02, 7.40725159E-02, 8.64105994E-02,
     &   1.03678160E-01, 1.29563885E-01, 1.72646788E-01, 2.58536509E-01,
     &   5.12399183E-01, 6.09176031E-01, 8.59229468E-01, 1.04939009E+00,
     &   1.20607465E+00, 1.34283055E+00, 1.46416078E+00, 1.57300573E+00,
     &   1.67023120E+00, 1.76034923E+00, 1.84545487E+00, 1.92323200E+00,
     &   1.99164610E+00, 2.05492993E+00, 2.11198770E+00, 2.16196343E+00,
     &   2.20944427E+00, 2.24833687E+00, 2.27870814E+00, 2.30449418E+00,
     &   2.32434170E+00, 2.33663659E+00, 2.33972980E+00, 2.33450347E+00,
     &   2.31941061E+00, 2.29274202E+00, 2.25264692E+00, 2.20757061E+00,
     &   2.13775637E+00, 2.04894385E+00, 1.93932738E+00, 1.78316339E+00,
     &   1.59510504E+00, 1.34086548E+00, 1.00771170E+00, 1.38066160E-01,
     &   7.29549386E-04, 4.56157264E-01, 1.00769533E+00, 1.34085077E+00,
     &   1.59509205E+00, 1.78315203E+00, 1.93931767E+00, 2.04893556E+00,
     &   2.13774946E+00, 2.20756503E+00, 2.25264241E+00, 2.29273872E+00,
     &   2.31940845E+00, 2.33450239E+00, 2.33972971E+00, 2.33663741E+00,
     &   2.32434352E+00, 2.30449689E+00, 2.27871167E+00, 2.24834115E+00,
     &   2.20944931E+00, 2.16196924E+00, 2.11199419E+00, 2.05493709E+00,
     &   1.99165390E+00, 1.92324041E+00, 1.84546388E+00, 1.76035882E+00,
     &   1.67024133E+00, 1.57301637E+00, 1.46417192E+00, 1.34284217E+00,
     &   1.20608673E+00, 1.04940262E+00, 8.59242429E-01, 6.09189406E-01,
     &   5.21627994E-01, 5.05205070E-01, 4.84834080E-01, 4.28570776E-01,
     &   3.77864350E-01, 2.58536198E-01, 1.72660531E-01, 1.29574113E-01,
     &   1.03689663E-01, 8.64243658E-02, 7.40862845E-02, 6.48308776E-02,
     &   5.76320259E-02, 5.18717928E-02, 4.71586461E-02, 4.32313024E-02,
     &   3.99077255E-02, 3.70589366E-02, 3.45902500E-02, 3.24299238E-02/
  
         DATA ((DGS  (I,J), I=1801,1900), J=2,2) /
     &   2.16297031E-02, 1.62313412E-02, 1.29938580E-02, 1.08369175E-02,
     &   9.29739974E-03, 8.14374715E-03, 7.24737650E-03, 5.94577737E-03,
     &   5.04719733E-03, 4.69502812E-03, 4.39033526E-03, 3.30155984E-03,
     &   2.51758991E-03, 2.06041359E-03, 1.76523289E-03, 1.56178860E-03,
     &   1.41521720E-03, 1.30616288E-03, 1.22309281E-03, 1.15870856E-03,
     &   1.10811908E-03, 1.06797134E-03, 1.03595946E-03, 1.01043597E-03,
     &   9.90214487E-04, 9.74447060E-04, 9.62552433E-04, 9.54188179E-04,
     &   9.49281682E-04, 9.48194237E-04, 9.49105931E-04, 9.47662748E-04,
     &   9.60663327E-04, 1.20321080E-03, 1.42877690E-03, 1.61265144E-03,
     &   1.76789552E-03, 1.89961707E-03, 2.01285813E-03, 2.11007899E-03,
     &   2.19346792E-03, 2.26398322E-03, 2.32323948E-03, 2.37225976E-03,
     &   2.41125338E-03, 2.44070941E-03, 2.46133393E-03, 2.47353087E-03,
     &   2.47720513E-03, 2.47277320E-03, 2.46032340E-03, 2.43946863E-03,
     &   2.41071921E-03, 2.37464509E-03, 2.32887616E-03, 2.27639827E-03,
     &   2.21397546E-03, 2.14273497E-03, 2.06100410E-03, 1.96853224E-03,
     &   1.86531944E-03, 1.75171492E-03, 1.61975073E-03, 1.47233715E-03,
     &   1.30844232E-03, 1.11790089E-03, 9.04623273E-04, 7.89805805E-04,
     &   1.24958199E-03, 1.48289216E-03, 1.66084006E-03, 1.80219826E-03,
     &   1.92263736E-03, 2.02582307E-03, 2.11131208E-03, 2.18623587E-03,
     &   2.25094526E-03, 2.30595041E-03, 2.35186086E-03, 2.39007128E-03,
     &   2.42019612E-03, 2.44424516E-03, 2.46079079E-03, 2.47112475E-03,
     &   2.47496606E-03, 2.47227449E-03, 2.46325594E-03, 2.44757087E-03,
     &   2.42503243E-03, 2.39586612E-03, 2.35955497E-03, 2.31538289E-03,
     &   2.25017657E-03, 2.20247507E-03, 2.13141252E-03, 2.04971198E-03,
     &   1.95583547E-03, 1.84745829E-03, 1.72200005E-03, 1.57393986E-03/
  
         DATA ((DGS  (I,J), I=1901,2000), J=2,2) /
     &   1.39723321E-03, 1.17441520E-03, 7.91374466E-04, 8.64686932E-04,
     &   8.21168895E-04, 8.05576252E-04, 8.00214725E-04, 7.98502869E-04,
     &   7.97530993E-04, 7.98192567E-04, 7.99993143E-04, 8.02065420E-04,
     &   8.04092094E-04, 8.05715026E-04, 8.06836087E-04, 8.07623082E-04,
     &   8.07829115E-04, 8.07568898E-04, 8.06678526E-04, 8.04944289E-04,
     &   8.02569846E-04, 7.99411271E-04, 7.95508702E-04, 7.90858544E-04,
     &   7.85384059E-04, 7.78927897E-04, 7.71602743E-04, 7.63346333E-04,
     &   7.54116944E-04, 7.43895708E-04, 7.32628645E-04, 7.20234835E-04,
     &   7.06485275E-04, 6.91387582E-04, 6.75278087E-04, 6.55373680E-04,
     &   6.33708031E-04, 6.10563450E-04, 5.78152504E-04, 5.91000621E-04,
     &   6.05810248E-04, 6.19558276E-04, 6.33458049E-04, 6.46356853E-04,
     &   6.57646028E-04, 6.68406155E-04, 6.78642025E-04, 6.88130822E-04,
     &   6.96977633E-04, 7.05251991E-04, 7.12942958E-04, 7.20021663E-04,
     &   7.26544321E-04, 7.32532503E-04, 7.37908872E-04, 7.42773144E-04,
     &   7.47140224E-04, 7.50970902E-04, 7.54255731E-04, 7.57011157E-04,
     &   7.59331924E-04, 7.61209442E-04, 7.62696530E-04, 7.63750694E-04,
     &   7.64311797E-04, 7.64484305E-04, 7.64437965E-04, 7.64286375E-04,
     &   7.64090027E-04, 7.64057168E-04, 7.64572197E-04, 7.62044828E-04,
     &   7.57009882E-04, 7.51395787E-04, 7.32623641E-04, 7.32040899E-04,
     &   7.31354791E-04, 7.30547422E-04, 7.29586757E-04, 7.28463321E-04,
     &   7.27198539E-04, 7.25776362E-04, 7.24177841E-04, 7.20125852E-04,
     &   7.15945533E-04, 7.11830533E-04, 7.07731143E-04, 7.03581396E-04,
     &   6.99310868E-04, 6.94930132E-04, 6.90469748E-04, 6.85828729E-04,
     &   6.80994109E-04, 6.75978840E-04, 6.70688283E-04, 6.65253860E-04,
     &   6.59591723E-04, 6.53636664E-04, 6.47249832E-04, 6.40842123E-04/
  
         DATA ((DGS  (I,J), I=2001,2100), J=2,2) /
     &   6.34006547E-04, 6.27055561E-04, 6.18936235E-04, 6.10492820E-04,
     &   6.01887327E-04, 5.93142738E-04, 5.83047412E-04, 5.72129380E-04,
     &   5.60725913E-04, 5.44924999E-04, 5.48333588E-04, 5.54825872E-04,
     &   5.62162032E-04, 5.69386279E-04, 5.75655309E-04, 5.82114389E-04,
     &   5.88633026E-04, 5.95002470E-04, 6.00285738E-04, 6.05593112E-04,
     &   6.10521547E-04, 6.15347605E-04, 6.19955941E-04, 6.24312087E-04,
     &   6.28479143E-04, 6.32505197E-04, 6.36276951E-04, 6.39819424E-04,
     &   6.43243959E-04, 6.46445513E-04, 6.49531043E-04, 6.52476573E-04,
     &   6.55257522E-04, 6.57919763E-04, 6.60520407E-04, 6.63089217E-04,
     &   6.65544842E-04, 6.66807050E-04, 6.68112937E-04, 6.69556054E-04,
     &   6.71279043E-04, 6.73445176E-04, 6.75424318E-04, 6.74426117E-04,
     &   6.73247480E-04, 6.65866153E-04, 6.58312868E-04, 6.59625544E-04,
     &   6.70771189E-04, 6.92252958E-04, 7.20079984E-04, 7.55272166E-04,
     &   7.93966218E-04, 8.38827030E-04, 8.88371502E-04, 9.40268359E-04,
     &   1.00033979E-03, 1.06427663E-03, 1.08925234E-03, 1.11422806E-03,
     &   1.06440320E-03, 1.01965425E-03, 9.82337671E-04, 9.47908124E-04,
     &   9.18562979E-04, 8.90996597E-04, 8.67141603E-04, 8.44440851E-04,
     &   8.23710745E-04, 8.05484094E-04, 7.87896027E-04, 7.72312666E-04,
     &   7.57171512E-04, 7.43672495E-04, 7.30482452E-04, 7.18662251E-04,
     &   7.07058071E-04, 6.96613781E-04, 6.86319206E-04, 6.77019086E-04,
     &   6.68209928E-04, 6.59483688E-04, 6.51563580E-04, 6.43695743E-04,
     &   6.36535649E-04, 6.29404914E-04, 6.22900179E-04, 6.16407550E-04,
     &   6.10472265E-04, 6.04536036E-04, 5.99098930E-04, 5.93650988E-04,
     &   5.88429899E-04, 5.83635430E-04, 5.78819535E-04, 5.74390382E-04,
     &   5.69934913E-04, 5.65831470E-04, 5.61698064E-04, 5.57886283E-04/
  
         DATA ((DGS  (I,J), I=2101,2174), J=2,2) /
     &   5.54041861E-04, 5.50492298E-04, 5.46908174E-04, 5.43595226E-04,
     &   5.40389634E-04, 5.37147687E-04, 5.34146418E-04, 5.31108130E-04,
     &   5.28292722E-04, 5.25439502E-04, 5.22793648E-04, 5.20110352E-04,
     &   5.17619572E-04, 5.15091489E-04, 5.12742919E-04, 5.10357330E-04,
     &   5.08040102E-04, 5.05884999E-04, 5.03693536E-04, 5.01587113E-04,
     &   4.99511313E-04, 4.97578146E-04, 4.95609782E-04, 4.93777439E-04,
     &   4.91906829E-04, 4.90164287E-04, 4.88387892E-04, 4.86730571E-04,
     &   4.85040105E-04, 4.83462090E-04, 4.81920857E-04, 4.80347561E-04,
     &   4.78877789E-04, 4.77376688E-04, 4.75973673E-04, 4.74540061E-04,
     &   4.73199494E-04, 4.71835050E-04, 4.70553039E-04, 4.69241882E-04,
     &   4.68014729E-04, 4.66759141E-04, 4.54689140E-04, 4.44958248E-04,
     &   4.43574443E-04, 4.05772654E-04, 3.96942362E-04, 3.83150930E-04,
     &   3.78580597E-04, 3.78361825E-04, 3.74133828E-04, 3.74122835E-04,
     &   3.71844232E-04, 3.71771456E-04, 3.71724044E-04, 3.70810773E-04,
     &   3.70262791E-04, 3.70169278E-04, 3.70019360E-04, 3.69803597E-04,
     &   3.69655808E-04, 3.69564803E-04, 3.68748306E-04, 3.68710061E-04,
     &   3.68495690E-04, 3.68456511E-04, 3.68346284E-04, 3.68213890E-04,
     &   3.68178532E-04, 3.68130394E-04, 3.67975363E-04, 3.67782151E-04,
     &   3.67702522E-04, 3.67210705E-04/

      save
      RETURN
      END

c      ++++ dalhadshigh15 ++++  

c update may 2016 
       subroutine dalhad_timelike1h16
       IMPLICIT NONE
       INTEGER NC,I,J
       PARAMETER(NC=200)
       REAL EMA(NC),DAM(NC,2),DGM(NC,2)
       COMMON /DATM16/EMA,DAM,DGM
c 
c******************************************************************************
c*                                                                            *
c*    --- intRdat1 ---                              Tue May 17 00:31:47 2016  *
c*                                                                            *
c***************************# FJ@HU Berlin #***********************************
c 
c  "Delta alpha"               Update version (29/09/2015) 
c 
c ISCAN:  2, Ismooth: 0, IGS: 0, DSET: WAVE, iplot: 0
c IKE,Emin,Emax,NP,N,EPSREL:  1, 6.0000E-01, 1.4000E+00, 100, 10, 1.00E-02, 
c ALS(sta)(sys):  0.1184( 0.0007)( 0.0000), MZINP:   91.1876, MTOP:    173.20
c NF,IOR,ICHK: 5,4,2,ISU2: 0,NFL: 5,INAG: 1,inotail:  0, sin2W:   0.23153
c CHPTCUT,EC,ECUT: 0.3180,     5.20,    11.50
c 
  
       DATA  (EMA  (I), I=   1, 100) /
     &   1.3005E+01, 1.3500E+01, 1.4000E+01, 1.4500E+01, 1.5000E+01,
     &   1.6000E+01, 1.7000E+01, 1.8000E+01, 1.9000E+01, 2.0000E+01,
     &   2.1000E+01, 2.2000E+01, 2.3000E+01, 2.4000E+01, 2.5000E+01,
     &   2.6000E+01, 2.7000E+01, 2.8000E+01, 2.9000E+01, 3.0000E+01,
     &   3.1000E+01, 3.2000E+01, 3.3000E+01, 3.4000E+01, 3.5000E+01,
     &   3.6000E+01, 3.7000E+01, 3.8000E+01, 3.9000E+01, 4.0000E+01,
     &   4.1000E+01, 4.2000E+01, 4.3000E+01, 4.4000E+01, 4.5000E+01,
     &   4.6000E+01, 4.7000E+01, 4.8000E+01, 4.9000E+01, 5.0000E+01,
     &   5.1000E+01, 5.2000E+01, 5.3000E+01, 5.4000E+01, 5.5000E+01,
     &   5.6000E+01, 5.7000E+01, 5.8000E+01, 5.9000E+01, 6.0000E+01,
     &   6.1000E+01, 6.2000E+01, 6.3000E+01, 6.4000E+01, 6.5000E+01,
     &   6.6000E+01, 6.7000E+01, 6.8000E+01, 6.9000E+01, 7.0000E+01,
     &   7.1000E+01, 7.2000E+01, 7.3000E+01, 7.4000E+01, 7.5000E+01,
     &   7.6000E+01, 7.7000E+01, 7.8000E+01, 7.9000E+01, 8.0000E+01,
     &   8.1000E+01, 8.2000E+01, 8.3000E+01, 8.4000E+01, 8.5000E+01,
     &   8.6000E+01, 8.7000E+01, 8.8000E+01, 8.9000E+01, 9.0000E+01,
     &   9.1000E+01, 9.2000E+01, 9.3000E+01, 9.4000E+01, 9.5000E+01,
     &   9.6000E+01, 9.7000E+01, 9.8000E+01, 9.9000E+01, 1.0000E+02,
     &   1.0900E+02, 1.1800E+02, 1.2700E+02, 1.3600E+02, 1.4500E+02,
     &   1.5400E+02, 1.6300E+02, 1.7200E+02, 1.8100E+02, 1.9000E+02/
  
       DATA  (EMA  (I), I= 101, 200) /
     &   1.9900E+02, 2.0800E+02, 2.1700E+02, 2.2600E+02, 2.3500E+02,
     &   2.4400E+02, 2.5300E+02, 2.6200E+02, 2.7100E+02, 2.8000E+02,
     &   2.8900E+02, 2.9800E+02, 3.0700E+02, 3.1600E+02, 3.2500E+02,
     &   3.3400E+02, 3.4300E+02, 3.5200E+02, 3.6100E+02, 3.7000E+02,
     &   3.7900E+02, 3.8800E+02, 3.9700E+02, 4.0600E+02, 4.1500E+02,
     &   4.2400E+02, 4.3300E+02, 4.4200E+02, 4.5100E+02, 4.6000E+02,
     &   4.6900E+02, 4.7800E+02, 4.8700E+02, 4.9600E+02, 5.0500E+02,
     &   5.1400E+02, 5.2300E+02, 5.3200E+02, 5.4100E+02, 5.5000E+02,
     &   5.5900E+02, 5.6800E+02, 5.7700E+02, 5.8600E+02, 5.9500E+02,
     &   6.0400E+02, 6.1300E+02, 6.2200E+02, 6.3100E+02, 6.4000E+02,
     &   6.4900E+02, 6.5800E+02, 6.6700E+02, 6.7600E+02, 6.8500E+02,
     &   6.9400E+02, 7.0300E+02, 7.1200E+02, 7.2100E+02, 7.3000E+02,
     &   7.3900E+02, 7.4800E+02, 7.5700E+02, 7.6600E+02, 7.7500E+02,
     &   7.8400E+02, 7.9300E+02, 8.0200E+02, 8.1100E+02, 8.2000E+02,
     &   8.2900E+02, 8.3800E+02, 8.4700E+02, 8.5600E+02, 8.6500E+02,
     &   8.7400E+02, 8.8300E+02, 8.9200E+02, 9.0100E+02, 9.1000E+02,
     &   9.1900E+02, 9.2800E+02, 9.3700E+02, 9.4600E+02, 9.5500E+02,
     &   9.6400E+02, 9.7300E+02, 9.8200E+02, 9.9100E+02, 1.0000E+03,
     &   2.0000E+03, 3.0000E+03, 4.0000E+03, 5.0000E+03, 6.0000E+03,
     &   8.0000E+03, 1.0000E+04, 1.2000E+04, 1.4000E+04, 2.0000E+04/
  
  
         DATA ((DAM  (I,J), I=   1, 100), J=1,1) /
     &   1.5797E-02, 1.6081E-02, 1.6282E-02, 1.6507E-02, 1.6724E-02,
     &   1.7131E-02, 1.7508E-02, 1.7862E-02, 1.8194E-02, 1.8508E-02,
     &   1.8804E-02, 1.9196E-02, 1.9355E-02, 1.9612E-02, 1.9858E-02,
     &   2.0095E-02, 2.0321E-02, 2.0539E-02, 2.0749E-02, 2.0952E-02,
     &   2.1148E-02, 2.1338E-02, 2.1521E-02, 2.1702E-02, 2.1905E-02,
     &   2.2040E-02, 2.2203E-02, 2.2362E-02, 2.2517E-02, 2.2668E-02,
     &   2.2814E-02, 2.2957E-02, 2.3105E-02, 2.3233E-02, 2.3367E-02,
     &   2.3560E-02, 2.3625E-02, 2.3750E-02, 2.3872E-02, 2.3992E-02,
     &   2.4109E-02, 2.4224E-02, 2.4337E-02, 2.4472E-02, 2.4557E-02,
     &   2.4663E-02, 2.4768E-02, 2.4996E-02, 2.4972E-02, 2.5072E-02,
     &   2.5170E-02, 2.5266E-02, 2.5361E-02, 2.5454E-02, 2.5546E-02,
     &   2.5636E-02, 2.5725E-02, 2.5813E-02, 2.5899E-02, 2.5984E-02,
     &   2.6068E-02, 2.6151E-02, 2.6232E-02, 2.6313E-02, 2.6392E-02,
     &   2.6471E-02, 2.6548E-02, 2.6624E-02, 2.6700E-02, 2.6774E-02,
     &   2.6847E-02, 2.6920E-02, 2.6992E-02, 2.7062E-02, 2.7132E-02,
     &   2.7202E-02, 2.7270E-02, 2.7337E-02, 2.7404E-02, 2.7470E-02,
     &   2.7536E-02, 2.7600E-02, 2.7664E-02, 2.7727E-02, 2.7790E-02,
     &   2.7852E-02, 2.7913E-02, 2.7974E-02, 2.8034E-02, 2.8093E-02,
     &   2.8602E-02, 2.9071E-02, 2.9506E-02, 2.9911E-02, 3.0290E-02,
     &   3.0647E-02, 3.0983E-02, 3.1302E-02, 3.1605E-02, 3.1893E-02/
  
         DATA ((DAM  (I,J), I= 101, 200), J=1,1) /
     &   3.2168E-02, 3.2431E-02, 3.2683E-02, 3.2926E-02, 3.3159E-02,
     &   3.3384E-02, 3.3600E-02, 3.3810E-02, 3.4013E-02, 3.4209E-02,
     &   3.4400E-02, 3.4585E-02, 3.4764E-02, 3.4939E-02, 3.5110E-02,
     &   3.5276E-02, 3.5437E-02, 3.5594E-02, 3.5750E-02, 3.5902E-02,
     &   3.6051E-02, 3.6196E-02, 3.6338E-02, 3.6478E-02, 3.6615E-02,
     &   3.6749E-02, 3.6881E-02, 3.7011E-02, 3.7139E-02, 3.7264E-02,
     &   3.7388E-02, 3.7510E-02, 3.7630E-02, 3.7748E-02, 3.7859E-02,
     &   3.7962E-02, 3.8064E-02, 3.8164E-02, 3.8262E-02, 3.8359E-02,
     &   3.8454E-02, 3.8547E-02, 3.8639E-02, 3.8730E-02, 3.8819E-02,
     &   3.8907E-02, 3.8993E-02, 3.9079E-02, 3.9163E-02, 3.9246E-02,
     &   3.9327E-02, 3.9408E-02, 3.9487E-02, 3.9566E-02, 3.9643E-02,
     &   3.9719E-02, 3.9795E-02, 3.9869E-02, 3.9943E-02, 4.0015E-02,
     &   4.0087E-02, 4.0158E-02, 4.0228E-02, 4.0297E-02, 4.0365E-02,
     &   4.0433E-02, 4.0499E-02, 4.0565E-02, 4.0631E-02, 4.0695E-02,
     &   4.0759E-02, 4.0822E-02, 4.0885E-02, 4.0946E-02, 4.1007E-02,
     &   4.1068E-02, 4.1128E-02, 4.1187E-02, 4.1246E-02, 4.1304E-02,
     &   4.1361E-02, 4.1418E-02, 4.1475E-02, 4.1531E-02, 4.1586E-02,
     &   4.1641E-02, 4.1695E-02, 4.1749E-02, 4.1802E-02, 4.1855E-02,
     &   4.5902E-02, 4.8265E-02, 4.9942E-02, 5.1242E-02, 5.2302E-02,
     &   5.3976E-02, 5.5275E-02, 5.6333E-02, 5.7230E-02, 5.9302E-02/
  
  
         DATA ((DAM  (I,J), I=   1, 100), J=2,2) /
     &   1.9452E-04, 1.8562E-04, 1.8003E-04, 1.7622E-04, 1.7346E-04,
     &   1.6969E-04, 1.6722E-04, 1.6547E-04, 1.6416E-04, 1.6314E-04,
     &   1.6233E-04, 1.6167E-04, 1.6111E-04, 1.6064E-04, 1.6024E-04,
     &   1.5990E-04, 1.5960E-04, 1.5934E-04, 1.5911E-04, 1.5890E-04,
     &   1.5872E-04, 1.5856E-04, 1.5841E-04, 1.5828E-04, 1.5816E-04,
     &   1.5805E-04, 1.5795E-04, 1.5786E-04, 1.5778E-04, 1.5770E-04,
     &   1.5763E-04, 1.5756E-04, 1.5750E-04, 1.5744E-04, 1.5739E-04,
     &   1.5735E-04, 1.5730E-04, 1.5725E-04, 1.5721E-04, 1.5718E-04,
     &   1.5714E-04, 1.5711E-04, 1.5708E-04, 1.5705E-04, 1.5702E-04,
     &   1.5699E-04, 1.5697E-04, 1.5696E-04, 1.5692E-04, 1.5690E-04,
     &   1.5688E-04, 1.5686E-04, 1.5685E-04, 1.5683E-04, 1.5681E-04,
     &   1.5680E-04, 1.5678E-04, 1.5677E-04, 1.5675E-04, 1.5674E-04,
     &   1.5673E-04, 1.5672E-04, 1.5671E-04, 1.5669E-04, 1.5668E-04,
     &   1.5667E-04, 1.5667E-04, 1.5666E-04, 1.5665E-04, 1.5664E-04,
     &   1.5663E-04, 1.5662E-04, 1.5662E-04, 1.5661E-04, 1.5660E-04,
     &   1.5659E-04, 1.5659E-04, 1.5658E-04, 1.5658E-04, 1.5657E-04,
     &   1.5656E-04, 1.5656E-04, 1.5655E-04, 1.5655E-04, 1.5654E-04,
     &   1.5654E-04, 1.5653E-04, 1.5653E-04, 1.5653E-04, 1.5652E-04,
     &   1.5649E-04, 1.5647E-04, 1.5645E-04, 1.5643E-04, 1.5642E-04,
     &   1.5641E-04, 1.5640E-04, 1.5639E-04, 1.5639E-04, 1.5638E-04/
  
         DATA ((DAM  (I,J), I= 101, 200), J=2,2) /
     &   1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5637E-04, 1.5637E-04,
     &   1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04,
     &   1.5636E-04, 1.5636E-04, 1.5636E-04, 1.5636E-04, 1.5636E-04,
     &   1.5636E-04, 1.5636E-04, 1.5636E-04, 1.5636E-04, 1.5636E-04,
     &   1.5636E-04, 1.5636E-04, 1.5636E-04, 1.5636E-04, 1.5636E-04,
     &   1.5636E-04, 1.5636E-04, 1.5636E-04, 1.5636E-04, 1.5636E-04,
     &   1.5636E-04, 1.5636E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04,
     &   1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04,
     &   1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04,
     &   1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04,
     &   1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04,
     &   1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04,
     &   1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04,
     &   1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5638E-04,
     &   1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5638E-04,
     &   1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5638E-04,
     &   1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5638E-04,
     &   1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5638E-04,
     &   1.5640E-04, 1.5641E-04, 1.5642E-04, 1.5642E-04, 1.5642E-04,
     &   1.5643E-04, 1.5644E-04, 1.5644E-04, 1.5645E-04, 1.5645E-04/
  

c      ++++ deghadshigh15 ++++  

c update may 2016 
c 
c******************************************************************************
c*                                                                            *
c*    --- intRdat1 ---                              Thu May 19 11:11:37 2016  *
c*                                                                            *
c***************************# FJ@HU Berlin #***********************************
c 
c  "Delta alpha"               Update version (29/09/2015) 
c 
c ISCAN:  2, Ismooth: 0, IGS: 0, DSET: WAVE, iplot: 0
c IKE,Emin,Emax,NP,N,EPSREL:  1, 6.0000E-01, 1.4000E+00, 100, 10, 1.00E-02, 
c ALS(sta)(sys):  0.1184( 0.0007)( 0.0000), MZINP:   91.1876, MTOP:    173.20
c NF,IOR,ICHK: 5,4,2,ISU2: 2,NFL: 5,INAG: 1,inotail:  0, sin2W:   0.23153
c CHPTCUT,EC,ECUT: 0.3180,     5.20,    11.50
c 
  
       DATA  (EMA  (I), I=   1, 100) /
     &   1.3005E+01, 1.3500E+01, 1.4000E+01, 1.4500E+01, 1.5000E+01,
     &   1.6000E+01, 1.7000E+01, 1.8000E+01, 1.9000E+01, 2.0000E+01,
     &   2.1000E+01, 2.2000E+01, 2.3000E+01, 2.4000E+01, 2.5000E+01,
     &   2.6000E+01, 2.7000E+01, 2.8000E+01, 2.9000E+01, 3.0000E+01,
     &   3.1000E+01, 3.2000E+01, 3.3000E+01, 3.4000E+01, 3.5000E+01,
     &   3.6000E+01, 3.7000E+01, 3.8000E+01, 3.9000E+01, 4.0000E+01,
     &   4.1000E+01, 4.2000E+01, 4.3000E+01, 4.4000E+01, 4.5000E+01,
     &   4.6000E+01, 4.7000E+01, 4.8000E+01, 4.9000E+01, 5.0000E+01,
     &   5.1000E+01, 5.2000E+01, 5.3000E+01, 5.4000E+01, 5.5000E+01,
     &   5.6000E+01, 5.7000E+01, 5.8000E+01, 5.9000E+01, 6.0000E+01,
     &   6.1000E+01, 6.2000E+01, 6.3000E+01, 6.4000E+01, 6.5000E+01,
     &   6.6000E+01, 6.7000E+01, 6.8000E+01, 6.9000E+01, 7.0000E+01,
     &   7.1000E+01, 7.2000E+01, 7.3000E+01, 7.4000E+01, 7.5000E+01,
     &   7.6000E+01, 7.7000E+01, 7.8000E+01, 7.9000E+01, 8.0000E+01,
     &   8.1000E+01, 8.2000E+01, 8.3000E+01, 8.4000E+01, 8.5000E+01,
     &   8.6000E+01, 8.7000E+01, 8.8000E+01, 8.9000E+01, 9.0000E+01,
     &   9.1000E+01, 9.2000E+01, 9.3000E+01, 9.4000E+01, 9.5000E+01,
     &   9.6000E+01, 9.7000E+01, 9.8000E+01, 9.9000E+01, 1.0000E+02,
     &   1.0900E+02, 1.1800E+02, 1.2700E+02, 1.3600E+02, 1.4500E+02,
     &   1.5400E+02, 1.6300E+02, 1.7200E+02, 1.8100E+02, 1.9000E+02/
  
       DATA  (EMA  (I), I= 101, 200) /
     &   1.9900E+02, 2.0800E+02, 2.1700E+02, 2.2600E+02, 2.3500E+02,
     &   2.4400E+02, 2.5300E+02, 2.6200E+02, 2.7100E+02, 2.8000E+02,
     &   2.8900E+02, 2.9800E+02, 3.0700E+02, 3.1600E+02, 3.2500E+02,
     &   3.3400E+02, 3.4300E+02, 3.5200E+02, 3.6100E+02, 3.7000E+02,
     &   3.7900E+02, 3.8800E+02, 3.9700E+02, 4.0600E+02, 4.1500E+02,
     &   4.2400E+02, 4.3300E+02, 4.4200E+02, 4.5100E+02, 4.6000E+02,
     &   4.6900E+02, 4.7800E+02, 4.8700E+02, 4.9600E+02, 5.0500E+02,
     &   5.1400E+02, 5.2300E+02, 5.3200E+02, 5.4100E+02, 5.5000E+02,
     &   5.5900E+02, 5.6800E+02, 5.7700E+02, 5.8600E+02, 5.9500E+02,
     &   6.0400E+02, 6.1300E+02, 6.2200E+02, 6.3100E+02, 6.4000E+02,
     &   6.4900E+02, 6.5800E+02, 6.6700E+02, 6.7600E+02, 6.8500E+02,
     &   6.9400E+02, 7.0300E+02, 7.1200E+02, 7.2100E+02, 7.3000E+02,
     &   7.3900E+02, 7.4800E+02, 7.5700E+02, 7.6600E+02, 7.7500E+02,
     &   7.8400E+02, 7.9300E+02, 8.0200E+02, 8.1100E+02, 8.2000E+02,
     &   8.2900E+02, 8.3800E+02, 8.4700E+02, 8.5600E+02, 8.6500E+02,
     &   8.7400E+02, 8.8300E+02, 8.9200E+02, 9.0100E+02, 9.1000E+02,
     &   9.1900E+02, 9.2800E+02, 9.3700E+02, 9.4600E+02, 9.5500E+02,
     &   9.6400E+02, 9.7300E+02, 9.8200E+02, 9.9100E+02, 1.0000E+03,
     &   2.0000E+03, 3.0000E+03, 4.0000E+03, 5.0000E+03, 6.0000E+03,
     &   8.0000E+03, 1.0000E+04, 1.2000E+04, 1.4000E+04, 2.0000E+04/
  
  
         DATA ((DGM  (I,J), I=   1, 100), J=1,1) /
     &   3.1586E-02, 3.2199E-02, 3.2635E-02, 3.3119E-02, 3.3581E-02,
     &   3.4444E-02, 3.5241E-02, 3.5984E-02, 3.6680E-02, 3.7337E-02,
     &   3.7955E-02, 3.8771E-02, 3.9103E-02, 3.9637E-02, 4.0148E-02,
     &   4.0640E-02, 4.1107E-02, 4.1559E-02, 4.1995E-02, 4.2415E-02,
     &   4.2821E-02, 4.3214E-02, 4.3594E-02, 4.3968E-02, 4.4387E-02,
     &   4.4666E-02, 4.5003E-02, 4.5331E-02, 4.5652E-02, 4.5964E-02,
     &   4.6265E-02, 4.6561E-02, 4.6867E-02, 4.7131E-02, 4.7407E-02,
     &   4.7808E-02, 4.7940E-02, 4.8198E-02, 4.8450E-02, 4.8697E-02,
     &   4.8940E-02, 4.9177E-02, 4.9410E-02, 4.9689E-02, 4.9863E-02,
     &   5.0084E-02, 5.0300E-02, 5.0772E-02, 5.0721E-02, 5.0927E-02,
     &   5.1129E-02, 5.1327E-02, 5.1523E-02, 5.1715E-02, 5.1904E-02,
     &   5.2091E-02, 5.2274E-02, 5.2455E-02, 5.2633E-02, 5.2809E-02,
     &   5.2982E-02, 5.3152E-02, 5.3321E-02, 5.3487E-02, 5.3650E-02,
     &   5.3812E-02, 5.3971E-02, 5.4129E-02, 5.4284E-02, 5.4438E-02,
     &   5.4589E-02, 5.4739E-02, 5.4886E-02, 5.5032E-02, 5.5177E-02,
     &   5.5319E-02, 5.5460E-02, 5.5600E-02, 5.5737E-02, 5.5874E-02,
     &   5.6008E-02, 5.6141E-02, 5.6273E-02, 5.6404E-02, 5.6533E-02,
     &   5.6660E-02, 5.6786E-02, 5.6911E-02, 5.7035E-02, 5.7158E-02,
     &   5.8208E-02, 5.9175E-02, 6.0071E-02, 6.0906E-02, 6.1688E-02,
     &   6.2423E-02, 6.3117E-02, 6.3774E-02, 6.4398E-02, 6.4992E-02/
  
         DATA ((DGM  (I,J), I= 101, 200), J=1,1) /
     &   6.5559E-02, 6.6102E-02, 6.6622E-02, 6.7121E-02, 6.7602E-02,
     &   6.8065E-02, 6.8512E-02, 6.8944E-02, 6.9362E-02, 6.9767E-02,
     &   7.0160E-02, 7.0541E-02, 7.0912E-02, 7.1272E-02, 7.1623E-02,
     &   7.1965E-02, 7.2299E-02, 7.2622E-02, 7.2943E-02, 7.3257E-02,
     &   7.3563E-02, 7.3863E-02, 7.4156E-02, 7.4444E-02, 7.4726E-02,
     &   7.5003E-02, 7.5276E-02, 7.5543E-02, 7.5806E-02, 7.6065E-02,
     &   7.6320E-02, 7.6571E-02, 7.6819E-02, 7.7063E-02, 7.7291E-02,
     &   7.7504E-02, 7.7714E-02, 7.7919E-02, 7.8122E-02, 7.8321E-02,
     &   7.8517E-02, 7.8710E-02, 7.8899E-02, 7.9086E-02, 7.9270E-02,
     &   7.9451E-02, 7.9630E-02, 7.9805E-02, 7.9979E-02, 8.0150E-02,
     &   8.0318E-02, 8.0484E-02, 8.0648E-02, 8.0810E-02, 8.0969E-02,
     &   8.1127E-02, 8.1282E-02, 8.1435E-02, 8.1587E-02, 8.1736E-02,
     &   8.1884E-02, 8.2030E-02, 8.2174E-02, 8.2317E-02, 8.2458E-02,
     &   8.2597E-02, 8.2734E-02, 8.2870E-02, 8.3005E-02, 8.3138E-02,
     &   8.3269E-02, 8.3400E-02, 8.3528E-02, 8.3656E-02, 8.3782E-02,
     &   8.3906E-02, 8.4030E-02, 8.4152E-02, 8.4273E-02, 8.4393E-02,
     &   8.4511E-02, 8.4629E-02, 8.4745E-02, 8.4860E-02, 8.4974E-02,
     &   8.5087E-02, 8.5199E-02, 8.5310E-02, 8.5420E-02, 8.5529E-02,
     &   9.3871E-02, 9.8743E-02, 1.0220E-01, 1.0488E-01, 1.0706E-01,
     &   1.1052E-01, 1.1319E-01, 1.1538E-01, 1.1722E-01, 1.2150E-01/
  
  
         DATA ((DGM  (I,J), I=   1, 100), J=2,2) /
     &   5.0159E-04, 4.6676E-04, 4.4496E-04, 4.3027E-04, 4.1976E-04,
     &   4.0577E-04, 3.9694E-04, 3.9088E-04, 3.8648E-04, 3.8315E-04,
     &   3.8054E-04, 3.7849E-04, 3.7675E-04, 3.7533E-04, 3.7413E-04,
     &   3.7312E-04, 3.7224E-04, 3.7148E-04, 3.7081E-04, 3.7022E-04,
     &   3.6970E-04, 3.6924E-04, 3.6883E-04, 3.6846E-04, 3.6813E-04,
     &   3.6781E-04, 3.6753E-04, 3.6728E-04, 3.6705E-04, 3.6684E-04,
     &   3.6664E-04, 3.6646E-04, 3.6630E-04, 3.6614E-04, 3.6599E-04,
     &   3.6589E-04, 3.6574E-04, 3.6562E-04, 3.6551E-04, 3.6541E-04,
     &   3.6531E-04, 3.6523E-04, 3.6514E-04, 3.6508E-04, 3.6499E-04,
     &   3.6492E-04, 3.6485E-04, 3.6484E-04, 3.6473E-04, 3.6467E-04,
     &   3.6462E-04, 3.6457E-04, 3.6452E-04, 3.6447E-04, 3.6443E-04,
     &   3.6439E-04, 3.6435E-04, 3.6431E-04, 3.6427E-04, 3.6424E-04,
     &   3.6421E-04, 3.6418E-04, 3.6415E-04, 3.6412E-04, 3.6409E-04,
     &   3.6406E-04, 3.6404E-04, 3.6401E-04, 3.6399E-04, 3.6397E-04,
     &   3.6394E-04, 3.6392E-04, 3.6390E-04, 3.6388E-04, 3.6387E-04,
     &   3.6385E-04, 3.6383E-04, 3.6381E-04, 3.6380E-04, 3.6378E-04,
     &   3.6377E-04, 3.6375E-04, 3.6374E-04, 3.6373E-04, 3.6371E-04,
     &   3.6370E-04, 3.6369E-04, 3.6368E-04, 3.6366E-04, 3.6365E-04,
     &   3.6357E-04, 3.6350E-04, 3.6345E-04, 3.6341E-04, 3.6338E-04,
     &   3.6335E-04, 3.6333E-04, 3.6331E-04, 3.6329E-04, 3.6328E-04/
  
         DATA ((DGM  (I,J), I= 101, 200), J=2,2) /
     &   3.6327E-04, 3.6326E-04, 3.6325E-04, 3.6325E-04, 3.6324E-04,
     &   3.6324E-04, 3.6323E-04, 3.6323E-04, 3.6322E-04, 3.6322E-04,
     &   3.6322E-04, 3.6322E-04, 3.6322E-04, 3.6321E-04, 3.6321E-04,
     &   3.6321E-04, 3.6321E-04, 3.6321E-04, 3.6321E-04, 3.6321E-04,
     &   3.6321E-04, 3.6321E-04, 3.6321E-04, 3.6321E-04, 3.6321E-04,
     &   3.6321E-04, 3.6321E-04, 3.6321E-04, 3.6321E-04, 3.6321E-04,
     &   3.6321E-04, 3.6321E-04, 3.6321E-04, 3.6321E-04, 3.6321E-04,
     &   3.6321E-04, 3.6321E-04, 3.6321E-04, 3.6321E-04, 3.6321E-04,
     &   3.6321E-04, 3.6321E-04, 3.6321E-04, 3.6322E-04, 3.6322E-04,
     &   3.6322E-04, 3.6322E-04, 3.6322E-04, 3.6322E-04, 3.6322E-04,
     &   3.6322E-04, 3.6322E-04, 3.6322E-04, 3.6322E-04, 3.6322E-04,
     &   3.6322E-04, 3.6322E-04, 3.6322E-04, 3.6322E-04, 3.6322E-04,
     &   3.6322E-04, 3.6322E-04, 3.6322E-04, 3.6322E-04, 3.6323E-04,
     &   3.6323E-04, 3.6323E-04, 3.6323E-04, 3.6323E-04, 3.6323E-04,
     &   3.6323E-04, 3.6323E-04, 3.6323E-04, 3.6323E-04, 3.6323E-04,
     &   3.6323E-04, 3.6323E-04, 3.6323E-04, 3.6323E-04, 3.6323E-04,
     &   3.6323E-04, 3.6323E-04, 3.6323E-04, 3.6323E-04, 3.6323E-04,
     &   3.6324E-04, 3.6324E-04, 3.6324E-04, 3.6324E-04, 3.6324E-04,
     &   3.6327E-04, 3.6329E-04, 3.6331E-04, 3.6332E-04, 3.6332E-04,
     &   3.6334E-04, 3.6335E-04, 3.6336E-04, 3.6337E-04, 3.6338E-04/
  
      save
      RETURN
      END

c      ++++ dalhadt15 ++++  

c update may 2016 
       subroutine dalhad_spacelike16
       IMPLICIT NONE
       INTEGER NA,NC,I,J
       PARAMETER(NA=979,NC=200)
       REAL ETA(NA),DAT(NA,2),DGT(NA,2)
       REAL XXX(NC),YGG(NC,2),Y3G(NC,2)
       COMMON /DATT16/ETA,DAT,DGT
       COMMON /DATH16/XXX,YGG,Y3G
c 
c******************************************************************************
c*                                                                            *
c*    --- intRdat1 ---                              Mon May 16 15:25:38 2016  *
c*                                                                            *
c***************************# FJ@HU Berlin #***********************************
c 
c  "Delta alpha"               Update version (29/09/2015) 
c 
c ISCAN: -1, Ismooth: 0, IGS: 0, DSET: WAVE, iplot: 0
c IKE,Emin,Emax,NP,N,EPSREL:  1, 6.0000E-01, 1.4000E+00, 100, 10, 1.00E-02, 
c ALS(sta)(sys):  0.1184( 0.0007)( 0.0000), MZINP:   91.1876, MTOP:    173.20
c NF,IOR,ICHK: 5,4,2,ISU2: 0,NFL: 5,INAG: 1,inotail:  0, sin2W:   0.23153
c CHPTCUT,EC,ECUT: 0.3180,     5.20,    11.50
c 
  
       DATA  (ETA  (I), I=   1, 100) /
     &  -1.0000E+03,-9.9000E+02,-9.8000E+02,-9.7000E+02,-9.6000E+02,
     &  -9.5000E+02,-9.4000E+02,-9.3000E+02,-9.2000E+02,-9.1000E+02,
     &  -9.0000E+02,-8.9000E+02,-8.8000E+02,-8.7000E+02,-8.6000E+02,
     &  -8.5000E+02,-8.4000E+02,-8.3000E+02,-8.2000E+02,-8.1000E+02,
     &  -8.0000E+02,-7.9000E+02,-7.8000E+02,-7.7000E+02,-7.6000E+02,
     &  -7.5000E+02,-7.4000E+02,-7.3000E+02,-7.2000E+02,-7.1000E+02,
     &  -7.0000E+02,-6.9000E+02,-6.8000E+02,-6.7000E+02,-6.6000E+02,
     &  -6.5000E+02,-6.4000E+02,-6.3000E+02,-6.2000E+02,-6.1000E+02,
     &  -6.0000E+02,-5.9000E+02,-5.8000E+02,-5.7000E+02,-5.6000E+02,
     &  -5.5000E+02,-5.4000E+02,-5.3000E+02,-5.2000E+02,-5.1000E+02,
     &  -5.0000E+02,-4.9000E+02,-4.8000E+02,-4.7000E+02,-4.6000E+02,
     &  -4.5000E+02,-4.4000E+02,-4.3000E+02,-4.2000E+02,-4.1000E+02,
     &  -4.0000E+02,-3.9000E+02,-3.8000E+02,-3.7000E+02,-3.6000E+02,
     &  -3.5000E+02,-3.4000E+02,-3.3000E+02,-3.2000E+02,-3.1000E+02,
     &  -3.0000E+02,-2.9000E+02,-2.8000E+02,-2.7000E+02,-2.6000E+02,
     &  -2.5000E+02,-2.4000E+02,-2.3000E+02,-2.2000E+02,-2.1000E+02,
     &  -2.0000E+02,-1.9000E+02,-1.8000E+02,-1.7000E+02,-1.6000E+02,
     &  -1.5000E+02,-1.4000E+02,-1.3000E+02,-1.2000E+02,-1.1000E+02,
     &  -1.0000E+02,-9.9000E+01,-9.8000E+01,-9.7000E+01,-9.6000E+01,
     &  -9.5000E+01,-9.4000E+01,-9.3000E+01,-9.2000E+01,-9.1000E+01/
  
       DATA  (ETA  (I), I= 101, 200) /
     &  -9.0000E+01,-8.9000E+01,-8.8000E+01,-8.7000E+01,-8.6000E+01,
     &  -8.5000E+01,-8.4000E+01,-8.3000E+01,-8.2000E+01,-8.1000E+01,
     &  -8.0000E+01,-7.9000E+01,-7.8000E+01,-7.7000E+01,-7.6000E+01,
     &  -7.5000E+01,-7.4000E+01,-7.3000E+01,-7.2000E+01,-7.1000E+01,
     &  -7.0000E+01,-6.9000E+01,-6.8000E+01,-6.7000E+01,-6.6000E+01,
     &  -6.5000E+01,-6.4000E+01,-6.3000E+01,-6.2000E+01,-6.1000E+01,
     &  -6.0000E+01,-5.9000E+01,-5.8000E+01,-5.7000E+01,-5.6000E+01,
     &  -5.5000E+01,-5.4000E+01,-5.3000E+01,-5.2000E+01,-5.1000E+01,
     &  -5.0000E+01,-4.9000E+01,-4.8000E+01,-4.7000E+01,-4.6000E+01,
     &  -4.5000E+01,-4.4000E+01,-4.3000E+01,-4.2000E+01,-4.1000E+01,
     &  -4.0000E+01,-3.9000E+01,-3.8000E+01,-3.7000E+01,-3.6000E+01,
     &  -3.5000E+01,-3.4000E+01,-3.3000E+01,-3.2000E+01,-3.1000E+01,
     &  -3.0000E+01,-2.9000E+01,-2.8000E+01,-2.7000E+01,-2.6000E+01,
     &  -2.5000E+01,-2.4000E+01,-2.3000E+01,-2.2000E+01,-2.1000E+01,
     &  -2.0000E+01,-1.9000E+01,-1.8000E+01,-1.7000E+01,-1.6000E+01,
     &  -1.5000E+01,-1.4000E+01,-1.3000E+01,-1.2000E+01,-1.1000E+01,
     &  -1.0000E+01,-9.9100E+00,-9.8200E+00,-9.7300E+00,-9.6400E+00,
     &  -9.5500E+00,-9.4600E+00,-9.3700E+00,-9.2800E+00,-9.1900E+00,
     &  -9.1000E+00,-9.0100E+00,-8.9200E+00,-8.8300E+00,-8.7400E+00,
     &  -8.6500E+00,-8.5600E+00,-8.4700E+00,-8.3800E+00,-8.2900E+00/
  
       DATA  (ETA  (I), I= 201, 300) /
     &  -8.2000E+00,-8.1100E+00,-8.0200E+00,-7.9300E+00,-7.8400E+00,
     &  -7.7500E+00,-7.6600E+00,-7.5700E+00,-7.4800E+00,-7.3900E+00,
     &  -7.3000E+00,-7.2100E+00,-7.1200E+00,-7.0300E+00,-6.9400E+00,
     &  -6.8500E+00,-6.7600E+00,-6.6700E+00,-6.5800E+00,-6.4900E+00,
     &  -6.4000E+00,-6.3100E+00,-6.2200E+00,-6.1300E+00,-6.0400E+00,
     &  -5.9500E+00,-5.8600E+00,-5.7700E+00,-5.6800E+00,-5.5900E+00,
     &  -5.5000E+00,-5.4100E+00,-5.3200E+00,-5.2300E+00,-5.1400E+00,
     &  -5.0500E+00,-4.9600E+00,-4.8700E+00,-4.7800E+00,-4.6900E+00,
     &  -4.6000E+00,-4.5100E+00,-4.4200E+00,-4.3300E+00,-4.2400E+00,
     &  -4.1500E+00,-4.0600E+00,-3.9700E+00,-3.8800E+00,-3.7900E+00,
     &  -3.7000E+00,-3.6100E+00,-3.5200E+00,-3.4300E+00,-3.3400E+00,
     &  -3.2500E+00,-3.1600E+00,-3.0700E+00,-2.9800E+00,-2.8900E+00,
     &  -2.8000E+00,-2.7100E+00,-2.6200E+00,-2.5300E+00,-2.4400E+00,
     &  -2.3500E+00,-2.2600E+00,-2.1700E+00,-2.0800E+00,-1.9900E+00,
     &  -1.9000E+00,-1.8100E+00,-1.7200E+00,-1.6300E+00,-1.5400E+00,
     &  -1.4500E+00,-1.3600E+00,-1.2700E+00,-1.1800E+00,-1.0900E+00,
     &  -1.0000E+00,-9.9100E-01,-9.8200E-01,-9.7300E-01,-9.6400E-01,
     &  -9.5500E-01,-9.4600E-01,-9.3700E-01,-9.2800E-01,-9.1900E-01,
     &  -9.1000E-01,-9.0100E-01,-8.9200E-01,-8.8300E-01,-8.7400E-01,
     &  -8.6500E-01,-8.5600E-01,-8.4700E-01,-8.3800E-01,-8.2900E-01/
  
       DATA  (ETA  (I), I= 301, 400) /
     &  -8.2000E-01,-8.1100E-01,-8.0200E-01,-7.9300E-01,-7.8400E-01,
     &  -7.7500E-01,-7.6600E-01,-7.5700E-01,-7.4800E-01,-7.3900E-01,
     &  -7.3000E-01,-7.2100E-01,-7.1200E-01,-7.0300E-01,-6.9400E-01,
     &  -6.8500E-01,-6.7600E-01,-6.6700E-01,-6.5800E-01,-6.4900E-01,
     &  -6.4000E-01,-6.3100E-01,-6.2200E-01,-6.1300E-01,-6.0400E-01,
     &  -5.9500E-01,-5.8600E-01,-5.7700E-01,-5.6800E-01,-5.5900E-01,
     &  -5.5000E-01,-5.4100E-01,-5.3200E-01,-5.2300E-01,-5.1400E-01,
     &  -5.0500E-01,-4.9600E-01,-4.8700E-01,-4.7800E-01,-4.6900E-01,
     &  -4.6000E-01,-4.5100E-01,-4.4200E-01,-4.3300E-01,-4.2400E-01,
     &  -4.1500E-01,-4.0600E-01,-3.9700E-01,-3.8800E-01,-3.7900E-01,
     &  -3.7000E-01,-3.6100E-01,-3.5200E-01,-3.4300E-01,-3.3400E-01,
     &  -3.2500E-01,-3.1600E-01,-3.0700E-01,-2.9800E-01,-2.8900E-01,
     &  -2.8000E-01,-2.7100E-01,-2.6200E-01,-2.5300E-01,-2.4400E-01,
     &  -2.3500E-01,-2.2600E-01,-2.1700E-01,-2.0800E-01,-1.9900E-01,
     &  -1.9000E-01,-1.8100E-01,-1.7200E-01,-1.6300E-01,-1.5400E-01,
     &  -1.4500E-01,-1.3600E-01,-1.2700E-01,-1.1800E-01,-1.0900E-01,
     &  -1.0000E-01,-9.9100E-02,-9.8200E-02,-9.7300E-02,-9.6400E-02,
     &  -9.5500E-02,-9.4600E-02,-9.3700E-02,-9.2800E-02,-9.1900E-02,
     &  -9.1000E-02,-9.0100E-02,-8.9200E-02,-8.8300E-02,-8.7400E-02,
     &  -8.6500E-02,-8.5600E-02,-8.4700E-02,-8.3800E-02,-8.2900E-02/
  
       DATA  (ETA  (I), I= 401, 500) /
     &  -8.2000E-02,-8.1100E-02,-8.0200E-02,-7.9300E-02,-7.8400E-02,
     &  -7.7500E-02,-7.6600E-02,-7.5700E-02,-7.4800E-02,-7.3900E-02,
     &  -7.3000E-02,-7.2100E-02,-7.1200E-02,-7.0300E-02,-6.9400E-02,
     &  -6.8500E-02,-6.7600E-02,-6.6700E-02,-6.5800E-02,-6.4900E-02,
     &  -6.4000E-02,-6.3100E-02,-6.2200E-02,-6.1300E-02,-6.0400E-02,
     &  -5.9500E-02,-5.8600E-02,-5.7700E-02,-5.6800E-02,-5.5900E-02,
     &  -5.5000E-02,-5.4100E-02,-5.3200E-02,-5.2300E-02,-5.1400E-02,
     &  -5.0500E-02,-4.9600E-02,-4.8700E-02,-4.7800E-02,-4.6900E-02,
     &  -4.6000E-02,-4.5100E-02,-4.4200E-02,-4.3300E-02,-4.2400E-02,
     &  -4.1500E-02,-4.0600E-02,-3.9700E-02,-3.8800E-02,-3.7900E-02,
     &  -3.7000E-02,-3.6100E-02,-3.5200E-02,-3.4300E-02,-3.3400E-02,
     &  -3.2500E-02,-3.1600E-02,-3.0700E-02,-2.9800E-02,-2.8900E-02,
     &  -2.8000E-02,-2.7100E-02,-2.6200E-02,-2.5300E-02,-2.4400E-02,
     &  -2.3500E-02,-2.2600E-02,-2.1700E-02,-2.0800E-02,-1.9900E-02,
     &  -1.9000E-02,-1.8100E-02,-1.7200E-02,-1.6300E-02,-1.5400E-02,
     &  -1.4500E-02,-1.3600E-02,-1.2700E-02,-1.1800E-02,-1.0900E-02,
     &  -1.0000E-02,-9.9100E-03,-9.8200E-03,-9.7300E-03,-9.6400E-03,
     &  -9.5500E-03,-9.4600E-03,-9.3700E-03,-9.2800E-03,-9.1900E-03,
     &  -9.1000E-03,-9.0100E-03,-8.9200E-03,-8.8300E-03,-8.7400E-03,
     &  -8.6500E-03,-8.5600E-03,-8.4700E-03,-8.3800E-03,-8.2900E-03/
  
       DATA  (ETA  (I), I= 501, 600) /
     &  -8.2000E-03,-8.1100E-03,-8.0200E-03,-7.9300E-03,-7.8400E-03,
     &  -7.7500E-03,-7.6600E-03,-7.5700E-03,-7.4800E-03,-7.3900E-03,
     &  -7.3000E-03,-7.2100E-03,-7.1200E-03,-7.0300E-03,-6.9400E-03,
     &  -6.8500E-03,-6.7600E-03,-6.6700E-03,-6.5800E-03,-6.4900E-03,
     &  -6.4000E-03,-6.3100E-03,-6.2200E-03,-6.1300E-03,-6.0400E-03,
     &  -5.9500E-03,-5.8600E-03,-5.7700E-03,-5.6800E-03,-5.5900E-03,
     &  -5.5000E-03,-5.4100E-03,-5.3200E-03,-5.2300E-03,-5.1400E-03,
     &  -5.0500E-03,-4.9600E-03,-4.8700E-03,-4.7800E-03,-4.6900E-03,
     &  -4.6000E-03,-4.5100E-03,-4.4200E-03,-4.3300E-03,-4.2400E-03,
     &  -4.1500E-03,-4.0600E-03,-3.9700E-03,-3.8800E-03,-3.7900E-03,
     &  -3.7000E-03,-3.6100E-03,-3.5200E-03,-3.4300E-03,-3.3400E-03,
     &  -3.2500E-03,-3.1600E-03,-3.0700E-03,-2.9800E-03,-2.8900E-03,
     &  -2.8000E-03,-2.7100E-03,-2.6200E-03,-2.5300E-03,-2.4400E-03,
     &  -2.3500E-03,-2.2600E-03,-2.1700E-03,-2.0800E-03,-1.9900E-03,
     &  -1.9000E-03,-1.8100E-03,-1.7200E-03,-1.6300E-03,-1.5400E-03,
     &  -1.4500E-03,-1.3600E-03,-1.2700E-03,-1.1800E-03,-1.0900E-03,
     &  -1.0000E-03,-9.9100E-04,-9.8200E-04,-9.7300E-04,-9.6400E-04,
     &  -9.5500E-04,-9.4600E-04,-9.3700E-04,-9.2800E-04,-9.1900E-04,
     &  -9.1000E-04,-9.0100E-04,-8.9200E-04,-8.8300E-04,-8.7400E-04,
     &  -8.6500E-04,-8.5600E-04,-8.4700E-04,-8.3800E-04,-8.2900E-04/
  
       DATA  (ETA  (I), I= 601, 700) /
     &  -8.2000E-04,-8.1100E-04,-8.0200E-04,-7.9300E-04,-7.8400E-04,
     &  -7.7500E-04,-7.6600E-04,-7.5700E-04,-7.4800E-04,-7.3900E-04,
     &  -7.3000E-04,-7.2100E-04,-7.1200E-04,-7.0300E-04,-6.9400E-04,
     &  -6.8500E-04,-6.7600E-04,-6.6700E-04,-6.5800E-04,-6.4900E-04,
     &  -6.4000E-04,-6.3100E-04,-6.2200E-04,-6.1300E-04,-6.0400E-04,
     &  -5.9500E-04,-5.8600E-04,-5.7700E-04,-5.6800E-04,-5.5900E-04,
     &  -5.5000E-04,-5.4100E-04,-5.3200E-04,-5.2300E-04,-5.1400E-04,
     &  -5.0500E-04,-4.9600E-04,-4.8700E-04,-4.7800E-04,-4.6900E-04,
     &  -4.6000E-04,-4.5100E-04,-4.4200E-04,-4.3300E-04,-4.2400E-04,
     &  -4.1500E-04,-4.0600E-04,-3.9700E-04,-3.8800E-04,-3.7900E-04,
     &  -3.7000E-04,-3.6100E-04,-3.5200E-04,-3.4300E-04,-3.3400E-04,
     &  -3.2500E-04,-3.1600E-04,-3.0700E-04,-2.9800E-04,-2.8900E-04,
     &  -2.8000E-04,-2.7100E-04,-2.6200E-04,-2.5300E-04,-2.4400E-04,
     &  -2.3500E-04,-2.2600E-04,-2.1700E-04,-2.0800E-04,-1.9900E-04,
     &  -1.9000E-04,-1.8100E-04,-1.7200E-04,-1.6300E-04,-1.5400E-04,
     &  -1.4500E-04,-1.3600E-04,-1.2700E-04,-1.1800E-04,-1.0900E-04,
     &  -1.0000E-04,-9.9100E-05,-9.8200E-05,-9.7300E-05,-9.6400E-05,
     &  -9.5500E-05,-9.4600E-05,-9.3700E-05,-9.2800E-05,-9.1900E-05,
     &  -9.1000E-05,-9.0100E-05,-8.9200E-05,-8.8300E-05,-8.7400E-05,
     &  -8.6500E-05,-8.5600E-05,-8.4700E-05,-8.3800E-05,-8.2900E-05/
  
       DATA  (ETA  (I), I= 701, 800) /
     &  -8.2000E-05,-8.1100E-05,-8.0200E-05,-7.9300E-05,-7.8400E-05,
     &  -7.7500E-05,-7.6600E-05,-7.5700E-05,-7.4800E-05,-7.3900E-05,
     &  -7.3000E-05,-7.2100E-05,-7.1200E-05,-7.0300E-05,-6.9400E-05,
     &  -6.8500E-05,-6.7600E-05,-6.6700E-05,-6.5800E-05,-6.4900E-05,
     &  -6.4000E-05,-6.3100E-05,-6.2200E-05,-6.1300E-05,-6.0400E-05,
     &  -5.9500E-05,-5.8600E-05,-5.7700E-05,-5.6800E-05,-5.5900E-05,
     &  -5.5000E-05,-5.4100E-05,-5.3200E-05,-5.2300E-05,-5.1400E-05,
     &  -5.0500E-05,-4.9600E-05,-4.8700E-05,-4.7800E-05,-4.6900E-05,
     &  -4.6000E-05,-4.5100E-05,-4.4200E-05,-4.3300E-05,-4.2400E-05,
     &  -4.1500E-05,-4.0600E-05,-3.9700E-05,-3.8800E-05,-3.7900E-05,
     &  -3.7000E-05,-3.6100E-05,-3.5200E-05,-3.4300E-05,-3.3400E-05,
     &  -3.2500E-05,-3.1600E-05,-3.0700E-05,-2.9800E-05,-2.8900E-05,
     &  -2.8000E-05,-2.7100E-05,-2.6200E-05,-2.5300E-05,-2.4400E-05,
     &  -2.3500E-05,-2.2600E-05,-2.1700E-05,-2.0800E-05,-1.9900E-05,
     &  -1.9000E-05,-1.8100E-05,-1.7200E-05,-1.6300E-05,-1.5400E-05,
     &  -1.4500E-05,-1.3600E-05,-1.2700E-05,-1.1800E-05,-1.0900E-05,
     &  -1.0000E-05,-9.9100E-06,-9.8200E-06,-9.7300E-06,-9.6400E-06,
     &  -9.5500E-06,-9.4600E-06,-9.3700E-06,-9.2800E-06,-9.1900E-06,
     &  -9.1000E-06,-9.0100E-06,-8.9200E-06,-8.8300E-06,-8.7400E-06,
     &  -8.6500E-06,-8.5600E-06,-8.4700E-06,-8.3800E-06,-8.2900E-06/
  
       DATA  (ETA  (I), I= 801, 900) /
     &  -8.2000E-06,-8.1100E-06,-8.0200E-06,-7.9300E-06,-7.8400E-06,
     &  -7.7500E-06,-7.6600E-06,-7.5700E-06,-7.4800E-06,-7.3900E-06,
     &  -7.3000E-06,-7.2100E-06,-7.1200E-06,-7.0300E-06,-6.9400E-06,
     &  -6.8500E-06,-6.7600E-06,-6.6700E-06,-6.5800E-06,-6.4900E-06,
     &  -6.4000E-06,-6.3100E-06,-6.2200E-06,-6.1300E-06,-6.0400E-06,
     &  -5.9500E-06,-5.8600E-06,-5.7700E-06,-5.6800E-06,-5.5900E-06,
     &  -5.5000E-06,-5.4100E-06,-5.3200E-06,-5.2300E-06,-5.1400E-06,
     &  -5.0500E-06,-4.9600E-06,-4.8700E-06,-4.7800E-06,-4.6900E-06,
     &  -4.6000E-06,-4.5100E-06,-4.4200E-06,-4.3300E-06,-4.2400E-06,
     &  -4.1500E-06,-4.0600E-06,-3.9700E-06,-3.8800E-06,-3.7900E-06,
     &  -3.7000E-06,-3.6100E-06,-3.5200E-06,-3.4300E-06,-3.3400E-06,
     &  -3.2500E-06,-3.1600E-06,-3.0700E-06,-2.9800E-06,-2.8900E-06,
     &  -2.8000E-06,-2.7100E-06,-2.6200E-06,-2.5300E-06,-2.4400E-06,
     &  -2.3500E-06,-2.2600E-06,-2.1700E-06,-2.0800E-06,-1.9900E-06,
     &  -1.9000E-06,-1.8100E-06,-1.7200E-06,-1.6300E-06,-1.5400E-06,
     &  -1.4500E-06,-1.3600E-06,-1.2700E-06,-1.1800E-06,-1.0900E-06,
     &  -1.0000E-06, 0.0000E+00, 1.0000E-06, 3.0010E-03, 6.0010E-03,
     &   9.0010E-03, 1.2000E-02, 1.5000E-02, 1.8000E-02, 2.1000E-02,
     &   2.4000E-02, 2.7000E-02, 3.0000E-02, 3.3000E-02, 3.6000E-02,
     &   3.9000E-02, 4.2000E-02, 4.5000E-02, 4.8000E-02, 5.1000E-02/
  
       DATA  (ETA  (I), I= 901, 979) /
     &   5.4000E-02, 5.7000E-02, 6.0000E-02, 6.3000E-02, 6.6000E-02,
     &   6.9000E-02, 7.2000E-02, 7.5000E-02, 7.8000E-02, 8.1000E-02,
     &   8.4000E-02, 8.7000E-02, 9.0000E-02, 9.3000E-02, 9.6000E-02,
     &   9.9000E-02, 1.0200E-01, 1.0500E-01, 1.0800E-01, 1.1100E-01,
     &   1.1400E-01, 1.1700E-01, 1.2000E-01, 1.2300E-01, 1.2600E-01,
     &   1.2900E-01, 1.3200E-01, 1.3500E-01, 1.3800E-01, 1.4100E-01,
     &   1.4400E-01, 1.4700E-01, 1.5000E-01, 1.5300E-01, 1.5600E-01,
     &   1.5900E-01, 1.6200E-01, 1.6500E-01, 1.6800E-01, 1.7100E-01,
     &   1.7400E-01, 1.7700E-01, 1.8000E-01, 1.8300E-01, 1.8600E-01,
     &   1.8900E-01, 1.9200E-01, 1.9500E-01, 1.9800E-01, 2.0100E-01,
     &   2.0400E-01, 2.0700E-01, 2.1000E-01, 2.1300E-01, 2.1600E-01,
     &   2.1900E-01, 2.2200E-01, 2.2500E-01, 2.2800E-01, 2.3100E-01,
     &   2.3400E-01, 2.3700E-01, 2.4000E-01, 2.4300E-01, 2.4600E-01,
     &   2.4900E-01, 2.5200E-01, 2.5500E-01, 2.5800E-01, 2.6100E-01,
     &   2.6400E-01, 2.6700E-01, 2.7000E-01, 2.7100E-01, 2.7300E-01,
     &   0.27370000D0, 0.27599999D0, 0.27630001D0, 0.27700001D0/
  
         DATA ((DAT  (I,J), I=   1, 100), J=1,1) /
     &   4.1003E-02, 4.0952E-02, 4.0900E-02, 4.0847E-02, 4.0794E-02,
     &   4.0740E-02, 4.0685E-02, 4.0630E-02, 4.0574E-02, 4.0518E-02,
     &   4.0461E-02, 4.0403E-02, 4.0344E-02, 4.0285E-02, 4.0225E-02,
     &   4.0164E-02, 4.0102E-02, 4.0040E-02, 3.9977E-02, 3.9912E-02,
     &   3.9847E-02, 3.9781E-02, 3.9714E-02, 3.9646E-02, 3.9577E-02,
     &   3.9507E-02, 3.9436E-02, 3.9364E-02, 3.9291E-02, 3.9217E-02,
     &   3.9141E-02, 3.9065E-02, 3.8987E-02, 3.8907E-02, 3.8827E-02,
     &   3.8745E-02, 3.8661E-02, 3.8577E-02, 3.8490E-02, 3.8402E-02,
     &   3.8313E-02, 3.8222E-02, 3.8129E-02, 3.8034E-02, 3.7937E-02,
     &   3.7839E-02, 3.7738E-02, 3.7636E-02, 3.7531E-02, 3.7424E-02,
     &   3.7315E-02, 3.7203E-02, 3.7089E-02, 3.6972E-02, 3.6852E-02,
     &   3.6730E-02, 3.6605E-02, 3.6476E-02, 3.6344E-02, 3.6209E-02,
     &   3.6070E-02, 3.5927E-02, 3.5781E-02, 3.5630E-02, 3.5475E-02,
     &   3.5315E-02, 3.5150E-02, 3.4980E-02, 3.4805E-02, 3.4623E-02,
     &   3.4436E-02, 3.4241E-02, 3.4040E-02, 3.3831E-02, 3.3613E-02,
     &   3.3387E-02, 3.3151E-02, 3.2905E-02, 3.2647E-02, 3.2378E-02,
     &   3.2094E-02, 3.1796E-02, 3.1481E-02, 3.1147E-02, 3.0793E-02,
     &   3.0416E-02, 3.0012E-02, 2.9578E-02, 2.9108E-02, 2.8597E-02,
     &   2.8036E-02, 2.7977E-02, 2.7918E-02, 2.7857E-02, 2.7796E-02,
     &   2.7735E-02, 2.7672E-02, 2.7609E-02, 2.7546E-02, 2.7481E-02/
  
         DATA ((DAT  (I,J), I= 101, 200), J=1,1) /
     &   2.7416E-02, 2.7350E-02, 2.7284E-02, 2.7217E-02, 2.7148E-02,
     &   2.7079E-02, 2.7010E-02, 2.6939E-02, 2.6868E-02, 2.6795E-02,
     &   2.6722E-02, 2.6648E-02, 2.6573E-02, 2.6497E-02, 2.6420E-02,
     &   2.6342E-02, 2.6262E-02, 2.6182E-02, 2.6101E-02, 2.6018E-02,
     &   2.5934E-02, 2.5850E-02, 2.5763E-02, 2.5676E-02, 2.5587E-02,
     &   2.5497E-02, 2.5405E-02, 2.5312E-02, 2.5218E-02, 2.5122E-02,
     &   2.5024E-02, 2.4925E-02, 2.4824E-02, 2.4721E-02, 2.4617E-02,
     &   2.4510E-02, 2.4402E-02, 2.4291E-02, 2.4179E-02, 2.4064E-02,
     &   2.3947E-02, 2.3827E-02, 2.3705E-02, 2.3581E-02, 2.3454E-02,
     &   2.3324E-02, 2.3191E-02, 2.3055E-02, 2.2916E-02, 2.2773E-02,
     &   2.2627E-02, 2.2477E-02, 2.2324E-02, 2.2166E-02, 2.2004E-02,
     &   2.1837E-02, 2.1666E-02, 2.1489E-02, 2.1307E-02, 2.1120E-02,
     &   2.0926E-02, 2.0725E-02, 2.0518E-02, 2.0303E-02, 2.0080E-02,
     &   1.9849E-02, 1.9608E-02, 1.9357E-02, 1.9095E-02, 1.8821E-02,
     &   1.8534E-02, 1.8232E-02, 1.7915E-02, 1.7580E-02, 1.7226E-02,
     &   1.6849E-02, 1.6447E-02, 1.6017E-02, 1.5555E-02, 1.5054E-02,
     &   1.4509E-02, 1.4458E-02, 1.4406E-02, 1.4354E-02, 1.4301E-02,
     &   1.4248E-02, 1.4194E-02, 1.4140E-02, 1.4085E-02, 1.4030E-02,
     &   1.3974E-02, 1.3918E-02, 1.3861E-02, 1.3804E-02, 1.3746E-02,
     &   1.3688E-02, 1.3629E-02, 1.3570E-02, 1.3510E-02, 1.3449E-02/
  
         DATA ((DAT  (I,J), I= 201, 300), J=1,1) /
     &   1.3388E-02, 1.3326E-02, 1.3263E-02, 1.3200E-02, 1.3137E-02,
     &   1.3072E-02, 1.3007E-02, 1.2942E-02, 1.2875E-02, 1.2808E-02,
     &   1.2740E-02, 1.2671E-02, 1.2602E-02, 1.2532E-02, 1.2461E-02,
     &   1.2389E-02, 1.2316E-02, 1.2243E-02, 1.2168E-02, 1.2093E-02,
     &   1.2017E-02, 1.1940E-02, 1.1862E-02, 1.1783E-02, 1.1702E-02,
     &   1.1621E-02, 1.1539E-02, 1.1456E-02, 1.1371E-02, 1.1286E-02,
     &   1.1199E-02, 1.1111E-02, 1.1022E-02, 1.0931E-02, 1.0839E-02,
     &   1.0746E-02, 1.0651E-02, 1.0555E-02, 1.0458E-02, 1.0359E-02,
     &   1.0258E-02, 1.0156E-02, 1.0052E-02, 9.9464E-03, 9.8390E-03,
     &   9.7296E-03, 9.6184E-03, 9.5052E-03, 9.3900E-03, 9.2726E-03,
     &   9.1530E-03, 9.0311E-03, 8.9069E-03, 8.7802E-03, 8.6507E-03,
     &   8.5187E-03, 8.3839E-03, 8.2462E-03, 8.1054E-03, 7.9614E-03,
     &   7.8140E-03, 7.6630E-03, 7.5083E-03, 7.3496E-03, 7.1867E-03,
     &   7.0193E-03, 6.8472E-03, 6.6699E-03, 6.4872E-03, 6.2986E-03,
     &   6.1035E-03, 5.9015E-03, 5.6919E-03, 5.4740E-03, 5.2469E-03,
     &   5.0096E-03, 4.7610E-03, 4.4999E-03, 4.2248E-03, 3.9342E-03,
     &   3.6265E-03, 3.5948E-03, 3.5628E-03, 3.5307E-03, 3.4983E-03,
     &   3.4658E-03, 3.4331E-03, 3.4002E-03, 3.3671E-03, 3.3338E-03,
     &   3.3003E-03, 3.2666E-03, 3.2327E-03, 3.1986E-03, 3.1643E-03,
     &   3.1298E-03, 3.0951E-03, 3.0602E-03, 3.0251E-03, 2.9898E-03/
  
         DATA ((DAT  (I,J), I= 301, 400), J=1,1) /
     &   2.9543E-03, 2.9186E-03, 2.8827E-03, 2.8465E-03, 2.8102E-03,
     &   2.7737E-03, 2.7370E-03, 2.7000E-03, 2.6629E-03, 2.6256E-03,
     &   2.5881E-03, 2.5504E-03, 2.5125E-03, 2.4744E-03, 2.4362E-03,
     &   2.3977E-03, 2.3591E-03, 2.3203E-03, 2.2813E-03, 2.2421E-03,
     &   2.2028E-03, 2.1634E-03, 2.1237E-03, 2.0840E-03, 2.0441E-03,
     &   2.0040E-03, 1.9639E-03, 1.9236E-03, 1.8832E-03, 1.8427E-03,
     &   1.8022E-03, 1.7615E-03, 1.7208E-03, 1.6800E-03, 1.6392E-03,
     &   1.5983E-03, 1.5574E-03, 1.5166E-03, 1.4757E-03, 1.4349E-03,
     &   1.3941E-03, 1.3533E-03, 1.3127E-03, 1.2721E-03, 1.2317E-03,
     &   1.1914E-03, 1.1512E-03, 1.1113E-03, 1.0715E-03, 1.0320E-03,
     &   9.9270E-04, 9.5370E-04, 9.1501E-04, 8.7666E-04, 8.3867E-04,
     &   8.0108E-04, 7.6390E-04, 7.2719E-04, 6.9096E-04, 6.5525E-04,
     &   6.2009E-04, 5.8552E-04, 5.5158E-04, 5.1829E-04, 4.8570E-04,
     &   4.5384E-04, 4.2276E-04, 3.9248E-04, 3.6304E-04, 3.3449E-04,
     &   3.0686E-04, 2.8018E-04, 2.5451E-04, 2.2987E-04, 2.0629E-04,
     &   1.8383E-04, 1.6251E-04, 1.4237E-04, 1.2345E-04, 1.0576E-04,
     &   8.9355E-05, 8.7785E-05, 8.6229E-05, 8.4686E-05, 8.3156E-05,
     &   8.1639E-05, 8.0135E-05, 7.8644E-05, 7.7167E-05, 7.5703E-05,
     &   7.4252E-05, 7.2815E-05, 7.1391E-05, 6.9980E-05, 6.8583E-05,
     &   6.7199E-05, 6.5829E-05, 6.4472E-05, 6.3129E-05, 6.1799E-05/
  
         DATA ((DAT  (I,J), I= 401, 500), J=1,1) /
     &   6.0483E-05, 5.9180E-05, 5.7891E-05, 5.6616E-05, 5.5354E-05,
     &   5.4106E-05, 5.2872E-05, 5.1651E-05, 5.0444E-05, 4.9251E-05,
     &   4.8072E-05, 4.6907E-05, 4.5755E-05, 4.4617E-05, 4.3494E-05,
     &   4.2384E-05, 4.1288E-05, 4.0206E-05, 3.9138E-05, 3.8084E-05,
     &   3.7043E-05, 3.6017E-05, 3.5006E-05, 3.4008E-05, 3.3024E-05,
     &   3.2054E-05, 3.1099E-05, 3.0157E-05, 2.9230E-05, 2.8317E-05,
     &   2.7418E-05, 2.6534E-05, 2.5663E-05, 2.4807E-05, 2.3966E-05,
     &   2.3138E-05, 2.2325E-05, 2.1526E-05, 2.0742E-05, 1.9971E-05,
     &   1.9216E-05, 1.8474E-05, 1.7747E-05, 1.7035E-05, 1.6336E-05,
     &   1.5653E-05, 1.4984E-05, 1.4329E-05, 1.3688E-05, 1.3063E-05,
     &   1.2451E-05, 1.1855E-05, 1.1272E-05, 1.0705E-05, 1.0152E-05,
     &   9.6132E-06, 9.0892E-06, 8.5799E-06, 8.0851E-06, 7.6050E-06,
     &   7.1395E-06, 6.6886E-06, 6.2523E-06, 5.8307E-06, 5.4238E-06,
     &   5.0315E-06, 4.6539E-06, 4.2910E-06, 3.9427E-06, 3.6092E-06,
     &   3.2904E-06, 2.9862E-06, 2.6968E-06, 2.4221E-06, 2.1622E-06,
     &   1.9169E-06, 1.6865E-06, 1.4707E-06, 1.2697E-06, 1.0835E-06,
     &   9.1196E-07, 8.9562E-07, 8.7943E-07, 8.6338E-07, 8.4749E-07,
     &   8.3174E-07, 8.1614E-07, 8.0069E-07, 7.8538E-07, 7.7023E-07,
     &   7.5522E-07, 7.4036E-07, 7.2564E-07, 7.1108E-07, 6.9666E-07,
     &   6.8238E-07, 6.6826E-07, 6.5428E-07, 6.4046E-07, 6.2677E-07/
  
         DATA ((DAT  (I,J), I= 501, 600), J=1,1) /
     &   6.1324E-07, 5.9986E-07, 5.8662E-07, 5.7353E-07, 5.6058E-07,
     &   5.4779E-07, 5.3514E-07, 5.2264E-07, 5.1029E-07, 4.9809E-07,
     &   4.8603E-07, 4.7412E-07, 4.6236E-07, 4.5074E-07, 4.3928E-07,
     &   4.2796E-07, 4.1679E-07, 4.0577E-07, 3.9489E-07, 3.8416E-07,
     &   3.7358E-07, 3.6315E-07, 3.5287E-07, 3.4273E-07, 3.3274E-07,
     &   3.2290E-07, 3.1321E-07, 3.0366E-07, 2.9426E-07, 2.8501E-07,
     &   2.7591E-07, 2.6695E-07, 2.5814E-07, 2.4948E-07, 2.4097E-07,
     &   2.3261E-07, 2.2439E-07, 2.1632E-07, 2.0840E-07, 2.0063E-07,
     &   1.9300E-07, 1.8552E-07, 1.7819E-07, 1.7101E-07, 1.6398E-07,
     &   1.5709E-07, 1.5035E-07, 1.4376E-07, 1.3731E-07, 1.3102E-07,
     &   1.2487E-07, 1.1887E-07, 1.1302E-07, 1.0731E-07, 1.0175E-07,
     &   9.6344E-08, 9.1082E-08, 8.5967E-08, 8.1001E-08, 7.6182E-08,
     &   7.1511E-08, 6.6988E-08, 6.2613E-08, 5.8385E-08, 5.4305E-08,
     &   5.0373E-08, 4.6588E-08, 4.2952E-08, 3.9463E-08, 3.6122E-08,
     &   3.2928E-08, 2.9883E-08, 2.6985E-08, 2.4235E-08, 2.1632E-08,
     &   1.9178E-08, 1.6871E-08, 1.4712E-08, 1.2701E-08, 1.0837E-08,
     &   9.1215E-09, 8.9580E-09, 8.7960E-09, 8.6356E-09, 8.4765E-09,
     &   8.3190E-09, 8.1629E-09, 8.0084E-09, 7.8553E-09, 7.7036E-09,
     &   7.5535E-09, 7.4048E-09, 7.2576E-09, 7.1119E-09, 6.9677E-09,
     &   6.8249E-09, 6.6836E-09, 6.5438E-09, 6.4055E-09, 6.2686E-09/
  
         DATA ((DAT  (I,J), I= 601, 700), J=1,1) /
     &   6.1333E-09, 5.9994E-09, 5.8670E-09, 5.7360E-09, 5.6066E-09,
     &   5.4786E-09, 5.3521E-09, 5.2270E-09, 5.1035E-09, 4.9814E-09,
     &   4.8608E-09, 4.7417E-09, 4.6241E-09, 4.5079E-09, 4.3932E-09,
     &   4.2800E-09, 4.1683E-09, 4.0580E-09, 3.9493E-09, 3.8420E-09,
     &   3.7362E-09, 3.6318E-09, 3.5290E-09, 3.4276E-09, 3.3277E-09,
     &   3.2292E-09, 3.1323E-09, 3.0368E-09, 2.9428E-09, 2.8503E-09,
     &   2.7592E-09, 2.6697E-09, 2.5816E-09, 2.4950E-09, 2.4099E-09,
     &   2.3262E-09, 2.2440E-09, 2.1633E-09, 2.0841E-09, 2.0064E-09,
     &   1.9301E-09, 1.8553E-09, 1.7820E-09, 1.7102E-09, 1.6398E-09,
     &   1.5709E-09, 1.5035E-09, 1.4376E-09, 1.3732E-09, 1.3102E-09,
     &   1.2487E-09, 1.1887E-09, 1.1302E-09, 1.0731E-09, 1.0176E-09,
     &   9.6346E-10, 9.1083E-10, 8.5969E-10, 8.1002E-10, 7.6184E-10,
     &   7.1512E-10, 6.6989E-10, 6.2613E-10, 5.8386E-10, 5.4306E-10,
     &   5.0373E-10, 4.6589E-10, 4.2952E-10, 3.9463E-10, 3.6122E-10,
     &   3.2929E-10, 2.9883E-10, 2.6985E-10, 2.4235E-10, 2.1633E-10,
     &   1.9178E-10, 1.6871E-10, 1.4712E-10, 1.2701E-10, 1.0837E-10,
     &   9.1215E-11, 8.9580E-11, 8.7961E-11, 8.6356E-11, 8.4766E-11,
     &   8.3190E-11, 8.1630E-11, 8.0084E-11, 7.8553E-11, 7.7036E-11,
     &   7.5535E-11, 7.4048E-11, 7.2576E-11, 7.1119E-11, 6.9677E-11,
     &   6.8249E-11, 6.6836E-11, 6.5438E-11, 6.4055E-11, 6.2687E-11/
  
         DATA ((DAT  (I,J), I= 701, 800), J=1,1) /
     &   6.1333E-11, 5.9994E-11, 5.8670E-11, 5.7360E-11, 5.6066E-11,
     &   5.4786E-11, 5.3521E-11, 5.2271E-11, 5.1035E-11, 4.9814E-11,
     &   4.8608E-11, 4.7417E-11, 4.6241E-11, 4.5079E-11, 4.3932E-11,
     &   4.2800E-11, 4.1683E-11, 4.0580E-11, 3.9493E-11, 3.8420E-11,
     &   3.7362E-11, 3.6318E-11, 3.5290E-11, 3.4276E-11, 3.3277E-11,
     &   3.2292E-11, 3.1323E-11, 3.0368E-11, 2.9428E-11, 2.8503E-11,
     &   2.7592E-11, 2.6697E-11, 2.5816E-11, 2.4950E-11, 2.4099E-11,
     &   2.3262E-11, 2.2440E-11, 2.1633E-11, 2.0841E-11, 2.0064E-11,
     &   1.9301E-11, 1.8553E-11, 1.7820E-11, 1.7102E-11, 1.6398E-11,
     &   1.5709E-11, 1.5035E-11, 1.4376E-11, 1.3732E-11, 1.3102E-11,
     &   1.2487E-11, 1.1887E-11, 1.1302E-11, 1.0731E-11, 1.0176E-11,
     &   9.6346E-12, 9.1083E-12, 8.5969E-12, 8.1002E-12, 7.6184E-12,
     &   7.1512E-12, 6.6989E-12, 6.2613E-12, 5.8386E-12, 5.4306E-12,
     &   5.0373E-12, 4.6589E-12, 4.2952E-12, 3.9463E-12, 3.6122E-12,
     &   3.2929E-12, 2.9883E-12, 2.6985E-12, 2.4235E-12, 2.1633E-12,
     &   1.9178E-12, 1.6871E-12, 1.4712E-12, 1.2701E-12, 1.0837E-12,
     &   9.1215E-13, 8.9580E-13, 8.7961E-13, 8.6356E-13, 8.4766E-13,
     &   8.3190E-13, 8.1630E-13, 8.0084E-13, 7.8553E-13, 7.7036E-13,
     &   7.5535E-13, 7.4048E-13, 7.2576E-13, 7.1119E-13, 6.9677E-13,
     &   6.8249E-13, 6.6836E-13, 6.5438E-13, 6.4055E-13, 6.2687E-13/
  
         DATA ((DAT  (I,J), I= 801, 900), J=1,1) /
     &   6.1333E-13, 5.9994E-13, 5.8670E-13, 5.7360E-13, 5.6066E-13,
     &   5.4786E-13, 5.3521E-13, 5.2271E-13, 5.1035E-13, 4.9814E-13,
     &   4.8608E-13, 4.7417E-13, 4.6241E-13, 4.5079E-13, 4.3932E-13,
     &   4.2800E-13, 4.1683E-13, 4.0580E-13, 3.9493E-13, 3.8420E-13,
     &   3.7362E-13, 3.6318E-13, 3.5290E-13, 3.4276E-13, 3.3277E-13,
     &   3.2292E-13, 3.1323E-13, 3.0368E-13, 2.9428E-13, 2.8503E-13,
     &   2.7592E-13, 2.6697E-13, 2.5816E-13, 2.4950E-13, 2.4099E-13,
     &   2.3262E-13, 2.2440E-13, 2.1633E-13, 2.0841E-13, 2.0064E-13,
     &   1.9301E-13, 1.8553E-13, 1.7820E-13, 1.7102E-13, 1.6398E-13,
     &   1.5709E-13, 1.5035E-13, 1.4376E-13, 1.3732E-13, 1.3102E-13,
     &   1.2487E-13, 1.1887E-13, 1.1302E-13, 1.0731E-13, 1.0176E-13,
     &   9.6346E-14, 9.1083E-14, 8.5969E-14, 8.1002E-14, 7.6184E-14,
     &   7.1512E-14, 6.6989E-14, 6.2614E-14, 5.8386E-14, 5.4306E-14,
     &   5.0373E-14, 4.6589E-14, 4.2952E-14, 3.9463E-14, 3.6122E-14,
     &   3.2929E-14, 2.9883E-14, 2.6985E-14, 2.4235E-14, 2.1633E-14,
     &   1.9178E-14, 1.6871E-14, 1.4712E-14, 1.2701E-14, 1.0837E-14,
     &   9.1215E-15, 0.0000E+00,-9.1215E-15,-8.2150E-08,-3.2851E-07,
     &  -7.3913E-07,-1.3139E-06,-2.0533E-06,-2.9574E-06,-4.0263E-06,
     &  -5.2604E-06,-6.6598E-06,-8.2250E-06,-9.9562E-06,-1.1854E-05,
     &  -1.3918E-05,-1.6150E-05,-1.8550E-05,-2.1119E-05,-2.3856E-05/
  
         DATA ((DAT  (I,J), I= 901, 979), J=1,1) /
     &  -2.6763E-05,-2.9841E-05,-3.3090E-05,-3.6510E-05,-4.0104E-05,
     &  -4.3870E-05,-4.7812E-05,-5.1929E-05,-5.6222E-05,-6.0693E-05,
     &  -6.5342E-05,-7.0172E-05,-7.5182E-05,-8.0374E-05,-8.5751E-05,
     &  -9.1312E-05,-9.7059E-05,-1.0299E-04,-1.0912E-04,-1.1544E-04,
     &  -1.2194E-04,-1.2865E-04,-1.3555E-04,-1.4265E-04,-1.4995E-04,
     &  -1.5745E-04,-1.6515E-04,-1.7307E-04,-1.8119E-04,-1.8952E-04,
     &  -1.9807E-04,-2.0683E-04,-2.1582E-04,-2.2503E-04,-2.3446E-04,
     &  -2.4412E-04,-2.5401E-04,-2.6414E-04,-2.7451E-04,-2.8512E-04,
     &  -2.9598E-04,-3.0708E-04,-3.1845E-04,-3.3007E-04,-3.4195E-04,
     &  -3.5411E-04,-3.6654E-04,-3.7925E-04,-3.9224E-04,-4.0553E-04,
     &  -4.1911E-04,-4.3301E-04,-4.4721E-04,-4.6173E-04,-4.7659E-04,
     &  -4.9178E-04,-5.0732E-04,-5.2322E-04,-5.3949E-04,-5.5614E-04,
     &  -5.7319E-04,-5.9066E-04,-6.0855E-04,-6.2689E-04,-6.4570E-04,
     &  -6.6501E-04,-6.8484E-04,-7.0523E-04,-7.2622E-04,-7.4787E-04,
     &  -7.7025E-04,-7.9344E-04,-8.1758E-04,-8.2587E-04,-8.4288E-04,
     &  -8.48989307E-04,-8.69785415E-04,-8.72599009E-04,-8.79283436E-04/
  
         DATA ((DAT  (I,J), I=   1, 100), J=2,2) /
     &   1.5636E-04, 1.5636E-04, 1.5636E-04, 1.5636E-04, 1.5636E-04,
     &   1.5636E-04, 1.5636E-04, 1.5636E-04, 1.5636E-04, 1.5636E-04,
     &   1.5636E-04, 1.5636E-04, 1.5636E-04, 1.5636E-04, 1.5636E-04,
     &   1.5636E-04, 1.5636E-04, 1.5636E-04, 1.5636E-04, 1.5636E-04,
     &   1.5636E-04, 1.5636E-04, 1.5636E-04, 1.5635E-04, 1.5635E-04,
     &   1.5635E-04, 1.5635E-04, 1.5635E-04, 1.5635E-04, 1.5635E-04,
     &   1.5635E-04, 1.5635E-04, 1.5635E-04, 1.5635E-04, 1.5635E-04,
     &   1.5635E-04, 1.5635E-04, 1.5635E-04, 1.5635E-04, 1.5635E-04,
     &   1.5634E-04, 1.5634E-04, 1.5634E-04, 1.5634E-04, 1.5634E-04,
     &   1.5634E-04, 1.5634E-04, 1.5634E-04, 1.5634E-04, 1.5634E-04,
     &   1.5634E-04, 1.5634E-04, 1.5633E-04, 1.5633E-04, 1.5633E-04,
     &   1.5633E-04, 1.5633E-04, 1.5633E-04, 1.5633E-04, 1.5633E-04,
     &   1.5632E-04, 1.5632E-04, 1.5632E-04, 1.5632E-04, 1.5632E-04,
     &   1.5632E-04, 1.5631E-04, 1.5631E-04, 1.5631E-04, 1.5631E-04,
     &   1.5630E-04, 1.5630E-04, 1.5630E-04, 1.5629E-04, 1.5629E-04,
     &   1.5629E-04, 1.5628E-04, 1.5628E-04, 1.5627E-04, 1.5627E-04,
     &   1.5626E-04, 1.5625E-04, 1.5624E-04, 1.5623E-04, 1.5622E-04,
     &   1.5621E-04, 1.5619E-04, 1.5617E-04, 1.5615E-04, 1.5611E-04,
     &   1.5607E-04, 1.5607E-04, 1.5606E-04, 1.5606E-04, 1.5605E-04,
     &   1.5605E-04, 1.5604E-04, 1.5604E-04, 1.5603E-04, 1.5603E-04/
  
         DATA ((DAT  (I,J), I= 101, 200), J=2,2) /
     &   1.5602E-04, 1.5601E-04, 1.5601E-04, 1.5600E-04, 1.5599E-04,
     &   1.5598E-04, 1.5598E-04, 1.5597E-04, 1.5596E-04, 1.5595E-04,
     &   1.5594E-04, 1.5594E-04, 1.5593E-04, 1.5592E-04, 1.5591E-04,
     &   1.5590E-04, 1.5589E-04, 1.5587E-04, 1.5586E-04, 1.5585E-04,
     &   1.5584E-04, 1.5583E-04, 1.5581E-04, 1.5580E-04, 1.5578E-04,
     &   1.5577E-04, 1.5575E-04, 1.5573E-04, 1.5572E-04, 1.5570E-04,
     &   1.5568E-04, 1.5566E-04, 1.5564E-04, 1.5561E-04, 1.5559E-04,
     &   1.5556E-04, 1.5554E-04, 1.5551E-04, 1.5548E-04, 1.5545E-04,
     &   1.5542E-04, 1.5538E-04, 1.5535E-04, 1.5531E-04, 1.5527E-04,
     &   1.5522E-04, 1.5517E-04, 1.5512E-04, 1.5507E-04, 1.5501E-04,
     &   1.5495E-04, 1.5489E-04, 1.5482E-04, 1.5474E-04, 1.5466E-04,
     &   1.5457E-04, 1.5447E-04, 1.5437E-04, 1.5425E-04, 1.5413E-04,
     &   1.5400E-04, 1.5385E-04, 1.5369E-04, 1.5351E-04, 1.5331E-04,
     &   1.5309E-04, 1.5284E-04, 1.5257E-04, 1.5226E-04, 1.5192E-04,
     &   1.5153E-04, 1.5108E-04, 1.5058E-04, 1.4999E-04, 1.4931E-04,
     &   1.4852E-04, 1.4760E-04, 1.4649E-04, 1.4517E-04, 1.4356E-04,
     &   1.4157E-04, 1.4137E-04, 1.4117E-04, 1.4096E-04, 1.4075E-04,
     &   1.4053E-04, 1.4031E-04, 1.4008E-04, 1.3985E-04, 1.3961E-04,
     &   1.3937E-04, 1.3912E-04, 1.3887E-04, 1.3861E-04, 1.3835E-04,
     &   1.3808E-04, 1.3780E-04, 1.3752E-04, 1.3723E-04, 1.3694E-04/
  
         DATA ((DAT  (I,J), I= 201, 300), J=2,2) /
     &   1.3663E-04, 1.3632E-04, 1.3601E-04, 1.3568E-04, 1.3535E-04,
     &   1.3501E-04, 1.3467E-04, 1.3431E-04, 1.3395E-04, 1.3357E-04,
     &   1.3319E-04, 1.3280E-04, 1.3239E-04, 1.3198E-04, 1.3156E-04,
     &   1.3112E-04, 1.3068E-04, 1.3022E-04, 1.2975E-04, 1.2927E-04,
     &   1.2877E-04, 1.2827E-04, 1.2775E-04, 1.2721E-04, 1.2666E-04,
     &   1.2609E-04, 1.2551E-04, 1.2491E-04, 1.2430E-04, 1.2367E-04,
     &   1.2302E-04, 1.2235E-04, 1.2166E-04, 1.2095E-04, 1.2022E-04,
     &   1.1947E-04, 1.1869E-04, 1.1790E-04, 1.1707E-04, 1.1623E-04,
     &   1.1535E-04, 1.1445E-04, 1.1352E-04, 1.1256E-04, 1.1157E-04,
     &   1.1054E-04, 1.0949E-04, 1.0839E-04, 1.0726E-04, 1.0609E-04,
     &   1.0489E-04, 1.0363E-04, 1.0234E-04, 1.0099E-04, 9.9599E-05,
     &   9.8154E-05, 9.6654E-05, 9.5096E-05, 9.3477E-05, 9.1793E-05,
     &   9.0041E-05, 8.8217E-05, 8.6315E-05, 8.4333E-05, 8.2264E-05,
     &   8.0104E-05, 7.7847E-05, 7.5487E-05, 7.3020E-05, 7.0439E-05,
     &   6.7739E-05, 6.4914E-05, 6.1960E-05, 5.8873E-05, 5.5650E-05,
     &   5.2291E-05, 4.8798E-05, 4.5174E-05, 4.1428E-05, 3.7575E-05,
     &   3.3631E-05, 3.3233E-05, 3.2834E-05, 3.2435E-05, 3.2035E-05,
     &   3.1634E-05, 3.1233E-05, 3.0831E-05, 3.0430E-05, 3.0027E-05,
     &   2.9625E-05, 2.9222E-05, 2.8819E-05, 2.8415E-05, 2.8012E-05,
     &   2.7608E-05, 2.7204E-05, 2.6800E-05, 2.6396E-05, 2.5992E-05/
  
         DATA ((DAT  (I,J), I= 301, 400), J=2,2) /
     &   2.5589E-05, 2.5185E-05, 2.4782E-05, 2.4379E-05, 2.3976E-05,
     &   2.3573E-05, 2.3171E-05, 2.2769E-05, 2.2368E-05, 2.1968E-05,
     &   2.1568E-05, 2.1168E-05, 2.0770E-05, 2.0372E-05, 1.9975E-05,
     &   1.9579E-05, 1.9185E-05, 1.8791E-05, 1.8398E-05, 1.8007E-05,
     &   1.7617E-05, 1.7228E-05, 1.6841E-05, 1.6456E-05, 1.6072E-05,
     &   1.5690E-05, 1.5309E-05, 1.4931E-05, 1.4554E-05, 1.4179E-05,
     &   1.3807E-05, 1.3437E-05, 1.3069E-05, 1.2704E-05, 1.2342E-05,
     &   1.1982E-05, 1.1624E-05, 1.1270E-05, 1.0919E-05, 1.0570E-05,
     &   1.0225E-05, 9.8834E-06, 9.5451E-06, 9.2104E-06, 8.8793E-06,
     &   8.5521E-06, 8.2288E-06, 7.9096E-06, 7.5945E-06, 7.2838E-06,
     &   6.9776E-06, 6.6759E-06, 6.3790E-06, 6.0869E-06, 5.7998E-06,
     &   5.5180E-06, 5.2413E-06, 4.9702E-06, 4.7046E-06, 4.4448E-06,
     &   4.1908E-06, 3.9428E-06, 3.7011E-06, 3.4656E-06, 3.2366E-06,
     &   3.0143E-06, 2.7987E-06, 2.5900E-06, 2.3884E-06, 2.1940E-06,
     &   2.0070E-06, 1.8274E-06, 1.6555E-06, 1.4914E-06, 1.3352E-06,
     &   1.1871E-06, 1.0471E-06, 9.1539E-07, 7.9213E-07, 6.7742E-07,
     &   5.7135E-07, 5.6122E-07, 5.5119E-07, 5.4123E-07, 5.3137E-07,
     &   5.2160E-07, 5.1191E-07, 5.0231E-07, 4.9280E-07, 4.8338E-07,
     &   4.7404E-07, 4.6480E-07, 4.5564E-07, 4.4657E-07, 4.3759E-07,
     &   4.2870E-07, 4.1990E-07, 4.1119E-07, 4.0256E-07, 3.9403E-07/
  
         DATA ((DAT  (I,J), I= 401, 500), J=2,2) /
     &   3.8558E-07, 3.7723E-07, 3.6896E-07, 3.6079E-07, 3.5270E-07,
     &   3.4470E-07, 3.3680E-07, 3.2898E-07, 3.2125E-07, 3.1362E-07,
     &   3.0607E-07, 2.9861E-07, 2.9125E-07, 2.8397E-07, 2.7679E-07,
     &   2.6969E-07, 2.6269E-07, 2.5578E-07, 2.4895E-07, 2.4222E-07,
     &   2.3558E-07, 2.2903E-07, 2.2258E-07, 2.1621E-07, 2.0993E-07,
     &   2.0375E-07, 1.9765E-07, 1.9165E-07, 1.8574E-07, 1.7992E-07,
     &   1.7420E-07, 1.6856E-07, 1.6302E-07, 1.5757E-07, 1.5221E-07,
     &   1.4694E-07, 1.4176E-07, 1.3668E-07, 1.3169E-07, 1.2679E-07,
     &   1.2198E-07, 1.1726E-07, 1.1264E-07, 1.0811E-07, 1.0367E-07,
     &   9.9326E-08, 9.5073E-08, 9.0912E-08, 8.6844E-08, 8.2868E-08,
     &   7.8985E-08, 7.5195E-08, 7.1498E-08, 6.7894E-08, 6.4382E-08,
     &   6.0963E-08, 5.7637E-08, 5.4405E-08, 5.1265E-08, 4.8218E-08,
     &   4.5264E-08, 4.2403E-08, 3.9636E-08, 3.6962E-08, 3.4381E-08,
     &   3.1893E-08, 2.9498E-08, 2.7197E-08, 2.4989E-08, 2.2874E-08,
     &   2.0853E-08, 1.8925E-08, 1.7090E-08, 1.5349E-08, 1.3701E-08,
     &   1.2147E-08, 1.0686E-08, 9.3188E-09, 8.0450E-09, 6.8648E-09,
     &   5.7781E-09, 5.6746E-09, 5.5720E-09, 5.4703E-09, 5.3696E-09,
     &   5.2698E-09, 5.1710E-09, 5.0730E-09, 4.9761E-09, 4.8800E-09,
     &   4.7849E-09, 4.6908E-09, 4.5975E-09, 4.5052E-09, 4.4139E-09,
     &   4.3234E-09, 4.2339E-09, 4.1454E-09, 4.0578E-09, 3.9711E-09/
  
         DATA ((DAT  (I,J), I= 501, 600), J=2,2) /
     &   3.8853E-09, 3.8005E-09, 3.7166E-09, 3.6337E-09, 3.5517E-09,
     &   3.4706E-09, 3.3905E-09, 3.3113E-09, 3.2330E-09, 3.1557E-09,
     &   3.0793E-09, 3.0038E-09, 2.9293E-09, 2.8557E-09, 2.7831E-09,
     &   2.7114E-09, 2.6406E-09, 2.5708E-09, 2.5019E-09, 2.4339E-09,
     &   2.3669E-09, 2.3008E-09, 2.2356E-09, 2.1714E-09, 2.1081E-09,
     &   2.0457E-09, 1.9843E-09, 1.9238E-09, 1.8643E-09, 1.8057E-09,
     &   1.7480E-09, 1.6913E-09, 1.6355E-09, 1.5806E-09, 1.5267E-09,
     &   1.4737E-09, 1.4216E-09, 1.3705E-09, 1.3203E-09, 1.2711E-09,
     &   1.2228E-09, 1.1754E-09, 1.1289E-09, 1.0834E-09, 1.0389E-09,
     &   9.9523E-10, 9.5253E-10, 9.1077E-10, 8.6994E-10, 8.3005E-10,
     &   7.9110E-10, 7.5308E-10, 7.1600E-10, 6.7985E-10, 6.4465E-10,
     &   6.1037E-10, 5.7704E-10, 5.4464E-10, 5.1317E-10, 4.8264E-10,
     &   4.5305E-10, 4.2439E-10, 3.9667E-10, 3.6989E-10, 3.4404E-10,
     &   3.1913E-10, 2.9515E-10, 2.7211E-10, 2.5001E-10, 2.2884E-10,
     &   2.0861E-10, 1.8932E-10, 1.7096E-10, 1.5354E-10, 1.3705E-10,
     &   1.2150E-10, 1.0688E-10, 9.3205E-11, 8.0463E-11, 6.8657E-11,
     &   5.7787E-11, 5.6752E-11, 5.5726E-11, 5.4709E-11, 5.3702E-11,
     &   5.2704E-11, 5.1715E-11, 5.0736E-11, 4.9766E-11, 4.8805E-11,
     &   4.7854E-11, 4.6912E-11, 4.5979E-11, 4.5056E-11, 4.4142E-11,
     &   4.3238E-11, 4.2343E-11, 4.1457E-11, 4.0581E-11, 3.9714E-11/
  
         DATA ((DAT  (I,J), I= 601, 700), J=2,2) /
     &   3.8856E-11, 3.8008E-11, 3.7169E-11, 3.6340E-11, 3.5519E-11,
     &   3.4709E-11, 3.3907E-11, 3.3115E-11, 3.2332E-11, 3.1559E-11,
     &   3.0795E-11, 3.0040E-11, 2.9295E-11, 2.8559E-11, 2.7833E-11,
     &   2.7115E-11, 2.6407E-11, 2.5709E-11, 2.5020E-11, 2.4340E-11,
     &   2.3670E-11, 2.3009E-11, 2.2357E-11, 2.1715E-11, 2.1082E-11,
     &   2.0458E-11, 1.9844E-11, 1.9239E-11, 1.8644E-11, 1.8057E-11,
     &   1.7481E-11, 1.6913E-11, 1.6355E-11, 1.5807E-11, 1.5267E-11,
     &   1.4737E-11, 1.4217E-11, 1.3705E-11, 1.3204E-11, 1.2711E-11,
     &   1.2228E-11, 1.1754E-11, 1.1290E-11, 1.0835E-11, 1.0389E-11,
     &   9.9525E-12, 9.5255E-12, 9.1078E-12, 8.6996E-12, 8.3007E-12,
     &   7.9111E-12, 7.5309E-12, 7.1601E-12, 6.7986E-12, 6.4465E-12,
     &   6.1038E-12, 5.7704E-12, 5.4464E-12, 5.1318E-12, 4.8265E-12,
     &   4.5305E-12, 4.2440E-12, 3.9668E-12, 3.6989E-12, 3.4404E-12,
     &   3.1913E-12, 2.9516E-12, 2.7212E-12, 2.5001E-12, 2.2884E-12,
     &   2.0861E-12, 1.8932E-12, 1.7096E-12, 1.5354E-12, 1.3705E-12,
     &   1.2150E-12, 1.0688E-12, 9.3205E-13, 8.0463E-13, 6.8657E-13,
     &   5.7788E-13, 5.6752E-13, 5.5726E-13, 5.4709E-13, 5.3702E-13,
     &   5.2704E-13, 5.1715E-13, 5.0736E-13, 4.9766E-13, 4.8805E-13,
     &   4.7854E-13, 4.6912E-13, 4.5979E-13, 4.5056E-13, 4.4142E-13,
     &   4.3238E-13, 4.2343E-13, 4.1457E-13, 4.0581E-13, 3.9714E-13/
  
         DATA ((DAT  (I,J), I= 701, 800), J=2,2) /
     &   3.8856E-13, 3.8008E-13, 3.7169E-13, 3.6340E-13, 3.5519E-13,
     &   3.4709E-13, 3.3907E-13, 3.3115E-13, 3.2332E-13, 3.1559E-13,
     &   3.0795E-13, 3.0040E-13, 2.9295E-13, 2.8559E-13, 2.7833E-13,
     &   2.7115E-13, 2.6408E-13, 2.5709E-13, 2.5020E-13, 2.4340E-13,
     &   2.3670E-13, 2.3009E-13, 2.2357E-13, 2.1715E-13, 2.1082E-13,
     &   2.0458E-13, 1.9844E-13, 1.9239E-13, 1.8644E-13, 1.8057E-13,
     &   1.7481E-13, 1.6913E-13, 1.6355E-13, 1.5807E-13, 1.5267E-13,
     &   1.4737E-13, 1.4217E-13, 1.3705E-13, 1.3204E-13, 1.2711E-13,
     &   1.2228E-13, 1.1754E-13, 1.1290E-13, 1.0835E-13, 1.0389E-13,
     &   9.9525E-14, 9.5255E-14, 9.1078E-14, 8.6996E-14, 8.3007E-14,
     &   7.9111E-14, 7.5309E-14, 7.1601E-14, 6.7986E-14, 6.4465E-14,
     &   6.1038E-14, 5.7704E-14, 5.4464E-14, 5.1318E-14, 4.8265E-14,
     &   4.5305E-14, 4.2440E-14, 3.9668E-14, 3.6989E-14, 3.4404E-14,
     &   3.1913E-14, 2.9516E-14, 2.7212E-14, 2.5001E-14, 2.2884E-14,
     &   2.0861E-14, 1.8932E-14, 1.7096E-14, 1.5354E-14, 1.3705E-14,
     &   1.2150E-14, 1.0688E-14, 9.3205E-15, 8.0463E-15, 6.8657E-15,
     &   5.7788E-15, 5.6752E-15, 5.5726E-15, 5.4709E-15, 5.3702E-15,
     &   5.2704E-15, 5.1715E-15, 5.0736E-15, 4.9766E-15, 4.8805E-15,
     &   4.7854E-15, 4.6912E-15, 4.5979E-15, 4.5056E-15, 4.4142E-15,
     &   4.3238E-15, 4.2343E-15, 4.1457E-15, 4.0581E-15, 3.9714E-15/
  
         DATA ((DAT  (I,J), I= 801, 900), J=2,2) /
     &   3.8856E-15, 3.8008E-15, 3.7169E-15, 3.6340E-15, 3.5519E-15,
     &   3.4709E-15, 3.3907E-15, 3.3115E-15, 3.2332E-15, 3.1559E-15,
     &   3.0795E-15, 3.0040E-15, 2.9295E-15, 2.8559E-15, 2.7833E-15,
     &   2.7115E-15, 2.6408E-15, 2.5709E-15, 2.5020E-15, 2.4340E-15,
     &   2.3670E-15, 2.3009E-15, 2.2357E-15, 2.1715E-15, 2.1082E-15,
     &   2.0458E-15, 1.9844E-15, 1.9239E-15, 1.8644E-15, 1.8057E-15,
     &   1.7481E-15, 1.6913E-15, 1.6355E-15, 1.5807E-15, 1.5267E-15,
     &   1.4737E-15, 1.4217E-15, 1.3705E-15, 1.3204E-15, 1.2711E-15,
     &   1.2228E-15, 1.1754E-15, 1.1290E-15, 1.0835E-15, 1.0389E-15,
     &   9.9525E-16, 9.5255E-16, 9.1078E-16, 8.6996E-16, 8.3007E-16,
     &   7.9111E-16, 7.5309E-16, 7.1601E-16, 6.7986E-16, 6.4465E-16,
     &   6.1038E-16, 5.7704E-16, 5.4464E-16, 5.1318E-16, 4.8265E-16,
     &   4.5305E-16, 4.2440E-16, 3.9668E-16, 3.6989E-16, 3.4404E-16,
     &   3.1913E-16, 2.9516E-16, 2.7212E-16, 2.5001E-16, 2.2884E-16,
     &   2.0861E-16, 1.8932E-16, 1.7096E-16, 1.5354E-16, 1.3705E-16,
     &   1.2150E-16, 1.0688E-16, 9.3205E-17, 8.0463E-17, 6.8657E-17,
     &   5.7788E-17, 0.0000E+00, 5.7788E-17, 5.2044E-10, 2.0811E-09,
     &   4.6823E-09, 8.3228E-09, 1.3006E-08, 1.8730E-08, 2.5497E-08,
     &   3.3308E-08, 4.2163E-08, 5.2063E-08, 6.3011E-08, 7.5006E-08,
     &   8.8051E-08, 1.0215E-07, 1.1730E-07, 1.3350E-07, 1.5076E-07/
  
         DATA ((DAT  (I,J), I= 901, 979), J=2,2) /
     &   1.6909E-07, 1.8847E-07, 2.0892E-07, 2.3043E-07, 2.5302E-07,
     &   2.7668E-07, 3.0141E-07, 3.2723E-07, 3.5412E-07, 3.8211E-07,
     &   4.1118E-07, 4.4135E-07, 4.7262E-07, 5.0499E-07, 5.3847E-07,
     &   5.7307E-07, 6.0878E-07, 6.4561E-07, 6.8358E-07, 7.2267E-07,
     &   7.6291E-07, 8.0430E-07, 8.4684E-07, 8.9054E-07, 9.3540E-07,
     &   9.8144E-07, 1.0287E-06, 1.0771E-06, 1.1267E-06, 1.1775E-06,
     &   1.2295E-06, 1.2828E-06, 1.3373E-06, 1.3930E-06, 1.4500E-06,
     &   1.5083E-06, 1.5679E-06, 1.6287E-06, 1.6908E-06, 1.7543E-06,
     &   1.8191E-06, 1.8853E-06, 1.9528E-06, 2.0217E-06, 2.0920E-06,
     &   2.1637E-06, 2.2368E-06, 2.3115E-06, 2.3876E-06, 2.4652E-06,
     &   2.5443E-06, 2.6250E-06, 2.7073E-06, 2.7913E-06, 2.8769E-06,
     &   2.9642E-06, 3.0532E-06, 3.1441E-06, 3.2368E-06, 3.3314E-06,
     &   3.4280E-06, 3.5266E-06, 3.6274E-06, 3.7305E-06, 3.8358E-06,
     &   3.9437E-06, 4.0543E-06, 4.1677E-06, 4.2843E-06, 4.4044E-06,
     &   4.5285E-06, 4.6571E-06, 4.7914E-06, 4.8377E-06, 4.9331E-06,
     &   4.96758151E-06, 5.08601143E-06, 5.10222737E-06, 5.14102690E-06/


c      ++++ deghadt15 ++++  

c update may 2016 
c 
c******************************************************************************
c*                                                                            *
c*    --- intRdat1 ---                              Thu May 19 03:20:58 2016  *
c*                                                                            *
c***************************# FJ@HU Berlin #***********************************
c 
c  "Delta alpha"               Update version (29/09/2015) 
c 
c ISCAN: -1, Ismooth: 0, IGS: 0, DSET: WAVE, iplot: 0
c IKE,Emin,Emax,NP,N,EPSREL:  1, 6.0000E-01, 1.4000E+00, 100, 10, 1.00E-02, 
c ALS(sta)(sys):  0.1184( 0.0007)( 0.0000), MZINP:   91.1876, MTOP:    173.20
c NF,IOR,ICHK: 5,4,2,ISU2: 2,NFL: 5,INAG: 1,inotail:  0, sin2W:   0.23153
c CHPTCUT,EC,ECUT: 0.3180,     5.20,    11.50
c 
  
       DATA  (ETA  (I), I=   1, 100) /
     &  -1.0000E+03,-9.9000E+02,-9.8000E+02,-9.7000E+02,-9.6000E+02,
     &  -9.5000E+02,-9.4000E+02,-9.3000E+02,-9.2000E+02,-9.1000E+02,
     &  -9.0000E+02,-8.9000E+02,-8.8000E+02,-8.7000E+02,-8.6000E+02,
     &  -8.5000E+02,-8.4000E+02,-8.3000E+02,-8.2000E+02,-8.1000E+02,
     &  -8.0000E+02,-7.9000E+02,-7.8000E+02,-7.7000E+02,-7.6000E+02,
     &  -7.5000E+02,-7.4000E+02,-7.3000E+02,-7.2000E+02,-7.1000E+02,
     &  -7.0000E+02,-6.9000E+02,-6.8000E+02,-6.7000E+02,-6.6000E+02,
     &  -6.5000E+02,-6.4000E+02,-6.3000E+02,-6.2000E+02,-6.1000E+02,
     &  -6.0000E+02,-5.9000E+02,-5.8000E+02,-5.7000E+02,-5.6000E+02,
     &  -5.5000E+02,-5.4000E+02,-5.3000E+02,-5.2000E+02,-5.1000E+02,
     &  -5.0000E+02,-4.9000E+02,-4.8000E+02,-4.7000E+02,-4.6000E+02,
     &  -4.5000E+02,-4.4000E+02,-4.3000E+02,-4.2000E+02,-4.1000E+02,
     &  -4.0000E+02,-3.9000E+02,-3.8000E+02,-3.7000E+02,-3.6000E+02,
     &  -3.5000E+02,-3.4000E+02,-3.3000E+02,-3.2000E+02,-3.1000E+02,
     &  -3.0000E+02,-2.9000E+02,-2.8000E+02,-2.7000E+02,-2.6000E+02,
     &  -2.5000E+02,-2.4000E+02,-2.3000E+02,-2.2000E+02,-2.1000E+02,
     &  -2.0000E+02,-1.9000E+02,-1.8000E+02,-1.7000E+02,-1.6000E+02,
     &  -1.5000E+02,-1.4000E+02,-1.3000E+02,-1.2000E+02,-1.1000E+02,
     &  -1.0000E+02,-9.9000E+01,-9.8000E+01,-9.7000E+01,-9.6000E+01,
     &  -9.5000E+01,-9.4000E+01,-9.3000E+01,-9.2000E+01,-9.1000E+01/
  
       DATA  (ETA  (I), I= 101, 200) /
     &  -9.0000E+01,-8.9000E+01,-8.8000E+01,-8.7000E+01,-8.6000E+01,
     &  -8.5000E+01,-8.4000E+01,-8.3000E+01,-8.2000E+01,-8.1000E+01,
     &  -8.0000E+01,-7.9000E+01,-7.8000E+01,-7.7000E+01,-7.6000E+01,
     &  -7.5000E+01,-7.4000E+01,-7.3000E+01,-7.2000E+01,-7.1000E+01,
     &  -7.0000E+01,-6.9000E+01,-6.8000E+01,-6.7000E+01,-6.6000E+01,
     &  -6.5000E+01,-6.4000E+01,-6.3000E+01,-6.2000E+01,-6.1000E+01,
     &  -6.0000E+01,-5.9000E+01,-5.8000E+01,-5.7000E+01,-5.6000E+01,
     &  -5.5000E+01,-5.4000E+01,-5.3000E+01,-5.2000E+01,-5.1000E+01,
     &  -5.0000E+01,-4.9000E+01,-4.8000E+01,-4.7000E+01,-4.6000E+01,
     &  -4.5000E+01,-4.4000E+01,-4.3000E+01,-4.2000E+01,-4.1000E+01,
     &  -4.0000E+01,-3.9000E+01,-3.8000E+01,-3.7000E+01,-3.6000E+01,
     &  -3.5000E+01,-3.4000E+01,-3.3000E+01,-3.2000E+01,-3.1000E+01,
     &  -3.0000E+01,-2.9000E+01,-2.8000E+01,-2.7000E+01,-2.6000E+01,
     &  -2.5000E+01,-2.4000E+01,-2.3000E+01,-2.2000E+01,-2.1000E+01,
     &  -2.0000E+01,-1.9000E+01,-1.8000E+01,-1.7000E+01,-1.6000E+01,
     &  -1.5000E+01,-1.4000E+01,-1.3000E+01,-1.2000E+01,-1.1000E+01,
     &  -1.0000E+01,-9.9100E+00,-9.8200E+00,-9.7300E+00,-9.6400E+00,
     &  -9.5500E+00,-9.4600E+00,-9.3700E+00,-9.2800E+00,-9.1900E+00,
     &  -9.1000E+00,-9.0100E+00,-8.9200E+00,-8.8300E+00,-8.7400E+00,
     &  -8.6500E+00,-8.5600E+00,-8.4700E+00,-8.3800E+00,-8.2900E+00/
  
       DATA  (ETA  (I), I= 201, 300) /
     &  -8.2000E+00,-8.1100E+00,-8.0200E+00,-7.9300E+00,-7.8400E+00,
     &  -7.7500E+00,-7.6600E+00,-7.5700E+00,-7.4800E+00,-7.3900E+00,
     &  -7.3000E+00,-7.2100E+00,-7.1200E+00,-7.0300E+00,-6.9400E+00,
     &  -6.8500E+00,-6.7600E+00,-6.6700E+00,-6.5800E+00,-6.4900E+00,
     &  -6.4000E+00,-6.3100E+00,-6.2200E+00,-6.1300E+00,-6.0400E+00,
     &  -5.9500E+00,-5.8600E+00,-5.7700E+00,-5.6800E+00,-5.5900E+00,
     &  -5.5000E+00,-5.4100E+00,-5.3200E+00,-5.2300E+00,-5.1400E+00,
     &  -5.0500E+00,-4.9600E+00,-4.8700E+00,-4.7800E+00,-4.6900E+00,
     &  -4.6000E+00,-4.5100E+00,-4.4200E+00,-4.3300E+00,-4.2400E+00,
     &  -4.1500E+00,-4.0600E+00,-3.9700E+00,-3.8800E+00,-3.7900E+00,
     &  -3.7000E+00,-3.6100E+00,-3.5200E+00,-3.4300E+00,-3.3400E+00,
     &  -3.2500E+00,-3.1600E+00,-3.0700E+00,-2.9800E+00,-2.8900E+00,
     &  -2.8000E+00,-2.7100E+00,-2.6200E+00,-2.5300E+00,-2.4400E+00,
     &  -2.3500E+00,-2.2600E+00,-2.1700E+00,-2.0800E+00,-1.9900E+00,
     &  -1.9000E+00,-1.8100E+00,-1.7200E+00,-1.6300E+00,-1.5400E+00,
     &  -1.4500E+00,-1.3600E+00,-1.2700E+00,-1.1800E+00,-1.0900E+00,
     &  -1.0000E+00,-9.9100E-01,-9.8200E-01,-9.7300E-01,-9.6400E-01,
     &  -9.5500E-01,-9.4600E-01,-9.3700E-01,-9.2800E-01,-9.1900E-01,
     &  -9.1000E-01,-9.0100E-01,-8.9200E-01,-8.8300E-01,-8.7400E-01,
     &  -8.6500E-01,-8.5600E-01,-8.4700E-01,-8.3800E-01,-8.2900E-01/
  
       DATA  (ETA  (I), I= 301, 400) /
     &  -8.2000E-01,-8.1100E-01,-8.0200E-01,-7.9300E-01,-7.8400E-01,
     &  -7.7500E-01,-7.6600E-01,-7.5700E-01,-7.4800E-01,-7.3900E-01,
     &  -7.3000E-01,-7.2100E-01,-7.1200E-01,-7.0300E-01,-6.9400E-01,
     &  -6.8500E-01,-6.7600E-01,-6.6700E-01,-6.5800E-01,-6.4900E-01,
     &  -6.4000E-01,-6.3100E-01,-6.2200E-01,-6.1300E-01,-6.0400E-01,
     &  -5.9500E-01,-5.8600E-01,-5.7700E-01,-5.6800E-01,-5.5900E-01,
     &  -5.5000E-01,-5.4100E-01,-5.3200E-01,-5.2300E-01,-5.1400E-01,
     &  -5.0500E-01,-4.9600E-01,-4.8700E-01,-4.7800E-01,-4.6900E-01,
     &  -4.6000E-01,-4.5100E-01,-4.4200E-01,-4.3300E-01,-4.2400E-01,
     &  -4.1500E-01,-4.0600E-01,-3.9700E-01,-3.8800E-01,-3.7900E-01,
     &  -3.7000E-01,-3.6100E-01,-3.5200E-01,-3.4300E-01,-3.3400E-01,
     &  -3.2500E-01,-3.1600E-01,-3.0700E-01,-2.9800E-01,-2.8900E-01,
     &  -2.8000E-01,-2.7100E-01,-2.6200E-01,-2.5300E-01,-2.4400E-01,
     &  -2.3500E-01,-2.2600E-01,-2.1700E-01,-2.0800E-01,-1.9900E-01,
     &  -1.9000E-01,-1.8100E-01,-1.7200E-01,-1.6300E-01,-1.5400E-01,
     &  -1.4500E-01,-1.3600E-01,-1.2700E-01,-1.1800E-01,-1.0900E-01,
     &  -1.0000E-01,-9.9100E-02,-9.8200E-02,-9.7300E-02,-9.6400E-02,
     &  -9.5500E-02,-9.4600E-02,-9.3700E-02,-9.2800E-02,-9.1900E-02,
     &  -9.1000E-02,-9.0100E-02,-8.9200E-02,-8.8300E-02,-8.7400E-02,
     &  -8.6500E-02,-8.5600E-02,-8.4700E-02,-8.3800E-02,-8.2900E-02/
  
       DATA  (ETA  (I), I= 401, 500) /
     &  -8.2000E-02,-8.1100E-02,-8.0200E-02,-7.9300E-02,-7.8400E-02,
     &  -7.7500E-02,-7.6600E-02,-7.5700E-02,-7.4800E-02,-7.3900E-02,
     &  -7.3000E-02,-7.2100E-02,-7.1200E-02,-7.0300E-02,-6.9400E-02,
     &  -6.8500E-02,-6.7600E-02,-6.6700E-02,-6.5800E-02,-6.4900E-02,
     &  -6.4000E-02,-6.3100E-02,-6.2200E-02,-6.1300E-02,-6.0400E-02,
     &  -5.9500E-02,-5.8600E-02,-5.7700E-02,-5.6800E-02,-5.5900E-02,
     &  -5.5000E-02,-5.4100E-02,-5.3200E-02,-5.2300E-02,-5.1400E-02,
     &  -5.0500E-02,-4.9600E-02,-4.8700E-02,-4.7800E-02,-4.6900E-02,
     &  -4.6000E-02,-4.5100E-02,-4.4200E-02,-4.3300E-02,-4.2400E-02,
     &  -4.1500E-02,-4.0600E-02,-3.9700E-02,-3.8800E-02,-3.7900E-02,
     &  -3.7000E-02,-3.6100E-02,-3.5200E-02,-3.4300E-02,-3.3400E-02,
     &  -3.2500E-02,-3.1600E-02,-3.0700E-02,-2.9800E-02,-2.8900E-02,
     &  -2.8000E-02,-2.7100E-02,-2.6200E-02,-2.5300E-02,-2.4400E-02,
     &  -2.3500E-02,-2.2600E-02,-2.1700E-02,-2.0800E-02,-1.9900E-02,
     &  -1.9000E-02,-1.8100E-02,-1.7200E-02,-1.6300E-02,-1.5400E-02,
     &  -1.4500E-02,-1.3600E-02,-1.2700E-02,-1.1800E-02,-1.0900E-02,
     &  -1.0000E-02,-9.9100E-03,-9.8200E-03,-9.7300E-03,-9.6400E-03,
     &  -9.5500E-03,-9.4600E-03,-9.3700E-03,-9.2800E-03,-9.1900E-03,
     &  -9.1000E-03,-9.0100E-03,-8.9200E-03,-8.8300E-03,-8.7400E-03,
     &  -8.6500E-03,-8.5600E-03,-8.4700E-03,-8.3800E-03,-8.2900E-03/
  
       DATA  (ETA  (I), I= 501, 600) /
     &  -8.2000E-03,-8.1100E-03,-8.0200E-03,-7.9300E-03,-7.8400E-03,
     &  -7.7500E-03,-7.6600E-03,-7.5700E-03,-7.4800E-03,-7.3900E-03,
     &  -7.3000E-03,-7.2100E-03,-7.1200E-03,-7.0300E-03,-6.9400E-03,
     &  -6.8500E-03,-6.7600E-03,-6.6700E-03,-6.5800E-03,-6.4900E-03,
     &  -6.4000E-03,-6.3100E-03,-6.2200E-03,-6.1300E-03,-6.0400E-03,
     &  -5.9500E-03,-5.8600E-03,-5.7700E-03,-5.6800E-03,-5.5900E-03,
     &  -5.5000E-03,-5.4100E-03,-5.3200E-03,-5.2300E-03,-5.1400E-03,
     &  -5.0500E-03,-4.9600E-03,-4.8700E-03,-4.7800E-03,-4.6900E-03,
     &  -4.6000E-03,-4.5100E-03,-4.4200E-03,-4.3300E-03,-4.2400E-03,
     &  -4.1500E-03,-4.0600E-03,-3.9700E-03,-3.8800E-03,-3.7900E-03,
     &  -3.7000E-03,-3.6100E-03,-3.5200E-03,-3.4300E-03,-3.3400E-03,
     &  -3.2500E-03,-3.1600E-03,-3.0700E-03,-2.9800E-03,-2.8900E-03,
     &  -2.8000E-03,-2.7100E-03,-2.6200E-03,-2.5300E-03,-2.4400E-03,
     &  -2.3500E-03,-2.2600E-03,-2.1700E-03,-2.0800E-03,-1.9900E-03,
     &  -1.9000E-03,-1.8100E-03,-1.7200E-03,-1.6300E-03,-1.5400E-03,
     &  -1.4500E-03,-1.3600E-03,-1.2700E-03,-1.1800E-03,-1.0900E-03,
     &  -1.0000E-03,-9.9100E-04,-9.8200E-04,-9.7300E-04,-9.6400E-04,
     &  -9.5500E-04,-9.4600E-04,-9.3700E-04,-9.2800E-04,-9.1900E-04,
     &  -9.1000E-04,-9.0100E-04,-8.9200E-04,-8.8300E-04,-8.7400E-04,
     &  -8.6500E-04,-8.5600E-04,-8.4700E-04,-8.3800E-04,-8.2900E-04/
  
       DATA  (ETA  (I), I= 601, 700) /
     &  -8.2000E-04,-8.1100E-04,-8.0200E-04,-7.9300E-04,-7.8400E-04,
     &  -7.7500E-04,-7.6600E-04,-7.5700E-04,-7.4800E-04,-7.3900E-04,
     &  -7.3000E-04,-7.2100E-04,-7.1200E-04,-7.0300E-04,-6.9400E-04,
     &  -6.8500E-04,-6.7600E-04,-6.6700E-04,-6.5800E-04,-6.4900E-04,
     &  -6.4000E-04,-6.3100E-04,-6.2200E-04,-6.1300E-04,-6.0400E-04,
     &  -5.9500E-04,-5.8600E-04,-5.7700E-04,-5.6800E-04,-5.5900E-04,
     &  -5.5000E-04,-5.4100E-04,-5.3200E-04,-5.2300E-04,-5.1400E-04,
     &  -5.0500E-04,-4.9600E-04,-4.8700E-04,-4.7800E-04,-4.6900E-04,
     &  -4.6000E-04,-4.5100E-04,-4.4200E-04,-4.3300E-04,-4.2400E-04,
     &  -4.1500E-04,-4.0600E-04,-3.9700E-04,-3.8800E-04,-3.7900E-04,
     &  -3.7000E-04,-3.6100E-04,-3.5200E-04,-3.4300E-04,-3.3400E-04,
     &  -3.2500E-04,-3.1600E-04,-3.0700E-04,-2.9800E-04,-2.8900E-04,
     &  -2.8000E-04,-2.7100E-04,-2.6200E-04,-2.5300E-04,-2.4400E-04,
     &  -2.3500E-04,-2.2600E-04,-2.1700E-04,-2.0800E-04,-1.9900E-04,
     &  -1.9000E-04,-1.8100E-04,-1.7200E-04,-1.6300E-04,-1.5400E-04,
     &  -1.4500E-04,-1.3600E-04,-1.2700E-04,-1.1800E-04,-1.0900E-04,
     &  -1.0000E-04,-9.9100E-05,-9.8200E-05,-9.7300E-05,-9.6400E-05,
     &  -9.5500E-05,-9.4600E-05,-9.3700E-05,-9.2800E-05,-9.1900E-05,
     &  -9.1000E-05,-9.0100E-05,-8.9200E-05,-8.8300E-05,-8.7400E-05,
     &  -8.6500E-05,-8.5600E-05,-8.4700E-05,-8.3800E-05,-8.2900E-05/
  
       DATA  (ETA  (I), I= 701, 800) /
     &  -8.2000E-05,-8.1100E-05,-8.0200E-05,-7.9300E-05,-7.8400E-05,
     &  -7.7500E-05,-7.6600E-05,-7.5700E-05,-7.4800E-05,-7.3900E-05,
     &  -7.3000E-05,-7.2100E-05,-7.1200E-05,-7.0300E-05,-6.9400E-05,
     &  -6.8500E-05,-6.7600E-05,-6.6700E-05,-6.5800E-05,-6.4900E-05,
     &  -6.4000E-05,-6.3100E-05,-6.2200E-05,-6.1300E-05,-6.0400E-05,
     &  -5.9500E-05,-5.8600E-05,-5.7700E-05,-5.6800E-05,-5.5900E-05,
     &  -5.5000E-05,-5.4100E-05,-5.3200E-05,-5.2300E-05,-5.1400E-05,
     &  -5.0500E-05,-4.9600E-05,-4.8700E-05,-4.7800E-05,-4.6900E-05,
     &  -4.6000E-05,-4.5100E-05,-4.4200E-05,-4.3300E-05,-4.2400E-05,
     &  -4.1500E-05,-4.0600E-05,-3.9700E-05,-3.8800E-05,-3.7900E-05,
     &  -3.7000E-05,-3.6100E-05,-3.5200E-05,-3.4300E-05,-3.3400E-05,
     &  -3.2500E-05,-3.1600E-05,-3.0700E-05,-2.9800E-05,-2.8900E-05,
     &  -2.8000E-05,-2.7100E-05,-2.6200E-05,-2.5300E-05,-2.4400E-05,
     &  -2.3500E-05,-2.2600E-05,-2.1700E-05,-2.0800E-05,-1.9900E-05,
     &  -1.9000E-05,-1.8100E-05,-1.7200E-05,-1.6300E-05,-1.5400E-05,
     &  -1.4500E-05,-1.3600E-05,-1.2700E-05,-1.1800E-05,-1.0900E-05,
     &  -1.0000E-05,-9.9100E-06,-9.8200E-06,-9.7300E-06,-9.6400E-06,
     &  -9.5500E-06,-9.4600E-06,-9.3700E-06,-9.2800E-06,-9.1900E-06,
     &  -9.1000E-06,-9.0100E-06,-8.9200E-06,-8.8300E-06,-8.7400E-06,
     &  -8.6500E-06,-8.5600E-06,-8.4700E-06,-8.3800E-06,-8.2900E-06/
  
       DATA  (ETA  (I), I= 801, 900) /
     &  -8.2000E-06,-8.1100E-06,-8.0200E-06,-7.9300E-06,-7.8400E-06,
     &  -7.7500E-06,-7.6600E-06,-7.5700E-06,-7.4800E-06,-7.3900E-06,
     &  -7.3000E-06,-7.2100E-06,-7.1200E-06,-7.0300E-06,-6.9400E-06,
     &  -6.8500E-06,-6.7600E-06,-6.6700E-06,-6.5800E-06,-6.4900E-06,
     &  -6.4000E-06,-6.3100E-06,-6.2200E-06,-6.1300E-06,-6.0400E-06,
     &  -5.9500E-06,-5.8600E-06,-5.7700E-06,-5.6800E-06,-5.5900E-06,
     &  -5.5000E-06,-5.4100E-06,-5.3200E-06,-5.2300E-06,-5.1400E-06,
     &  -5.0500E-06,-4.9600E-06,-4.8700E-06,-4.7800E-06,-4.6900E-06,
     &  -4.6000E-06,-4.5100E-06,-4.4200E-06,-4.3300E-06,-4.2400E-06,
     &  -4.1500E-06,-4.0600E-06,-3.9700E-06,-3.8800E-06,-3.7900E-06,
     &  -3.7000E-06,-3.6100E-06,-3.5200E-06,-3.4300E-06,-3.3400E-06,
     &  -3.2500E-06,-3.1600E-06,-3.0700E-06,-2.9800E-06,-2.8900E-06,
     &  -2.8000E-06,-2.7100E-06,-2.6200E-06,-2.5300E-06,-2.4400E-06,
     &  -2.3500E-06,-2.2600E-06,-2.1700E-06,-2.0800E-06,-1.9900E-06,
     &  -1.9000E-06,-1.8100E-06,-1.7200E-06,-1.6300E-06,-1.5400E-06,
     &  -1.4500E-06,-1.3600E-06,-1.2700E-06,-1.1800E-06,-1.0900E-06,
     &  -1.0000E-06, 0.0000E+00, 1.0000E-06, 3.0010E-03, 6.0010E-03,
     &   9.0010E-03, 1.2000E-02, 1.5000E-02, 1.8000E-02, 2.1000E-02,
     &   2.4000E-02, 2.7000E-02, 3.0000E-02, 3.3000E-02, 3.6000E-02,
     &   3.9000E-02, 4.2000E-02, 4.5000E-02, 4.8000E-02, 5.1000E-02/
  
       DATA  (ETA  (I), I= 901, 979) /
     &   5.4000E-02, 5.7000E-02, 6.0000E-02, 6.3000E-02, 6.6000E-02,
     &   6.9000E-02, 7.2000E-02, 7.5000E-02, 7.8000E-02, 8.1000E-02,
     &   8.4000E-02, 8.7000E-02, 9.0000E-02, 9.3000E-02, 9.6000E-02,
     &   9.9000E-02, 1.0200E-01, 1.0500E-01, 1.0800E-01, 1.1100E-01,
     &   1.1400E-01, 1.1700E-01, 1.2000E-01, 1.2300E-01, 1.2600E-01,
     &   1.2900E-01, 1.3200E-01, 1.3500E-01, 1.3800E-01, 1.4100E-01,
     &   1.4400E-01, 1.4700E-01, 1.5000E-01, 1.5300E-01, 1.5600E-01,
     &   1.5900E-01, 1.6200E-01, 1.6500E-01, 1.6800E-01, 1.7100E-01,
     &   1.7400E-01, 1.7700E-01, 1.8000E-01, 1.8300E-01, 1.8600E-01,
     &   1.8900E-01, 1.9200E-01, 1.9500E-01, 1.9800E-01, 2.0100E-01,
     &   2.0400E-01, 2.0700E-01, 2.1000E-01, 2.1300E-01, 2.1600E-01,
     &   2.1900E-01, 2.2200E-01, 2.2500E-01, 2.2800E-01, 2.3100E-01,
     &   2.3400E-01, 2.3700E-01, 2.4000E-01, 2.4300E-01, 2.4600E-01,
     &   2.4900E-01, 2.5200E-01, 2.5500E-01, 2.5800E-01, 2.6100E-01,
     &   2.6400E-01, 2.6700E-01, 2.7000E-01, 2.7100E-01, 2.7300E-01,
     &   0.27370000D0, 0.27599999D0, 0.27630001D0, 0.27700001D0/
  
         DATA ((DGT  (I,J), I=   1, 100), J=1,1) /
     &   8.3773E-02, 8.3667E-02, 8.3559E-02, 8.3451E-02, 8.3341E-02,
     &   8.3230E-02, 8.3118E-02, 8.3004E-02, 8.2889E-02, 8.2773E-02,
     &   8.2655E-02, 8.2536E-02, 8.2415E-02, 8.2293E-02, 8.2169E-02,
     &   8.2043E-02, 8.1916E-02, 8.1787E-02, 8.1657E-02, 8.1524E-02,
     &   8.1390E-02, 8.1254E-02, 8.1116E-02, 8.0976E-02, 8.0834E-02,
     &   8.0690E-02, 8.0543E-02, 8.0395E-02, 8.0244E-02, 8.0091E-02,
     &   7.9935E-02, 7.9777E-02, 7.9616E-02, 7.9453E-02, 7.9286E-02,
     &   7.9118E-02, 7.8946E-02, 7.8771E-02, 7.8593E-02, 7.8411E-02,
     &   7.8227E-02, 7.8039E-02, 7.7847E-02, 7.7652E-02, 7.7453E-02,
     &   7.7250E-02, 7.7043E-02, 7.6831E-02, 7.6615E-02, 7.6395E-02,
     &   7.6169E-02, 7.5939E-02, 7.5704E-02, 7.5463E-02, 7.5216E-02,
     &   7.4964E-02, 7.4705E-02, 7.4440E-02, 7.4169E-02, 7.3890E-02,
     &   7.3604E-02, 7.3309E-02, 7.3007E-02, 7.2696E-02, 7.2377E-02,
     &   7.2047E-02, 7.1707E-02, 7.1357E-02, 7.0995E-02, 7.0621E-02,
     &   7.0234E-02, 6.9834E-02, 6.9418E-02, 6.8987E-02, 6.8539E-02,
     &   6.8073E-02, 6.7587E-02, 6.7079E-02, 6.6548E-02, 6.5992E-02,
     &   6.5407E-02, 6.4792E-02, 6.4143E-02, 6.3456E-02, 6.2726E-02,
     &   6.1948E-02, 6.1116E-02, 6.0220E-02, 5.9253E-02, 5.8199E-02,
     &   5.7044E-02, 5.6922E-02, 5.6799E-02, 5.6675E-02, 5.6549E-02,
     &   5.6422E-02, 5.6294E-02, 5.6164E-02, 5.6033E-02, 5.5900E-02/
  
         DATA ((DGT  (I,J), I= 101, 200), J=1,1) /
     &   5.5766E-02, 5.5630E-02, 5.5493E-02, 5.5355E-02, 5.5214E-02,
     &   5.5072E-02, 5.4928E-02, 5.4783E-02, 5.4636E-02, 5.4487E-02,
     &   5.4336E-02, 5.4183E-02, 5.4028E-02, 5.3871E-02, 5.3713E-02,
     &   5.3552E-02, 5.3389E-02, 5.3223E-02, 5.3055E-02, 5.2885E-02,
     &   5.2713E-02, 5.2538E-02, 5.2361E-02, 5.2180E-02, 5.1997E-02,
     &   5.1812E-02, 5.1623E-02, 5.1432E-02, 5.1237E-02, 5.1039E-02,
     &   5.0838E-02, 5.0634E-02, 5.0426E-02, 5.0214E-02, 4.9998E-02,
     &   4.9779E-02, 4.9556E-02, 4.9328E-02, 4.9097E-02, 4.8860E-02,
     &   4.8619E-02, 4.8373E-02, 4.8122E-02, 4.7866E-02, 4.7604E-02,
     &   4.7337E-02, 4.7063E-02, 4.6783E-02, 4.6497E-02, 4.6204E-02,
     &   4.5903E-02, 4.5595E-02, 4.5279E-02, 4.4954E-02, 4.4621E-02,
     &   4.4279E-02, 4.3926E-02, 4.3563E-02, 4.3189E-02, 4.2804E-02,
     &   4.2405E-02, 4.1994E-02, 4.1568E-02, 4.1127E-02, 4.0670E-02,
     &   4.0195E-02, 3.9701E-02, 3.9187E-02, 3.8650E-02, 3.8089E-02,
     &   3.7502E-02, 3.6885E-02, 3.6237E-02, 3.5553E-02, 3.4830E-02,
     &   3.4063E-02, 3.3246E-02, 3.2373E-02, 3.1435E-02, 3.0422E-02,
     &   2.9322E-02, 2.9218E-02, 2.9113E-02, 2.9007E-02, 2.8901E-02,
     &   2.8794E-02, 2.8685E-02, 2.8576E-02, 2.8466E-02, 2.8355E-02,
     &   2.8243E-02, 2.8130E-02, 2.8016E-02, 2.7901E-02, 2.7784E-02,
     &   2.7667E-02, 2.7549E-02, 2.7429E-02, 2.7309E-02, 2.7187E-02/
  
         DATA ((DGT  (I,J), I= 201, 300), J=1,1) /
     &   2.7064E-02, 2.6940E-02, 2.6815E-02, 2.6688E-02, 2.6560E-02,
     &   2.6431E-02, 2.6301E-02, 2.6169E-02, 2.6036E-02, 2.5901E-02,
     &   2.5765E-02, 2.5628E-02, 2.5489E-02, 2.5348E-02, 2.5206E-02,
     &   2.5063E-02, 2.4917E-02, 2.4770E-02, 2.4621E-02, 2.4471E-02,
     &   2.4319E-02, 2.4165E-02, 2.4008E-02, 2.3851E-02, 2.3688E-02,
     &   2.3526E-02, 2.3362E-02, 2.3196E-02, 2.3028E-02, 2.2857E-02,
     &   2.2684E-02, 2.2508E-02, 2.2330E-02, 2.2150E-02, 2.1967E-02,
     &   2.1781E-02, 2.1593E-02, 2.1401E-02, 2.1207E-02, 2.1010E-02,
     &   2.0809E-02, 2.0605E-02, 2.0398E-02, 2.0188E-02, 1.9974E-02,
     &   1.9756E-02, 1.9534E-02, 1.9309E-02, 1.9079E-02, 1.8845E-02,
     &   1.8607E-02, 1.8363E-02, 1.8116E-02, 1.7863E-02, 1.7605E-02,
     &   1.7341E-02, 1.7072E-02, 1.6796E-02, 1.6515E-02, 1.6226E-02,
     &   1.5931E-02, 1.5629E-02, 1.5318E-02, 1.5000E-02, 1.4674E-02,
     &   1.4338E-02, 1.3991E-02, 1.3634E-02, 1.3265E-02, 1.2884E-02,
     &   1.2490E-02, 1.2081E-02, 1.1656E-02, 1.1213E-02, 1.0751E-02,
     &   1.0268E-02, 9.7605E-03, 9.2268E-03, 8.6644E-03, 8.0695E-03,
     &   7.4390E-03, 7.3739E-03, 7.3084E-03, 7.2425E-03, 7.1762E-03,
     &   7.1095E-03, 7.0424E-03, 6.9749E-03, 6.9070E-03, 6.8386E-03,
     &   6.7699E-03, 6.7008E-03, 6.6313E-03, 6.5613E-03, 6.4909E-03,
     &   6.4201E-03, 6.3489E-03, 6.2773E-03, 6.2053E-03, 6.1328E-03/
  
         DATA ((DGT  (I,J), I= 301, 400), J=1,1) /
     &   6.0600E-03, 5.9867E-03, 5.9130E-03, 5.8389E-03, 5.7643E-03,
     &   5.6894E-03, 5.6140E-03, 5.5382E-03, 5.4621E-03, 5.3855E-03,
     &   5.3085E-03, 5.2311E-03, 5.1533E-03, 5.0752E-03, 4.9967E-03,
     &   4.9178E-03, 4.8385E-03, 4.7589E-03, 4.6789E-03, 4.5986E-03,
     &   4.5179E-03, 4.4369E-03, 4.3557E-03, 4.2741E-03, 4.1922E-03,
     &   4.1101E-03, 4.0277E-03, 3.9451E-03, 3.8623E-03, 3.7793E-03,
     &   3.6960E-03, 3.6127E-03, 3.5292E-03, 3.4455E-03, 3.3618E-03,
     &   3.2781E-03, 3.1943E-03, 3.1104E-03, 3.0266E-03, 2.9429E-03,
     &   2.8593E-03, 2.7758E-03, 2.6924E-03, 2.6092E-03, 2.5263E-03,
     &   2.4437E-03, 2.3614E-03, 2.2795E-03, 2.1980E-03, 2.1169E-03,
     &   2.0364E-03, 1.9565E-03, 1.8771E-03, 1.7985E-03, 1.7206E-03,
     &   1.6435E-03, 1.5673E-03, 1.4921E-03, 1.4178E-03, 1.3445E-03,
     &   1.2725E-03, 1.2016E-03, 1.1320E-03, 1.0637E-03, 9.9685E-04,
     &   9.3151E-04, 8.6774E-04, 8.0562E-04, 7.4523E-04, 6.8666E-04,
     &   6.2996E-04, 5.7523E-04, 5.2254E-04, 4.7197E-04, 4.2359E-04,
     &   3.7748E-04, 3.3372E-04, 2.9237E-04, 2.5351E-04, 2.1721E-04,
     &   1.8352E-04, 1.8029E-04, 1.7710E-04, 1.7393E-04, 1.7079E-04,
     &   1.6767E-04, 1.6458E-04, 1.6152E-04, 1.5849E-04, 1.5548E-04,
     &   1.5250E-04, 1.4955E-04, 1.4663E-04, 1.4373E-04, 1.4086E-04,
     &   1.3802E-04, 1.3521E-04, 1.3242E-04, 1.2966E-04, 1.2693E-04/
  
         DATA ((DGT  (I,J), I= 401, 500), J=1,1) /
     &   1.2423E-04, 1.2155E-04, 1.1891E-04, 1.1629E-04, 1.1369E-04,
     &   1.1113E-04, 1.0860E-04, 1.0609E-04, 1.0361E-04, 1.0116E-04,
     &   9.8739E-05, 9.6346E-05, 9.3981E-05, 9.1644E-05, 8.9336E-05,
     &   8.7057E-05, 8.4806E-05, 8.2583E-05, 8.0390E-05, 7.8225E-05,
     &   7.6089E-05, 7.3981E-05, 7.1903E-05, 6.9854E-05, 6.7833E-05,
     &   6.5841E-05, 6.3879E-05, 6.1945E-05, 6.0041E-05, 5.8166E-05,
     &   5.6320E-05, 5.4503E-05, 5.2715E-05, 5.0957E-05, 4.9228E-05,
     &   4.7528E-05, 4.5858E-05, 4.4217E-05, 4.2606E-05, 4.1024E-05,
     &   3.9471E-05, 3.7948E-05, 3.6455E-05, 3.4991E-05, 3.3557E-05,
     &   3.2153E-05, 3.0778E-05, 2.9433E-05, 2.8118E-05, 2.6833E-05,
     &   2.5577E-05, 2.4351E-05, 2.3156E-05, 2.1989E-05, 2.0853E-05,
     &   1.9747E-05, 1.8671E-05, 1.7625E-05, 1.6608E-05, 1.5622E-05,
     &   1.4666E-05, 1.3740E-05, 1.2843E-05, 1.1977E-05, 1.1141E-05,
     &   1.0336E-05, 9.5600E-06, 8.8145E-06, 8.0991E-06, 7.4140E-06,
     &   6.7591E-06, 6.1343E-06, 5.5398E-06, 4.9756E-06, 4.4415E-06,
     &   3.9378E-06, 3.4643E-06, 3.0211E-06, 2.6082E-06, 2.2256E-06,
     &   1.8734E-06, 1.8398E-06, 1.8065E-06, 1.7736E-06, 1.7409E-06,
     &   1.7086E-06, 1.6765E-06, 1.6448E-06, 1.6133E-06, 1.5822E-06,
     &   1.5514E-06, 1.5209E-06, 1.4906E-06, 1.4607E-06, 1.4311E-06,
     &   1.4018E-06, 1.3728E-06, 1.3440E-06, 1.3156E-06, 1.2875E-06/
  
         DATA ((DGT  (I,J), I= 501, 600), J=1,1) /
     &   1.2597E-06, 1.2322E-06, 1.2050E-06, 1.1781E-06, 1.1516E-06,
     &   1.1253E-06, 1.0993E-06, 1.0736E-06, 1.0482E-06, 1.0232E-06,
     &   9.9841E-07, 9.7395E-07, 9.4979E-07, 9.2593E-07, 9.0237E-07,
     &   8.7912E-07, 8.5618E-07, 8.3353E-07, 8.1119E-07, 7.8916E-07,
     &   7.6742E-07, 7.4599E-07, 7.2487E-07, 7.0404E-07, 6.8352E-07,
     &   6.6331E-07, 6.4339E-07, 6.2378E-07, 6.0448E-07, 5.8547E-07,
     &   5.6677E-07, 5.4838E-07, 5.3028E-07, 5.1250E-07, 4.9501E-07,
     &   4.7783E-07, 4.6095E-07, 4.4437E-07, 4.2810E-07, 4.1213E-07,
     &   3.9647E-07, 3.8111E-07, 3.6605E-07, 3.5129E-07, 3.3684E-07,
     &   3.2270E-07, 3.0885E-07, 2.9531E-07, 2.8207E-07, 2.6914E-07,
     &   2.5651E-07, 2.4418E-07, 2.3216E-07, 2.2044E-07, 2.0902E-07,
     &   1.9791E-07, 1.8710E-07, 1.7660E-07, 1.6639E-07, 1.5649E-07,
     &   1.4690E-07, 1.3761E-07, 1.2862E-07, 1.1994E-07, 1.1155E-07,
     &   1.0348E-07, 9.5703E-08, 8.8232E-08, 8.1065E-08, 7.4202E-08,
     &   6.7642E-08, 6.1386E-08, 5.5433E-08, 4.9783E-08, 4.4438E-08,
     &   3.9395E-08, 3.4657E-08, 3.0222E-08, 2.6090E-08, 2.2262E-08,
     &   1.8737E-08, 1.8402E-08, 1.8069E-08, 1.7739E-08, 1.7413E-08,
     &   1.7089E-08, 1.6768E-08, 1.6451E-08, 1.6136E-08, 1.5825E-08,
     &   1.5517E-08, 1.5211E-08, 1.4909E-08, 1.4609E-08, 1.4313E-08,
     &   1.4020E-08, 1.3730E-08, 1.3442E-08, 1.3158E-08, 1.2877E-08/
  
         DATA ((DGT  (I,J), I= 601, 700), J=1,1) /
     &   1.2599E-08, 1.2324E-08, 1.2052E-08, 1.1783E-08, 1.1517E-08,
     &   1.1254E-08, 1.0994E-08, 1.0738E-08, 1.0484E-08, 1.0233E-08,
     &   9.9852E-09, 9.7405E-09, 9.4989E-09, 9.2602E-09, 9.0247E-09,
     &   8.7921E-09, 8.5626E-09, 8.3361E-09, 8.1127E-09, 7.8923E-09,
     &   7.6749E-09, 7.4605E-09, 7.2492E-09, 7.0410E-09, 6.8357E-09,
     &   6.6335E-09, 6.4344E-09, 6.2383E-09, 6.0452E-09, 5.8551E-09,
     &   5.6681E-09, 5.4841E-09, 5.3032E-09, 5.1253E-09, 4.9504E-09,
     &   4.7785E-09, 4.6097E-09, 4.4440E-09, 4.2812E-09, 4.1215E-09,
     &   3.9649E-09, 3.8112E-09, 3.6606E-09, 3.5131E-09, 3.3686E-09,
     &   3.2271E-09, 3.0886E-09, 2.9532E-09, 2.8208E-09, 2.6915E-09,
     &   2.5652E-09, 2.4419E-09, 2.3217E-09, 2.2045E-09, 2.0903E-09,
     &   1.9792E-09, 1.8711E-09, 1.7660E-09, 1.6640E-09, 1.5650E-09,
     &   1.4690E-09, 1.3761E-09, 1.2862E-09, 1.1994E-09, 1.1156E-09,
     &   1.0348E-09, 9.5704E-10, 8.8233E-10, 8.1066E-10, 7.4202E-10,
     &   6.7642E-10, 6.1386E-10, 5.5433E-10, 4.9784E-10, 4.4438E-10,
     &   3.9396E-10, 3.4657E-10, 3.0222E-10, 2.6090E-10, 2.2262E-10,
     &   1.8738E-10, 1.8402E-10, 1.8069E-10, 1.7739E-10, 1.7413E-10,
     &   1.7089E-10, 1.6769E-10, 1.6451E-10, 1.6136E-10, 1.5825E-10,
     &   1.5517E-10, 1.5211E-10, 1.4909E-10, 1.4609E-10, 1.4313E-10,
     &   1.4020E-10, 1.3730E-10, 1.3442E-10, 1.3158E-10, 1.2877E-10/
  
         DATA ((DGT  (I,J), I= 701, 800), J=1,1) /
     &   1.2599E-10, 1.2324E-10, 1.2052E-10, 1.1783E-10, 1.1517E-10,
     &   1.1254E-10, 1.0994E-10, 1.0738E-10, 1.0484E-10, 1.0233E-10,
     &   9.9852E-11, 9.7405E-11, 9.4989E-11, 9.2603E-11, 9.0247E-11,
     &   8.7921E-11, 8.5626E-11, 8.3361E-11, 8.1127E-11, 7.8923E-11,
     &   7.6749E-11, 7.4606E-11, 7.2492E-11, 7.0410E-11, 6.8357E-11,
     &   6.6336E-11, 6.4344E-11, 6.2383E-11, 6.0452E-11, 5.8551E-11,
     &   5.6681E-11, 5.4841E-11, 5.3032E-11, 5.1253E-11, 4.9504E-11,
     &   4.7785E-11, 4.6097E-11, 4.4440E-11, 4.2812E-11, 4.1215E-11,
     &   3.9649E-11, 3.8112E-11, 3.6606E-11, 3.5131E-11, 3.3686E-11,
     &   3.2271E-11, 3.0886E-11, 2.9532E-11, 2.8208E-11, 2.6915E-11,
     &   2.5652E-11, 2.4419E-11, 2.3217E-11, 2.2045E-11, 2.0903E-11,
     &   1.9792E-11, 1.8711E-11, 1.7660E-11, 1.6640E-11, 1.5650E-11,
     &   1.4690E-11, 1.3761E-11, 1.2862E-11, 1.1994E-11, 1.1156E-11,
     &   1.0348E-11, 9.5704E-12, 8.8233E-12, 8.1066E-12, 7.4202E-12,
     &   6.7642E-12, 6.1386E-12, 5.5433E-12, 4.9784E-12, 4.4438E-12,
     &   3.9396E-12, 3.4657E-12, 3.0222E-12, 2.6090E-12, 2.2262E-12,
     &   1.8738E-12, 1.8402E-12, 1.8069E-12, 1.7739E-12, 1.7413E-12,
     &   1.7089E-12, 1.6769E-12, 1.6451E-12, 1.6136E-12, 1.5825E-12,
     &   1.5517E-12, 1.5211E-12, 1.4909E-12, 1.4609E-12, 1.4313E-12,
     &   1.4020E-12, 1.3730E-12, 1.3442E-12, 1.3158E-12, 1.2877E-12/
  
         DATA ((DGT  (I,J), I= 801, 900), J=1,1) /
     &   1.2599E-12, 1.2324E-12, 1.2052E-12, 1.1783E-12, 1.1517E-12,
     &   1.1254E-12, 1.0994E-12, 1.0738E-12, 1.0484E-12, 1.0233E-12,
     &   9.9852E-13, 9.7405E-13, 9.4989E-13, 9.2603E-13, 9.0247E-13,
     &   8.7921E-13, 8.5626E-13, 8.3361E-13, 8.1127E-13, 7.8923E-13,
     &   7.6749E-13, 7.4606E-13, 7.2492E-13, 7.0410E-13, 6.8357E-13,
     &   6.6336E-13, 6.4344E-13, 6.2383E-13, 6.0452E-13, 5.8551E-13,
     &   5.6681E-13, 5.4841E-13, 5.3032E-13, 5.1253E-13, 4.9504E-13,
     &   4.7785E-13, 4.6097E-13, 4.4440E-13, 4.2812E-13, 4.1215E-13,
     &   3.9649E-13, 3.8112E-13, 3.6606E-13, 3.5131E-13, 3.3686E-13,
     &   3.2271E-13, 3.0886E-13, 2.9532E-13, 2.8208E-13, 2.6915E-13,
     &   2.5652E-13, 2.4419E-13, 2.3217E-13, 2.2045E-13, 2.0903E-13,
     &   1.9792E-13, 1.8711E-13, 1.7660E-13, 1.6640E-13, 1.5650E-13,
     &   1.4690E-13, 1.3761E-13, 1.2862E-13, 1.1994E-13, 1.1156E-13,
     &   1.0348E-13, 9.5704E-14, 8.8233E-14, 8.1066E-14, 7.4202E-14,
     &   6.7642E-14, 6.1386E-14, 5.5433E-14, 4.9784E-14, 4.4438E-14,
     &   3.9396E-14, 3.4657E-14, 3.0222E-14, 2.6090E-14, 2.2262E-14,
     &   1.8738E-14, 0.0000E+00,-1.8738E-14,-1.6875E-07,-6.7483E-07,
     &  -1.5183E-06,-2.6990E-06,-4.2180E-06,-6.0752E-06,-8.2710E-06,
     &  -1.0806E-05,-1.3681E-05,-1.6896E-05,-2.0453E-05,-2.4351E-05,
     &  -2.8593E-05,-3.3178E-05,-3.8108E-05,-4.3385E-05,-4.9009E-05/
  
         DATA ((DGT  (I,J), I= 901, 979), J=1,1) /
     &  -5.4981E-05,-6.1304E-05,-6.7979E-05,-7.5007E-05,-8.2390E-05,
     &  -9.0129E-05,-9.8228E-05,-1.0669E-04,-1.1551E-04,-1.2470E-04,
     &  -1.3425E-04,-1.4417E-04,-1.5447E-04,-1.6514E-04,-1.7619E-04,
     &  -1.8762E-04,-1.9943E-04,-2.1163E-04,-2.2422E-04,-2.3720E-04,
     &  -2.5058E-04,-2.6436E-04,-2.7855E-04,-2.9314E-04,-3.0815E-04,
     &  -3.2357E-04,-3.3941E-04,-3.5568E-04,-3.7239E-04,-3.8952E-04,
     &  -4.0710E-04,-4.2513E-04,-4.4361E-04,-4.6255E-04,-4.8195E-04,
     &  -5.0183E-04,-5.2218E-04,-5.4302E-04,-5.6435E-04,-5.8619E-04,
     &  -6.0853E-04,-6.3139E-04,-6.5478E-04,-6.7871E-04,-7.0318E-04,
     &  -7.2821E-04,-7.5380E-04,-7.7997E-04,-8.0674E-04,-8.3411E-04,
     &  -8.6210E-04,-8.9073E-04,-9.2000E-04,-9.4994E-04,-9.8057E-04,
     &  -1.0119E-03,-1.0439E-03,-1.0767E-03,-1.1103E-03,-1.1447E-03,
     &  -1.1799E-03,-1.2159E-03,-1.2529E-03,-1.2908E-03,-1.3297E-03,
     &  -1.3696E-03,-1.4106E-03,-1.4528E-03,-1.4963E-03,-1.5411E-03,
     &  -1.5875E-03,-1.6356E-03,-1.6857E-03,-1.7029E-03,-1.7383E-03,
     &  -1.75101521E-03,-1.79432626E-03,-1.80019217E-03,-1.81413516E-03/
  
         DATA ((DGT  (I,J), I=   1, 100), J=2,2) /
     &   3.6320E-04, 3.6320E-04, 3.6320E-04, 3.6320E-04, 3.6320E-04,
     &   3.6320E-04, 3.6320E-04, 3.6320E-04, 3.6320E-04, 3.6319E-04,
     &   3.6319E-04, 3.6319E-04, 3.6319E-04, 3.6319E-04, 3.6319E-04,
     &   3.6319E-04, 3.6319E-04, 3.6319E-04, 3.6319E-04, 3.6319E-04,
     &   3.6319E-04, 3.6318E-04, 3.6318E-04, 3.6318E-04, 3.6318E-04,
     &   3.6318E-04, 3.6318E-04, 3.6318E-04, 3.6318E-04, 3.6318E-04,
     &   3.6317E-04, 3.6317E-04, 3.6317E-04, 3.6317E-04, 3.6317E-04,
     &   3.6317E-04, 3.6317E-04, 3.6317E-04, 3.6316E-04, 3.6316E-04,
     &   3.6316E-04, 3.6316E-04, 3.6316E-04, 3.6316E-04, 3.6315E-04,
     &   3.6315E-04, 3.6315E-04, 3.6315E-04, 3.6315E-04, 3.6314E-04,
     &   3.6314E-04, 3.6314E-04, 3.6314E-04, 3.6314E-04, 3.6313E-04,
     &   3.6313E-04, 3.6313E-04, 3.6312E-04, 3.6312E-04, 3.6312E-04,
     &   3.6311E-04, 3.6311E-04, 3.6311E-04, 3.6310E-04, 3.6310E-04,
     &   3.6309E-04, 3.6309E-04, 3.6308E-04, 3.6308E-04, 3.6307E-04,
     &   3.6307E-04, 3.6306E-04, 3.6305E-04, 3.6304E-04, 3.6304E-04,
     &   3.6303E-04, 3.6302E-04, 3.6300E-04, 3.6299E-04, 3.6298E-04,
     &   3.6296E-04, 3.6294E-04, 3.6292E-04, 3.6289E-04, 3.6286E-04,
     &   3.6283E-04, 3.6278E-04, 3.6273E-04, 3.6267E-04, 3.6259E-04,
     &   3.6248E-04, 3.6247E-04, 3.6246E-04, 3.6245E-04, 3.6243E-04,
     &   3.6242E-04, 3.6241E-04, 3.6239E-04, 3.6238E-04, 3.6236E-04/
  
         DATA ((DGT  (I,J), I= 101, 200), J=2,2) /
     &   3.6235E-04, 3.6233E-04, 3.6231E-04, 3.6229E-04, 3.6228E-04,
     &   3.6226E-04, 3.6224E-04, 3.6222E-04, 3.6220E-04, 3.6218E-04,
     &   3.6215E-04, 3.6213E-04, 3.6211E-04, 3.6208E-04, 3.6206E-04,
     &   3.6203E-04, 3.6200E-04, 3.6198E-04, 3.6195E-04, 3.6191E-04,
     &   3.6188E-04, 3.6185E-04, 3.6181E-04, 3.6178E-04, 3.6174E-04,
     &   3.6170E-04, 3.6166E-04, 3.6161E-04, 3.6157E-04, 3.6152E-04,
     &   3.6147E-04, 3.6142E-04, 3.6136E-04, 3.6131E-04, 3.6125E-04,
     &   3.6118E-04, 3.6112E-04, 3.6104E-04, 3.6097E-04, 3.6089E-04,
     &   3.6081E-04, 3.6072E-04, 3.6063E-04, 3.6053E-04, 3.6042E-04,
     &   3.6031E-04, 3.6019E-04, 3.6007E-04, 3.5993E-04, 3.5979E-04,
     &   3.5964E-04, 3.5947E-04, 3.5930E-04, 3.5911E-04, 3.5890E-04,
     &   3.5868E-04, 3.5845E-04, 3.5819E-04, 3.5791E-04, 3.5761E-04,
     &   3.5728E-04, 3.5692E-04, 3.5653E-04, 3.5610E-04, 3.5563E-04,
     &   3.5511E-04, 3.5454E-04, 3.5390E-04, 3.5319E-04, 3.5240E-04,
     &   3.5152E-04, 3.5053E-04, 3.4941E-04, 3.4814E-04, 3.4669E-04,
     &   3.4503E-04, 3.4311E-04, 3.4088E-04, 3.3825E-04, 3.3513E-04,
     &   3.3136E-04, 3.3098E-04, 3.3059E-04, 3.3020E-04, 3.2980E-04,
     &   3.2939E-04, 3.2898E-04, 3.2855E-04, 3.2812E-04, 3.2768E-04,
     &   3.2723E-04, 3.2677E-04, 3.2630E-04, 3.2582E-04, 3.2533E-04,
     &   3.2483E-04, 3.2432E-04, 3.2380E-04, 3.2326E-04, 3.2272E-04/
  
         DATA ((DGT  (I,J), I= 201, 300), J=2,2) /
     &   3.2216E-04, 3.2159E-04, 3.2101E-04, 3.2041E-04, 3.1980E-04,
     &   3.1917E-04, 3.1853E-04, 3.1787E-04, 3.1720E-04, 3.1651E-04,
     &   3.1580E-04, 3.1507E-04, 3.1433E-04, 3.1356E-04, 3.1278E-04,
     &   3.1197E-04, 3.1114E-04, 3.1029E-04, 3.0942E-04, 3.0852E-04,
     &   3.0759E-04, 3.0664E-04, 3.0566E-04, 3.0465E-04, 3.0361E-04,
     &   3.0254E-04, 3.0144E-04, 3.0030E-04, 2.9912E-04, 2.9791E-04,
     &   2.9666E-04, 2.9536E-04, 2.9402E-04, 2.9264E-04, 2.9121E-04,
     &   2.8973E-04, 2.8820E-04, 2.8661E-04, 2.8497E-04, 2.8326E-04,
     &   2.8149E-04, 2.7965E-04, 2.7775E-04, 2.7576E-04, 2.7370E-04,
     &   2.7156E-04, 2.6933E-04, 2.6700E-04, 2.6458E-04, 2.6205E-04,
     &   2.5941E-04, 2.5666E-04, 2.5378E-04, 2.5078E-04, 2.4763E-04,
     &   2.4433E-04, 2.4088E-04, 2.3726E-04, 2.3347E-04, 2.2948E-04,
     &   2.2530E-04, 2.2090E-04, 2.1628E-04, 2.1142E-04, 2.0631E-04,
     &   2.0093E-04, 1.9526E-04, 1.8930E-04, 1.8303E-04, 1.7643E-04,
     &   1.6949E-04, 1.6220E-04, 1.5456E-04, 1.4655E-04, 1.3819E-04,
     &   1.2946E-04, 1.2040E-04, 1.1101E-04, 1.0135E-04, 9.1449E-05,
     &   8.1381E-05, 8.0367E-05, 7.9354E-05, 7.8339E-05, 7.7324E-05,
     &   7.6308E-05, 7.5292E-05, 7.4276E-05, 7.3259E-05, 7.2243E-05,
     &   7.1226E-05, 7.0210E-05, 6.9194E-05, 6.8178E-05, 6.7163E-05,
     &   6.6148E-05, 6.5134E-05, 6.4121E-05, 6.3109E-05, 6.2099E-05/
  
         DATA ((DGT  (I,J), I= 301, 400), J=2,2) /
     &   6.1089E-05, 6.0081E-05, 5.9074E-05, 5.8069E-05, 5.7066E-05,
     &   5.6065E-05, 5.5066E-05, 5.4070E-05, 5.3075E-05, 5.2084E-05,
     &   5.1095E-05, 5.0109E-05, 4.9126E-05, 4.8146E-05, 4.7170E-05,
     &   4.6197E-05, 4.5228E-05, 4.4263E-05, 4.3302E-05, 4.2346E-05,
     &   4.1394E-05, 4.0447E-05, 3.9504E-05, 3.8567E-05, 3.7635E-05,
     &   3.6708E-05, 3.5787E-05, 3.4872E-05, 3.3963E-05, 3.3061E-05,
     &   3.2165E-05, 3.1275E-05, 3.0393E-05, 2.9518E-05, 2.8650E-05,
     &   2.7790E-05, 2.6938E-05, 2.6094E-05, 2.5259E-05, 2.4432E-05,
     &   2.3614E-05, 2.2805E-05, 2.2006E-05, 2.1216E-05, 2.0436E-05,
     &   1.9666E-05, 1.8907E-05, 1.8158E-05, 1.7420E-05, 1.6694E-05,
     &   1.5979E-05, 1.5276E-05, 1.4585E-05, 1.3906E-05, 1.3240E-05,
     &   1.2587E-05, 1.1947E-05, 1.1320E-05, 1.0708E-05, 1.0109E-05,
     &   9.5245E-06, 8.9548E-06, 8.4000E-06, 7.8604E-06, 7.3363E-06,
     &   6.8280E-06, 6.3358E-06, 5.8599E-06, 5.4006E-06, 4.9583E-06,
     &   4.5332E-06, 4.1255E-06, 3.7356E-06, 3.3636E-06, 3.0100E-06,
     &   2.6748E-06, 2.3584E-06, 2.0610E-06, 1.7829E-06, 1.5242E-06,
     &   1.2851E-06, 1.2623E-06, 1.2397E-06, 1.2173E-06, 1.1951E-06,
     &   1.1730E-06, 1.1512E-06, 1.1296E-06, 1.1082E-06, 1.0870E-06,
     &   1.0659E-06, 1.0451E-06, 1.0245E-06, 1.0041E-06, 9.8388E-07,
     &   9.6386E-07, 9.4405E-07, 9.2444E-07, 9.0503E-07, 8.8582E-07/
  
         DATA ((DGT  (I,J), I= 401, 500), J=2,2) /
     &   8.6681E-07, 8.4801E-07, 8.2941E-07, 8.1101E-07, 7.9281E-07,
     &   7.7482E-07, 7.5703E-07, 7.3944E-07, 7.2206E-07, 7.0488E-07,
     &   6.8790E-07, 6.7113E-07, 6.5456E-07, 6.3819E-07, 6.2203E-07,
     &   6.0608E-07, 5.9033E-07, 5.7478E-07, 5.5944E-07, 5.4430E-07,
     &   5.2937E-07, 5.1464E-07, 5.0012E-07, 4.8581E-07, 4.7170E-07,
     &   4.5779E-07, 4.4409E-07, 4.3060E-07, 4.1731E-07, 4.0423E-07,
     &   3.9136E-07, 3.7869E-07, 3.6623E-07, 3.5398E-07, 3.4193E-07,
     &   3.3009E-07, 3.1846E-07, 3.0703E-07, 2.9582E-07, 2.8481E-07,
     &   2.7400E-07, 2.6341E-07, 2.5302E-07, 2.4284E-07, 2.3287E-07,
     &   2.2310E-07, 2.1355E-07, 2.0420E-07, 1.9506E-07, 1.8613E-07,
     &   1.7740E-07, 1.6889E-07, 1.6058E-07, 1.5249E-07, 1.4460E-07,
     &   1.3692E-07, 1.2945E-07, 1.2218E-07, 1.1513E-07, 1.0829E-07,
     &   1.0165E-07, 9.5229E-08, 8.9013E-08, 8.3006E-08, 7.7209E-08,
     &   7.1622E-08, 6.6243E-08, 6.1075E-08, 5.6116E-08, 5.1367E-08,
     &   4.6827E-08, 4.2497E-08, 3.8377E-08, 3.4467E-08, 3.0767E-08,
     &   2.7277E-08, 2.3996E-08, 2.0926E-08, 1.8066E-08, 1.5415E-08,
     &   1.2975E-08, 1.2742E-08, 1.2512E-08, 1.2284E-08, 1.2058E-08,
     &   1.1834E-08, 1.1612E-08, 1.1392E-08, 1.1174E-08, 1.0958E-08,
     &   1.0745E-08, 1.0533E-08, 1.0324E-08, 1.0117E-08, 9.9114E-09,
     &   9.7084E-09, 9.5074E-09, 9.3086E-09, 9.1118E-09, 8.9171E-09/
  
         DATA ((DGT  (I,J), I= 501, 600), J=2,2) /
     &   8.7246E-09, 8.5341E-09, 8.3458E-09, 8.1595E-09, 7.9754E-09,
     &   7.7933E-09, 7.6134E-09, 7.4356E-09, 7.2598E-09, 7.0862E-09,
     &   6.9146E-09, 6.7452E-09, 6.5779E-09, 6.4126E-09, 6.2495E-09,
     &   6.0885E-09, 5.9295E-09, 5.7727E-09, 5.6180E-09, 5.4653E-09,
     &   5.3148E-09, 5.1664E-09, 5.0201E-09, 4.8759E-09, 4.7337E-09,
     &   4.5937E-09, 4.4558E-09, 4.3200E-09, 4.1863E-09, 4.0547E-09,
     &   3.9252E-09, 3.7978E-09, 3.6725E-09, 3.5493E-09, 3.4282E-09,
     &   3.3092E-09, 3.1923E-09, 3.0775E-09, 2.9648E-09, 2.8542E-09,
     &   2.7457E-09, 2.6393E-09, 2.5350E-09, 2.4328E-09, 2.3328E-09,
     &   2.2348E-09, 2.1389E-09, 2.0451E-09, 1.9535E-09, 1.8639E-09,
     &   1.7764E-09, 1.6910E-09, 1.6078E-09, 1.5266E-09, 1.4476E-09,
     &   1.3706E-09, 1.2957E-09, 1.2230E-09, 1.1523E-09, 1.0838E-09,
     &   1.0173E-09, 9.5298E-10, 8.9073E-10, 8.3059E-10, 7.7254E-10,
     &   7.1661E-10, 6.6277E-10, 6.1103E-10, 5.6140E-10, 5.1387E-10,
     &   4.6844E-10, 4.2511E-10, 3.8389E-10, 3.4476E-10, 3.0774E-10,
     &   2.7282E-10, 2.4001E-10, 2.0929E-10, 1.8068E-10, 1.5417E-10,
     &   1.2976E-10, 1.2744E-10, 1.2513E-10, 1.2285E-10, 1.2059E-10,
     &   1.1835E-10, 1.1613E-10, 1.1393E-10, 1.1175E-10, 1.0959E-10,
     &   1.0746E-10, 1.0534E-10, 1.0325E-10, 1.0117E-10, 9.9122E-11,
     &   9.7091E-11, 9.5081E-11, 9.3092E-11, 9.1124E-11, 8.9178E-11/
  
         DATA ((DGT  (I,J), I= 601, 700), J=2,2) /
     &   8.7252E-11, 8.5347E-11, 8.3463E-11, 8.1600E-11, 7.9759E-11,
     &   7.7938E-11, 7.6138E-11, 7.4360E-11, 7.2602E-11, 7.0866E-11,
     &   6.9150E-11, 6.7455E-11, 6.5782E-11, 6.4129E-11, 6.2498E-11,
     &   6.0887E-11, 5.9298E-11, 5.7730E-11, 5.6182E-11, 5.4656E-11,
     &   5.3150E-11, 5.1666E-11, 5.0203E-11, 4.8760E-11, 4.7339E-11,
     &   4.5939E-11, 4.4560E-11, 4.3201E-11, 4.1864E-11, 4.0548E-11,
     &   3.9253E-11, 3.7979E-11, 3.6726E-11, 3.5494E-11, 3.4283E-11,
     &   3.3092E-11, 3.1923E-11, 3.0775E-11, 2.9648E-11, 2.8543E-11,
     &   2.7458E-11, 2.6394E-11, 2.5351E-11, 2.4329E-11, 2.3328E-11,
     &   2.2348E-11, 2.1389E-11, 2.0452E-11, 1.9535E-11, 1.8639E-11,
     &   1.7764E-11, 1.6911E-11, 1.6078E-11, 1.5266E-11, 1.4476E-11,
     &   1.3706E-11, 1.2957E-11, 1.2230E-11, 1.1523E-11, 1.0838E-11,
     &   1.0173E-11, 9.5298E-12, 8.9074E-12, 8.3059E-12, 7.7255E-12,
     &   7.1661E-12, 6.6277E-12, 6.1103E-12, 5.6140E-12, 5.1387E-12,
     &   4.6844E-12, 4.2511E-12, 3.8389E-12, 3.4476E-12, 3.0774E-12,
     &   2.7282E-12, 2.4001E-12, 2.0929E-12, 1.8068E-12, 1.5417E-12,
     &   1.2976E-12, 1.2744E-12, 1.2513E-12, 1.2285E-12, 1.2059E-12,
     &   1.1835E-12, 1.1613E-12, 1.1393E-12, 1.1175E-12, 1.0959E-12,
     &   1.0746E-12, 1.0534E-12, 1.0325E-12, 1.0117E-12, 9.9122E-13,
     &   9.7091E-13, 9.5081E-13, 9.3092E-13, 9.1124E-13, 8.9178E-13/
  
         DATA ((DGT  (I,J), I= 701, 800), J=2,2) /
     &   8.7252E-13, 8.5347E-13, 8.3463E-13, 8.1601E-13, 7.9759E-13,
     &   7.7938E-13, 7.6138E-13, 7.4360E-13, 7.2602E-13, 7.0866E-13,
     &   6.9150E-13, 6.7455E-13, 6.5782E-13, 6.4129E-13, 6.2498E-13,
     &   6.0887E-13, 5.9298E-13, 5.7730E-13, 5.6182E-13, 5.4656E-13,
     &   5.3150E-13, 5.1666E-13, 5.0203E-13, 4.8760E-13, 4.7339E-13,
     &   4.5939E-13, 4.4560E-13, 4.3201E-13, 4.1864E-13, 4.0548E-13,
     &   3.9253E-13, 3.7979E-13, 3.6726E-13, 3.5494E-13, 3.4283E-13,
     &   3.3092E-13, 3.1923E-13, 3.0775E-13, 2.9648E-13, 2.8543E-13,
     &   2.7458E-13, 2.6394E-13, 2.5351E-13, 2.4329E-13, 2.3328E-13,
     &   2.2348E-13, 2.1389E-13, 2.0452E-13, 1.9535E-13, 1.8639E-13,
     &   1.7764E-13, 1.6911E-13, 1.6078E-13, 1.5266E-13, 1.4476E-13,
     &   1.3706E-13, 1.2957E-13, 1.2230E-13, 1.1523E-13, 1.0838E-13,
     &   1.0173E-13, 9.5298E-14, 8.9074E-14, 8.3059E-14, 7.7255E-14,
     &   7.1661E-14, 6.6277E-14, 6.1103E-14, 5.6140E-14, 5.1387E-14,
     &   4.6844E-14, 4.2511E-14, 3.8389E-14, 3.4476E-14, 3.0774E-14,
     &   2.7282E-14, 2.4001E-14, 2.0929E-14, 1.8068E-14, 1.5417E-14,
     &   1.2976E-14, 1.2744E-14, 1.2513E-14, 1.2285E-14, 1.2059E-14,
     &   1.1835E-14, 1.1613E-14, 1.1393E-14, 1.1175E-14, 1.0959E-14,
     &   1.0746E-14, 1.0534E-14, 1.0325E-14, 1.0117E-14, 9.9122E-15,
     &   9.7091E-15, 9.5081E-15, 9.3092E-15, 9.1124E-15, 8.9178E-15/
  
         DATA ((DGT  (I,J), I= 801, 900), J=2,2) /
     &   8.7252E-15, 8.5347E-15, 8.3463E-15, 8.1601E-15, 7.9759E-15,
     &   7.7938E-15, 7.6138E-15, 7.4360E-15, 7.2602E-15, 7.0866E-15,
     &   6.9150E-15, 6.7455E-15, 6.5782E-15, 6.4129E-15, 6.2498E-15,
     &   6.0887E-15, 5.9298E-15, 5.7730E-15, 5.6182E-15, 5.4656E-15,
     &   5.3150E-15, 5.1666E-15, 5.0203E-15, 4.8760E-15, 4.7339E-15,
     &   4.5939E-15, 4.4560E-15, 4.3201E-15, 4.1864E-15, 4.0548E-15,
     &   3.9253E-15, 3.7979E-15, 3.6726E-15, 3.5494E-15, 3.4283E-15,
     &   3.3092E-15, 3.1923E-15, 3.0775E-15, 2.9648E-15, 2.8543E-15,
     &   2.7458E-15, 2.6394E-15, 2.5351E-15, 2.4329E-15, 2.3328E-15,
     &   2.2348E-15, 2.1389E-15, 2.0452E-15, 1.9535E-15, 1.8639E-15,
     &   1.7764E-15, 1.6911E-15, 1.6078E-15, 1.5266E-15, 1.4476E-15,
     &   1.3706E-15, 1.2957E-15, 1.2230E-15, 1.1523E-15, 1.0838E-15,
     &   1.0173E-15, 9.5298E-16, 8.9074E-16, 8.3059E-16, 7.7255E-16,
     &   7.1661E-16, 6.6277E-16, 6.1103E-16, 5.6140E-16, 5.1387E-16,
     &   4.6844E-16, 4.2511E-16, 3.8389E-16, 3.4476E-16, 3.0774E-16,
     &   2.7282E-16, 2.4001E-16, 2.0929E-16, 1.8068E-16, 1.5417E-16,
     &   1.2976E-16, 0.0000E+00, 1.2976E-16, 1.1686E-09, 4.6731E-09,
     &   1.0514E-08, 1.8688E-08, 2.9203E-08, 4.2056E-08, 5.7250E-08,
     &   7.4786E-08, 9.4665E-08, 1.1689E-07, 1.4146E-07, 1.6839E-07,
     &   1.9767E-07, 2.2930E-07, 2.6330E-07, 2.9966E-07, 3.3839E-07/
  
         DATA ((DGT  (I,J), I= 901, 979), J=2,2) /
     &   3.7949E-07, 4.2297E-07, 4.6884E-07, 5.1708E-07, 5.6773E-07,
     &   6.2077E-07, 6.7622E-07, 7.3407E-07, 7.9435E-07, 8.5705E-07,
     &   9.2218E-07, 9.8976E-07, 1.0598E-06, 1.1323E-06, 1.2072E-06,
     &   1.2846E-06, 1.3645E-06, 1.4469E-06, 1.5319E-06, 1.6193E-06,
     &   1.7092E-06, 1.8017E-06, 1.8968E-06, 1.9944E-06, 2.0946E-06,
     &   2.1974E-06, 2.3028E-06, 2.4108E-06, 2.5215E-06, 2.6348E-06,
     &   2.7508E-06, 2.8695E-06, 2.9909E-06, 3.1151E-06, 3.2420E-06,
     &   3.3717E-06, 3.5042E-06, 3.6395E-06, 3.7776E-06, 3.9187E-06,
     &   4.0626E-06, 4.2095E-06, 4.3594E-06, 4.5122E-06, 4.6681E-06,
     &   4.8271E-06, 4.9892E-06, 5.1545E-06, 5.3229E-06, 5.4947E-06,
     &   5.6697E-06, 5.8481E-06, 6.0300E-06, 6.2153E-06, 6.4042E-06,
     &   6.5968E-06, 6.7931E-06, 6.9933E-06, 7.1974E-06, 7.4056E-06,
     &   7.6180E-06, 7.8348E-06, 8.0562E-06, 8.2823E-06, 8.5134E-06,
     &   8.7499E-06, 8.9920E-06, 9.2402E-06, 9.4951E-06, 9.7574E-06,
     &   1.0028E-05, 1.0308E-05, 1.0601E-05, 1.0702E-05, 1.0909E-05,
     &   1.09836974E-05, 1.12405441E-05, 1.12756708E-05, 1.13596672E-05/


c      ++++ dalhadthigh15 ++++  

c update may 2016 
c 
c******************************************************************************
c*                                                                            *
c*    --- intRdat1 ---                              Sat May 21 16:43:16 2016  *
c*                                                                            *
c***************************# FJ@HU Berlin #***********************************
c 
c  "Delta alpha"               Update version (29/09/2015) 
c 
c ISCAN: -2, Ismooth: 0, IGS: 0, DSET: WAVE, iplot: 0
c IKE,Emin,Emax,NP,N,EPSREL:  1,-2.0000E+04,-1.0000E+03, 100, 10, 1.00E-02, 
c ALS(sta)(sys):  0.1184( 0.0007)( 0.0000), MZINP:   91.1876, MTOP:    173.20
c NF,IOR,ICHK: 5,4,2,ISU2: 0,NFL: 5,INAG: 1,inotail:  0, sin2W:   0.23153
c CHPTCUT,EC,ECUT: 0.3180,     5.20,    11.50
c 
  
       DATA  (XXX  (I), I=   1, 100) /
     &  -2.0000E+04,-1.9701E+04,-1.9407E+04,-1.9117E+04,-1.8831E+04,
     &  -1.8550E+04,-1.8273E+04,-1.8000E+04,-1.7731E+04,-1.7466E+04,
     &  -1.7205E+04,-1.6948E+04,-1.6695E+04,-1.6445E+04,-1.6199E+04,
     &  -1.5957E+04,-1.5719E+04,-1.5484E+04,-1.5253E+04,-1.5025E+04,
     &  -1.4800E+04,-1.4579E+04,-1.4361E+04,-1.4147E+04,-1.3935E+04,
     &  -1.3727E+04,-1.3522E+04,-1.3320E+04,-1.3121E+04,-1.2925E+04,
     &  -1.2732E+04,-1.2542E+04,-1.2354E+04,-1.2170E+04,-1.1988E+04,
     &  -1.1809E+04,-1.1632E+04,-1.1459E+04,-1.1287E+04,-1.1119E+04,
     &  -1.0953E+04,-1.0789E+04,-1.0628E+04,-1.0469E+04,-1.0313E+04,
     &  -1.0158E+04,-1.0007E+04,-9.8572E+03,-9.7099E+03,-9.5648E+03,
     &  -9.4219E+03,-9.2811E+03,-9.1424E+03,-9.0058E+03,-8.8713E+03,
     &  -8.7387E+03,-8.6082E+03,-8.4796E+03,-8.3529E+03,-8.2281E+03,
     &  -8.1051E+03,-7.9840E+03,-7.8647E+03,-7.7472E+03,-7.6315E+03,
     &  -7.5174E+03,-7.4051E+03,-7.2945E+03,-7.1855E+03,-7.0781E+03,
     &  -6.9724E+03,-6.8682E+03,-6.7656E+03,-6.6645E+03,-6.5649E+03,
     &  -6.4668E+03,-6.3702E+03,-6.2750E+03,-6.1813E+03,-6.0889E+03,
     &  -5.9980E+03,-5.9083E+03,-5.8201E+03,-5.7331E+03,-5.6474E+03,
     &  -5.5631E+03,-5.4799E+03,-5.3981E+03,-5.3174E+03,-5.2380E+03,
     &  -5.1597E+03,-5.0826E+03,-5.0067E+03,-4.9319E+03,-4.8582E+03,
     &  -4.7856E+03,-4.7141E+03,-4.6437E+03,-4.5743E+03,-4.5059E+03/
  
       DATA  (XXX  (I), I= 101, 200) /
     &  -4.4386E+03,-4.3723E+03,-4.3070E+03,-4.2426E+03,-4.1792E+03,
     &  -4.1168E+03,-4.0553E+03,-3.9947E+03,-3.9350E+03,-3.8762E+03,
     &  -3.8183E+03,-3.7612E+03,-3.7050E+03,-3.6497E+03,-3.5951E+03,
     &  -3.5414E+03,-3.4885E+03,-3.4364E+03,-3.3850E+03,-3.3345E+03,
     &  -3.2847E+03,-3.2356E+03,-3.1872E+03,-3.1396E+03,-3.0927E+03,
     &  -3.0465E+03,-3.0010E+03,-2.9561E+03,-2.9120E+03,-2.8685E+03,
     &  -2.8256E+03,-2.7834E+03,-2.7418E+03,-2.7008E+03,-2.6605E+03,
     &  -2.6207E+03,-2.5816E+03,-2.5430E+03,-2.5050E+03,-2.4676E+03,
     &  -2.4307E+03,-2.3944E+03,-2.3586E+03,-2.3234E+03,-2.2887E+03,
     &  -2.2545E+03,-2.2208E+03,-2.1876E+03,-2.1549E+03,-2.1227E+03,
     &  -2.0910E+03,-2.0598E+03,-2.0290E+03,-1.9987E+03,-1.9688E+03,
     &  -1.9394E+03,-1.9104E+03,-1.8819E+03,-1.8538E+03,-1.8261E+03,
     &  -1.7988E+03,-1.7719E+03,-1.7454E+03,-1.7193E+03,-1.6937E+03,
     &  -1.6683E+03,-1.6434E+03,-1.6189E+03,-1.5947E+03,-1.5709E+03,
     &  -1.5474E+03,-1.5243E+03,-1.5015E+03,-1.4791E+03,-1.4570E+03,
     &  -1.4352E+03,-1.4137E+03,-1.3926E+03,-1.3718E+03,-1.3513E+03,
     &  -1.3311E+03,-1.3112E+03,-1.2916E+03,-1.2723E+03,-1.2533E+03,
     &  -1.2346E+03,-1.2162E+03,-1.1980E+03,-1.1801E+03,-1.1625E+03,
     &  -1.1451E+03,-1.1280E+03,-1.1111E+03,-1.0945E+03,-1.0782E+03,
     &  -1.0621E+03,-1.0462E+03,-1.0306E+03,-1.0152E+03,-1.0000E+03/
  
  
         DATA ((YGG  (I,J), I=   1, 100), J=1,1) /
     &   5.7039E-02, 5.6953E-02, 5.6866E-02, 5.6780E-02, 5.6693E-02,
     &   5.6607E-02, 5.6521E-02, 5.6434E-02, 5.6348E-02, 5.6262E-02,
     &   5.6175E-02, 5.6089E-02, 5.6003E-02, 5.5917E-02, 5.5830E-02,
     &   5.5744E-02, 5.5658E-02, 5.5572E-02, 5.5486E-02, 5.5400E-02,
     &   5.5314E-02, 5.5228E-02, 5.5142E-02, 5.5056E-02, 5.4970E-02,
     &   5.4884E-02, 5.4799E-02, 5.4713E-02, 5.4627E-02, 5.4541E-02,
     &   5.4456E-02, 5.4370E-02, 5.4284E-02, 5.4199E-02, 5.4113E-02,
     &   5.4028E-02, 5.3942E-02, 5.3857E-02, 5.3772E-02, 5.3686E-02,
     &   5.3601E-02, 5.3516E-02, 5.3431E-02, 5.3346E-02, 5.3261E-02,
     &   5.3176E-02, 5.3091E-02, 5.3006E-02, 5.2921E-02, 5.2836E-02,
     &   5.2751E-02, 5.2667E-02, 5.2582E-02, 5.2497E-02, 5.2413E-02,
     &   5.2328E-02, 5.2244E-02, 5.2160E-02, 5.2075E-02, 5.1991E-02,
     &   5.1907E-02, 5.1823E-02, 5.1739E-02, 5.1655E-02, 5.1571E-02,
     &   5.1487E-02, 5.1404E-02, 5.1320E-02, 5.1237E-02, 5.1153E-02,
     &   5.1070E-02, 5.0986E-02, 5.0903E-02, 5.0820E-02, 5.0737E-02,
     &   5.0654E-02, 5.0571E-02, 5.0488E-02, 5.0405E-02, 5.0322E-02,
     &   5.0240E-02, 5.0157E-02, 5.0075E-02, 4.9992E-02, 4.9910E-02,
     &   4.9828E-02, 4.9746E-02, 4.9664E-02, 4.9582E-02, 4.9500E-02,
     &   4.9419E-02, 4.9337E-02, 4.9256E-02, 4.9174E-02, 4.9093E-02,
     &   4.9012E-02, 4.8931E-02, 4.8850E-02, 4.8769E-02, 4.8688E-02/
  
         DATA ((YGG  (I,J), I= 101, 200), J=1,1) /
     &   4.8607E-02, 4.8527E-02, 4.8446E-02, 4.8366E-02, 4.8286E-02,
     &   4.8206E-02, 4.8126E-02, 4.8046E-02, 4.7966E-02, 4.7886E-02,
     &   4.7807E-02, 4.7727E-02, 4.7648E-02, 4.7568E-02, 4.7489E-02,
     &   4.7410E-02, 4.7331E-02, 4.7252E-02, 4.7174E-02, 4.7095E-02,
     &   4.7017E-02, 4.6938E-02, 4.6860E-02, 4.6782E-02, 4.6704E-02,
     &   4.6626E-02, 4.6548E-02, 4.6470E-02, 4.6393E-02, 4.6315E-02,
     &   4.6238E-02, 4.6160E-02, 4.6083E-02, 4.6006E-02, 4.5929E-02,
     &   4.5852E-02, 4.5775E-02, 4.5698E-02, 4.5622E-02, 4.5545E-02,
     &   4.5469E-02, 4.5392E-02, 4.5316E-02, 4.5240E-02, 4.5164E-02,
     &   4.5088E-02, 4.5012E-02, 4.4936E-02, 4.4860E-02, 4.4784E-02,
     &   4.4708E-02, 4.4633E-02, 4.4557E-02, 4.4482E-02, 4.4406E-02,
     &   4.4331E-02, 4.4255E-02, 4.4180E-02, 4.4105E-02, 4.4029E-02,
     &   4.3954E-02, 4.3879E-02, 4.3804E-02, 4.3728E-02, 4.3653E-02,
     &   4.3578E-02, 4.3503E-02, 4.3428E-02, 4.3353E-02, 4.3278E-02,
     &   4.3203E-02, 4.3127E-02, 4.3052E-02, 4.2977E-02, 4.2902E-02,
     &   4.2827E-02, 4.2751E-02, 4.2676E-02, 4.2601E-02, 4.2525E-02,
     &   4.2450E-02, 4.2374E-02, 4.2299E-02, 4.2223E-02, 4.2148E-02,
     &   4.2072E-02, 4.1996E-02, 4.1920E-02, 4.1844E-02, 4.1768E-02,
     &   4.1692E-02, 4.1616E-02, 4.1540E-02, 4.1463E-02, 4.1387E-02,
     &   4.1310E-02, 4.1234E-02, 4.1157E-02, 4.1080E-02, 4.1003E-02/
  
  
         DATA ((YGG  (I,J), I=   1, 100), J=2,2) /
     &   1.5644E-04, 1.5644E-04, 1.5644E-04, 1.5644E-04, 1.5644E-04,
     &   1.5644E-04, 1.5644E-04, 1.5644E-04, 1.5643E-04, 1.5643E-04,
     &   1.5643E-04, 1.5643E-04, 1.5643E-04, 1.5643E-04, 1.5643E-04,
     &   1.5643E-04, 1.5643E-04, 1.5643E-04, 1.5643E-04, 1.5643E-04,
     &   1.5643E-04, 1.5643E-04, 1.5643E-04, 1.5643E-04, 1.5643E-04,
     &   1.5643E-04, 1.5643E-04, 1.5643E-04, 1.5643E-04, 1.5643E-04,
     &   1.5643E-04, 1.5643E-04, 1.5643E-04, 1.5643E-04, 1.5643E-04,
     &   1.5643E-04, 1.5643E-04, 1.5643E-04, 1.5642E-04, 1.5642E-04,
     &   1.5642E-04, 1.5642E-04, 1.5642E-04, 1.5642E-04, 1.5642E-04,
     &   1.5642E-04, 1.5642E-04, 1.5642E-04, 1.5642E-04, 1.5642E-04,
     &   1.5642E-04, 1.5642E-04, 1.5642E-04, 1.5642E-04, 1.5642E-04,
     &   1.5642E-04, 1.5642E-04, 1.5642E-04, 1.5642E-04, 1.5642E-04,
     &   1.5642E-04, 1.5642E-04, 1.5642E-04, 1.5642E-04, 1.5642E-04,
     &   1.5642E-04, 1.5641E-04, 1.5641E-04, 1.5641E-04, 1.5641E-04,
     &   1.5641E-04, 1.5641E-04, 1.5641E-04, 1.5641E-04, 1.5641E-04,
     &   1.5641E-04, 1.5641E-04, 1.5641E-04, 1.5641E-04, 1.5641E-04,
     &   1.5641E-04, 1.5641E-04, 1.5641E-04, 1.5641E-04, 1.5641E-04,
     &   1.5641E-04, 1.5641E-04, 1.5641E-04, 1.5641E-04, 1.5641E-04,
     &   1.5641E-04, 1.5641E-04, 1.5641E-04, 1.5641E-04, 1.5640E-04,
     &   1.5640E-04, 1.5640E-04, 1.5640E-04, 1.5640E-04, 1.5640E-04/
  
         DATA ((YGG  (I,J), I= 101, 200), J=2,2) /
     &   1.5640E-04, 1.5640E-04, 1.5640E-04, 1.5640E-04, 1.5640E-04,
     &   1.5640E-04, 1.5640E-04, 1.5640E-04, 1.5640E-04, 1.5640E-04,
     &   1.5640E-04, 1.5640E-04, 1.5640E-04, 1.5640E-04, 1.5640E-04,
     &   1.5640E-04, 1.5640E-04, 1.5640E-04, 1.5640E-04, 1.5640E-04,
     &   1.5640E-04, 1.5640E-04, 1.5639E-04, 1.5639E-04, 1.5639E-04,
     &   1.5639E-04, 1.5639E-04, 1.5639E-04, 1.5639E-04, 1.5639E-04,
     &   1.5639E-04, 1.5639E-04, 1.5639E-04, 1.5639E-04, 1.5639E-04,
     &   1.5639E-04, 1.5639E-04, 1.5639E-04, 1.5639E-04, 1.5639E-04,
     &   1.5639E-04, 1.5639E-04, 1.5639E-04, 1.5639E-04, 1.5639E-04,
     &   1.5639E-04, 1.5639E-04, 1.5639E-04, 1.5639E-04, 1.5638E-04,
     &   1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5638E-04,
     &   1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5638E-04,
     &   1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5638E-04,
     &   1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5638E-04,
     &   1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5638E-04, 1.5637E-04,
     &   1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04,
     &   1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04,
     &   1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04,
     &   1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04, 1.5637E-04,
     &   1.5637E-04, 1.5637E-04, 1.5636E-04, 1.5636E-04, 1.5636E-04/
  

c      ++++ deghadthigh15 ++++  

c update may 2016 
c 
c******************************************************************************
c*                                                                            *
c*    --- intRdat1 ---                              Sat May 21 19:24:44 2016  *
c*                                                                            *
c***************************# FJ@HU Berlin #***********************************
c 
c  "Delta alpha"               Update version (29/09/2015) 
c 
c ISCAN: -2, Ismooth: 0, IGS: 0, DSET: WAVE, iplot: 0
c IKE,Emin,Emax,NP,N,EPSREL:  1,-2.0000E+04,-1.0000E+03, 100, 10, 1.00E-02, 
c ALS(sta)(sys):  0.1184( 0.0007)( 0.0000), MZINP:   91.1876, MTOP:    173.20
c NF,IOR,ICHK: 5,4,2,ISU2: 2,NFL: 5,INAG: 1,inotail:  0, sin2W:   0.23153
c CHPTCUT,EC,ECUT: 0.3180,     5.20,    11.50
c 
  
       DATA  (XXX  (I), I=   1, 100) /
     &  -2.0000E+04,-1.9701E+04,-1.9407E+04,-1.9117E+04,-1.8831E+04,
     &  -1.8550E+04,-1.8273E+04,-1.8000E+04,-1.7731E+04,-1.7466E+04,
     &  -1.7205E+04,-1.6948E+04,-1.6695E+04,-1.6445E+04,-1.6199E+04,
     &  -1.5957E+04,-1.5719E+04,-1.5484E+04,-1.5253E+04,-1.5025E+04,
     &  -1.4800E+04,-1.4579E+04,-1.4361E+04,-1.4147E+04,-1.3935E+04,
     &  -1.3727E+04,-1.3522E+04,-1.3320E+04,-1.3121E+04,-1.2925E+04,
     &  -1.2732E+04,-1.2542E+04,-1.2354E+04,-1.2170E+04,-1.1988E+04,
     &  -1.1809E+04,-1.1632E+04,-1.1459E+04,-1.1287E+04,-1.1119E+04,
     &  -1.0953E+04,-1.0789E+04,-1.0628E+04,-1.0469E+04,-1.0313E+04,
     &  -1.0158E+04,-1.0007E+04,-9.8572E+03,-9.7099E+03,-9.5648E+03,
     &  -9.4219E+03,-9.2811E+03,-9.1424E+03,-9.0058E+03,-8.8713E+03,
     &  -8.7387E+03,-8.6082E+03,-8.4796E+03,-8.3529E+03,-8.2281E+03,
     &  -8.1051E+03,-7.9840E+03,-7.8647E+03,-7.7472E+03,-7.6315E+03,
     &  -7.5174E+03,-7.4051E+03,-7.2945E+03,-7.1855E+03,-7.0781E+03,
     &  -6.9724E+03,-6.8682E+03,-6.7656E+03,-6.6645E+03,-6.5649E+03,
     &  -6.4668E+03,-6.3702E+03,-6.2750E+03,-6.1813E+03,-6.0889E+03,
     &  -5.9980E+03,-5.9083E+03,-5.8201E+03,-5.7331E+03,-5.6474E+03,
     &  -5.5631E+03,-5.4799E+03,-5.3981E+03,-5.3174E+03,-5.2380E+03,
     &  -5.1597E+03,-5.0826E+03,-5.0067E+03,-4.9319E+03,-4.8582E+03,
     &  -4.7856E+03,-4.7141E+03,-4.6437E+03,-4.5743E+03,-4.5059E+03/
  
       DATA  (XXX  (I), I= 101, 200) /
     &  -4.4386E+03,-4.3723E+03,-4.3070E+03,-4.2426E+03,-4.1792E+03,
     &  -4.1168E+03,-4.0553E+03,-3.9947E+03,-3.9350E+03,-3.8762E+03,
     &  -3.8183E+03,-3.7612E+03,-3.7050E+03,-3.6497E+03,-3.5951E+03,
     &  -3.5414E+03,-3.4885E+03,-3.4364E+03,-3.3850E+03,-3.3345E+03,
     &  -3.2847E+03,-3.2356E+03,-3.1872E+03,-3.1396E+03,-3.0927E+03,
     &  -3.0465E+03,-3.0010E+03,-2.9561E+03,-2.9120E+03,-2.8685E+03,
     &  -2.8256E+03,-2.7834E+03,-2.7418E+03,-2.7008E+03,-2.6605E+03,
     &  -2.6207E+03,-2.5816E+03,-2.5430E+03,-2.5050E+03,-2.4676E+03,
     &  -2.4307E+03,-2.3944E+03,-2.3586E+03,-2.3234E+03,-2.2887E+03,
     &  -2.2545E+03,-2.2208E+03,-2.1876E+03,-2.1549E+03,-2.1227E+03,
     &  -2.0910E+03,-2.0598E+03,-2.0290E+03,-1.9987E+03,-1.9688E+03,
     &  -1.9394E+03,-1.9104E+03,-1.8819E+03,-1.8538E+03,-1.8261E+03,
     &  -1.7988E+03,-1.7719E+03,-1.7454E+03,-1.7193E+03,-1.6937E+03,
     &  -1.6683E+03,-1.6434E+03,-1.6189E+03,-1.5947E+03,-1.5709E+03,
     &  -1.5474E+03,-1.5243E+03,-1.5015E+03,-1.4791E+03,-1.4570E+03,
     &  -1.4352E+03,-1.4137E+03,-1.3926E+03,-1.3718E+03,-1.3513E+03,
     &  -1.3311E+03,-1.3112E+03,-1.2916E+03,-1.2723E+03,-1.2533E+03,
     &  -1.2346E+03,-1.2162E+03,-1.1980E+03,-1.1801E+03,-1.1625E+03,
     &  -1.1451E+03,-1.1280E+03,-1.1111E+03,-1.0945E+03,-1.0782E+03,
     &  -1.0621E+03,-1.0462E+03,-1.0306E+03,-1.0152E+03,-1.0000E+03/
  
  
         DATA ((YGG  (I,J), I=   1, 100), J=1,1) /
     &   1.1683E-01, 1.1665E-01, 1.1647E-01, 1.1630E-01, 1.1612E-01,
     &   1.1594E-01, 1.1576E-01, 1.1558E-01, 1.1541E-01, 1.1523E-01,
     &   1.1505E-01, 1.1487E-01, 1.1469E-01, 1.1452E-01, 1.1434E-01,
     &   1.1416E-01, 1.1398E-01, 1.1381E-01, 1.1363E-01, 1.1345E-01,
     &   1.1327E-01, 1.1310E-01, 1.1292E-01, 1.1274E-01, 1.1257E-01,
     &   1.1239E-01, 1.1221E-01, 1.1203E-01, 1.1186E-01, 1.1168E-01,
     &   1.1150E-01, 1.1133E-01, 1.1115E-01, 1.1098E-01, 1.1080E-01,
     &   1.1062E-01, 1.1045E-01, 1.1027E-01, 1.1009E-01, 1.0992E-01,
     &   1.0974E-01, 1.0957E-01, 1.0939E-01, 1.0922E-01, 1.0904E-01,
     &   1.0887E-01, 1.0869E-01, 1.0852E-01, 1.0834E-01, 1.0817E-01,
     &   1.0799E-01, 1.0782E-01, 1.0764E-01, 1.0747E-01, 1.0729E-01,
     &   1.0712E-01, 1.0695E-01, 1.0677E-01, 1.0660E-01, 1.0642E-01,
     &   1.0625E-01, 1.0608E-01, 1.0590E-01, 1.0573E-01, 1.0556E-01,
     &   1.0539E-01, 1.0521E-01, 1.0504E-01, 1.0487E-01, 1.0470E-01,
     &   1.0452E-01, 1.0435E-01, 1.0418E-01, 1.0401E-01, 1.0384E-01,
     &   1.0367E-01, 1.0350E-01, 1.0333E-01, 1.0315E-01, 1.0298E-01,
     &   1.0281E-01, 1.0264E-01, 1.0247E-01, 1.0230E-01, 1.0213E-01,
     &   1.0197E-01, 1.0180E-01, 1.0163E-01, 1.0146E-01, 1.0129E-01,
     &   1.0112E-01, 1.0095E-01, 1.0079E-01, 1.0062E-01, 1.0045E-01,
     &   1.0028E-01, 1.0012E-01, 9.9948E-02, 9.9781E-02, 9.9615E-02/
  
         DATA ((YGG  (I,J), I= 101, 200), J=1,1) /
     &   9.9449E-02, 9.9283E-02, 9.9117E-02, 9.8951E-02, 9.8786E-02,
     &   9.8620E-02, 9.8455E-02, 9.8291E-02, 9.8126E-02, 9.7962E-02,
     &   9.7798E-02, 9.7634E-02, 9.7470E-02, 9.7307E-02, 9.7144E-02,
     &   9.6981E-02, 9.6818E-02, 9.6656E-02, 9.6493E-02, 9.6331E-02,
     &   9.6170E-02, 9.6008E-02, 9.5847E-02, 9.5686E-02, 9.5525E-02,
     &   9.5364E-02, 9.5203E-02, 9.5043E-02, 9.4883E-02, 9.4723E-02,
     &   9.4564E-02, 9.4404E-02, 9.4245E-02, 9.4086E-02, 9.3927E-02,
     &   9.3769E-02, 9.3610E-02, 9.3452E-02, 9.3294E-02, 9.3136E-02,
     &   9.2979E-02, 9.2821E-02, 9.2664E-02, 9.2507E-02, 9.2350E-02,
     &   9.2193E-02, 9.2036E-02, 9.1880E-02, 9.1723E-02, 9.1567E-02,
     &   9.1411E-02, 9.1255E-02, 9.1099E-02, 9.0944E-02, 9.0788E-02,
     &   9.0632E-02, 9.0477E-02, 9.0322E-02, 9.0166E-02, 9.0011E-02,
     &   8.9856E-02, 8.9701E-02, 8.9546E-02, 8.9391E-02, 8.9236E-02,
     &   8.9081E-02, 8.8926E-02, 8.8771E-02, 8.8617E-02, 8.8462E-02,
     &   8.8307E-02, 8.8152E-02, 8.7997E-02, 8.7842E-02, 8.7687E-02,
     &   8.7532E-02, 8.7377E-02, 8.7221E-02, 8.7066E-02, 8.6911E-02,
     &   8.6755E-02, 8.6600E-02, 8.6444E-02, 8.6288E-02, 8.6132E-02,
     &   8.5976E-02, 8.5820E-02, 8.5664E-02, 8.5507E-02, 8.5350E-02,
     &   8.5193E-02, 8.5036E-02, 8.4879E-02, 8.4722E-02, 8.4564E-02,
     &   8.4406E-02, 8.4248E-02, 8.4090E-02, 8.3931E-02, 8.3773E-02/
  
  
         DATA ((YGG  (I,J), I=   1, 100), J=2,2) /
     &   3.6335E-04, 3.6335E-04, 3.6335E-04, 3.6335E-04, 3.6335E-04,
     &   3.6335E-04, 3.6335E-04, 3.6335E-04, 3.6335E-04, 3.6335E-04,
     &   3.6334E-04, 3.6334E-04, 3.6334E-04, 3.6334E-04, 3.6334E-04,
     &   3.6334E-04, 3.6334E-04, 3.6334E-04, 3.6334E-04, 3.6334E-04,
     &   3.6334E-04, 3.6334E-04, 3.6334E-04, 3.6334E-04, 3.6334E-04,
     &   3.6333E-04, 3.6333E-04, 3.6333E-04, 3.6333E-04, 3.6333E-04,
     &   3.6333E-04, 3.6333E-04, 3.6333E-04, 3.6333E-04, 3.6333E-04,
     &   3.6333E-04, 3.6333E-04, 3.6333E-04, 3.6333E-04, 3.6332E-04,
     &   3.6332E-04, 3.6332E-04, 3.6332E-04, 3.6332E-04, 3.6332E-04,
     &   3.6332E-04, 3.6332E-04, 3.6332E-04, 3.6332E-04, 3.6332E-04,
     &   3.6332E-04, 3.6332E-04, 3.6332E-04, 3.6332E-04, 3.6331E-04,
     &   3.6331E-04, 3.6331E-04, 3.6331E-04, 3.6331E-04, 3.6331E-04,
     &   3.6331E-04, 3.6331E-04, 3.6331E-04, 3.6331E-04, 3.6331E-04,
     &   3.6331E-04, 3.6331E-04, 3.6331E-04, 3.6330E-04, 3.6330E-04,
     &   3.6330E-04, 3.6330E-04, 3.6330E-04, 3.6330E-04, 3.6330E-04,
     &   3.6330E-04, 3.6330E-04, 3.6330E-04, 3.6330E-04, 3.6330E-04,
     &   3.6330E-04, 3.6330E-04, 3.6329E-04, 3.6329E-04, 3.6329E-04,
     &   3.6329E-04, 3.6329E-04, 3.6329E-04, 3.6329E-04, 3.6329E-04,
     &   3.6329E-04, 3.6329E-04, 3.6329E-04, 3.6329E-04, 3.6329E-04,
     &   3.6329E-04, 3.6328E-04, 3.6328E-04, 3.6328E-04, 3.6328E-04/
  
         DATA ((YGG  (I,J), I= 101, 200), J=2,2) /
     &   3.6328E-04, 3.6328E-04, 3.6328E-04, 3.6328E-04, 3.6328E-04,
     &   3.6328E-04, 3.6328E-04, 3.6328E-04, 3.6328E-04, 3.6327E-04,
     &   3.6327E-04, 3.6327E-04, 3.6327E-04, 3.6327E-04, 3.6327E-04,
     &   3.6327E-04, 3.6327E-04, 3.6327E-04, 3.6327E-04, 3.6327E-04,
     &   3.6327E-04, 3.6327E-04, 3.6327E-04, 3.6326E-04, 3.6326E-04,
     &   3.6326E-04, 3.6326E-04, 3.6326E-04, 3.6326E-04, 3.6326E-04,
     &   3.6326E-04, 3.6326E-04, 3.6326E-04, 3.6326E-04, 3.6326E-04,
     &   3.6326E-04, 3.6325E-04, 3.6325E-04, 3.6325E-04, 3.6325E-04,
     &   3.6325E-04, 3.6325E-04, 3.6325E-04, 3.6325E-04, 3.6325E-04,
     &   3.6325E-04, 3.6325E-04, 3.6325E-04, 3.6325E-04, 3.6324E-04,
     &   3.6324E-04, 3.6324E-04, 3.6324E-04, 3.6324E-04, 3.6324E-04,
     &   3.6324E-04, 3.6324E-04, 3.6324E-04, 3.6324E-04, 3.6324E-04,
     &   3.6324E-04, 3.6324E-04, 3.6323E-04, 3.6323E-04, 3.6323E-04,
     &   3.6323E-04, 3.6323E-04, 3.6323E-04, 3.6323E-04, 3.6323E-04,
     &   3.6323E-04, 3.6323E-04, 3.6323E-04, 3.6323E-04, 3.6322E-04,
     &   3.6322E-04, 3.6322E-04, 3.6322E-04, 3.6322E-04, 3.6322E-04,
     &   3.6322E-04, 3.6322E-04, 3.6322E-04, 3.6322E-04, 3.6322E-04,
     &   3.6321E-04, 3.6321E-04, 3.6321E-04, 3.6321E-04, 3.6321E-04,
     &   3.6321E-04, 3.6321E-04, 3.6321E-04, 3.6321E-04, 3.6321E-04,
     &   3.6321E-04, 3.6320E-04, 3.6320E-04, 3.6320E-04, 3.6320E-04/
  
      save
      RETURN
      END
