c;;;;;;;;;;;;;;;;;;;;;;;;;;;;; -*- Mode: Fortran -*- ;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; hadr5n17.f ---
c;; Author          : Friedrich Jegerlehner
c;; Created On      : Thu Jul  6 12:20:17 2017
c;; Last Modified By: Friedrich Jegerlehner
c;; Last Modified On: Tue Jan 14 02:54:47 2020
c;; RCS: $Id$
c;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; Copyright (C) 2019 Friedrich Jegerlehner
c;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;;
       subroutine hadr5x(e,st2,der,errdersta,errdersys,
     &     deg,errdegsta,errdegsys)
c Provides real part of 5 flavor hadronic contribution to photon vacuum polarization
c single precision HADR5 ; double precision DHADR5
c ******************************************************************
c *                                                                *
c *      subroutine for the evaluation of the light hadron         *
c *           contributions to Delta_r  and  Delta_g               *
c *                    using fits to the                           *
c *          QED vacuum polarization from e^+ e^- data             *
c *                                                                *
c *    F. Jegerlehner, Institut fr Physik                         *
c *                    Humboldt-Universitt zu Berlin              *
c *                    Newtonstrae 15, D-12489 Berlin, Germany    *
c *    E-mail: fjeger@physik.hu-berlin.de                          *
c *    Phone :   +49-3375 500 277                                  *
c *                                                                *
c *    Reference: F. Jegerlehner, Z. Phys. C32 (1986) 195          *
c *               H. Burkhardt et al., Z. Phys. C42 (1989) 497     *
c *               S. Eidelman, F. Jegerlehner, Z. Phys. C (1995)   *
c *               F. Jegerlehner,                                  *
c *                    Nucl.Phys.Proc.Suppl.131:213-222,2004       *
c *                    Nucl.Phys.Proc.Suppl.162:22-32,2006         *
c *                    Nucl.Phys.Proc.Suppl.181-182:135-140,2008   *
c *                                                                *
c ******************************************************************
c       VERSION: 23/12/2019
c
C  Notation: E energy ( momentum transfer ): E>0 timelike , E<0 spacelike
c      sin2ell=0.23153 ! pm 0.00016 LEPEEWG Phys Rep 427 (2006) 257
C            st2 is sin^2(Theta); st2=0.23153 is the reference value
C  the routine returns the hadronic contribution of 5 flavors (u,d,s,c,b)
C                 to   DER=Delta_r with hadronic error ERRDER
C                and   DEG=Delta_g with hadronic error ERRDEG
C  The effective value of the fine structure constant alphaQED at energy
C  E is alphaQED(E)=alphaQED(0)/(1-Delta_r) ,similarly for the SU(2)
C  coupling alphaSU2(E)=alphaSU2(0)/(1-Delta_g), where Delta_r(g) is the
C  sum of leptonic, hadronic contributions (top to be added).
C
      IMPLICIT NONE
      INTEGER NA,NB,NC,IJ,I,ini
      PARAMETER(NA=979,NB=2174,NC=200)
      real e,st2,der,errdersta,errdersys,deg,errdegsta,errdegsys,fac
      real dal(3),dg2(3)
      REAL ETA(NA),DAT(NA,3),DGT(NA,3)
      REAL ESA(NB),DAS(NB,3),DGS(NB,3)
      REAL EMA(NC),DAM(NC,3),DGM(NC,3)
      REAL XXX(NC),YGG(NC,3),Y3G(NC,3)
      COMMON /DATM19/EMA,DAM,DGM
      COMMON /DATT19/ETA,DAT,DGT
      COMMON /DATS19/ESA,DAS,DGS
      COMMON /DATH19/XXX,YGG,Y3G
      data ini/0/
C initialize data

      call dalhad_spacelike19

      call dalhad_timelike19

      call dalhad_timelike1h19

c boundaries of data sets
c space-like and unphysical region  -1.0000E+03 - 0.27599999D0 (ETA,DAT)
c time-like R data range 0.28207018D0 - 38.29000000D0          (ESA,DAS)
c time-like1 pQCD range 1.3005E+01 - 2.0000E+04                (EMA,DAM)
c 2 pion threshold at 0.27914....
      if (st2.eq.0.0) then
         fac=1.0
      else
         fac=0.23153d0/st2
      endif
      if (e.eq.0.0) then
        der=0.0
        errdersta=0.0
        errdersys=0.0
        deg=0.0
        errdegsta=0.0
        errdegsys=0.0
        return
c first bridge cracks between regions
      else if ((e.ge.XXX(NC)).and.(e.lt.ETA(1))) then
        DO I=1,3
        dal(i)=YGG(NC,I)
     &    +(DAT(1,I)-YGG(NC,I))/(ETA(1)-XXX(NC))*(E-XXX(NC))
        dg2(i)=Y3G(NC,I)
     &    +(DAT(1,I)-Y3G(NC,I))/(ETA(1)-XXX(NC))*(E-XXX(NC))
        ENDDO
        der=dal(1)
        errdersta=dal(2)
        errdersys=dal(3)
        deg=dg2(1)*fac
        errdegsta=dg2(2)*fac
        errdegsys=dg2(3)*fac
        return
      else if ((e.ge.ETA(NA)).and.(e.lt.ESA(1))) then
        DO I=1,3
        dal(i)=DAT(NA,I)
     &    +(DAS(1,I)-DAT(NA,I))/(ESA(1)-ETA(NA))*(E-ETA(NA))
        dg2(i)=DGT(NA,I)
     &    +(DGS(1,I)-DGT(NA,I))/(ESA(1)-ETA(NA))*(E-ETA(NA))
        ENDDO
        der=dal(1)
        errdersta=dal(2)
        errdersys=dal(3)
        deg=dg2(1)*fac
        errdegsta=dg2(2)*fac
        errdegsys=dg2(3)*fac
        return
      else if ((e.lt.ETA(NA)).and.(e.ge.ETA(1))) then
        ij=NA
        do while (ETA(ij).gt.e)
          ij=ij-1
        enddo
        DO I=1,3
        dal(i)=DAT(IJ,I)
     &    +(DAT(IJ+1,I)-DAT(IJ,I))/(ETA(IJ+1)-ETA(IJ))*(E-ETA(IJ))
        dg2(i)=DGT(IJ,I)
     &    +(DGT(IJ+1,I)-DGT(IJ,I))/(ETA(IJ+1)-ETA(IJ))*(E-ETA(IJ))
        ENDDO
        der=dal(1)
        errdersta=dal(2)
        errdersys=dal(3)
        deg=dg2(1)*fac
        errdegsta=dg2(2)*fac
        errdegsys=dg2(3)*fac
        return
      else if ((e.lt.EMA(NC)).and.(e.ge.EMA(1))) then
        ij=NC
        do while (EMA(ij).gt.e)
          ij=ij-1
        enddo
        DO I=1,3
        dal(i)=DAM(IJ,I)
     &    +(DAM(IJ+1,I)-DAM(IJ,I))/(EMA(IJ+1)-EMA(IJ))*(E-EMA(IJ))
        dg2(i)=DGM(IJ,I)
     &    +(DGM(IJ+1,I)-DGM(IJ,I))/(EMA(IJ+1)-EMA(IJ))*(E-EMA(IJ))
        ENDDO
        der=dal(1)
        errdersta=dal(2)
        errdersys=dal(3)
        deg=dg2(1)*fac
        errdegsta=dg2(2)*fac
        errdegsys=dg2(3)*fac
        return
      else if ((e.lt.ESA(NB)).and.(e.ge.ESA(1))) then
        if ((e.gt.3.96).and.(e.lt.4.52).and.(ini.eq.0)) then
c           write (*,*) ' ******************************************'
c           write (*,*) ' * Note: fluctuations in the Psi(4,5,6)   *'
c           write (*,*) ' * region are because we are using the    *'
c           write (*,*) ' * CB and BES data without any smoothing  *'
c           write (*,*) ' *********** FJ@HU Berlin *****************'
           ini=1
        endif
        ij=NB
        do while (ESA(ij).gt.e)
          ij=ij-1
        enddo
        DO I=1,3
        dal(i)=DAS(IJ,I)
     &    +(DAS(IJ+1,I)-DAS(IJ,I))/(ESA(IJ+1)-ESA(IJ))*(E-ESA(IJ))
        dg2(i)=DGS(IJ,I)
     &    +(DGS(IJ+1,I)-DGS(IJ,I))/(ESA(IJ+1)-ESA(IJ))*(E-ESA(IJ))
        ENDDO
        der=dal(1)
        errdersta=dal(2)
        errdersys=dal(3)
        deg=dg2(1)*fac
        errdegsta=dg2(2)*fac
        errdegsys=dg2(3)*fac
        return
      else if ((e.lt.XXX(NC)).and.(e.ge.XXX(1))) then
        ij=NC
        do while (XXX(ij).gt.e)
          ij=ij-1
        enddo
        DO I=1,3
        dal(i)=YGG(IJ,I)
     &    +(YGG(IJ+1,I)-YGG(IJ,I))/(XXX(IJ+1)-XXX(IJ))*(E-XXX(IJ))
        dg2(i)=Y3G(IJ,I)
     &    +(Y3G(IJ+1,I)-Y3G(IJ,I))/(XXX(IJ+1)-XXX(IJ))*(E-XXX(IJ))
        ENDDO
        der=dal(1)
        errdersta=dal(2)
        errdersys=dal(3)
        deg=dg2(1)*fac
        errdegsta=dg2(2)*fac
        errdegsys=dg2(3)*fac
        return
      else
         if (ini.le.10) then
            write(*,*) ' out of range! e=',e
            ini=ini+1
            if (ini.eq.10) write(*,*) ' Warning: continuing anyway ...'
         endif
      endif
      return
      end

       subroutine dhadr5x(de,dst2,dder,derrdersta,derrdersys,
     &     ddeg,derrdegsta,derrdegsys)
c **********************************************************************
c *    F. Jegerlehner, University of Silesia, PL-40007 Katowice, Poland *
c **********************************************************************
c Converts hadr5x to double precision variables in dhadr5x
c
       implicit none
       real *8 de,dst2,dder,ddeg,derrdersta,derrdersys,
     &      derrdegsta,derrdegsys
       real    se,sst2,sder,sdeg,serrdersta,serrdersys,
     &      serrdegsta,serrdegsys
       se  =sngl(de)
       sst2=sngl(dst2)
       call hadr5x(se,sst2,sder,serrdersta,serrdersys,sdeg,
     &      serrdegsta,serrdegsys)
       dder   =dble(sder)
       ddeg   =dble(sdeg)
       derrdersta=dble(serrdersta)
       derrdersys=dble(serrdersys)
       derrdegsta=dble(serrdegsta)
       derrdegsys=dble(serrdegsys)
       return
       end
c
c Following datasets obtained with run parameters specified in the header
c

c      ++++ dalhadslow19 ++++

c update 12/23/19
       subroutine dalhad_timelike19
       IMPLICIT NONE
       INTEGER NB,I,J
       PARAMETER(NB=2174)
       REAL ESA(NB),DAS(NB,3),DGS(NB,3)
       COMMON /DATS19/ESA,DAS,DGS

c******************************************************************************
c*                                                                            *
c*    --- intRdatx ---                              Mon Dec 23 00:45:02 2019  *
c*                                                                            *
c***************************# FJ@HU Berlin #***********************************

c  "Delta alpha"               Update version (24/02/2019)

c ISCAN:  1, Ismooth: 0, IGS: 0, DSET:     , iplot: 0 itestflavorspl: 0
c IKE,Emin,Emax,NP,N,EPSREL:  1, 6.0000E-01, 1.4000E+00, 100, 10, 1.00E-02,
c ALS(sta)(sys):  0.1184( 0.0007)( 0.0000), MZINP:   91.1876, MTOP:    173.20
c NF,IOR,ICHK: 5,4,2,ISU2: 0,NFL: 5,INAG: 1,inotail:  0, sin2W:   0.23153
c CHPTCUT,EC,ECUT: 0.3180,     5.20,    11.50


       DATA  (ESA  (I), I=   1, 100) /
     &   0.28207018D0, 0.28800000D0, 0.29400000D0, 0.30000000D0,
     &   0.30600000D0, 0.30900000D0, 0.31500000D0, 0.32600000D0,
     &   0.33100000D0, 0.33600000D0, 0.34100000D0, 0.34600000D0,
     &   0.35200000D0, 0.35600000D0, 0.36100000D0, 0.36600000D0,
     &   0.37100000D0, 0.37600000D0, 0.38000000D0, 0.38500000D0,
     &   0.38900000D0, 0.39400000D0, 0.39800000D0, 0.40300000D0,
     &   0.40700000D0, 0.41100000D0, 0.41600000D0, 0.41871100D0,
     &   0.42000000D0, 0.42400000D0, 0.42800000D0, 0.43200000D0,
     &   0.43600000D0, 0.44000000D0, 0.44400000D0, 0.44800000D0,
     &   0.45200000D0, 0.45600000D0, 0.46000000D0, 0.46400000D0,
     &   0.46700000D0, 0.47100000D0, 0.47500000D0, 0.47800000D0,
     &   0.48200000D0, 0.48600000D0, 0.48900000D0, 0.49300000D0,
     &   0.49600000D0, 0.50000000D0, 0.52000000D0, 0.54000000D0,
     &   0.56600000D0, 0.59200000D0, 0.60000000D0, 0.60200000D0,
     &   0.60800000D0, 0.61600000D0, 0.62400000D0, 0.63200000D0,
     &   0.64000000D0, 0.64300000D0, 0.64800000D0, 0.65600000D0,
     &   0.66300000D0, 0.67100000D0, 0.67800000D0, 0.68600000D0,
     &   0.68800000D0, 0.69300000D0, 0.69600000D0, 0.70000000D0,
     &   0.70700000D0, 0.71400000D0, 0.72100000D0, 0.72800000D0,
     &   0.73500000D0, 0.74200000D0, 0.74800000D0, 0.75000000D0,
     &   0.75500000D0, 0.75744000D0, 0.76007300D0, 0.76207300D0,
     &   0.76507300D0, 0.76851700D0, 0.77060500D0, 0.77205400D0,
     &   0.77314300D0, 0.77400300D0, 0.77470900D0, 0.77530500D0,
     &   0.77582000D0, 0.77627100D0, 0.77667300D0, 0.77726600D0,
     &   0.77746600D0, 0.77766900D0, 0.77795100D0, 0.77821500D0/
       DATA  (ESA  (I), I= 101, 200) /
     &   0.77846300D0, 0.77869800D0, 0.77892200D0, 0.77913700D0,
     &   0.77934400D0, 0.77954500D0, 0.77974200D0, 0.77993500D0,
     &   0.78012700D0, 0.78031800D0, 0.78051000D0, 0.78070600D0,
     &   0.78090800D0, 0.78112100D0, 0.78135100D0, 0.78161200D0,
     &   0.78193600D0, 0.78268000D0, 0.78341500D0, 0.78373400D0,
     &   0.78399100D0, 0.78421700D0, 0.78442600D0, 0.78462500D0,
     &   0.78481700D0, 0.78500500D0, 0.78519100D0, 0.78537800D0,
     &   0.78556600D0, 0.78575600D0, 0.78595100D0, 0.78615100D0,
     &   0.78635800D0, 0.78657300D0, 0.78679800D0, 0.78703400D0,
     &   0.78728400D0, 0.78755000D0, 0.78773500D0, 0.78793500D0,
     &   0.78814300D0, 0.78847900D0, 0.78884800D0, 0.78925800D0,
     &   0.78971900D0, 0.79024600D0, 0.79085900D0, 0.79158800D0,
     &   0.79248000D0, 0.79361500D0, 0.79513900D0, 0.79736300D0,
     &   0.79800000D0, 0.80000000D0, 0.80200000D0, 0.80400000D0,
     &   0.80600000D0, 0.80800000D0, 0.81200000D0, 0.81400000D0,
     &   0.81600000D0, 0.82000000D0, 0.82200000D0, 0.82400000D0,
     &   0.82600000D0, 0.82800000D0, 0.83000000D0, 0.83100000D0,
     &   0.83700000D0, 0.84300000D0, 0.84900000D0, 0.85400000D0,
     &   0.86000000D0, 0.86600000D0, 0.87200000D0, 0.87700000D0,
     &   0.88300000D0, 0.88900000D0, 0.89400000D0, 0.89700000D0,
     &   0.90250000D0, 0.90500000D0, 0.91100000D0, 0.91600000D0,
     &   0.92200000D0, 0.92700000D0, 0.93300000D0, 0.93800000D0,
     &   0.94300000D0, 0.94900000D0, 0.95200000D0, 0.95400000D0,
     &   0.95800000D0, 0.96000000D0, 0.96200000D0, 0.96400000D0,
     &   0.96600000D0, 0.96800000D0, 0.97000000D0, 0.97400000D0/
       DATA  (ESA  (I), I= 201, 300) /
     &   0.97800000D0, 0.98000000D0, 0.98070000D0, 0.98140000D0,
     &   0.98210000D0, 0.98280000D0, 0.98350000D0, 0.98420000D0,
     &   0.98490000D0, 0.98560000D0, 0.98630000D0, 0.98700000D0,
     &   0.98770000D0, 0.98840000D0, 0.98910000D0, 0.98980000D0,
     &   0.99050000D0, 0.99120000D0, 0.99190000D0, 0.99260000D0,
     &   0.99330000D0, 0.99400000D0, 0.99470000D0, 0.99540000D0,
     &   0.99610000D0, 0.99680000D0, 0.99750000D0, 0.99820000D0,
     &   0.99890000D0, 0.99960000D0, 0.99990000D0, 1.00001000D0,
     &   1.00030000D0, 1.00100000D0, 1.00170000D0, 1.00240000D0,
     &   1.00310000D0, 1.00380000D0, 1.00450000D0, 1.00520000D0,
     &   1.00590000D0, 1.00660000D0, 1.00730000D0, 1.00800000D0,
     &   1.00870000D0, 1.00940000D0, 1.01010000D0, 1.01080000D0,
     &   1.01150000D0, 1.01220000D0, 1.01290000D0, 1.01360000D0,
     &   1.01430000D0, 1.01500000D0, 1.01520000D0, 1.01540000D0,
     &   1.01560000D0, 1.01580000D0, 1.01600000D0, 1.01620000D0,
     &   1.01640000D0, 1.01660000D0, 1.01680000D0, 1.01700000D0,
     &   1.01720000D0, 1.01740000D0, 1.01770000D0, 1.01790000D0,
     &   1.01800000D0, 1.01820000D0, 1.01840000D0, 1.01860000D0,
     &   1.01880000D0, 1.01900000D0, 1.01920000D0, 1.01940000D0,
     &   1.01960000D0, 1.01980000D0, 1.02000000D0, 1.02020000D0,
     &   1.02040000D0, 1.02060000D0, 1.02080000D0, 1.02100000D0,
     &   1.02120000D0, 1.02140000D0, 1.02160000D0, 1.02180000D0,
     &   1.02200000D0, 1.02220000D0, 1.02240000D0, 1.02270000D0,
     &   1.02290000D0, 1.02300000D0, 1.02320000D0, 1.02340000D0,
     &   1.02360000D0, 1.02380000D0, 1.02400000D0, 1.02420000D0/
       DATA  (ESA  (I), I= 301, 400) /
     &   1.02440000D0, 1.02460000D0, 1.02480000D0, 1.02500000D0,
     &   1.02570000D0, 1.02640000D0, 1.02710000D0, 1.02780000D0,
     &   1.02850000D0, 1.02920000D0, 1.02990000D0, 1.03060000D0,
     &   1.03130000D0, 1.03200000D0, 1.03270000D0, 1.03340000D0,
     &   1.03410000D0, 1.03480000D0, 1.03550000D0, 1.03620000D0,
     &   1.03690000D0, 1.03760000D0, 1.03830000D0, 1.03900000D0,
     &   1.03970000D0, 1.03999000D0, 1.04001000D0, 1.04040000D0,
     &   1.04110000D0, 1.04180000D0, 1.04250000D0, 1.04320000D0,
     &   1.04390000D0, 1.04460000D0, 1.04530000D0, 1.04600000D0,
     &   1.04670000D0, 1.04740000D0, 1.04810000D0, 1.04880000D0,
     &   1.04950000D0, 1.05020000D0, 1.05090000D0, 1.05160000D0,
     &   1.05230000D0, 1.05300000D0, 1.05370000D0, 1.05440000D0,
     &   1.05510000D0, 1.05580000D0, 1.05650000D0, 1.05720000D0,
     &   1.05790000D0, 1.05860000D0, 1.05930000D0, 1.06000000D0,
     &   1.06200000D0, 1.06700000D0, 1.07700000D0, 1.08700000D0,
     &   1.09700000D0, 1.10700000D0, 1.11700000D0, 1.12700000D0,
     &   1.13700000D0, 1.13800000D0, 1.15200000D0, 1.16300000D0,
     &   1.16700000D0, 1.17700000D0, 1.18700000D0, 1.18800000D0,
     &   1.19700000D0, 1.20700000D0, 1.21200000D0, 1.21700000D0,
     &   1.22700000D0, 1.23700000D0, 1.24700000D0, 1.25700000D0,
     &   1.26300000D0, 1.26700000D0, 1.27700000D0, 1.28700000D0,
     &   1.28800000D0, 1.29700000D0, 1.30700000D0, 1.31200000D0,
     &   1.31700000D0, 1.32700000D0, 1.33700000D0, 1.34700000D0,
     &   1.35000000D0, 1.35700000D0, 1.36200000D0, 1.36700000D0,
     &   1.37700000D0, 1.38700000D0, 1.38800000D0, 1.39500000D0/
       DATA  (ESA  (I), I= 401, 500) /
     &   1.41000000D0, 1.41900000D0, 1.42700000D0, 1.43500000D0,
     &   1.43800000D0, 1.45200000D0, 1.45700000D0, 1.46000000D0,
     &   1.46400000D0, 1.47200000D0, 1.48100000D0, 1.49000000D0,
     &   1.49100000D0, 1.50000000D0, 1.50100000D0, 1.50600000D0,
     &   1.51000000D0, 1.52300000D0, 1.52500000D0, 1.53800000D0,
     &   1.53900000D0, 1.54000000D0, 1.55000000D0, 1.56000000D0,
     &   1.56200000D0, 1.57400000D0, 1.57500000D0, 1.58000000D0,
     &   1.58100000D0, 1.58700000D0, 1.59000000D0, 1.59900000D0,
     &   1.60000000D0, 1.61000000D0, 1.61900000D0, 1.62400000D0,
     &   1.62500000D0, 1.63800000D0, 1.64000000D0, 1.65000000D0,
     &   1.65700000D0, 1.66000000D0, 1.66200000D0, 1.66300000D0,
     &   1.67300000D0, 1.67500000D0, 1.68100000D0, 1.68700000D0,
     &   1.69000000D0, 1.70000000D0, 1.71000000D0, 1.71200000D0,
     &   1.71300000D0, 1.71900000D0, 1.72200000D0, 1.72500000D0,
     &   1.73800000D0, 1.74000000D0, 1.74600000D0, 1.75000000D0,
     &   1.75600000D0, 1.75700000D0, 1.76000000D0, 1.76200000D0,
     &   1.76400000D0, 1.76900000D0, 1.77500000D0, 1.78100000D0,
     &   1.79000000D0, 1.79900000D0, 1.80000000D0, 1.81200000D0,
     &   1.81900000D0, 1.82500000D0, 1.82600000D0, 1.83400000D0,
     &   1.83500000D0, 1.83800000D0, 1.84500000D0, 1.85700000D0,
     &   1.85900000D0, 1.87400000D0, 1.88100000D0, 1.88600000D0,
     &   1.90000000D0, 1.91900000D0, 1.92500000D0, 1.93000000D0,
     &   1.93800000D0, 1.94000000D0, 1.94500000D0, 1.94600000D0,
     &   1.95000000D0, 1.95500000D0, 1.96000000D0, 1.96200000D0,
     &   1.98100000D0, 1.99050000D0, 2.00500000D0, 2.05000000D0/
       DATA  (ESA  (I), I= 501, 600) /
     &   2.10000000D0, 2.15010000D0, 2.22000000D0, 2.26000000D0,
     &   2.30000000D0, 2.40000000D0, 2.45000000D0, 2.50000000D0,
     &   2.55000000D0, 2.60000000D0, 2.65000000D0, 2.70000000D0,
     &   2.75000000D0, 2.80000000D0, 2.85000000D0, 2.90000000D0,
     &   2.91000000D0, 2.92000000D0, 2.93000000D0, 2.94000000D0,
     &   2.95000000D0, 2.96000000D0, 2.97000000D0, 2.97200000D0,
     &   2.97400000D0, 2.97600000D0, 2.97800000D0, 2.98000000D0,
     &   2.98200000D0, 2.98400000D0, 2.98600000D0, 2.98800000D0,
     &   2.99000000D0, 2.99900000D0, 3.00500000D0, 3.00800000D0,
     &   3.01000000D0, 3.01200000D0, 3.01400000D0, 3.01600000D0,
     &   3.01800000D0, 3.02000000D0, 3.02100000D0, 3.02200000D0,
     &   3.02300000D0, 3.02400000D0, 3.02500000D0, 3.02600000D0,
     &   3.02700000D0, 3.02800000D0, 3.02900000D0, 3.03000000D0,
     &   3.03100000D0, 3.03200000D0, 3.03300000D0, 3.03400000D0,
     &   3.03500000D0, 3.03600000D0, 3.03700000D0, 3.03800000D0,
     &   3.03900000D0, 3.04000000D0, 3.04100000D0, 3.04200000D0,
     &   3.04300000D0, 3.04400000D0, 3.04500000D0, 3.04600000D0,
     &   3.04700000D0, 3.04800000D0, 3.04900000D0, 3.05000000D0,
     &   3.05100000D0, 3.05200000D0, 3.05300000D0, 3.05400000D0,
     &   3.05500000D0, 3.05600000D0, 3.05700000D0, 3.05800000D0,
     &   3.05900000D0, 3.06000000D0, 3.06050000D0, 3.06100000D0,
     &   3.06150000D0, 3.06200000D0, 3.06250000D0, 3.06300000D0,
     &   3.06350000D0, 3.06400000D0, 3.06450000D0, 3.06500000D0,
     &   3.06550000D0, 3.06600000D0, 3.06650000D0, 3.06700000D0,
     &   3.06750000D0, 3.06800000D0, 3.06850000D0, 3.06900000D0/
       DATA  (ESA  (I), I= 601, 700) /
     &   3.06950000D0, 3.07000000D0, 3.07050000D0, 3.07100000D0,
     &   3.07150000D0, 3.07200000D0, 3.07250000D0, 3.07300000D0,
     &   3.07350000D0, 3.07400000D0, 3.07450000D0, 3.07500000D0,
     &   3.07550000D0, 3.07600000D0, 3.07650000D0, 3.07700000D0,
     &   3.07750000D0, 3.07800000D0, 3.07850000D0, 3.07900000D0,
     &   3.07970000D0, 3.08000000D0, 3.08050000D0, 3.08100000D0,
     &   3.08150000D0, 3.08200000D0, 3.08250000D0, 3.08300000D0,
     &   3.08350000D0, 3.08400000D0, 3.08450000D0, 3.08500000D0,
     &   3.08550000D0, 3.08587000D0, 3.08688300D0, 3.08789600D0,
     &   3.08890900D0, 3.08992200D0, 3.09093500D0, 3.09194800D0,
     &   3.09296100D0, 3.09397400D0, 3.09498700D0, 3.09600000D0,
     &   3.09605000D0, 3.09610000D0, 3.09615000D0, 3.09620000D0,
     &   3.09625000D0, 3.09630000D0, 3.09635000D0, 3.09640000D0,
     &   3.09645000D0, 3.09650000D0, 3.09655000D0, 3.09660000D0,
     &   3.09665000D0, 3.09665500D0, 3.09673500D0, 3.09677100D0,
     &   3.09679300D0, 3.09680900D0, 3.09682000D0, 3.09682900D0,
     &   3.09683600D0, 3.09684300D0, 3.09684800D0, 3.09685300D0,
     &   3.09685700D0, 3.09686100D0, 3.09686400D0, 3.09686800D0,
     &   3.09687000D0, 3.09687300D0, 3.09687600D0, 3.09687800D0,
     &   3.09688100D0, 3.09688300D0, 3.09688500D0, 3.09688700D0,
     &   3.09688900D0, 3.09689100D0, 3.09689300D0, 3.09689500D0,
     &   3.09689700D0, 3.09689900D0, 3.09690100D0, 3.09690400D0,
     &   3.09690600D0, 3.09690900D0, 3.09691200D0, 3.09692000D0,
     &   3.09692800D0, 3.09693100D0, 3.09693400D0, 3.09693600D0,
     &   3.09693900D0, 3.09694100D0, 3.09694300D0, 3.09694500D0/
       DATA  (ESA  (I), I= 701, 800) /
     &   3.09694700D0, 3.09694900D0, 3.09695100D0, 3.09695300D0,
     &   3.09695500D0, 3.09695700D0, 3.09695900D0, 3.09696200D0,
     &   3.09696400D0, 3.09696700D0, 3.09697000D0, 3.09697200D0,
     &   3.09697600D0, 3.09697900D0, 3.09698300D0, 3.09698700D0,
     &   3.09699200D0, 3.09699700D0, 3.09700400D0, 3.09701100D0,
     &   3.09702000D0, 3.09703100D0, 3.09704700D0, 3.09706900D0,
     &   3.09710500D0, 3.09715000D0, 3.09720000D0, 3.09725000D0,
     &   3.09730000D0, 3.09735000D0, 3.09740000D0, 3.09745000D0,
     &   3.09750000D0, 3.09755000D0, 3.09760000D0, 3.09765000D0,
     &   3.09770000D0, 3.09775000D0, 3.09780000D0, 3.09785000D0,
     &   3.09790000D0, 3.09795000D0, 3.09800000D0, 3.09898700D0,
     &   3.09997400D0, 3.10096100D0, 3.10194800D0, 3.10293500D0,
     &   3.10392200D0, 3.10490900D0, 3.10589600D0, 3.10688300D0,
     &   3.10787000D0, 3.10800000D0, 3.10900000D0, 3.11000000D0,
     &   3.11100000D0, 3.11200000D0, 3.11300000D0, 3.11400000D0,
     &   3.11500000D0, 3.11600000D0, 3.11700000D0, 3.11800000D0,
     &   3.11900000D0, 3.12000000D0, 3.12100000D0, 3.12200000D0,
     &   3.12300000D0, 3.12400000D0, 3.12500000D0, 3.12600000D0,
     &   3.12700000D0, 3.12800000D0, 3.12900000D0, 3.13000000D0,
     &   3.13100000D0, 3.13200000D0, 3.13300000D0, 3.13400000D0,
     &   3.13500000D0, 3.13600000D0, 3.13700000D0, 3.13800000D0,
     &   3.13900000D0, 3.14000000D0, 3.14100000D0, 3.14200000D0,
     &   3.14300000D0, 3.14400000D0, 3.14500000D0, 3.14600000D0,
     &   3.14700000D0, 3.14800000D0, 3.14900000D0, 3.15000000D0,
     &   3.15100000D0, 3.15200000D0, 3.15300000D0, 3.15400000D0/
       DATA  (ESA  (I), I= 801, 900) /
     &   3.15500000D0, 3.15600000D0, 3.15700000D0, 3.15800000D0,
     &   3.15900000D0, 3.16000000D0, 3.16100000D0, 3.16200000D0,
     &   3.16300000D0, 3.16400000D0, 3.16500000D0, 3.16600000D0,
     &   3.16700000D0, 3.16800000D0, 3.16900000D0, 3.17000000D0,
     &   3.17100000D0, 3.17200000D0, 3.17300000D0, 3.17400000D0,
     &   3.17500000D0, 3.17600000D0, 3.17700000D0, 3.17800000D0,
     &   3.17900000D0, 3.18000000D0, 3.18100000D0, 3.18200000D0,
     &   3.18300000D0, 3.18400000D0, 3.18500000D0, 3.18600000D0,
     &   3.18700000D0, 3.18800000D0, 3.18900000D0, 3.19000000D0,
     &   3.19100000D0, 3.19200000D0, 3.19300000D0, 3.19400000D0,
     &   3.19500000D0, 3.19600000D0, 3.19700000D0, 3.19800000D0,
     &   3.19900000D0, 3.19990000D0, 3.20100000D0, 3.20200000D0,
     &   3.20300000D0, 3.20400000D0, 3.20500000D0, 3.20600000D0,
     &   3.20700000D0, 3.20800000D0, 3.20900000D0, 3.21000000D0,
     &   3.25000000D0, 3.30000000D0, 3.37000000D0, 3.45000000D0,
     &   3.50000000D0, 3.52000000D0, 3.54000000D0, 3.56000000D0,
     &   3.57500000D0, 3.59000000D0, 3.59500000D0, 3.60500000D0,
     &   3.61000000D0, 3.61500000D0, 3.62000000D0, 3.62500000D0,
     &   3.63000000D0, 3.63500000D0, 3.64000000D0, 3.64500000D0,
     &   3.65000000D0, 3.65200000D0, 3.65400000D0, 3.65600000D0,
     &   3.65800000D0, 3.66000000D0, 3.66200000D0, 3.66300000D0,
     &   3.66400000D0, 3.66500000D0, 3.66600000D0, 3.66700000D0,
     &   3.66800000D0, 3.66900000D0, 3.67000000D0, 3.67100000D0,
     &   3.67200000D0, 3.67300000D0, 3.67400000D0, 3.67496000D0,
     &   3.67551000D0, 3.67606000D0, 3.67661000D0, 3.67716000D0/
       DATA  (ESA  (I), I= 901,1000) /
     &   3.67771000D0, 3.67826000D0, 3.67881000D0, 3.67936000D0,
     &   3.67991000D0, 3.68046000D0, 3.68101000D0, 3.68156000D0,
     &   3.68211000D0, 3.68266000D0, 3.68321000D0, 3.68376000D0,
     &   3.68431000D0, 3.68486000D0, 3.68527300D0, 3.68541000D0,
     &   3.68552000D0, 3.68569900D0, 3.68574600D0, 3.68578100D0,
     &   3.68580900D0, 3.68583200D0, 3.68585100D0, 3.68586800D0,
     &   3.68588300D0, 3.68589500D0, 3.68590700D0, 3.68591800D0,
     &   3.68592800D0, 3.68593700D0, 3.68594600D0, 3.68595300D0,
     &   3.68596100D0, 3.68596800D0, 3.68597500D0, 3.68598200D0,
     &   3.68598800D0, 3.68599500D0, 3.68600100D0, 3.68600700D0,
     &   3.68601300D0, 3.68602000D0, 3.68602600D0, 3.68603300D0,
     &   3.68603900D0, 3.68604700D0, 3.68605500D0, 3.68606600D0,
     &   3.68607500D0, 3.68608500D0, 3.68609000D0, 3.68609500D0,
     &   3.68610000D0, 3.68611400D0, 3.68612500D0, 3.68613300D0,
     &   3.68614100D0, 3.68614700D0, 3.68615400D0, 3.68616000D0,
     &   3.68616700D0, 3.68617300D0, 3.68617900D0, 3.68618500D0,
     &   3.68619200D0, 3.68619800D0, 3.68620500D0, 3.68621200D0,
     &   3.68621900D0, 3.68622700D0, 3.68623400D0, 3.68624300D0,
     &   3.68625200D0, 3.68626200D0, 3.68627300D0, 3.68628500D0,
     &   3.68629700D0, 3.68631200D0, 3.68632900D0, 3.68634800D0,
     &   3.68637100D0, 3.68639900D0, 3.68643400D0, 3.68648100D0,
     &   3.68654800D0, 3.68651000D0, 3.68660000D0, 3.68667000D0,
     &   3.68672000D0, 3.68685000D0, 3.68690700D0, 3.68706000D0,
     &   3.68761000D0, 3.68816000D0, 3.68871000D0, 3.68926000D0,
     &   3.68981000D0, 3.69036000D0, 3.69091000D0, 3.69146000D0/
       DATA  (ESA  (I), I=1001,1100) /
     &   3.69201000D0, 3.69256000D0, 3.69311000D0, 3.69366000D0,
     &   3.69421000D0, 3.69476000D0, 3.69531000D0, 3.69586000D0,
     &   3.69641000D0, 3.69696000D0, 3.69796000D0, 3.69950000D0,
     &   3.70050000D0, 3.70250000D0, 3.70450000D0, 3.70700000D0,
     &   3.71200000D0, 3.72128100D0, 3.73082800D0, 3.73660200D0,
     &   3.74060200D0, 3.74360300D0, 3.74597300D0, 3.74791700D0,
     &   3.74955600D0, 3.75096800D0, 3.75220600D0, 3.75330900D0,
     &   3.75430100D0, 3.75520500D0, 3.75603600D0, 3.75680500D0,
     &   3.75752300D0, 3.75819800D0, 3.75883700D0, 3.75944400D0,
     &   3.76002600D0, 3.76058600D0, 3.76112800D0, 3.76165500D0,
     &   3.76217200D0, 3.76268100D0, 3.76318700D0, 3.76369200D0,
     &   3.76420200D0, 3.76472200D0, 3.76526100D0, 3.76582800D0,
     &   3.76644200D0, 3.76713800D0, 3.76800500D0, 3.77000000D0,
     &   3.77199500D0, 3.77286200D0, 3.77355800D0, 3.77417200D0,
     &   3.77474000D0, 3.77527800D0, 3.77579800D0, 3.77630800D0,
     &   3.77681400D0, 3.77731900D0, 3.77782800D0, 3.77834500D0,
     &   3.77887200D0, 3.77941400D0, 3.77997400D0, 3.78055600D0,
     &   3.78116300D0, 3.78180200D0, 3.78247700D0, 3.78319500D0,
     &   3.78396400D0, 3.78479500D0, 3.78569900D0, 3.78669100D0,
     &   3.78779400D0, 3.78903200D0, 3.79044400D0, 3.79208300D0,
     &   3.79402700D0, 3.79639700D0, 3.79939800D0, 3.80339800D0,
     &   3.80917200D0, 3.81871900D0, 3.82200000D0, 3.83200000D0,
     &   3.85000000D0, 3.85200000D0, 3.86700000D0, 3.87300000D0,
     &   3.87800000D0, 3.88400000D0, 3.89000000D0, 3.89600000D0,
     &   3.90000000D0, 3.90200000D0, 3.90800000D0, 3.91400000D0/
       DATA  (ESA  (I), I=1101,1200) /
     &   3.92000000D0, 3.92600000D0, 3.93200000D0, 3.93800000D0,
     &   3.94400000D0, 3.95000000D0, 3.95600000D0, 3.96200000D0,
     &   3.96800000D0, 3.97400000D0, 3.98000000D0, 3.98600000D0,
     &   3.99200000D0, 3.99800000D0, 4.00000000D0, 4.00400000D0,
     &   4.01000000D0, 4.01600000D0, 4.02000000D0, 4.02200000D0,
     &   4.02800000D0, 4.03000000D0, 4.03400000D0, 4.04000000D0,
     &   4.04600000D0, 4.05000000D0, 4.05200000D0, 4.05800000D0,
     &   4.06400000D0, 4.07000000D0, 4.07600000D0, 4.08200000D0,
     &   4.08800000D0, 4.09000000D0, 4.09400000D0, 4.10000000D0,
     &   4.10500000D0, 4.10900000D0, 4.11000000D0, 4.11300000D0,
     &   4.11800000D0, 4.12100000D0, 4.12500000D0, 4.12900000D0,
     &   4.13000000D0, 4.13300000D0, 4.13600000D0, 4.14100000D0,
     &   4.14700000D0, 4.15000000D0, 4.15300000D0, 4.15900000D0,
     &   4.16500000D0, 4.17100000D0, 4.17700000D0, 4.18000000D0,
     &   4.18200000D0, 4.18500000D0, 4.19000000D0, 4.19500000D0,
     &   4.19900000D0, 4.20100000D0, 4.20800000D0, 4.21200000D0,
     &   4.21400000D0, 4.22000000D0, 4.22600000D0, 4.23200000D0,
     &   4.23800000D0, 4.24000000D0, 4.24400000D0, 4.25000000D0,
     &   4.25600000D0, 4.26200000D0, 4.26800000D0, 4.27400000D0,
     &   4.28000000D0, 4.29200000D0, 4.30000000D0, 4.30400000D0,
     &   4.31000000D0, 4.31600000D0, 4.32800000D0, 4.33000000D0,
     &   4.34000000D0, 4.35000000D0, 4.35200000D0, 4.36400000D0,
     &   4.37600000D0, 4.38000000D0, 4.38200000D0, 4.38800000D0,
     &   4.39000000D0, 4.39400000D0, 4.40000000D0, 4.40600000D0,
     &   4.41000000D0, 4.41200000D0, 4.41800000D0, 4.42400000D0/
       DATA  (ESA  (I), I=1201,1300) /
     &   4.43000000D0, 4.43600000D0, 4.44000000D0, 4.44200000D0,
     &   4.44800000D0, 4.45000000D0, 4.46000000D0, 4.47200000D0,
     &   4.48400000D0, 4.49600000D0, 4.50450000D0, 4.54000000D0,
     &   4.55000000D0, 4.57000000D0, 4.60000000D0, 4.62000000D0,
     &   4.65000000D0, 4.66000000D0, 4.70000000D0, 4.71000000D0,
     &   4.80000000D0, 4.90000000D0, 4.98000000D0, 5.06500000D0,
     &   5.15000000D0, 5.22500000D0, 5.25000000D0, 5.30000000D0,
     &   5.40000000D0, 5.45000000D0, 5.50000000D0, 5.60000000D0,
     &   5.70000000D0, 5.75000000D0, 5.80000000D0, 5.85000000D0,
     &   5.90000000D0, 5.95000000D0, 6.00000000D0, 6.05000000D0,
     &   6.10000000D0, 6.15000000D0, 6.20000000D0, 6.25000000D0,
     &   6.30000000D0, 6.35000000D0, 6.40000000D0, 6.45000000D0,
     &   6.50000000D0, 6.55000000D0, 6.60000000D0, 6.65000000D0,
     &   6.70000000D0, 6.75000000D0, 6.80000000D0, 6.85000000D0,
     &   6.90000000D0, 6.95000000D0, 7.00000000D0, 7.05000000D0,
     &   7.10000000D0, 7.15000000D0, 7.20000000D0, 7.25000000D0,
     &   7.30000000D0, 7.35000000D0, 7.40000000D0, 7.44000000D0,
     &   7.50000000D0, 7.60000000D0, 7.70000000D0, 7.80000000D0,
     &   7.90000000D0, 8.00000000D0, 8.10000000D0, 8.20000000D0,
     &   8.30000000D0, 8.40000000D0, 8.50000000D0, 8.60000000D0,
     &   8.70000000D0, 8.80000000D0, 8.90000000D0, 8.91000000D0,
     &   9.00000000D0, 9.10000000D0, 9.20000000D0, 9.28000000D0,
     &   9.30000000D0, 9.35000000D0, 9.39000000D0, 9.40000000D0,
     &   9.41000000D0, 9.41500000D0, 9.42000000D0, 9.42400000D0,
     &   9.42800000D0, 9.43000000D0, 9.43200000D0, 9.43400000D0/
       DATA  (ESA  (I), I=1301,1400) /
     &   9.43600000D0, 9.43800000D0, 9.44000000D0, 9.44100000D0,
     &   9.44200000D0, 9.44300000D0, 9.44400000D0, 9.44500000D0,
     &   9.44600000D0, 9.44700000D0, 9.44800000D0, 9.44900000D0,
     &   9.44937000D0, 9.44959000D0, 9.44981000D0, 9.45003000D0,
     &   9.45025000D0, 9.45047000D0, 9.45069000D0, 9.45091000D0,
     &   9.45113000D0, 9.45135000D0, 9.45157000D0, 9.45179000D0,
     &   9.45201000D0, 9.45223000D0, 9.45245000D0, 9.45267000D0,
     &   9.45289000D0, 9.45311000D0, 9.45333000D0, 9.45355000D0,
     &   9.45377000D0, 9.45399000D0, 9.45421000D0, 9.45443000D0,
     &   9.45465000D0, 9.45487000D0, 9.45509000D0, 9.45531000D0,
     &   9.45553000D0, 9.45575000D0, 9.45597000D0, 9.45619000D0,
     &   9.45641000D0, 9.45663000D0, 9.45685000D0, 9.45707000D0,
     &   9.45729000D0, 9.45751000D0, 9.45773000D0, 9.45795000D0,
     &   9.45817000D0, 9.45839000D0, 9.45861000D0, 9.45883000D0,
     &   9.45905000D0, 9.45927000D0, 9.45949000D0, 9.45971000D0,
     &   9.45993000D0, 9.46000000D0, 9.46014595D0, 9.46019268D0,
     &   9.46021366D0, 9.46022606D0, 9.46023464D0, 9.46024132D0,
     &   9.46024704D0, 9.46025085D0, 9.46025467D0, 9.46025848D0,
     &   9.46026039D0, 9.46026325D0, 9.46026516D0, 9.46026802D0,
     &   9.46026897D0, 9.46027088D0, 9.46027279D0, 9.46027470D0,
     &   9.46027565D0, 9.46027660D0, 9.46027851D0, 9.46027946D0,
     &   9.46028042D0, 9.46028233D0, 9.46028328D0, 9.46028423D0,
     &   9.46028519D0, 9.46028709D0, 9.46028805D0, 9.46028900D0,
     &   9.46029091D0, 9.46029186D0, 9.46029377D0, 9.46029568D0,
     &   9.46030045D0, 9.46030426D0, 9.46030617D0, 9.46030807D0/
       DATA  (ESA  (I), I=1401,1500) /
     &   9.46030998D0, 9.46031094D0, 9.46031189D0, 9.46031284D0,
     &   9.46031475D0, 9.46031570D0, 9.46031666D0, 9.46031761D0,
     &   9.46031952D0, 9.46032047D0, 9.46032143D0, 9.46032333D0,
     &   9.46032429D0, 9.46032619D0, 9.46032715D0, 9.46032906D0,
     &   9.46033096D0, 9.46033287D0, 9.46033478D0, 9.46033669D0,
     &   9.46033955D0, 9.46034145D0, 9.46034527D0, 9.46034908D0,
     &   9.46035290D0, 9.46035862D0, 9.46036530D0, 9.46037388D0,
     &   9.46038628D0, 9.46040726D0, 9.46045494D0, 9.46059000D0,
     &   9.46081000D0, 9.46103000D0, 9.46125000D0, 9.46147000D0,
     &   9.46169000D0, 9.46191000D0, 9.46213000D0, 9.46235000D0,
     &   9.46257000D0, 9.46279000D0, 9.46301000D0, 9.46323000D0,
     &   9.46345000D0, 9.46367000D0, 9.46389000D0, 9.46411000D0,
     &   9.46433000D0, 9.46455000D0, 9.46477000D0, 9.46499000D0,
     &   9.46521000D0, 9.46543000D0, 9.46565000D0, 9.46587000D0,
     &   9.46609000D0, 9.46631000D0, 9.46653000D0, 9.46675000D0,
     &   9.46697000D0, 9.46719000D0, 9.46741000D0, 9.46763000D0,
     &   9.46785000D0, 9.46807000D0, 9.46829000D0, 9.46851000D0,
     &   9.46873000D0, 9.46895000D0, 9.46917000D0, 9.46939000D0,
     &   9.46961000D0, 9.46983000D0, 9.47005000D0, 9.47027000D0,
     &   9.47049000D0, 9.47071000D0, 9.47093000D0, 9.47115000D0,
     &   9.47137000D0, 9.47337000D0, 9.47500000D0, 9.47700000D0,
     &   9.48000000D0, 9.48400000D0, 9.49000000D0, 9.49500000D0,
     &   9.50500000D0, 9.51000000D0, 9.51500000D0, 9.52000000D0,
     &   9.52500000D0, 9.53000000D0, 9.54000000D0, 9.55000000D0,
     &   9.60000000D0, 9.70000000D0, 9.80000000D0, 9.90000000D0/
       DATA  (ESA  (I), I=1501,1600) /
     &   9.95000000D0, 9.97500000D0, 9.99000000D0,10.00000000D0,
     &  10.01000000D0,10.01100000D0,10.01200000D0,10.01226000D0,
     &  10.01281000D0,10.01336000D0,10.01391000D0,10.01446000D0,
     &  10.01501000D0,10.01556000D0,10.01611000D0,10.01666000D0,
     &  10.01721000D0,10.01776000D0,10.01831000D0,10.01886000D0,
     &  10.01941000D0,10.01996000D0,10.02051000D0,10.02106000D0,
     &  10.02161000D0,10.02216000D0,10.02250000D0,10.02253750D0,
     &  10.02257500D0,10.02261250D0,10.02265000D0,10.02268750D0,
     &  10.02272500D0,10.02276250D0,10.02280000D0,10.02283750D0,
     &  10.02287500D0,10.02291250D0,10.02295000D0,10.02298750D0,
     &  10.02302500D0,10.02306250D0,10.02310000D0,10.02313750D0,
     &  10.02317500D0,10.02321250D0,10.02322000D0,10.02322500D0,
     &  10.02323000D0,10.02323400D0,10.02323700D0,10.02324000D0,
     &  10.02324200D0,10.02324400D0,10.02324600D0,10.02324800D0,
     &  10.02324930D0,10.02324950D0,10.02324960D0,10.02324980D0,
     &  10.02324988D0,10.02324994D0,10.02325000D0,10.02325060D0,
     &  10.02325150D0,10.02325250D0,10.02325340D0,10.02325440D0,
     &  10.02325540D0,10.02325630D0,10.02325680D0,10.02325730D0,
     &  10.02325780D0,10.02325850D0,10.02325900D0,10.02325950D0,
     &  10.02325980D0,10.02325990D0,10.02325995D0,10.02326000D0,
     &  10.02326005D0,10.02326010D0,10.02326050D0,10.02326100D0,
     &  10.02326150D0,10.02326390D0,10.02326490D0,10.02326680D0,
     &  10.02326770D0,10.02326870D0,10.02326970D0,10.02327060D0,
     &  10.02327160D0,10.02327250D0,10.02327350D0,10.02327440D0,
     &  10.02327540D0,10.02327630D0,10.02327730D0,10.02327820D0/
       DATA  (ESA  (I), I=1601,1700) /
     &  10.02328010D0,10.02328110D0,10.02328210D0,10.02328300D0,
     &  10.02328490D0,10.02328590D0,10.02328750D0,10.02329000D0,
     &  10.02330000D0,10.02331000D0,10.02332000D0,10.02333000D0,
     &  10.02334000D0,10.02335000D0,10.02336250D0,10.02339000D0,
     &  10.02340000D0,10.02341000D0,10.02342000D0,10.02343750D0,
     &  10.02347500D0,10.02351250D0,10.02355000D0,10.02358750D0,
     &  10.02362500D0,10.02366250D0,10.02370000D0,10.02373750D0,
     &  10.02377500D0,10.02381250D0,10.02385000D0,10.02388750D0,
     &  10.02392500D0,10.02396250D0,10.02400000D0,10.02436000D0,
     &  10.02491000D0,10.02546000D0,10.02656000D0,10.02766000D0,
     &  10.02876000D0,10.02986000D0,10.03096000D0,10.03206000D0,
     &  10.03316000D0,10.03426000D0,10.03500000D0,10.03600000D0,
     &  10.03700000D0,10.03800000D0,10.03900000D0,10.04000000D0,
     &  10.04500000D0,10.05000000D0,10.05500000D0,10.06000000D0,
     &  10.06500000D0,10.07000000D0,10.07500000D0,10.08000000D0,
     &  10.20000000D0,10.25000000D0,10.26000000D0,10.27000000D0,
     &  10.28000000D0,10.29000000D0,10.29500000D0,10.30000000D0,
     &  10.31000000D0,10.32000000D0,10.33000000D0,10.34000000D0,
     &  10.34420000D0,10.34493333D0,10.34566667D0,10.34640000D0,
     &  10.34713333D0,10.34786667D0,10.34860000D0,10.34933333D0,
     &  10.35006667D0,10.35080000D0,10.35153333D0,10.35226667D0,
     &  10.35300000D0,10.35309167D0,10.35318333D0,10.35327500D0,
     &  10.35336667D0,10.35345833D0,10.35355000D0,10.35364167D0,
     &  10.35373333D0,10.35382500D0,10.35391667D0,10.35400833D0,
     &  10.35410000D0,10.35419167D0,10.35428333D0,10.35437500D0/
       DATA  (ESA  (I), I=1701,1800) /
     &  10.35440000D0,10.35446667D0,10.35455833D0,10.35465000D0,
     &  10.35474167D0,10.35483334D0,10.35492500D0,10.35501667D0,
     &  10.35510834D0,10.35512330D0,10.35514660D0,10.35515710D0,
     &  10.35516340D0,10.35516780D0,10.35517110D0,10.35517370D0,
     &  10.35517580D0,10.35517760D0,10.35517920D0,10.35518060D0,
     &  10.35518180D0,10.35518290D0,10.35518390D0,10.35518480D0,
     &  10.35518570D0,10.35518650D0,10.35518720D0,10.35518790D0,
     &  10.35518860D0,10.35518930D0,10.35518990D0,10.35519050D0,
     &  10.35519110D0,10.35519170D0,10.35519230D0,10.35519280D0,
     &  10.35519340D0,10.35519400D0,10.35519460D0,10.35519530D0,
     &  10.35519600D0,10.35519680D0,10.35519770D0,10.35519970D0,
     &  10.35520000D0,10.35520100D0,10.35520230D0,10.35520320D0,
     &  10.35520400D0,10.35520470D0,10.35520540D0,10.35520600D0,
     &  10.35520660D0,10.35520720D0,10.35520770D0,10.35520830D0,
     &  10.35520890D0,10.35520950D0,10.35521010D0,10.35521070D0,
     &  10.35521140D0,10.35521210D0,10.35521280D0,10.35521350D0,
     &  10.35521430D0,10.35521520D0,10.35521610D0,10.35521710D0,
     &  10.35521820D0,10.35521940D0,10.35522080D0,10.35522240D0,
     &  10.35522420D0,10.35522630D0,10.35522890D0,10.35523220D0,
     &  10.35523660D0,10.35524290D0,10.35525340D0,10.35527670D0,
     &  10.35529000D0,10.35529300D0,10.35529700D0,10.35531000D0,
     &  10.35532500D0,10.35538334D0,10.35547500D0,10.35556667D0,
     &  10.35565834D0,10.35575000D0,10.35584167D0,10.35593334D0,
     &  10.35602500D0,10.35611667D0,10.35620834D0,10.35630000D0,
     &  10.35639167D0,10.35648334D0,10.35657500D0,10.35666667D0/
       DATA  (ESA  (I), I=1801,1900) /
     &  10.35740000D0,10.35813333D0,10.35886667D0,10.35960000D0,
     &  10.36033333D0,10.36106667D0,10.36180000D0,10.36326667D0,
     &  10.36473333D0,10.36546667D0,10.36620000D0,10.37000000D0,
     &  10.37500000D0,10.38000000D0,10.38500000D0,10.39000000D0,
     &  10.39500000D0,10.40000000D0,10.40500000D0,10.41000000D0,
     &  10.41500000D0,10.42000000D0,10.42500000D0,10.43000000D0,
     &  10.43500000D0,10.44000000D0,10.44500000D0,10.45000000D0,
     &  10.45500000D0,10.46000000D0,10.46500000D0,10.47000000D0,
     &  10.47300000D0,10.52897000D0,10.54217000D0,10.54889000D0,
     &  10.55317000D0,10.55622000D0,10.55855000D0,10.56042000D0,
     &  10.56197000D0,10.56328000D0,10.56442000D0,10.56543000D0,
     &  10.56633000D0,10.56714000D0,10.56788000D0,10.56857000D0,
     &  10.56921000D0,10.56980000D0,10.57036000D0,10.57090000D0,
     &  10.57141000D0,10.57189000D0,10.57237000D0,10.57282000D0,
     &  10.57327000D0,10.57371000D0,10.57415000D0,10.57459000D0,
     &  10.57503000D0,10.57547000D0,10.57594000D0,10.57643000D0,
     &  10.57695000D0,10.57755000D0,10.57829000D0,10.58000000D0,
     &  10.58171000D0,10.58245000D0,10.58305000D0,10.58357000D0,
     &  10.58406000D0,10.58453000D0,10.58497000D0,10.58541000D0,
     &  10.58585000D0,10.58629000D0,10.58673000D0,10.58718000D0,
     &  10.58763000D0,10.58811000D0,10.58859000D0,10.58910000D0,
     &  10.58964000D0,10.59020000D0,10.59079000D0,10.59143000D0,
     &  10.59212000D0,10.59286000D0,10.59367000D0,10.59457000D0,
     &  10.59558000D0,10.59671000D0,10.59803000D0,10.59958000D0,
     &  10.60145000D0,10.60378000D0,10.60683000D0,10.61111000D0/
       DATA  (ESA  (I), I=1901,2000) /
     &  10.61783000D0,10.63103000D0,10.68700000D0,10.69000000D0,
     &  10.71309000D0,10.72975000D0,10.74256000D0,10.75286000D0,
     &  10.76141000D0,10.76869000D0,10.77500000D0,10.78057000D0,
     &  10.78554000D0,10.79004000D0,10.79414000D0,10.79791000D0,
     &  10.80141000D0,10.80468000D0,10.80775000D0,10.81065000D0,
     &  10.81341000D0,10.81604000D0,10.81858000D0,10.82102000D0,
     &  10.82339000D0,10.82570000D0,10.82796000D0,10.83019000D0,
     &  10.83240000D0,10.83460000D0,10.83681000D0,10.83906000D0,
     &  10.84135000D0,10.84374000D0,10.84626000D0,10.84900000D0,
     &  10.85212000D0,10.85601000D0,10.86500000D0,10.87288000D0,
     &  10.87627000D0,10.87895000D0,10.88128000D0,10.88340000D0,
     &  10.88539000D0,10.88728000D0,10.88910000D0,10.89088000D0,
     &  10.89262000D0,10.89434000D0,10.89605000D0,10.89775000D0,
     &  10.89946000D0,10.90119000D0,10.90292000D0,10.90468000D0,
     &  10.90648000D0,10.90830000D0,10.91017000D0,10.91208000D0,
     &  10.91406000D0,10.91609000D0,10.91820000D0,10.92038000D0,
     &  10.92266000D0,10.92504000D0,10.92754000D0,10.93017000D0,
     &  10.93295000D0,10.93590000D0,10.93905000D0,10.94242000D0,
     &  10.94606000D0,10.95000000D0,10.97500000D0,10.97638000D0,
     &  10.97771000D0,10.97900000D0,10.98025000D0,10.98146000D0,
     &  10.98263000D0,10.98377000D0,10.98489000D0,10.98598000D0,
     &  10.98705000D0,10.98810000D0,10.98913000D0,10.99014000D0,
     &  10.99114000D0,10.99213000D0,10.99310000D0,10.99407000D0,
     &  10.99504000D0,10.99600000D0,10.99697000D0,10.99793000D0,
     &  10.99890000D0,10.99988000D0,11.00087000D0,11.00188000D0/
       DATA  (ESA  (I), I=2001,2100) /
     &  11.00290000D0,11.00396000D0,11.00506000D0,11.00621000D0,
     &  11.00742000D0,11.00873000D0,11.01018000D0,11.01186000D0,
     &  11.01399000D0,11.01900000D0,11.02401000D0,11.02614000D0,
     &  11.02782000D0,11.02927000D0,11.03058000D0,11.03179000D0,
     &  11.03294000D0,11.03404000D0,11.03510000D0,11.03612000D0,
     &  11.03713000D0,11.03812000D0,11.03910000D0,11.04007000D0,
     &  11.04103000D0,11.04200000D0,11.04296000D0,11.04392000D0,
     &  11.04490000D0,11.04587000D0,11.04686000D0,11.04786000D0,
     &  11.04887000D0,11.04990000D0,11.05095000D0,11.05202000D0,
     &  11.05311000D0,11.05423000D0,11.05537000D0,11.05654000D0,
     &  11.05775000D0,11.05900000D0,11.06029000D0,11.06162000D0,
     &  11.06300000D0,11.07000000D0,11.09200000D0,11.11500000D0,
     &  11.13700000D0,11.16000000D0,11.18200000D0,11.20500000D0,
     &  11.22700000D0,11.25000000D0,11.27300000D0,11.29500000D0,
     &  11.31800000D0,11.34000000D0,11.50000000D0,11.65500000D0,
     &  11.67700000D0,11.70000000D0,11.72200000D0,11.74500000D0,
     &  11.76700000D0,11.79000000D0,11.81200000D0,11.83500000D0,
     &  11.85800000D0,11.88000000D0,11.90300000D0,11.92500000D0,
     &  11.94800000D0,11.97000000D0,11.99300000D0,12.01500000D0,
     &  12.03800000D0,12.06000000D0,12.08300000D0,12.10500000D0,
     &  12.12700000D0,12.15000000D0,12.17200000D0,12.19500000D0,
     &  12.21700000D0,12.24000000D0,12.26200000D0,12.28500000D0,
     &  12.30700000D0,12.33000000D0,12.35200000D0,12.37500000D0,
     &  12.39800000D0,12.42000000D0,12.44300000D0,12.46500000D0,
     &  12.48800000D0,12.51000000D0,12.53300000D0,12.55500000D0/
       DATA  (ESA  (I), I=2101,2174) /
     &  12.57800000D0,12.60000000D0,12.62300000D0,12.64500000D0,
     &  12.66700000D0,12.69000000D0,12.71200000D0,12.73500000D0,
     &  12.75700000D0,12.78000000D0,12.80200000D0,12.82500000D0,
     &  12.84700000D0,12.87000000D0,12.89200000D0,12.91500000D0,
     &  12.93800000D0,12.96000000D0,12.98300000D0,13.00500000D0,
     &  13.02800000D0,13.05000000D0,13.07300000D0,13.09500000D0,
     &  13.11800000D0,13.14000000D0,13.16300000D0,13.18500000D0,
     &  13.20800000D0,13.23000000D0,13.25200000D0,13.27500000D0,
     &  13.29700000D0,13.32000000D0,13.34200000D0,13.36500000D0,
     &  13.38700000D0,13.41000000D0,13.43200000D0,13.45500000D0,
     &  13.47700000D0,13.50000000D0,13.75000000D0,14.00000000D0,
     &  14.04000000D0,16.00000000D0,17.00000000D0,20.00000000D0,
     &  21.95000000D0,22.05000000D0,25.00000000D0,25.01000000D0,
     &  27.50000000D0,27.60000000D0,27.66000000D0,29.00000000D0,
     &  29.93000000D0,30.10000000D0,30.38000000D0,30.80000000D0,
     &  31.10000000D0,31.29000000D0,33.20000000D0,33.30000000D0,
     &  33.89000000D0,34.00000000D0,34.50000000D0,34.70000000D0,
     &  34.86000000D0,35.01000000D0,35.45000000D0,36.10000000D0,
     &  36.38000000D0,38.29000000D0/

         DATA ((DAS  (I,J), I=   1, 100), J=1,1) /
     &  -9.24518286E-04,-9.83163273E-04,-1.03973327E-03,-1.09717737E-03,
     &  -1.15676923E-03,-1.18683995E-03,-1.23765286E-03,-1.32971094E-03,
     &  -1.36738489E-03,-1.40562062E-03,-1.46379820E-03,-1.52052208E-03,
     &  -1.58454730E-03,-1.62658279E-03,-1.67783592E-03,-1.72973154E-03,
     &  -1.78177607E-03,-1.83381417E-03,-1.87663107E-03,-1.93221696E-03,
     &  -1.97768560E-03,-2.03405855E-03,-2.07897548E-03,-2.13785134E-03,
     &  -2.18657166E-03,-2.23467747E-03,-2.29368167E-03,-2.32793746E-03,
     &  -2.34336858E-03,-2.39443666E-03,-2.44619898E-03,-2.49797915E-03,
     &  -2.55039106E-03,-2.60384231E-03,-2.65986323E-03,-2.71814459E-03,
     &  -2.77649861E-03,-2.83669911E-03,-2.89895960E-03,-2.96366827E-03,
     &  -3.01441433E-03,-3.07707565E-03,-3.14435501E-03,-3.19454680E-03,
     &  -3.26324837E-03,-3.33333152E-03,-3.38809895E-03,-3.46412720E-03,
     &  -3.52200453E-03,-3.60276069E-03,-4.04868870E-03,-4.52808624E-03,
     &  -5.21112285E-03,-6.08582410E-03,-6.38131361E-03,-6.46103156E-03,
     &  -6.69899458E-03,-7.01401898E-03,-7.34601994E-03,-7.68292545E-03,
     &  -8.03746330E-03,-8.16122312E-03,-8.37748424E-03,-8.73233942E-03,
     &  -9.02926372E-03,-9.34006670E-03,-9.59077020E-03,-9.81954789E-03,
     &  -9.85432396E-03,-9.93027445E-03,-9.95988974E-03,-1.02911593E-02,
     &  -9.87982972E-03,-9.63738852E-03,-9.19663335E-03,-8.64140384E-03,
     &  -7.85993960E-03,-6.95388686E-03,-6.12636850E-03,-5.88637483E-03,
     &  -5.47944057E-03,-5.03030364E-03,-4.72064720E-03,-4.55086222E-03,
     &  -4.44445739E-03,-4.67318035E-03,-5.05602101E-03,-5.38985059E-03,
     &  -5.67050056E-03,-5.97420862E-03,-6.22441370E-03,-6.40714646E-03,
     &  -6.56504474E-03,-6.59788398E-03,-6.57041352E-03,-6.52989118E-03,
     &  -6.44163263E-03,-6.35205020E-03,-6.22760565E-03,-6.07254359E-03/

         DATA ((DAS  (I,J), I= 101, 200), J=1,1) /
     &  -5.77442913E-03,-5.49093073E-03,-5.18358566E-03,-4.82350176E-03,
     &  -4.45837502E-03,-3.97424015E-03,-3.49973980E-03,-3.03487397E-03,
     &  -2.45746047E-03,-1.79499413E-03,-1.09241604E-03,-2.01105495E-04,
     &   7.17490063E-04, 1.68610815E-03, 2.92544074E-03, 4.41093300E-03,
     &   6.43321711E-03, 1.14290621E-02, 1.61562652E-02, 1.79343467E-02,
     &   1.92048687E-02, 2.01935358E-02, 2.10169724E-02, 2.16656307E-02,
     &   2.22239740E-02, 2.26759989E-02, 2.30480452E-02, 2.33565018E-02,
     &   2.35704672E-02, 2.37107081E-02, 2.38439981E-02, 2.45769340E-02,
     &   2.39653932E-02, 2.39755990E-02, 2.39168049E-02, 2.38101742E-02,
     &   2.37567122E-02, 2.36197653E-02, 2.35119269E-02, 2.32497702E-02,
     &   2.30906427E-02, 2.27937605E-02, 2.24910854E-02, 2.21322720E-02,
     &   2.17268399E-02, 2.12061222E-02, 2.06235892E-02, 1.99651274E-02,
     &   1.92378963E-02, 1.84065721E-02, 1.75212743E-02, 1.65118092E-02,
     &   1.62987276E-02, 1.57430085E-02, 1.51872894E-02, 1.47409742E-02,
     &   1.44550859E-02, 1.41376813E-02, 1.36300581E-02, 1.35219360E-02,
     &   1.33520985E-02, 1.32079276E-02, 1.31102444E-02, 1.29815927E-02,
     &   1.28915124E-02, 1.27863239E-02, 1.26743780E-02, 1.26154082E-02,
     &   1.22735040E-02, 1.19287613E-02, 1.15851424E-02, 1.12919291E-02,
     &   1.09431912E-02, 1.05867347E-02, 1.02145948E-02, 9.90574778E-03,
     &   9.51008154E-03, 9.09310204E-03, 8.79315027E-03, 8.60799963E-03,
     &   8.25549569E-03, 8.10436102E-03, 7.72885111E-03, 7.41080539E-03,
     &   7.01758897E-03, 6.66912634E-03, 6.23949074E-03, 5.86132416E-03,
     &   5.45624857E-03, 4.94930137E-03, 4.68297096E-03, 4.49905805E-03,
     &   4.11837953E-03, 3.91610477E-03, 3.69227007E-03, 3.45787543E-03,
     &   3.20553048E-03, 2.95861787E-03, 2.69609340E-03, 2.13668683E-03/

         DATA ((DAS  (I,J), I= 201, 300), J=1,1) /
     &   1.50979544E-03, 1.13892979E-03, 9.99559691E-04, 8.59278228E-04,
     &   7.11655698E-04, 5.61611270E-04, 4.21693396E-04, 2.41730668E-04,
     &   6.35933519E-05,-9.61636755E-05,-2.77284391E-04,-4.76897349E-04,
     &  -6.53282286E-04,-8.54228133E-04,-1.05812360E-03,-1.27176613E-03,
     &  -1.49523766E-03,-1.72663578E-03,-1.96387931E-03,-2.20686620E-03,
     &  -2.46991186E-03,-2.66612761E-03,-3.03015036E-03,-3.33653553E-03,
     &  -3.65469550E-03,-3.99523423E-03,-4.35402586E-03,-4.74746741E-03,
     &  -5.15936846E-03,-5.59509868E-03,-5.76948306E-03,-6.02688998E-03,
     &  -6.08416615E-03,-6.70505111E-03,-7.28575824E-03,-7.88697914E-03,
     &  -8.59358982E-03,-9.29241930E-03,-1.01226599E-02,-1.08563984E-02,
     &  -1.17348088E-02,-1.27009585E-02,-1.35865247E-02,-1.49263058E-02,
     &  -1.62660868E-02,-1.77001553E-02,-1.93350209E-02,-2.12189040E-02,
     &  -2.33262975E-02,-2.57474871E-02,-2.85548111E-02,-3.17935894E-02,
     &  -3.55533109E-02,-3.91712269E-02,-4.11473336E-02,-4.24971324E-02,
     &  -4.43608406E-02,-4.52639988E-02,-4.47193897E-02,-4.80231896E-02,
     &  -4.93646865E-02,-5.05576390E-02,-5.16282379E-02,-5.24768425E-02,
     &  -5.30141381E-02,-5.31232661E-02,-5.21923899E-02,-5.05209413E-02,
     &  -4.93389519E-02,-4.60977260E-02,-4.15495192E-02,-3.55459941E-02,
     &  -2.57103631E-02,-1.91242583E-02,-9.07847610E-03, 1.70449777E-03,
     &   1.26742658E-02, 2.32814541E-02, 3.30101684E-02, 4.14817208E-02,
     &   4.85174341E-02, 5.40716928E-02, 5.82076923E-02, 6.10980776E-02,
     &   6.29397712E-02, 6.39158776E-02, 6.42222164E-02, 6.40098716E-02,
     &   6.34061781E-02, 6.25144860E-02, 6.08607184E-02, 6.03804832E-02,
     &   5.80413182E-02, 5.74588273E-02, 5.62039411E-02, 5.48356754E-02,
     &   5.34905314E-02, 5.21743047E-02, 5.08710816E-02, 4.96316971E-02/

         DATA ((DAS  (I,J), I= 301, 400), J=1,1) /
     &   4.94544767E-02, 4.72602116E-02, 4.61335272E-02, 4.50732268E-02,
     &   4.16078780E-02, 3.86779455E-02, 3.60771128E-02, 3.38312310E-02,
     &   3.18772879E-02, 2.98074013E-02, 2.86461994E-02, 2.72987648E-02,
     &   2.60963666E-02, 2.50201753E-02, 2.40424723E-02, 2.31540514E-02,
     &   2.23904865E-02, 2.16442272E-02, 2.09770687E-02, 2.03469568E-02,
     &   1.98599235E-02, 1.93271174E-02, 1.93175610E-02, 1.83662910E-02,
     &   1.79624341E-02, 1.78350672E-02, 1.80593608E-02, 1.76846627E-02,
     &   1.71614858E-02, 1.68086575E-02, 1.64683014E-02, 1.61806448E-02,
     &   1.59048266E-02, 1.56359802E-02, 1.53824836E-02, 1.51457193E-02,
     &   1.49205933E-02, 1.47061815E-02, 1.45070331E-02, 1.43098163E-02,
     &   1.41268293E-02, 1.39470049E-02, 1.37744708E-02, 1.36307738E-02,
     &   1.34479920E-02, 1.32996181E-02, 1.31489757E-02, 1.29897533E-02,
     &   1.28354392E-02, 1.27113409E-02, 1.25894256E-02, 1.24619576E-02,
     &   1.23345713E-02, 1.22152186E-02, 1.20923304E-02, 1.19730566E-02,
     &   1.16585430E-02, 1.09697557E-02, 9.87123666E-03, 9.04581953E-03,
     &   8.42304755E-03, 7.90128531E-03, 7.45699214E-03, 7.07182474E-03,
     &   6.73107913E-03, 6.69925922E-03, 6.29362094E-03, 6.01560278E-03,
     &   5.92643802E-03, 5.69147141E-03, 5.48961920E-03, 5.44992292E-03,
     &   5.26721543E-03, 5.07448165E-03, 4.98758212E-03, 4.88776120E-03,
     &   4.72799649E-03, 4.57471802E-03, 4.43638981E-03, 4.31007515E-03,
     &   4.24244829E-03, 4.19282557E-03, 4.07000050E-03, 3.94851403E-03,
     &   3.93262863E-03, 3.83883666E-03, 3.74363104E-03, 3.70513871E-03,
     &   3.66076135E-03, 3.57800277E-03, 3.47771343E-03, 3.37448360E-03,
     &   3.33928510E-03, 3.27968673E-03, 3.23668843E-03, 3.20070470E-03,
     &   3.14420613E-03, 3.07876185E-03, 3.07305875E-03, 3.02100782E-03/

         DATA ((DAS  (I,J), I= 401, 500), J=1,1) /
     &   2.95069328E-03, 2.94193881E-03, 2.95342542E-03, 2.99179724E-03,
     &   2.95248152E-03, 2.98845419E-03, 2.99120646E-03, 2.99672452E-03,
     &   3.01082342E-03, 3.04719285E-03, 3.09824867E-03, 3.15661727E-03,
     &   3.16383648E-03, 3.23055519E-03, 3.23815994E-03, 3.27415199E-03,
     &   3.30139516E-03, 3.37247365E-03, 3.37971304E-03, 3.40435652E-03,
     &   3.40565681E-03, 3.40715303E-03, 3.41719010E-03, 3.43645494E-03,
     &   3.44155295E-03, 3.47204151E-03, 3.47475011E-03, 3.48637537E-03,
     &   3.48901424E-03, 3.50702534E-03, 3.51921585E-03, 3.55664556E-03,
     &   3.56442651E-03, 3.64349328E-03, 3.71986114E-03, 3.76397862E-03,
     &   3.77244261E-03, 3.92378957E-03, 3.95180174E-03, 4.11878678E-03,
     &   4.26409490E-03, 4.32460768E-03, 4.36671589E-03, 4.38752015E-03,
     &   4.59847835E-03, 4.64140632E-03, 4.75707923E-03, 4.86173284E-03,
     &   4.90943795E-03, 5.04520404E-03, 5.14354376E-03, 5.16191486E-03,
     &   5.17225787E-03, 5.23276036E-03, 5.25791753E-03, 5.28201029E-03,
     &   5.38514136E-03, 5.39776380E-03, 5.41918916E-03, 5.42344232E-03,
     &   5.41903654E-03, 5.41823582E-03, 5.41107880E-03, 5.40422416E-03,
     &   5.39993524E-03, 5.37306539E-03, 5.33724420E-03, 5.29521156E-03,
     &   5.22304476E-03, 5.13990903E-03, 5.13192565E-03, 4.99827618E-03,
     &   4.89679532E-03, 4.79914657E-03, 4.78082929E-03, 4.66309942E-03,
     &   4.65522017E-03, 4.63317349E-03, 4.59207773E-03, 4.60302766E-03,
     &   4.63758082E-03, 4.67276018E-03, 4.68378069E-03, 4.68930326E-03,
     &   4.69683791E-03, 4.67689874E-03, 4.65816527E-03, 4.63850701E-03,
     &   4.60040460E-03, 4.59022960E-03, 4.56251638E-03, 4.55628896E-03,
     &   4.53299480E-03, 4.50597610E-03, 4.48042063E-03, 4.47035565E-03,
     &   4.42480088E-03, 4.33040861E-03, 4.41123799E-03, 4.47154574E-03/

         DATA ((DAS  (I,J), I= 501, 600), J=1,1) /
     &   4.56392022E-03, 4.68040314E-03, 4.84292124E-03, 4.82981647E-03,
     &   4.80384863E-03, 4.70133311E-03, 4.60406072E-03, 4.47661521E-03,
     &   4.32311947E-03, 4.15253980E-03, 3.93752023E-03, 3.66174692E-03,
     &   3.28899284E-03, 2.76030319E-03, 2.00332253E-03, 8.63549375E-04,
     &   5.65312846E-04, 2.34364573E-04,-1.35403212E-04,-5.51430825E-04,
     &  -1.02300647E-03,-1.56217716E-03,-2.18519067E-03,-2.32159008E-03,
     &  -2.46231538E-03,-2.60790666E-03,-2.75805438E-03,-2.91338412E-03,
     &  -3.07404956E-03,-3.24047489E-03,-3.41263474E-03,-3.59113630E-03,
     &  -3.77630312E-03,-4.70996266E-03,-5.42743294E-03,-5.82137086E-03,
     &  -6.09950127E-03,-6.39089177E-03,-6.69638191E-03,-7.01693071E-03,
     &  -7.35356312E-03,-7.70761382E-03,-7.89157676E-03,-8.08044275E-03,
     &  -8.27439499E-03,-8.47363851E-03,-8.67846223E-03,-8.88895153E-03,
     &  -9.10535159E-03,-9.32814187E-03,-9.55749036E-03,-9.79369145E-03,
     &  -1.00370520E-02,-1.02879027E-02,-1.05465964E-02,-1.08135004E-02,
     &  -1.10890175E-02,-1.13735368E-02,-1.16676693E-02,-1.19716573E-02,
     &  -1.22861658E-02,-1.26117154E-02,-1.29488965E-02,-1.32983457E-02,
     &  -1.36607448E-02,-1.40368288E-02,-1.44273844E-02,-1.48332486E-02,
     &  -1.52554081E-02,-1.56948134E-02,-1.61526172E-02,-1.66299262E-02,
     &  -1.71280034E-02,-1.76482362E-02,-1.81921837E-02,-1.87615133E-02,
     &  -1.93579245E-02,-1.99835502E-02,-2.06404383E-02,-2.13310600E-02,
     &  -2.20580819E-02,-2.28244994E-02,-2.32235137E-02,-2.36336277E-02,
     &  -2.40553121E-02,-2.44890658E-02,-2.49354113E-02,-2.53949068E-02,
     &  -2.58680746E-02,-2.63557519E-02,-2.68583808E-02,-2.73766827E-02,
     &  -2.79116299E-02,-2.84635161E-02,-2.90337515E-02,-2.96231836E-02,
     &  -3.02325658E-02,-3.08630122E-02,-3.15156350E-02,-3.21916264E-02/

         DATA ((DAS  (I,J), I= 601, 700), J=1,1) /
     &  -3.28922918E-02,-3.36189786E-02,-3.43731805E-02,-3.51564794E-02,
     &  -3.59705893E-02,-3.68173683E-02,-3.76988308E-02,-3.86171443E-02,
     &  -3.95746711E-02,-4.05739791E-02,-4.16181755E-02,-4.27097459E-02,
     &  -4.38522486E-02,-4.50493843E-02,-4.63051790E-02,-4.76239912E-02,
     &  -4.90108054E-02,-5.04709662E-02,-5.20104527E-02,-5.36359228E-02,
     &  -5.60702110E-02,-5.71753346E-02,-5.91068783E-02,-6.11599238E-02,
     &  -6.33462936E-02,-6.56794115E-02,-6.81745809E-02,-7.08493149E-02,
     &  -7.37237407E-02,-7.68211152E-02,-8.01684566E-02,-8.37974153E-02,
     &  -8.77449848E-02,-9.08968340E-02,-1.00719797E-01,-1.12758474E-01,
     &  -1.27861534E-01,-1.47376295E-01,-1.73588473E-01,-2.10762176E-01,
     &  -2.69962298E-01,-3.61313092E-01,-5.53191372E-01,-1.16864961E+00,
     &  -1.23606870E+00,-1.31168325E+00,-1.39720822E+00,-1.49422872E+00,
     &  -1.60578740E+00,-1.73517164E+00,-1.88699172E+00,-2.06758321E+00,
     &  -2.28588512E+00,-2.55491028E+00,-2.89446905E+00,-3.32145128E+00,
     &  -3.92890628E+00,-3.99969969E+00,-5.58487812E+00,-6.74021817E+00,
     &  -7.66851107E+00,-4.78259072E+00,-9.10031207E+00,-9.29780365E+00,
     &  -1.00727027E+01,-1.05094629E+01,-1.08127636E+01,-1.10928083E+01,
     &  -1.12873799E+01,-1.14439701E+01,-1.15294510E+01,-1.15903172E+01,
     &  -1.15936807E+01,-1.15583713E+01,-1.14661116E+01,-1.13674138E+01,
     &  -1.11539529E+01,-1.09618687E+01,-1.07248052E+01,-1.02319891E+01,
     &  -9.74959628E+00,-9.21933470E+00,-8.64427971E+00,-8.72241965E+00,
     &  -8.13809120E+00,-7.49013854E+00,-6.78009262E+00,-5.60549086E+00,
     &  -4.75633794E+00,-3.43344604E+00,-1.96701496E+00, 1.98011221E+00,
     &   5.61479380E+00, 6.78772049E+00, 7.82827522E+00, 8.44393310E+00,
     &   9.24842306E+00, 9.24625460E+00, 9.77034605E+00, 1.02456613E+01/

         DATA ((DAS  (I,J), I= 701, 800), J=1,1) /
     &   1.06696673E+01, 1.09722372E+01, 1.12017112E+01, 1.13188104E+01,
     &   1.14352826E+01, 1.15191688E+01, 1.15740556E+01, 1.16090016E+01,
     &   8.99902819E+00, 9.52139914E+00, 1.14861647E+01, 1.14179143E+01,
     &   1.12488035E+01, 1.10994901E+01, 1.08792265E+01, 1.06431582E+01,
     &   1.03364476E+01, 1.00887307E+01, 9.35410879E+00, 9.16898080E+00,
     &   8.65188081E+00, 8.06915475E+00, 7.31967551E+00, 6.39417621E+00,
     &   5.38757346E+00, 4.44047241E+00, 3.70479172E+00, 3.17394605E+00,
     &   2.77424667E+00, 2.46302439E+00, 2.21410593E+00, 2.01062683E+00,
     &   1.84131448E+00, 1.69814597E+00, 1.57563928E+00, 1.46960683E+00,
     &   1.37694836E+00, 1.29529012E+00, 1.22278852E+00, 1.15798904E+00,
     &   1.09972893E+00, 1.04706801E+00, 9.99237560E-01, 5.26727199E-01,
     &   3.58967301E-01, 2.73212883E-01, 2.22452341E-01, 1.85284141E-01,
     &   1.59702138E-01, 1.40586093E-01, 1.25724194E-01, 1.13829650E-01,
     &   1.04091466E-01, 1.02938680E-01, 9.49042849E-02, 8.81022239E-02,
     &   8.22687626E-02, 7.72105697E-02, 7.27825255E-02, 6.88736807E-02,
     &   6.53975946E-02, 6.22862547E-02, 5.94850986E-02, 5.69497322E-02,
     &   5.46440958E-02, 5.25382675E-02, 5.06073117E-02, 4.88303047E-02,
     &   4.71895254E-02, 4.56697420E-02, 4.42583757E-02, 4.29437714E-02,
     &   4.17167513E-02, 4.05684896E-02, 3.94917210E-02, 3.84799643E-02,
     &   3.75274982E-02, 3.66292504E-02, 3.57807146E-02, 3.49779714E-02,
     &   3.42172297E-02, 3.34953428E-02, 3.28092886E-02, 3.21566468E-02,
     &   3.15349496E-02, 3.09420414E-02, 3.03759638E-02, 2.98349486E-02,
     &   2.93173477E-02, 2.88216765E-02, 2.83464078E-02, 2.78908485E-02,
     &   2.74531664E-02, 2.70324388E-02, 2.66279658E-02, 2.62386843E-02,
     &   2.58637645E-02, 2.55024126E-02, 2.51539110E-02, 2.48175840E-02/

         DATA ((DAS  (I,J), I= 801, 900), J=1,1) /
     &   2.44926924E-02, 2.41791382E-02, 2.38757391E-02, 2.35821043E-02,
     &   2.32980535E-02, 2.30230052E-02, 2.27565376E-02, 2.24982548E-02,
     &   2.22477920E-02, 2.20048057E-02, 2.17689733E-02, 2.15399935E-02,
     &   2.13175252E-02, 2.11014783E-02, 2.08914297E-02, 2.06872056E-02,
     &   2.04885726E-02, 2.02953269E-02, 2.01072744E-02, 1.99242338E-02,
     &   1.97460257E-02, 1.95725157E-02, 1.94036497E-02, 1.92389844E-02,
     &   1.90787335E-02, 1.89226813E-02, 1.87707296E-02, 1.86228142E-02,
     &   1.84788893E-02, 1.83389309E-02, 1.82029451E-02, 1.80710399E-02,
     &   1.79431768E-02, 1.78195398E-02, 1.77004988E-02, 1.75864017E-02,
     &   1.73024562E-02, 1.71796956E-02, 1.70594206E-02, 1.69415533E-02,
     &   1.68260184E-02, 1.67128412E-02, 1.66017537E-02, 1.64926854E-02,
     &   1.63858018E-02, 1.62912857E-02, 1.61779428E-02, 1.60768949E-02,
     &   1.59776842E-02, 1.58802614E-02, 1.57846473E-02, 1.56906335E-02,
     &   1.55982624E-02, 1.55073219E-02, 1.54182071E-02, 1.53305037E-02,
     &   1.26931883E-02, 1.08165084E-02, 9.13955590E-03, 7.63378004E-03,
     &   6.64070663E-03, 6.17956925E-03, 5.64458169E-03, 4.99730184E-03,
     &   4.39897933E-03, 3.64993977E-03, 3.35650671E-03, 2.68302355E-03,
     &   2.30669325E-03, 1.87621045E-03, 1.37507068E-03, 7.91833316E-04,
     &   1.09408837E-04,-6.99140856E-04,-1.67188377E-03,-2.86723904E-03,
     &  -4.37596886E-03,-5.09827341E-03,-5.90751123E-03,-6.82104784E-03,
     &  -7.86119503E-03,-9.05710608E-03,-1.04476318E-02,-1.12316825E-02,
     &  -1.20856945E-02,-1.30195687E-02,-1.40452780E-02,-1.51773050E-02,
     &  -1.64331080E-02,-1.78346452E-02,-1.94090593E-02,-2.11907689E-02,
     &  -2.32239786E-02,-2.55666175E-02,-2.82953911E-02,-3.13753709E-02,
     &  -3.33915346E-02,-3.56287949E-02,-3.81258396E-02,-4.09309614E-02/

         DATA ((DAS  (I,J), I= 901,1000), J=1,1) /
     &  -4.41052442E-02,-4.77271096E-02,-5.18989585E-02,-5.67571316E-02,
     &  -6.24872875E-02,-6.93491892E-02,-7.77177205E-02,-8.81558481E-02,
     &  -1.01550581E-01,-1.19392578E-01,-1.44414474E-01,-1.82361331E-01,
     &  -2.49483081E-01,-3.68204477E-01,-5.38191671E-01,-6.36770173E-01,
     &  -7.44666968E-01,-1.01077742E+00,-1.10696463E+00,-1.13765500E+00,
     &  -1.25300425E+00,-1.29728004E+00,-1.35644103E+00,-1.39641270E+00,
     &  -1.40540349E+00,-1.43140332E+00,-1.47272940E+00,-1.48690585E+00,
     &  -1.49529185E+00,-1.49827910E+00,-1.49604922E+00,-1.49009413E+00,
     &  -1.47807528E+00,-1.46239909E+00,-1.44318323E+00,-1.41686011E+00,
     &  -1.38575996E+00,-1.34589729E+00,-1.30553675E+00,-1.25918154E+00,
     &  -1.20650886E+00,-1.13675288E+00,-1.06965516E+00,-9.82754669E-01,
     &  -9.00934376E-01,-7.81646594E-01,-6.51465174E-01,-4.57036135E-01,
     &  -2.87561008E-01,-9.23504854E-02, 6.52787473E-03, 1.05407361E-01,
     &   2.03648999E-01, 4.70118720E-01, 6.64576282E-01, 7.94783351E-01,
     &   9.14100067E-01, 9.95943643E-01, 1.08287272E+00, 1.14999556E+00,
     &   1.21978137E+00, 1.27247966E+00, 1.31886029E+00, 1.35924930E+00,
     &   1.39914024E+00, 1.43023127E+00, 1.45658098E+00, 1.47585391E+00,
     &   1.49155384E+00, 1.50359839E+00, 1.50957482E+00, 1.51183043E+00,
     &   1.50886718E+00, 1.50050589E+00, 1.48635454E+00, 1.44486971E+00,
     &   1.41890372E+00, 1.41011057E+00, 1.37016350E+00, 1.31089533E+00,
     &   1.26677824E+00, 1.15194169E+00, 1.12077787E+00, 1.02461224E+00,
     &   9.08347055E-01, 9.71467839E-01, 8.32712378E-01, 7.47360886E-01,
     &   6.95809157E-01, 5.89251885E-01, 5.52095076E-01, 4.75128596E-01,
     &   5.77029016E-01, 2.22325508E-01, 1.74055434E-01, 1.43951440E-01,
     &   1.23167581E-01, 1.07902837E-01, 9.61972144E-02, 8.69268975E-02/

         DATA ((DAS  (I,J), I=1001,1100), J=1,1) /
     &   7.93978963E-02, 7.31570962E-02, 6.78943264E-02, 6.33819713E-02,
     &   5.94937966E-02, 5.61565351E-02, 5.31984395E-02, 5.05690787E-02,
     &   4.82202830E-02, 4.61107427E-02, 4.27591535E-02, 3.85757198E-02,
     &   3.63331676E-02, 3.26594914E-02, 2.97753943E-02, 2.69306145E-02,
     &   2.28747190E-02, 1.82908933E-02, 1.54237853E-02, 1.40905863E-02,
     &   1.33984909E-02, 1.28867605E-02, 1.25093169E-02, 1.22031825E-02,
     &   1.19774899E-02, 1.17799569E-02, 1.16132380E-02, 1.14709618E-02,
     &   1.13477126E-02, 1.12674174E-02, 1.11455868E-02, 1.10625505E-02,
     &   1.09893635E-02, 1.09248898E-02, 1.08684000E-02, 1.08188647E-02,
     &   1.07757709E-02, 1.07386942E-02, 1.07072427E-02, 1.06812543E-02,
     &   1.06260111E-02, 1.06452020E-02, 1.06352398E-02, 1.06309918E-02,
     &   1.06316312E-02, 1.06406897E-02, 1.06590589E-02, 1.06857261E-02,
     &   1.07253157E-02, 1.07839667E-02, 1.08783267E-02, 1.11799367E-02,
     &   1.15767147E-02, 1.17654030E-02, 1.19185728E-02, 1.20523833E-02,
     &   1.21732870E-02, 1.22837823E-02, 1.23862197E-02, 1.24812572E-02,
     &   1.25698037E-02, 1.26530060E-02, 1.27281670E-02, 1.27987004E-02,
     &   1.28230879E-02, 1.28885373E-02, 1.29412081E-02, 1.29959481E-02,
     &   1.30334902E-02, 1.30724593E-02, 1.30870036E-02, 1.30928793E-02,
     &   1.30885798E-02, 1.30730649E-02, 1.30451851E-02, 1.30034506E-02,
     &   1.29390458E-02, 1.28611985E-02, 1.27724099E-02, 1.26583083E-02,
     &   1.25160550E-02, 1.23369356E-02, 1.21131144E-02, 1.18272470E-02,
     &   1.14417666E-02, 1.08869870E-02, 1.07213175E-02, 1.02813060E-02,
     &   9.62017343E-03, 9.55201826E-03, 9.09453255E-03, 8.90841719E-03,
     &   8.76024905E-03, 8.63426928E-03, 8.43850232E-03, 8.23057470E-03,
     &   8.18543762E-03, 8.17904197E-03, 8.14313012E-03, 8.07845760E-03/

         DATA ((DAS  (I,J), I=1101,1200), J=1,1) /
     &   8.04283694E-03, 8.05275031E-03, 8.18771025E-03, 8.17212286E-03,
     &   8.02250564E-03, 7.96166265E-03, 7.84465483E-03, 7.74406766E-03,
     &   7.64348048E-03, 7.54289331E-03, 7.47132975E-03, 7.29555941E-03,
     &   7.03254374E-03, 6.67077138E-03, 6.43997950E-03, 6.08622161E-03,
     &   5.85435227E-03, 5.97369777E-03, 6.52472939E-03, 6.90189716E-03,
     &   7.75177896E-03, 8.11002477E-03, 8.40281928E-03, 8.72909234E-03,
     &   8.98183767E-03, 8.92896214E-03, 8.86181191E-03, 8.99195638E-03,
     &   9.37337970E-03, 9.09226108E-03, 8.74129626E-03, 8.80198483E-03,
     &   8.91954681E-03, 9.00311882E-03, 9.12421427E-03, 9.13868562E-03,
     &   9.05460111E-03, 8.87618712E-03, 8.76976319E-03, 8.56589741E-03,
     &   8.53999370E-03, 8.67531406E-03, 8.82827984E-03, 9.01931626E-03,
     &   9.10360753E-03, 9.28215824E-03, 9.35714211E-03, 9.25315675E-03,
     &   9.35133172E-03, 9.57377823E-03, 9.81010731E-03, 1.00494217E-02,
     &   1.01915334E-02, 1.03120393E-02, 1.04611032E-02, 1.05748076E-02,
     &   1.06566907E-02, 1.07437967E-02, 1.08676985E-02, 1.09744552E-02,
     &   1.10679604E-02, 1.11643953E-02, 1.10064861E-02, 1.05165639E-02,
     &   1.04089433E-02, 1.04846794E-02, 1.06442223E-02, 1.04544677E-02,
     &   1.04513264E-02, 1.05856234E-02, 1.06706844E-02, 9.93173108E-03,
     &   9.14658807E-03, 8.88346986E-03, 8.69440511E-03, 8.99075544E-03,
     &   8.99659782E-03, 8.87031158E-03, 8.75953880E-03, 8.69506801E-03,
     &   8.60356439E-03, 8.48170287E-03, 8.14273003E-03, 8.16534822E-03,
     &   8.19290182E-03, 8.13049647E-03, 8.10785094E-03, 8.09304168E-03,
     &   8.14285980E-03, 8.16804060E-03, 8.18187871E-03, 8.21930305E-03,
     &   8.20617001E-03, 8.24690794E-03, 8.62538291E-03, 9.02948284E-03,
     &   9.14295775E-03, 9.17646130E-03, 9.19612799E-03, 9.06531049E-03/

         DATA ((DAS  (I,J), I=1201,1300), J=1,1) /
     &   9.33188415E-03, 9.66132040E-03, 9.78712917E-03, 9.83928357E-03,
     &   9.92921554E-03, 9.94243424E-03, 9.85098298E-03, 9.69035413E-03,
     &   9.63392330E-03, 9.74328087E-03, 9.74207032E-03, 9.41406213E-03,
     &   9.42797111E-03, 9.59244709E-03, 9.55243987E-03, 9.37548876E-03,
     &   9.28575421E-03, 9.31647492E-03, 9.41607302E-03, 9.42097966E-03,
     &   9.64234826E-03, 9.74077973E-03, 9.96504860E-03, 1.02033343E-02,
     &   1.02129823E-02, 1.02214953E-02, 1.02243329E-02, 1.03502662E-02,
     &   1.05396223E-02, 1.06192815E-02, 1.06931262E-02, 1.08286089E-02,
     &   1.09525207E-02, 1.10111865E-02, 1.10680174E-02, 1.11232105E-02,
     &   1.11769241E-02, 1.12292883E-02, 1.12804112E-02, 1.13303835E-02,
     &   1.13792826E-02, 1.14271751E-02, 1.14741182E-02, 1.15201619E-02,
     &   1.15653501E-02, 1.16097211E-02, 1.16533087E-02, 1.16961434E-02,
     &   1.17382513E-02, 1.17796561E-02, 1.18203788E-02, 1.18604377E-02,
     &   1.18998495E-02, 1.19386283E-02, 1.19767868E-02, 1.20143360E-02,
     &   1.20512850E-02, 1.20876419E-02, 1.21234130E-02, 1.21586033E-02,
     &   1.21932166E-02, 1.22272552E-02, 1.22607202E-02, 1.22936113E-02,
     &   1.23259267E-02, 1.23576634E-02, 1.23888169E-02, 1.24133156E-02,
     &   1.24493476E-02, 1.25074490E-02, 1.25630202E-02, 1.26159225E-02,
     &   1.26659705E-02, 1.27129195E-02, 1.27564456E-02, 1.27961186E-02,
     &   1.28313609E-02, 1.28613819E-02, 1.28850746E-02, 1.29008386E-02,
     &   1.29062764E-02, 1.28975914E-02, 1.28683894E-02, 1.28639599E-02,
     &   1.28068925E-02, 1.26885429E-02, 1.24515410E-02, 1.20477361E-02,
     &   1.18802871E-02, 1.11870475E-02, 9.90530823E-03, 9.31521207E-03,
     &   8.48773322E-03, 7.93557426E-03, 7.24510687E-03, 6.55456745E-03,
     &   5.69156998E-03, 5.17397096E-03, 4.58268470E-03, 3.90083905E-03/

         DATA ((DAS  (I,J), I=1301,1400), J=1,1) /
     &   3.10599157E-03, 2.16762322E-03, 1.04314270E-03, 3.92951987E-04,
     &  -3.28749054E-04,-1.13442790E-03,-2.03963814E-03,-3.06404701E-03,
     &  -4.23291490E-03,-5.57930191E-03,-7.14756295E-03,-8.99985030E-03,
     &  -9.77506914E-03,-1.02639497E-02,-1.07778229E-02,-1.13321449E-02,
     &  -1.18646910E-02,-1.24172679E-02,-1.11311178E-02,-1.36229861E-02,
     &  -1.42726805E-02,-1.49554067E-02,-1.56733055E-02,-1.64288938E-02,
     &  -1.72250382E-02,-1.80649635E-02,-1.89521872E-02,-1.98910509E-02,
     &  -2.08858092E-02,-2.19416688E-02,-2.30643918E-02,-2.42604966E-02,
     &  -2.55373840E-02,-2.69034899E-02,-2.83684714E-02,-2.99434211E-02,
     &  -3.16412215E-02,-3.34767485E-02,-3.54674584E-02,-3.76338695E-02,
     &  -4.00002850E-02,-4.25957045E-02,-4.54550037E-02,-4.86204265E-02,
     &  -5.21434655E-02,-5.60909663E-02,-6.05408778E-02,-6.55971895E-02,
     &  -7.13926933E-02,-7.81022386E-02,-8.59604684E-02,-9.52898481E-02,
     &  -1.06545966E-01,-1.20394154E-01,-1.37845763E-01, 6.69083134E-05,
     &  -1.91159551E-01,-2.34870209E-01,-3.02260915E-01,-4.19671011E-01,
     &  -6.75160979E-01,-8.33558131E-01,-1.60031791E+00,-2.23180749E+00,
     &  -2.69035418E+00,-3.04398656E+00,-3.18391260E-03,-3.58971786E+00,
     &  -3.82635540E+00,-3.99104561E+00,-4.15823571E+00,-4.32296586E+00,
     &  -4.40237913E+00,-4.51367323E+00,-4.58108062E+00,-4.66462913E+00,
     &  -4.68476986E+00,-4.72175409E+00,-4.72996468E+00,-4.71994856E+00,
     &  -4.70468092E+00,-4.68163820E+00,-5.22498695E-02,-5.09860178E-02,
     &  -4.49471298E+00,-4.33282027E+00,-4.23231671E+00,-4.11902021E+00,
     &  -3.98839700E+00,-3.67781882E+00,-3.49673853E+00,-3.30050144E+00,
     &  -2.85443848E+00,-2.60752059E+00,-7.92182715E-01,-1.46726513E+00,
     &   1.71693728E-01, 1.47372337E+00, 2.07347172E+00, 2.61604858E+00/

         DATA ((DAS  (I,J), I=1401,1500), J=1,1) /
     &   3.09819083E+00, 3.31492834E+00, 3.51224044E+00, 3.69252383E+00,
     &   4.00461518E+00, 4.13575655E+00, 4.25288203E+00, 4.36075558E+00,
     &   4.51911020E+00, 4.58083675E+00, 4.63324903E+00, 4.70707117E+00,
     &   4.73090756E+00, 4.75525235E+00, 4.75744273E+00, 4.74484197E+00,
     &   4.71366392E+00, 4.73397423E+00, 4.69232069E+00, 4.54444358E+00,
     &   4.43581860E+00, 4.35607578E+00, 4.18834389E+00, 4.02128256E+00,
     &   3.85648951E+00, 3.61974342E+00, 3.36502349E+00, 3.07340611E+00,
     &   2.71876056E+00, 2.26043311E+00, 1.61890839E+00, 8.89797282E-01,
     &   5.14831853E-01, 3.64330651E-01, 2.83301491E-01, 2.32683394E-01,
     &   1.98066916E-01, 1.72946828E-01, 1.53783924E-01, 1.38767042E-01,
     &   1.26660148E-01, 1.16692237E-01, 1.08342611E-01, 1.01246902E-01,
     &   9.51424724E-02, 8.98352627E-02, 3.82725693E-02, 8.10622061E-02,
     &   7.73902783E-02, 7.41030818E-02, 7.11385942E-02, 6.84524393E-02,
     &   6.60072853E-02, 6.37721321E-02, 6.17210874E-02, 5.98323492E-02,
     &   5.80874310E-02, 1.76262197E-02, 5.49681715E-02, 5.35685855E-02,
     &   5.22616455E-02, 5.10385012E-02, 4.98914025E-02, 4.88135363E-02,
     &   4.77988927E-02, 4.68421552E-02, 4.59386108E-02, 4.50840798E-02,
     &   4.42748634E-02, 4.35077159E-02, 4.27798636E-02, 4.20891289E-02,
     &   4.14344606E-02, 4.08190750E-02, 4.02399805E-02, 3.96207615E-02,
     &   3.90533611E-02, 3.85152991E-02, 3.80020136E-02, 3.75110481E-02,
     &   3.70406315E-02, 3.35118060E-02, 3.13621125E-02, 2.93130437E-02,
     &   2.70505371E-02, 2.49933064E-02, 2.31841735E-02, 2.27414083E-02,
     &   2.10876242E-02, 1.86003285E-02, 1.70107431E-02, 1.69370148E-02,
     &   1.67787257E-02, 1.66033242E-02, 1.62694976E-02, 1.59824849E-02,
     &   1.50685125E-02, 1.42614241E-02, 1.37874866E-02, 1.31554998E-02/

         DATA ((DAS  (I,J), I=1501,1600), J=1,1) /
     &   1.24085394E-02, 1.15485218E-02, 1.04438322E-02, 8.93086442E-03,
     &   5.15177661E-03, 4.43567023E-03, 3.59249049E-03, 3.34817153E-03,
     &   2.79130750E-03, 2.17260125E-03, 1.48113192E-03, 7.03244889E-04,
     &  -1.78361072E-04,-1.18593887E-03,-2.34860227E-03,-3.70519306E-03,
     &  -5.30871900E-03,-7.23346930E-03,-9.58690859E-03,-1.25305821E-02,
     &  -1.63192135E-02,-2.13799262E-02,-2.84920624E-02,-3.93165066E-02,
     &  -5.69716039E-02,-9.22525819E-02,-1.39647847E-01,-1.47604607E-01,
     &  -1.56431446E-01,-1.66279180E-01,-1.77335584E-01,-1.89837231E-01,
     &  -2.04088178E-01,-2.20480687E-01,-2.39537466E-01,-2.61963990E-01,
     &  -2.88738931E-01,-3.21259646E-01,-3.61592239E-01,-4.12924311E-01,
     &  -4.80442236E-01,-5.73168529E-01,-7.08288136E-01,-9.22812345E-01,
     &  -1.31212384E+00,-2.19287750E+00,-2.50151500E+00,-2.63581040E+00,
     &  -3.01379068E+00,-3.24384336E+00,-3.41045266E+00,-2.95906728E+00,
     &  -2.59669017E+00,-3.91407764E+00,-3.78249448E+00,-2.55806976E-01,
     &  -1.35633297E+00,-1.32470939E+00,-1.30893768E+00,-3.13350313E+00,
     &  -3.11476745E+00,-3.10059648E+00,-3.08632429E+00,-2.93822521E+00,
     &  -2.69918713E+00,-2.41312815E+00,-2.14056059E+00,-2.25602909E+00,
     &  -1.91303580E+00,-1.57410414E+00,-1.37502332E+00,-1.18432132E+00,
     &  -9.71618165E-01,-6.64812605E-01,-4.40805579E-01, 4.03484916E-02,
     &   4.04516653E-02, 4.04860319E-02, 4.05032105E-02, 4.05203861E-02,
     &   4.05375586E-02, 4.05547281E-02, 4.06919721E-02, 4.08632482E-02,
     &   4.10342131E-02, 1.69391297E+00, 2.04553323E+00, 2.64191959E+00,
     &   2.87271088E+00, 3.08906797E+00, 3.25010693E+00, 3.16479212E+00,
     &   3.36737010E+00, 3.55500188E+00, 3.61120328E+00, 3.64298494E+00,
     &   3.65299601E+00, 3.65233809E+00, 3.64083884E+00, 3.62269299E+00/

         DATA ((DAS  (I,J), I=1601,1700), J=1,1) /
     &   2.63439830E+00, 3.52692087E+00, 3.48332853E+00, 3.44023395E+00,
     &   3.33209171E+00, 3.27610782E+00, 3.18518221E+00, 3.03981405E+00,
     &   2.52930906E+00, 2.13152812E+00, 1.82978362E+00, 1.59827554E+00,
     &   1.41653523E+00, 1.24440924E+00, 1.08048831E+00, 8.98152660E-01,
     &   8.36678715E-01, 7.83070930E-01, 7.35841176E-01, 6.66019024E-01,
     &   5.53655762E-01, 4.74186111E-01, 4.15050578E-01, 3.69348591E-01,
     &   3.32974157E-01, 3.03343263E-01, 2.78741251E-01, 2.57989023E-01,
     &   2.40250286E-01, 2.24912807E-01, 2.11520384E-01, 1.99725657E-01,
     &   1.89258692E-01, 1.79906806E-01, 1.71502634E-01, 1.19951409E-01,
     &   8.46176659E-02, 6.69579238E-02, 4.93267073E-02, 4.03695181E-02,
     &   3.50533894E-02, 3.15171275E-02, 2.89935126E-02, 2.71015562E-02,
     &   2.56302337E-02, 2.44531429E-02, 2.37853148E-02, 2.30059723E-02,
     &   2.23399046E-02, 2.17640360E-02, 2.12611606E-02, 2.08181867E-02,
     &   1.92123409E-02, 1.82037060E-02, 1.75097868E-02, 1.70017764E-02,
     &   1.66120751E-02, 1.62988691E-02, 1.60493236E-02, 1.58417057E-02,
     &   1.40707276E-02, 1.35786273E-02, 1.34547045E-02, 1.33112934E-02,
     &   1.31406533E-02, 1.29289181E-02, 1.28012806E-02, 1.26543499E-02,
     &   1.22766422E-02, 1.17084500E-02, 1.07211504E-02, 8.48256186E-03,
     &   6.34709164E-03, 5.79618213E-03, 5.16083706E-03, 4.41994376E-03,
     &   3.54466774E-03, 2.49461122E-03, 1.21142085E-03,-3.92605964E-04,
     &  -2.45560717E-03,-5.20871762E-03,-9.07334563E-03,-1.48940700E-02,
     &  -2.44733004E-02,-2.61434276E-02,-2.79654555E-02,-2.99614775E-02,
     &  -3.21578458E-02,-3.45930093E-02,-3.72809951E-02,-4.02952996E-02,
     &  -4.36860951E-02,-4.75294575E-02,-5.19219051E-02,-5.69895126E-02,
     &  -6.29022453E-02,-6.98898069E-02,-7.82736343E-02,-8.85200733E-02/

         DATA ((DAS  (I,J), I=1701,1800), J=1,1) /
     &  -9.17192904E-02,-1.01328818E-01,-1.17791908E-01,-1.39740186E-01,
     &  -1.70457246E-01,-2.16501486E-01,-2.93143202E-01,-4.45928563E-01,
     &  -8.97510173E-01,-1.06966182E+00,-1.51446773E+00,-1.73265685E+00,
     &  -2.13150583E+00,-2.37430812E+00,-2.59074782E+00,-2.78691559E+00,
     &  -2.95565258E+00,-3.19359447E+00,-3.26464309E+00, 3.18724268E-02,
     &  -3.53991076E+00,-3.65459803E+00,-3.75204021E+00,-3.83271619E+00,
     &  -2.65844562E+00,-3.96760324E+00,-4.01704727E+00,-4.06233537E+00,
     &  -4.10177333E+00,-4.13231876E+00,-4.20147763E+00,-4.13878303E+00,
     &  -4.11191470E+00,-3.85222792E+00,-3.63282132E+00, 8.68043076E-02,
     &  -3.82147953E+00,-3.65343599E+00,-3.44391584E+00,-3.14610794E+00,
     &  -2.79216939E+00,-2.32404970E+00,-1.77843889E+00, 8.19433772E-01,
     &   1.39495558E-02, 8.25148383E-01, 1.80643857E+00, 2.35401619E+00,
     &   2.82764431E+00, 3.18507317E+00, 2.68975797E+00, 2.98405021E+00,
     &   3.26784258E+00, 3.53740920E+00, 3.74822083E+00, 3.98086173E+00,
     &   4.18724666E+00, 1.58820768E+00, 1.72857372E+00, 1.87474122E+00,
     &   2.98432123E-01, 2.24046449E+00, 2.43703498E+00, 2.64285108E+00,
     &   2.88887727E+00, 3.17769358E+00, 3.47564603E+00, 3.81014466E+00,
     &   3.61022280E+00, 3.33769426E+00, 3.06325883E+00, 2.81906207E+00,
     &   2.63610651E+00, 1.03205755E+00, 2.62695568E+00, 2.39841897E+00,
     &   2.16102305E+00, 1.88017495E+00, 1.54241814E+00, 1.09761705E+00,
     &   9.41858960E-01, 9.12645129E-01, 6.13873948E-01, 7.76309142E-01,
     &   6.86125717E-01, 4.73859172E-01, 3.21099028E-01, 2.44454151E-01,
     &   1.98408891E-01, 1.67695781E-01, 1.45747316E-01, 1.29282497E-01,
     &   1.16475953E-01, 1.06228482E-01, 9.78436071E-02, 9.08565551E-02,
     &   8.49440957E-02, 7.98746817E-02, 7.54826777E-02, 7.16388797E-02/

         DATA ((DAS  (I,J), I=1801,1900), J=1,1) /
     &   5.24195973E-02, 4.28097745E-02, 3.70449165E-02, 3.32073947E-02,
     &   3.04540556E-02, 2.83782818E-02, 2.67697600E-02, 2.44327104E-02,
     &   2.28148943E-02, 2.21790474E-02, 2.16277576E-02, 1.96427836E-02,
     &   1.81847080E-02, 1.73077391E-02, 1.67191476E-02, 1.62944315E-02,
     &   1.59715734E-02, 1.57161225E-02, 1.55072724E-02, 1.53311665E-02,
     &   1.50417214E-02, 1.50545591E-02, 1.49438769E-02, 1.48435568E-02,
     &   1.47517323E-02, 1.46669496E-02, 1.45873623E-02, 1.45116811E-02,
     &   1.44385387E-02, 1.43663189E-02, 1.42926371E-02, 1.42111434E-02,
     &   1.41567745E-02, 1.31424324E-02, 1.29032093E-02, 1.26053993E-02,
     &   1.23690896E-02, 1.21738888E-02, 1.21517107E-02, 1.18806167E-02,
     &   1.17539529E-02, 1.16549563E-02, 1.15717439E-02, 1.15027690E-02,
     &   1.14773101E-02, 1.14054827E-02, 1.13755877E-02, 1.13572791E-02,
     &   1.13505583E-02, 1.13549031E-02, 1.13700329E-02, 1.13963588E-02,
     &   1.14263533E-02, 1.14800731E-02, 1.15398362E-02, 1.16086402E-02,
     &   1.16913071E-02, 1.17862148E-02, 1.18958466E-02, 1.20208592E-02,
     &   1.21617195E-02, 1.23186546E-02, 1.25039692E-02, 1.27160749E-02,
     &   1.29610236E-02, 1.32663288E-02, 1.36698620E-02, 1.46560952E-02,
     &   1.55883802E-02, 1.59401410E-02, 1.61950330E-02, 1.63923516E-02,
     &   1.65578011E-02, 1.66979380E-02, 1.68130712E-02, 1.69132811E-02,
     &   1.69993202E-02, 1.70722968E-02, 1.71375542E-02, 1.71823867E-02,
     &   1.72214219E-02, 1.72522947E-02, 1.72731195E-02, 1.72855996E-02,
     &   1.72893405E-02, 1.72843806E-02, 1.72712824E-02, 1.72486637E-02,
     &   1.72167631E-02, 1.71758379E-02, 1.71251693E-02, 1.70470057E-02,
     &   1.69592889E-02, 1.68611502E-02, 1.68076589E-02, 1.66934131E-02,
     &   1.65612401E-02, 1.64042095E-02, 1.62227764E-02, 1.60019734E-02/

         DATA ((DAS  (I,J), I=1901,2000), J=1,1) /
     &   1.57244230E-02, 1.53488794E-02, 1.46934756E-02, 1.46583459E-02,
     &   1.44693175E-02, 1.44168488E-02, 1.43021185E-02, 1.42497390E-02,
     &   1.42077793E-02, 1.41729790E-02, 1.41436689E-02, 1.41186621E-02,
     &   1.40847743E-02, 1.40788524E-02, 1.40631425E-02, 1.40498875E-02,
     &   1.40371194E-02, 1.40295245E-02, 1.40223639E-02, 1.40169836E-02,
     &   1.40133012E-02, 1.40112849E-02, 1.39886779E-02, 1.39916452E-02,
     &   1.40151535E-02, 1.40195135E-02, 1.40265373E-02, 1.40351889E-02,
     &   1.40457689E-02, 1.40585045E-02, 1.40736247E-02, 1.40914873E-02,
     &   1.41123632E-02, 1.41370712E-02, 1.41663500E-02, 1.41989311E-02,
     &   1.42466456E-02, 1.43081206E-02, 1.44663856E-02, 1.45858710E-02,
     &   1.46634932E-02, 1.47051106E-02, 1.47388624E-02, 1.47673808E-02,
     &   1.47921426E-02, 1.48138031E-02, 1.48329269E-02, 1.48499826E-02,
     &   1.48650945E-02, 1.48785458E-02, 1.48904922E-02, 1.49010015E-02,
     &   1.49102486E-02, 1.49183106E-02, 1.49251344E-02, 1.49308736E-02,
     &   1.49355614E-02, 1.49391632E-02, 1.49446373E-02, 1.49435785E-02,
     &   1.49440250E-02, 1.49434664E-02, 1.49420081E-02, 1.49395303E-02,
     &   1.49359972E-02, 1.49313940E-02, 1.49255902E-02, 1.49187614E-02,
     &   1.49107351E-02, 1.49121501E-02, 1.49031258E-02, 1.48922908E-02,
     &   1.48791799E-02, 1.48502660E-02, 1.47333605E-02, 1.47589873E-02,
     &   1.47561199E-02, 1.47536611E-02, 1.47516107E-02, 1.47499656E-02,
     &   1.47487193E-02, 1.47478568E-02, 1.47473726E-02, 1.47472608E-02,
     &   1.47452777E-02, 1.47456813E-02, 1.47470855E-02, 1.47030669E-02,
     &   1.47050226E-02, 1.47075028E-02, 1.47547800E-02, 1.47577825E-02,
     &   1.47612445E-02, 1.47651869E-02, 1.47696828E-02, 1.47746545E-02,
     &   1.47802191E-02, 1.47864054E-02, 1.47932406E-02, 1.48008284E-02/

         DATA ((DAS  (I,J), I=2001,2100), J=1,1) /
     &   1.48091259E-02, 1.48184251E-02, 1.48288002E-02, 1.48404250E-02,
     &   1.48534935E-02, 1.48685725E-02, 1.48863249E-02, 1.49081644E-02,
     &   1.49375360E-02, 1.50118429E-02, 1.50862542E-02, 1.51165101E-02,
     &   1.51510026E-02, 1.51726610E-02, 1.51913515E-02, 1.51818072E-02,
     &   1.51946541E-02, 1.52063512E-02, 1.52170675E-02, 1.52268626E-02,
     &   1.52360645E-02, 1.52446102E-02, 1.52526167E-02, 1.52601094E-02,
     &   1.52670849E-02, 1.52736770E-02, 1.52797975E-02, 1.52854533E-02,
     &   1.52911499E-02, 1.52959975E-02, 1.53003011E-02, 1.53045598E-02,
     &   1.53084840E-02, 1.53121150E-02, 1.53154512E-02, 1.53184921E-02,
     &   1.53212365E-02, 1.53237090E-02, 1.53258870E-02, 1.53277910E-02,
     &   1.53305957E-02, 1.53316198E-02, 1.53323783E-02, 1.53328334E-02,
     &   1.53329184E-02, 1.53289602E-02, 1.52881089E-02, 1.52317250E-02,
     &   1.52286787E-02, 1.52036467E-02, 1.51846126E-02, 1.51656897E-02,
     &   1.51503765E-02, 1.51352597E-02, 1.51211327E-02, 1.51054736E-02,
     &   1.50888750E-02, 1.50717939E-02, 1.50502642E-02, 1.50287346E-02,
     &   1.50525118E-02, 1.50753787E-02, 1.50958052E-02, 1.51159832E-02,
     &   1.51343836E-02, 1.51528549E-02, 1.51699158E-02, 1.51872192E-02,
     &   1.52040592E-02, 1.52197939E-02, 1.52359045E-02, 1.52510293E-02,
     &   1.52665772E-02, 1.52812233E-02, 1.52963227E-02, 1.53105814E-02,
     &   1.53253130E-02, 1.53392502E-02, 1.53536730E-02, 1.53673374E-02,
     &   1.53808824E-02, 1.53949238E-02, 1.54082476E-02, 1.54220716E-02,
     &   1.54351995E-02, 1.54488298E-02, 1.54617817E-02, 1.54752370E-02,
     &   1.54880292E-02, 1.55013249E-02, 1.55139707E-02, 1.55271192E-02,
     &   1.55401968E-02, 1.55526417E-02, 1.55655875E-02, 1.55779105E-02,
     &   1.55907329E-02, 1.56029412E-02, 1.56156472E-02, 1.56277473E-02/

         DATA ((DAS  (I,J), I=2101,2174), J=1,1) /
     &   1.56403430E-02, 1.56523404E-02, 1.56648314E-02, 1.56767310E-02,
     &   1.56885844E-02, 1.57009283E-02, 1.57126902E-02, 1.57249405E-02,
     &   1.57366147E-02, 1.57464592E-02, 1.57581038E-02, 1.57702390E-02,
     &   1.57818104E-02, 1.57938706E-02, 1.58053714E-02, 1.58173589E-02,
     &   1.58293102E-02, 1.58407084E-02, 1.58525903E-02, 1.58639232E-02,
     &   1.58757377E-02, 1.58870069E-02, 1.58987558E-02, 1.59100612E-02,
     &   1.59218397E-02, 1.59329880E-02, 1.59446117E-02, 1.59557001E-02,
     &   1.59672619E-02, 1.59782918E-02, 1.59892935E-02, 1.60007655E-02,
     &   1.60117105E-02, 1.60231237E-02, 1.60340130E-02, 1.60453686E-02,
     &   1.60562034E-02, 1.61037455E-02, 1.61145136E-02, 1.61257222E-02,
     &   1.61363969E-02, 1.61475086E-02, 1.62313337E-02, 1.63482348E-02,
     &   1.63666769E-02, 1.71964259E-02, 1.75740848E-02, 1.85730590E-02,
     &   1.92464814E-02, 1.91651068E-02, 1.99228661E-02, 1.99252725E-02,
     &   2.04950774E-02, 2.05168680E-02, 2.05299232E-02, 2.08132880E-02,
     &   2.10021080E-02, 2.10360017E-02, 2.10914023E-02, 2.11735267E-02,
     &   2.12314802E-02, 2.12678835E-02, 2.16212871E-02, 2.16392255E-02,
     &   2.17471949E-02, 2.17665097E-02, 2.20632883E-02, 2.18847832E-02,
     &   2.19443801E-02, 2.19704013E-02, 2.20122075E-02, 2.21204263E-02,
     &   2.21664337E-02, 2.24709762E-02/

         DATA ((DAS  (I,J), I=   1, 100), J=2,2) /
     &   1.67173048E-06, 1.86527564E-06, 2.02572824E-06, 2.19531225E-06,
     &   2.40417280E-06, 2.37117470E-06, 3.73580962E-06, 3.05436283E-06,
     &   3.45445883E-06, 3.92286821E-06, 3.67558783E-06, 3.60309370E-06,
     &   3.44425296E-06, 3.68141600E-06, 3.96208495E-06, 4.27576524E-06,
     &   4.39105564E-06, 4.38758785E-06, 4.36316846E-06, 4.42804554E-06,
     &   4.53705114E-06, 4.63635810E-06, 4.69566561E-06, 4.72965753E-06,
     &   4.79295769E-06, 4.97267017E-06, 5.24803659E-06, 5.36195101E-06,
     &   5.40951614E-06, 5.52317378E-06, 5.64987965E-06, 5.68087273E-06,
     &   5.84069113E-06, 6.00786279E-06, 6.19462134E-06, 6.40798806E-06,
     &   6.63920294E-06, 6.96955001E-06, 7.26751813E-06, 7.57407150E-06,
     &   7.90075647E-06, 8.30584968E-06, 8.77656823E-06, 9.22291466E-06,
     &   9.95937705E-06, 1.10685455E-05, 1.19542387E-05, 1.22778116E-05,
     &   1.22741343E-05, 1.20752817E-05, 1.47559264E-05, 1.78400702E-05,
     &   2.10286233E-05, 2.10645722E-05, 2.37784320E-05, 2.40764985E-05,
     &   2.13331361E-05, 1.91676912E-05, 1.82078811E-05, 1.82530164E-05,
     &   1.88940812E-05, 1.88754433E-05, 1.93270929E-05, 1.93397679E-05,
     &   2.01657252E-05, 2.08764850E-05, 2.18173697E-05, 2.31118548E-05,
     &   2.38775366E-05, 2.44601861E-05, 2.63070247E-05, 2.52163500E-05,
     &   2.58815708E-05, 2.69624136E-05, 2.89691054E-05, 3.00181270E-05,
     &   3.06415663E-05, 3.31421495E-05, 3.66632341E-05, 6.75704287E-05,
     &   6.36014807E-05, 6.68465681E-05, 6.76096025E-05, 6.86231910E-05,
     &   7.03073772E-05, 7.93862973E-05, 8.81809748E-05, 9.36377461E-05,
     &   9.94205986E-05, 1.04467029E-04, 1.08264481E-04, 1.12179756E-04,
     &   1.15562921E-04, 1.18338373E-04, 1.21905584E-04, 1.27167665E-04,
     &   1.27782002E-04, 1.28405554E-04, 1.29271770E-04, 1.28664951E-04/

         DATA ((DAS  (I,J), I= 101, 200), J=2,2) /
     &   1.27915040E-04, 1.26857786E-04, 1.25738903E-04, 1.24611682E-04,
     &   1.23762036E-04, 1.21976086E-04, 1.20225678E-04, 1.18510811E-04,
     &   1.16110064E-04, 1.13502699E-04, 1.10361617E-04, 1.05768769E-04,
     &   1.01035323E-04, 9.60441159E-05, 8.90095160E-05, 7.99509382E-05,
     &   6.75070327E-05, 4.34964510E-05, 5.29035864E-05, 6.52417493E-05,
     &   7.55912773E-05, 8.42538283E-05, 9.16820960E-05, 9.80764520E-05,
     &   1.03465374E-04, 1.07777304E-04, 1.11505537E-04, 1.14661409E-04,
     &   1.17343092E-04, 1.19430756E-04, 1.21182001E-04, 1.27755339E-04,
     &   1.23811802E-04, 1.24757239E-04, 1.25099282E-04, 1.26066410E-04,
     &   1.25258493E-04, 1.25310565E-04, 1.25067239E-04, 1.23651564E-04,
     &   1.22555716E-04, 1.20726213E-04, 1.18582741E-04, 1.15982488E-04,
     &   1.13060819E-04, 1.09418306E-04, 1.05288258E-04, 1.00987482E-04,
     &   9.55659668E-05, 8.94395325E-05, 8.19170679E-05, 7.45857423E-05,
     &   7.28315157E-05, 6.55304312E-05, 5.82293466E-05, 5.40621628E-05,
     &   5.09311448E-05, 4.86162840E-05, 4.31033085E-05, 3.93993618E-05,
     &   3.75009412E-05, 3.62369338E-05, 3.57037100E-05, 3.40128136E-05,
     &   3.23504593E-05, 3.08020208E-05, 2.98276226E-05, 2.92280672E-05,
     &   2.70545708E-05, 2.57738810E-05, 2.47856476E-05, 2.38226146E-05,
     &   2.37479128E-05, 2.34254794E-05, 2.33962221E-05, 2.32454457E-05,
     &   2.27967047E-05, 2.22913554E-05, 2.22121182E-05, 2.20890446E-05,
     &   2.20791733E-05, 2.21185505E-05, 2.22929078E-05, 2.25353982E-05,
     &   2.33537914E-05, 2.39841365E-05, 2.49606960E-05, 2.55962390E-05,
     &   2.65621448E-05, 2.79382639E-05, 2.89836787E-05, 2.97746877E-05,
     &   3.16417806E-05, 3.25968958E-05, 3.34749165E-05, 3.47226618E-05,
     &   3.63809467E-05, 3.86807056E-05, 4.05713163E-05, 4.27307760E-05/

         DATA ((DAS  (I,J), I= 201, 300), J=2,2) /
     &   4.51310451E-05, 4.73332534E-05, 4.82272495E-05, 4.91830036E-05,
     &   5.01978471E-05, 5.12727014E-05, 5.24047431E-05, 5.35400054E-05,
     &   5.46426889E-05, 5.58038481E-05, 5.70255191E-05, 5.83172693E-05,
     &   5.96778658E-05, 6.10532859E-05, 6.24887871E-05, 6.40438311E-05,
     &   6.56854940E-05, 6.74370583E-05, 6.93264589E-05, 7.13682954E-05,
     &   7.35796930E-05, 7.59815332E-05, 7.85962621E-05, 8.14515949E-05,
     &   8.45909303E-05, 8.80478650E-05, 9.18742775E-05, 9.62066469E-05,
     &   1.01257241E-04, 1.07843555E-04, 1.14068842E-04, 1.21778386E-04,
     &   1.12740853E-04, 1.13156383E-04, 1.15764580E-04, 1.19211209E-04,
     &   1.24474041E-04, 1.29257638E-04, 1.34977936E-04, 1.40453900E-04,
     &   1.46958860E-04, 1.54165482E-04, 1.60777319E-04, 1.70933305E-04,
     &   1.81089291E-04, 1.92323494E-04, 2.05023260E-04, 2.19460917E-04,
     &   2.35970077E-04, 2.54955125E-04, 2.76871960E-04, 3.02232387E-04,
     &   3.31853590E-04, 3.60639732E-04, 3.76421256E-04, 3.87179271E-04,
     &   4.01893698E-04, 4.09056082E-04, 4.04821804E-04, 4.31107205E-04,
     &   4.41538853E-04, 4.51031856E-04, 4.59465685E-04, 4.66175150E-04,
     &   4.70388760E-04, 4.71184895E-04, 4.63714557E-04, 4.50361749E-04,
     &   4.40854185E-04, 4.14964186E-04, 3.78665058E-04, 3.30833627E-04,
     &   2.52770931E-04, 2.00851182E-04, 1.22503388E-04, 4.75892670E-05,
     &   6.78576071E-05, 1.46741865E-04, 2.23017703E-04, 2.90223372E-04,
     &   3.46191671E-04, 3.90526369E-04, 4.23525981E-04, 4.46665142E-04,
     &   4.61443585E-04, 4.69344804E-04, 4.71832091E-04, 4.70211905E-04,
     &   4.65528228E-04, 4.58595979E-04, 4.45562105E-04, 4.42016860E-04,
     &   4.23565003E-04, 4.19007101E-04, 4.09214468E-04, 3.98614546E-04,
     &   3.88233692E-04, 3.78211951E-04, 3.68255273E-04, 3.58155641E-04/

         DATA ((DAS  (I,J), I= 301, 400), J=2,2) /
     &   3.56817600E-04, 3.39827902E-04, 3.30901780E-04, 3.22320996E-04,
     &   2.95516838E-04, 2.72689081E-04, 2.53697924E-04, 2.37066995E-04,
     &   2.22554766E-04, 2.06954429E-04, 1.98715302E-04, 1.88885518E-04,
     &   1.80180221E-04, 1.72441215E-04, 1.65539145E-04, 1.59371035E-04,
     &   1.53855546E-04, 1.48894712E-04, 1.44409042E-04, 1.40332130E-04,
     &   1.21594799E-04, 1.17897567E-04, 1.18561505E-04, 1.12527216E-04,
     &   1.12731431E-04, 1.22499700E-04, 1.22350628E-04, 1.08670055E-04,
     &   1.03607551E-04, 1.00520220E-04, 9.79570832E-05, 9.58502406E-05,
     &   9.43777715E-05, 9.21032630E-05, 9.01029492E-05, 8.83893296E-05,
     &   8.66117262E-05, 8.48278654E-05, 8.30804122E-05, 8.13231074E-05,
     &   7.95702477E-05, 7.77525067E-05, 7.59201921E-05, 7.42261482E-05,
     &   7.26028514E-05, 7.08443019E-05, 6.92674889E-05, 6.78356037E-05,
     &   6.64710311E-05, 6.51420864E-05, 6.39021556E-05, 6.26927370E-05,
     &   6.13861054E-05, 6.01317186E-05, 5.89067403E-05, 5.77002839E-05,
     &   5.47617297E-05, 4.89668096E-05, 4.11259772E-05, 3.73667972E-05,
     &   3.43605378E-05, 3.20309680E-05, 3.02557710E-05, 2.89202342E-05,
     &   2.70934648E-05, 2.67378091E-05, 2.45746873E-05, 2.51822829E-05,
     &   2.49480404E-05, 2.39632462E-05, 2.36196997E-05, 2.32566914E-05,
     &   2.29843674E-05, 2.30620753E-05, 2.23095628E-05, 2.33496411E-05,
     &   2.22115573E-05, 2.18706838E-05, 2.22974511E-05, 2.25643922E-05,
     &   2.13852197E-05, 2.20393707E-05, 2.09174634E-05, 2.15111105E-05,
     &   2.11931446E-05, 2.12801858E-05, 2.17159945E-05, 2.10726858E-05,
     &   2.21123082E-05, 2.17309196E-05, 2.14181877E-05, 2.22674959E-05,
     &   2.19354807E-05, 2.36156656E-05, 2.27744247E-05, 2.39713944E-05,
     &   2.36661960E-05, 2.52004762E-05, 2.48852617E-05, 2.50214781E-05/

         DATA ((DAS  (I,J), I= 401, 500), J=2,2) /
     &   2.60657818E-05, 2.78321342E-05, 2.92620848E-05, 2.77848319E-05,
     &   2.72211098E-05, 2.80166498E-05, 2.95297371E-05, 3.07763202E-05,
     &   3.09426714E-05, 2.98875276E-05, 3.08624877E-05, 3.08100436E-05,
     &   3.14936743E-05, 3.41755357E-05, 3.34175488E-05, 3.15869798E-05,
     &   3.24142930E-05, 3.08195633E-05, 3.10890703E-05, 3.33950238E-05,
     &   3.37882139E-05, 3.44879228E-05, 3.56544632E-05, 3.47041344E-05,
     &   3.37030962E-05, 3.50475187E-05, 3.51080338E-05, 3.82989759E-05,
     &   3.83923264E-05, 3.78658184E-05, 3.95370413E-05, 3.79930631E-05,
     &   3.87189361E-05, 3.99307114E-05, 4.20411494E-05, 3.96559647E-05,
     &   3.97596458E-05, 3.89599370E-05, 4.04453965E-05, 4.39520469E-05,
     &   4.32836630E-05, 4.41323233E-05, 4.29159645E-05, 4.31017842E-05,
     &   4.22405666E-05, 4.17979074E-05, 4.60312881E-05, 4.34819065E-05,
     &   4.51729944E-05, 4.23008040E-05, 4.70453916E-05, 4.69069989E-05,
     &   4.70635601E-05, 4.76466155E-05, 4.52712977E-05, 4.28443234E-05,
     &   4.23299365E-05, 4.36153262E-05, 4.66994861E-05, 4.72540821E-05,
     &   4.73870874E-05, 4.69400741E-05, 4.72604881E-05, 4.65447745E-05,
     &   4.71779946E-05, 4.75576259E-05, 4.69286613E-05, 4.86143751E-05,
     &   5.18600199E-05, 5.23858318E-05, 5.32336984E-05, 5.70028263E-05,
     &   6.26705683E-05, 6.91106404E-05, 7.05347327E-05, 8.01662523E-05,
     &   8.31550383E-05, 7.82734542E-05, 7.65761133E-05, 7.83917726E-05,
     &   7.80999603E-05, 9.37552723E-05, 1.01703496E-04, 1.01540942E-04,
     &   8.68171591E-05, 9.17447767E-05, 9.61863919E-05, 1.00581247E-04,
     &   9.07823412E-05, 9.18806559E-05, 9.07797455E-05, 9.12796498E-05,
     &   9.35315746E-05, 9.19372192E-05, 8.45885046E-05, 8.16178221E-05,
     &   8.61478718E-05, 9.55904627E-05, 8.79707265E-05, 9.69231578E-05/

         DATA ((DAS  (I,J), I= 501, 600), J=2,2) /
     &   1.00454676E-04, 1.03268153E-04, 1.07193542E-04, 1.06455552E-04,
     &   1.11070654E-04, 1.14697637E-04, 1.07523711E-04, 1.06665669E-04,
     &   1.11435761E-04, 1.20360905E-04, 1.32567777E-04, 1.48520368E-04,
     &   1.69597578E-04, 1.98641369E-04, 2.40437160E-04, 3.03785483E-04,
     &   3.21296251E-04, 3.39681490E-04, 3.60907570E-04, 3.85065805E-04,
     &   4.12616174E-04, 4.44182501E-04, 4.81009790E-04, 4.89040380E-04,
     &   4.97377150E-04, 5.06018464E-04, 5.14971249E-04, 5.24248525E-04,
     &   5.33695741E-04, 5.43506174E-04, 5.53696918E-04, 5.64291448E-04,
     &   5.75283827E-04, 6.30355519E-04, 6.73168240E-04, 6.96780255E-04,
     &   7.13396135E-04, 7.30791508E-04, 7.49046864E-04, 7.68194562E-04,
     &   7.88313718E-04, 8.09476485E-04, 8.20476388E-04, 8.31776283E-04,
     &   8.43378588E-04, 8.55305125E-04, 8.67559905E-04, 8.80166051E-04,
     &   8.93129347E-04, 9.06474428E-04, 9.20209061E-04, 9.34359633E-04,
     &   9.48936179E-04, 9.63963271E-04, 9.79461923E-04, 9.95454519E-04,
     &   1.01196492E-03, 1.02901858E-03, 1.04664263E-03, 1.06486609E-03,
     &   1.08371997E-03, 1.10323750E-03, 1.12345424E-03, 1.14440838E-03,
     &   1.16614093E-03, 1.18869603E-03, 1.21212117E-03, 1.23646763E-03,
     &   1.26179076E-03, 1.28815044E-03, 1.31561157E-03, 1.34424454E-03,
     &   1.37412695E-03, 1.40534129E-03, 1.43797857E-03, 1.47213826E-03,
     &   1.50792931E-03, 1.54547137E-03, 1.58489602E-03, 1.62634842E-03,
     &   1.66998901E-03, 1.71599563E-03, 1.73994677E-03, 1.76456591E-03,
     &   1.78987996E-03, 1.81592014E-03, 1.84271675E-03, 1.87030459E-03,
     &   1.89871784E-03, 1.92799543E-03, 1.95817608E-03, 1.98930349E-03,
     &   2.02142161E-03, 2.05457977E-03, 2.08882802E-03, 2.12422223E-03,
     &   2.16081965E-03, 2.19868387E-03, 2.23788060E-03, 2.27848260E-03/

         DATA ((DAS  (I,J), I= 601, 700), J=2,2) /
     &   2.32056537E-03, 2.36421318E-03, 2.40951327E-03, 2.45656203E-03,
     &   2.50546263E-03, 2.55632657E-03, 2.60927446E-03, 2.66443701E-03,
     &   2.72195607E-03, 2.78198591E-03, 2.84469452E-03, 2.91026524E-03,
     &   2.97889856E-03, 3.05081398E-03, 3.12625253E-03, 3.20548003E-03,
     &   3.28878871E-03, 3.37650269E-03, 3.46898027E-03, 3.56662080E-03,
     &   3.71284700E-03, 3.77922082E-03, 3.89523532E-03, 4.01854028E-03,
     &   4.14984452E-03, 4.28995299E-03, 4.43978121E-03, 4.60037738E-03,
     &   4.77294482E-03, 4.95887432E-03, 5.15977953E-03, 5.37754606E-03,
     &   5.61438873E-03, 5.80345695E-03, 6.39245068E-03, 7.11373914E-03,
     &   8.01752782E-03, 9.18310487E-03, 1.07434612E-02, 1.29400228E-02,
     &   1.62614430E-02, 2.18688495E-02, 3.33583675E-02, 7.01439942E-02,
     &   7.41730724E-02, 7.86913853E-02, 8.37934447E-02, 8.95996671E-02,
     &   9.62658524E-02, 1.03997123E-01, 1.13068961E-01, 1.23859905E-01,
     &   1.36904108E-01, 1.52979146E-01, 1.73259433E-01, 1.99598364E-01,
     &   2.35077878E-01, 2.39307962E-01, 3.34024927E-01, 4.03061249E-01,
     &   4.58527947E-01, 5.06766968E-01, 5.43938993E-01, 5.76417932E-01,
     &   6.02447538E-01, 6.28378790E-01, 6.46192023E-01, 6.62685604E-01,
     &   6.74324067E-01, 6.83918670E-01, 6.89310363E-01, 6.93334907E-01,
     &   6.93677132E-01, 6.91651665E-01, 6.86034722E-01, 6.79964133E-01,
     &   6.66833368E-01, 6.55068310E-01, 6.40626813E-01, 6.23281718E-01,
     &   6.02815114E-01, 5.79026600E-01, 5.51742899E-01, 5.20828626E-01,
     &   4.86197802E-01, 4.47825557E-01, 4.05759223E-01, 3.36039791E-01,
     &   2.85497986E-01, 2.04480273E-01, 1.18637847E-01, 1.18555986E-01,
     &   3.35968108E-01, 4.05693300E-01, 4.67418718E-01, 5.03925940E-01,
     &   5.51694925E-01, 5.78983329E-01, 6.02776520E-01, 6.23247733E-01/

         DATA ((DAS  (I,J), I= 701, 800), J=2,2) /
     &   6.40597336E-01, 6.55043210E-01, 6.66812498E-01, 6.76134761E-01,
     &   6.83236547E-01, 6.88337231E-01, 6.91645994E-01, 6.93676422E-01,
     &   6.93337286E-01, 6.90694983E-01, 6.85921069E-01, 6.81764406E-01,
     &   6.71596941E-01, 6.62706229E-01, 6.49645833E-01, 6.35642608E-01,
     &   6.17372851E-01, 5.98747853E-01, 5.72776192E-01, 5.47507852E-01,
     &   5.16636025E-01, 4.81817605E-01, 4.37032797E-01, 3.85693131E-01,
     &   3.21587662E-01, 2.64997101E-01, 2.21038934E-01, 1.89320158E-01,
     &   1.65437366E-01, 1.46841166E-01, 1.31967682E-01, 1.19809249E-01,
     &   1.09689287E-01, 1.01137502E-01, 9.38172457E-02, 8.74814005E-02,
     &   8.19445838E-02, 7.70650663E-02, 7.27326740E-02, 6.88604921E-02,
     &   6.53790486E-02, 6.22321566E-02, 5.93738896E-02, 3.11337564E-02,
     &   2.11009172E-02, 1.59614621E-02, 1.28375098E-02, 1.07378837E-02,
     &   9.22976591E-03, 8.09407021E-03, 7.20801310E-03, 6.49743936E-03,
     &   5.91491794E-03, 5.84592507E-03, 5.36484133E-03, 4.95730014E-03,
     &   4.60763723E-03, 4.30434195E-03, 4.03876619E-03, 3.80428657E-03,
     &   3.59574469E-03, 3.40906331E-03, 3.24097722E-03, 3.08884064E-03,
     &   2.95048704E-03, 2.82412525E-03, 2.70826159E-03, 2.60164059E-03,
     &   2.50319927E-03, 2.41203167E-03, 2.32736085E-03, 2.24851673E-03,
     &   2.17491833E-03, 2.10605941E-03, 2.04149686E-03, 1.98084115E-03,
     &   1.92374846E-03, 1.86991416E-03, 1.81906740E-03, 1.77096655E-03,
     &   1.72539534E-03, 1.68215965E-03, 1.64108473E-03, 1.60201284E-03,
     &   1.56480125E-03, 1.52932045E-03, 1.49545272E-03, 1.46309075E-03,
     &   1.43213658E-03, 1.40250055E-03, 1.37410046E-03, 1.34686081E-03,
     &   1.32071215E-03, 1.29559045E-03, 1.27143661E-03, 1.24819597E-03,
     &   1.22581792E-03, 1.20425555E-03, 1.18346525E-03, 1.16340652E-03/

         DATA ((DAS  (I,J), I= 801, 900), J=2,2) /
     &   1.14404161E-03, 1.12533535E-03, 1.10725490E-03, 1.08976959E-03,
     &   1.07285070E-03, 1.05647137E-03, 1.04060638E-03, 1.02523211E-03,
     &   1.01032632E-03, 9.95868129E-04, 9.81837891E-04, 9.68217087E-04,
     &   9.54988265E-04, 9.42134957E-04, 9.29641616E-04, 9.17493551E-04,
     &   9.05676868E-04, 8.94178421E-04, 8.82985762E-04, 8.72087097E-04,
     &   8.61471245E-04, 8.51127599E-04, 8.41046095E-04, 8.31217174E-04,
     &   8.21631761E-04, 8.12281231E-04, 8.03157387E-04, 7.94252441E-04,
     &   7.85558987E-04, 7.77069988E-04, 7.68778761E-04, 7.60678956E-04,
     &   7.52764552E-04, 7.45029846E-04, 7.37469443E-04, 7.30078257E-04,
     &   7.22851530E-04, 7.15784805E-04, 7.08873955E-04, 7.02115202E-04,
     &   6.95505150E-04, 6.89040842E-04, 6.82719827E-04, 6.76540281E-04,
     &   6.70501169E-04, 6.65185155E-04, 6.58677851E-04, 6.52867925E-04,
     &   6.47169529E-04, 6.41579590E-04, 6.36095153E-04, 6.30713368E-04,
     &   6.25431499E-04, 6.20246913E-04, 6.15157079E-04, 6.10158589E-04,
     &   4.67880669E-04, 3.74588958E-04, 3.14758348E-04, 3.05306350E-04,
     &   3.34339000E-04, 3.58292916E-04, 3.92916774E-04, 4.42765144E-04,
     &   4.94468652E-04, 5.64648404E-04, 5.93760928E-04, 6.63380457E-04,
     &   7.05371105E-04, 7.53626154E-04, 8.09380955E-04, 8.74580593E-04,
     &   9.51712939E-04, 1.04425711E-03, 1.15720741E-03, 1.29802175E-03,
     &   1.47824160E-03, 1.56523498E-03, 1.66314996E-03, 1.77414553E-03,
     &   1.90101882E-03, 2.04742534E-03, 2.21821216E-03, 2.31473155E-03,
     &   2.42001294E-03, 2.53529890E-03, 2.66208636E-03, 2.80217872E-03,
     &   2.95778463E-03, 3.13162360E-03, 3.32709657E-03, 3.54850027E-03,
     &   3.80135565E-03, 4.09286506E-03, 4.43261587E-03, 4.81622416E-03,
     &   5.06736684E-03, 5.34604899E-03, 5.65705897E-03, 6.00636383E-03/

         DATA ((DAS  (I,J), I= 901,1000), J=2,2) /
     &   6.40149451E-03, 6.85209469E-03, 7.37071665E-03, 7.97400681E-03,
     &   8.68452227E-03, 9.53360714E-03, 1.05661293E-02, 1.18486457E-02,
     &   1.34842883E-02, 1.56418301E-02, 1.86176132E-02, 2.29835078E-02,
     &   3.00009129E-02, 4.30841380E-02, 6.36615369E-02, 7.53733660E-02,
     &   8.81480097E-02, 1.19579599E-01, 1.30903650E-01, 1.40249638E-01,
     &   1.48186548E-01, 1.54869878E-01, 1.60353207E-01, 1.65074645E-01,
     &   1.68940583E-01, 1.71707439E-01, 1.74058001E-01, 1.75721989E-01,
     &   1.76708933E-01, 1.77065388E-01, 1.76812258E-01, 1.76120585E-01,
     &   1.74716849E-01, 1.72879993E-01, 1.70404012E-01, 1.67218352E-01,
     &   1.63868607E-01, 1.59175386E-01, 1.54430440E-01, 1.48976804E-01,
     &   1.42779476E-01, 1.34571688E-01, 1.26676364E-01, 1.16450558E-01,
     &   1.06822370E-01, 9.27849345E-02, 7.74652562E-02, 5.45843348E-02,
     &   3.46397700E-02, 1.16669403E-02, 1.63038587E-04, 1.16086782E-02,
     &   2.31698166E-02, 5.45291051E-02, 7.74133903E-02, 9.27361034E-02,
     &   1.06776960E-01, 1.16407908E-01, 1.26637084E-01, 1.34535381E-01,
     &   1.42746689E-01, 1.48947041E-01, 1.54403682E-01, 1.59151591E-01,
     &   1.63848190E-01, 1.67200748E-01, 1.70389577E-01, 1.72868595E-01,
     &   1.74708349E-01, 1.76115219E-01, 1.76809479E-01, 1.77065726E-01,
     &   1.76712160E-01, 1.75728169E-01, 1.74067138E-01, 1.71719483E-01,
     &   1.68955234E-01, 1.65092178E-01, 1.60373571E-01, 1.54892939E-01,
     &   1.48212336E-01, 1.40278117E-01, 1.30934757E-01, 1.19613323E-01,
     &   1.05888162E-01, 1.13339671E-01, 9.69555294E-02, 8.68639052E-02,
     &   8.07628423E-02, 6.81242445E-02, 6.37028756E-02, 5.41851379E-02,
     &   3.50851396E-02, 2.58871290E-02, 2.04989443E-02, 1.69644172E-02,
     &   1.44688091E-02, 1.26132829E-02, 1.11798047E-02, 1.00391968E-02/

         DATA ((DAS  (I,J), I=1001,1100), J=2,2) /
     &   9.11008630E-03, 8.33867513E-03, 7.68797663E-03, 7.13172838E-03,
     &   6.65076844E-03, 6.23078799E-03, 5.86088737E-03, 5.53261851E-03,
     &   5.23933192E-03, 4.97572263E-03, 4.55901805E-03, 4.03887524E-03,
     &   3.76068751E-03, 3.30611064E-03, 2.95039189E-03, 2.60159232E-03,
     &   2.10643712E-03, 1.56183194E-03, 1.24009630E-03, 1.10637864E-03,
     &   1.03171244E-03, 9.83100171E-04, 9.48785972E-04, 9.22917649E-04,
     &   9.02620643E-04, 8.86148560E-04, 8.72374501E-04, 8.60626843E-04,
     &   8.50397323E-04, 8.41337297E-04, 8.33201015E-04, 8.25775980E-04,
     &   8.18935749E-04, 8.12544491E-04, 8.06493481E-04, 8.00753644E-04,
     &   7.95220809E-04, 7.89839146E-04, 7.84577776E-04, 7.79390762E-04,
     &   7.74226605E-04, 7.69038703E-04, 7.63781186E-04, 7.58415352E-04,
     &   7.52874048E-04, 7.47084824E-04, 7.40942041E-04, 7.34322434E-04,
     &   7.26992583E-04, 7.18521741E-04, 7.07835344E-04, 6.83810678E-04,
     &   6.63338685E-04, 6.56362146E-04, 6.51752258E-04, 6.48422930E-04,
     &   6.45929157E-04, 6.44037495E-04, 6.42599136E-04, 6.41494491E-04,
     &   6.40646081E-04, 6.39981671E-04, 6.39439240E-04, 6.38948022E-04,
     &   6.38476927E-04, 6.37964929E-04, 6.37252073E-04, 6.36511212E-04,
     &   6.35455841E-04, 6.34153432E-04, 6.32489903E-04, 6.30408540E-04,
     &   6.27860415E-04, 6.24707945E-04, 6.20912116E-04, 6.16365838E-04,
     &   6.10891341E-04, 6.04276661E-04, 5.96732294E-04, 5.87603676E-04,
     &   5.76781041E-04, 5.63672423E-04, 5.47990218E-04, 5.27877454E-04,
     &   5.02461794E-04, 4.67074449E-04, 4.56982506E-04, 4.26550016E-04,
     &   3.84221136E-04, 3.80213629E-04, 3.53423744E-04, 3.44868232E-04,
     &   3.38009416E-04, 3.28429342E-04, 3.20255934E-04, 3.12707700E-04,
     &   3.08383605E-04, 3.05744793E-04, 2.99258263E-04, 2.92691787E-04/

         DATA ((DAS  (I,J), I=1101,1200), J=2,2) /
     &   2.87103919E-04, 2.81783502E-04, 2.76962161E-04, 2.72599553E-04,
     &   2.68449279E-04, 2.64875992E-04, 2.61450447E-04, 2.57311759E-04,
     &   2.53173072E-04, 2.49034384E-04, 2.45556210E-04, 2.41837124E-04,
     &   2.39705855E-04, 2.36914006E-04, 2.37032836E-04, 2.34784660E-04,
     &   2.32366324E-04, 2.30347455E-04, 2.31381089E-04, 2.30907329E-04,
     &   2.30949515E-04, 2.31044923E-04, 2.30143082E-04, 2.36982642E-04,
     &   2.43045738E-04, 2.81442809E-04, 3.23330589E-04, 2.98973236E-04,
     &   2.29619212E-04, 2.36391661E-04, 2.34686693E-04, 2.54928407E-04,
     &   2.48193729E-04, 2.35767695E-04, 2.21975549E-04, 2.16909892E-04,
     &   2.11934221E-04, 2.09238764E-04, 2.08795838E-04, 2.05293668E-04,
     &   2.02796116E-04, 2.02648700E-04, 2.03129938E-04, 2.02320480E-04,
     &   2.02047522E-04, 2.00638245E-04, 1.99739208E-04, 1.98645935E-04,
     &   1.97894651E-04, 1.99905711E-04, 1.99760188E-04, 2.02443196E-04,
     &   2.02101240E-04, 2.26769198E-04, 2.43131462E-04, 2.40264857E-04,
     &   2.28473742E-04, 2.22328572E-04, 2.05185449E-04, 1.98368780E-04,
     &   2.18170270E-04, 2.46468691E-04, 2.46273662E-04, 2.01385701E-04,
     &   2.00703052E-04, 1.98268389E-04, 2.01414400E-04, 1.93019442E-04,
     &   1.91033632E-04, 1.92636135E-04, 1.90867699E-04, 1.86428269E-04,
     &   1.85690283E-04, 1.85254808E-04, 1.88297280E-04, 1.85409071E-04,
     &   1.81686141E-04, 1.81366690E-04, 1.94538804E-04, 1.98432174E-04,
     &   1.97328194E-04, 2.01428867E-04, 1.78245189E-04, 1.86035788E-04,
     &   2.01376699E-04, 1.96978487E-04, 1.95975436E-04, 1.99374875E-04,
     &   2.01654113E-04, 2.06219592E-04, 2.03953286E-04, 1.84184014E-04,
     &   2.05537632E-04, 2.83543599E-04, 2.41293647E-04, 1.87884388E-04,
     &   1.84243810E-04, 1.80796929E-04, 1.76480626E-04, 1.71756764E-04/

         DATA ((DAS  (I,J), I=1201,1300), J=2,2) /
     &   1.73409218E-04, 1.74435830E-04, 1.79726115E-04, 1.77315440E-04,
     &   1.78063231E-04, 1.81589415E-04, 1.77707576E-04, 1.84649175E-04,
     &   1.98088741E-04, 2.35715512E-04, 2.26407044E-04, 2.25046968E-04,
     &   2.51057879E-04, 2.03767746E-04, 1.95890910E-04, 2.00429010E-04,
     &   2.33179273E-04, 2.31337401E-04, 2.67311253E-04, 2.69861380E-04,
     &   3.38226821E-04, 4.24743056E-04, 5.68546850E-04, 7.21338382E-04,
     &   4.83632066E-04, 2.73891199E-04, 2.03977576E-04, 1.58417988E-04,
     &   1.23585105E-04, 1.14798165E-04, 1.08435375E-04, 9.97335450E-05,
     &   9.39523340E-05, 9.17017218E-05, 8.97480070E-05, 8.80289719E-05,
     &   8.64991493E-05, 8.51245309E-05, 8.38749291E-05, 8.27383841E-05,
     &   8.16949818E-05, 8.07319571E-05, 7.98389505E-05, 7.90074571E-05,
     &   7.82304197E-05, 7.75019256E-05, 7.68169778E-05, 7.61713189E-05,
     &   7.55612948E-05, 7.49837485E-05, 7.44359355E-05, 7.39154568E-05,
     &   7.34202045E-05, 7.29483187E-05, 7.24981507E-05, 7.20682347E-05,
     &   7.16572627E-05, 7.12640648E-05, 7.08875925E-05, 7.05269043E-05,
     &   7.01811544E-05, 6.98495821E-05, 6.95315047E-05, 6.92263093E-05,
     &   6.89334481E-05, 6.86524331E-05, 6.83828327E-05, 6.81751149E-05,
     &   6.78764133E-05, 6.74117691E-05, 6.69872717E-05, 6.66021012E-05,
     &   6.62563196E-05, 6.59510162E-05, 6.56885533E-05, 6.54729636E-05,
     &   6.53105870E-05, 6.52111099E-05, 6.51893132E-05, 6.52685819E-05,
     &   6.54845850E-05, 6.58999808E-05, 6.66272137E-05, 6.67241711E-05,
     &   6.78915518E-05, 7.02144735E-05, 7.50976999E-05, 8.44922457E-05,
     &   8.87597462E-05, 1.08210216E-04, 1.48805294E-04, 1.68586401E-04,
     &   1.96959676E-04, 2.16173864E-04, 2.40419839E-04, 2.64846165E-04,
     &   2.95551249E-04, 3.14038053E-04, 3.35206540E-04, 3.59669701E-04/

         DATA ((DAS  (I,J), I=1301,1400), J=2,2) /
     &   3.88242323E-04, 4.22031479E-04, 4.62580507E-04, 4.86047184E-04,
     &   5.12107403E-04, 5.41211122E-04, 5.73918718E-04, 6.10937069E-04,
     &   6.53170774E-04, 7.01796486E-04, 7.58373502E-04, 8.25013065E-04,
     &   8.52773114E-04, 8.70191754E-04, 8.88343584E-04, 9.07275726E-04,
     &   9.27039426E-04, 9.47690518E-04, 9.69289950E-04, 9.91904379E-04,
     &   1.01560686E-03, 1.04047763E-03, 1.06660502E-03, 1.09408648E-03,
     &   1.12302982E-03, 1.15355460E-03, 1.18579377E-03, 1.21989563E-03,
     &   1.25602608E-03, 1.29437134E-03, 1.33514112E-03, 1.37857248E-03,
     &   1.42493440E-03, 1.47453342E-03, 1.52772031E-03, 1.58489846E-03,
     &   1.64653408E-03, 1.71316892E-03, 1.78543620E-03, 1.86408084E-03,
     &   1.94998514E-03, 2.04420200E-03, 2.14799831E-03, 2.26291213E-03,
     &   2.39082940E-03, 2.53408814E-03, 2.69562262E-03, 2.87916623E-03,
     &   3.08954352E-03, 3.33310011E-03, 3.61835340E-03, 3.95700851E-03,
     &   4.36560318E-03, 4.86828835E-03, 5.50177755E-03, 6.32473314E-03,
     &   7.43704188E-03, 9.02371502E-03, 1.14699891E-02, 1.57318679E-02,
     &   2.50059776E-02, 3.07560462E-02, 5.85808557E-02, 8.15111374E-02,
     &   9.81320913E-02, 1.10992660E-01, 1.21559190E-01, 1.30806064E-01,
     &   1.39385874E-01, 1.45358808E-01, 1.51438794E-01, 1.57418406E-01,
     &   1.60301457E-01, 1.64341997E-01, 1.66763042E-01, 1.69768199E-01,
     &   1.70551358E-01, 1.71709751E-01, 1.72191155E-01, 1.71828337E-01,
     &   1.71274731E-01, 1.70438941E-01, 1.67792148E-01, 1.65936062E-01,
     &   1.63654844E-01, 1.57777258E-01, 1.54128266E-01, 1.49957879E-01,
     &   1.45186345E-01, 1.33998039E-01, 1.27427291E-01, 1.20304010E-01,
     &   1.04112197E-01, 9.51492396E-02, 7.54249008E-02, 5.37102599E-02,
     &   5.73466399E-03, 5.29970606E-02, 7.47674190E-02, 9.44624944E-02/

         DATA ((DAS  (I,J), I=1401,1500), J=2,2) /
     &   1.11962894E-01, 1.19830561E-01, 1.26993188E-01, 1.33537706E-01,
     &   1.44866936E-01, 1.49627382E-01, 1.53878951E-01, 1.57560203E-01,
     &   1.63496875E-01, 1.65782920E-01, 1.67685334E-01, 1.70364941E-01,
     &   1.71230204E-01, 1.72114000E-01, 1.72193561E-01, 1.71736231E-01,
     &   1.70604503E-01, 1.68929211E-01, 1.66835279E-01, 1.64422970E-01,
     &   1.60391257E-01, 1.57527222E-01, 1.51535717E-01, 1.45471512E-01,
     &   1.39480472E-01, 1.30895110E-01, 1.21640454E-01, 1.11063898E-01,
     &   9.81906522E-02, 8.15539479E-02, 5.82670592E-02, 3.18016141E-02,
     &   1.81898592E-02, 1.27267619E-02, 9.78544834E-03, 7.94803354E-03,
     &   6.69146538E-03, 5.77798362E-03, 5.08398754E-03, 4.53887322E-03,
     &   4.09938266E-03, 3.73753460E-03, 3.43442846E-03, 3.17683741E-03,
     &   2.95522813E-03, 2.76255576E-03, 2.59350179E-03, 2.44397574E-03,
     &   2.31078014E-03, 2.19137941E-03, 2.08373707E-03, 1.98619866E-03,
     &   1.89740620E-03, 1.81623463E-03, 1.74174389E-03, 1.67314247E-03,
     &   1.60975910E-03, 1.55102081E-03, 1.49643555E-03, 1.44557840E-03,
     &   1.39808050E-03, 1.35362007E-03, 1.31191520E-03, 1.27271777E-03,
     &   1.23580863E-03, 1.20099340E-03, 1.16809915E-03, 1.13697145E-03,
     &   1.10747201E-03, 1.07947661E-03, 1.05287334E-03, 1.02756113E-03,
     &   1.00344847E-03, 9.80452279E-04, 9.58496998E-04, 9.37513725E-04,
     &   9.17439510E-04, 8.98216724E-04, 8.79792510E-04, 8.62118296E-04,
     &   8.45149373E-04, 7.17197987E-04, 6.38799421E-04, 5.63658055E-04,
     &   4.79810681E-04, 4.01448826E-04, 3.24251229E-04, 2.81039634E-04,
     &   2.24706028E-04, 2.05706837E-04, 1.90676098E-04, 1.79099131E-04,
     &   1.69135665E-04, 1.61739997E-04, 1.51858533E-04, 1.42120775E-04,
     &   1.31555401E-04, 1.35514796E-04, 1.48173979E-04, 1.94565351E-04/

         DATA ((DAS  (I,J), I=1501,1600), J=2,2) /
     &   2.86571286E-04, 4.19112546E-04, 6.00845992E-04, 8.54726508E-04,
     &   1.49540810E-03, 1.61714662E-03, 1.76056403E-03, 1.80213233E-03,
     &   1.89689250E-03, 2.00219681E-03, 2.11990500E-03, 2.25234162E-03,
     &   2.40245226E-03, 2.57402371E-03, 2.77200753E-03, 3.00300813E-03,
     &   3.27602807E-03, 3.60367420E-03, 4.00415313E-03, 4.50477733E-03,
     &   5.14846265E-03, 6.00673555E-03, 7.20832927E-03, 9.01070537E-03,
     &   1.20144834E-02, 1.80207887E-02, 2.60778320E-02, 2.74301971E-02,
     &   2.89304025E-02, 3.06040767E-02, 3.24831313E-02, 3.46078095E-02,
     &   3.70295932E-02, 3.98154161E-02, 4.30539235E-02, 4.68650380E-02,
     &   5.14150627E-02, 5.69414494E-02, 6.37952864E-02, 7.25182389E-02,
     &   8.39914335E-02, 9.97485818E-02, 1.22709356E-01, 1.59163213E-01,
     &   2.25318242E-01, 3.74979139E-01, 4.27443258E-01, 4.68736501E-01,
     &   5.14761422E-01, 5.53387382E-01, 5.81243636E-01, 6.04869007E-01,
     &   6.15754650E-01, 6.20076573E-01, 6.14640647E-01, 5.95378742E-01,
     &   5.73201564E-01, 5.69005081E-01, 5.66823419E-01, 5.62290223E-01,
     &   5.60412677E-01, 5.58980136E-01, 5.57526554E-01, 5.41806365E-01,
     &   5.14000194E-01, 4.76793199E-01, 4.37371724E-01, 3.86876818E-01,
     &   3.29501237E-01, 2.72384144E-01, 2.38632336E-01, 2.03605366E-01,
     &   1.67461590E-01, 1.15326165E-01, 7.72598802E-02, 3.87452104E-02,
     &   1.55134227E-02, 7.76002220E-03, 3.88294710E-03, 9.35557749E-05,
     &   3.87503576E-03, 7.75210965E-03, 3.87373140E-02, 7.72520359E-02,
     &   1.15318406E-01, 2.85489469E-01, 3.47407233E-01, 4.46620026E-01,
     &   4.84778291E-01, 5.20628711E-01, 5.49935274E-01, 5.71128080E-01,
     &   5.89452940E-01, 6.01741177E-01, 6.11297218E-01, 6.16690843E-01,
     &   6.19638596E-01, 6.19962469E-01, 6.18159540E-01, 6.14917119E-01/

         DATA ((DAS  (I,J), I=1601,1700), J=2,2) /
     &   6.04199695E-01, 5.96976251E-01, 5.88967567E-01, 5.81245928E-01,
     &   5.63863290E-01, 5.54349920E-01, 5.38898945E-01, 5.14764976E-01,
     &   4.27447755E-01, 3.59805743E-01, 3.08587154E-01, 2.69240477E-01,
     &   2.38356226E-01, 2.13594632E-01, 1.88870516E-01, 1.50268421E-01,
     &   1.39822238E-01, 1.30717908E-01, 1.22715193E-01, 1.10822250E-01,
     &   9.17284378E-02, 7.82241767E-02, 6.81752514E-02, 6.04088100E-02,
     &   5.42278835E-02, 4.91926282E-02, 4.50119177E-02, 4.14854558E-02,
     &   3.84709491E-02, 3.58645456E-02, 3.35886837E-02, 3.15842731E-02,
     &   2.98055082E-02, 2.82163355E-02, 2.67879903E-02, 1.80267206E-02,
     &   1.20204152E-02, 9.01663542E-03, 6.01266011E-03, 4.51070984E-03,
     &   3.60962066E-03, 3.00897474E-03, 2.58000239E-03, 2.25833342E-03,
     &   2.00820440E-03, 1.80815889E-03, 1.69470994E-03, 1.56235302E-03,
     &   1.44930130E-03, 1.35162772E-03, 1.26640574E-03, 1.19140998E-03,
     &   9.20264724E-04, 7.51139041E-04, 6.36065582E-04, 5.53075080E-04,
     &   4.90982915E-04, 4.43790536E-04, 4.04281923E-04, 3.71935377E-04,
     &   1.84720604E-04, 1.77823030E-04, 1.85339582E-04, 1.89573653E-04,
     &   2.00166991E-04, 2.18877504E-04, 2.30016029E-04, 2.46005756E-04,
     &   2.87591720E-04, 3.57783111E-04, 4.87651000E-04, 7.96667840E-04,
     &   1.09672536E-03, 1.17449669E-03, 1.26428911E-03, 1.36910284E-03,
     &   1.49303283E-03, 1.64181691E-03, 1.82373429E-03, 2.05121172E-03,
     &   2.34377638E-03, 2.73394658E-03, 3.28032963E-03, 4.10008458E-03,
     &   5.46652781E-03, 5.70419238E-03, 5.96343635E-03, 6.24740273E-03,
     &   6.55976833E-03, 6.90497743E-03, 7.28858474E-03, 7.71732354E-03,
     &   8.19959891E-03, 8.74623177E-03, 9.37095027E-03, 1.00916855E-02,
     &   1.09326113E-02, 1.19264033E-02, 1.31187603E-02, 1.45761398E-02/

         DATA ((DAS  (I,J), I=1701,1800), J=2,2) /
     &   1.50315311E-02, 1.63977015E-02, 1.87391076E-02, 2.18606283E-02,
     &   2.62292657E-02, 3.27782266E-02, 4.36779304E-02, 6.54074019E-02,
     &   1.29632629E-01, 1.54116378E-01, 2.17377834E-01, 2.65486894E-01,
     &   3.05126786E-01, 3.39724893E-01, 3.70420431E-01, 3.97957299E-01,
     &   4.22554532E-01, 4.45353643E-01, 4.66884662E-01, 4.86561627E-01,
     &   5.03869824E-01, 5.19880106E-01, 5.34315247E-01, 5.46958690E-01,
     &   5.58970946E-01, 5.68810452E-01, 5.76494128E-01, 5.83017783E-01,
     &   5.88039041E-01, 5.91149543E-01, 5.91932102E-01, 5.90609884E-01,
     &   5.86791515E-01, 5.80044584E-01, 5.69900858E-01, 5.58496927E-01,
     &   5.40834506E-01, 5.18365679E-01, 4.90633624E-01, 4.51125436E-01,
     &   4.03548239E-01, 3.39227746E-01, 2.54942608E-01, 3.49292528E-02,
     &   1.03671470E-04, 1.15403881E-01, 2.54938465E-01, 3.39224024E-01,
     &   4.03544953E-01, 4.51122561E-01, 4.90631168E-01, 5.18363581E-01,
     &   5.40832758E-01, 5.58495516E-01, 5.69899717E-01, 5.80043750E-01,
     &   5.86790970E-01, 5.90609608E-01, 5.91932078E-01, 5.91149752E-01,
     &   5.88039499E-01, 5.83018469E-01, 5.76495021E-01, 5.68811533E-01,
     &   5.58972221E-01, 5.46960160E-01, 5.34316889E-01, 5.19881917E-01,
     &   5.03871798E-01, 4.86563754E-01, 4.66886941E-01, 4.45356069E-01,
     &   4.22557094E-01, 3.97959990E-01, 3.70423249E-01, 3.39727833E-01,
     &   3.05129844E-01, 2.65490064E-01, 2.17381113E-01, 1.54119762E-01,
     &   1.31967443E-01, 1.27812577E-01, 1.22658883E-01, 1.08424723E-01,
     &   9.55964063E-02, 6.54073228E-02, 4.36814069E-02, 3.27808136E-02,
     &   2.62321748E-02, 2.18641095E-02, 1.87425888E-02, 1.64009589E-02,
     &   1.45796203E-02, 1.31222402E-02, 1.19297641E-02, 1.09360896E-02,
     &   1.00951630E-02, 9.37435365E-03, 8.74970703E-03, 8.20307294E-03/

         DATA ((DAS  (I,J), I=1801,1900), J=2,2) /
     &   5.46998932E-03, 4.10352867E-03, 3.28375069E-03, 2.73734004E-03,
     &   2.34712992E-03, 2.05453700E-03, 1.82703316E-03, 1.49627895E-03,
     &   1.26748646E-03, 1.17765809E-03, 1.09983823E-03, 8.20604480E-04,
     &   6.17323416E-04, 4.96819744E-04, 4.17425192E-04, 3.61355154E-04,
     &   3.19815893E-04, 2.87936439E-04, 2.62806975E-04, 2.42583357E-04,
     &   2.26049281E-04, 2.12368538E-04, 2.00945697E-04, 1.91354540E-04,
     &   1.83288711E-04, 1.76533118E-04, 1.70950658E-04, 1.66485144E-04,
     &   1.63191900E-04, 1.61338757E-04, 1.59938581E-04, 1.56490776E-04,
     &   1.57911617E-04, 1.84419766E-04, 1.90671464E-04, 2.10695562E-04,
     &   2.28681395E-04, 2.44007144E-04, 2.57560234E-04, 2.69133835E-04,
     &   2.79171474E-04, 2.87722567E-04, 2.94858583E-04, 3.00803027E-04,
     &   3.05543960E-04, 3.09091097E-04, 3.11578193E-04, 3.13038138E-04,
     &   3.13422704E-04, 3.12834305E-04, 3.11261717E-04, 3.08638725E-04,
     &   3.05058264E-04, 3.00588021E-04, 2.94915890E-04, 2.88426022E-04,
     &   2.80723839E-04, 2.71934855E-04, 2.61872775E-04, 2.50507107E-04,
     &   2.37843138E-04, 2.23943836E-04, 2.07842794E-04, 1.89944566E-04,
     &   1.70172588E-04, 1.47429748E-04, 1.22446525E-04, 1.09217188E-04,
     &   1.62868902E-04, 1.90944291E-04, 2.12551001E-04, 2.29800427E-04,
     &   2.44540427E-04, 2.57201721E-04, 2.67702730E-04, 2.76921939E-04,
     &   2.84889837E-04, 2.91668508E-04, 2.97335678E-04, 3.02046926E-04,
     &   3.05769428E-04, 3.08747244E-04, 3.10790232E-04, 3.12077094E-04,
     &   3.12576388E-04, 3.12253489E-04, 3.11152510E-04, 3.09253223E-04,
     &   3.06479498E-04, 3.02896970E-04, 2.98458060E-04, 2.92765969E-04,
     &   2.86378177E-04, 2.79231441E-04, 2.70518034E-04, 2.60570396E-04,
     &   2.49261447E-04, 2.36091459E-04, 2.21254517E-04, 2.03682403E-04/

         DATA ((DAS  (I,J), I=1901,2000), J=2,2) /
     &   1.83792428E-04, 1.58324362E-04, 1.28045623E-04, 1.26422678E-04,
     &   1.16401654E-04, 1.13362452E-04, 1.12633023E-04, 1.12198131E-04,
     &   1.11151893E-04, 1.10655013E-04, 1.10698806E-04, 1.10885785E-04,
     &   1.11131150E-04, 1.11273678E-04, 1.11312712E-04, 1.11409809E-04,
     &   1.11429545E-04, 1.11480549E-04, 1.11472577E-04, 1.11277852E-04,
     &   1.11057649E-04, 1.10723519E-04, 1.10337729E-04, 1.09892247E-04,
     &   1.09367060E-04, 1.08670343E-04, 1.07894467E-04, 1.07034098E-04,
     &   1.06082370E-04, 1.05045494E-04, 1.03931367E-04, 1.02763092E-04,
     &   1.01360841E-04, 9.98847238E-05, 9.85705262E-05, 9.57503331E-05,
     &   9.29549435E-05, 9.06737167E-05, 8.66643196E-05, 8.98588514E-05,
     &   9.21940827E-05, 9.39730705E-05, 9.61399843E-05, 9.80212253E-05,
     &   9.92216499E-05, 1.00459154E-04, 1.01728527E-04, 1.02854118E-04,
     &   1.03936262E-04, 1.04988583E-04, 1.05996421E-04, 1.06951653E-04,
     &   1.07856527E-04, 1.08716229E-04, 1.09517700E-04, 1.10279301E-04,
     &   1.11004852E-04, 1.11693282E-04, 1.12309352E-04, 1.12872392E-04,
     &   1.13426044E-04, 1.13973700E-04, 1.14536624E-04, 1.15086384E-04,
     &   1.15578621E-04, 1.16073971E-04, 1.16672899E-04, 1.17435183E-04,
     &   1.18385110E-04, 1.19632146E-04, 1.21385259E-04, 1.21468903E-04,
     &   1.20211805E-04, 1.18739533E-04, 1.19391474E-04, 1.19896555E-04,
     &   1.20369727E-04, 1.20810285E-04, 1.21202698E-04, 1.21543606E-04,
     &   1.21846230E-04, 1.22111016E-04, 1.22338685E-04, 1.21236108E-04,
     &   1.20126920E-04, 1.19122545E-04, 1.18195826E-04, 1.17309087E-04,
     &   1.16437151E-04, 1.15587088E-04, 1.14758305E-04, 1.13927871E-04,
     &   1.13089937E-04, 1.12230990E-04, 1.11338318E-04, 1.10441463E-04,
     &   1.09535480E-04, 1.08581189E-04, 1.07579294E-04, 1.06619717E-04/

         DATA ((DAS  (I,J), I=2001,2100), J=2,2) /
     &   1.05516680E-04, 1.04521405E-04, 1.03017915E-04, 1.01495952E-04,
     &   1.00044453E-04, 9.87393238E-05, 9.69092545E-05, 9.49276484E-05,
     &   9.31275358E-05, 9.08198286E-05, 9.20473771E-05, 9.34371250E-05,
     &   9.51300490E-05, 9.67343100E-05, 9.78413648E-05, 9.91288131E-05,
     &   1.00509970E-04, 1.01889323E-04, 1.02754573E-04, 1.03743608E-04,
     &   1.04588743E-04, 1.05473820E-04, 1.06324289E-04, 1.07126971E-04,
     &   1.07921923E-04, 1.08709947E-04, 1.09461280E-04, 1.10176416E-04,
     &   1.10894686E-04, 1.11595116E-04, 1.12303906E-04, 1.13019966E-04,
     &   1.13729605E-04, 1.14450347E-04, 1.15214298E-04, 1.16037432E-04,
     &   1.16874317E-04, 1.17134879E-04, 1.17494370E-04, 1.17998292E-04,
     &   1.18719603E-04, 1.19742708E-04, 1.20725275E-04, 1.20184861E-04,
     &   1.19614964E-04, 1.16715536E-04, 1.17543126E-04, 1.18471648E-04,
     &   1.19135527E-04, 1.20536232E-04, 1.22435271E-04, 1.24897253E-04,
     &   1.27647208E-04, 1.30603230E-04, 1.33698225E-04, 1.36619331E-04,
     &   1.39844863E-04, 1.42924215E-04, 1.05700655E-04, 6.84770951E-05,
     &   6.75069804E-05, 6.66979671E-05, 6.60630091E-05, 6.55053624E-05,
     &   6.50489020E-05, 6.46339786E-05, 6.42844189E-05, 6.39588791E-05,
     &   6.36668009E-05, 6.34134657E-05, 6.31715549E-05, 6.29588997E-05,
     &   6.27534335E-05, 6.25709255E-05, 6.23929518E-05, 6.22335531E-05,
     &   6.20769600E-05, 6.19357700E-05, 6.17962252E-05, 6.16697130E-05,
     &   6.15493794E-05, 6.14295945E-05, 6.13202818E-05, 6.12110441E-05,
     &   6.11109980E-05, 6.10106880E-05, 6.09185342E-05, 6.08258716E-05,
     &   6.07405146E-05, 6.06544711E-05, 6.05750248E-05, 6.04947627E-05,
     &   6.04171809E-05, 6.03453289E-05, 6.02725305E-05, 6.02049909E-05,
     &   6.01364486E-05, 6.00727581E-05, 6.00080258E-05, 5.99477905E-05/

         DATA ((DAS  (I,J), I=2101,2174), J=2,2) /
     &   5.98864875E-05, 5.98293700E-05, 5.97711690E-05, 5.97168782E-05,
     &   5.96638752E-05, 5.96097787E-05, 5.95592377E-05, 5.95076033E-05,
     &   5.94593170E-05, 5.94099413E-05, 5.93637271E-05, 5.93164306E-05,
     &   5.92721267E-05, 5.92267500E-05, 5.91842126E-05, 5.91406138E-05,
     &   5.90978737E-05, 5.90577664E-05, 5.90166173E-05, 5.89668893E-05,
     &   5.89271955E-05, 5.88899048E-05, 5.88516033E-05, 5.88156002E-05,
     &   5.87786008E-05, 5.87438031E-05, 5.87080240E-05, 5.86743571E-05,
     &   5.86397239E-05, 5.86071198E-05, 5.85750124E-05, 5.85419611E-05,
     &   5.85108260E-05, 5.84787622E-05, 5.84485449E-05, 5.84174139E-05,
     &   5.83880643E-05, 5.83578159E-05, 5.83292880E-05, 5.82998758E-05,
     &   5.82721269E-05, 5.82435083E-05, 5.79556548E-05, 5.77034614E-05,
     &   5.76658866E-05, 5.63849748E-05, 5.59728390E-05, 5.51676628E-05,
     &   5.48376228E-05, 5.48233109E-05, 5.44820441E-05, 5.44811055E-05,
     &   5.42803486E-05, 5.42734724E-05, 5.42693847E-05, 5.41849584E-05,
     &   5.41332449E-05, 5.41243317E-05, 5.41099915E-05, 5.40892439E-05,
     &   5.40749591E-05, 5.40661331E-05, 5.39859833E-05, 5.39821773E-05,
     &   5.39604279E-05, 5.39565022E-05, 5.39391451E-05, 5.39324186E-05,
     &   5.39271234E-05, 5.39222272E-05, 5.39082333E-05, 5.38885185E-05,
     &   5.38803598E-05, 5.38295261E-05/

         DATA ((DAS  (I,J), I=   1, 100), J=3,3) /
     &   4.32910747E-06, 4.58550308E-06, 4.86820835E-06, 5.17932293E-06,
     &   5.52890501E-06, 5.71326120E-06, 6.23015280E-06, 6.70013747E-06,
     &   7.03018644E-06, 7.35286948E-06, 7.24207826E-06, 7.12976611E-06,
     &   7.07842024E-06, 7.08133723E-06, 7.13627575E-06, 7.23714654E-06,
     &   7.42723803E-06, 7.63652839E-06, 7.81755725E-06, 8.06795680E-06,
     &   8.26660696E-06, 8.48888994E-06, 8.65740925E-06, 8.90288285E-06,
     &   9.12005173E-06, 9.33193196E-06, 9.60758189E-06, 9.76325760E-06,
     &   9.83958723E-06, 1.00859669E-05, 1.03414949E-05, 1.05999179E-05,
     &   1.08656368E-05, 1.11305374E-05, 1.13987370E-05, 1.16711766E-05,
     &   1.19474279E-05, 1.22297085E-05, 1.25205025E-05, 1.28206661E-05,
     &   1.30513899E-05, 1.33653812E-05, 1.36897800E-05, 1.39404263E-05,
     &   1.42857675E-05, 1.46436734E-05, 1.49159990E-05, 1.52806891E-05,
     &   1.55614516E-05, 1.59437806E-05, 1.79673575E-05, 2.03202043E-05,
     &   2.42320360E-05, 3.05786618E-05, 3.31086599E-05, 3.37548205E-05,
     &   3.59024441E-05, 3.94454540E-05, 4.36776004E-05, 4.63794633E-05,
     &   4.90747084E-05, 4.95963093E-05, 4.91915226E-05, 4.97008531E-05,
     &   5.07336321E-05, 5.20800799E-05, 5.30893046E-05, 5.40567683E-05,
     &   5.44922072E-05, 5.53230696E-05, 5.57619289E-05, 5.56291985E-05,
     &   5.65993157E-05, 5.79338691E-05, 5.91043335E-05, 6.16758645E-05,
     &   6.49292400E-05, 7.09670566E-05, 7.96368234E-05, 8.43273212E-05,
     &   1.00409635E-04, 1.04073537E-04, 1.12393032E-04, 1.20955608E-04,
     &   1.37971648E-04, 1.64934178E-04, 1.86467874E-04, 2.03828684E-04,
     &   2.18669276E-04, 2.31302897E-04, 2.42112323E-04, 2.51699511E-04,
     &   2.59983743E-04, 2.66081071E-04, 2.71884384E-04, 2.80444993E-04,
     &   2.82153212E-04, 2.83887054E-04, 2.86295642E-04, 2.86602672E-04/

         DATA ((DAS  (I,J), I= 101, 200), J=3,3) /
     &   2.86213300E-04, 2.85101657E-04, 2.83330196E-04, 2.80840540E-04,
     &   2.77802362E-04, 2.72482867E-04, 2.67269233E-04, 2.62161459E-04,
     &   2.54725093E-04, 2.45979263E-04, 2.35726851E-04, 2.21875793E-04,
     &   2.07600723E-04, 1.92548298E-04, 1.72199909E-04, 1.46570404E-04,
     &   1.12397588E-04, 4.95929313E-05, 1.02944950E-04, 1.37060648E-04,
     &   1.63048328E-04, 1.84096642E-04, 2.01782928E-04, 2.16938992E-04,
     &   2.30005433E-04, 2.41250347E-04, 2.50963013E-04, 2.59421408E-04,
     &   2.66692575E-04, 2.72883344E-04, 2.78045767E-04, 2.94685480E-04,
     &   2.85310936E-04, 2.88160486E-04, 2.89180091E-04, 2.88147807E-04,
     &   2.89722548E-04, 2.88857785E-04, 2.87754612E-04, 2.83749033E-04,
     &   2.81226540E-04, 2.76978103E-04, 2.71920755E-04, 2.65928414E-04,
     &   2.59034999E-04, 2.49966128E-04, 2.39620817E-04, 2.27848334E-04,
     &   2.14532398E-04, 1.99341816E-04, 1.81690526E-04, 1.60741483E-04,
     &   1.55805560E-04, 1.42729855E-04, 1.29654151E-04, 1.20190229E-04,
     &   1.12865892E-04, 1.08196144E-04, 9.42480212E-05, 8.55840357E-05,
     &   7.99771651E-05, 7.25420959E-05, 6.96610912E-05, 6.68853363E-05,
     &   6.43389287E-05, 6.18483311E-05, 5.94336044E-05, 5.83029214E-05,
     &   5.28123625E-05, 4.91017335E-05, 4.64938722E-05, 4.49371463E-05,
     &   4.37278476E-05, 4.27568232E-05, 4.18259949E-05, 4.11409276E-05,
     &   4.05266588E-05, 4.03940526E-05, 4.06509344E-05, 4.08810336E-05,
     &   4.14459822E-05, 4.17459018E-05, 4.24521146E-05, 4.31296345E-05,
     &   4.43467150E-05, 4.57076457E-05, 4.77881913E-05, 4.97785356E-05,
     &   5.20468688E-05, 5.45987645E-05, 5.60102705E-05, 5.67495651E-05,
     &   5.87462546E-05, 5.87774377E-05, 5.78587696E-05, 5.90618701E-05,
     &   6.14266236E-05, 6.42404941E-05, 6.66106090E-05, 6.95670180E-05/

         DATA ((DAS  (I,J), I= 201, 300), J=3,3) /
     &   7.24922893E-05, 7.49326796E-05, 7.59777572E-05, 7.71036810E-05,
     &   7.83159993E-05, 7.96222243E-05, 8.10219231E-05, 8.25293016E-05,
     &   8.41436854E-05, 8.58571704E-05, 8.76548130E-05, 8.95249105E-05,
     &   9.14499694E-05, 9.34293645E-05, 9.54461708E-05, 9.75327137E-05,
     &   9.97256772E-05, 1.01945309E-04, 1.04225468E-04, 1.06595205E-04,
     &   1.09085009E-04, 1.11727990E-04, 1.14575995E-04, 1.17685858E-04,
     &   1.21122794E-04, 1.24970672E-04, 1.29351327E-04, 1.34471099E-04,
     &   1.40855973E-04, 1.50963237E-04, 1.62731638E-04, 1.81088257E-04,
     &   1.58689556E-04, 1.57775825E-04, 1.61529936E-04, 1.66525618E-04,
     &   1.72647338E-04, 1.79348139E-04, 1.87328276E-04, 1.94973814E-04,
     &   2.04015853E-04, 2.14006020E-04, 2.23168828E-04, 2.37148097E-04,
     &   2.51127367E-04, 2.66575024E-04, 2.84028932E-04, 3.03869933E-04,
     &   3.26565992E-04, 3.52689038E-04, 3.82900586E-04, 4.17998280E-04,
     &   4.58668362E-04, 4.98112906E-04, 5.19660630E-04, 5.34335613E-04,
     &   5.54627451E-04, 5.64427781E-04, 5.58519936E-04, 5.94621267E-04,
     &   6.09059472E-04, 6.22196709E-04, 6.33830426E-04, 6.43062765E-04,
     &   6.48878250E-04, 6.50000442E-04, 6.39785434E-04, 6.21504327E-04,
     &   6.08496862E-04, 5.73049917E-04, 5.23377261E-04, 4.57995864E-04,
     &   3.51490547E-04, 2.81108835E-04, 1.76490525E-04, 8.38894596E-05,
     &   1.06938394E-04, 2.08351878E-04, 3.10961193E-04, 4.02327615E-04,
     &   4.78707996E-04, 5.39312271E-04, 5.84450686E-04, 6.16098585E-04,
     &   6.36288874E-04, 6.47085286E-04, 6.50484967E-04, 6.48200643E-04,
     &   6.41670647E-04, 6.32070469E-04, 6.14085861E-04, 6.09014056E-04,
     &   5.83538078E-04, 5.77212931E-04, 5.63564107E-04, 5.48752690E-04,
     &   5.34164766E-04, 5.19839219E-04, 5.05867697E-04, 4.92314828E-04/

         DATA ((DAS  (I,J), I= 301, 400), J=3,3) /
     &   4.90470812E-04, 4.66618210E-04, 4.54498365E-04, 4.42857424E-04,
     &   4.05710280E-04, 3.73678431E-04, 3.46062489E-04, 3.22160863E-04,
     &   3.01366636E-04, 2.79102634E-04, 2.67181482E-04, 2.53047926E-04,
     &   2.40507376E-04, 2.29343295E-04, 2.19382874E-04, 2.10491632E-04,
     &   2.02571871E-04, 1.95523373E-04, 1.89282025E-04, 1.83824743E-04,
     &   1.80778630E-04, 1.77335553E-04, 1.80186174E-04, 1.76789491E-04,
     &   1.90555684E-04, 2.63088973E-04, 2.50123297E-04, 1.70676459E-04,
     &   1.52402542E-04, 1.41652032E-04, 1.34618133E-04, 1.29384348E-04,
     &   1.25405347E-04, 1.22213898E-04, 1.19571965E-04, 1.17331471E-04,
     &   1.15391279E-04, 1.13675709E-04, 1.12125806E-04, 1.10723225E-04,
     &   1.09416751E-04, 1.08083066E-04, 1.06802807E-04, 1.05609980E-04,
     &   1.04455644E-04, 1.03227815E-04, 1.02026217E-04, 1.00899896E-04,
     &   9.97879019E-05, 9.87911945E-05, 9.77273540E-05, 9.66230907E-05,
     &   9.57045261E-05, 9.47382653E-05, 9.37994257E-05, 9.29076579E-05,
     &   9.04609036E-05, 8.44642915E-05, 7.52354482E-05, 7.06136153E-05,
     &   6.88706280E-05, 6.84559391E-05, 6.90810354E-05, 7.05854319E-05,
     &   7.29768683E-05, 7.32730797E-05, 7.81534337E-05, 8.16604020E-05,
     &   8.28287468E-05, 8.60214016E-05, 8.99263759E-05, 9.03318007E-05,
     &   9.39703463E-05, 9.64676377E-05, 9.71752402E-05, 9.77553745E-05,
     &   9.96103079E-05, 1.02659175E-04, 1.06474965E-04, 1.10534374E-04,
     &   1.13109013E-04, 1.14933689E-04, 1.19956894E-04, 1.25675767E-04,
     &   1.26290936E-04, 1.32133872E-04, 1.39409712E-04, 1.43340821E-04,
     &   1.47432681E-04, 1.56761502E-04, 1.67774445E-04, 1.81018571E-04,
     &   1.85488105E-04, 1.96436038E-04, 2.04828190E-04, 2.14057879E-04,
     &   2.36522445E-04, 2.67093124E-04, 2.71083364E-04, 3.08325793E-04/

         DATA ((DAS  (I,J), I= 401, 500), J=3,3) /
     &   3.78609593E-04, 3.93438519E-04, 3.47368502E-04, 3.09796575E-04,
     &   2.97767359E-04, 2.60092350E-04, 2.48602407E-04, 2.42469775E-04,
     &   2.35094328E-04, 2.21890218E-04, 2.08818852E-04, 1.96759459E-04,
     &   1.95468153E-04, 1.84683185E-04, 1.83569716E-04, 1.77975540E-04,
     &   1.73432524E-04, 1.58059952E-04, 1.55483887E-04, 1.38299788E-04,
     &   1.36969631E-04, 1.35646512E-04, 1.22959515E-04, 1.12487772E-04,
     &   1.10723268E-04, 1.01142677E-04, 1.00382266E-04, 9.68503560E-05,
     &   9.61824375E-05, 9.22834219E-05, 9.01516736E-05, 8.38630306E-05,
     &   8.31982011E-05, 7.69792241E-05, 7.30508745E-05, 7.19704076E-05,
     &   7.18618963E-05, 7.35075752E-05, 7.41513027E-05, 7.81639375E-05,
     &   8.14508522E-05, 8.30334014E-05, 8.41183796E-05, 8.46638609E-05,
     &   9.04608054E-05, 9.18613348E-05, 9.65266211E-05, 1.01796026E-04,
     &   1.04689003E-04, 1.16651189E-04, 1.32556696E-04, 1.35972541E-04,
     &   1.37755564E-04, 1.49327591E-04, 1.55527329E-04, 1.61929925E-04,
     &   1.89159232E-04, 1.93052541E-04, 2.03885849E-04, 2.10425806E-04,
     &   2.18741422E-04, 2.19991333E-04, 2.23481391E-04, 2.25710292E-04,
     &   2.27929627E-04, 2.33378641E-04, 2.39622720E-04, 2.46825143E-04,
     &   2.60253100E-04, 2.76002508E-04, 2.77892144E-04, 3.04587239E-04,
     &   3.25641135E-04, 3.50615668E-04, 3.55787095E-04, 3.88528565E-04,
     &   3.88975077E-04, 3.90355305E-04, 3.94241251E-04, 3.63337822E-04,
     &   3.55136685E-04, 3.13324280E-04, 3.00208161E-04, 2.92273203E-04,
     &   2.74857487E-04, 2.61019846E-04, 2.59128958E-04, 2.58565464E-04,
     &   2.59130112E-04, 2.59544509E-04, 2.61049810E-04, 2.61429681E-04,
     &   2.63138519E-04, 2.65556155E-04, 2.68277303E-04, 2.69421661E-04,
     &   2.80167199E-04, 2.83593386E-04, 2.84180527E-04, 2.24961963E-04/

         DATA ((DAS  (I,J), I= 501, 600), J=3,3) /
     &   1.93840984E-04, 1.79519926E-04, 1.59539049E-04, 1.53101831E-04,
     &   1.48136501E-04, 1.40364212E-04, 1.39413689E-04, 1.39036205E-04,
     &   1.39800120E-04, 1.42458848E-04, 1.47224605E-04, 1.54671690E-04,
     &   1.66028109E-04, 1.83373383E-04, 2.09999450E-04, 2.52506479E-04,
     &   2.63929332E-04, 2.76799653E-04, 2.91391211E-04, 3.08017934E-04,
     &   3.27091066E-04, 3.49146502E-04, 3.74891946E-04, 3.80562172E-04,
     &   3.86425990E-04, 3.92493049E-04, 3.98773650E-04, 4.05278803E-04,
     &   4.12020282E-04, 4.19010667E-04, 4.26263582E-04, 4.33793444E-04,
     &   4.41615893E-04, 4.80988540E-04, 5.11605772E-04, 5.28510890E-04,
     &   5.40448289E-04, 5.52967163E-04, 5.66105004E-04, 5.79907800E-04,
     &   5.94426198E-04, 6.09716127E-04, 6.17669478E-04, 6.25839424E-04,
     &   6.34234766E-04, 6.42864816E-04, 6.51739328E-04, 6.60868761E-04,
     &   6.70264158E-04, 6.79937109E-04, 6.89899884E-04, 7.00165492E-04,
     &   7.10747726E-04, 7.21661234E-04, 7.32921578E-04, 7.44545311E-04,
     &   7.56550055E-04, 7.68954588E-04, 7.81778946E-04, 7.95044612E-04,
     &   8.08774186E-04, 8.22992420E-04, 8.37725454E-04, 8.53001431E-04,
     &   8.68850580E-04, 8.85305418E-04, 9.02400965E-04, 9.20175059E-04,
     &   9.38668553E-04, 9.57925568E-04, 9.77993912E-04, 9.98925472E-04,
     &   1.02077666E-03, 1.04360891E-03, 1.06748926E-03, 1.09249100E-03,
     &   1.11869440E-03, 1.14618759E-03, 1.17506755E-03, 1.20544120E-03,
     &   1.23742676E-03, 1.27115526E-03, 1.28871798E-03, 1.30677230E-03,
     &   1.32533904E-03, 1.34444021E-03, 1.36409911E-03, 1.38434042E-03,
     &   1.40519028E-03, 1.42667643E-03, 1.44882832E-03, 1.47167725E-03,
     &   1.49525649E-03, 1.51960150E-03, 1.54475004E-03, 1.57074241E-03,
     &   1.59762165E-03, 1.62543376E-03, 1.65422800E-03, 1.68405715E-03/

         DATA ((DAS  (I,J), I= 601, 700), J=3,3) /
     &   1.71497783E-03, 1.74705089E-03, 1.78034177E-03, 1.81492096E-03,
     &   1.85086451E-03, 1.88825456E-03, 1.92717999E-03, 1.96773710E-03,
     &   2.01003043E-03, 2.05417363E-03, 2.10029049E-03, 2.14851612E-03,
     &   2.19899822E-03, 2.25189862E-03, 2.30739502E-03, 2.36568294E-03,
     &   2.42697804E-03, 2.49151882E-03, 2.55956962E-03, 2.63142435E-03,
     &   2.73904260E-03, 2.78789481E-03, 2.87328791E-03, 2.96405255E-03,
     &   3.06071140E-03, 3.16385719E-03, 3.27416490E-03, 3.39240650E-03,
     &   3.51946904E-03, 3.65637691E-03, 3.80431952E-03, 3.96468586E-03,
     &   4.13910830E-03, 4.27835252E-03, 4.71215759E-03, 5.24344213E-03,
     &   5.90919994E-03, 6.76785354E-03, 7.91739628E-03, 9.53572096E-03,
     &   1.19828878E-02, 1.61144648E-02, 2.45802283E-02, 5.16852289E-02,
     &   5.46540136E-02, 5.79832870E-02, 6.17426894E-02, 6.60209488E-02,
     &   7.09328650E-02, 7.66295808E-02, 8.33140832E-02, 9.12652957E-02,
     &   1.00876804E-01, 1.12721559E-01, 1.27664918E-01, 1.47072542E-01,
     &   1.73215333E-01, 1.76332235E-01, 2.46123668E-01, 2.96992530E-01,
     &   3.37862725E-01, 3.73407264E-01, 4.00797176E-01, 4.24729024E-01,
     &   4.43908733E-01, 4.63015971E-01, 4.76141510E-01, 4.88294675E-01,
     &   4.96870384E-01, 5.03940092E-01, 5.07912918E-01, 5.10878371E-01,
     &   5.11130536E-01, 5.09638087E-01, 5.05499287E-01, 5.01026222E-01,
     &   4.91350922E-01, 4.82681932E-01, 4.72040829E-01, 4.59260233E-01,
     &   4.44179579E-01, 4.26651201E-01, 4.06547422E-01, 3.83768485E-01,
     &   3.58251038E-01, 3.29976755E-01, 2.98980511E-01, 2.47608305E-01,
     &   2.10366982E-01, 1.50669737E-01, 8.74174683E-02, 8.73571495E-02,
     &   2.47555486E-01, 2.98931937E-01, 3.44413819E-01, 3.71313875E-01,
     &   4.06512073E-01, 4.26619317E-01, 4.44151141E-01, 4.59235192E-01/

         DATA ((DAS  (I,J), I= 701, 800), J=3,3) /
     &   4.72019109E-01, 4.82663438E-01, 4.91335544E-01, 4.98204579E-01,
     &   5.03437474E-01, 5.07195873E-01, 5.09633909E-01, 5.11130014E-01,
     &   5.10880124E-01, 5.08933164E-01, 5.05415543E-01, 5.02352739E-01,
     &   4.94860923E-01, 4.88309872E-01, 4.78686422E-01, 4.68368258E-01,
     &   4.54906332E-01, 4.41182650E-01, 4.22045637E-01, 4.03426861E-01,
     &   3.80679201E-01, 3.55023525E-01, 3.22024195E-01, 2.84194972E-01,
     &   2.36959369E-01, 1.95261070E-01, 1.62870851E-01, 1.39499131E-01,
     &   1.21901294E-01, 1.08198840E-01, 9.72394409E-02, 8.82806057E-02,
     &   8.08238013E-02, 7.45224957E-02, 6.91286325E-02, 6.44601248E-02,
     &   6.03803750E-02, 5.67849508E-02, 5.35926716E-02, 5.07394949E-02,
     &   4.81742306E-02, 4.58554779E-02, 4.37493963E-02, 2.29410728E-02,
     &   1.55486513E-02, 1.17618821E-02, 9.46022125E-03, 7.91332809E-03,
     &   6.80228408E-03, 5.96565734E-03, 5.31297581E-03, 4.78959903E-03,
     &   4.36057764E-03, 4.30976782E-03, 3.95549352E-03, 3.65540952E-03,
     &   3.39797372E-03, 3.17470460E-03, 2.97922984E-03, 2.80666910E-03,
     &   2.65322144E-03, 2.51588259E-03, 2.39224662E-03, 2.28036401E-03,
     &   2.17863837E-03, 2.08574992E-03, 2.00059806E-03, 1.92225774E-03,
     &   1.84994570E-03, 1.78299439E-03, 1.72083142E-03, 1.66296305E-03,
     &   1.60896103E-03, 1.55845257E-03, 1.51111112E-03, 1.46664971E-03,
     &   1.42481494E-03, 1.38538241E-03, 1.34815257E-03, 1.31294738E-03,
     &   1.27960754E-03, 1.24799001E-03, 1.21796608E-03, 1.18941953E-03,
     &   1.16224522E-03, 1.13634777E-03, 1.11164045E-03, 1.08804424E-03,
     &   1.06548698E-03, 1.04390238E-03, 1.02323074E-03, 1.00341564E-03,
     &   9.84406231E-04, 9.66155201E-04, 9.48619428E-04, 9.31758496E-04,
     &   9.15535173E-04, 8.99914941E-04, 8.84865751E-04, 8.70357810E-04/

         DATA ((DAS  (I,J), I= 801, 900), J=3,3) /
     &   8.56363360E-04, 8.42856586E-04, 8.29813382E-04, 8.17211359E-04,
     &   8.05029244E-04, 7.93247429E-04, 7.81847460E-04, 7.70812034E-04,
     &   7.60124914E-04, 7.49770855E-04, 7.39735528E-04, 7.30005461E-04,
     &   7.20567530E-04, 7.11411150E-04, 7.02523740E-04, 6.93895169E-04,
     &   6.85515470E-04, 6.77375253E-04, 6.69465681E-04, 6.61778431E-04,
     &   6.54304930E-04, 6.47039294E-04, 6.39973923E-04, 6.33102374E-04,
     &   6.26418645E-04, 6.19917169E-04, 6.13592817E-04, 6.07440912E-04,
     &   6.01457242E-04, 5.95638098E-04, 5.89980316E-04, 5.84481349E-04,
     &   5.79139363E-04, 5.73953380E-04, 5.68923487E-04, 5.64051136E-04,
     &   5.59339625E-04, 5.54387728E-04, 5.49546837E-04, 5.44813543E-04,
     &   5.40184577E-04, 5.35656804E-04, 5.31227213E-04, 5.26892702E-04,
     &   5.22651131E-04, 5.18910421E-04, 5.14262053E-04, 5.10281149E-04,
     &   5.06382725E-04, 5.02564667E-04, 4.98824426E-04, 4.95160060E-04,
     &   4.91569527E-04, 4.88050350E-04, 4.84602159E-04, 4.81221596E-04,
     &   3.79913978E-04, 3.22303039E-04, 3.00868682E-04, 3.35917086E-04,
     &   3.99534765E-04, 4.39851714E-04, 4.93364850E-04, 5.66064590E-04,
     &   6.39303894E-04, 7.37087662E-04, 7.77444607E-04, 8.70989749E-04,
     &   9.27216116E-04, 9.91589577E-04, 1.06512724E-03, 1.15143493E-03,
     &   1.25364923E-03, 1.37630218E-03, 1.52596255E-03, 1.71243380E-03,
     &   1.95097523E-03, 2.06609682E-03, 2.19563390E-03, 2.34245938E-03,
     &   2.51026442E-03, 2.70387205E-03, 2.92970678E-03, 3.05732558E-03,
     &   3.19651961E-03, 3.34893499E-03, 3.51654552E-03, 3.70173847E-03,
     &   3.90742883E-03, 4.13721362E-03, 4.39558376E-03, 4.68822007E-03,
     &   5.02241530E-03, 5.40768945E-03, 5.85670998E-03, 6.36368142E-03,
     &   6.69558272E-03, 7.06387487E-03, 7.47488575E-03, 7.93650017E-03/

         DATA ((DAS  (I,J), I= 901,1000), J=3,3) /
     &   8.45866990E-03, 9.05413822E-03, 9.73949206E-03, 1.05367285E-02,
     &   1.14756551E-02, 1.25976910E-02, 1.39621256E-02, 1.56569094E-02,
     &   1.78183238E-02, 2.06693902E-02, 2.46017036E-02, 3.03709496E-02,
     &   3.96439767E-02, 5.69325511E-02, 8.41241335E-02, 9.96004856E-02,
     &   1.16481270E-01, 1.58015878E-01, 1.72979804E-01, 1.85329860E-01,
     &   1.95817921E-01, 2.04649465E-01, 2.11895294E-01, 2.18134336E-01,
     &   2.23242898E-01, 2.26899101E-01, 2.30005201E-01, 2.32204043E-01,
     &   2.33508219E-01, 2.33979248E-01, 2.33644755E-01, 2.32730758E-01,
     &   2.30875822E-01, 2.28448548E-01, 2.25176716E-01, 2.20967093E-01,
     &   2.16540644E-01, 2.10338887E-01, 2.04068780E-01, 1.96862188E-01,
     &   1.88672862E-01, 1.77826855E-01, 1.67393747E-01, 1.53881073E-01,
     &   1.41158108E-01, 1.22608636E-01, 1.02364770E-01, 7.21292534E-02,
     &   4.57739090E-02, 1.54168122E-02, 1.99389053E-04, 1.53398220E-02,
     &   3.06171489E-02, 7.20562712E-02, 1.02296233E-01, 1.22544109E-01,
     &   1.41098102E-01, 1.53824714E-01, 1.67341841E-01, 1.77778878E-01,
     &   1.88629535E-01, 1.96822859E-01, 2.04033421E-01, 2.10307443E-01,
     &   2.16513664E-01, 2.20943830E-01, 2.25157641E-01, 2.28433486E-01,
     &   2.30864589E-01, 2.32723667E-01, 2.33641084E-01, 2.33979695E-01,
     &   2.33512482E-01, 2.32212208E-01, 2.30017275E-01, 2.26915017E-01,
     &   2.23262258E-01, 2.18157506E-01, 2.11922203E-01, 2.04679939E-01,
     &   1.95851999E-01, 1.85367493E-01, 1.73020910E-01, 1.58060442E-01,
     &   1.39923619E-01, 1.49770258E-01, 1.28119781E-01, 1.14784417E-01,
     &   1.06722297E-01, 9.00212867E-02, 8.41787609E-02, 7.16017437E-02,
     &   4.63624362E-02, 3.42078986E-02, 2.70877744E-02, 2.24171269E-02,
     &   1.91193372E-02, 1.66673707E-02, 1.47731112E-02, 1.32658597E-02/

         DATA ((DAS  (I,J), I=1001,1100), J=3,3) /
     &   1.20380874E-02, 1.10187039E-02, 1.01588348E-02, 9.42377537E-03,
     &   8.78820448E-03, 8.23321413E-03, 7.74440233E-03, 7.31060414E-03,
     &   6.92303303E-03, 6.57467747E-03, 6.02401430E-03, 5.33666016E-03,
     &   4.96904196E-03, 4.36833636E-03, 3.89831142E-03, 3.43746840E-03,
     &   2.78312183E-03, 2.06326007E-03, 1.63744965E-03, 1.46028087E-03,
     &   1.36101262E-03, 1.29663392E-03, 1.25107848E-03, 1.21682812E-03,
     &   1.18993784E-03, 1.16810504E-03, 1.14988883E-03, 1.13431769E-03,
     &   1.12078507E-03, 1.10879203E-03, 1.09800290E-03, 1.08818518E-03,
     &   1.07912121E-03, 1.07065695E-03, 1.06266282E-03, 1.05505611E-03,
     &   1.04772313E-03, 1.04060439E-03, 1.03363122E-03, 1.02675033E-03,
     &   1.01988306E-03, 1.01299032E-03, 1.00599209E-03, 9.98848302E-04,
     &   9.91461005E-04, 9.83742848E-04, 9.75542801E-04, 9.66704342E-04,
     &   9.56910174E-04, 9.45582105E-04, 9.31280328E-04, 8.99097409E-04,
     &   8.71635758E-04, 8.62284060E-04, 8.56111344E-04, 8.51657921E-04,
     &   8.48327575E-04, 8.45811522E-04, 8.43898865E-04, 8.42440120E-04,
     &   8.41329354E-04, 8.40428333E-04, 8.39718920E-04, 8.39067306E-04,
     &   8.38392991E-04, 8.37636386E-04, 8.36768325E-04, 8.35866161E-04,
     &   8.34683964E-04, 8.33072876E-04, 8.30972524E-04, 8.28301975E-04,
     &   8.24963970E-04, 8.20845850E-04, 8.15829819E-04, 8.09781313E-04,
     &   8.02520253E-04, 7.93668676E-04, 7.83573016E-04, 7.71307268E-04,
     &   7.56685822E-04, 7.39131739E-04, 7.17746115E-04, 6.91081765E-04,
     &   6.56357358E-04, 6.07621397E-04, 5.93105693E-04, 5.53979423E-04,
     &   4.97209050E-04, 4.91671126E-04, 4.54817312E-04, 4.41804445E-04,
     &   4.31721827E-04, 4.21063125E-04, 4.10971753E-04, 4.00818738E-04,
     &   3.94265202E-04, 3.91041965E-04, 3.81992077E-04, 3.73553285E-04/

         DATA ((DAS  (I,J), I=1101,1200), J=3,3) /
     &   3.65648053E-04, 3.58267895E-04, 3.51569628E-04, 3.44985314E-04,
     &   3.38665899E-04, 3.34448872E-04, 3.28401675E-04, 3.21224278E-04,
     &   3.14046881E-04, 3.06869484E-04, 3.03233849E-04, 2.99119078E-04,
     &   2.94218581E-04, 2.90411812E-04, 2.89779680E-04, 2.88001340E-04,
     &   2.81937920E-04, 2.76174371E-04, 2.74635401E-04, 2.74570565E-04,
     &   2.74040025E-04, 2.72435052E-04, 2.69448079E-04, 2.69684237E-04,
     &   2.73512684E-04, 2.82314846E-04, 2.88746364E-04, 2.76662875E-04,
     &   2.49204916E-04, 2.45388242E-04, 2.43358916E-04, 2.42900229E-04,
     &   2.39735546E-04, 2.37608055E-04, 2.35170660E-04, 2.33203697E-04,
     &   2.31667169E-04, 2.30064838E-04, 2.29551404E-04, 2.28227090E-04,
     &   2.26394885E-04, 2.25427370E-04, 2.24209393E-04, 2.23214280E-04,
     &   2.23118196E-04, 2.22647224E-04, 2.21735432E-04, 2.19272632E-04,
     &   2.17106829E-04, 2.16279657E-04, 2.15595840E-04, 2.14916630E-04,
     &   2.15930835E-04, 2.19136862E-04, 2.19216312E-04, 2.16791913E-04,
     &   2.14747367E-04, 2.12296711E-04, 2.08106754E-04, 2.04417851E-04,
     &   2.02341781E-04, 2.02410607E-04, 2.01224690E-04, 1.99305275E-04,
     &   1.99017267E-04, 1.97466022E-04, 1.97483174E-04, 1.94847978E-04,
     &   1.93543308E-04, 1.93395097E-04, 1.93159960E-04, 1.91378115E-04,
     &   1.90501444E-04, 1.89111368E-04, 1.87480615E-04, 1.86380259E-04,
     &   1.85307996E-04, 1.83488902E-04, 1.83552247E-04, 1.84582723E-04,
     &   1.88761243E-04, 1.90993965E-04, 1.77431530E-04, 1.77025699E-04,
     &   1.76302611E-04, 1.74372845E-04, 1.74719219E-04, 1.78605262E-04,
     &   1.77156348E-04, 1.75973484E-04, 1.75107190E-04, 1.68929998E-04,
     &   1.71678560E-04, 1.88227226E-04, 1.83795773E-04, 1.71756128E-04,
     &   1.69169398E-04, 1.68217709E-04, 1.66768580E-04, 1.66281561E-04/

         DATA ((DAS  (I,J), I=1201,1300), J=3,3) /
     &   1.65643789E-04, 1.64672332E-04, 1.63536885E-04, 1.62988428E-04,
     &   1.62066829E-04, 1.61856074E-04, 1.60647000E-04, 1.59468323E-04,
     &   1.58321004E-04, 1.57067855E-04, 1.56745026E-04, 1.55783525E-04,
     &   1.57561411E-04, 1.53199384E-04, 1.48768628E-04, 1.47013734E-04,
     &   1.44995464E-04, 1.44610535E-04, 1.42882256E-04, 1.41833621E-04,
     &   1.36566319E-04, 1.36379190E-04, 1.31921069E-04, 1.27184316E-04,
     &   1.83052905E-04, 2.32348719E-04, 2.48780657E-04, 2.09682899E-04,
     &   1.76230605E-04, 1.66952780E-04, 1.59927208E-04, 1.49827323E-04,
     &   1.42781295E-04, 1.39969521E-04, 1.37501516E-04, 1.35312236E-04,
     &   1.33352809E-04, 1.31585713E-04, 1.29977207E-04, 1.28512544E-04,
     &   1.27168758E-04, 1.25930501E-04, 1.24785105E-04, 1.23722008E-04,
     &   1.22732327E-04, 1.21808529E-04, 1.20944181E-04, 1.20133752E-04,
     &   1.19372463E-04, 1.18656156E-04, 1.17981206E-04, 1.17344430E-04,
     &   1.16743032E-04, 1.16174543E-04, 1.15636779E-04, 1.15127807E-04,
     &   1.14645911E-04, 1.14189570E-04, 1.13757434E-04, 1.13348308E-04,
     &   1.12961138E-04, 1.12594995E-04, 1.12249069E-04, 1.11922660E-04,
     &   1.11615169E-04, 1.11326094E-04, 1.11055028E-04, 1.10850924E-04,
     &   1.10565741E-04, 1.10145848E-04, 1.09795358E-04, 1.09515857E-04,
     &   1.09310731E-04, 1.09185535E-04, 1.09148558E-04, 1.09211711E-04,
     &   1.09391912E-04, 1.09713301E-04, 1.10210948E-04, 1.10938390E-04,
     &   1.11974645E-04, 1.13456308E-04, 1.15624716E-04, 1.15894424E-04,
     &   1.18958983E-04, 1.24561409E-04, 1.35582576E-04, 1.55844571E-04,
     &   1.64860829E-04, 2.05212183E-04, 2.87558415E-04, 3.27248751E-04,
     &   3.83918723E-04, 4.22177598E-04, 4.70362006E-04, 5.18827073E-04,
     &   5.79669851E-04, 6.16269134E-04, 6.58153783E-04, 7.06531618E-04/

         DATA ((DAS  (I,J), I=1301,1400), J=3,3) /
     &   7.63008102E-04, 8.29764661E-04, 9.09842411E-04, 9.56171769E-04,
     &   1.00761193E-03, 1.06504954E-03, 1.12958886E-03, 1.20262282E-03,
     &   1.28593399E-03, 1.38184103E-03, 1.49341665E-03, 1.62482121E-03,
     &   1.67955638E-03, 1.71390009E-03, 1.74968860E-03, 1.78701477E-03,
     &   1.82597959E-03, 1.86669308E-03, 1.90927534E-03, 1.95385773E-03,
     &   2.00058421E-03, 2.04961290E-03, 2.10111784E-03, 2.15529108E-03,
     &   2.21234507E-03, 2.27251540E-03, 2.33606409E-03, 2.40328335E-03,
     &   2.47450012E-03, 2.55008133E-03, 2.63044026E-03, 2.71604403E-03,
     &   2.80742274E-03, 2.90518039E-03, 3.01000826E-03, 3.12270124E-03,
     &   3.24417803E-03, 3.37550618E-03, 3.51793356E-03, 3.67292805E-03,
     &   3.84222831E-03, 4.02790921E-03, 4.23246722E-03, 4.45893324E-03,
     &   4.71102366E-03, 4.99334581E-03, 5.31168186E-03, 5.67338884E-03,
     &   6.08797392E-03, 6.56794170E-03, 7.13007649E-03, 7.79744424E-03,
     &   8.60263404E-03, 9.59323744E-03, 1.08416017E-02, 1.24633262E-02,
     &   1.46552465E-02, 1.77819437E-02, 2.26025603E-02, 3.10009863E-02,
     &   4.92764554E-02, 6.06074786E-02, 1.15438732E-01, 1.60624879E-01,
     &   1.93377937E-01, 2.18720824E-01, 2.39543104E-01, 2.57764886E-01,
     &   2.74672159E-01, 2.86442352E-01, 2.98423500E-01, 3.10206854E-01,
     &   3.15888161E-01, 3.23850402E-01, 3.28621284E-01, 3.34543210E-01,
     &   3.36086494E-01, 3.38369210E-01, 3.39317860E-01, 3.38602894E-01,
     &   3.37511966E-01, 3.35864967E-01, 3.30649228E-01, 3.26991647E-01,
     &   3.22496305E-01, 3.10914003E-01, 3.03723343E-01, 2.95505227E-01,
     &   2.86102498E-01, 2.64054953E-01, 2.51106715E-01, 2.37069660E-01,
     &   2.05162263E-01, 1.87499964E-01, 1.48631412E-01, 1.05840792E-01,
     &   1.13005314E-02, 1.04435370E-01, 1.47335786E-01, 1.86146672E-01/

         DATA ((DAS  (I,J), I=1401,1500), J=3,3) /
     &   2.20632755E-01, 2.36136687E-01, 2.50251277E-01, 2.63147826E-01,
     &   2.85473074E-01, 2.94853954E-01, 3.03232045E-01, 3.10486279E-01,
     &   3.22185013E-01, 3.26689867E-01, 3.30438742E-01, 3.35719143E-01,
     &   3.37424221E-01, 3.39165820E-01, 3.39322602E-01, 3.38421392E-01,
     &   3.36191221E-01, 3.32889911E-01, 3.28763634E-01, 3.24009965E-01,
     &   3.16065119E-01, 3.10421286E-01, 2.98614497E-01, 2.86664444E-01,
     &   2.74858572E-01, 2.57940359E-01, 2.39703241E-01, 2.18861204E-01,
     &   1.93493336E-01, 1.60709241E-01, 1.14820369E-01, 6.26678632E-02,
     &   3.58446817E-02, 2.50791490E-02, 1.92830135E-02, 1.56622081E-02,
     &   1.31860123E-02, 1.13858985E-02, 1.00183007E-02, 8.94408755E-03,
     &   8.07801537E-03, 7.36494464E-03, 6.76763009E-03, 6.26000709E-03,
     &   5.82328936E-03, 5.44359444E-03, 5.11044168E-03, 4.81577047E-03,
     &   4.55327990E-03, 4.31797339E-03, 4.10583787E-03, 3.91361309E-03,
     &   3.73862302E-03, 3.57865062E-03, 3.43184347E-03, 3.29664177E-03,
     &   3.17172277E-03, 3.05595736E-03, 2.94837592E-03, 2.84814110E-03,
     &   2.75452602E-03, 2.66689663E-03, 2.58469734E-03, 2.50743927E-03,
     &   2.43469052E-03, 2.36606809E-03, 2.30123116E-03, 2.23987545E-03,
     &   2.18172843E-03, 2.12654532E-03, 2.07410559E-03, 2.02421007E-03,
     &   1.97667843E-03, 1.93134698E-03, 1.88806680E-03, 1.84670212E-03,
     &   1.80712889E-03, 1.76923355E-03, 1.73291194E-03, 1.69806835E-03,
     &   1.66461468E-03, 1.41234338E-03, 1.25775021E-03, 1.10956029E-03,
     &   9.44174752E-04, 7.89591388E-04, 6.37358423E-04, 5.52312014E-04,
     &   4.39973428E-04, 4.01537830E-04, 3.70602903E-04, 3.43605450E-04,
     &   3.21236572E-04, 3.02393796E-04, 2.72483050E-04, 2.49938234E-04,
     &   1.90998134E-04, 1.59274936E-04, 1.61754316E-04, 2.07390650E-04/

         DATA ((DAS  (I,J), I=1501,1600), J=3,3) /
     &   3.00506876E-04, 4.31628805E-04, 6.11751189E-04, 8.65326411E-04,
     &   1.50786843E-03, 1.63016785E-03, 1.77426891E-03, 1.81604000E-03,
     &   1.91127000E-03, 2.01710829E-03, 2.13542693E-03, 2.26856600E-03,
     &   2.41948957E-03, 2.59200858E-03, 2.79110654E-03, 3.02342809E-03,
     &   3.29803401E-03, 3.62760930E-03, 4.03047645E-03, 4.53411873E-03,
     &   5.18172302E-03, 6.04526487E-03, 7.25429298E-03, 9.06788057E-03,
     &   1.20904254E-02, 1.81343804E-02, 2.62420189E-02, 2.76028808E-02,
     &   2.91125128E-02, 3.07967047E-02, 3.26875686E-02, 3.48256009E-02,
     &   3.72626073E-02, 4.00659426E-02, 4.33248092E-02, 4.71598843E-02,
     &   5.17385170E-02, 5.72996521E-02, 6.41965864E-02, 7.29743916E-02,
     &   8.45197360E-02, 1.00375977E-01, 1.23481151E-01, 1.60164268E-01,
     &   2.26735358E-01, 3.77337510E-01, 4.30131591E-01, 4.71684539E-01,
     &   5.17998925E-01, 5.56867814E-01, 5.84899264E-01, 6.08673222E-01,
     &   6.19627328E-01, 6.23976433E-01, 6.18506319E-01, 5.99123270E-01,
     &   5.76806613E-01, 5.72583738E-01, 5.70388354E-01, 5.65826647E-01,
     &   5.63937293E-01, 5.62495742E-01, 5.61033019E-01, 5.45213961E-01,
     &   5.17232908E-01, 4.79791908E-01, 4.40122500E-01, 3.89310018E-01,
     &   3.31573586E-01, 2.74097268E-01, 2.40133188E-01, 2.04885926E-01,
     &   1.68514835E-01, 1.16051526E-01, 7.77458508E-02, 3.89890093E-02,
     &   1.56112870E-02, 7.80941817E-03, 3.90854872E-03, 1.34500178E-04,
     &   3.90059002E-03, 7.80145646E-03, 3.89810632E-02, 7.77379571E-02,
     &   1.16043719E-01, 2.87285016E-01, 3.49592198E-01, 4.49428968E-01,
     &   4.87827220E-01, 5.23903114E-01, 5.53393995E-01, 5.74720088E-01,
     &   5.93160199E-01, 6.05525721E-01, 6.15141862E-01, 6.20569409E-01,
     &   6.23535702E-01, 6.23861611E-01, 6.22047343E-01, 6.18784529E-01/

         DATA ((DAS  (I,J), I=1601,1700), J=3,3) /
     &   6.07999701E-01, 6.00730826E-01, 5.92671773E-01, 5.84901571E-01,
     &   5.67409608E-01, 5.57836406E-01, 5.42288255E-01, 5.18002500E-01,
     &   4.30136116E-01, 3.62068685E-01, 3.10527969E-01, 2.70933830E-01,
     &   2.39855341E-01, 2.14938016E-01, 1.90058405E-01, 1.51213536E-01,
     &   1.40701656E-01, 1.31540068E-01, 1.23487024E-01, 1.11519287E-01,
     &   9.23053962E-02, 7.87162113E-02, 6.86040939E-02, 6.07888156E-02,
     &   5.45690240E-02, 4.95021091E-02, 4.52951136E-02, 4.17464813E-02,
     &   3.87130241E-02, 3.60902369E-02, 3.38000700E-02, 3.17830617E-02,
     &   2.99931183E-02, 2.83939595E-02, 2.69566397E-02, 1.81403511E-02,
     &   1.20963978E-02, 9.07385395E-03, 6.05124027E-03, 4.54009666E-03,
     &   3.63359041E-03, 3.02941304E-03, 2.59799806E-03, 2.27456069E-03,
     &   2.02310887E-03, 1.82204717E-03, 1.70803926E-03, 1.57507835E-03,
     &   1.46153741E-03, 1.36346393E-03, 1.27790985E-03, 1.20263153E-03,
     &   9.30745541E-04, 7.61382202E-04, 6.46085911E-04, 5.62777541E-04,
     &   4.99956861E-04, 4.51046721E-04, 4.12001186E-04, 3.80210705E-04,
     &   1.89996130E-04, 1.89350860E-04, 1.92255617E-04, 1.96731245E-04,
     &   2.03381825E-04, 2.13261998E-04, 2.19961011E-04, 2.28299883E-04,
     &   2.52314585E-04, 2.93786816E-04, 3.75218939E-04, 5.79271883E-04,
     &   7.83416116E-04, 8.36752558E-04, 8.98482766E-04, 9.70705873E-04,
     &   1.05628730E-03, 1.15923896E-03, 1.28535078E-03, 1.44331280E-03,
     &   1.64678177E-03, 1.91850794E-03, 2.29945301E-03, 2.87154289E-03,
     &   3.82589429E-03, 3.99193875E-03, 4.17307246E-03, 4.37149315E-03,
     &   4.58977195E-03, 4.83101640E-03, 5.09911061E-03, 5.39876280E-03,
     &   5.73585024E-03, 6.11793941E-03, 6.55462972E-03, 7.05845947E-03,
     &   7.64633208E-03, 8.34109607E-03, 9.17470639E-03, 1.01936330E-02/

         DATA ((DAS  (I,J), I=1701,1800), J=3,3) /
     &   1.05120254E-02, 1.14672132E-02, 1.31042936E-02, 1.52868675E-02,
     &   1.83414853E-02, 2.29206902E-02, 3.05421438E-02, 4.57362921E-02,
     &   9.06455179E-02, 1.07765711E-01, 1.52001129E-01, 1.85641283E-01,
     &   2.13359394E-01, 2.37552049E-01, 2.59015841E-01, 2.78270941E-01,
     &   2.95470507E-01, 3.11412740E-01, 3.26468263E-01, 3.40227342E-01,
     &   3.52330065E-01, 3.63525223E-01, 3.73618967E-01, 3.82459871E-01,
     &   3.90859417E-01, 3.97739673E-01, 4.03112469E-01, 4.07674122E-01,
     &   4.11185226E-01, 4.13360239E-01, 4.13907441E-01, 4.12982883E-01,
     &   4.10312896E-01, 4.05595117E-01, 3.98502137E-01, 3.90527960E-01,
     &   3.78177547E-01, 3.62466263E-01, 3.43074678E-01, 3.15448655E-01,
     &   2.82180394E-01, 2.37204419E-01, 1.78268213E-01, 2.44247565E-02,
     &   1.78149190E-04, 8.06961112E-02, 1.78265317E-01, 2.37201817E-01,
     &   2.82178097E-01, 3.15446645E-01, 3.43072960E-01, 3.62464796E-01,
     &   3.78176324E-01, 3.90526974E-01, 3.98501339E-01, 4.05594534E-01,
     &   4.10312515E-01, 4.12982691E-01, 4.13907425E-01, 4.13360385E-01,
     &   4.11185547E-01, 4.07674601E-01, 4.03113093E-01, 3.97740428E-01,
     &   3.90860309E-01, 3.82460898E-01, 3.73620116E-01, 3.63526490E-01,
     &   3.52331445E-01, 3.40228830E-01, 3.26469857E-01, 3.11414437E-01,
     &   2.95472299E-01, 2.78272822E-01, 2.59017812E-01, 2.37554104E-01,
     &   2.13361532E-01, 1.85643500E-01, 1.52003422E-01, 1.07768077E-01,
     &   9.22781297E-02, 8.93728523E-02, 8.57691475E-02, 7.58159588E-02,
     &   6.68458056E-02, 4.57362373E-02, 3.05445756E-02, 2.29225008E-02,
     &   1.83435220E-02, 1.52893054E-02, 1.31067328E-02, 1.14694975E-02,
     &   1.01960750E-02, 9.17714983E-03, 8.34345844E-03, 7.64877887E-03,
     &   7.06090811E-03, 6.55702933E-03, 6.12039220E-03, 5.73830530E-03/

         DATA ((DAS  (I,J), I=1801,1900), J=3,3) /
     &   3.82837266E-03, 2.87405375E-03, 2.30200555E-03, 1.92111134E-03,
     &   1.64944513E-03, 1.44604521E-03, 1.28816121E-03, 1.05928050E-03,
     &   9.01693741E-04, 8.40085316E-04, 7.86879113E-04, 5.97831579E-04,
     &   4.63767711E-04, 3.87363292E-04, 3.39453138E-04, 3.07654139E-04,
     &   2.85770289E-04, 2.70325422E-04, 2.59286116E-04, 2.51375228E-04,
     &   2.45668663E-04, 2.41540212E-04, 2.38624932E-04, 2.36664083E-04,
     &   2.35473524E-04, 2.34922305E-04, 2.34918343E-04, 2.35398870E-04,
     &   2.36324123E-04, 2.37673502E-04, 2.39442483E-04, 2.41638825E-04,
     &   2.48136285E-04, 3.69357223E-04, 3.97946045E-04, 4.51504189E-04,
     &   4.96100906E-04, 5.33867757E-04, 5.66144027E-04, 5.93873742E-04,
     &   6.17603815E-04, 6.37640349E-04, 6.54500144E-04, 6.68430026E-04,
     &   6.79507013E-04, 6.87892821E-04, 6.93766722E-04, 6.97251140E-04,
     &   6.98335700E-04, 6.97115390E-04, 6.93625739E-04, 6.87766059E-04,
     &   6.79667166E-04, 6.69491637E-04, 6.56579566E-04, 6.41767508E-04,
     &   6.24140283E-04, 6.04023199E-04, 5.80936128E-04, 5.54809891E-04,
     &   5.25644603E-04, 4.93533307E-04, 4.56226177E-04, 4.14534148E-04,
     &   3.68160321E-04, 3.14206067E-04, 2.53737968E-04, 2.21272113E-04,
     &   3.51818623E-04, 4.17876867E-04, 4.68224644E-04, 5.08204936E-04,
     &   5.42262628E-04, 5.71435195E-04, 5.95605096E-04, 6.16784832E-04,
     &   6.35077855E-04, 6.50627665E-04, 6.63604621E-04, 6.74409838E-04,
     &   6.82927268E-04, 6.89726875E-04, 6.94410859E-04, 6.97336077E-04,
     &   6.98421717E-04, 6.97669525E-04, 6.95129621E-04, 6.90697017E-04,
     &   6.84342955E-04, 6.76114561E-04, 6.65859480E-04, 6.52729750E-04,
     &   6.37995275E-04, 6.21510170E-04, 6.01460059E-04, 5.78384589E-04,
     &   5.51827980E-04, 5.21241364E-04, 4.85684382E-04, 4.43815480E-04/

         DATA ((DAS  (I,J), I=1901,2000), J=3,3) /
     &   3.93485764E-04, 3.30589874E-04, 2.48368841E-04, 2.43961782E-04,
     &   2.35288123E-04, 2.32136129E-04, 2.31071287E-04, 2.30963612E-04,
     &   2.31326737E-04, 2.31913615E-04, 2.32582719E-04, 2.33249383E-04,
     &   2.33855896E-04, 2.34365867E-04, 2.34750165E-04, 2.34989320E-04,
     &   2.35068345E-04, 2.34974469E-04, 2.34697679E-04, 2.34229448E-04,
     &   2.33560557E-04, 2.32686462E-04, 2.31591216E-04, 2.30277929E-04,
     &   2.28731486E-04, 2.26944910E-04, 2.24911594E-04, 2.22614351E-04,
     &   2.20043601E-04, 2.17191056E-04, 2.14036150E-04, 2.10542073E-04,
     &   2.06723539E-04, 2.02506340E-04, 1.97884002E-04, 1.92785009E-04,
     &   1.87115043E-04, 1.80718017E-04, 1.72114670E-04, 1.74390617E-04,
     &   1.77871238E-04, 1.81330683E-04, 1.84661640E-04, 1.87831734E-04,
     &   1.90841786E-04, 1.93668421E-04, 1.96314195E-04, 1.98794586E-04,
     &   2.01090893E-04, 2.03217241E-04, 2.05176449E-04, 2.06962554E-04,
     &   2.08591666E-04, 2.10067195E-04, 2.11370210E-04, 2.12522386E-04,
     &   2.13525679E-04, 2.14367943E-04, 2.15062730E-04, 2.15605245E-04,
     &   2.16002532E-04, 2.16250046E-04, 2.16352170E-04, 2.16309340E-04,
     &   2.16122642E-04, 2.15793923E-04, 2.15323987E-04, 2.14715925E-04,
     &   2.13972178E-04, 2.13096664E-04, 2.12092395E-04, 2.10967867E-04,
     &   2.09724766E-04, 2.08373745E-04, 1.99556548E-04, 1.98928445E-04,
     &   1.98283481E-04, 1.97615171E-04, 1.96921990E-04, 1.96203005E-04,
     &   1.95457918E-04, 1.94680033E-04, 1.93861348E-04, 1.93008585E-04,
     &   1.92113563E-04, 1.91175730E-04, 1.90195005E-04, 1.89171791E-04,
     &   1.88095923E-04, 1.86966883E-04, 1.85797017E-04, 1.84562669E-04,
     &   1.83262841E-04, 1.81911524E-04, 1.80480807E-04, 1.79001177E-04,
     &   1.77443673E-04, 1.75809257E-04, 1.74048168E-04, 1.72249061E-04/

         DATA ((DAS  (I,J), I=2001,2100), J=3,3) /
     &   1.70383638E-04, 1.68403175E-04, 1.66315717E-04, 1.64115270E-04,
     &   1.61802526E-04, 1.59332525E-04, 1.56684304E-04, 1.53798896E-04,
     &   1.50554085E-04, 1.45595877E-04, 1.45214377E-04, 1.46339066E-04,
     &   1.47643086E-04, 1.48994386E-04, 1.50346153E-04, 1.51669052E-04,
     &   1.52964495E-04, 1.54216284E-04, 1.55416678E-04, 1.56552509E-04,
     &   1.57647353E-04, 1.58682697E-04, 1.59663541E-04, 1.60585666E-04,
     &   1.61447032E-04, 1.62262697E-04, 1.63013387E-04, 1.63706254E-04,
     &   1.64353126E-04, 1.64933213E-04, 1.65464100E-04, 1.65938803E-04,
     &   1.66357182E-04, 1.66722875E-04, 1.67035030E-04, 1.67293354E-04,
     &   1.67498093E-04, 1.67651138E-04, 1.67751689E-04, 1.67801773E-04,
     &   1.67802223E-04, 1.67753543E-04, 1.67656918E-04, 1.67511197E-04,
     &   1.67315201E-04, 1.65823796E-04, 1.60205946E-04, 1.57598074E-04,
     &   1.58685866E-04, 1.62549350E-04, 1.68009473E-04, 1.74964357E-04,
     &   1.82463740E-04, 1.90983233E-04, 2.00102638E-04, 2.09381724E-04,
     &   2.19738475E-04, 2.30424884E-04, 2.26013456E-04, 2.21602028E-04,
     &   2.12788317E-04, 2.04863596E-04, 1.98253372E-04, 1.92157136E-04,
     &   1.86966131E-04, 1.82096527E-04, 1.77889811E-04, 1.73894646E-04,
     &   1.70254597E-04, 1.67061833E-04, 1.63988773E-04, 1.61273227E-04,
     &   1.58641986E-04, 1.56302724E-04, 1.54023551E-04, 1.51987008E-04,
     &   1.49993528E-04, 1.48204578E-04, 1.46446455E-04, 1.44862831E-04,
     &   1.43367099E-04, 1.41889754E-04, 1.40552759E-04, 1.39228387E-04,
     &   1.38026562E-04, 1.36833015E-04, 1.35747259E-04, 1.34666478E-04,
     &   1.33681132E-04, 1.32698237E-04, 1.31800331E-04, 1.30902940E-04,
     &   1.30045131E-04, 1.29259363E-04, 1.28472001E-04, 1.27749594E-04,
     &   1.27024593E-04, 1.26358406E-04, 1.25688861E-04, 1.25072775E-04/

         DATA ((DAS  (I,J), I=2101,2174), J=3,3) /
     &   1.24452752E-04, 1.23881493E-04, 1.23305863E-04, 1.22774862E-04,
     &   1.22262061E-04, 1.21744449E-04, 1.21266170E-04, 1.20782888E-04,
     &   1.20335867E-04, 1.19883966E-04, 1.19465315E-04, 1.19041456E-04,
     &   1.18648654E-04, 1.18250608E-04, 1.17881406E-04, 1.17506956E-04,
     &   1.17143791E-04, 1.16806524E-04, 1.16464054E-04, 1.16123868E-04,
     &   1.15800309E-04, 1.15499380E-04, 1.15193367E-04, 1.14908285E-04,
     &   1.14618720E-04, 1.14348805E-04, 1.14073963E-04, 1.13817832E-04,
     &   1.13556864E-04, 1.13313515E-04, 1.13076077E-04, 1.12833942E-04,
     &   1.12607960E-04, 1.12377379E-04, 1.12162063E-04, 1.11942247E-04,
     &   1.11736874E-04, 1.11527360E-04, 1.11331320E-04, 1.11130980E-04,
     &   1.10943619E-04, 1.10752058E-04, 1.08917593E-04, 1.07446188E-04,
     &   1.07237412E-04, 1.01534574E-04, 1.00178993E-04, 9.79992524E-05,
     &   9.72575550E-05, 9.72130679E-05, 9.65017219E-05, 9.64998485E-05,
     &   9.61085820E-05, 9.60967267E-05, 9.60878364E-05, 9.59293597E-05,
     &   9.58338144E-05, 9.58174978E-05, 9.57913216E-05, 9.57536074E-05,
     &   9.57277439E-05, 9.57118037E-05, 9.55681552E-05, 9.55614161E-05,
     &   9.55246557E-05, 9.55177437E-05, 9.55175025E-05, 9.54739470E-05,
     &   9.54728486E-05, 9.54644196E-05, 9.54318276E-05, 9.53976704E-05,
     &   9.53835862E-05, 9.52965213E-05/


c      ++++ deghadslow19 ++++

c update 12/28/19

c******************************************************************************
c*                                                                            *
c*    --- intRdatx ---                              Sat Dec 28 11:45:39 2019  *
c*                                                                            *
c***************************# FJ@HU Berlin #***********************************

c  "Delta alpha"               Update version (24/02/2019)

c ISCAN:  1, Ismooth: 0, IGS: 0, DSET: WAVE, iplot: 0 itestflavorspl: 0
c IKE,Emin,Emax,NP,N,EPSREL:  1, 6.0000E-01, 1.4000E+00, 100, 10, 1.00E-02,
c ALS(sta)(sys):  0.1184( 0.0007)( 0.0000), MZINP:   91.1876, MTOP:    173.20
c NF,IOR,ICHK: 5,4,2,ISU2: 2,NFL: 5,INAG: 1,inotail:  0, sin2W:   0.23153
c CHPTCUT,EC,ECUT: 0.3180,     5.20,    11.50


         DATA ((DGS  (I,J), I=   1, 100), J=1,1) /
     &  -1.91589187E-03,-2.03781555E-03,-2.15584275E-03,-2.27657569E-03,
     &  -2.40009894E-03,-2.46399323E-03,-2.56470619E-03,-2.75763972E-03,
     &  -2.83374614E-03,-2.90853059E-03,-3.02698529E-03,-3.14046329E-03,
     &  -3.27093610E-03,-3.35727000E-03,-3.46354876E-03,-3.57140659E-03,
     &  -3.67982438E-03,-3.78809610E-03,-3.87699589E-03,-3.99155780E-03,
     &  -4.08477652E-03,-4.19910049E-03,-4.29079720E-03,-4.40993215E-03,
     &  -4.50848341E-03,-4.60573708E-03,-4.72478267E-03,-4.79436199E-03,
     &  -4.82579409E-03,-4.92932195E-03,-5.03404048E-03,-5.13915009E-03,
     &  -5.24582907E-03,-5.35483833E-03,-5.46897602E-03,-5.58769394E-03,
     &  -5.70665224E-03,-5.82939082E-03,-5.95623550E-03,-6.08793016E-03,
     &  -6.18776225E-03,-6.32031224E-03,-6.44943237E-03,-6.55507698E-03,
     &  -6.69359807E-03,-6.83480679E-03,-6.94467391E-03,-7.09790144E-03,
     &  -7.21383197E-03,-7.37670079E-03,-8.27080667E-03,-9.21579594E-03,
     &  -1.05723845E-02,-1.22940304E-02,-1.28680774E-02,-1.30230176E-02,
     &  -1.34791273E-02,-1.40965021E-02,-1.48323828E-02,-1.53901042E-02,
     &  -1.60422061E-02,-1.62659633E-02,-1.66594009E-02,-1.72974926E-02,
     &  -1.78052389E-02,-1.83075158E-02,-1.86917146E-02,-1.89761710E-02,
     &  -1.90053323E-02,-1.89683845E-02,-1.89223870E-02,-1.88117866E-02,
     &  -1.82900528E-02,-1.73995032E-02,-1.61454230E-02,-1.42945713E-02,
     &  -1.19040626E-02,-8.93769053E-03,-5.96778585E-03,-4.86077742E-03,
     &  -1.97116528E-03,-5.64427027E-04, 9.81785286E-04, 2.18926126E-03,
     &   3.98848625E-03, 5.99853665E-03, 7.13141305E-03, 7.95136293E-03,
     &   8.57044926E-03, 9.04311099E-03, 9.45592926E-03, 9.80443277E-03,
     &   1.01055726E-02, 1.03941261E-02, 1.06952776E-02, 1.11395136E-02,
     &   1.12937777E-02, 1.14503557E-02, 1.16678680E-02, 1.18316492E-02/

         DATA ((DGS  (I,J), I= 101, 200), J=1,1) /
     &   1.19742208E-02, 1.20890680E-02, 1.21960337E-02, 1.22732737E-02,
     &   1.23688704E-02, 1.24248499E-02, 1.24797154E-02, 1.25334668E-02,
     &   1.25780601E-02, 1.26168319E-02, 1.26454202E-02, 1.26540545E-02,
     &   1.26629532E-02, 1.26723365E-02, 1.27011150E-02, 1.26999325E-02,
     &   1.26921277E-02, 1.26564622E-02, 1.27490386E-02, 1.28358412E-02,
     &   1.29329290E-02, 1.30394434E-02, 1.31675186E-02, 1.33157018E-02,
     &   1.34702602E-02, 1.36197859E-02, 1.37798126E-02, 1.39634209E-02,
     &   1.41634663E-02, 1.43626601E-02, 1.45703663E-02, 1.47893799E-02,
     &   1.50110325E-02, 1.51948339E-02, 1.54434530E-02, 1.56191403E-02,
     &   1.57636398E-02, 1.58326188E-02, 1.57858901E-02, 1.58352540E-02,
     &   1.59693273E-02, 1.61569548E-02, 1.63471563E-02, 1.64932358E-02,
     &   1.66597181E-02, 1.68258838E-02, 1.69851414E-02, 1.71628072E-02,
     &   1.73875572E-02, 1.75700196E-02, 1.78770079E-02, 1.83501631E-02,
     &   1.84852397E-02, 1.89228859E-02, 1.93605321E-02, 1.97551115E-02,
     &   2.00667245E-02, 2.03607125E-02, 2.05445759E-02, 2.06669306E-02,
     &   2.08250431E-02, 2.10430777E-02, 2.11102882E-02, 2.11434096E-02,
     &   2.11440433E-02, 2.11312710E-02, 2.10761974E-02, 2.10095175E-02,
     &   2.08399347E-02, 2.04849342E-02, 2.00314388E-02, 1.95945618E-02,
     &   1.90179652E-02, 1.83872499E-02, 1.77061630E-02, 1.71098631E-02,
     &   1.63093029E-02, 1.55668173E-02, 1.49018470E-02, 1.45058001E-02,
     &   1.37517986E-02, 1.34262320E-02, 1.25701921E-02, 1.18303839E-02,
     &   1.09121945E-02, 1.00953111E-02, 9.06341097E-03, 8.09938648E-03,
     &   7.14053666E-03, 5.82370789E-03, 5.13536384E-03, 4.63944752E-03,
     &   3.59693427E-03, 3.03047873E-03, 2.43653901E-03, 1.80479351E-03,
     &   1.09892162E-03, 3.96080706E-04,-3.51627781E-04,-1.98124615E-03/

         DATA ((DGS  (I,J), I= 201, 300), J=1,1) /
     &  -3.81070747E-03,-4.89143284E-03,-5.29016862E-03,-5.70869788E-03,
     &  -6.14299654E-03,-6.58731525E-03,-6.99717352E-03,-7.53833554E-03,
     &  -8.08435065E-03,-8.56499829E-03,-9.11604406E-03,-9.68771079E-03,
     &  -1.02886963E-02,-1.09020870E-02,-1.15595604E-02,-1.22415679E-02,
     &  -1.29680589E-02,-1.37147722E-02,-1.44974125E-02,-1.52986057E-02,
     &  -1.61836446E-02,-1.70999818E-02,-1.80672923E-02,-1.90981579E-02,
     &  -2.01918863E-02,-2.13550028E-02,-2.25838810E-02,-2.39473377E-02,
     &  -2.53828507E-02,-2.69165263E-02,-2.75543413E-02,-2.84041085E-02,
     &  -2.87214651E-02,-3.08918496E-02,-3.29121192E-02,-3.49414564E-02,
     &  -3.71952000E-02,-3.95132799E-02,-4.22671524E-02,-4.46861964E-02,
     &  -4.75862911E-02,-5.07565385E-02,-5.36751321E-02,-5.80198124E-02,
     &  -6.23644926E-02,-6.70910213E-02,-7.24518597E-02,-7.85381951E-02,
     &  -8.54003308E-02,-9.32803599E-02,-1.02361680E-01,-1.12887405E-01,
     &  -1.25091259E-01,-1.36981286E-01,-1.43233899E-01,-1.47611224E-01,
     &  -1.53648905E-01,-1.56588562E-01,-1.54827863E-01,-1.65531677E-01,
     &  -1.69847307E-01,-1.73752630E-01,-1.77218065E-01,-1.79971125E-01,
     &  -1.81726000E-01,-1.82065393E-01,-1.79047979E-01,-1.73635056E-01,
     &  -1.69823618E-01,-1.59311460E-01,-1.44578256E-01,-1.25128885E-01,
     &  -9.32703828E-02,-7.19732652E-02,-3.94094640E-02,-4.47305409E-03,
     &   3.10784925E-02, 6.54191774E-02, 9.69474276E-02, 1.24391569E-01,
     &   1.47185632E-01, 1.65187304E-01, 1.78582528E-01, 1.87901612E-01,
     &   1.93878321E-01, 1.97060508E-01, 1.98063494E-01, 1.97379244E-01,
     &   1.95450256E-01, 1.92550360E-01, 1.87190075E-01, 1.85658818E-01,
     &   1.78086316E-01, 1.76200820E-01, 1.72149622E-01, 1.67723501E-01,
     &   1.63373762E-01, 1.59114987E-01, 1.54894477E-01, 1.50890692E-01/

         DATA ((DGS  (I,J), I= 301, 400), J=1,1) /
     &   1.50328731E-01, 1.43225603E-01, 1.39590990E-01, 1.36090326E-01,
     &   1.24970666E-01, 1.15380610E-01, 1.07072787E-01, 9.98745839E-02,
     &   9.35818117E-02, 8.69004581E-02, 8.31761550E-02, 7.88557007E-02,
     &   7.49897225E-02, 7.15287376E-02, 6.84105788E-02, 6.55938775E-02,
     &   6.31040477E-02, 6.07328307E-02, 5.86306481E-02, 5.66255993E-02,
     &   5.52548880E-02, 5.36592498E-02, 5.37745780E-02, 5.09898816E-02,
     &   5.00597493E-02, 5.05517166E-02, 5.11402187E-02, 4.91364018E-02,
     &   4.73804316E-02, 4.62054640E-02, 4.51267200E-02, 4.41835966E-02,
     &   4.32839562E-02, 4.24452076E-02, 4.16544478E-02, 4.09058252E-02,
     &   4.02010824E-02, 3.95200662E-02, 3.88819944E-02, 3.82876826E-02,
     &   3.77163574E-02, 3.71651634E-02, 3.66502568E-02, 3.61399009E-02,
     &   3.56627818E-02, 3.52045877E-02, 3.47532714E-02, 3.42850744E-02,
     &   3.38315059E-02, 3.35090608E-02, 3.30769091E-02, 3.27006445E-02,
     &   3.23247983E-02, 3.19453188E-02, 3.15986601E-02, 3.12444184E-02,
     &   3.03025696E-02, 2.82416071E-02, 2.50251947E-02, 2.26230258E-02,
     &   2.08443665E-02, 1.93770921E-02, 1.81578640E-02, 1.71007478E-02,
     &   1.61909765E-02, 1.61037071E-02, 1.50599548E-02, 1.43501088E-02,
     &   1.41053142E-02, 1.35352632E-02, 1.30223839E-02, 1.29401015E-02,
     &   1.25054794E-02, 1.20131720E-02, 1.17791765E-02, 1.15461091E-02,
     &   1.11573760E-02, 1.07749991E-02, 1.04550619E-02, 1.01562007E-02,
     &   9.97409766E-03, 9.85213191E-03, 9.56314503E-03, 9.24287121E-03,
     &   9.25710331E-03, 9.04453470E-03, 8.81239862E-03, 8.69682556E-03,
     &   8.59172354E-03, 8.36392742E-03, 8.16110266E-03, 7.94565449E-03,
     &   7.88858772E-03, 7.73625849E-03, 7.63876072E-03, 7.57075511E-03,
     &   7.41153152E-03, 7.30090659E-03, 7.27406326E-03, 7.18368051E-03/

         DATA ((DGS  (I,J), I= 401, 500), J=1,1) /
     &   6.98758368E-03, 6.91477323E-03, 6.88655837E-03, 6.90608302E-03,
     &   6.92122899E-03, 6.99449548E-03, 7.02225004E-03, 7.03473905E-03,
     &   7.06326611E-03, 7.12304469E-03, 7.21575216E-03, 7.32637840E-03,
     &   7.34047285E-03, 7.47225977E-03, 7.48048047E-03, 7.54979924E-03,
     &   7.59857312E-03, 7.74708255E-03, 7.76536331E-03, 7.87219418E-03,
     &   7.87933263E-03, 7.88684502E-03, 8.27884850E-03, 8.01903510E-03,
     &   8.03100693E-03, 8.11595669E-03, 8.12392906E-03, 8.16368293E-03,
     &   8.17373764E-03, 8.23341527E-03, 8.26670841E-03, 8.35373351E-03,
     &   8.36735946E-03, 8.51112823E-03, 8.60437961E-03, 8.65725565E-03,
     &   8.66443772E-03, 8.80983970E-03, 8.83821947E-03, 8.99790451E-03,
     &   9.14338475E-03, 9.21132712E-03, 9.25746676E-03, 9.27899237E-03,
     &   9.52454607E-03, 9.57580093E-03, 9.70530498E-03, 9.83114495E-03,
     &   9.89195659E-03, 1.00579186E-02, 1.01977594E-02, 1.02240270E-02,
     &   1.02411815E-02, 1.03376257E-02, 1.03802389E-02, 1.04194866E-02,
     &   1.05874758E-02, 1.06118825E-02, 1.06463860E-02, 1.06587682E-02,
     &   1.06546288E-02, 1.06526642E-02, 1.06374862E-02, 1.06216484E-02,
     &   1.06024491E-02, 1.05551022E-02, 1.04827224E-02, 1.03931624E-02,
     &   1.02330371E-02, 1.00542857E-02, 1.00432365E-02, 9.75592766E-03,
     &   9.54660303E-03, 9.33602253E-03, 9.29192643E-03, 9.01106276E-03,
     &   9.00029567E-03, 8.96977610E-03, 8.91062188E-03, 9.05572243E-03,
     &   9.10923353E-03, 9.32637329E-03, 9.39438590E-03, 9.43491195E-03,
     &   9.51750728E-03, 9.55765494E-03, 9.54025820E-03, 9.52190110E-03,
     &   9.47425361E-03, 9.46009506E-03, 9.41833016E-03, 9.40360485E-03,
     &   9.22213476E-03, 9.31665235E-03, 9.29465683E-03, 9.28389434E-03,
     &   9.23533077E-03, 9.02669876E-03, 9.22270108E-03, 9.43412993E-03/

         DATA ((DGS  (I,J), I= 501, 600), J=1,1) /
     &   9.74664456E-03, 1.00652162E-02, 1.05096904E-02, 1.05424776E-02,
     &   1.05492723E-02, 1.04991019E-02, 1.03952561E-02, 1.02286236E-02,
     &   1.00191113E-02, 9.79079188E-03, 9.49211649E-03, 9.09716681E-03,
     &   8.54113842E-03, 7.72140588E-03, 6.52513789E-03, 4.70582494E-03,
     &   4.22881908E-03, 3.69909752E-03, 3.10666902E-03, 2.43963331E-03,
     &   1.68272672E-03, 8.16681719E-04,-1.84919562E-04,-4.04296352E-04,
     &  -6.30632491E-04,-8.64976443E-04,-1.10653394E-03,-1.35652151E-03,
     &  -1.61512730E-03,-1.88311338E-03,-2.16025990E-03,-2.44770098E-03,
     &  -2.74594489E-03,-4.25455225E-03,-5.41131498E-03,-6.04617265E-03,
     &  -6.49474107E-03,-6.96488494E-03,-7.45790701E-03,-7.97531757E-03,
     &  -8.51871243E-03,-9.09029211E-03,-9.38729472E-03,-9.69223877E-03,
     &  -1.00054126E-02,-1.03271457E-02,-1.06579425E-02,-1.09978610E-02,
     &  -1.13472974E-02,-1.17071457E-02,-1.20776192E-02,-1.24591951E-02,
     &  -1.28523680E-02,-1.32576736E-02,-1.36756840E-02,-1.41069911E-02,
     &  -1.45522497E-02,-1.50120711E-02,-1.54875311E-02,-1.59788920E-02,
     &  -1.64873115E-02,-1.70136150E-02,-1.75587582E-02,-1.81237731E-02,
     &  -1.87097633E-02,-1.93179203E-02,-1.99495160E-02,-2.06058976E-02,
     &  -2.12886990E-02,-2.19994350E-02,-2.27400120E-02,-2.35121877E-02,
     &  -2.43180022E-02,-2.51597025E-02,-2.60398389E-02,-2.69611190E-02,
     &  -2.79262348E-02,-2.89387177E-02,-3.00018093E-02,-3.11195425E-02,
     &  -3.22962353E-02,-3.35367596E-02,-3.41826406E-02,-3.48464999E-02,
     &  -3.55290999E-02,-3.62312500E-02,-3.69537942E-02,-3.76976375E-02,
     &  -3.84635890E-02,-3.92531187E-02,-4.00668221E-02,-4.09058757E-02,
     &  -4.17719732E-02,-4.26653173E-02,-4.35884928E-02,-4.45428381E-02,
     &  -4.55294562E-02,-4.65501915E-02,-4.76068457E-02,-4.87013494E-02/

         DATA ((DGS  (I,J), I= 601, 700), J=1,1) /
     &  -4.98358313E-02,-5.10124594E-02,-5.22336612E-02,-5.35019930E-02,
     &  -5.48202288E-02,-5.61913793E-02,-5.76187106E-02,-5.91057284E-02,
     &  -6.06562591E-02,-6.22744612E-02,-6.39655343E-02,-6.57331976E-02,
     &  -6.75833325E-02,-6.95219552E-02,-7.15555913E-02,-7.36912656E-02,
     &  -7.59371089E-02,-7.83017503E-02,-8.07948728E-02,-8.34272641E-02,
     &  -8.73694887E-02,-8.91592749E-02,-9.22874043E-02,-9.56123418E-02,
     &  -9.91532255E-02,-1.02931792E-01,-1.06972826E-01,-1.11304697E-01,
     &  -1.15960001E-01,-1.20976407E-01,-1.26397674E-01,-1.32275104E-01,
     &  -1.38668537E-01,-1.43773209E-01,-1.59682457E-01,-1.79180302E-01,
     &  -2.03641591E-01,-2.35248354E-01,-2.77702573E-01,-3.37910785E-01,
     &  -4.33794320E-01,-5.81751053E-01,-8.92528183E-01,-1.88936106E+00,
     &  -1.99855709E+00,-2.12102696E+00,-2.25954835E+00,-2.41668860E+00,
     &  -2.59737577E+00,-2.80693429E+00,-3.05283126E+00,-3.34532814E+00,
     &  -3.69890311E+00,-4.13463251E+00,-4.68460249E+00,-5.37616867E+00,
     &  -6.36003961E+00,-6.47470085E+00,-9.04215176E+00,-1.09134105E+01,
     &  -1.24169298E+01,-7.74271848E+00,-1.47359614E+01,-1.50558307E+01,
     &  -1.63109041E+01,-1.70183075E+01,-1.75095516E+01,-1.79631289E+01,
     &  -1.82782688E+01,-1.85318919E+01,-1.86703419E+01,-1.87689244E+01,
     &  -1.87743722E+01,-1.87171829E+01,-1.85677535E+01,-1.84078966E+01,
     &  -1.80621625E+01,-1.77510513E+01,-1.73670889E+01,-1.65688940E+01,
     &  -1.57875814E+01,-1.49287376E+01,-1.39973438E+01,-1.41239039E+01,
     &  -1.31774902E+01,-1.21280268E+01,-1.09779935E+01,-9.07553732E+00,
     &  -7.70019783E+00,-5.55756201E+00,-3.18244144E+00, 3.21056472E+00,
     &   9.09751510E+00, 1.09972582E+01, 1.26826036E+01, 1.36797604E+01,
     &   1.49827611E+01, 1.49792489E+01, 1.58280992E+01, 1.65979485E+01/

         DATA ((DGS  (I,J), I= 701, 800), J=1,1) /
     &   1.72846943E+01, 1.77747549E+01, 1.81464248E+01, 1.83360858E+01,
     &   1.85247311E+01, 1.86605984E+01, 1.87494963E+01, 1.88060971E+01,
     &   1.45788260E+01, 1.54248897E+01, 1.86071429E+01, 1.84966004E+01,
     &   1.82226983E+01, 1.79808613E+01, 1.76241091E+01, 1.72417585E+01,
     &   1.67449915E+01, 1.63437742E+01, 1.51539361E+01, 1.48540916E+01,
     &   1.40165652E+01, 1.30727468E+01, 1.18588448E+01, 1.03598501E+01,
     &   8.72949538E+00, 7.19551320E+00, 6.00396024E+00, 5.14417047E+00,
     &   4.49679318E+00, 3.99271876E+00, 3.58955537E+00, 3.25998833E+00,
     &   2.98575983E+00, 2.75387546E+00, 2.55545623E+00, 2.38371968E+00,
     &   2.23364444E+00, 2.10138584E+00, 1.98395788E+00, 1.87900475E+00,
     &   1.78464317E+00, 1.69935040E+00, 1.62188133E+00, 8.56575464E-01,
     &   5.84861905E-01, 4.45969388E-01, 3.63755071E-01, 3.03555722E-01,
     &   2.62122079E-01, 2.31161105E-01, 2.07090404E-01, 1.87825746E-01,
     &   1.72053763E-01, 1.70186678E-01, 1.57174284E-01, 1.46157809E-01,
     &   1.36710112E-01, 1.28518104E-01, 1.21346717E-01, 1.15016264E-01,
     &   1.09386686E-01, 1.04347924E-01, 9.98116113E-02, 9.57057390E-02,
     &   9.19719646E-02, 8.85618110E-02, 8.54348830E-02, 8.25573016E-02,
     &   7.99003671E-02, 7.74393403E-02, 7.51540487E-02, 7.30253516E-02,
     &   7.10386561E-02, 6.91794560E-02, 6.74360466E-02, 6.57979350E-02,
     &   6.42558568E-02, 6.28015974E-02, 6.14278602E-02, 6.01283532E-02,
     &   5.88968336E-02, 5.77282507E-02, 5.66176485E-02, 5.55612184E-02,
     &   5.45549135E-02, 5.35952427E-02, 5.26790345E-02, 5.18034347E-02,
     &   5.09657646E-02, 5.01636235E-02, 4.93944521E-02, 4.86574317E-02,
     &   4.79492301E-02, 4.72684299E-02, 4.66140505E-02, 4.59842871E-02,
     &   4.53778024E-02, 4.47933043E-02, 4.42296341E-02, 4.36856971E-02/

         DATA ((DGS  (I,J), I= 801, 900), J=1,1) /
     &   4.31602357E-02, 4.26533548E-02, 4.21627962E-02, 4.16879914E-02,
     &   4.12288092E-02, 4.07842352E-02, 4.03535854E-02, 3.99362170E-02,
     &   3.95315423E-02, 3.91390071E-02, 3.87580918E-02, 3.83883120E-02,
     &   3.80290882E-02, 3.76803739E-02, 3.73413957E-02, 3.70119103E-02,
     &   3.66915391E-02, 3.63799603E-02, 3.60768669E-02, 3.57819740E-02,
     &   3.54949951E-02, 3.52157361E-02, 3.49441710E-02, 3.46794292E-02,
     &   3.44220545E-02, 3.41716596E-02, 3.39281070E-02, 3.36913294E-02,
     &   3.34612956E-02, 3.32380184E-02, 3.30215734E-02, 3.28122524E-02,
     &   3.26100288E-02, 3.24153536E-02, 3.22290777E-02, 3.20520017E-02,
     &   3.15067005E-02, 3.13081015E-02, 3.11135294E-02, 3.09228577E-02,
     &   3.07359639E-02, 3.05529407E-02, 3.03732460E-02, 3.01967633E-02,
     &   3.00238836E-02, 2.98709921E-02, 2.96876684E-02, 2.95242377E-02,
     &   2.93637830E-02, 2.92062257E-02, 2.90516377E-02, 2.88995951E-02,
     &   2.87502142E-02, 2.86030612E-02, 2.84590127E-02, 2.83171960E-02,
     &   2.40413721E-02, 2.10315799E-02, 1.83558113E-02, 1.59443676E-02,
     &   1.43472152E-02, 1.36018873E-02, 1.27305380E-02, 1.16609354E-02,
     &   1.06441696E-02, 9.24212922E-03, 8.51507681E-03, 7.43034882E-03,
     &   7.45125965E-03, 7.12792389E-03, 6.20799022E-03, 5.20464053E-03,
     &   4.06280847E-03, 2.72857476E-03, 1.13550970E-03,-8.13508005E-04,
     &  -3.26693239E-03,-4.44006844E-03,-5.75369067E-03,-7.23596061E-03,
     &  -8.92304597E-03,-1.08621912E-02,-1.31163438E-02,-1.43871568E-02,
     &  -1.57712398E-02,-1.72846312E-02,-1.89467247E-02,-2.07809795E-02,
     &  -2.28156734E-02,-2.50863729E-02,-2.76370416E-02,-3.05234274E-02,
     &  -3.38171280E-02,-3.76119701E-02,-4.20321918E-02,-4.70212022E-02,
     &  -5.02869723E-02,-5.39108364E-02,-5.79554556E-02,-6.24990481E-02/

         DATA ((DGS  (I,J), I= 901,1000), J=1,1) /
     &  -6.76405491E-02,-7.35069750E-02,-8.02641808E-02,-8.81329922E-02,
     &  -9.74141133E-02,-1.08528272E-01,-1.22082653E-01,-1.38989072E-01,
     &  -1.60684182E-01,-1.89582343E-01,-2.30109500E-01,-2.91570699E-01,
     &  -4.00285317E-01,-5.92573839E-01,-8.67895508E-01,-1.02755926E+00,
     &  -1.20231547E+00,-1.63332412E+00,-1.78911477E+00,-1.83882275E+00,
     &  -2.02564940E+00,-2.09736115E+00,-2.19318188E+00,-2.25792241E+00,
     &  -2.27248444E+00,-2.31459535E+00,-2.38152957E+00,-2.40449061E+00,
     &  -2.41807310E+00,-2.42291143E+00,-2.41929977E+00,-2.40965455E+00,
     &  -2.39018810E+00,-2.36479800E+00,-2.33367483E+00,-2.29104031E+00,
     &  -2.24066862E+00,-2.17610464E+00,-2.11073426E+00,-2.03565456E+00,
     &  -1.95034270E+00,-1.83736170E+00,-1.72868616E+00,-1.58793687E+00,
     &  -1.45541577E+00,-1.26221006E+00,-1.05136035E+00,-7.36451306E-01,
     &  -4.61959132E-01,-1.45784353E-01, 1.43650247E-02, 1.74516226E-01,
     &   3.33634330E-01, 7.65224822E-01, 1.08018006E+00, 1.29107131E+00,
     &   1.48432389E+00, 1.61688269E+00, 1.75767829E+00, 1.86639450E+00,
     &   1.97942382E+00, 2.06477716E+00, 2.13989804E+00, 2.20531453E+00,
     &   2.26992431E+00, 2.32028121E+00, 2.36295880E+00, 2.39417440E+00,
     &   2.41960296E+00, 2.43911104E+00, 2.44879082E+00, 2.45244415E+00,
     &   2.44764468E+00, 2.43410223E+00, 2.41118185E+00, 2.34399051E+00,
     &   2.30193441E+00, 2.28769248E+00, 2.22299178E+00, 2.12699746E+00,
     &   2.05554275E+00, 1.86954651E+00, 1.81907169E+00, 1.66331600E+00,
     &   1.47500585E+00, 1.57724011E+00, 1.35250334E+00, 1.21426287E+00,
     &   1.13076656E+00, 9.58179916E-01, 8.97998484E-01, 7.73338873E-01,
     &   9.38382827E-01, 3.63883418E-01, 2.85702157E-01, 2.36943782E-01,
     &   2.03280867E-01, 1.78557051E-01, 1.59597807E-01, 1.44582935E-01/

         DATA ((DGS  (I,J), I=1001,1100), J=1,1) /
     &   1.32388408E-01, 1.22280334E-01, 1.13756340E-01, 1.06447768E-01,
     &   1.00150160E-01, 9.47448482E-02, 8.99536601E-02, 8.56949144E-02,
     &   8.18905926E-02, 7.84737868E-02, 7.30452378E-02, 6.62693442E-02,
     &   6.26370863E-02, 5.66868394E-02, 5.20154812E-02, 4.74078298E-02,
     &   4.08386641E-02, 3.34149089E-02, 2.87721879E-02, 2.66136836E-02,
     &   2.54933751E-02, 2.46650726E-02, 2.40541800E-02, 2.35587190E-02,
     &   2.31934971E-02, 2.28738458E-02, 2.26040726E-02, 2.23738641E-02,
     &   2.21744526E-02, 2.20445956E-02, 2.18474517E-02, 2.17131296E-02,
     &   2.15947500E-02, 2.14904750E-02, 2.13991240E-02, 2.13190307E-02,
     &   2.12493658E-02, 2.11894422E-02, 2.11386261E-02, 2.10966556E-02,
     &   2.10073007E-02, 2.10385024E-02, 2.10224858E-02, 2.10157248E-02,
     &   2.10168813E-02, 2.10316770E-02, 2.10615579E-02, 2.11048862E-02,
     &   2.11691565E-02, 2.12643207E-02, 2.14173643E-02, 2.19063648E-02,
     &   2.25495132E-02, 2.28553450E-02, 2.31036063E-02, 2.33204920E-02,
     &   2.35164618E-02, 2.36955664E-02, 2.38616156E-02, 2.40156774E-02,
     &   2.41592253E-02, 2.42941177E-02, 2.44159875E-02, 2.45303647E-02,
     &   2.45700041E-02, 2.46761546E-02, 2.47616136E-02, 2.48504299E-02,
     &   2.49113989E-02, 2.49746886E-02, 2.49984288E-02, 2.50081412E-02,
     &   2.50013882E-02, 2.49764879E-02, 2.49315820E-02, 2.48642620E-02,
     &   2.47602563E-02, 2.46345193E-02, 2.44911101E-02, 2.43067714E-02,
     &   2.40769283E-02, 2.37875034E-02, 2.34258700E-02, 2.29640525E-02,
     &   2.23414545E-02, 2.14458643E-02, 2.11785734E-02, 2.04691182E-02,
     &   1.94042613E-02, 1.92945460E-02, 1.85586817E-02, 1.82593075E-02,
     &   1.80210601E-02, 1.78191083E-02, 1.75041361E-02, 1.71694778E-02,
     &   1.70977853E-02, 1.70881352E-02, 1.70321013E-02, 1.69294924E-02/

         DATA ((DGS  (I,J), I=1101,1200), J=1,1) /
     &   1.68739442E-02, 1.68921518E-02, 1.71128976E-02, 1.70898126E-02,
     &   1.68496489E-02, 1.67532730E-02, 1.65659322E-02, 1.64051996E-02,
     &   1.62444671E-02, 1.60837345E-02, 1.59700739E-02, 1.56875540E-02,
     &   1.52637726E-02, 1.46799745E-02, 1.43068865E-02, 1.37353743E-02,
     &   1.33620520E-02, 1.35575571E-02, 1.44514772E-02, 1.50630719E-02,
     &   1.64417292E-02, 1.70226878E-02, 1.74983798E-02, 1.80290631E-02,
     &   1.84406462E-02, 1.83564557E-02, 1.82484094E-02, 1.84613129E-02,
     &   1.90812038E-02, 1.86280522E-02, 1.80618294E-02, 1.81623584E-02,
     &   1.83549571E-02, 1.84910279E-02, 1.86885610E-02, 1.87140577E-02,
     &   1.85795863E-02, 1.82920171E-02, 1.81200022E-02, 1.77908899E-02,
     &   1.77507769E-02, 1.79710693E-02, 1.82203110E-02, 1.85311876E-02,
     &   1.86680698E-02, 1.89583187E-02, 1.90807934E-02, 1.89137096E-02,
     &   1.90749575E-02, 1.94362173E-02, 1.98199852E-02, 2.02096852E-02,
     &   2.04420803E-02, 2.06395123E-02, 2.08830601E-02, 2.10681764E-02,
     &   2.12013879E-02, 2.13432806E-02, 2.15451408E-02, 2.17191141E-02,
     &   2.18714363E-02, 2.20281067E-02, 2.17744123E-02, 2.09824284E-02,
     &   2.08089775E-02, 2.09345244E-02, 2.11960266E-02, 2.08916485E-02,
     &   2.08891456E-02, 2.11074323E-02, 2.12466416E-02, 2.00518401E-02,
     &   1.87823302E-02, 1.83585326E-02, 1.80547629E-02, 1.85370717E-02,
     &   1.85486567E-02, 1.83484468E-02, 1.81720935E-02, 1.80690873E-02,
     &   1.79229804E-02, 1.77278610E-02, 1.71832221E-02, 1.72205724E-02,
     &   1.72688424E-02, 1.71713185E-02, 1.71353489E-02, 1.71156015E-02,
     &   1.72005076E-02, 1.72426934E-02, 1.72658063E-02, 1.73285174E-02,
     &   1.73079440E-02, 1.73753194E-02, 1.79904070E-02, 1.86469935E-02,
     &   1.88321696E-02, 1.88871258E-02, 1.89210516E-02, 1.87112394E-02/

         DATA ((DGS  (I,J), I=1201,1300), J=1,1) /
     &   1.91450614E-02, 1.96806949E-02, 1.98858323E-02, 1.99709889E-02,
     &   2.01186977E-02, 2.01407897E-02, 1.99960731E-02, 1.97399768E-02,
     &   1.96526283E-02, 1.98337833E-02, 1.98346688E-02, 1.93152011E-02,
     &   1.93410245E-02, 1.96139758E-02, 1.95589251E-02, 1.92787653E-02,
     &   1.91430020E-02, 1.91959285E-02, 1.93697544E-02, 1.93808150E-02,
     &   1.97668803E-02, 1.99558514E-02, 2.03415956E-02, 2.07514488E-02,
     &   2.07894334E-02, 2.08229493E-02, 2.08341212E-02, 2.10506384E-02,
     &   2.13816640E-02, 2.15224715E-02, 2.16536090E-02, 2.18953511E-02,
     &   2.21173511E-02, 2.22226438E-02, 2.23247117E-02, 2.24238730E-02,
     &   2.25203834E-02, 2.26144523E-02, 2.27062531E-02, 2.27959312E-02,
     &   2.28836102E-02, 2.29693953E-02, 2.30533770E-02, 2.31356336E-02,
     &   2.32162374E-02, 2.32952379E-02, 2.33726907E-02, 2.34486412E-02,
     &   2.35231268E-02, 2.35961815E-02, 2.36678338E-02, 2.37381073E-02,
     &   2.38070229E-02, 2.38745967E-02, 2.39408417E-02, 2.40057677E-02,
     &   2.40693808E-02, 2.41316845E-02, 2.41926789E-02, 2.42523610E-02,
     &   2.43107248E-02, 2.43677611E-02, 2.44234573E-02, 2.44777975E-02,
     &   2.45307621E-02, 2.45823279E-02, 2.46324676E-02, 2.46715311E-02,
     &   2.47283357E-02, 2.48180520E-02, 2.49012095E-02, 2.49772866E-02,
     &   2.50456165E-02, 2.51053452E-02, 2.51553679E-02, 2.51942399E-02,
     &   2.52200420E-02, 2.52301694E-02, 2.52209943E-02, 2.51872965E-02,
     &   2.51212682E-02, 2.50105838E-02, 2.48344982E-02, 2.48120579E-02,
     &   2.45548313E-02, 2.40920103E-02, 2.32458276E-02, 2.18757190E-02,
     &   2.13178840E-02, 1.90339045E-02, 1.48514528E-02, 1.29323503E-02,
     &   1.02443022E-02, 8.45189805E-03, 6.21147222E-03, 3.97157190E-03,
     &   1.17302719E-03,-5.05150618E-04,-2.42202437E-03,-4.63224831E-03/

         DATA ((DGS  (I,J), I=1301,1400), J=1,1) /
     &  -7.20852072E-03,-1.02497026E-02,-1.38937607E-02,-1.60006892E-02,
     &  -1.83392624E-02,-2.09498664E-02,-2.38828844E-02,-2.72020248E-02,
     &  -3.09891141E-02,-3.53512446E-02,-4.04320961E-02,-4.64329998E-02,
     &  -4.89444600E-02,-5.05282660E-02,-5.21930314E-02,-5.39888233E-02,
     &  -5.57140762E-02,-5.75042149E-02,-5.33381269E-02,-6.14102533E-02,
     &  -6.35149868E-02,-6.57267212E-02,-6.80523909E-02,-7.05001489E-02,
     &  -7.30792810E-02,-7.58002335E-02,-7.86744008E-02,-8.17158466E-02,
     &  -8.49383528E-02,-8.83587859E-02,-9.19958111E-02,-9.58705434E-02,
     &  -1.00006956E-01,-1.04432377E-01,-1.09178087E-01,-1.14280019E-01,
     &  -1.19779905E-01,-1.25725931E-01,-1.32174644E-01,-1.39192510E-01,
     &  -1.46858254E-01,-1.55265816E-01,-1.64528169E-01,-1.74782154E-01,
     &  -1.86194573E-01,-1.98981960E-01,-2.13396817E-01,-2.29775999E-01,
     &  -2.48549661E-01,-2.70284197E-01,-2.95739691E-01,-3.25960712E-01,
     &  -3.62423057E-01,-4.07281947E-01,-4.63813480E-01,-1.70702169E-02,
     &  -6.36514285E-01,-7.78107307E-01,-9.96407607E-01,-1.37673677E+00,
     &  -2.20435098E+00,-2.71745043E+00,-5.20123150E+00,-7.24682905E+00,
     &  -8.73220903E+00,-9.87773789E+00,-2.76017189E-02,-1.16455368E+01,
     &  -1.24120818E+01,-1.29455663E+01,-1.34871487E+01,-1.40207625E+01,
     &  -1.42780076E+01,-1.46385249E+01,-1.48568791E+01,-1.51275196E+01,
     &  -1.51927619E+01,-1.53125657E+01,-1.53391624E+01,-1.53067170E+01,
     &  -1.52572602E+01,-1.51826174E+01,-1.86542141E-01,-1.82448120E-01,
     &  -1.45771066E+01,-1.40526841E+01,-1.37271208E+01,-1.33601171E+01,
     &  -1.29369866E+01,-1.19309241E+01,-1.13443469E+01,-1.07086720E+01,
     &  -9.26373084E+00,-8.46388451E+00,-2.58342223E+00,-4.77023095E+00,
     &   5.38883061E-01, 4.75657516E+00, 6.69935265E+00, 8.45693331E+00/

         DATA ((DGS  (I,J), I=1401,1500), J=1,1) /
     &   1.00187469E+01, 1.07208292E+01, 1.13599865E+01, 1.19439823E+01,
     &   1.29549464E+01, 1.33797554E+01, 1.37591624E+01, 1.41085995E+01,
     &   1.46215609E+01, 1.48215131E+01, 1.49912933E+01, 1.52304269E+01,
     &   1.53076406E+01, 1.53865012E+01, 1.53935965E+01, 1.53527786E+01,
     &   1.52517829E+01, 1.53175745E+01, 1.51826453E+01, 1.47036239E+01,
     &   1.43517527E+01, 1.40934401E+01, 1.35501026E+01, 1.30089373E+01,
     &   1.24751197E+01, 1.17082231E+01, 1.08831035E+01, 9.93846115E+00,
     &   8.78965031E+00, 7.30498053E+00, 5.22687592E+00, 2.86505084E+00,
     &   1.65041725E+00, 1.16289537E+00, 9.00415734E-01, 7.36447302E-01,
     &   6.24313251E-01, 5.42941055E-01, 4.80866092E-01, 4.32221423E-01,
     &   3.93003143E-01, 3.60713715E-01, 3.33666432E-01, 3.10680989E-01,
     &   2.90906616E-01, 2.73714695E-01, 1.06686411E-01, 2.45295620E-01,
     &   2.33400903E-01, 2.22752454E-01, 2.13149363E-01, 2.04447881E-01,
     &   1.96527078E-01, 1.89286535E-01, 1.82642379E-01, 1.76523986E-01,
     &   1.70871472E-01, 3.98044525E-02, 1.60766861E-01, 1.56232989E-01,
     &   1.51999227E-01, 1.48036906E-01, 1.44320921E-01, 1.40829203E-01,
     &   1.37542283E-01, 1.34442940E-01, 1.31515906E-01, 1.28747642E-01,
     &   1.26126167E-01, 1.23640966E-01, 1.21283056E-01, 1.19045381E-01,
     &   1.16924536E-01, 1.14930941E-01, 1.13054905E-01, 1.11048893E-01,
     &   1.09210737E-01, 1.07467619E-01, 1.05804759E-01, 1.04214201E-01,
     &   1.02690207E-01, 9.12577264E-02, 8.42929736E-02, 7.76539054E-02,
     &   7.03227122E-02, 6.36557553E-02, 5.77910641E-02, 5.63533307E-02,
     &   5.09886516E-02, 4.29274361E-02, 3.77743188E-02, 3.75316022E-02,
     &   3.70149963E-02, 3.64429774E-02, 3.53539713E-02, 3.44166472E-02,
     &   3.14183178E-02, 2.87294715E-02, 2.71208181E-02, 2.50010149E-02/

         DATA ((DGS  (I,J), I=1501,1600), J=1,1) /
     &   2.25453827E-02, 1.97415921E-02, 1.61524213E-02, 1.12442909E-02,
     &  -1.00452096E-03,-3.32493230E-03,-6.05697500E-03,-6.84858789E-03,
     &  -8.65284128E-03,-1.06574214E-02,-1.28977046E-02,-1.54179223E-02,
     &  -1.82741190E-02,-2.15383789E-02,-2.53050108E-02,-2.96998358E-02,
     &  -3.48945630E-02,-4.11298390E-02,-4.87537763E-02,-5.82896716E-02,
     &  -7.05626570E-02,-8.69563220E-02,-1.09995207E-01,-1.45059449E-01,
     &  -2.02250370E-01,-3.16537177E-01,-4.70065902E-01,-4.95840429E-01,
     &  -5.24433421E-01,-5.56333422E-01,-5.92148689E-01,-6.32645566E-01,
     &  -6.78808986E-01,-7.31909612E-01,-7.93640666E-01,-8.66287404E-01,
     &  -9.53020062E-01,-1.05836513E+00,-1.18901537E+00,-1.35529646E+00,
     &  -1.57400873E+00,-1.87437899E+00,-2.31207481E+00,-3.00698764E+00,
     &  -4.26809260E+00,-7.12113653E+00,-8.12091251E+00,-8.55593845E+00,
     &  -9.78033792E+00,-1.05255524E+01,-1.10652535E+01,-9.60307129E+00,
     &  -8.42921546E+00,-1.26966565E+01,-1.22704164E+01,-8.46343151E-01,
     &  -4.41130023E+00,-4.30886128E+00,-4.25777167E+00,-1.01681250E+01,
     &  -1.01074341E+01,-1.00615298E+01,-1.00152976E+01,-9.53555712E+00,
     &  -8.76123602E+00,-7.83459913E+00,-6.95166529E+00,-7.32570486E+00,
     &  -6.21463946E+00,-5.11673096E+00,-4.47184426E+00,-3.85409926E+00,
     &  -3.16508544E+00,-2.17124372E+00,-1.44561304E+00, 1.12999495E-01,
     &   1.13333707E-01, 1.13445032E-01, 1.13500679E-01, 1.13556316E-01,
     &   1.13611943E-01, 1.13667560E-01, 1.14112138E-01, 1.14666956E-01,
     &   1.15220765E-01, 5.46942579E+00, 6.60843671E+00, 8.54032363E+00,
     &   9.28793072E+00, 9.98878083E+00, 1.05104377E+01, 1.02340756E+01,
     &   1.08902908E+01, 1.14980904E+01, 1.16801447E+01, 1.17830957E+01,
     &   1.18155248E+01, 1.18133936E+01, 1.17761438E+01, 1.17173636E+01/

         DATA ((DGS  (I,J), I=1601,1700), J=1,1) /
     &   8.51595975E+00, 1.14071269E+01, 1.12659173E+01, 1.11263202E+01,
     &   1.07760127E+01, 1.05946629E+01, 1.03001257E+01, 9.82923154E+00,
     &   8.17554194E+00, 6.88700174E+00, 5.90955442E+00, 5.15962539E+00,
     &   4.57091017E+00, 4.01333880E+00, 3.48234628E+00, 2.89170254E+00,
     &   2.69256869E+00, 2.51891585E+00, 2.36592351E+00, 2.13974712E+00,
     &   1.77576641E+00, 1.51833867E+00, 1.32677966E+00, 1.17873623E+00,
     &   1.06090773E+00, 9.64923720E-01, 8.85229877E-01, 8.18006720E-01,
     &   7.60545226E-01, 7.10862183E-01, 6.67479797E-01, 6.29272862E-01,
     &   5.95366974E-01, 5.65073187E-01, 5.37849349E-01, 3.70858122E-01,
     &   2.56400390E-01, 1.99194424E-01, 1.42080469E-01, 1.13064475E-01,
     &   9.58430442E-02, 8.43871739E-02, 7.62115925E-02, 7.00821559E-02,
     &   6.53152877E-02, 6.15015306E-02, 5.93376946E-02, 5.68124431E-02,
     &   5.46541251E-02, 5.27879913E-02, 5.11583056E-02, 4.97226603E-02,
     &   4.45172585E-02, 4.12464166E-02, 3.89950420E-02, 3.73458873E-02,
     &   3.60799759E-02, 3.50618594E-02, 3.42499616E-02, 3.35738826E-02,
     &   2.77527885E-02, 2.61239075E-02, 2.57155425E-02, 2.52440563E-02,
     &   2.46843741E-02, 2.39915790E-02, 2.35746652E-02, 2.30952557E-02,
     &   2.18648379E-02, 2.00173869E-02, 1.68123202E-02, 9.55393445E-03,
     &   2.63357386E-03, 8.48497135E-04,-1.21009342E-03,-3.61058854E-03,
     &  -6.44639241E-03,-9.84836642E-03,-1.40055358E-02,-1.92019973E-02,
     &  -2.58852243E-02,-3.48039366E-02,-4.73232112E-02,-6.61789101E-02,
     &  -9.72096166E-02,-1.02619758E-01,-1.08521954E-01,-1.14987773E-01,
     &  -1.22102579E-01,-1.29990918E-01,-1.38698230E-01,-1.48462593E-01,
     &  -1.59446531E-01,-1.71896479E-01,-1.86125090E-01,-2.02540761E-01,
     &  -2.21694064E-01,-2.44329082E-01,-2.71487054E-01,-3.04678623E-01/

         DATA ((DGS  (I,J), I=1701,1800), J=1,1) /
     &  -3.15041932E-01,-3.46170321E-01,-3.99499620E-01,-4.70597204E-01,
     &  -5.70099686E-01,-7.19251829E-01,-9.67519019E-01,-1.46243992E+00,
     &  -2.92525782E+00,-3.48291228E+00,-4.92378147E+00,-5.63056610E+00,
     &  -6.92256604E+00,-7.70908061E+00,-8.41019828E+00,-9.04564871E+00,
     &  -9.59224199E+00,-1.03630122E+01,-1.05931615E+01, 8.53107893E-02,
     &  -1.14848418E+01,-1.18563507E+01,-1.21719972E+01,-1.24333326E+01,
     &  -8.62949322E+00,-1.28702751E+01,-1.30304402E+01,-1.31771429E+01,
     &  -1.33048951E+01,-1.34038416E+01,-1.36278694E+01,-1.34247814E+01,
     &  -1.33377463E+01,-1.24965374E+01,-1.17858090E+01, 2.63252781E-01,
     &  -1.23969335E+01,-1.18525865E+01,-1.11738835E+01,-1.02091881E+01,
     &  -9.06266746E+00,-7.54627747E+00,-5.77886887E+00, 2.63647486E+00,
     &   2.72528420E-02, 2.65498631E+00, 5.83370032E+00, 7.60748005E+00,
     &   9.14171360E+00, 1.02995405E+01, 8.69505547E+00, 9.64836256E+00,
     &   1.05676572E+01, 1.14408700E+01, 1.21237566E+01, 1.28773551E+01,
     &   1.35459022E+01, 5.12678039E+00, 5.58147106E+00, 6.05495452E+00,
     &   9.48783215E-01, 7.23964962E+00, 7.87640456E+00, 8.54310899E+00,
     &   9.34006683E+00, 1.02756356E+01, 1.12407993E+01, 1.23243477E+01,
     &   1.16767366E+01, 1.07939292E+01, 9.90494469E+00, 9.11391296E+00,
     &   8.52126116E+00, 3.32523152E+00, 8.49161870E+00, 7.75131491E+00,
     &   6.98231329E+00, 6.07255609E+00, 4.97845334E+00, 3.53759977E+00,
     &   3.03304929E+00, 2.93841631E+00, 1.97060052E+00, 2.49678026E+00,
     &   2.20464718E+00, 1.51704764E+00, 1.02220842E+00, 7.73930988E-01,
     &   6.24775544E-01, 5.25285854E-01, 4.54187666E-01, 4.00852768E-01,
     &   3.59368199E-01, 3.26173286E-01, 2.99011920E-01, 2.76378552E-01,
     &   2.57226135E-01, 2.40804612E-01, 2.26577438E-01, 2.14126079E-01/

         DATA ((DGS  (I,J), I=1801,1900), J=1,1) /
     &   1.51868149E-01, 1.20738344E-01, 1.02063615E-01, 8.96321468E-02,
     &   8.07126941E-02, 7.39880936E-02, 6.87770714E-02, 6.12056117E-02,
     &   5.59639801E-02, 5.39037646E-02, 5.21174575E-02, 4.56848851E-02,
     &   4.09582845E-02, 3.81140774E-02, 3.62040191E-02, 3.48248076E-02,
     &   3.37755486E-02, 3.29446452E-02, 3.22646988E-02, 3.16908229E-02,
     &   3.07498070E-02, 3.07879842E-02, 3.04260426E-02, 3.00976688E-02,
     &   2.97968173E-02, 2.95187774E-02, 2.92575702E-02, 2.90090174E-02,
     &   2.87686902E-02, 2.85313540E-02, 2.82892835E-02, 2.80219096E-02,
     &   2.78437664E-02, 2.45202079E-02, 2.37363778E-02, 2.27671598E-02,
     &   2.19988027E-02, 2.13644373E-02, 2.12910316E-02, 2.04116168E-02,
     &   2.00002723E-02, 1.96787120E-02, 1.94083958E-02, 1.91842869E-02,
     &   1.91012137E-02, 1.88679988E-02, 1.87706633E-02, 1.87108934E-02,
     &   1.86886938E-02, 1.87023727E-02, 1.87510075E-02, 1.88359240E-02,
     &   1.89327442E-02, 1.91064383E-02, 1.92997085E-02, 1.95222851E-02,
     &   1.97897685E-02, 2.00969102E-02, 2.04517483E-02, 2.08564095E-02,
     &   2.13124065E-02, 2.18204748E-02, 2.24204539E-02, 2.31072043E-02,
     &   2.39003235E-02, 2.48889032E-02, 2.61955815E-02, 2.93891630E-02,
     &   3.24079889E-02, 3.35469605E-02, 3.43722358E-02, 3.50110660E-02,
     &   3.55466823E-02, 3.60003154E-02, 3.63729754E-02, 3.66972931E-02,
     &   3.69757069E-02, 3.72118072E-02, 3.74229028E-02, 3.75678286E-02,
     &   3.76939752E-02, 3.77936611E-02, 3.78607984E-02, 3.79008847E-02,
     &   3.79126417E-02, 3.78962004E-02, 3.78533769E-02, 3.77796798E-02,
     &   3.76758823E-02, 3.75428179E-02, 3.73781447E-02, 3.71243466E-02,
     &   3.68395287E-02, 3.65208711E-02, 3.63467139E-02, 3.59755998E-02,
     &   3.55462007E-02, 3.50359726E-02, 3.44462173E-02, 3.37281106E-02/

         DATA ((DGS  (I,J), I=1901,2000), J=1,1) /
     &   3.28245562E-02, 3.15992617E-02, 2.94390996E-02, 2.93233146E-02,
     &   2.86957661E-02, 2.85148470E-02, 2.81347914E-02, 2.79583683E-02,
     &   2.78168521E-02, 2.76993636E-02, 2.76002975E-02, 2.75156575E-02,
     &   2.74026426E-02, 2.73805264E-02, 2.73269656E-02, 2.72815733E-02,
     &   2.72382230E-02, 2.72115100E-02, 2.71863352E-02, 2.71670388E-02,
     &   2.71533341E-02, 2.71451116E-02, 2.70702489E-02, 2.70782952E-02,
     &   2.71529268E-02, 2.71655711E-02, 2.71868773E-02, 2.72134776E-02,
     &   2.72463387E-02, 2.72861887E-02, 2.73337602E-02, 2.73901917E-02,
     &   2.74563606E-02, 2.75348819E-02, 2.76281294E-02, 2.77319386E-02,
     &   2.78845342E-02, 2.80812269E-02, 2.85882902E-02, 2.89705677E-02,
     &   2.92200723E-02, 2.93532153E-02, 2.94613515E-02, 2.95523904E-02,
     &   2.96313808E-02, 2.97007710E-02, 2.97618094E-02, 2.98158039E-02,
     &   2.98637346E-02, 2.99070270E-02, 2.99459384E-02, 2.99798485E-02,
     &   3.00086409E-02, 3.00325968E-02, 3.00522756E-02, 3.00688778E-02,
     &   3.00827178E-02, 3.00933143E-02, 3.01094959E-02, 3.01046551E-02,
     &   3.01044332E-02, 3.01012291E-02, 3.00951896E-02, 3.00855880E-02,
     &   3.00723576E-02, 3.00558162E-02, 3.00354130E-02, 3.00113601E-02,
     &   2.99834417E-02, 2.99860738E-02, 2.99546251E-02, 2.99172411E-02,
     &   2.98723972E-02, 2.97760969E-02, 2.93806612E-02, 2.94627573E-02,
     &   2.94525851E-02, 2.94437636E-02, 2.94362921E-02, 2.94301603E-02,
     &   2.94253476E-02, 2.94217979E-02, 2.94194876E-02, 2.94184037E-02,
     &   2.94112713E-02, 2.94118840E-02, 2.94157515E-02, 2.92724933E-02,
     &   2.92781671E-02, 2.92855472E-02, 2.94380525E-02, 2.94471380E-02,
     &   2.94577121E-02, 2.94698493E-02, 2.94837731E-02, 2.94992448E-02,
     &   2.95166306E-02, 2.95360238E-02, 2.95575130E-02, 2.95814269E-02/

         DATA ((DGS  (I,J), I=2001,2100), J=1,1) /
     &   2.96076331E-02, 2.96370585E-02, 2.96699427E-02, 2.97068422E-02,
     &   2.97483795E-02, 2.97963635E-02, 2.98529161E-02, 2.99225571E-02,
     &   3.00163023E-02, 3.02537182E-02, 3.04914760E-02, 3.05880891E-02,
     &   3.06987216E-02, 3.07679311E-02, 3.08276183E-02, 3.07959095E-02,
     &   3.08367728E-02, 3.08739441E-02, 3.09079647E-02, 3.09390274E-02,
     &   3.09681754E-02, 3.09952108E-02, 3.10205060E-02, 3.10441440E-02,
     &   3.10661128E-02, 3.10868334E-02, 3.11060332E-02, 3.11237273E-02,
     &   3.11415410E-02, 3.11566110E-02, 3.11699059E-02, 3.11830490E-02,
     &   3.11951022E-02, 3.12061926E-02, 3.12163150E-02, 3.12254680E-02,
     &   3.12336478E-02, 3.12409273E-02, 3.12472400E-02, 3.12526454E-02,
     &   3.12609428E-02, 3.12634464E-02, 3.12650638E-02, 3.12656725E-02,
     &   3.12650500E-02, 3.12476764E-02, 3.11010675E-02, 3.09035289E-02,
     &   3.08794254E-02, 3.07833876E-02, 3.07066736E-02, 3.06319846E-02,
     &   3.05683613E-02, 3.05047721E-02, 3.04444188E-02, 3.03797551E-02,
     &   3.03113853E-02, 3.02414591E-02, 3.00748799E-02, 2.99083006E-02,
     &   2.99718106E-02, 3.00317778E-02, 3.00844742E-02, 3.01357772E-02,
     &   3.01819566E-02, 3.02277813E-02, 3.02696720E-02, 3.03117679E-02,
     &   3.03523913E-02, 3.03900671E-02, 3.04283858E-02, 3.04641421E-02,
     &   3.05006979E-02, 3.05349613E-02, 3.05701248E-02, 3.06031924E-02,
     &   3.06372263E-02, 3.06693121E-02, 3.07024080E-02, 3.07336700E-02,
     &   3.07645746E-02, 3.07965291E-02, 3.08267775E-02, 3.08580913E-02,
     &   3.08877655E-02, 3.09185147E-02, 3.09476795E-02, 3.09779250E-02,
     &   3.10066328E-02, 3.10364238E-02, 3.10647173E-02, 3.10940944E-02,
     &   3.11232735E-02, 3.11510059E-02, 3.11798200E-02, 3.12072163E-02,
     &   3.12356915E-02, 3.12627749E-02, 3.12909337E-02, 3.13177241E-02/

         DATA ((DGS  (I,J), I=2101,2174), J=1,1) /
     &   3.13455859E-02, 3.13721006E-02, 3.13996824E-02, 3.14259365E-02,
     &   3.14520683E-02, 3.14792603E-02, 3.15051510E-02, 3.15320971E-02,
     &   3.15577581E-02, 3.15769677E-02, 3.16050904E-02, 3.16317069E-02,
     &   3.16570705E-02, 3.16834892E-02, 3.17086675E-02, 3.17348960E-02,
     &   3.17610301E-02, 3.17859410E-02, 3.18118949E-02, 3.18366365E-02,
     &   3.18624164E-02, 3.18869940E-02, 3.19126052E-02, 3.19373417E-02,
     &   3.19630921E-02, 3.19871570E-02, 3.20124867E-02, 3.20366024E-02,
     &   3.20617363E-02, 3.20857038E-02, 3.21096001E-02, 3.21345076E-02,
     &   3.21582611E-02, 3.21830210E-02, 3.22066350E-02, 3.22312506E-02,
     &   3.22547282E-02, 3.23545957E-02, 3.23779111E-02, 3.24021733E-02,
     &   3.24252733E-02, 3.24493123E-02, 3.26330983E-02, 3.28846193E-02,
     &   3.29242302E-02, 3.46914989E-02, 3.54885808E-02, 3.75830785E-02,
     &   3.89865600E-02, 3.88176219E-02, 4.03930599E-02, 4.03980562E-02,
     &   4.15800653E-02, 4.16252800E-02, 4.16523913E-02, 4.22394930E-02,
     &   4.26305439E-02, 4.27007322E-02, 4.28154483E-02, 4.29854774E-02,
     &   4.31054465E-02, 4.31807971E-02, 4.39118662E-02, 4.39498522E-02,
     &   4.41725515E-02, 4.42124971E-02, 4.48280660E-02, 4.44569481E-02,
     &   4.45806033E-02, 4.46344129E-02, 4.47204264E-02, 4.49441557E-02,
     &   4.50392619E-02, 4.56686904E-02/

         DATA ((DGS  (I,J), I=   1, 100), J=2,2) /
     &   3.46015528E-06, 3.87189637E-06, 4.20519650E-06, 4.55396605E-06,
     &   4.98364854E-06, 4.88375507E-06, 7.91128449E-06, 6.09328378E-06,
     &   6.75241849E-06, 7.66730936E-06, 7.25276681E-06, 7.25028546E-06,
     &   7.02202312E-06, 7.41393548E-06, 7.82454105E-06, 8.26659312E-06,
     &   8.49564452E-06, 8.61480129E-06, 8.65745323E-06, 8.82595052E-06,
     &   9.03173409E-06, 9.24939840E-06, 9.35660945E-06, 9.48612647E-06,
     &   9.68040737E-06, 1.00474078E-05, 1.05506179E-05, 1.07827756E-05,
     &   1.08597181E-05, 1.11201123E-05, 1.14438502E-05, 1.15442763E-05,
     &   1.19070479E-05, 1.22710307E-05, 1.26639609E-05, 1.31057207E-05,
     &   1.35869721E-05, 1.42702196E-05, 1.48801472E-05, 1.55076014E-05,
     &   1.61820672E-05, 1.70054384E-05, 1.79622136E-05, 1.88750095E-05,
     &   2.03839340E-05, 2.26651518E-05, 2.44790181E-05, 2.51229728E-05,
     &   2.50770134E-05, 2.46353618E-05, 3.01509346E-05, 3.61310679E-05,
     &   4.15735346E-05, 4.24933165E-05, 4.67133694E-05, 4.74122687E-05,
     &   4.28016854E-05, 3.93115009E-05, 3.75950723E-05, 3.75432328E-05,
     &   3.88249973E-05, 3.87218763E-05, 3.96117524E-05, 3.94042928E-05,
     &   4.10075177E-05, 4.22745857E-05, 4.40386788E-05, 4.64686488E-05,
     &   4.80589337E-05, 4.89735264E-05, 5.28869653E-05, 4.99919710E-05,
     &   5.05522780E-05, 5.17262777E-05, 5.46319372E-05, 5.45904304E-05,
     &   5.22738631E-05, 5.25307576E-05, 5.27801151E-05, 1.27167646E-04,
     &   1.06886740E-04, 1.12017856E-04, 1.07396778E-04, 1.03234556E-04,
     &   9.26223128E-05, 9.24378416E-05, 9.66462333E-05, 9.22447790E-05,
     &   9.21992359E-05, 9.21227908E-05, 8.92498984E-05, 9.10206834E-05,
     &   9.25508081E-05, 9.84398348E-05, 1.09726118E-04, 1.26374790E-04,
     &   1.18302411E-04, 1.10108946E-04, 9.87268916E-05, 9.03380616E-05/

         DATA ((DGS  (I,J), I= 101, 200), J=2,2) /
     &   8.28069577E-05, 7.96808351E-05, 8.02500312E-05, 8.44983692E-05,
     &   9.13008165E-05, 9.69031864E-05, 1.02394066E-04, 1.07773456E-04,
     &   1.12040519E-04, 1.16244732E-04, 1.19996774E-04, 1.19551062E-04,
     &   1.19091707E-04, 1.18607336E-04, 1.15234874E-04, 1.08873323E-04,
     &   9.78456114E-05, 7.50338067E-05, 8.50581958E-05, 9.76060435E-05,
     &   1.07150191E-04, 1.14425442E-04, 1.17692443E-04, 1.20279481E-04,
     &   1.18863725E-04, 1.13755253E-04, 1.06930887E-04, 9.86143944E-05,
     &   9.00045101E-05, 8.05072844E-05, 7.42138170E-05, 7.30100312E-05,
     &   7.92553099E-05, 9.09391849E-05, 1.06281194E-04, 1.20980190E-04,
     &   1.30310096E-04, 1.28697803E-04, 1.14744264E-04, 1.00311463E-04,
     &   1.00203291E-04, 9.41437368E-05, 8.98233981E-05, 8.35620265E-05,
     &   8.00032305E-05, 7.78879162E-05, 7.60415947E-05, 7.77253579E-05,
     &   7.60285198E-05, 7.52450919E-05, 7.16297418E-05, 7.68698736E-05,
     &   7.74297087E-05, 6.54588856E-05, 5.34880625E-05, 4.95645493E-05,
     &   4.77075490E-05, 4.56887804E-05, 4.58178064E-05, 4.28344327E-05,
     &   4.51889294E-05, 5.44113268E-05, 5.65311721E-05, 5.36111558E-05,
     &   5.01964435E-05, 4.72391858E-05, 4.64036747E-05, 4.57038812E-05,
     &   4.52351726E-05, 4.51028565E-05, 4.52359488E-05, 4.51182534E-05,
     &   4.71662632E-05, 4.77735231E-05, 4.88874628E-05, 4.96816815E-05,
     &   5.01635245E-05, 5.05240210E-05, 5.15050284E-05, 5.19840011E-05,
     &   5.33640041E-05, 5.41082181E-05, 5.60783981E-05, 5.79191827E-05,
     &   6.11413411E-05, 6.39053479E-05, 6.79877363E-05, 7.15600717E-05,
     &   7.58873643E-05, 8.17866385E-05, 8.53946341E-05, 8.80524321E-05,
     &   9.41404417E-05, 9.75283148E-05, 1.00977013E-04, 1.05091184E-04,
     &   1.09803184E-04, 1.15342604E-04, 1.20720209E-04, 1.30730264E-04/

         DATA ((DGS  (I,J), I= 201, 300), J=2,2) /
     &   1.42024414E-04, 1.49269695E-04, 1.52095116E-04, 1.55084873E-04,
     &   1.58247553E-04, 1.61594183E-04, 1.65127476E-04, 1.68780153E-04,
     &   1.72511321E-04, 1.76424824E-04, 1.80523298E-04, 1.84813622E-04,
     &   1.89304831E-04, 1.93936234E-04, 1.98774282E-04, 2.03899122E-04,
     &   2.09312270E-04, 2.15076164E-04, 2.21269031E-04, 2.27961420E-04,
     &   2.35235634E-04, 2.43188784E-04, 2.51922862E-04, 2.61552753E-04,
     &   2.72204163E-04, 2.84073687E-04, 2.97247479E-04, 3.12149778E-04,
     &   3.29393659E-04, 3.51655041E-04, 3.71965411E-04, 3.98515906E-04,
     &   3.68096549E-04, 3.69246335E-04, 3.77995335E-04, 3.89114314E-04,
     &   3.98907814E-04, 4.14502031E-04, 4.33165088E-04, 4.51029948E-04,
     &   4.72252258E-04, 4.95757496E-04, 5.17308544E-04, 5.50392787E-04,
     &   5.83477030E-04, 6.20047263E-04, 6.61371058E-04, 7.08331304E-04,
     &   7.62010480E-04, 8.23722251E-04, 8.94953952E-04, 9.77397342E-04,
     &   1.07350684E-03, 1.16684377E-03, 1.21798273E-03, 1.25282887E-03,
     &   1.30059193E-03, 1.32379872E-03, 1.31001103E-03, 1.39526149E-03,
     &   1.42913318E-03, 1.45995403E-03, 1.48731735E-03, 1.50907371E-03,
     &   1.52274181E-03, 1.52532808E-03, 1.50111233E-03, 1.45782796E-03,
     &   1.42701175E-03, 1.34307645E-03, 1.22537845E-03, 1.07024959E-03,
     &   8.16877436E-04, 6.48187518E-04, 3.92844951E-04, 1.43493163E-04,
     &   2.12393932E-04, 4.71921215E-04, 7.20150843E-04, 9.38356212E-04,
     &   1.11992056E-03, 1.26368475E-03, 1.37066488E-03, 1.44566232E-03,
     &   1.49354706E-03, 1.51915055E-03, 1.52721560E-03, 1.52194201E-03,
     &   1.50671729E-03, 1.48420991E-03, 1.44191262E-03, 1.43034160E-03,
     &   1.37044848E-03, 1.35564319E-03, 1.32380583E-03, 1.28932581E-03,
     &   1.25552219E-03, 1.22277934E-03, 1.19037360E-03, 1.15778842E-03/

         DATA ((DGS  (I,J), I= 301, 400), J=2,2) /
     &   1.15348629E-03, 1.09816160E-03, 1.06929422E-03, 1.04154891E-03,
     &   9.54545926E-04, 8.80288258E-04, 8.18106337E-04, 7.63834373E-04,
     &   7.16555412E-04, 6.65771225E-04, 6.39024879E-04, 6.07114096E-04,
     &   5.78891231E-04, 5.53837570E-04, 5.31526875E-04, 5.11613896E-04,
     &   4.93819516E-04, 4.77844650E-04, 4.63439806E-04, 4.50388522E-04,
     &   3.91856011E-04, 3.79620534E-04, 3.82251884E-04, 3.65974130E-04,
     &   3.71963046E-04, 4.20717299E-04, 4.18435571E-04, 3.67512184E-04,
     &   3.50062453E-04, 3.40481911E-04, 3.32035233E-04, 3.25538285E-04,
     &   3.19135859E-04, 3.12095391E-04, 3.06510957E-04, 2.99457810E-04,
     &   2.92783227E-04, 2.86174253E-04, 2.78729118E-04, 2.72726313E-04,
     &   2.65545705E-04, 2.58017676E-04, 2.51652600E-04, 2.46164848E-04,
     &   2.40025218E-04, 2.34884604E-04, 2.29198863E-04, 2.24135444E-04,
     &   2.19277495E-04, 2.14497910E-04, 2.09987713E-04, 2.05680420E-04,
     &   2.01051197E-04, 1.96668381E-04, 1.92408103E-04, 1.88117099E-04,
     &   1.77583201E-04, 1.56403262E-04, 1.27572912E-04, 1.12675637E-04,
     &   1.01195303E-04, 9.28147492E-05, 8.56675321E-05, 7.94312606E-05,
     &   7.35078044E-05, 7.26387256E-05, 6.65062675E-05, 6.54700456E-05,
     &   6.44097509E-05, 6.12796239E-05, 5.97416599E-05, 5.89407661E-05,
     &   5.74107344E-05, 5.65251048E-05, 5.46470895E-05, 5.59882208E-05,
     &   5.29230580E-05, 5.16650619E-05, 5.16912508E-05, 5.15768587E-05,
     &   4.91677879E-05, 5.00031076E-05, 4.74307732E-05, 4.78875490E-05,
     &   4.72793107E-05, 4.69666321E-05, 4.72449884E-05, 4.58799755E-05,
     &   4.72175833E-05, 4.61757098E-05, 4.59538598E-05, 4.66257389E-05,
     &   4.59761905E-05, 4.86430057E-05, 4.73190236E-05, 4.96470713E-05,
     &   4.86643674E-05, 5.12452428E-05, 5.06352549E-05, 5.08781336E-05/

         DATA ((DGS  (I,J), I= 401, 500), J=2,2) /
     &   5.15470698E-05, 5.51235712E-05, 5.70551754E-05, 5.37981547E-05,
     &   5.28104495E-05, 5.41370473E-05, 5.67806046E-05, 5.89698235E-05,
     &   5.91024868E-05, 5.68667409E-05, 5.84918930E-05, 5.86593834E-05,
     &   5.98997199E-05, 6.47728613E-05, 6.34820470E-05, 6.03637887E-05,
     &   6.18905507E-05, 5.91386063E-05, 5.95956473E-05, 6.39337104E-05,
     &   6.47086999E-05, 6.60776134E-05, 6.80061630E-05, 6.62858369E-05,
     &   6.42956118E-05, 6.63519580E-05, 6.63838380E-05, 7.19734111E-05,
     &   7.20825934E-05, 7.05999648E-05, 7.33719942E-05, 7.00833584E-05,
     &   7.12731780E-05, 7.30760775E-05, 7.62795266E-05, 7.22919581E-05,
     &   7.24667722E-05, 7.11449862E-05, 7.35738133E-05, 7.93177227E-05,
     &   7.82768906E-05, 7.95848341E-05, 7.76716444E-05, 7.80731420E-05,
     &   7.68186406E-05, 7.61608357E-05, 8.38886616E-05, 7.97826333E-05,
     &   8.30130440E-05, 7.77339282E-05, 8.62092628E-05, 8.60555991E-05,
     &   8.63926200E-05, 8.78656983E-05, 8.37663962E-05, 7.95384018E-05,
     &   7.91066205E-05, 8.14822472E-05, 8.70464643E-05, 8.81774733E-05,
     &   8.86926516E-05, 8.79175252E-05, 8.85703786E-05, 8.73651794E-05,
     &   8.85703875E-05, 8.95024199E-05, 8.88203141E-05, 9.21841599E-05,
     &   9.85365243E-05, 1.00314809E-04, 1.01861538E-04, 1.10911163E-04,
     &   1.23274194E-04, 1.37590084E-04, 1.40690984E-04, 1.66244199E-04,
     &   1.72860117E-04, 1.62415082E-04, 1.58564641E-04, 1.61894685E-04,
     &   1.62041599E-04, 1.99228737E-04, 2.17000106E-04, 2.16742244E-04,
     &   1.84106596E-04, 1.93526329E-04, 2.03039277E-04, 2.12648990E-04,
     &   1.91285169E-04, 1.93896132E-04, 1.93029299E-04, 1.94287267E-04,
     &   4.14516138E-04, 1.94523722E-04, 1.77265366E-04, 1.70669779E-04,
     &   1.80124270E-04, 2.00304993E-04, 1.84644538E-04, 2.03338369E-04/

         DATA ((DGS  (I,J), I= 501, 600), J=2,2) /
     &   2.10013832E-04, 2.14853211E-04, 2.21605159E-04, 2.18325576E-04,
     &   2.26957997E-04, 2.29003420E-04, 2.07257865E-04, 1.98595788E-04,
     &   2.01950181E-04, 2.13771737E-04, 2.31376286E-04, 2.55029581E-04,
     &   2.86774720E-04, 3.31473361E-04, 3.96937539E-04, 4.96452328E-04,
     &   5.24935773E-04, 5.53476470E-04, 5.87251578E-04, 6.25972966E-04,
     &   6.70252738E-04, 7.20964571E-04, 7.80476778E-04, 7.93398744E-04,
     &   8.06869422E-04, 8.20867662E-04, 8.35395810E-04, 8.50472511E-04,
     &   8.65633344E-04, 8.81412411E-04, 8.97835819E-04, 9.14946331E-04,
     &   9.32700231E-04, 1.02169430E-03, 1.09099773E-03, 1.12926379E-03,
     &   1.15615079E-03, 1.18429338E-03, 1.21385169E-03, 1.24484026E-03,
     &   1.27740021E-03, 1.31164362E-03, 1.32944219E-03, 1.34773359E-03,
     &   1.36650974E-03, 1.38581755E-03, 1.40565183E-03, 1.42606143E-03,
     &   1.44704426E-03, 1.46865144E-03, 1.49088435E-03, 1.51379673E-03,
     &   1.53739387E-03, 1.56172148E-03, 1.58681346E-03, 1.61270595E-03,
     &   1.63943753E-03, 1.66704934E-03, 1.69558530E-03, 1.72509235E-03,
     &   1.75562071E-03, 1.78722412E-03, 1.81996019E-03, 1.85389075E-03,
     &   1.88908219E-03, 1.92560597E-03, 1.96353902E-03, 2.00296434E-03,
     &   2.04397157E-03, 2.08665769E-03, 2.13112775E-03, 2.17749575E-03,
     &   2.22588859E-03, 2.27643879E-03, 2.32929373E-03, 2.38461452E-03,
     &   2.44257764E-03, 2.50337684E-03, 2.56722528E-03, 2.63435801E-03,
     &   2.70503493E-03, 2.77954404E-03, 2.81833335E-03, 2.85820547E-03,
     &   2.89920226E-03, 2.94137600E-03, 2.98477406E-03, 3.02945448E-03,
     &   3.07547098E-03, 3.12288824E-03, 3.17176736E-03, 3.22218069E-03,
     &   3.27419790E-03, 3.32790037E-03, 3.38336767E-03, 3.44069180E-03,
     &   3.49996400E-03, 3.56128871E-03, 3.62477094E-03, 3.69052992E-03/

         DATA ((DGS  (I,J), I= 601, 700), J=2,2) /
     &   3.75868628E-03, 3.82937861E-03, 3.90274627E-03, 3.97894620E-03,
     &   4.05814555E-03, 4.14052488E-03, 4.22627953E-03, 4.31562121E-03,
     &   4.40877968E-03, 4.50600477E-03, 4.60756861E-03, 4.71376812E-03,
     &   4.82492802E-03, 4.94140371E-03, 5.06358538E-03, 5.19190420E-03,
     &   5.32683282E-03, 5.46889687E-03, 5.61867603E-03, 5.77681774E-03,
     &   6.01365025E-03, 6.12115170E-03, 6.30905268E-03, 6.50876209E-03,
     &   6.72142728E-03, 6.94835269E-03, 7.19102046E-03, 7.45112897E-03,
     &   7.73062657E-03, 8.03176647E-03, 8.35716163E-03, 8.70986673E-03,
     &   9.09346839E-03, 9.39969262E-03, 1.03536572E-02, 1.15218943E-02,
     &   1.29857194E-02, 1.48735528E-02, 1.74007946E-02, 2.09584731E-02,
     &   2.63380404E-02, 3.54201298E-02, 5.40292430E-02, 1.13609465E-01,
     &   1.20135204E-01, 1.27453337E-01, 1.35716941E-01, 1.45121050E-01,
     &   1.55918007E-01, 1.68440041E-01, 1.83133338E-01, 2.00611005E-01,
     &   2.21738186E-01, 2.47774290E-01, 2.80621470E-01, 3.23281597E-01,
     &   3.80746367E-01, 3.87597666E-01, 5.41006997E-01, 6.52822404E-01,
     &   7.42659626E-01, 8.20790465E-01, 8.80996528E-01, 9.33601384E-01,
     &   9.75760510E-01, 1.01776034E+00, 1.04661173E+00, 1.07332573E+00,
     &   1.09217609E+00, 1.10771609E+00, 1.11644880E+00, 1.12296720E+00,
     &   1.12352148E+00, 1.12024092E+00, 1.11114337E+00, 1.10131108E+00,
     &   1.08004370E+00, 1.06098830E+00, 1.03759798E+00, 1.00950481E+00,
     &   9.76355859E-01, 9.37826540E-01, 8.93636208E-01, 8.43565578E-01,
     &   7.87475399E-01, 7.25325388E-01, 6.57192206E-01, 5.44270393E-01,
     &   4.62409826E-01, 3.31188633E-01, 1.92153044E-01, 1.92020457E-01,
     &   5.44154291E-01, 6.57085434E-01, 7.57059658E-01, 8.16188964E-01,
     &   8.93558506E-01, 9.37756456E-01, 9.76293350E-01, 1.00944977E+00/

         DATA ((DGS  (I,J), I= 701, 800), J=2,2) /
     &   1.03755023E+00, 1.06094765E+00, 1.08000990E+00, 1.09510880E+00,
     &   1.10661128E+00, 1.11487266E+00, 1.12023173E+00, 1.12352034E+00,
     &   1.12297105E+00, 1.11869142E+00, 1.11095930E+00, 1.10422691E+00,
     &   1.08775907E+00, 1.07335913E+00, 1.05220573E+00, 1.02952526E+00,
     &   9.99934451E-01, 9.69768277E-01, 9.27703002E-01, 8.86776868E-01,
     &   8.36774988E-01, 7.80380967E-01, 7.07844781E-01, 6.24691950E-01,
     &   5.20862850E-01, 4.29205352E-01, 3.58008043E-01, 3.06634394E-01,
     &   2.67952376E-01, 2.37832846E-01, 2.13742851E-01, 1.94050318E-01,
     &   1.77659415E-01, 1.63808426E-01, 1.51952096E-01, 1.41690177E-01,
     &   1.32722414E-01, 1.24819252E-01, 1.17802248E-01, 1.11530628E-01,
     &   1.05891872E-01, 1.00794975E-01, 9.61655530E-02, 5.04261291E-02,
     &   3.41763411E-02, 2.58521743E-02, 2.07924379E-02, 1.73917624E-02,
     &   1.49491294E-02, 1.31096939E-02, 1.16745882E-02, 1.05237057E-02,
     &   9.58022433E-03, 9.46848011E-03, 8.68929409E-03, 8.02922204E-03,
     &   7.46289318E-03, 6.97166419E-03, 6.54152804E-03, 6.16175712E-03,
     &   5.82399658E-03, 5.52164268E-03, 5.24940686E-03, 5.00300390E-03,
     &   4.77892480E-03, 4.57426840E-03, 4.38661546E-03, 4.21393254E-03,
     &   4.05449800E-03, 3.90684448E-03, 3.76971361E-03, 3.64202009E-03,
     &   3.52282291E-03, 3.41130217E-03, 3.30674019E-03, 3.20850604E-03,
     &   3.11604285E-03, 3.02885725E-03, 2.94651055E-03, 2.86861139E-03,
     &   2.79480950E-03, 2.72479047E-03, 2.65827128E-03, 2.59499644E-03,
     &   2.53473478E-03, 2.47727655E-03, 2.42243109E-03, 2.37002460E-03,
     &   2.31989842E-03, 2.27190733E-03, 2.22591822E-03, 2.18180882E-03,
     &   2.13946664E-03, 2.09878797E-03, 2.05967711E-03, 2.02204555E-03,
     &   1.98581134E-03, 1.95089846E-03, 1.91723636E-03, 1.88475939E-03/

         DATA ((DGS  (I,J), I= 801, 900), J=2,2) /
     &   1.85340647E-03, 1.82312064E-03, 1.79384872E-03, 1.76554105E-03,
     &   1.73815115E-03, 1.71163551E-03, 1.68595334E-03, 1.66106636E-03,
     &   1.63693862E-03, 1.61353631E-03, 1.59082764E-03, 1.56878265E-03,
     &   1.54737310E-03, 1.52657236E-03, 1.50635530E-03, 1.48669817E-03,
     &   1.46757851E-03, 1.44897509E-03, 1.43086780E-03, 1.41323762E-03,
     &   1.39606650E-03, 1.37933736E-03, 1.36303397E-03, 1.34714097E-03,
     &   1.33164377E-03, 1.31652852E-03, 1.30178210E-03, 1.28739205E-03,
     &   1.27334656E-03, 1.25963445E-03, 1.24624514E-03, 1.23316861E-03,
     &   1.22039543E-03, 1.20791674E-03, 1.19572423E-03, 1.18381017E-03,
     &   1.17216749E-03, 1.16078968E-03, 1.14967096E-03, 1.13880624E-03,
     &   1.12819135E-03, 1.11782312E-03, 1.10769963E-03, 1.09782055E-03,
     &   1.08818762E-03, 1.07972965E-03, 1.06920012E-03, 1.05978028E-03,
     &   1.05054075E-03, 1.04147660E-03, 1.03258308E-03, 1.02385562E-03,
     &   1.01528983E-03, 1.00688150E-03, 9.98626601E-04, 9.90518450E-04,
     &   7.62690515E-04, 6.14215091E-04, 5.19792389E-04, 5.03250942E-04,
     &   5.46854602E-04, 5.84408521E-04, 6.39463833E-04, 7.19512536E-04,
     &   8.02697858E-04, 9.15843000E-04, 9.62903368E-04, 1.07546167E-03,
     &   1.14331114E-03, 1.22141278E-03, 1.31153251E-03, 1.41701923E-03,
     &   1.54186459E-03, 1.69168960E-03, 1.87457679E-03, 2.10260195E-03,
     &   2.39445625E-03, 2.53534096E-03, 2.69391558E-03, 2.87367688E-03,
     &   3.07915520E-03, 3.31627119E-03, 3.59287544E-03, 3.74919802E-03,
     &   3.91971216E-03, 4.10643040E-03, 4.31177727E-03, 4.53867363E-03,
     &   4.79069678E-03, 5.07225136E-03, 5.38884577E-03, 5.74743923E-03,
     &   6.15697380E-03, 6.62911486E-03, 7.17939068E-03, 7.80070101E-03,
     &   8.20746411E-03, 8.65883184E-03, 9.16255969E-03, 9.72831228E-03/

         DATA ((DGS  (I,J), I= 901,1000), J=2,2) /
     &   1.03682871E-02, 1.10981038E-02, 1.19380927E-02, 1.29152153E-02,
     &   1.40660065E-02, 1.54412332E-02, 1.71135660E-02, 1.91908046E-02,
     &   2.18399879E-02, 2.53344706E-02, 3.01542266E-02, 3.72254914E-02,
     &   4.85913031E-02, 6.97816841E-02, 1.03110084E-01, 1.22079271E-01,
     &   1.42769857E-01, 1.93678361E-01, 2.12019479E-01, 2.27156807E-01,
     &   2.40011906E-01, 2.50836633E-01, 2.59717765E-01, 2.67364891E-01,
     &   2.73626398E-01, 2.78107766E-01, 2.81914879E-01, 2.84609975E-01,
     &   2.86208489E-01, 2.86785824E-01, 2.86375840E-01, 2.85255564E-01,
     &   2.82981990E-01, 2.80006906E-01, 2.75996658E-01, 2.70836970E-01,
     &   2.65411519E-01, 2.57810095E-01, 2.50124893E-01, 2.41291853E-01,
     &   2.31254287E-01, 2.17960456E-01, 2.05172711E-01, 1.88610377E-01,
     &   1.73015980E-01, 1.50280100E-01, 1.25467422E-01, 8.84080981E-02,
     &   5.61046758E-02, 1.88965034E-02, 2.65528302E-04, 1.88021387E-02,
     &   3.75272481E-02, 8.83186446E-02, 1.25383417E-01, 1.50201011E-01,
     &   1.72942432E-01, 1.88541299E-01, 2.05109090E-01, 2.17901651E-01,
     &   2.31201182E-01, 2.41243648E-01, 2.50081554E-01, 2.57771554E-01,
     &   2.65378450E-01, 2.70808456E-01, 2.75973277E-01, 2.79988445E-01,
     &   2.82968221E-01, 2.85246873E-01, 2.86371340E-01, 2.86786373E-01,
     &   2.86213715E-01, 2.84619983E-01, 2.81929678E-01, 2.78127274E-01,
     &   2.73650128E-01, 2.67393289E-01, 2.59750747E-01, 2.50873984E-01,
     &   2.40053675E-01, 2.27202933E-01, 2.12069862E-01, 1.93732983E-01,
     &   1.71502880E-01, 1.83571796E-01, 1.57035048E-01, 1.40690043E-01,
     &   1.30808392E-01, 1.10338155E-01, 1.03177038E-01, 8.77615345E-02,
     &   5.68260232E-02, 4.19283707E-02, 3.32013429E-02, 2.74766112E-02,
     &   2.34345755E-02, 2.04292553E-02, 1.81075095E-02, 1.62601142E-02/

         DATA ((DGS  (I,J), I=1001,1100), J=2,2) /
     &   1.47552728E-02, 1.35058509E-02, 1.24519425E-02, 1.15510114E-02,
     &   1.07720218E-02, 1.00917983E-02, 9.49268703E-03, 8.96100513E-03,
     &   8.48598260E-03, 8.05902722E-03, 7.38411120E-03, 6.54166240E-03,
     &   6.09109656E-03, 5.35484392E-03, 4.77870774E-03, 4.21377996E-03,
     &   3.41181424E-03, 2.52976940E-03, 2.00869581E-03, 1.79213477E-03,
     &   1.67121138E-03, 1.59248353E-03, 1.53691185E-03, 1.49501843E-03,
     &   1.46214782E-03, 1.43547168E-03, 1.41316500E-03, 1.39414005E-03,
     &   1.37757372E-03, 1.36290137E-03, 1.34972502E-03, 1.33770052E-03,
     &   1.32662311E-03, 1.31627281E-03, 1.30647355E-03, 1.29717823E-03,
     &   1.28821817E-03, 1.27950293E-03, 1.27098253E-03, 1.26258256E-03,
     &   1.25421963E-03, 1.24581827E-03, 1.23730420E-03, 1.22861475E-03,
     &   1.21964117E-03, 1.21026614E-03, 1.20031859E-03, 1.18959893E-03,
     &   1.17772916E-03, 1.16401177E-03, 1.14670667E-03, 1.10780251E-03,
     &   1.07465160E-03, 1.06335427E-03, 1.05588928E-03, 1.05049790E-03,
     &   1.04645952E-03, 1.04339613E-03, 1.04106676E-03, 1.03927776E-03,
     &   1.03790366E-03, 1.03682751E-03, 1.03594887E-03, 1.03515315E-03,
     &   1.03439001E-03, 1.03356063E-03, 1.03240600E-03, 1.03120601E-03,
     &   1.02949675E-03, 1.02738746E-03, 1.02469342E-03, 1.02132278E-03,
     &   1.01719633E-03, 1.01209129E-03, 1.00594447E-03, 9.98582489E-04,
     &   9.89717491E-04, 9.79006311E-04, 9.66789681E-04, 9.52007858E-04,
     &   9.34483100E-04, 9.13256951E-04, 8.87863790E-04, 8.55297072E-04,
     &   8.14144581E-04, 7.56847704E-04, 7.40507748E-04, 6.91236059E-04,
     &   6.22706667E-04, 6.16218839E-04, 5.72849345E-04, 5.58999184E-04,
     &   5.47895963E-04, 5.32388624E-04, 5.19158282E-04, 5.06940118E-04,
     &   4.99940691E-04, 4.95669570E-04, 4.85170276E-04, 4.74542032E-04/

         DATA ((DGS  (I,J), I=1101,1200), J=2,2) /
     &   4.65497574E-04, 4.56886169E-04, 4.49082525E-04, 4.42021311E-04,
     &   4.35303852E-04, 4.29520037E-04, 4.23975411E-04, 4.17277607E-04,
     &   4.10579804E-04, 4.03882001E-04, 3.98253096E-04, 3.92234739E-04,
     &   3.88784734E-04, 3.84266438E-04, 3.84457700E-04, 3.80819775E-04,
     &   3.76905841E-04, 3.73638074E-04, 3.75307980E-04, 3.74540903E-04,
     &   3.74606555E-04, 3.74760013E-04, 3.73299794E-04, 3.84359423E-04,
     &   3.94164143E-04, 4.56283219E-04, 5.24069508E-04, 4.84648323E-04,
     &   3.72440718E-04, 3.83392316E-04, 3.80632485E-04, 4.13374063E-04,
     &   4.02477166E-04, 3.82376208E-04, 3.60068016E-04, 3.51873922E-04,
     &   3.43826232E-04, 3.39466361E-04, 3.38749829E-04, 3.33086173E-04,
     &   3.29046352E-04, 3.28806966E-04, 3.29583648E-04, 3.28273561E-04,
     &   3.27831910E-04, 3.25552433E-04, 3.24097922E-04, 3.22328748E-04,
     &   3.21112176E-04, 3.24362518E-04, 3.24126286E-04, 3.28462274E-04,
     &   3.27907554E-04, 3.67797520E-04, 3.94263190E-04, 3.89625161E-04,
     &   3.70551665E-04, 3.60611642E-04, 3.32887151E-04, 3.21864125E-04,
     &   3.53882777E-04, 3.99656420E-04, 3.99339355E-04, 3.26737317E-04,
     &   3.25633032E-04, 3.21695006E-04, 3.26780089E-04, 3.13205714E-04,
     &   3.09993824E-04, 3.12583971E-04, 3.09724030E-04, 3.02545967E-04,
     &   3.01351506E-04, 3.00646071E-04, 3.05562763E-04, 3.00892476E-04,
     &   2.94873245E-04, 2.94354027E-04, 3.15645983E-04, 3.21940127E-04,
     &   3.20153860E-04, 3.26783189E-04, 2.89300519E-04, 3.01892990E-04,
     &   3.26694186E-04, 3.19580545E-04, 3.17958338E-04, 3.23452827E-04,
     &   3.27136314E-04, 3.34518567E-04, 3.30853275E-04, 2.98888181E-04,
     &   3.33414076E-04, 4.59615830E-04, 3.91249406E-04, 3.04867128E-04,
     &   2.98980942E-04, 2.93408631E-04, 2.86430748E-04, 2.78794861E-04/

         DATA ((DGS  (I,J), I=1201,1300), J=2,2) /
     &   2.81464446E-04, 2.83122647E-04, 2.91672949E-04, 2.87775978E-04,
     &   2.88983708E-04, 2.94683335E-04, 2.88406897E-04, 2.99626323E-04,
     &   3.21354468E-04, 3.82213150E-04, 3.67153450E-04, 3.64949461E-04,
     &   4.07030836E-04, 3.30527919E-04, 3.17786699E-04, 3.25123372E-04,
     &   3.78095635E-04, 3.75115032E-04, 4.33320540E-04, 4.37446827E-04,
     &   5.48098597E-04, 6.88164772E-04, 9.21032451E-04, 1.16845436E-03,
     &   7.83602371E-04, 4.44027087E-04, 3.30835326E-04, 2.57176643E-04,
     &   2.00927517E-04, 1.86755347E-04, 1.76500013E-04, 1.62488284E-04,
     &   1.53192605E-04, 1.49578208E-04, 1.46443368E-04, 1.43687730E-04,
     &   1.41237991E-04, 1.39039344E-04, 1.37043188E-04, 1.35230241E-04,
     &   1.33568481E-04, 1.32037395E-04, 1.30620355E-04, 1.29303728E-04,
     &   1.28076222E-04, 1.26928396E-04, 1.25852293E-04, 1.24841153E-04,
     &   1.23889198E-04, 1.22991463E-04, 1.22143652E-04, 1.21342040E-04,
     &   1.20583382E-04, 1.19864845E-04, 1.19183950E-04, 1.18538529E-04,
     &   1.17926686E-04, 1.17346763E-04, 1.16797323E-04, 1.16277120E-04,
     &   1.15785091E-04, 1.15320338E-04, 1.14882119E-04, 1.14469841E-04,
     &   1.14083057E-04, 1.13721458E-04, 1.13384878E-04, 1.13133608E-04,
     &   1.12786827E-04, 1.12290508E-04, 1.11900129E-04, 1.11622934E-04,
     &   1.11469878E-04, 1.11456638E-04, 1.11605126E-04, 1.11945762E-04,
     &   1.12520958E-04, 1.13390608E-04, 1.14641048E-04, 1.16402971E-04,
     &   1.18867141E-04, 1.22357571E-04, 1.27431889E-04, 1.28060500E-04,
     &   1.35157451E-04, 1.47873157E-04, 1.71855598E-04, 2.13159404E-04,
     &   2.30730908E-04, 3.05843665E-04, 4.50668615E-04, 5.18654175E-04,
     &   6.14698853E-04, 6.79087501E-04, 7.59825821E-04, 8.40742554E-04,
     &   9.42025963E-04, 1.00283023E-03, 1.07232736E-03, 1.15250268E-03/

         DATA ((DGS  (I,J), I=1301,1400), J=2,2) /
     &   1.24599519E-03, 1.35639069E-03, 1.48868813E-03, 1.56517888E-03,
     &   1.65007192E-03, 1.74482477E-03, 1.85125261E-03, 1.97164518E-03,
     &   2.10893273E-03, 2.26692674E-03, 2.45067823E-03, 2.66702612E-03,
     &   2.75712791E-03, 2.81365840E-03, 2.87256396E-03, 2.93399721E-03,
     &   2.99812417E-03, 3.06512571E-03, 3.13519923E-03, 3.20856069E-03,
     &   3.28544671E-03, 3.36611722E-03, 3.45085833E-03, 3.53998573E-03,
     &   3.63384865E-03, 3.73283436E-03, 3.83737356E-03, 3.94794663E-03,
     &   4.06509099E-03, 4.18940982E-03, 4.32158246E-03, 4.46237680E-03,
     &   4.61266423E-03, 4.77343762E-03, 4.94583332E-03, 5.13115801E-03,
     &   5.33092187E-03, 5.54687979E-03, 5.78108305E-03, 6.03594460E-03,
     &   6.31432252E-03, 6.61962753E-03, 6.95596344E-03, 7.32831254E-03,
     &   7.74278397E-03, 8.20695138E-03, 8.73031975E-03, 9.32498280E-03,
     &   1.00065687E-02, 1.07956331E-02, 1.17197664E-02, 1.28168854E-02,
     &   1.41405610E-02, 1.57690260E-02, 1.78212073E-02, 2.04871310E-02,
     &   2.40903624E-02, 2.92302124E-02, 3.71545858E-02, 5.09602858E-02,
     &   8.10022111E-02, 9.96285633E-02, 1.89762125E-01, 2.64040692E-01,
     &   3.17881308E-01, 3.59540826E-01, 3.93769213E-01, 4.23722809E-01,
     &   4.51515572E-01, 4.70863822E-01, 4.90558849E-01, 5.09928734E-01,
     &   5.19267864E-01, 5.32356471E-01, 5.40199013E-01, 5.49933681E-01,
     &   5.52470584E-01, 5.56222992E-01, 5.57782416E-01, 5.56607130E-01,
     &   5.54813824E-01, 5.52106430E-01, 5.43532617E-01, 5.37520160E-01,
     &   5.30130560E-01, 5.11091171E-01, 4.99270914E-01, 4.85761690E-01,
     &   4.70305158E-01, 4.34062641E-01, 4.12777877E-01, 3.89703279E-01,
     &   3.37252793E-01, 3.08218892E-01, 2.44325413E-01, 1.73984694E-01,
     &   1.85756345E-02, 1.71674412E-01, 2.42195618E-01, 3.05994303E-01/

         DATA ((DGS  (I,J), I=1401,1500), J=2,2) /
     &   3.62683725E-01, 3.88169626E-01, 4.11371679E-01, 4.32571474E-01,
     &   4.69270490E-01, 4.84691106E-01, 4.98463302E-01, 5.10388063E-01,
     &   5.29618848E-01, 5.37024083E-01, 5.43186613E-01, 5.51866720E-01,
     &   5.54669586E-01, 5.57532486E-01, 5.57790210E-01, 5.56308770E-01,
     &   5.52642738E-01, 5.47215930E-01, 5.40433013E-01, 5.32618767E-01,
     &   5.19558754E-01, 5.10281226E-01, 4.90872816E-01, 4.71228906E-01,
     &   4.51822003E-01, 4.24011259E-01, 3.94032451E-01, 3.59771588E-01,
     &   3.18071005E-01, 2.64179369E-01, 1.88745637E-01, 1.03015498E-01,
     &   5.89225478E-02, 4.12257154E-02, 3.16977501E-02, 2.57456674E-02,
     &   2.16751335E-02, 1.87159666E-02, 1.64677843E-02, 1.47018779E-02,
     &   1.32781207E-02, 1.21058726E-02, 1.11239084E-02, 1.02893822E-02,
     &   9.57141253E-03, 8.94717884E-03, 8.39945272E-03, 7.91498386E-03,
     &   7.48341458E-03, 7.09653136E-03, 6.74773734E-03, 6.43167323E-03,
     &   6.14394008E-03, 5.88089345E-03, 5.63948825E-03, 5.41716043E-03,
     &   5.21173559E-03, 5.02135770E-03, 4.84443284E-03, 4.67958455E-03,
     &   4.52561796E-03, 4.38149078E-03, 4.24628974E-03, 4.11921118E-03,
     &   3.99954513E-03, 3.88666199E-03, 3.78000154E-03, 3.67906356E-03,
     &   3.58340009E-03, 3.49260871E-03, 3.40632692E-03, 3.32422734E-03,
     &   3.24601351E-03, 3.17141636E-03, 3.10019107E-03, 3.03211448E-03,
     &   2.96698267E-03, 2.90460896E-03, 2.84482212E-03, 2.78746482E-03,
     &   2.73239224E-03, 2.31696633E-03, 2.06223718E-03, 1.81788894E-03,
     &   1.54487981E-03, 1.28919778E-03, 1.03643094E-03, 8.94283407E-04,
     &   7.07676114E-04, 6.44210236E-04, 5.93715859E-04, 5.54606031E-04,
     &   5.20758839E-04, 4.95501556E-04, 4.61545253E-04, 4.27789376E-04,
     &   3.90838360E-04, 4.05004343E-04, 4.49314072E-04, 6.07363509E-04/

         DATA ((DGS  (I,J), I=1501,1600), J=2,2) /
     &   9.12949568E-04, 1.34720767E-03, 1.93907649E-03, 2.76364006E-03,
     &   4.84119908E-03, 5.23576841E-03, 5.70056271E-03, 5.83527282E-03,
     &   6.14235207E-03, 6.48358717E-03, 6.86500245E-03, 7.29412786E-03,
     &   7.78050509E-03, 8.33640078E-03, 8.97785459E-03, 9.72626039E-03,
     &   1.06107803E-02, 1.16722520E-02, 1.29696525E-02, 1.45914556E-02,
     &   1.66766797E-02, 1.94570219E-02, 2.33494903E-02, 2.91880859E-02,
     &   3.89184078E-02, 5.83748799E-02, 8.44743046E-02, 8.88550577E-02,
     &   9.37147129E-02, 9.91362895E-02, 1.05223159E-01, 1.12105682E-01,
     &   1.19950625E-01, 1.28974808E-01, 1.39465382E-01, 1.51810814E-01,
     &   1.66549813E-01, 1.84451562E-01, 2.06653349E-01, 2.34909801E-01,
     &   2.72075171E-01, 3.23117642E-01, 3.97494972E-01, 5.15580727E-01,
     &   7.29878116E-01, 1.21467782E+00, 1.38462595E+00, 1.51838803E+00,
     &   1.66747753E+00, 1.79259941E+00, 1.88283477E+00, 1.95936493E+00,
     &   1.99462702E+00, 2.00862712E+00, 1.99101841E+00, 1.92862291E+00,
     &   1.85678391E+00, 1.84319016E+00, 1.83612305E+00, 1.82143857E+00,
     &   1.81535660E+00, 1.81071614E+00, 1.80600752E+00, 1.75508478E+00,
     &   1.66501166E+00, 1.54448626E+00, 1.41678744E+00, 1.25321823E+00,
     &   1.06736030E+00, 8.82339687E-01, 7.73006743E-01, 6.59543137E-01,
     &   5.42461837E-01, 3.73578440E-01, 2.50269504E-01, 1.25508065E-01,
     &   5.02526775E-02, 2.51366438E-02, 1.25769937E-02, 2.52388499E-04,
     &   1.25513640E-02, 2.51110120E-02, 1.25482486E-01, 2.50244093E-01,
     &   3.73553307E-01, 9.24792045E-01, 1.12536357E+00, 1.44674566E+00,
     &   1.57035254E+00, 1.68648356E+00, 1.78141693E+00, 1.85006723E+00,
     &   1.90942734E+00, 1.94923288E+00, 1.98018797E+00, 1.99765965E+00,
     &   2.00720837E+00, 2.00825750E+00, 2.00241723E+00, 1.99191399E+00/

         DATA ((DGS  (I,J), I=1601,1700), J=2,2) /
     &   1.95719681E+00, 1.93379776E+00, 1.90785506E+00, 1.88284219E+00,
     &   1.82653424E+00, 1.79571738E+00, 1.74566671E+00, 1.66748904E+00,
     &   1.38464052E+00, 1.16552632E+00, 9.99612865E-01, 8.72156342E-01,
     &   7.72112333E-01, 6.91901576E-01, 6.11812224E-01, 4.86767638E-01,
     &   4.52929094E-01, 4.23437244E-01, 3.97513879E-01, 3.58988814E-01,
     &   2.97137816E-01, 2.53393170E-01, 2.20841466E-01, 1.95683461E-01,
     &   1.75661447E-01, 1.59350630E-01, 1.45807958E-01, 1.34384608E-01,
     &   1.24619644E-01, 1.16176658E-01, 1.08804402E-01, 1.02311465E-01,
     &   9.65494658E-02, 9.14016169E-02, 8.67747400E-02, 5.83940954E-02,
     &   3.89376233E-02, 2.92072956E-02, 1.94762144E-02, 1.46106747E-02,
     &   1.16915172E-02, 9.74559205E-03, 8.35577296E-03, 7.31354417E-03,
     &   6.50305651E-03, 5.85480570E-03, 5.48714739E-03, 5.05818380E-03,
     &   4.69175620E-03, 4.37514360E-03, 4.09886607E-03, 3.85571489E-03,
     &   2.97631429E-03, 2.42739871E-03, 2.05359144E-03, 1.78373170E-03,
     &   1.58159886E-03, 1.42778271E-03, 1.29883948E-03, 1.19311723E-03,
     &   5.74598243E-04, 5.51356273E-04, 5.76759328E-04, 5.91034889E-04,
     &   6.26625010E-04, 6.89162573E-04, 7.26235544E-04, 7.79290384E-04,
     &   9.16598646E-04, 1.14695518E-03, 1.57086541E-03, 2.57529279E-03,
     &   3.54874740E-03, 3.80093223E-03, 4.09205753E-03, 4.43184199E-03,
     &   4.83355043E-03, 5.31576943E-03, 5.90531805E-03, 6.64245050E-03,
     &   7.59042149E-03, 8.85456830E-03, 1.06247388E-02, 1.32804487E-02,
     &   1.77070575E-02, 1.84769619E-02, 1.93167690E-02, 2.02366598E-02,
     &   2.12485449E-02, 2.23668205E-02, 2.36094803E-02, 2.49983355E-02,
     &   2.65606129E-02, 2.83313648E-02, 3.03550612E-02, 3.26897866E-02,
     &   3.54138476E-02, 3.86330917E-02, 4.24955516E-02, 4.72165042E-02/

         DATA ((DGS  (I,J), I=1701,1800), J=2,2) /
     &   4.86916717E-02, 5.31171599E-02, 6.07017585E-02, 7.08133990E-02,
     &   8.49648508E-02, 1.06179071E-01, 1.41486744E-01, 2.11875512E-01,
     &   4.19921676E-01, 4.99232408E-01, 7.04156583E-01, 8.59997281E-01,
     &   9.88403622E-01, 1.10047800E+00, 1.19991071E+00, 1.28911146E+00,
     &   1.36878980E+00, 1.44264344E+00, 1.51238933E+00, 1.57612934E+00,
     &   1.63219614E+00, 1.68405859E+00, 1.73081864E+00, 1.77177482E+00,
     &   1.81068637E+00, 1.84255969E+00, 1.86744959E+00, 1.88858180E+00,
     &   1.90484726E+00, 1.91492318E+00, 1.91745814E+00, 1.91317505E+00,
     &   1.90080613E+00, 1.87895065E+00, 1.84609187E+00, 1.80915087E+00,
     &   1.75193662E+00, 1.67915287E+00, 1.58931984E+00, 1.46134013E+00,
     &   1.30722231E+00, 1.09886758E+00, 8.25840950E-01, 1.13146943E-01,
     &   2.91737531E-04, 3.73830191E-01, 8.25827533E-01, 1.09885552E+00,
     &   1.30721167E+00, 1.46133082E+00, 1.58931188E+00, 1.67914607E+00,
     &   1.75193095E+00, 1.80914630E+00, 1.84608817E+00, 1.87894795E+00,
     &   1.90080436E+00, 1.91317416E+00, 1.91745806E+00, 1.91492386E+00,
     &   1.90484875E+00, 1.88858402E+00, 1.86745248E+00, 1.84256319E+00,
     &   1.81069050E+00, 1.77177958E+00, 1.73082396E+00, 1.68406445E+00,
     &   1.63220254E+00, 1.57613623E+00, 1.51239671E+00, 1.44265130E+00,
     &   1.36879810E+00, 1.28912018E+00, 1.19991983E+00, 1.10048752E+00,
     &   9.88413526E-01, 8.60007549E-01, 7.04167205E-01, 4.99243369E-01,
     &   4.27484888E-01, 4.14025943E-01, 3.97331471E-01, 3.51222455E-01,
     &   3.09667411E-01, 2.11875255E-01, 1.41498005E-01, 1.06187451E-01,
     &   8.49742743E-02, 7.08246759E-02, 6.07130354E-02, 5.31277118E-02,
     &   4.72277787E-02, 4.25068240E-02, 3.86439787E-02, 3.54251152E-02,
     &   3.27010513E-02, 3.03660860E-02, 2.83426225E-02, 2.65718667E-02/

         DATA ((DGS  (I,J), I=1801,1900), J=2,2) /
     &   1.77182711E-02, 1.32916064E-02, 1.06358225E-02, 8.86556339E-03,
     &   7.60128821E-03, 6.65322670E-03, 5.91600990E-03, 4.84407410E-03,
     &   4.10242645E-03, 3.81118643E-03, 3.55884620E-03, 2.65299581E-03,
     &   1.99278547E-03, 1.60075007E-03, 1.34191924E-03, 1.15868684E-03,
     &   1.02257182E-03, 9.17799508E-04, 8.34947797E-04, 7.68046411E-04,
     &   7.13159358E-04, 6.67582101E-04, 6.29389927E-04, 5.97207141E-04,
     &   5.70047768E-04, 5.47223907E-04, 5.28304504E-04, 5.13128383E-04,
     &   5.01911030E-04, 4.95591086E-04, 4.90812258E-04, 4.79014802E-04,
     &   4.83837422E-04, 5.73811435E-04, 5.95030963E-04, 6.62121031E-04,
     &   7.22033774E-04, 7.72885851E-04, 8.17732231E-04, 8.55951069E-04,
     &   8.89047524E-04, 9.17209678E-04, 9.40690430E-04, 9.60236886E-04,
     &   9.75817753E-04, 9.87470765E-04, 9.95639199E-04, 1.00043344E-03,
     &   1.00169649E-03, 9.99764947E-04, 9.94601497E-04, 9.85987285E-04,
     &   9.74225160E-04, 9.59534318E-04, 9.40884170E-04, 9.19531561E-04,
     &   8.94169917E-04, 8.65200366E-04, 8.31992174E-04, 7.94420514E-04,
     &   7.52469131E-04, 7.06299905E-04, 6.52617155E-04, 5.92622502E-04,
     &   5.25821342E-04, 4.47989460E-04, 3.60480698E-04, 3.12712864E-04,
     &   5.00969058E-04, 5.95990374E-04, 6.68346583E-04, 7.25778311E-04,
     &   7.74674747E-04, 8.16567054E-04, 8.51246790E-04, 8.81650779E-04,
     &   9.07899237E-04, 9.30210802E-04, 9.48851331E-04, 9.64339426E-04,
     &   9.76572051E-04, 9.86354529E-04, 9.93064539E-04, 9.97290614E-04,
     &   9.98930387E-04, 9.97870497E-04, 9.94255589E-04, 9.88018487E-04,
     &   9.78907720E-04, 9.67136723E-04, 9.52546216E-04, 9.33812918E-04,
     &   9.12789995E-04, 8.89269298E-04, 8.60544639E-04, 8.27709252E-04,
     &   7.90319333E-04, 7.46681996E-04, 6.97377165E-04, 6.38738602E-04/

         DATA ((DGS  (I,J), I=1901,2000), J=2,2) /
     &   5.71948890E-04, 4.85503651E-04, 3.80419664E-04, 3.74787148E-04,
     &   3.39056263E-04, 3.28102656E-04, 3.25478981E-04, 3.23917543E-04,
     &   3.20125732E-04, 3.18327581E-04, 3.18498759E-04, 3.19190592E-04,
     &   3.20093694E-04, 3.20620911E-04, 3.20770390E-04, 3.21130634E-04,
     &   3.21208715E-04, 3.21400183E-04, 3.21376621E-04, 3.20672882E-04,
     &   3.19875766E-04, 3.18662697E-04, 3.17260000E-04, 3.15637783E-04,
     &   3.13722146E-04, 3.11175481E-04, 3.08333578E-04, 3.05174937E-04,
     &   3.01671872E-04, 2.97844428E-04, 2.93718750E-04, 2.89377586E-04,
     &   2.84145159E-04, 2.78610901E-04, 2.73660642E-04, 2.62950036E-04,
     &   2.52212243E-04, 2.43351178E-04, 2.27528033E-04, 2.40197872E-04,
     &   2.49318734E-04, 2.56199238E-04, 2.64505967E-04, 2.71658890E-04,
     &   2.76197924E-04, 2.80856409E-04, 2.85614142E-04, 2.89816749E-04,
     &   2.93843178E-04, 2.97746055E-04, 3.01472826E-04, 3.04995441E-04,
     &   3.08324049E-04, 3.11479276E-04, 3.14414686E-04, 3.17198793E-04,
     &   3.19846485E-04, 3.22354641E-04, 3.24596133E-04, 3.26642231E-04,
     &   3.28651827E-04, 3.30637333E-04, 3.32675735E-04, 3.34664269E-04,
     &   3.36443299E-04, 3.38231851E-04, 3.40391337E-04, 3.43135138E-04,
     &   3.46547781E-04, 3.51017290E-04, 3.57281784E-04, 3.57585089E-04,
     &   3.53104874E-04, 3.47844522E-04, 3.50216917E-04, 3.52024973E-04,
     &   3.53717338E-04, 3.55291833E-04, 3.56693369E-04, 3.57910345E-04,
     &   3.58990189E-04, 3.59934721E-04, 3.60746712E-04, 3.56821111E-04,
     &   3.52864345E-04, 3.49274815E-04, 3.45957017E-04, 3.42776936E-04,
     &   3.39644611E-04, 3.36585641E-04, 3.33598130E-04, 3.30599455E-04,
     &   3.27568248E-04, 3.24455151E-04, 3.21213339E-04, 3.17949513E-04,
     &   3.14645329E-04, 3.11156879E-04, 3.07485196E-04, 3.03959613E-04/

         DATA ((DGS  (I,J), I=2001,2100), J=2,2) /
     &   2.99895290E-04, 2.96217418E-04, 2.90640160E-04, 2.84968045E-04,
     &   2.79532539E-04, 2.74622458E-04, 2.67697482E-04, 2.60143785E-04,
     &   2.53228927E-04, 2.44286470E-04, 2.49071647E-04, 2.54448115E-04,
     &   2.60950883E-04, 2.67070608E-04, 2.71271804E-04, 2.76135165E-04,
     &   2.81327495E-04, 2.86488769E-04, 2.89715281E-04, 2.93392379E-04,
     &   2.96526018E-04, 2.99799322E-04, 3.02936932E-04, 3.05891552E-04,
     &   3.08811436E-04, 3.11699904E-04, 3.14448531E-04, 3.17060031E-04,
     &   3.19678427E-04, 3.22227498E-04, 3.24802777E-04, 3.27400251E-04,
     &   3.29970377E-04, 3.32576655E-04, 3.35334747E-04, 3.38301505E-04,
     &   3.41312745E-04, 3.42250456E-04, 3.43542777E-04, 3.45352130E-04,
     &   3.47938280E-04, 3.51599775E-04, 3.55109900E-04, 3.53183112E-04,
     &   3.51149167E-04, 3.40768043E-04, 3.43773222E-04, 3.47135805E-04,
     &   3.49543980E-04, 3.54580051E-04, 3.61373595E-04, 3.70139819E-04,
     &   3.79885119E-04, 3.90315016E-04, 4.01188067E-04, 4.11410897E-04,
     &   4.22656285E-04, 4.33354685E-04, 2.92402865E-04, 1.51451045E-04,
     &   1.46874237E-04, 1.43011760E-04, 1.39951543E-04, 1.37243927E-04,
     &   1.35014591E-04, 1.32978878E-04, 1.31257835E-04, 1.29650871E-04,
     &   1.28206344E-04, 1.26951900E-04, 1.25753226E-04, 1.24699300E-04,
     &   1.23681219E-04, 1.22777409E-04, 1.21896836E-04, 1.21109087E-04,
     &   1.20336302E-04, 1.19640674E-04, 1.18954418E-04, 1.18333514E-04,
     &   1.17744196E-04, 1.17158933E-04, 1.16626156E-04, 1.16095125E-04,
     &   1.15610091E-04, 1.15125144E-04, 1.14680922E-04, 1.14235581E-04,
     &   1.13826611E-04, 1.13415647E-04, 1.13037411E-04, 1.12656540E-04,
     &   1.12289640E-04, 1.11950988E-04, 1.11609050E-04, 1.11292916E-04,
     &   1.10973214E-04, 1.10677198E-04, 1.10377417E-04, 1.10099471E-04/

         DATA ((DGS  (I,J), I=2101,2174), J=2,2) /
     &   1.09817628E-04, 1.09555990E-04, 1.09290373E-04, 1.09043520E-04,
     &   1.08803402E-04, 1.08559246E-04, 1.08331996E-04, 1.08100704E-04,
     &   1.07885228E-04, 1.07665725E-04, 1.07461057E-04, 1.07252393E-04,
     &   1.07057677E-04, 1.06859006E-04, 1.06673477E-04, 1.06484043E-04,
     &   1.06299066E-04, 1.06126146E-04, 1.05949410E-04, 1.05719200E-04,
     &   1.05549907E-04, 1.05391472E-04, 1.05229365E-04, 1.05077567E-04,
     &   1.04922162E-04, 1.04776559E-04, 1.04627416E-04, 1.04487608E-04,
     &   1.04344327E-04, 1.04209947E-04, 1.04078098E-04, 1.03942878E-04,
     &   1.03815970E-04, 1.03685760E-04, 1.03563500E-04, 1.03438005E-04,
     &   1.03320124E-04, 1.03199074E-04, 1.03085322E-04, 1.02968468E-04,
     &   1.02858617E-04, 1.02745727E-04, 1.01633543E-04, 1.00694873E-04,
     &   1.00557933E-04, 9.63417347E-05, 9.51611198E-05, 9.30602368E-05,
     &   9.22655958E-05, 9.22318950E-05, 9.14453635E-05, 9.14432433E-05,
     &   9.09947941E-05, 9.09796068E-05, 9.09705835E-05, 9.07850845E-05,
     &   9.06722602E-05, 9.06528742E-05, 9.06217214E-05, 9.05767285E-05,
     &   9.05458050E-05, 9.05267208E-05, 9.03541727E-05, 9.03460124E-05,
     &   9.02994391E-05, 9.02910430E-05, 9.02539588E-05, 9.02396039E-05,
     &   9.02283099E-05, 9.02178719E-05, 9.01880659E-05, 9.01461415E-05,
     &   9.01288142E-05, 9.00211507E-05/

         DATA ((DGS  (I,J), I=   1, 100), J=3,3) /
     &   7.96903730E-06, 8.44807944E-06, 8.97579074E-06, 9.55349103E-06,
     &   1.01959459E-05, 1.05229127E-05, 1.14952220E-05, 1.19877971E-05,
     &   1.26635412E-05, 1.33443229E-05, 1.32241745E-05, 1.31462570E-05,
     &   1.31569964E-05, 1.32060256E-05, 1.33482933E-05, 1.35602860E-05,
     &   1.38912880E-05, 1.42666036E-05, 1.46049430E-05, 1.50572605E-05,
     &   1.54097079E-05, 1.58297273E-05, 1.61576180E-05, 1.66208005E-05,
     &   1.70104890E-05, 1.74004252E-05, 1.79146038E-05, 1.82057185E-05,
     &   1.83494097E-05, 1.88083922E-05, 1.92772930E-05, 1.97547031E-05,
     &   2.02502077E-05, 2.07429709E-05, 2.12416060E-05, 2.17475129E-05,
     &   2.22601924E-05, 2.27859613E-05, 2.33301156E-05, 2.38922495E-05,
     &   2.43232708E-05, 2.49102463E-05, 2.55190616E-05, 2.59911775E-05,
     &   2.66413076E-05, 2.73145203E-05, 2.78253480E-05, 2.85105838E-05,
     &   2.90391683E-05, 2.97565967E-05, 3.35756423E-05, 3.80480075E-05,
     &   4.56050911E-05, 5.78176466E-05, 6.27646738E-05, 6.40845796E-05,
     &   6.85853865E-05, 7.60161887E-05, 8.48989615E-05, 9.03101910E-05,
     &   9.54832318E-05, 9.63281519E-05, 9.48450539E-05, 9.49825313E-05,
     &   9.63395197E-05, 9.80890216E-05, 9.91048100E-05, 9.95885724E-05,
     &   1.00050798E-04, 1.00556667E-04, 1.00630591E-04, 9.88616833E-05,
     &   9.80321838E-05, 9.70927181E-05, 9.43025095E-05, 9.28792734E-05,
     &   8.97100500E-05, 8.79735805E-05, 8.66002478E-05, 8.75016419E-05,
     &   8.53287067E-05, 8.36959561E-05, 8.50583132E-05, 8.81138910E-05,
     &   9.67173481E-05, 1.09401135E-04, 1.17744628E-04, 1.20200339E-04,
     &   1.21730355E-04, 1.21479126E-04, 1.19651466E-04, 1.23548824E-04,
     &   1.26916508E-04, 1.34804695E-04, 1.44113257E-04, 1.57844543E-04,
     &   1.42479493E-04, 1.26883967E-04, 1.05219247E-04, 8.86186048E-05/

         DATA ((DGS  (I,J), I= 101, 200), J=3,3) /
     &   6.78786530E-05, 5.24700000E-05, 4.14447832E-05, 3.75383399E-05,
     &   4.04737794E-05, 4.81185430E-05, 5.56111721E-05, 6.29516665E-05,
     &   7.02047009E-05, 7.66524781E-05, 8.22168391E-05, 8.39870871E-05,
     &   8.58115264E-05, 8.77353163E-05, 8.68943765E-05, 8.35490178E-05,
     &   7.67099614E-05, 5.40122191E-05, 3.92359176E-05, 3.82706729E-05,
     &   3.92689945E-05, 4.05453402E-05, 4.07387181E-05, 4.04287999E-05,
     &   3.95486420E-05, 3.85604132E-05, 3.87546458E-05, 4.13917426E-05,
     &   4.82020383E-05, 5.93211907E-05, 7.36116691E-05, 9.05544297E-05,
     &   1.09370271E-04, 1.28831252E-04, 1.47335647E-04, 1.63491121E-04,
     &   1.73148681E-04, 1.71313184E-04, 1.56652246E-04, 1.41793256E-04,
     &   1.43060523E-04, 1.37535340E-04, 1.32190933E-04, 1.23691766E-04,
     &   1.15714229E-04, 1.07592092E-04, 9.82283377E-05, 8.89902669E-05,
     &   8.25086845E-05, 7.98023050E-05, 7.95900073E-05, 8.19148992E-05,
     &   8.26280207E-05, 7.91139071E-05, 7.55997934E-05, 7.08775004E-05,
     &   6.94600537E-05, 6.79168106E-05, 5.44561919E-05, 5.59728727E-05,
     &   5.94232338E-05, 6.39068536E-05, 6.44817136E-05, 6.36886978E-05,
     &   6.25166228E-05, 6.04617186E-05, 5.78344956E-05, 5.66387414E-05,
     &   5.19765308E-05, 5.13205713E-05, 5.24233169E-05, 5.40601029E-05,
     &   5.68500799E-05, 5.91702606E-05, 6.07737533E-05, 6.18973144E-05,
     &   6.33649835E-05, 6.58216359E-05, 6.86652152E-05, 7.05179241E-05,
     &   7.41960809E-05, 7.59544481E-05, 8.01100513E-05, 8.37961928E-05,
     &   8.92107664E-05, 9.45846865E-05, 1.02172393E-04, 1.09287923E-04,
     &   1.17300510E-04, 1.27009252E-04, 1.32303290E-04, 1.35661569E-04,
     &   1.43395331E-04, 1.47601926E-04, 1.46926271E-04, 1.51173384E-04,
     &   1.58300013E-04, 1.66690370E-04, 1.74791013E-04, 1.88074275E-04/

         DATA ((DGS  (I,J), I= 201, 300), J=3,3) /
     &   2.02356416E-04, 2.12095075E-04, 2.16014888E-04, 2.20161784E-04,
     &   2.24620487E-04, 2.29290339E-04, 2.34245030E-04, 2.39492667E-04,
     &   2.45024043E-04, 2.50816308E-04, 2.56831308E-04, 2.63046118E-04,
     &   2.69440599E-04, 2.76005781E-04, 2.82756147E-04, 2.89764798E-04,
     &   2.97170435E-04, 3.04785762E-04, 3.12732769E-04, 3.21099265E-04,
     &   3.29990266E-04, 3.38648993E-04, 3.49830117E-04, 3.61072593E-04,
     &   3.74726662E-04, 3.87113626E-04, 4.02454366E-04, 4.20018285E-04,
     &   4.41058559E-04, 4.71172849E-04, 5.00450342E-04, 5.43408455E-04,
     &   4.94799590E-04, 4.97823366E-04, 5.11591007E-04, 5.28694174E-04,
     &   5.49041523E-04, 5.71286907E-04, 5.97651137E-04, 6.22851285E-04,
     &   6.52590626E-04, 6.85394435E-04, 7.15470876E-04, 7.61256578E-04,
     &   8.07042280E-04, 8.57561742E-04, 9.14578772E-04, 9.79321534E-04,
     &   1.05330238E-03, 1.13836283E-03, 1.23664254E-03, 1.35071405E-03,
     &   1.48279092E-03, 1.61084525E-03, 1.68076824E-03, 1.72839691E-03,
     &   1.79423190E-03, 1.82604023E-03, 1.80689619E-03, 1.92399682E-03,
     &   1.97084419E-03, 2.01347020E-03, 2.05122171E-03, 2.08118835E-03,
     &   2.10007947E-03, 2.10375379E-03, 2.07069771E-03, 2.01147196E-03,
     &   1.96932271E-03, 1.85444085E-03, 1.69342016E-03, 1.48142020E-03,
     &   1.13585629E-03, 9.07308109E-04, 5.66659812E-04, 2.61425331E-04,
     &   3.38746747E-04, 6.71212125E-04, 1.00495568E-03, 1.30155916E-03,
     &   1.54934220E-03, 1.74589128E-03, 1.89227123E-03, 1.99491119E-03,
     &   2.06041247E-03, 2.09546965E-03, 2.10655823E-03, 2.09923029E-03,
     &   2.07814782E-03, 2.04712061E-03, 1.98893268E-03, 1.97262433E-03,
     &   1.89017892E-03, 1.86973295E-03, 1.82561004E-03, 1.77772677E-03,
     &   1.73057363E-03, 1.68427704E-03, 1.63913415E-03, 1.59535165E-03/

         DATA ((DGS  (I,J), I= 301, 400), J=3,3) /
     &   1.58948481E-03, 1.51236054E-03, 1.47322893E-03, 1.43564434E-03,
     &   1.31580009E-03, 1.21251569E-03, 1.12351153E-03, 1.04650739E-03,
     &   9.79536850E-04, 9.07823588E-04, 8.69469202E-04, 8.23959823E-04,
     &   7.83566182E-04, 7.47584637E-04, 7.15454169E-04, 6.86739262E-04,
     &   6.61125255E-04, 6.38282514E-04, 6.18035235E-04, 6.00255158E-04,
     &   5.82865576E-04, 5.68730070E-04, 5.73803180E-04, 5.54881971E-04,
     &   5.78285682E-04, 7.28900316E-04, 7.03711399E-04, 5.23190335E-04,
     &   4.71531758E-04, 4.38647758E-04, 4.15451613E-04, 3.97429773E-04,
     &   3.82808542E-04, 3.70549048E-04, 3.60022370E-04, 3.50817087E-04,
     &   3.42645005E-04, 3.35294254E-04, 3.28573355E-04, 3.22417143E-04,
     &   3.16660675E-04, 3.11354582E-04, 3.06460801E-04, 3.01584176E-04,
     &   2.97321919E-04, 2.92726009E-04, 2.88710239E-04, 2.84568379E-04,
     &   2.80509460E-04, 2.76565536E-04, 2.72703407E-04, 2.68739674E-04,
     &   2.65338155E-04, 2.61844323E-04, 2.58345792E-04, 2.55035703E-04,
     &   2.45861186E-04, 2.23367873E-04, 1.85791323E-04, 1.62738848E-04,
     &   1.49384770E-04, 1.40795786E-04, 1.36073765E-04, 1.34535084E-04,
     &   1.36572063E-04, 1.36961094E-04, 1.44137392E-04, 1.48833548E-04,
     &   1.50239261E-04, 1.54869973E-04, 1.62012809E-04, 1.62785834E-04,
     &   1.69323798E-04, 1.71297387E-04, 1.70671689E-04, 1.69509536E-04,
     &   1.68034571E-04, 1.69567618E-04, 1.72776348E-04, 1.76285842E-04,
     &   1.78734956E-04, 1.80517788E-04, 1.85870436E-04, 1.92604648E-04,
     &   1.93370991E-04, 2.00881228E-04, 2.10699715E-04, 2.16283126E-04,
     &   2.22154956E-04, 2.35494567E-04, 2.51663564E-04, 2.72312634E-04,
     &   2.79295741E-04, 2.96591421E-04, 3.09615017E-04, 3.23853452E-04,
     &   3.58613394E-04, 4.06851900E-04, 4.12816132E-04, 4.64714835E-04/

         DATA ((DGS  (I,J), I= 401, 500), J=3,3) /
     &   5.44543453E-04, 5.58732275E-04, 5.04213624E-04, 4.62169642E-04,
     &   4.49472981E-04, 4.02179639E-04, 3.89677754E-04, 3.82972034E-04,
     &   3.74791005E-04, 3.60610001E-04, 3.46662400E-04, 3.33836536E-04,
     &   3.32475374E-04, 3.20749593E-04, 3.19488834E-04, 3.13305965E-04,
     &   3.08627976E-04, 2.94112034E-04, 2.91552993E-04, 2.73132862E-04,
     &   2.71574366E-04, 2.69979247E-04, 2.53515496E-04, 2.36615868E-04,
     &   2.33834363E-04, 2.19312733E-04, 2.18164158E-04, 2.12702228E-04,
     &   2.11667933E-04, 2.05700506E-04, 2.02472391E-04, 1.91665122E-04,
     &   1.90406421E-04, 1.77845327E-04, 1.67097536E-04, 1.61509895E-04,
     &   1.60445995E-04, 1.48266885E-04, 1.46658760E-04, 1.39841044E-04,
     &   1.36512177E-04, 1.35562361E-04, 1.35104999E-04, 1.34931214E-04,
     &   1.35415448E-04, 1.36061576E-04, 1.39217592E-04, 1.44106318E-04,
     &   1.47225754E-04, 1.60828258E-04, 1.78509249E-04, 1.82664648E-04,
     &   1.84836525E-04, 1.99048448E-04, 2.06639589E-04, 2.14621966E-04,
     &   2.52782151E-04, 2.58608349E-04, 2.75476562E-04, 2.86098018E-04,
     &   3.00595281E-04, 3.02892348E-04, 3.09556291E-04, 3.13973749E-04,
     &   3.18281075E-04, 3.28413094E-04, 3.40291282E-04, 3.52260025E-04,
     &   3.71593915E-04, 3.93744160E-04, 3.96438155E-04, 4.34683019E-04,
     &   4.64783608E-04, 4.99226235E-04, 5.06213361E-04, 5.50693022E-04,
     &   5.51337169E-04, 5.53302230E-04, 5.58057229E-04, 5.17560392E-04,
     &   5.06876758E-04, 4.54011740E-04, 4.38490237E-04, 4.29592237E-04,
     &   4.12281793E-04, 4.05153133E-04, 4.07093069E-04, 4.10409440E-04,
     &   4.17902602E-04, 4.20165421E-04, 4.29518279E-04, 4.31853727E-04,
     &   5.66538829E-04, 4.41443626E-04, 4.42925823E-04, 4.44868564E-04,
     &   4.68956728E-04, 4.91031855E-04, 4.95641893E-04, 3.76402765E-04/

         DATA ((DGS  (I,J), I= 501, 600), J=3,3) /
     &   3.17121048E-04, 2.91662479E-04, 2.56142440E-04, 2.45361360E-04,
     &   2.37264967E-04, 2.24969762E-04, 2.24754410E-04, 2.24509329E-04,
     &   2.25748689E-04, 2.30352709E-04, 2.38422861E-04, 2.50779108E-04,
     &   2.69579029E-04, 2.98438982E-04, 3.42485931E-04, 4.11884997E-04,
     &   4.30282973E-04, 4.51031326E-04, 4.74589315E-04, 5.01448209E-04,
     &   5.32266570E-04, 5.67907911E-04, 6.09515823E-04, 6.18680079E-04,
     &   6.28157390E-04, 6.37963373E-04, 6.48114699E-04, 6.58629188E-04,
     &   6.69525901E-04, 6.80825157E-04, 6.92549138E-04, 7.04721058E-04,
     &   7.17366281E-04, 7.81104269E-04, 8.30597379E-04, 8.57927640E-04,
     &   8.77229083E-04, 8.97477116E-04, 9.18726979E-04, 9.41053120E-04,
     &   9.64537534E-04, 9.89270775E-04, 1.00213654E-03, 1.01535288E-03,
     &   1.02893406E-03, 1.04289518E-03, 1.05725196E-03, 1.07202147E-03,
     &   1.08722169E-03, 1.10287141E-03, 1.11899048E-03, 1.13559998E-03,
     &   1.15272225E-03, 1.17038099E-03, 1.18860143E-03, 1.20741035E-03,
     &   1.22683630E-03, 1.24690970E-03, 1.26766298E-03, 1.28913105E-03,
     &   1.31135018E-03, 1.33436079E-03, 1.35820509E-03, 1.38292867E-03,
     &   1.40858049E-03, 1.43521320E-03, 1.46288352E-03, 1.49165281E-03,
     &   1.52158731E-03, 1.55275844E-03, 1.58524362E-03, 1.61912686E-03,
     &   1.65449952E-03, 1.69146109E-03, 1.73012011E-03, 1.77059528E-03,
     &   1.81301661E-03, 1.85752682E-03, 1.90428295E-03, 1.95345818E-03,
     &   2.00524396E-03, 2.05985251E-03, 2.08828801E-03, 2.11751966E-03,
     &   2.14758119E-03, 2.17850827E-03, 2.21033863E-03, 2.24311223E-03,
     &   2.27687141E-03, 2.31166110E-03, 2.34752900E-03, 2.38452579E-03,
     &   2.42270539E-03, 2.46212518E-03, 2.50284636E-03, 2.54493418E-03,
     &   2.58845834E-03, 2.63349337E-03, 2.68011904E-03, 2.72842082E-03/

         DATA ((DGS  (I,J), I= 601, 700), J=3,3) /
     &   2.77849045E-03, 2.83042645E-03, 2.88433481E-03, 2.94032973E-03,
     &   2.99853434E-03, 3.05908172E-03, 3.12211578E-03, 3.18779253E-03,
     &   3.25628127E-03, 3.32776607E-03, 3.40244745E-03, 3.48054419E-03,
     &   3.56229557E-03, 3.64796367E-03, 3.73783630E-03, 3.83223013E-03,
     &   3.93149448E-03, 4.03601560E-03, 4.14622165E-03, 4.26258866E-03,
     &   4.43687495E-03, 4.51599084E-03, 4.65428505E-03, 4.80127921E-03,
     &   4.95781983E-03, 5.12486697E-03, 5.30351384E-03, 5.49501077E-03,
     &   5.70079448E-03, 5.92252415E-03, 6.16212618E-03, 6.42185031E-03,
     &   6.70434038E-03, 6.92985749E-03, 7.63244321E-03, 8.49291193E-03,
     &   9.57118102E-03, 1.09618751E-02, 1.28237102E-02, 1.54448122E-02,
     &   1.94083564E-02, 2.61000724E-02, 3.98116986E-02, 8.37125611E-02,
     &   8.85209826E-02, 9.39132735E-02, 1.00002227E-01, 1.06931553E-01,
     &   1.14887188E-01, 1.24113932E-01, 1.34940555E-01, 1.47818819E-01,
     &   1.63386198E-01, 1.82570678E-01, 2.06773846E-01, 2.38207606E-01,
     &   2.80550051E-01, 2.85598375E-01, 3.98636803E-01, 4.81027091E-01,
     &   5.47222932E-01, 6.04793019E-01, 6.49155378E-01, 6.87916849E-01,
     &   7.18981466E-01, 7.49928705E-01, 7.71187623E-01, 7.90871624E-01,
     &   8.04761363E-01, 8.16211889E-01, 8.22646519E-01, 8.27449546E-01,
     &   8.27857968E-01, 8.25440708E-01, 8.18737257E-01, 8.11492409E-01,
     &   7.95821708E-01, 7.81780886E-01, 7.64545912E-01, 7.43845686E-01,
     &   7.19420144E-01, 6.91030122E-01, 6.58468826E-01, 6.21574680E-01,
     &   5.80245078E-01, 5.34450337E-01, 4.84246945E-01, 4.01041410E-01,
     &   3.40723110E-01, 2.44033841E-01, 1.41586645E-01, 1.41488949E-01,
     &   4.00955861E-01, 4.84168271E-01, 5.57833481E-01, 6.01402440E-01,
     &   6.58411572E-01, 6.90978481E-01, 7.19374085E-01, 7.43805127E-01/

         DATA ((DGS  (I,J), I= 701, 800), J=3,3) /
     &   7.64510733E-01, 7.81750932E-01, 7.95796800E-01, 8.06922306E-01,
     &   8.15397819E-01, 8.21485149E-01, 8.25433940E-01, 8.27857122E-01,
     &   8.27452385E-01, 8.24298971E-01, 8.18601619E-01, 8.13640916E-01,
     &   8.01506718E-01, 7.90896239E-01, 7.75309517E-01, 7.58597593E-01,
     &   7.36793843E-01, 7.14566137E-01, 6.83570673E-01, 6.53414576E-01,
     &   6.16571088E-01, 5.75017602E-01, 5.21569890E-01, 4.60299390E-01,
     &   3.83793750E-01, 3.16256661E-01, 2.63795503E-01, 2.25941253E-01,
     &   1.97438731E-01, 1.75245410E-01, 1.57494905E-01, 1.42984635E-01,
     &   1.30907145E-01, 1.20701171E-01, 1.11964945E-01, 1.04403550E-01,
     &   9.77957416E-02, 9.19723769E-02, 8.68019702E-02, 8.21807943E-02,
     &   7.80259389E-02, 7.42703487E-02, 7.08592134E-02, 3.71568318E-02,
     &   2.51836581E-02, 1.90504182E-02, 1.53225503E-02, 1.28171508E-02,
     &   1.10176797E-02, 9.66267231E-03, 8.60559450E-03, 7.75794731E-03,
     &   7.06312468E-03, 6.98083618E-03, 6.40708004E-03, 5.92109466E-03,
     &   5.50418575E-03, 5.14261613E-03, 4.82606482E-03, 4.54662730E-03,
     &   4.29814738E-03, 4.07575917E-03, 3.87556586E-03, 3.69440995E-03,
     &   3.52970580E-03, 3.37931584E-03, 3.24145750E-03, 3.11463254E-03,
     &   2.99757230E-03, 2.88919560E-03, 2.78857547E-03, 2.69491223E-03,
     &   2.60751208E-03, 2.52577146E-03, 2.44916142E-03, 2.37721726E-03,
     &   2.30952848E-03, 2.24573168E-03, 2.18550369E-03, 2.12855619E-03,
     &   2.07463118E-03, 2.02349703E-03, 1.97494528E-03, 1.92878776E-03,
     &   1.88485420E-03, 1.84299015E-03, 1.80305519E-03, 1.76492138E-03,
     &   1.72847188E-03, 1.69359904E-03, 1.66020722E-03, 1.62820410E-03,
     &   1.59750769E-03, 1.56804124E-03, 1.53973541E-03, 1.51252457E-03,
     &   1.48634850E-03, 1.46115136E-03, 1.43688133E-03, 1.41349024E-03/

         DATA ((DGS  (I,J), I= 801, 900), J=3,3) /
     &   1.39093321E-03, 1.36916854E-03, 1.34815732E-03, 1.32786355E-03,
     &   1.30825264E-03, 1.28929308E-03, 1.27095513E-03, 1.25321093E-03,
     &   1.23603436E-03, 1.21940091E-03, 1.20328758E-03, 1.18767277E-03,
     &   1.17253493E-03, 1.15785883E-03, 1.14362275E-03, 1.12981113E-03,
     &   1.11640820E-03, 1.10339916E-03, 1.09077012E-03, 1.07850811E-03,
     &   1.06659884E-03, 1.05503524E-03, 1.04380481E-03, 1.03289790E-03,
     &   1.02230570E-03, 1.01202024E-03, 1.00203439E-03, 9.92341975E-04,
     &   9.82937803E-04, 9.73817784E-04, 9.64979079E-04, 9.56420312E-04,
     &   9.48141860E-04, 9.40146265E-04, 9.32438829E-04, 9.25028481E-04,
     &   9.17929130E-04, 9.10005393E-04, 9.02260654E-04, 8.94689390E-04,
     &   8.87286307E-04, 8.80046323E-04, 8.72964563E-04, 8.66035743E-04,
     &   8.59257175E-04, 8.53279787E-04, 8.45639720E-04, 8.39279511E-04,
     &   8.33052149E-04, 8.26954413E-04, 8.20981753E-04, 8.15131240E-04,
     &   8.09399565E-04, 8.03782099E-04, 7.98280097E-04, 7.92886265E-04,
     &   6.23154366E-04, 5.28022021E-04, 4.91849922E-04, 5.47429722E-04,
     &   6.49944618E-04, 7.15091531E-04, 8.01656570E-04, 9.19370264E-04,
     &   1.03806497E-03, 1.19676591E-03, 1.26241749E-03, 1.41359539E-03,
     &   1.50449643E-03, 1.60859370E-03, 1.72720229E-03, 1.86664610E-03,
     &   2.03193288E-03, 2.23037209E-03, 2.47258696E-03, 2.77444652E-03,
     &   3.16065982E-03, 3.34706463E-03, 3.55681965E-03, 3.79457784E-03,
     &   4.06631751E-03, 4.37985018E-03, 4.74558127E-03, 4.95225866E-03,
     &   5.17768432E-03, 5.42452439E-03, 5.69597575E-03, 5.99590497E-03,
     &   6.32903331E-03, 6.70118673E-03, 7.11963898E-03, 7.59359105E-03,
     &   8.13485473E-03, 8.75884915E-03, 9.48609132E-03, 1.03071952E-02,
     &   1.08447526E-02, 1.14412509E-02, 1.21069391E-02, 1.28545879E-02/

         DATA ((DGS  (I,J), I= 901,1000), J=3,3) /
     &   1.37003158E-02, 1.46647627E-02, 1.57747936E-02, 1.70660365E-02,
     &   1.85867693E-02, 2.04040770E-02, 2.26139885E-02, 2.53589536E-02,
     &   2.88597019E-02, 3.34774525E-02, 3.98464562E-02, 4.91906672E-02,
     &   6.42098144E-02, 9.22114272E-02, 1.36252554E-01, 1.61318989E-01,
     &   1.88660127E-01, 2.55932090E-01, 2.80168571E-01, 3.00171469E-01,
     &   3.17158567E-01, 3.31462671E-01, 3.43198453E-01, 3.53303584E-01,
     &   3.61577720E-01, 3.67499528E-01, 3.72530355E-01, 3.76091733E-01,
     &   3.78204055E-01, 3.78966962E-01, 3.78425197E-01, 3.76944833E-01,
     &   3.73940466E-01, 3.70009106E-01, 3.64709848E-01, 3.57891689E-01,
     &   3.50722343E-01, 3.40677603E-01, 3.30522157E-01, 3.18849925E-01,
     &   3.05585997E-01, 2.88019148E-01, 2.71121054E-01, 2.49235111E-01,
     &   2.28628226E-01, 1.98584380E-01, 1.65796191E-01, 1.16824922E-01,
     &   7.41382197E-02, 2.49700771E-02, 3.28658785E-04, 2.48453794E-02,
     &   4.95894257E-02, 1.16706716E-01, 1.65685184E-01, 1.98479869E-01,
     &   2.28531037E-01, 2.49143829E-01, 2.71036984E-01, 2.87941442E-01,
     &   3.05515823E-01, 3.18786225E-01, 3.30464888E-01, 3.40626674E-01,
     &   3.50678645E-01, 3.57854011E-01, 3.64678953E-01, 3.69984711E-01,
     &   3.73922272E-01, 3.76933348E-01, 3.78419250E-01, 3.78967687E-01,
     &   3.78210960E-01, 3.76104958E-01, 3.72549911E-01, 3.67525306E-01,
     &   3.61609077E-01, 3.53341111E-01, 3.43242036E-01, 3.31512027E-01,
     &   3.17213761E-01, 3.00232422E-01, 2.80235149E-01, 2.56004270E-01,
     &   2.26628773E-01, 2.42576986E-01, 2.07510564E-01, 1.85911802E-01,
     &   1.72853904E-01, 1.45803939E-01, 1.36341032E-01, 1.15970535E-01,
     &   7.50914335E-02, 5.54052171E-02, 4.38730415E-02, 3.63081865E-02,
     &   3.09668952E-02, 2.69955503E-02, 2.39275015E-02, 2.14862741E-02/

         DATA ((DGS  (I,J), I=1001,1100), J=3,3) /
     &   1.94977081E-02, 1.78466613E-02, 1.64539735E-02, 1.52634350E-02,
     &   1.42340341E-02, 1.33351463E-02, 1.25434452E-02, 1.18408475E-02,
     &   1.12131219E-02, 1.06489121E-02, 9.75703915E-03, 8.64378073E-03,
     &   8.04837813E-03, 7.07546679E-03, 6.31421324E-03, 5.56783742E-03,
     &   4.50808282E-03, 3.34226403E-03, 2.65270247E-03, 2.36580901E-03,
     &   2.20506723E-03, 2.10082350E-03, 2.02706021E-03, 1.97160287E-03,
     &   1.92806328E-03, 1.89271287E-03, 1.86321846E-03, 1.83800693E-03,
     &   1.81609616E-03, 1.79667824E-03, 1.77920968E-03, 1.76331400E-03,
     &   1.74863879E-03, 1.73493463E-03, 1.72199173E-03, 1.70967616E-03,
     &   1.69780383E-03, 1.68627845E-03, 1.67498885E-03, 1.66384872E-03,
     &   1.65273073E-03, 1.64157160E-03, 1.63024176E-03, 1.61867638E-03,
     &   1.60671689E-03, 1.59422188E-03, 1.58094687E-03, 1.56663850E-03,
     &   1.55078314E-03, 1.53244485E-03, 1.50929299E-03, 1.45719631E-03,
     &   1.41274332E-03, 1.39760539E-03, 1.38761320E-03, 1.38040394E-03,
     &   1.37501250E-03, 1.37093906E-03, 1.36784226E-03, 1.36548012E-03,
     &   1.36368120E-03, 1.36222174E-03, 1.36107238E-03, 1.36001656E-03,
     &   1.35892398E-03, 1.35769818E-03, 1.35629196E-03, 1.35483050E-03,
     &   1.35291577E-03, 1.35030683E-03, 1.34690598E-03, 1.34258221E-03,
     &   1.33717811E-03, 1.33051135E-03, 1.32239128E-03, 1.31260012E-03,
     &   1.30084645E-03, 1.28651873E-03, 1.27017726E-03, 1.25032388E-03,
     &   1.22665819E-03, 1.19824670E-03, 1.16363500E-03, 1.12048176E-03,
     &   1.06428740E-03, 9.85424776E-04, 9.61937570E-04, 8.98633214E-04,
     &   8.06794502E-04, 7.97836690E-04, 7.38228755E-04, 7.17183845E-04,
     &   7.00878617E-04, 6.83641686E-04, 6.67323130E-04, 6.50906741E-04,
     &   6.40311027E-04, 6.35099938E-04, 6.20468935E-04, 6.06826465E-04/

         DATA ((DGS  (I,J), I=1101,1200), J=3,3) /
     &   5.94047098E-04, 5.82116938E-04, 5.71288945E-04, 5.60646075E-04,
     &   5.50431957E-04, 5.43612056E-04, 5.33839215E-04, 5.22245567E-04,
     &   5.10651919E-04, 4.99058271E-04, 4.93180655E-04, 4.86530855E-04,
     &   4.78614675E-04, 4.72463089E-04, 4.71439498E-04, 4.68563512E-04,
     &   4.58774896E-04, 4.49471314E-04, 4.46982303E-04, 4.46873612E-04,
     &   4.46006075E-04, 4.43415091E-04, 4.38592858E-04, 4.38961367E-04,
     &   4.45120324E-04, 4.59303543E-04, 4.69671378E-04, 4.50176110E-04,
     &   4.05919008E-04, 3.99760735E-04, 3.96481248E-04, 3.95731040E-04,
     &   3.90624130E-04, 3.87195244E-04, 3.83264013E-04, 3.80086591E-04,
     &   3.77604154E-04, 3.75017907E-04, 3.74189767E-04, 3.72053041E-04,
     &   3.69095533E-04, 3.67533252E-04, 3.65566266E-04, 3.63957986E-04,
     &   3.63801597E-04, 3.63038490E-04, 3.61566241E-04, 3.57595520E-04,
     &   3.54101266E-04, 3.52765608E-04, 3.51660582E-04, 3.50557864E-04,
     &   3.52178884E-04, 3.57326213E-04, 3.57444377E-04, 3.53539383E-04,
     &   3.50247481E-04, 3.46301250E-04, 3.39555815E-04, 3.33617528E-04,
     &   3.30274234E-04, 3.30381615E-04, 3.28464479E-04, 3.25373647E-04,
     &   3.24907688E-04, 3.22405727E-04, 3.22423966E-04, 3.18181006E-04,
     &   3.16076026E-04, 3.15834917E-04, 3.15451187E-04, 3.12580186E-04,
     &   3.11163304E-04, 3.08922101E-04, 3.06294731E-04, 3.04519319E-04,
     &   3.02789228E-04, 2.99852088E-04, 2.99942810E-04, 3.01591797E-04,
     &   3.08294072E-04, 3.11873023E-04, 2.90082061E-04, 2.89428287E-04,
     &   2.88255253E-04, 2.85146656E-04, 2.85699829E-04, 2.91921024E-04,
     &   2.89581636E-04, 2.87678821E-04, 2.86286462E-04, 2.76371184E-04,
     &   2.80776836E-04, 3.07339136E-04, 3.00214544E-04, 2.80883298E-04,
     &   2.76730062E-04, 2.75201742E-04, 2.72871631E-04, 2.72084285E-04/

         DATA ((DGS  (I,J), I=1201,1300), J=3,3) /
     &   2.71055370E-04, 2.69491691E-04, 2.67667492E-04, 2.66786362E-04,
     &   2.65303125E-04, 2.64963306E-04, 2.63015600E-04, 2.61115016E-04,
     &   2.59265164E-04, 2.57246290E-04, 2.56721205E-04, 2.55149478E-04,
     &   2.57989092E-04, 2.50986247E-04, 2.43870188E-04, 2.41047275E-04,
     &   2.37797510E-04, 2.37175136E-04, 2.34386066E-04, 2.32703417E-04,
     &   2.24239000E-04, 2.23903866E-04, 2.16772716E-04, 2.09195869E-04,
     &   2.98972035E-04, 3.78186300E-04, 4.04591055E-04, 3.41578433E-04,
     &   2.87788635E-04, 2.72906387E-04, 2.61654178E-04, 2.45517488E-04,
     &   2.34304456E-04, 2.29846035E-04, 2.25943427E-04, 2.22492193E-04,
     &   2.19413929E-04, 2.16648513E-04, 2.14141982E-04, 2.11870732E-04,
     &   2.09798148E-04, 2.07899815E-04, 2.06155643E-04, 2.04548940E-04,
     &   2.03065730E-04, 2.01694227E-04, 2.00424429E-04, 1.99247809E-04,
     &   1.98157067E-04, 1.97145930E-04, 1.96209004E-04, 1.95341640E-04,
     &   1.94539839E-04, 1.93800164E-04, 1.93119675E-04, 1.92495873E-04,
     &   1.91926658E-04, 1.91410286E-04, 1.90945346E-04, 1.90530736E-04,
     &   1.90165639E-04, 1.89849518E-04, 1.89582098E-04, 1.89363369E-04,
     &   1.89193577E-04, 1.89073229E-04, 1.89003100E-04, 1.88983847E-04,
     &   1.89017971E-04, 1.89250120E-04, 1.89716707E-04, 1.90441068E-04,
     &   1.91454092E-04, 1.92796222E-04, 1.94520358E-04, 1.96696159E-04,
     &   1.99416563E-04, 2.02808013E-04, 2.07047163E-04, 2.12395465E-04,
     &   2.19224008E-04, 2.28160638E-04, 2.40278465E-04, 2.41730804E-04,
     &   2.57635873E-04, 2.84775606E-04, 3.33995381E-04, 4.16867832E-04,
     &   4.51863454E-04, 6.00840610E-04, 8.87064522E-04, 1.02125064E-03,
     &   1.21072723E-03, 1.33771410E-03, 1.49691631E-03, 1.65644687E-03,
     &   1.85610798E-03, 1.97596301E-03, 2.11294671E-03, 2.27097118E-03/

         DATA ((DGS  (I,J), I=1301,1400), J=3,3) /
     &   2.45523661E-03, 2.67280888E-03, 2.93353865E-03, 3.08428249E-03,
     &   3.25158314E-03, 3.43831271E-03, 3.64804813E-03, 3.88530151E-03,
     &   4.15584708E-03, 4.46719570E-03, 4.82930074E-03, 5.25563889E-03,
     &   5.43319422E-03, 5.54459353E-03, 5.66067306E-03, 5.78173359E-03,
     &   5.90810226E-03, 6.04013547E-03, 6.17822226E-03, 6.32278814E-03,
     &   6.47429944E-03, 6.63326834E-03, 6.80025866E-03, 6.97589251E-03,
     &   7.16085804E-03, 7.35591843E-03, 7.56192240E-03, 7.77981659E-03,
     &   8.01066000E-03, 8.25564128E-03, 8.51609911E-03, 8.79354672E-03,
     &   9.08970122E-03, 9.40651916E-03, 9.74623983E-03, 1.01114381E-02,
     &   1.05050900E-02, 1.09306537E-02, 1.13921713E-02, 1.18943979E-02,
     &   1.24429654E-02, 1.30445951E-02, 1.37073738E-02, 1.44411196E-02,
     &   1.52578711E-02, 1.61725529E-02, 1.72038952E-02, 1.83757300E-02,
     &   1.97188539E-02, 2.12737733E-02, 2.30948581E-02, 2.52568262E-02,
     &   2.78652443E-02, 3.10742764E-02, 3.51182788E-02, 4.03717147E-02,
     &   4.74721982E-02, 5.76007241E-02, 7.32163987E-02, 1.00421747E-01,
     &   1.59622009E-01, 1.96326878E-01, 3.73943013E-01, 5.20315483E-01,
     &   6.26413166E-01, 7.08506922E-01, 7.75956979E-01, 8.34983183E-01,
     &   8.89751274E-01, 9.27878709E-01, 9.66689498E-01, 1.00485957E+00,
     &   1.02326315E+00, 1.04905540E+00, 1.06450982E+00, 1.08369284E+00,
     &   1.08869203E+00, 1.09608648E+00, 1.09915947E+00, 1.09684346E+00,
     &   1.09330959E+00, 1.08797444E+00, 1.07107898E+00, 1.05923090E+00,
     &   1.04466905E+00, 1.00715025E+00, 9.83857390E-01, 9.57236273E-01,
     &   9.26777813E-01, 8.55358734E-01, 8.13415229E-01, 7.67944698E-01,
     &   6.64586387E-01, 6.07372524E-01, 4.81464785E-01, 3.42852194E-01,
     &   3.66049462E-02, 3.38299578E-01, 4.77267837E-01, 6.02988775E-01/

         DATA ((DGS  (I,J), I=1401,1500), J=3,3) /
     &   7.14700283E-01, 7.64922499E-01, 8.10644191E-01, 8.52420258E-01,
     &   9.24738908E-01, 9.55126591E-01, 9.82265920E-01, 1.00576471E+00,
     &   1.04366067E+00, 1.05825334E+00, 1.07039715E+00, 1.08750207E+00,
     &   1.09302536E+00, 1.09866696E+00, 1.09917483E+00, 1.09625552E+00,
     &   1.08903128E+00, 1.07833727E+00, 1.06497094E+00, 1.04957228E+00,
     &   1.02383637E+00, 1.00555418E+00, 9.67308197E-01, 9.28598139E-01,
     &   8.90355125E-01, 8.35551599E-01, 7.76475714E-01, 7.08961659E-01,
     &   6.26786982E-01, 5.20588758E-01, 3.71939934E-01, 2.03001132E-01,
     &   1.16112086E-01, 8.12389191E-02, 6.24632258E-02, 5.07341249E-02,
     &   4.27127819E-02, 3.68814858E-02, 3.24512477E-02, 2.89713769E-02,
     &   2.61657415E-02, 2.38557269E-02, 2.19206837E-02, 2.02761803E-02,
     &   1.88613619E-02, 1.76312587E-02, 1.65519204E-02, 1.55972364E-02,
     &   1.47467957E-02, 1.39844130E-02, 1.32970886E-02, 1.26742615E-02,
     &   1.21072631E-02, 1.15889119E-02, 1.11132072E-02, 1.06750962E-02,
     &   1.02702943E-02, 9.89514379E-03, 9.54650384E-03, 9.22166199E-03,
     &   8.91826364E-03, 8.63425489E-03, 8.36783603E-03, 8.11742340E-03,
     &   7.88161793E-03, 7.65917896E-03, 7.44900250E-03, 7.25010288E-03,
     &   7.06159729E-03, 6.88269276E-03, 6.71267498E-03, 6.55089877E-03,
     &   6.39677997E-03, 6.24978833E-03, 6.10944146E-03, 5.97529956E-03,
     &   5.84696079E-03, 5.72405731E-03, 5.60625169E-03, 5.49323391E-03,
     &   5.38471857E-03, 4.56618664E-03, 4.06431808E-03, 3.58294936E-03,
     &   3.04522436E-03, 2.54185640E-03, 2.04486292E-03, 1.76622263E-03,
     &   1.39638249E-03, 1.26916480E-03, 1.16631931E-03, 1.07614518E-03,
     &   1.00106267E-03, 9.37494953E-04, 8.35816079E-04, 7.58353637E-04,
     &   5.50227192E-04, 4.32021198E-04, 4.42031146E-04, 6.10343264E-04/

         DATA ((DGS  (I,J), I=1501,1600), J=3,3) /
     &   9.32172344E-04, 1.36980412E-03, 1.96174981E-03, 2.78903664E-03,
     &   4.87643333E-03, 5.27320455E-03, 5.74059881E-03, 5.87606610E-03,
     &   6.18487949E-03, 6.52805668E-03, 6.91166187E-03, 7.34327529E-03,
     &   7.83249852E-03, 8.39167641E-03, 9.03695214E-03, 9.78984981E-03,
     &   1.06797202E-02, 1.17476541E-02, 1.30530039E-02, 1.46847967E-02,
     &   1.67829287E-02, 1.95805518E-02, 2.34973158E-02, 2.93724421E-02,
     &   3.91637692E-02, 5.87424135E-02, 8.50058642E-02, 8.94141558E-02,
     &   9.43043613E-02, 9.97600224E-02, 1.05885160E-01, 1.12810956E-01,
     &   1.20705225E-01, 1.29786151E-01, 1.40342689E-01, 1.52765752E-01,
     &   1.67597436E-01, 1.85611761E-01, 2.07953168E-01, 2.36387320E-01,
     &   2.73786421E-01, 3.25149900E-01, 3.99994999E-01, 5.18823418E-01,
     &   7.34468577E-01, 1.22231733E+00, 1.39333431E+00, 1.52793765E+00,
     &   1.67796482E+00, 1.80387364E+00, 1.89467651E+00, 1.97168799E+00,
     &   2.00717186E+00, 2.02126000E+00, 2.00354055E+00, 1.94075263E+00,
     &   1.86846181E+00, 1.85478256E+00, 1.84767101E+00, 1.83289417E+00,
     &   1.82677395E+00, 1.82210430E+00, 1.81736607E+00, 1.76612306E+00,
     &   1.67548345E+00, 1.55420002E+00, 1.42569807E+00, 1.26110012E+00,
     &   1.07407328E+00, 8.87889020E-01, 7.77868451E-01, 6.63691242E-01,
     &   5.45873589E-01, 3.75928052E-01, 2.51843619E-01, 1.26297610E-01,
     &   5.05691960E-02, 2.52956633E-02, 1.26579477E-02, 3.33798072E-04,
     &   1.26321606E-02, 2.52698712E-02, 1.26271870E-01, 2.51818049E-01,
     &   3.75902762E-01, 9.30608372E-01, 1.13244135E+00, 1.45584471E+00,
     &   1.58022898E+00, 1.69709039E+00, 1.79262082E+00, 1.86170289E+00,
     &   1.92143633E+00, 1.96149222E+00, 1.99264199E+00, 2.01022356E+00,
     &   2.01983233E+00, 2.02088806E+00, 2.01501106E+00, 2.00444176E+00/

         DATA ((DGS  (I,J), I=1601,1700), J=3,3) /
     &   1.96950624E+00, 1.94596002E+00, 1.91985416E+00, 1.89468398E+00,
     &   1.83802189E+00, 1.80701122E+00, 1.75664576E+00, 1.67797640E+00,
     &   1.39334897E+00, 1.17285670E+00, 1.00589976E+00, 8.77641628E-01,
     &   7.76968416E-01, 6.96253192E-01, 6.15660138E-01, 4.89829118E-01,
     &   4.55777756E-01, 4.26100426E-01, 4.00014025E-01, 3.61246670E-01,
     &   2.99006686E-01, 2.54986930E-01, 2.22230512E-01, 1.96914294E-01,
     &   1.76766369E-01, 1.60352982E-01, 1.46725150E-01, 1.35229968E-01,
     &   1.25403603E-01, 1.16907530E-01, 1.09488922E-01, 1.02955162E-01,
     &   9.71569375E-02, 9.19767259E-02, 8.73207628E-02, 5.87617550E-02,
     &   3.91831162E-02, 2.93917929E-02, 1.95999105E-02, 1.47041656E-02,
     &   1.17670358E-02, 9.80924700E-03, 8.41109187E-03, 7.36271180E-03,
     &   6.54751702E-03, 5.89555305E-03, 5.52580108E-03, 5.09449922E-03,
     &   4.72610593E-03, 4.40781784E-03, 4.13008528E-03, 3.88564044E-03,
     &   3.00196426E-03, 2.45043198E-03, 2.07408749E-03, 1.80141614E-03,
     &   1.59517124E-03, 1.43405319E-03, 1.30496050E-03, 1.19944476E-03,
     &   5.48547097E-04, 5.46326523E-04, 5.56894167E-04, 5.73084820E-04,
     &   5.96971653E-04, 6.32136941E-04, 6.55792159E-04, 6.85054763E-04,
     &   7.68398896E-04, 9.10007395E-04, 1.18312887E-03, 1.85567926E-03,
     &   2.52242075E-03, 2.69617658E-03, 2.89713036E-03, 3.13208139E-03,
     &   3.41031119E-03, 3.74481519E-03, 4.15434819E-03, 4.66705958E-03,
     &   5.32718734E-03, 6.20842627E-03, 7.44346385E-03, 9.29768383E-03,
     &   1.23901739E-02, 1.29281753E-02, 1.35150557E-02, 1.41579341E-02,
     &   1.48651394E-02, 1.56467378E-02, 1.65153112E-02, 1.74861110E-02,
     &   1.85781757E-02, 1.98160155E-02, 2.12307258E-02, 2.28629224E-02,
     &   2.47673608E-02, 2.70180548E-02, 2.97185165E-02, 3.30192774E-02/

         DATA ((DGS  (I,J), I=1701,1800), J=3,3) /
     &   3.40506882E-02, 3.71449429E-02, 4.24481021E-02, 4.95182901E-02,
     &   5.94133083E-02, 7.42469535E-02, 9.89354193E-02, 1.48154274E-01,
     &   2.93629807E-01, 3.49087629E-01, 4.92380384E-01, 6.01351660E-01,
     &   6.91139530E-01, 7.69507312E-01, 8.39035433E-01, 9.01408885E-01,
     &   9.57123806E-01, 1.00876582E+00, 1.05753549E+00, 1.10210557E+00,
     &   1.14131018E+00, 1.17757489E+00, 1.21027177E+00, 1.23891030E+00,
     &   1.26611913E+00, 1.28840648E+00, 1.30581069E+00, 1.32058735E+00,
     &   1.33196095E+00, 1.33900651E+00, 1.34077908E+00, 1.33778414E+00,
     &   1.32913519E+00, 1.31385279E+00, 1.29087635E+00, 1.26504543E+00,
     &   1.22503847E+00, 1.17414458E+00, 1.11132901E+00, 1.02183945E+00,
     &   9.14072862E-01, 7.68381230E-01, 5.77467917E-01, 7.91191395E-02,
     &   5.05522459E-04, 2.61400467E-01, 5.77458535E-01, 7.68372799E-01,
     &   9.14065421E-01, 1.02183293E+00, 1.11132344E+00, 1.17413983E+00,
     &   1.22503451E+00, 1.26504223E+00, 1.29087376E+00, 1.31385090E+00,
     &   1.32913396E+00, 1.33778351E+00, 1.34077902E+00, 1.33900699E+00,
     &   1.33196199E+00, 1.32058891E+00, 1.30581272E+00, 1.28840893E+00,
     &   1.26612201E+00, 1.23891363E+00, 1.21027549E+00, 1.17757899E+00,
     &   1.14131465E+00, 1.10211039E+00, 1.05754066E+00, 1.00877132E+00,
     &   9.57129610E-01, 9.01414982E-01, 8.39041815E-01, 7.69513971E-01,
     &   6.91146455E-01, 6.01358840E-01, 4.92387812E-01, 3.49095294E-01,
     &   2.98918363E-01, 2.89507232E-01, 2.77833670E-01, 2.45592077E-01,
     &   2.16534849E-01, 1.48154097E-01, 9.89432967E-02, 7.42528186E-02,
     &   5.94199060E-02, 4.95261874E-02, 4.24560037E-02, 3.71523426E-02,
     &   3.30271880E-02, 2.97264320E-02, 2.70257077E-02, 2.47752874E-02,
     &   2.28708551E-02, 2.12384997E-02, 1.98239618E-02, 1.85861295E-02/

         DATA ((DGS  (I,J), I=1801,1900), J=3,3) /
     &   1.23982047E-02, 9.30582176E-03, 7.45173939E-03, 6.21686978E-03,
     &   5.33582905E-03, 4.67592962E-03, 4.16347652E-03, 3.42004531E-03,
     &   2.90758786E-03, 2.70703931E-03, 2.53371741E-03, 1.91647172E-03,
     &   1.47630011E-03, 1.22356037E-03, 1.06382306E-03, 9.56979437E-04,
     &   8.82923067E-04, 8.30326061E-04, 7.92530671E-04, 7.65328041E-04,
     &   7.45638212E-04, 7.31356606E-04, 7.21253254E-04, 7.14450271E-04,
     &   7.10319039E-04, 7.08409883E-04, 7.08404697E-04, 7.10085090E-04,
     &   7.13311187E-04, 7.18008647E-04, 7.24158013E-04, 7.31781503E-04,
     &   7.53690864E-04, 1.16244652E-03, 1.25884771E-03, 1.43600174E-03,
     &   1.58289428E-03, 1.70696599E-03, 1.81281321E-03, 1.90363610E-03,
     &   1.98128676E-03, 2.04680532E-03, 2.10190660E-03, 2.14741358E-03,
     &   2.18358925E-03, 2.21096965E-03, 2.23014539E-03, 2.24151944E-03,
     &   2.24505976E-03, 2.24107697E-03, 2.22968645E-03, 2.21055774E-03,
     &   2.18411484E-03, 2.15088443E-03, 2.10870503E-03, 2.06030123E-03,
     &   2.00267128E-03, 1.93686268E-03, 1.86128262E-03, 1.77567217E-03,
     &   1.67998612E-03, 1.57446564E-03, 1.45159898E-03, 1.31384855E-03,
     &   1.15988462E-03, 9.79290564E-04, 7.73724086E-04, 6.60931755E-04,
     &   1.10538837E-03, 1.32491853E-03, 1.49115599E-03, 1.62270771E-03,
     &   1.73452917E-03, 1.83016681E-03, 1.90931817E-03, 1.97862132E-03,
     &   2.03844091E-03, 2.08926480E-03, 2.13166300E-03, 2.16695501E-03,
     &   2.19476813E-03, 2.21696789E-03, 2.23225851E-03, 2.24180703E-03,
     &   2.24535080E-03, 2.24289595E-03, 2.23460580E-03, 2.22013663E-03,
     &   2.19939268E-03, 2.17252501E-03, 2.13903220E-03, 2.09611963E-03,
     &   2.04796218E-03, 1.99408305E-03, 1.92848895E-03, 1.85294112E-03,
     &   1.76591211E-03, 1.66554722E-03, 1.54866514E-03, 1.41067638E-03/

         DATA ((DGS  (I,J), I=1901,2000), J=3,3) /
     &   1.24411709E-03, 1.03439911E-03, 7.55129475E-04, 7.40160582E-04,
     &   7.10117986E-04, 6.99172589E-04, 6.95480381E-04, 6.95120098E-04,
     &   6.96398438E-04, 6.98454166E-04, 7.00793943E-04, 7.03123035E-04,
     &   7.05240882E-04, 7.07021366E-04, 7.08363651E-04, 7.09200488E-04,
     &   7.09480099E-04, 7.09158256E-04, 7.08200070E-04, 7.06575599E-04,
     &   7.04252193E-04, 7.01213124E-04, 6.97401623E-04, 6.92826782E-04,
     &   6.87433743E-04, 6.81195358E-04, 6.74085075E-04, 6.66038436E-04,
     &   6.57016437E-04, 6.46983304E-04, 6.35858359E-04, 6.23500941E-04,
     &   6.09949637E-04, 5.94923477E-04, 5.78375824E-04, 5.60018919E-04,
     &   5.39466945E-04, 5.16082467E-04, 4.84257852E-04, 4.92743267E-04,
     &   5.05636322E-04, 5.18374755E-04, 5.30574516E-04, 5.42129175E-04,
     &   5.53053535E-04, 5.63273366E-04, 5.72806638E-04, 5.81716596E-04,
     &   5.89943196E-04, 5.97543275E-04, 6.04531493E-04, 6.10890081E-04,
     &   6.16679404E-04, 6.21914538E-04, 6.26531514E-04, 6.30609963E-04,
     &   6.34158753E-04, 6.37136139E-04, 6.39590696E-04, 6.41506962E-04,
     &   6.42910277E-04, 6.43785547E-04, 6.44148455E-04, 6.44000387E-04,
     &   6.43345218E-04, 6.42189519E-04, 6.40535653E-04, 6.38393643E-04,
     &   6.35771662E-04, 6.32682604E-04, 6.29135880E-04, 6.25160501E-04,
     &   6.20761140E-04, 6.15973963E-04, 5.84571530E-04, 5.82322781E-04,
     &   5.80011889E-04, 5.77615443E-04, 5.75127743E-04, 5.72545194E-04,
     &   5.69866459E-04, 5.67067150E-04, 5.64118050E-04, 5.61042926E-04,
     &   5.57811786E-04, 5.54422045E-04, 5.50872779E-04, 5.47164771E-04,
     &   5.43260368E-04, 5.39156728E-04, 5.34897781E-04, 5.30396282E-04,
     &   5.25647112E-04, 5.20699914E-04, 5.15450721E-04, 5.10009470E-04,
     &   5.04267563E-04, 4.98225829E-04, 4.91696446E-04, 4.85004622E-04/

         DATA ((DGS  (I,J), I=2001,2100), J=3,3) /
     &   4.78042196E-04, 4.70622543E-04, 4.62769511E-04, 4.54453406E-04,
     &   4.45668517E-04, 4.36233047E-04, 4.26051645E-04, 4.14876034E-04,
     &   4.02197405E-04, 3.82573816E-04, 3.81059599E-04, 3.85552245E-04,
     &   3.90737518E-04, 3.96086126E-04, 4.01412760E-04, 4.06603593E-04,
     &   4.11666427E-04, 4.16540332E-04, 4.21197825E-04, 4.25590626E-04,
     &   4.29812275E-04, 4.33793431E-04, 4.37555391E-04, 4.41083834E-04,
     &   4.44372708E-04, 4.47480939E-04, 4.50336429E-04, 4.52967699E-04,
     &   4.55420699E-04, 4.57680632E-04, 4.59717950E-04, 4.61474329E-04,
     &   4.62999468E-04, 4.64379351E-04, 4.65556641E-04, 4.66530637E-04,
     &   4.67302607E-04, 4.67879928E-04, 4.68259771E-04, 4.68449894E-04,
     &   4.68453507E-04, 4.68272504E-04, 4.67911286E-04, 4.67365424E-04,
     &   4.66630327E-04, 4.61021272E-04, 4.39718986E-04, 4.29752784E-04,
     &   4.33965566E-04, 4.48750548E-04, 4.69404943E-04, 4.95373032E-04,
     &   5.23011857E-04, 5.54025411E-04, 5.86840525E-04, 6.19887999E-04,
     &   6.56429390E-04, 6.93811788E-04, 6.78603020E-04, 6.63394253E-04,
     &   6.32455746E-04, 6.04440463E-04, 5.80909510E-04, 5.59060462E-04,
     &   5.40331215E-04, 5.22646274E-04, 5.07270087E-04, 4.92574651E-04,
     &   4.79100201E-04, 4.67209143E-04, 4.55695186E-04, 4.45460668E-04,
     &   4.35486459E-04, 4.26568580E-04, 4.17831293E-04, 4.09981341E-04,
     &   4.02256097E-04, 3.95286888E-04, 3.88402377E-04, 3.82169728E-04,
     &   3.76254392E-04, 3.70383324E-04, 3.65044645E-04, 3.59731656E-04,
     &   3.54888202E-04, 3.50056546E-04, 3.45641941E-04, 3.41228689E-04,
     &   3.37188201E-04, 3.33141185E-04, 3.29429215E-04, 3.25704768E-04,
     &   3.22130579E-04, 3.18844229E-04, 3.15539087E-04, 3.12495695E-04,
     &   3.09430634E-04, 3.06604529E-04, 3.03754643E-04, 3.01123686E-04/

         DATA ((DGS  (I,J), I=2101,2174), J=3,3) /
     &   2.98467432E-04, 2.96012417E-04, 2.93531055E-04, 2.91235234E-04,
     &   2.89011781E-04, 2.86761072E-04, 2.84675612E-04, 2.82562617E-04,
     &   2.80602989E-04, 2.78614884E-04, 2.76770320E-04, 2.74898202E-04,
     &   2.73159103E-04, 2.71392674E-04, 2.69750515E-04, 2.68081325E-04,
     &   2.66458858E-04, 2.64948926E-04, 2.63412549E-04, 2.61879931E-04,
     &   2.60422364E-04, 2.59064171E-04, 2.57680478E-04, 2.56394056E-04,
     &   2.55075475E-04, 2.53848589E-04, 2.52597245E-04, 2.51429213E-04,
     &   2.50237259E-04, 2.49124084E-04, 2.48036382E-04, 2.46925571E-04,
     &   2.45887414E-04, 2.44826695E-04, 2.43834891E-04, 2.42821068E-04,
     &   2.41872689E-04, 2.40914558E-04, 2.40007041E-04, 2.39078570E-04,
     &   2.38209291E-04, 2.37319579E-04, 2.28744199E-04, 2.21820271E-04,
     &   2.20834611E-04, 1.93892010E-04, 1.87637863E-04, 1.77989403E-04,
     &   1.74890251E-04, 1.74695104E-04, 1.71826436E-04, 1.71819034E-04,
     &   1.70291362E-04, 1.70247024E-04, 1.70211502E-04, 1.69604385E-04,
     &   1.69241370E-04, 1.69179642E-04, 1.69080757E-04, 1.68938591E-04,
     &   1.68841295E-04, 1.68781412E-04, 1.68244237E-04, 1.68219215E-04,
     &   1.68084254E-04, 1.68058624E-04, 1.68082657E-04, 1.67895052E-04,
     &   1.67898220E-04, 1.67867181E-04, 1.67739666E-04, 1.67613967E-04,
     &   1.67562224E-04, 1.67243470E-04/
       save
       RETURN
       END


c      ++++ dalhadshigh19 ++++

c update 12/23/19
       subroutine dalhad_timelike1h19
       IMPLICIT NONE
       INTEGER NC,I,J
       PARAMETER(NC=200)
       REAL EMA(NC),DAM(NC,3),DGM(NC,3)
       COMMON /DATM19/EMA,DAM,DGM

c******************************************************************************
c*                                                                            *
c*    --- intRdatx ---                              Mon Dec 23 16:55:49 2019  *
c*                                                                            *
c***************************# FJ@HU Berlin #***********************************

c  "Delta alpha"               Update version (24/02/2019)

c ISCAN:  2, Ismooth: 0, IGS: 0, DSET: WAVE, iplot: 0 itestflavorspl: 0
c IKE,Emin,Emax,NP,N,EPSREL:  1, 6.0000E-01, 1.4000E+00, 100, 10, 1.00E-02,
c ALS(sta)(sys):  0.1184( 0.0007)( 0.0000), MZINP:   91.1876, MTOP:    173.20
c NF,IOR,ICHK: 5,4,2,ISU2: 0,NFL: 5,INAG: 1,inotail:  0, sin2W:   0.23153
c CHPTCUT,EC,ECUT: 0.3180,     5.20,    11.50


       DATA  (EMA  (I), I=   1, 100) /
     &   1.3005E+01, 1.3500E+01, 1.4000E+01, 1.4500E+01, 1.5000E+01,
     &   1.6000E+01, 1.7000E+01, 1.8000E+01, 1.9000E+01, 2.0000E+01,
     &   2.1000E+01, 2.2000E+01, 2.3000E+01, 2.4000E+01, 2.5000E+01,
     &   2.6000E+01, 2.7000E+01, 2.8000E+01, 2.9000E+01, 3.0000E+01,
     &   3.1000E+01, 3.2000E+01, 3.3000E+01, 3.4000E+01, 3.5000E+01,
     &   3.6000E+01, 3.7000E+01, 3.8000E+01, 3.9000E+01, 4.0000E+01,
     &   4.1000E+01, 4.2000E+01, 4.3000E+01, 4.4000E+01, 4.5000E+01,
     &   4.6000E+01, 4.7000E+01, 4.8000E+01, 4.9000E+01, 5.0000E+01,
     &   5.1000E+01, 5.2000E+01, 5.3000E+01, 5.4000E+01, 5.5000E+01,
     &   5.6000E+01, 5.7000E+01, 5.8000E+01, 5.9000E+01, 6.0000E+01,
     &   6.1000E+01, 6.2000E+01, 6.3000E+01, 6.4000E+01, 6.5000E+01,
     &   6.6000E+01, 6.7000E+01, 6.8000E+01, 6.9000E+01, 7.0000E+01,
     &   7.1000E+01, 7.2000E+01, 7.3000E+01, 7.4000E+01, 7.5000E+01,
     &   7.6000E+01, 7.7000E+01, 7.8000E+01, 7.9000E+01, 8.0000E+01,
     &   8.1000E+01, 8.2000E+01, 8.3000E+01, 8.4000E+01, 8.5000E+01,
     &   8.6000E+01, 8.7000E+01, 8.8000E+01, 8.9000E+01, 9.0000E+01,
     &   9.1000E+01, 9.2000E+01, 9.3000E+01, 9.4000E+01, 9.5000E+01,
     &   9.6000E+01, 9.7000E+01, 9.8000E+01, 9.9000E+01, 1.0000E+02,
     &   1.0900E+02, 1.1800E+02, 1.2700E+02, 1.3600E+02, 1.4500E+02,
     &   1.5400E+02, 1.6300E+02, 1.7200E+02, 1.8100E+02, 1.9000E+02/

       DATA  (EMA  (I), I= 101, 200) /
     &   1.9900E+02, 2.0800E+02, 2.1700E+02, 2.2600E+02, 2.3500E+02,
     &   2.4400E+02, 2.5300E+02, 2.6200E+02, 2.7100E+02, 2.8000E+02,
     &   2.8900E+02, 2.9800E+02, 3.0700E+02, 3.1600E+02, 3.2500E+02,
     &   3.3400E+02, 3.4300E+02, 3.5200E+02, 3.6100E+02, 3.7000E+02,
     &   3.7900E+02, 3.8800E+02, 3.9700E+02, 4.0600E+02, 4.1500E+02,
     &   4.2400E+02, 4.3300E+02, 4.4200E+02, 4.5100E+02, 4.6000E+02,
     &   4.6900E+02, 4.7800E+02, 4.8700E+02, 4.9600E+02, 5.0500E+02,
     &   5.1400E+02, 5.2300E+02, 5.3200E+02, 5.4100E+02, 5.5000E+02,
     &   5.5900E+02, 5.6800E+02, 5.7700E+02, 5.8600E+02, 5.9500E+02,
     &   6.0400E+02, 6.1300E+02, 6.2200E+02, 6.3100E+02, 6.4000E+02,
     &   6.4900E+02, 6.5800E+02, 6.6700E+02, 6.7600E+02, 6.8500E+02,
     &   6.9400E+02, 7.0300E+02, 7.1200E+02, 7.2100E+02, 7.3000E+02,
     &   7.3900E+02, 7.4800E+02, 7.5700E+02, 7.6600E+02, 7.7500E+02,
     &   7.8400E+02, 7.9300E+02, 8.0200E+02, 8.1100E+02, 8.2000E+02,
     &   8.2900E+02, 8.3800E+02, 8.4700E+02, 8.5600E+02, 8.6500E+02,
     &   8.7400E+02, 8.8300E+02, 8.9200E+02, 9.0100E+02, 9.1000E+02,
     &   9.1900E+02, 9.2800E+02, 9.3700E+02, 9.4600E+02, 9.5500E+02,
     &   9.6400E+02, 9.7300E+02, 9.8200E+02, 9.9100E+02, 1.0000E+03,
     &   2.0000E+03, 3.0000E+03, 4.0000E+03, 5.0000E+03, 6.0000E+03,
     &   8.0000E+03, 1.0000E+04, 1.2000E+04, 1.4000E+04, 2.0000E+04/


         DATA ((DAM  (I,J), I=   1, 100), J=1,1) /
     &   1.5864E-02, 1.6148E-02, 1.6348E-02, 1.6574E-02, 1.6790E-02,
     &   1.7196E-02, 1.7574E-02, 1.7927E-02, 1.8259E-02, 1.8573E-02,
     &   1.8869E-02, 1.9261E-02, 1.9420E-02, 1.9677E-02, 1.9923E-02,
     &   2.0160E-02, 2.0385E-02, 2.0603E-02, 2.0813E-02, 2.1016E-02,
     &   2.1212E-02, 2.1402E-02, 2.1585E-02, 2.1767E-02, 2.1969E-02,
     &   2.2104E-02, 2.2267E-02, 2.2426E-02, 2.2581E-02, 2.2732E-02,
     &   2.2878E-02, 2.3021E-02, 2.3169E-02, 2.3297E-02, 2.3430E-02,
     &   2.3624E-02, 2.3689E-02, 2.3813E-02, 2.3936E-02, 2.4056E-02,
     &   2.4173E-02, 2.4288E-02, 2.4401E-02, 2.4536E-02, 2.4620E-02,
     &   2.4727E-02, 2.4832E-02, 2.5060E-02, 2.5036E-02, 2.5136E-02,
     &   2.5234E-02, 2.5330E-02, 2.5425E-02, 2.5518E-02, 2.5610E-02,
     &   2.5700E-02, 2.5789E-02, 2.5877E-02, 2.5963E-02, 2.6048E-02,
     &   2.6132E-02, 2.6215E-02, 2.6296E-02, 2.6377E-02, 2.6456E-02,
     &   2.6534E-02, 2.6612E-02, 2.6688E-02, 2.6763E-02, 2.6838E-02,
     &   2.6911E-02, 2.6984E-02, 2.7055E-02, 2.7126E-02, 2.7196E-02,
     &   2.7265E-02, 2.7334E-02, 2.7401E-02, 2.7468E-02, 2.7534E-02,
     &   2.7599E-02, 2.7664E-02, 2.7728E-02, 2.7791E-02, 2.7854E-02,
     &   2.7915E-02, 2.7977E-02, 2.8037E-02, 2.8097E-02, 2.8157E-02,
     &   2.8666E-02, 2.9135E-02, 2.9570E-02, 2.9975E-02, 3.0354E-02,
     &   3.0711E-02, 3.1047E-02, 3.1366E-02, 3.1668E-02, 3.1957E-02/

         DATA ((DAM  (I,J), I= 101, 200), J=1,1) /
     &   3.2232E-02, 3.2495E-02, 3.2747E-02, 3.2989E-02, 3.3223E-02,
     &   3.3447E-02, 3.3664E-02, 3.3874E-02, 3.4076E-02, 3.4273E-02,
     &   3.4463E-02, 3.4648E-02, 3.4828E-02, 3.5003E-02, 3.5173E-02,
     &   3.5339E-02, 3.5501E-02, 3.5658E-02, 3.5814E-02, 3.5966E-02,
     &   3.6114E-02, 3.6260E-02, 3.6402E-02, 3.6542E-02, 3.6679E-02,
     &   3.6813E-02, 3.6945E-02, 3.7075E-02, 3.7202E-02, 3.7328E-02,
     &   3.7452E-02, 3.7574E-02, 3.7694E-02, 3.7812E-02, 3.7922E-02,
     &   3.8026E-02, 3.8128E-02, 3.8227E-02, 3.8326E-02, 3.8422E-02,
     &   3.8517E-02, 3.8611E-02, 3.8703E-02, 3.8793E-02, 3.8883E-02,
     &   3.8970E-02, 3.9057E-02, 3.9142E-02, 3.9226E-02, 3.9309E-02,
     &   3.9391E-02, 3.9472E-02, 3.9551E-02, 3.9629E-02, 3.9707E-02,
     &   3.9783E-02, 3.9858E-02, 3.9933E-02, 4.0006E-02, 4.0079E-02,
     &   4.0151E-02, 4.0221E-02, 4.0291E-02, 4.0360E-02, 4.0429E-02,
     &   4.0496E-02, 4.0563E-02, 4.0629E-02, 4.0694E-02, 4.0759E-02,
     &   4.0823E-02, 4.0886E-02, 4.0948E-02, 4.1010E-02, 4.1071E-02,
     &   4.1132E-02, 4.1192E-02, 4.1251E-02, 4.1309E-02, 4.1368E-02,
     &   4.1425E-02, 4.1482E-02, 4.1538E-02, 4.1594E-02, 4.1650E-02,
     &   4.1704E-02, 4.1759E-02, 4.1813E-02, 4.1866E-02, 4.1919E-02,
     &   4.5966E-02, 4.8329E-02, 5.0006E-02, 5.1306E-02, 5.2366E-02,
     &   5.4040E-02, 5.5338E-02, 5.6397E-02, 5.7294E-02, 5.9366E-02/


         DATA ((DAM  (I,J), I=   1, 100), J=2,2) /
     &   5.8967E-05, 5.8244E-05, 5.7703E-05, 5.7278E-05, 5.6930E-05,
     &   5.6385E-05, 5.5973E-05, 5.5648E-05, 5.5385E-05, 5.5168E-05,
     &   5.4985E-05, 5.4830E-05, 5.4698E-05, 5.4582E-05, 5.4482E-05,
     &   5.4394E-05, 5.4316E-05, 5.4247E-05, 5.4185E-05, 5.4130E-05,
     &   5.4080E-05, 5.4035E-05, 5.3994E-05, 5.3957E-05, 5.3923E-05,
     &   5.3891E-05, 5.3863E-05, 5.3837E-05, 5.3813E-05, 5.3790E-05,
     &   5.3770E-05, 5.3750E-05, 5.3732E-05, 5.3716E-05, 5.3700E-05,
     &   5.3686E-05, 5.3672E-05, 5.3660E-05, 5.3648E-05, 5.3636E-05,
     &   5.3626E-05, 5.3616E-05, 5.3607E-05, 5.3598E-05, 5.3589E-05,
     &   5.3582E-05, 5.3574E-05, 5.3567E-05, 5.3560E-05, 5.3554E-05,
     &   5.3548E-05, 5.3542E-05, 5.3536E-05, 5.3531E-05, 5.3526E-05,
     &   5.3521E-05, 5.3517E-05, 5.3512E-05, 5.3508E-05, 5.3504E-05,
     &   5.3500E-05, 5.3497E-05, 5.3493E-05, 5.3490E-05, 5.3487E-05,
     &   5.3483E-05, 5.3480E-05, 5.3478E-05, 5.3475E-05, 5.3472E-05,
     &   5.3470E-05, 5.3467E-05, 5.3465E-05, 5.3462E-05, 5.3460E-05,
     &   5.3458E-05, 5.3456E-05, 5.3454E-05, 5.3452E-05, 5.3450E-05,
     &   5.3448E-05, 5.3447E-05, 5.3445E-05, 5.3443E-05, 5.3442E-05,
     &   5.3440E-05, 5.3439E-05, 5.3437E-05, 5.3436E-05, 5.3434E-05,
     &   5.3424E-05, 5.3416E-05, 5.3409E-05, 5.3404E-05, 5.3399E-05,
     &   5.3396E-05, 5.3393E-05, 5.3390E-05, 5.3388E-05, 5.3386E-05/

         DATA ((DAM  (I,J), I= 101, 200), J=2,2) /
     &   5.3385E-05, 5.3383E-05, 5.3382E-05, 5.3381E-05, 5.3380E-05,
     &   5.3379E-05, 5.3378E-05, 5.3378E-05, 5.3377E-05, 5.3376E-05,
     &   5.3376E-05, 5.3375E-05, 5.3375E-05, 5.3374E-05, 5.3374E-05,
     &   5.3374E-05, 5.3373E-05, 5.3373E-05, 5.3373E-05, 5.3373E-05,
     &   5.3372E-05, 5.3372E-05, 5.3372E-05, 5.3372E-05, 5.3372E-05,
     &   5.3372E-05, 5.3371E-05, 5.3371E-05, 5.3371E-05, 5.3371E-05,
     &   5.3371E-05, 5.3371E-05, 5.3371E-05, 5.3371E-05, 5.3370E-05,
     &   5.3370E-05, 5.3370E-05, 5.3370E-05, 5.3370E-05, 5.3370E-05,
     &   5.3370E-05, 5.3370E-05, 5.3370E-05, 5.3370E-05, 5.3370E-05,
     &   5.3370E-05, 5.3370E-05, 5.3370E-05, 5.3369E-05, 5.3369E-05,
     &   5.3369E-05, 5.3369E-05, 5.3369E-05, 5.3369E-05, 5.3369E-05,
     &   5.3369E-05, 5.3369E-05, 5.3369E-05, 5.3369E-05, 5.3369E-05,
     &   5.3369E-05, 5.3369E-05, 5.3369E-05, 5.3369E-05, 5.3369E-05,
     &   5.3369E-05, 5.3369E-05, 5.3369E-05, 5.3369E-05, 5.3369E-05,
     &   5.3369E-05, 5.3369E-05, 5.3369E-05, 5.3369E-05, 5.3369E-05,
     &   5.3369E-05, 5.3369E-05, 5.3369E-05, 5.3369E-05, 5.3369E-05,
     &   5.3369E-05, 5.3369E-05, 5.3369E-05, 5.3369E-05, 5.3369E-05,
     &   5.3369E-05, 5.3369E-05, 5.3369E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05/


         DATA ((DAM  (I,J), I=   1, 100), J=3,3) /
     &   1.1612E-04, 1.1075E-04, 1.0745E-04, 1.0523E-04, 1.0365E-04,
     &   1.0153E-04, 1.0018E-04, 9.9234E-05, 9.8536E-05, 9.7999E-05,
     &   9.7574E-05, 9.7242E-05, 9.6944E-05, 9.6705E-05, 9.6502E-05,
     &   9.6328E-05, 9.6177E-05, 9.6045E-05, 9.5929E-05, 9.5827E-05,
     &   9.5736E-05, 9.5655E-05, 9.5582E-05, 9.5518E-05, 9.5465E-05,
     &   9.5403E-05, 9.5354E-05, 9.5309E-05, 9.5268E-05, 9.5231E-05,
     &   9.5195E-05, 9.5163E-05, 9.5137E-05, 9.5106E-05, 9.5081E-05,
     &   9.5073E-05, 9.5035E-05, 9.5015E-05, 9.4996E-05, 9.4978E-05,
     &   9.4961E-05, 9.4945E-05, 9.4931E-05, 9.4925E-05, 9.4904E-05,
     &   9.4891E-05, 9.4880E-05, 9.4897E-05, 9.4859E-05, 9.4849E-05,
     &   9.4840E-05, 9.4831E-05, 9.4823E-05, 9.4815E-05, 9.4807E-05,
     &   9.4800E-05, 9.4793E-05, 9.4787E-05, 9.4781E-05, 9.4775E-05,
     &   9.4769E-05, 9.4764E-05, 9.4759E-05, 9.4754E-05, 9.4750E-05,
     &   9.4745E-05, 9.4741E-05, 9.4737E-05, 9.4733E-05, 9.4730E-05,
     &   9.4726E-05, 9.4723E-05, 9.4719E-05, 9.4716E-05, 9.4713E-05,
     &   9.4711E-05, 9.4708E-05, 9.4705E-05, 9.4703E-05, 9.4700E-05,
     &   9.4698E-05, 9.4696E-05, 9.4693E-05, 9.4691E-05, 9.4689E-05,
     &   9.4687E-05, 9.4686E-05, 9.4684E-05, 9.4682E-05, 9.4680E-05,
     &   9.4668E-05, 9.4659E-05, 9.4653E-05, 9.4648E-05, 9.4644E-05,
     &   9.4642E-05, 9.4640E-05, 9.4639E-05, 9.4638E-05, 9.4638E-05/

         DATA ((DAM  (I,J), I= 101, 200), J=3,3) /
     &   9.4638E-05, 9.4638E-05, 9.4638E-05, 9.4639E-05, 9.4639E-05,
     &   9.4640E-05, 9.4641E-05, 9.4641E-05, 9.4642E-05, 9.4643E-05,
     &   9.4644E-05, 9.4645E-05, 9.4646E-05, 9.4647E-05, 9.4648E-05,
     &   9.4648E-05, 9.4649E-05, 9.4650E-05, 9.4651E-05, 9.4652E-05,
     &   9.4653E-05, 9.4654E-05, 9.4655E-05, 9.4656E-05, 9.4657E-05,
     &   9.4658E-05, 9.4659E-05, 9.4660E-05, 9.4661E-05, 9.4662E-05,
     &   9.4663E-05, 9.4663E-05, 9.4664E-05, 9.4665E-05, 9.4666E-05,
     &   9.4667E-05, 9.4668E-05, 9.4668E-05, 9.4669E-05, 9.4670E-05,
     &   9.4670E-05, 9.4671E-05, 9.4672E-05, 9.4673E-05, 9.4673E-05,
     &   9.4674E-05, 9.4675E-05, 9.4675E-05, 9.4676E-05, 9.4676E-05,
     &   9.4677E-05, 9.4678E-05, 9.4678E-05, 9.4679E-05, 9.4680E-05,
     &   9.4680E-05, 9.4681E-05, 9.4681E-05, 9.4682E-05, 9.4682E-05,
     &   9.4683E-05, 9.4684E-05, 9.4684E-05, 9.4685E-05, 9.4685E-05,
     &   9.4686E-05, 9.4686E-05, 9.4687E-05, 9.4687E-05, 9.4688E-05,
     &   9.4688E-05, 9.4689E-05, 9.4689E-05, 9.4690E-05, 9.4690E-05,
     &   9.4691E-05, 9.4691E-05, 9.4692E-05, 9.4692E-05, 9.4693E-05,
     &   9.4693E-05, 9.4694E-05, 9.4694E-05, 9.4694E-05, 9.4695E-05,
     &   9.4695E-05, 9.4696E-05, 9.4696E-05, 9.4696E-05, 9.4697E-05,
     &   9.4729E-05, 9.4743E-05, 9.4759E-05, 9.4768E-05, 9.4773E-05,
     &   9.4788E-05, 9.4796E-05, 9.4800E-05, 9.4809E-05, 9.4822E-05/


c      ++++ deghadshigh19 ++++

c update 12/22/19

c******************************************************************************
c*                                                                            *
c*    --- intRdatx ---                              Sun Dec 22 18:47:22 2019  *
c*                                                                            *
c***************************# FJ@HU Berlin #***********************************

c  "Delta alpha"               Update version (24/02/2019)

c ISCAN:  2, Ismooth: 0, IGS: 0, DSET: WAVE, iplot: 0 itestflavorspl: 0
c IKE,Emin,Emax,NP,N,EPSREL:  1, 6.0000E-01, 1.4000E+00, 100, 10, 1.00E-02,
c ALS(sta)(sys):  0.1184( 0.0007)( 0.0000), MZINP:   91.1876, MTOP:    173.20
c NF,IOR,ICHK: 5,4,2,ISU2: 2,NFL: 5,INAG: 1,inotail:  0, sin2W:   0.23153
c CHPTCUT,EC,ECUT: 0.3180,     5.20,    11.50


         DATA ((DGM  (I,J), I=   1, 100), J=1,1) /
     &   3.1837E-02, 3.2449E-02, 3.2885E-02, 3.3368E-02, 3.3829E-02,
     &   3.4691E-02, 3.5489E-02, 3.6231E-02, 3.6927E-02, 3.7583E-02,
     &   3.8201E-02, 3.9017E-02, 3.9348E-02, 3.9882E-02, 4.0393E-02,
     &   4.0885E-02, 4.1352E-02, 4.1804E-02, 4.2239E-02, 4.2659E-02,
     &   4.3066E-02, 4.3458E-02, 4.3838E-02, 4.4212E-02, 4.4631E-02,
     &   4.4910E-02, 4.5247E-02, 4.5575E-02, 4.5896E-02, 4.6208E-02,
     &   4.6509E-02, 4.6805E-02, 4.7111E-02, 4.7375E-02, 4.7651E-02,
     &   4.8052E-02, 4.8183E-02, 4.8441E-02, 4.8694E-02, 4.8941E-02,
     &   4.9183E-02, 4.9421E-02, 4.9654E-02, 4.9932E-02, 5.0107E-02,
     &   5.0327E-02, 5.0544E-02, 5.1015E-02, 5.0965E-02, 5.1170E-02,
     &   5.1372E-02, 5.1571E-02, 5.1766E-02, 5.1958E-02, 5.2148E-02,
     &   5.2334E-02, 5.2518E-02, 5.2698E-02, 5.2877E-02, 5.3052E-02,
     &   5.3225E-02, 5.3396E-02, 5.3564E-02, 5.3730E-02, 5.3894E-02,
     &   5.4055E-02, 5.4215E-02, 5.4372E-02, 5.4528E-02, 5.4681E-02,
     &   5.4832E-02, 5.4982E-02, 5.5130E-02, 5.5276E-02, 5.5420E-02,
     &   5.5563E-02, 5.5704E-02, 5.5843E-02, 5.5981E-02, 5.6117E-02,
     &   5.6252E-02, 5.6385E-02, 5.6517E-02, 5.6647E-02, 5.6776E-02,
     &   5.6903E-02, 5.7030E-02, 5.7155E-02, 5.7279E-02, 5.7401E-02,
     &   5.8451E-02, 5.9418E-02, 6.0315E-02, 6.1150E-02, 6.1932E-02,
     &   6.2667E-02, 6.3360E-02, 6.4017E-02, 6.4641E-02, 6.5235E-02/

         DATA ((DGM  (I,J), I= 101, 200), J=1,1) /
     &   6.5802E-02, 6.6345E-02, 6.6865E-02, 6.7365E-02, 6.7845E-02,
     &   6.8308E-02, 6.8755E-02, 6.9187E-02, 6.9605E-02, 7.0010E-02,
     &   7.0403E-02, 7.0784E-02, 7.1155E-02, 7.1516E-02, 7.1867E-02,
     &   7.2209E-02, 7.2542E-02, 7.2866E-02, 7.3187E-02, 7.3500E-02,
     &   7.3806E-02, 7.4106E-02, 7.4400E-02, 7.4687E-02, 7.4970E-02,
     &   7.5247E-02, 7.5519E-02, 7.5786E-02, 7.6050E-02, 7.6308E-02,
     &   7.6563E-02, 7.6815E-02, 7.7062E-02, 7.7306E-02, 7.7534E-02,
     &   7.7747E-02, 7.7957E-02, 7.8163E-02, 7.8365E-02, 7.8564E-02,
     &   7.8760E-02, 7.8953E-02, 7.9143E-02, 7.9329E-02, 7.9513E-02,
     &   7.9694E-02, 7.9873E-02, 8.0049E-02, 8.0222E-02, 8.0393E-02,
     &   8.0561E-02, 8.0727E-02, 8.0891E-02, 8.1053E-02, 8.1212E-02,
     &   8.1370E-02, 8.1525E-02, 8.1678E-02, 8.1830E-02, 8.1979E-02,
     &   8.2127E-02, 8.2273E-02, 8.2417E-02, 8.2560E-02, 8.2701E-02,
     &   8.2840E-02, 8.2977E-02, 8.3114E-02, 8.3248E-02, 8.3381E-02,
     &   8.3513E-02, 8.3643E-02, 8.3771E-02, 8.3899E-02, 8.4025E-02,
     &   8.4150E-02, 8.4273E-02, 8.4395E-02, 8.4516E-02, 8.4636E-02,
     &   8.4755E-02, 8.4872E-02, 8.4988E-02, 8.5103E-02, 8.5218E-02,
     &   8.5331E-02, 8.5442E-02, 8.5553E-02, 8.5663E-02, 8.5772E-02,
     &   9.4115E-02, 9.8987E-02, 1.0244E-01, 1.0512E-01, 1.0731E-01,
     &   1.1076E-01, 1.1344E-01, 1.1562E-01, 1.1747E-01, 1.2174E-01/


         DATA ((DGM  (I,J), I=   1, 100), J=2,2) /
     &   1.0572E-04, 1.0275E-04, 1.0069E-04, 9.9190E-05, 9.8030E-05,
     &   9.6342E-05, 9.5161E-05, 9.4283E-05, 9.3603E-05, 9.3060E-05,
     &   9.2617E-05, 9.2249E-05, 9.1938E-05, 9.1673E-05, 9.1445E-05,
     &   9.1247E-05, 9.1074E-05, 9.0921E-05, 9.0785E-05, 9.0664E-05,
     &   9.0556E-05, 9.0459E-05, 9.0371E-05, 9.0291E-05, 9.0219E-05,
     &   9.0152E-05, 9.0092E-05, 9.0036E-05, 8.9985E-05, 8.9938E-05,
     &   8.9895E-05, 8.9854E-05, 8.9817E-05, 8.9782E-05, 8.9750E-05,
     &   8.9720E-05, 8.9691E-05, 8.9665E-05, 8.9640E-05, 8.9617E-05,
     &   8.9595E-05, 8.9574E-05, 8.9555E-05, 8.9537E-05, 8.9519E-05,
     &   8.9503E-05, 8.9487E-05, 8.9473E-05, 8.9459E-05, 8.9446E-05,
     &   8.9433E-05, 8.9421E-05, 8.9410E-05, 8.9399E-05, 8.9389E-05,
     &   8.9379E-05, 8.9369E-05, 8.9360E-05, 8.9352E-05, 8.9343E-05,
     &   8.9336E-05, 8.9328E-05, 8.9321E-05, 8.9314E-05, 8.9307E-05,
     &   8.9301E-05, 8.9295E-05, 8.9289E-05, 8.9283E-05, 8.9278E-05,
     &   8.9272E-05, 8.9267E-05, 8.9262E-05, 8.9258E-05, 8.9253E-05,
     &   8.9249E-05, 8.9245E-05, 8.9240E-05, 8.9237E-05, 8.9233E-05,
     &   8.9229E-05, 8.9225E-05, 8.9222E-05, 8.9219E-05, 8.9215E-05,
     &   8.9212E-05, 8.9209E-05, 8.9206E-05, 8.9203E-05, 8.9201E-05,
     &   8.9179E-05, 8.9162E-05, 8.9149E-05, 8.9138E-05, 8.9129E-05,
     &   8.9122E-05, 8.9116E-05, 8.9111E-05, 8.9106E-05, 8.9102E-05/

         DATA ((DGM  (I,J), I= 101, 200), J=2,2) /
     &   8.9099E-05, 8.9096E-05, 8.9094E-05, 8.9091E-05, 8.9089E-05,
     &   8.9088E-05, 8.9086E-05, 8.9085E-05, 8.9083E-05, 8.9082E-05,
     &   8.9081E-05, 8.9080E-05, 8.9079E-05, 8.9078E-05, 8.9078E-05,
     &   8.9077E-05, 8.9076E-05, 8.9076E-05, 8.9075E-05, 8.9075E-05,
     &   8.9074E-05, 8.9074E-05, 8.9074E-05, 8.9073E-05, 8.9073E-05,
     &   8.9072E-05, 8.9072E-05, 8.9072E-05, 8.9072E-05, 8.9071E-05,
     &   8.9071E-05, 8.9071E-05, 8.9071E-05, 8.9070E-05, 8.9070E-05,
     &   8.9070E-05, 8.9070E-05, 8.9070E-05, 8.9070E-05, 8.9069E-05,
     &   8.9069E-05, 8.9069E-05, 8.9069E-05, 8.9069E-05, 8.9069E-05,
     &   8.9069E-05, 8.9069E-05, 8.9068E-05, 8.9068E-05, 8.9068E-05,
     &   8.9068E-05, 8.9068E-05, 8.9068E-05, 8.9068E-05, 8.9068E-05,
     &   8.9068E-05, 8.9068E-05, 8.9068E-05, 8.9068E-05, 8.9067E-05,
     &   8.9067E-05, 8.9067E-05, 8.9067E-05, 8.9067E-05, 8.9067E-05,
     &   8.9067E-05, 8.9067E-05, 8.9067E-05, 8.9067E-05, 8.9067E-05,
     &   8.9067E-05, 8.9067E-05, 8.9067E-05, 8.9067E-05, 8.9067E-05,
     &   8.9067E-05, 8.9067E-05, 8.9067E-05, 8.9067E-05, 8.9067E-05,
     &   8.9067E-05, 8.9067E-05, 8.9066E-05, 8.9066E-05, 8.9066E-05,
     &   8.9066E-05, 8.9066E-05, 8.9066E-05, 8.9066E-05, 8.9066E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05/


         DATA ((DGM  (I,J), I=   1, 100), J=3,3) /
     &   2.6188E-04, 2.3732E-04, 2.2182E-04, 2.1135E-04, 2.0385E-04,
     &   1.9389E-04, 1.8764E-04, 1.8338E-04, 1.8030E-04, 1.7799E-04,
     &   1.7619E-04, 1.7483E-04, 1.7359E-04, 1.7263E-04, 1.7183E-04,
     &   1.7114E-04, 1.7055E-04, 1.7005E-04, 1.6960E-04, 1.6922E-04,
     &   1.6887E-04, 1.6857E-04, 1.6830E-04, 1.6806E-04, 1.6787E-04,
     &   1.6763E-04, 1.6745E-04, 1.6729E-04, 1.6714E-04, 1.6700E-04,
     &   1.6688E-04, 1.6676E-04, 1.6667E-04, 1.6655E-04, 1.6646E-04,
     &   1.6645E-04, 1.6630E-04, 1.6622E-04, 1.6616E-04, 1.6609E-04,
     &   1.6603E-04, 1.6598E-04, 1.6592E-04, 1.6591E-04, 1.6583E-04,
     &   1.6578E-04, 1.6574E-04, 1.6583E-04, 1.6567E-04, 1.6563E-04,
     &   1.6560E-04, 1.6557E-04, 1.6554E-04, 1.6551E-04, 1.6548E-04,
     &   1.6546E-04, 1.6543E-04, 1.6541E-04, 1.6539E-04, 1.6537E-04,
     &   1.6535E-04, 1.6533E-04, 1.6531E-04, 1.6530E-04, 1.6528E-04,
     &   1.6526E-04, 1.6525E-04, 1.6523E-04, 1.6522E-04, 1.6521E-04,
     &   1.6519E-04, 1.6518E-04, 1.6517E-04, 1.6516E-04, 1.6515E-04,
     &   1.6514E-04, 1.6513E-04, 1.6512E-04, 1.6511E-04, 1.6510E-04,
     &   1.6509E-04, 1.6508E-04, 1.6508E-04, 1.6507E-04, 1.6506E-04,
     &   1.6506E-04, 1.6505E-04, 1.6504E-04, 1.6504E-04, 1.6503E-04,
     &   1.6498E-04, 1.6495E-04, 1.6493E-04, 1.6491E-04, 1.6489E-04,
     &   1.6488E-04, 1.6487E-04, 1.6487E-04, 1.6486E-04, 1.6486E-04/

         DATA ((DGM  (I,J), I= 101, 200), J=3,3) /
     &   1.6486E-04, 1.6486E-04, 1.6486E-04, 1.6486E-04, 1.6486E-04,
     &   1.6486E-04, 1.6486E-04, 1.6486E-04, 1.6486E-04, 1.6486E-04,
     &   1.6487E-04, 1.6487E-04, 1.6487E-04, 1.6487E-04, 1.6487E-04,
     &   1.6488E-04, 1.6488E-04, 1.6488E-04, 1.6488E-04, 1.6488E-04,
     &   1.6489E-04, 1.6489E-04, 1.6489E-04, 1.6489E-04, 1.6490E-04,
     &   1.6490E-04, 1.6490E-04, 1.6490E-04, 1.6491E-04, 1.6491E-04,
     &   1.6491E-04, 1.6491E-04, 1.6492E-04, 1.6492E-04, 1.6492E-04,
     &   1.6492E-04, 1.6492E-04, 1.6493E-04, 1.6493E-04, 1.6493E-04,
     &   1.6493E-04, 1.6493E-04, 1.6493E-04, 1.6494E-04, 1.6494E-04,
     &   1.6494E-04, 1.6494E-04, 1.6494E-04, 1.6495E-04, 1.6495E-04,
     &   1.6495E-04, 1.6495E-04, 1.6495E-04, 1.6495E-04, 1.6495E-04,
     &   1.6496E-04, 1.6496E-04, 1.6496E-04, 1.6496E-04, 1.6496E-04,
     &   1.6496E-04, 1.6497E-04, 1.6497E-04, 1.6497E-04, 1.6497E-04,
     &   1.6497E-04, 1.6497E-04, 1.6497E-04, 1.6498E-04, 1.6498E-04,
     &   1.6498E-04, 1.6498E-04, 1.6498E-04, 1.6498E-04, 1.6498E-04,
     &   1.6498E-04, 1.6499E-04, 1.6499E-04, 1.6499E-04, 1.6499E-04,
     &   1.6499E-04, 1.6499E-04, 1.6499E-04, 1.6499E-04, 1.6500E-04,
     &   1.6500E-04, 1.6500E-04, 1.6500E-04, 1.6500E-04, 1.6500E-04,
     &   1.6509E-04, 1.6512E-04, 1.6517E-04, 1.6519E-04, 1.6520E-04,
     &   1.6524E-04, 1.6527E-04, 1.6527E-04, 1.6530E-04, 1.6533E-04/

       save
       RETURN
       END

c      ++++ dalhadt19 ++++

c update 12/25/19
       subroutine dalhad_spacelike19
       IMPLICIT NONE
       INTEGER NA,NC,I,J
       PARAMETER(NA=979,NC=200)
       REAL ETA(NA),DAT(NA,3),DGT(NA,3)
       REAL XXX(NC),YGG(NC,3),Y3G(NC,3)
       COMMON /DATT19/ETA,DAT,DGT
       COMMON /DATH19/XXX,YGG,Y3G

c******************************************************************************
c*                                                                            *
c*    --- intRdatx ---                              Wed Dec 25 17:40:23 2019  *
c*                                                                            *
c***************************# FJ@HU Berlin #***********************************

c  "Delta alpha"               Update version (24/02/2019)

c ISCAN: -1, Ismooth: 0, IGS: 0, DSET: WAVE, iplot: 0 itestflavorspl: 0
c IKE,Emin,Emax,NP,N,EPSREL:  1, 6.0000E-01, 1.4000E+00, 100, 10, 1.00E-02,
c ALS(sta)(sys):  0.1184( 0.0007)( 0.0000), MZINP:   91.1876, MTOP:    173.20
c NF,IOR,ICHK: 5,4,2,ISU2: 0,NFL: 5,INAG: 1,inotail:  0, sin2W:   0.23153
c CHPTCUT,EC,ECUT: 0.3180,     5.20,    11.50


       DATA  (ETA  (I), I=   1, 100) /
     &  -1.0000E+03,-9.9000E+02,-9.8000E+02,-9.7000E+02,-9.6000E+02,
     &  -9.5000E+02,-9.4000E+02,-9.3000E+02,-9.2000E+02,-9.1000E+02,
     &  -9.0000E+02,-8.9000E+02,-8.8000E+02,-8.7000E+02,-8.6000E+02,
     &  -8.5000E+02,-8.4000E+02,-8.3000E+02,-8.2000E+02,-8.1000E+02,
     &  -8.0000E+02,-7.9000E+02,-7.8000E+02,-7.7000E+02,-7.6000E+02,
     &  -7.5000E+02,-7.4000E+02,-7.3000E+02,-7.2000E+02,-7.1000E+02,
     &  -7.0000E+02,-6.9000E+02,-6.8000E+02,-6.7000E+02,-6.6000E+02,
     &  -6.5000E+02,-6.4000E+02,-6.3000E+02,-6.2000E+02,-6.1000E+02,
     &  -6.0000E+02,-5.9000E+02,-5.8000E+02,-5.7000E+02,-5.6000E+02,
     &  -5.5000E+02,-5.4000E+02,-5.3000E+02,-5.2000E+02,-5.1000E+02,
     &  -5.0000E+02,-4.9000E+02,-4.8000E+02,-4.7000E+02,-4.6000E+02,
     &  -4.5000E+02,-4.4000E+02,-4.3000E+02,-4.2000E+02,-4.1000E+02,
     &  -4.0000E+02,-3.9000E+02,-3.8000E+02,-3.7000E+02,-3.6000E+02,
     &  -3.5000E+02,-3.4000E+02,-3.3000E+02,-3.2000E+02,-3.1000E+02,
     &  -3.0000E+02,-2.9000E+02,-2.8000E+02,-2.7000E+02,-2.6000E+02,
     &  -2.5000E+02,-2.4000E+02,-2.3000E+02,-2.2000E+02,-2.1000E+02,
     &  -2.0000E+02,-1.9000E+02,-1.8000E+02,-1.7000E+02,-1.6000E+02,
     &  -1.5000E+02,-1.4000E+02,-1.3000E+02,-1.2000E+02,-1.1000E+02,
     &  -1.0000E+02,-9.9000E+01,-9.8000E+01,-9.7000E+01,-9.6000E+01,
     &  -9.5000E+01,-9.4000E+01,-9.3000E+01,-9.2000E+01,-9.1000E+01/

       DATA  (ETA  (I), I= 101, 200) /
     &  -9.0000E+01,-8.9000E+01,-8.8000E+01,-8.7000E+01,-8.6000E+01,
     &  -8.5000E+01,-8.4000E+01,-8.3000E+01,-8.2000E+01,-8.1000E+01,
     &  -8.0000E+01,-7.9000E+01,-7.8000E+01,-7.7000E+01,-7.6000E+01,
     &  -7.5000E+01,-7.4000E+01,-7.3000E+01,-7.2000E+01,-7.1000E+01,
     &  -7.0000E+01,-6.9000E+01,-6.8000E+01,-6.7000E+01,-6.6000E+01,
     &  -6.5000E+01,-6.4000E+01,-6.3000E+01,-6.2000E+01,-6.1000E+01,
     &  -6.0000E+01,-5.9000E+01,-5.8000E+01,-5.7000E+01,-5.6000E+01,
     &  -5.5000E+01,-5.4000E+01,-5.3000E+01,-5.2000E+01,-5.1000E+01,
     &  -5.0000E+01,-4.9000E+01,-4.8000E+01,-4.7000E+01,-4.6000E+01,
     &  -4.5000E+01,-4.4000E+01,-4.3000E+01,-4.2000E+01,-4.1000E+01,
     &  -4.0000E+01,-3.9000E+01,-3.8000E+01,-3.7000E+01,-3.6000E+01,
     &  -3.5000E+01,-3.4000E+01,-3.3000E+01,-3.2000E+01,-3.1000E+01,
     &  -3.0000E+01,-2.9000E+01,-2.8000E+01,-2.7000E+01,-2.6000E+01,
     &  -2.5000E+01,-2.4000E+01,-2.3000E+01,-2.2000E+01,-2.1000E+01,
     &  -2.0000E+01,-1.9000E+01,-1.8000E+01,-1.7000E+01,-1.6000E+01,
     &  -1.5000E+01,-1.4000E+01,-1.3000E+01,-1.2000E+01,-1.1000E+01,
     &  -1.0000E+01,-9.9100E+00,-9.8200E+00,-9.7300E+00,-9.6400E+00,
     &  -9.5500E+00,-9.4600E+00,-9.3700E+00,-9.2800E+00,-9.1900E+00,
     &  -9.1000E+00,-9.0100E+00,-8.9200E+00,-8.8300E+00,-8.7400E+00,
     &  -8.6500E+00,-8.5600E+00,-8.4700E+00,-8.3800E+00,-8.2900E+00/

       DATA  (ETA  (I), I= 201, 300) /
     &  -8.2000E+00,-8.1100E+00,-8.0200E+00,-7.9300E+00,-7.8400E+00,
     &  -7.7500E+00,-7.6600E+00,-7.5700E+00,-7.4800E+00,-7.3900E+00,
     &  -7.3000E+00,-7.2100E+00,-7.1200E+00,-7.0300E+00,-6.9400E+00,
     &  -6.8500E+00,-6.7600E+00,-6.6700E+00,-6.5800E+00,-6.4900E+00,
     &  -6.4000E+00,-6.3100E+00,-6.2200E+00,-6.1300E+00,-6.0400E+00,
     &  -5.9500E+00,-5.8600E+00,-5.7700E+00,-5.6800E+00,-5.5900E+00,
     &  -5.5000E+00,-5.4100E+00,-5.3200E+00,-5.2300E+00,-5.1400E+00,
     &  -5.0500E+00,-4.9600E+00,-4.8700E+00,-4.7800E+00,-4.6900E+00,
     &  -4.6000E+00,-4.5100E+00,-4.4200E+00,-4.3300E+00,-4.2400E+00,
     &  -4.1500E+00,-4.0600E+00,-3.9700E+00,-3.8800E+00,-3.7900E+00,
     &  -3.7000E+00,-3.6100E+00,-3.5200E+00,-3.4300E+00,-3.3400E+00,
     &  -3.2500E+00,-3.1600E+00,-3.0700E+00,-2.9800E+00,-2.8900E+00,
     &  -2.8000E+00,-2.7100E+00,-2.6200E+00,-2.5300E+00,-2.4400E+00,
     &  -2.3500E+00,-2.2600E+00,-2.1700E+00,-2.0800E+00,-1.9900E+00,
     &  -1.9000E+00,-1.8100E+00,-1.7200E+00,-1.6300E+00,-1.5400E+00,
     &  -1.4500E+00,-1.3600E+00,-1.2700E+00,-1.1800E+00,-1.0900E+00,
     &  -1.0000E+00,-9.9100E-01,-9.8200E-01,-9.7300E-01,-9.6400E-01,
     &  -9.5500E-01,-9.4600E-01,-9.3700E-01,-9.2800E-01,-9.1900E-01,
     &  -9.1000E-01,-9.0100E-01,-8.9200E-01,-8.8300E-01,-8.7400E-01,
     &  -8.6500E-01,-8.5600E-01,-8.4700E-01,-8.3800E-01,-8.2900E-01/

       DATA  (ETA  (I), I= 301, 400) /
     &  -8.2000E-01,-8.1100E-01,-8.0200E-01,-7.9300E-01,-7.8400E-01,
     &  -7.7500E-01,-7.6600E-01,-7.5700E-01,-7.4800E-01,-7.3900E-01,
     &  -7.3000E-01,-7.2100E-01,-7.1200E-01,-7.0300E-01,-6.9400E-01,
     &  -6.8500E-01,-6.7600E-01,-6.6700E-01,-6.5800E-01,-6.4900E-01,
     &  -6.4000E-01,-6.3100E-01,-6.2200E-01,-6.1300E-01,-6.0400E-01,
     &  -5.9500E-01,-5.8600E-01,-5.7700E-01,-5.6800E-01,-5.5900E-01,
     &  -5.5000E-01,-5.4100E-01,-5.3200E-01,-5.2300E-01,-5.1400E-01,
     &  -5.0500E-01,-4.9600E-01,-4.8700E-01,-4.7800E-01,-4.6900E-01,
     &  -4.6000E-01,-4.5100E-01,-4.4200E-01,-4.3300E-01,-4.2400E-01,
     &  -4.1500E-01,-4.0600E-01,-3.9700E-01,-3.8800E-01,-3.7900E-01,
     &  -3.7000E-01,-3.6100E-01,-3.5200E-01,-3.4300E-01,-3.3400E-01,
     &  -3.2500E-01,-3.1600E-01,-3.0700E-01,-2.9800E-01,-2.8900E-01,
     &  -2.8000E-01,-2.7100E-01,-2.6200E-01,-2.5300E-01,-2.4400E-01,
     &  -2.3500E-01,-2.2600E-01,-2.1700E-01,-2.0800E-01,-1.9900E-01,
     &  -1.9000E-01,-1.8100E-01,-1.7200E-01,-1.6300E-01,-1.5400E-01,
     &  -1.4500E-01,-1.3600E-01,-1.2700E-01,-1.1800E-01,-1.0900E-01,
     &  -1.0000E-01,-9.9100E-02,-9.8200E-02,-9.7300E-02,-9.6400E-02,
     &  -9.5500E-02,-9.4600E-02,-9.3700E-02,-9.2800E-02,-9.1900E-02,
     &  -9.1000E-02,-9.0100E-02,-8.9200E-02,-8.8300E-02,-8.7400E-02,
     &  -8.6500E-02,-8.5600E-02,-8.4700E-02,-8.3800E-02,-8.2900E-02/

       DATA  (ETA  (I), I= 401, 500) /
     &  -8.2000E-02,-8.1100E-02,-8.0200E-02,-7.9300E-02,-7.8400E-02,
     &  -7.7500E-02,-7.6600E-02,-7.5700E-02,-7.4800E-02,-7.3900E-02,
     &  -7.3000E-02,-7.2100E-02,-7.1200E-02,-7.0300E-02,-6.9400E-02,
     &  -6.8500E-02,-6.7600E-02,-6.6700E-02,-6.5800E-02,-6.4900E-02,
     &  -6.4000E-02,-6.3100E-02,-6.2200E-02,-6.1300E-02,-6.0400E-02,
     &  -5.9500E-02,-5.8600E-02,-5.7700E-02,-5.6800E-02,-5.5900E-02,
     &  -5.5000E-02,-5.4100E-02,-5.3200E-02,-5.2300E-02,-5.1400E-02,
     &  -5.0500E-02,-4.9600E-02,-4.8700E-02,-4.7800E-02,-4.6900E-02,
     &  -4.6000E-02,-4.5100E-02,-4.4200E-02,-4.3300E-02,-4.2400E-02,
     &  -4.1500E-02,-4.0600E-02,-3.9700E-02,-3.8800E-02,-3.7900E-02,
     &  -3.7000E-02,-3.6100E-02,-3.5200E-02,-3.4300E-02,-3.3400E-02,
     &  -3.2500E-02,-3.1600E-02,-3.0700E-02,-2.9800E-02,-2.8900E-02,
     &  -2.8000E-02,-2.7100E-02,-2.6200E-02,-2.5300E-02,-2.4400E-02,
     &  -2.3500E-02,-2.2600E-02,-2.1700E-02,-2.0800E-02,-1.9900E-02,
     &  -1.9000E-02,-1.8100E-02,-1.7200E-02,-1.6300E-02,-1.5400E-02,
     &  -1.4500E-02,-1.3600E-02,-1.2700E-02,-1.1800E-02,-1.0900E-02,
     &  -1.0000E-02,-9.9100E-03,-9.8200E-03,-9.7300E-03,-9.6400E-03,
     &  -9.5500E-03,-9.4600E-03,-9.3700E-03,-9.2800E-03,-9.1900E-03,
     &  -9.1000E-03,-9.0100E-03,-8.9200E-03,-8.8300E-03,-8.7400E-03,
     &  -8.6500E-03,-8.5600E-03,-8.4700E-03,-8.3800E-03,-8.2900E-03/

       DATA  (ETA  (I), I= 501, 600) /
     &  -8.2000E-03,-8.1100E-03,-8.0200E-03,-7.9300E-03,-7.8400E-03,
     &  -7.7500E-03,-7.6600E-03,-7.5700E-03,-7.4800E-03,-7.3900E-03,
     &  -7.3000E-03,-7.2100E-03,-7.1200E-03,-7.0300E-03,-6.9400E-03,
     &  -6.8500E-03,-6.7600E-03,-6.6700E-03,-6.5800E-03,-6.4900E-03,
     &  -6.4000E-03,-6.3100E-03,-6.2200E-03,-6.1300E-03,-6.0400E-03,
     &  -5.9500E-03,-5.8600E-03,-5.7700E-03,-5.6800E-03,-5.5900E-03,
     &  -5.5000E-03,-5.4100E-03,-5.3200E-03,-5.2300E-03,-5.1400E-03,
     &  -5.0500E-03,-4.9600E-03,-4.8700E-03,-4.7800E-03,-4.6900E-03,
     &  -4.6000E-03,-4.5100E-03,-4.4200E-03,-4.3300E-03,-4.2400E-03,
     &  -4.1500E-03,-4.0600E-03,-3.9700E-03,-3.8800E-03,-3.7900E-03,
     &  -3.7000E-03,-3.6100E-03,-3.5200E-03,-3.4300E-03,-3.3400E-03,
     &  -3.2500E-03,-3.1600E-03,-3.0700E-03,-2.9800E-03,-2.8900E-03,
     &  -2.8000E-03,-2.7100E-03,-2.6200E-03,-2.5300E-03,-2.4400E-03,
     &  -2.3500E-03,-2.2600E-03,-2.1700E-03,-2.0800E-03,-1.9900E-03,
     &  -1.9000E-03,-1.8100E-03,-1.7200E-03,-1.6300E-03,-1.5400E-03,
     &  -1.4500E-03,-1.3600E-03,-1.2700E-03,-1.1800E-03,-1.0900E-03,
     &  -1.0000E-03,-9.9100E-04,-9.8200E-04,-9.7300E-04,-9.6400E-04,
     &  -9.5500E-04,-9.4600E-04,-9.3700E-04,-9.2800E-04,-9.1900E-04,
     &  -9.1000E-04,-9.0100E-04,-8.9200E-04,-8.8300E-04,-8.7400E-04,
     &  -8.6500E-04,-8.5600E-04,-8.4700E-04,-8.3800E-04,-8.2900E-04/

       DATA  (ETA  (I), I= 601, 700) /
     &  -8.2000E-04,-8.1100E-04,-8.0200E-04,-7.9300E-04,-7.8400E-04,
     &  -7.7500E-04,-7.6600E-04,-7.5700E-04,-7.4800E-04,-7.3900E-04,
     &  -7.3000E-04,-7.2100E-04,-7.1200E-04,-7.0300E-04,-6.9400E-04,
     &  -6.8500E-04,-6.7600E-04,-6.6700E-04,-6.5800E-04,-6.4900E-04,
     &  -6.4000E-04,-6.3100E-04,-6.2200E-04,-6.1300E-04,-6.0400E-04,
     &  -5.9500E-04,-5.8600E-04,-5.7700E-04,-5.6800E-04,-5.5900E-04,
     &  -5.5000E-04,-5.4100E-04,-5.3200E-04,-5.2300E-04,-5.1400E-04,
     &  -5.0500E-04,-4.9600E-04,-4.8700E-04,-4.7800E-04,-4.6900E-04,
     &  -4.6000E-04,-4.5100E-04,-4.4200E-04,-4.3300E-04,-4.2400E-04,
     &  -4.1500E-04,-4.0600E-04,-3.9700E-04,-3.8800E-04,-3.7900E-04,
     &  -3.7000E-04,-3.6100E-04,-3.5200E-04,-3.4300E-04,-3.3400E-04,
     &  -3.2500E-04,-3.1600E-04,-3.0700E-04,-2.9800E-04,-2.8900E-04,
     &  -2.8000E-04,-2.7100E-04,-2.6200E-04,-2.5300E-04,-2.4400E-04,
     &  -2.3500E-04,-2.2600E-04,-2.1700E-04,-2.0800E-04,-1.9900E-04,
     &  -1.9000E-04,-1.8100E-04,-1.7200E-04,-1.6300E-04,-1.5400E-04,
     &  -1.4500E-04,-1.3600E-04,-1.2700E-04,-1.1800E-04,-1.0900E-04,
     &  -1.0000E-04,-9.9100E-05,-9.8200E-05,-9.7300E-05,-9.6400E-05,
     &  -9.5500E-05,-9.4600E-05,-9.3700E-05,-9.2800E-05,-9.1900E-05,
     &  -9.1000E-05,-9.0100E-05,-8.9200E-05,-8.8300E-05,-8.7400E-05,
     &  -8.6500E-05,-8.5600E-05,-8.4700E-05,-8.3800E-05,-8.2900E-05/

       DATA  (ETA  (I), I= 701, 800) /
     &  -8.2000E-05,-8.1100E-05,-8.0200E-05,-7.9300E-05,-7.8400E-05,
     &  -7.7500E-05,-7.6600E-05,-7.5700E-05,-7.4800E-05,-7.3900E-05,
     &  -7.3000E-05,-7.2100E-05,-7.1200E-05,-7.0300E-05,-6.9400E-05,
     &  -6.8500E-05,-6.7600E-05,-6.6700E-05,-6.5800E-05,-6.4900E-05,
     &  -6.4000E-05,-6.3100E-05,-6.2200E-05,-6.1300E-05,-6.0400E-05,
     &  -5.9500E-05,-5.8600E-05,-5.7700E-05,-5.6800E-05,-5.5900E-05,
     &  -5.5000E-05,-5.4100E-05,-5.3200E-05,-5.2300E-05,-5.1400E-05,
     &  -5.0500E-05,-4.9600E-05,-4.8700E-05,-4.7800E-05,-4.6900E-05,
     &  -4.6000E-05,-4.5100E-05,-4.4200E-05,-4.3300E-05,-4.2400E-05,
     &  -4.1500E-05,-4.0600E-05,-3.9700E-05,-3.8800E-05,-3.7900E-05,
     &  -3.7000E-05,-3.6100E-05,-3.5200E-05,-3.4300E-05,-3.3400E-05,
     &  -3.2500E-05,-3.1600E-05,-3.0700E-05,-2.9800E-05,-2.8900E-05,
     &  -2.8000E-05,-2.7100E-05,-2.6200E-05,-2.5300E-05,-2.4400E-05,
     &  -2.3500E-05,-2.2600E-05,-2.1700E-05,-2.0800E-05,-1.9900E-05,
     &  -1.9000E-05,-1.8100E-05,-1.7200E-05,-1.6300E-05,-1.5400E-05,
     &  -1.4500E-05,-1.3600E-05,-1.2700E-05,-1.1800E-05,-1.0900E-05,
     &  -1.0000E-05,-9.9100E-06,-9.8200E-06,-9.7300E-06,-9.6400E-06,
     &  -9.5500E-06,-9.4600E-06,-9.3700E-06,-9.2800E-06,-9.1900E-06,
     &  -9.1000E-06,-9.0100E-06,-8.9200E-06,-8.8300E-06,-8.7400E-06,
     &  -8.6500E-06,-8.5600E-06,-8.4700E-06,-8.3800E-06,-8.2900E-06/

       DATA  (ETA  (I), I= 801, 900) /
     &  -8.2000E-06,-8.1100E-06,-8.0200E-06,-7.9300E-06,-7.8400E-06,
     &  -7.7500E-06,-7.6600E-06,-7.5700E-06,-7.4800E-06,-7.3900E-06,
     &  -7.3000E-06,-7.2100E-06,-7.1200E-06,-7.0300E-06,-6.9400E-06,
     &  -6.8500E-06,-6.7600E-06,-6.6700E-06,-6.5800E-06,-6.4900E-06,
     &  -6.4000E-06,-6.3100E-06,-6.2200E-06,-6.1300E-06,-6.0400E-06,
     &  -5.9500E-06,-5.8600E-06,-5.7700E-06,-5.6800E-06,-5.5900E-06,
     &  -5.5000E-06,-5.4100E-06,-5.3200E-06,-5.2300E-06,-5.1400E-06,
     &  -5.0500E-06,-4.9600E-06,-4.8700E-06,-4.7800E-06,-4.6900E-06,
     &  -4.6000E-06,-4.5100E-06,-4.4200E-06,-4.3300E-06,-4.2400E-06,
     &  -4.1500E-06,-4.0600E-06,-3.9700E-06,-3.8800E-06,-3.7900E-06,
     &  -3.7000E-06,-3.6100E-06,-3.5200E-06,-3.4300E-06,-3.3400E-06,
     &  -3.2500E-06,-3.1600E-06,-3.0700E-06,-2.9800E-06,-2.8900E-06,
     &  -2.8000E-06,-2.7100E-06,-2.6200E-06,-2.5300E-06,-2.4400E-06,
     &  -2.3500E-06,-2.2600E-06,-2.1700E-06,-2.0800E-06,-1.9900E-06,
     &  -1.9000E-06,-1.8100E-06,-1.7200E-06,-1.6300E-06,-1.5400E-06,
     &  -1.4500E-06,-1.3600E-06,-1.2700E-06,-1.1800E-06,-1.0900E-06,
     &  -1.0000E-06, 0.0000E+00, 1.0000E-06, 3.0010E-03, 6.0010E-03,
     &   9.0010E-03, 1.2000E-02, 1.5000E-02, 1.8000E-02, 2.1000E-02,
     &   2.4000E-02, 2.7000E-02, 3.0000E-02, 3.3000E-02, 3.6000E-02,
     &   3.9000E-02, 4.2000E-02, 4.5000E-02, 4.8000E-02, 5.1000E-02/

       DATA  (ETA  (I), I= 901, 979) /
     &   5.4000E-02, 5.7000E-02, 6.0000E-02, 6.3000E-02, 6.6000E-02,
     &   6.9000E-02, 7.2000E-02, 7.5000E-02, 7.8000E-02, 8.1000E-02,
     &   8.4000E-02, 8.7000E-02, 9.0000E-02, 9.3000E-02, 9.6000E-02,
     &   9.9000E-02, 1.0200E-01, 1.0500E-01, 1.0800E-01, 1.1100E-01,
     &   1.1400E-01, 1.1700E-01, 1.2000E-01, 1.2300E-01, 1.2600E-01,
     &   1.2900E-01, 1.3200E-01, 1.3500E-01, 1.3800E-01, 1.4100E-01,
     &   1.4400E-01, 1.4700E-01, 1.5000E-01, 1.5300E-01, 1.5600E-01,
     &   1.5900E-01, 1.6200E-01, 1.6500E-01, 1.6800E-01, 1.7100E-01,
     &   1.7400E-01, 1.7700E-01, 1.8000E-01, 1.8300E-01, 1.8600E-01,
     &   1.8900E-01, 1.9200E-01, 1.9500E-01, 1.9800E-01, 2.0100E-01,
     &   2.0400E-01, 2.0700E-01, 2.1000E-01, 2.1300E-01, 2.1600E-01,
     &   2.1900E-01, 2.2200E-01, 2.2500E-01, 2.2800E-01, 2.3100E-01,
     &   2.3400E-01, 2.3700E-01, 2.4000E-01, 2.4300E-01, 2.4600E-01,
     &   2.4900E-01, 2.5200E-01, 2.5500E-01, 2.5800E-01, 2.6100E-01,
     &   2.6400E-01, 2.6700E-01, 2.7000E-01, 2.7100E-01, 2.7300E-01,
     &   0.27370000D0, 0.27599999D0, 0.27630001D0, 0.27700001D0/

         DATA ((DAT  (I,J), I=   1, 100), J=1,1) /
     &   4.1079E-02, 4.1027E-02, 4.0975E-02, 4.0922E-02, 4.0869E-02,
     &   4.0815E-02, 4.0761E-02, 4.0706E-02, 4.0650E-02, 4.0594E-02,
     &   4.0536E-02, 4.0479E-02, 4.0420E-02, 4.0361E-02, 4.0301E-02,
     &   4.0240E-02, 4.0178E-02, 4.0115E-02, 4.0052E-02, 3.9988E-02,
     &   3.9923E-02, 3.9857E-02, 3.9790E-02, 3.9722E-02, 3.9653E-02,
     &   3.9583E-02, 3.9512E-02, 3.9440E-02, 3.9367E-02, 3.9292E-02,
     &   3.9217E-02, 3.9140E-02, 3.9062E-02, 3.8983E-02, 3.8902E-02,
     &   3.8820E-02, 3.8737E-02, 3.8652E-02, 3.8566E-02, 3.8478E-02,
     &   3.8388E-02, 3.8297E-02, 3.8204E-02, 3.8109E-02, 3.8013E-02,
     &   3.7914E-02, 3.7814E-02, 3.7711E-02, 3.7607E-02, 3.7500E-02,
     &   3.7390E-02, 3.7279E-02, 3.7164E-02, 3.7048E-02, 3.6928E-02,
     &   3.6805E-02, 3.6680E-02, 3.6552E-02, 3.6420E-02, 3.6284E-02,
     &   3.6146E-02, 3.6003E-02, 3.5856E-02, 3.5706E-02, 3.5550E-02,
     &   3.5391E-02, 3.5226E-02, 3.5056E-02, 3.4880E-02, 3.4699E-02,
     &   3.4511E-02, 3.4317E-02, 3.4115E-02, 3.3906E-02, 3.3689E-02,
     &   3.3463E-02, 3.3227E-02, 3.2981E-02, 3.2723E-02, 3.2453E-02,
     &   3.2170E-02, 3.1871E-02, 3.1556E-02, 3.1223E-02, 3.0869E-02,
     &   3.0491E-02, 3.0087E-02, 2.9653E-02, 2.9183E-02, 2.8672E-02,
     &   2.8112E-02, 2.8053E-02, 2.7993E-02, 2.7933E-02, 2.7872E-02,
     &   2.7810E-02, 2.7748E-02, 2.7685E-02, 2.7621E-02, 2.7557E-02/

         DATA ((DAT  (I,J), I= 101, 200), J=1,1) /
     &   2.7492E-02, 2.7426E-02, 2.7359E-02, 2.7292E-02, 2.7224E-02,
     &   2.7155E-02, 2.7085E-02, 2.7015E-02, 2.6943E-02, 2.6871E-02,
     &   2.6798E-02, 2.6723E-02, 2.6648E-02, 2.6572E-02, 2.6495E-02,
     &   2.6417E-02, 2.6338E-02, 2.6258E-02, 2.6176E-02, 2.6094E-02,
     &   2.6010E-02, 2.5925E-02, 2.5839E-02, 2.5751E-02, 2.5663E-02,
     &   2.5572E-02, 2.5481E-02, 2.5388E-02, 2.5293E-02, 2.5197E-02,
     &   2.5100E-02, 2.5000E-02, 2.4899E-02, 2.4797E-02, 2.4692E-02,
     &   2.4586E-02, 2.4477E-02, 2.4367E-02, 2.4254E-02, 2.4139E-02,
     &   2.4022E-02, 2.3903E-02, 2.3781E-02, 2.3656E-02, 2.3529E-02,
     &   2.3399E-02, 2.3266E-02, 2.3130E-02, 2.2991E-02, 2.2848E-02,
     &   2.2702E-02, 2.2552E-02, 2.2399E-02, 2.2241E-02, 2.2079E-02,
     &   2.1912E-02, 2.1741E-02, 2.1564E-02, 2.1382E-02, 2.1195E-02,
     &   2.1001E-02, 2.0800E-02, 2.0593E-02, 2.0378E-02, 2.0155E-02,
     &   1.9923E-02, 1.9682E-02, 1.9431E-02, 1.9169E-02, 1.8895E-02,
     &   1.8608E-02, 1.8307E-02, 1.7989E-02, 1.7654E-02, 1.7299E-02,
     &   1.6922E-02, 1.6520E-02, 1.6090E-02, 1.5627E-02, 1.5126E-02,
     &   1.4581E-02, 1.4529E-02, 1.4477E-02, 1.4425E-02, 1.4372E-02,
     &   1.4318E-02, 1.4264E-02, 1.4210E-02, 1.4155E-02, 1.4100E-02,
     &   1.4044E-02, 1.3988E-02, 1.3931E-02, 1.3874E-02, 1.3816E-02,
     &   1.3758E-02, 1.3699E-02, 1.3639E-02, 1.3579E-02, 1.3518E-02/

         DATA ((DAT  (I,J), I= 201, 300), J=1,1) /
     &   1.3457E-02, 1.3395E-02, 1.3333E-02, 1.3269E-02, 1.3206E-02,
     &   1.3141E-02, 1.3076E-02, 1.3010E-02, 1.2944E-02, 1.2876E-02,
     &   1.2808E-02, 1.2739E-02, 1.2670E-02, 1.2599E-02, 1.2528E-02,
     &   1.2456E-02, 1.2384E-02, 1.2310E-02, 1.2235E-02, 1.2160E-02,
     &   1.2083E-02, 1.2006E-02, 1.1928E-02, 1.1849E-02, 1.1768E-02,
     &   1.1687E-02, 1.1604E-02, 1.1521E-02, 1.1436E-02, 1.1350E-02,
     &   1.1263E-02, 1.1175E-02, 1.1086E-02, 1.0995E-02, 1.0903E-02,
     &   1.0809E-02, 1.0714E-02, 1.0618E-02, 1.0520E-02, 1.0421E-02,
     &   1.0320E-02, 1.0217E-02, 1.0113E-02, 1.0007E-02, 9.8992E-03,
     &   9.7895E-03, 9.6779E-03, 9.5643E-03, 9.4486E-03, 9.3308E-03,
     &   9.2108E-03, 9.0884E-03, 8.9637E-03, 8.8331E-03, 8.7033E-03,
     &   8.5707E-03, 8.4354E-03, 8.2970E-03, 8.1556E-03, 8.0109E-03,
     &   7.8628E-03, 7.7111E-03, 7.5556E-03, 7.3961E-03, 7.2324E-03,
     &   7.0672E-03, 6.8941E-03, 6.7158E-03, 6.5319E-03, 6.3421E-03,
     &   6.1459E-03, 5.9425E-03, 5.7315E-03, 5.5121E-03, 5.2833E-03,
     &   5.0443E-03, 4.7940E-03, 4.5310E-03, 4.2539E-03, 3.9609E-03,
     &   3.6510E-03, 3.6190E-03, 3.5869E-03, 3.5545E-03, 3.5202E-03,
     &   3.4875E-03, 3.4546E-03, 3.4214E-03, 3.3881E-03, 3.3546E-03,
     &   3.3208E-03, 3.2869E-03, 3.2528E-03, 3.2185E-03, 3.1839E-03,
     &   3.1492E-03, 3.1143E-03, 3.0791E-03, 3.0438E-03, 3.0082E-03/

         DATA ((DAT  (I,J), I= 301, 400), J=1,1) /
     &   2.9725E-03, 2.9365E-03, 2.9004E-03, 2.8640E-03, 2.8274E-03,
     &   2.7907E-03, 2.7537E-03, 2.7165E-03, 2.6792E-03, 2.6416E-03,
     &   2.6039E-03, 2.5659E-03, 2.5278E-03, 2.4894E-03, 2.4509E-03,
     &   2.4122E-03, 2.3733E-03, 2.3343E-03, 2.2950E-03, 2.2556E-03,
     &   2.2161E-03, 2.1763E-03, 2.1365E-03, 2.0965E-03, 2.0563E-03,
     &   2.0161E-03, 1.9756E-03, 1.9351E-03, 1.8945E-03, 1.8537E-03,
     &   1.8129E-03, 1.7720E-03, 1.7310E-03, 1.6900E-03, 1.6489E-03,
     &   1.6078E-03, 1.5667E-03, 1.5256E-03, 1.4845E-03, 1.4434E-03,
     &   1.4023E-03, 1.3613E-03, 1.3204E-03, 1.2796E-03, 1.2389E-03,
     &   1.1984E-03, 1.1580E-03, 1.1178E-03, 1.0783E-03, 1.0385E-03,
     &   9.9893E-04, 9.5969E-04, 9.2075E-04, 8.8215E-04, 8.4391E-04,
     &   8.0608E-04, 7.6867E-04, 7.3172E-04, 6.9525E-04, 6.5931E-04,
     &   6.2393E-04, 5.8915E-04, 5.5499E-04, 5.2149E-04, 4.8870E-04,
     &   4.5664E-04, 4.2536E-04, 3.9489E-04, 3.6527E-04, 3.3654E-04,
     &   3.0874E-04, 2.8190E-04, 2.5606E-04, 2.3127E-04, 2.0755E-04,
     &   1.8495E-04, 1.6350E-04, 1.4324E-04, 1.2420E-04, 1.0640E-04,
     &   8.9897E-05, 8.8318E-05, 8.6752E-05, 8.5199E-05, 8.3660E-05,
     &   8.2134E-05, 8.0621E-05, 7.9121E-05, 7.7635E-05, 7.6162E-05,
     &   7.4702E-05, 7.3256E-05, 7.1824E-05, 7.0404E-05, 6.8999E-05,
     &   6.7606E-05, 6.6228E-05, 6.4863E-05, 6.3511E-05, 6.2173E-05/

         DATA ((DAT  (I,J), I= 401, 500), J=1,1) /
     &   6.0849E-05, 5.9539E-05, 5.8242E-05, 5.6959E-05, 5.5689E-05,
     &   5.4434E-05, 5.3192E-05, 5.1964E-05, 5.0750E-05, 4.9549E-05,
     &   4.8363E-05, 4.7190E-05, 4.6032E-05, 4.4887E-05, 4.3757E-05,
     &   4.2640E-05, 4.1537E-05, 4.0449E-05, 3.9374E-05, 3.8314E-05,
     &   3.7267E-05, 3.6235E-05, 3.5217E-05, 3.4213E-05, 3.3224E-05,
     &   3.2248E-05, 3.1287E-05, 3.0340E-05, 2.9407E-05, 2.8488E-05,
     &   2.7584E-05, 2.6694E-05, 2.5819E-05, 2.4957E-05, 2.4110E-05,
     &   2.3278E-05, 2.2460E-05, 2.1656E-05, 2.0867E-05, 2.0092E-05,
     &   1.9332E-05, 1.8586E-05, 1.7854E-05, 1.7137E-05, 1.6435E-05,
     &   1.5747E-05, 1.5074E-05, 1.4415E-05, 1.3771E-05, 1.3142E-05,
     &   1.2527E-05, 1.1926E-05, 1.1341E-05, 1.0769E-05, 1.0213E-05,
     &   9.6712E-06, 9.1441E-06, 8.6317E-06, 8.1339E-06, 7.6509E-06,
     &   7.1825E-06, 6.7289E-06, 6.2900E-06, 5.8659E-06, 5.4565E-06,
     &   5.0619E-06, 4.6820E-06, 4.3169E-06, 3.9665E-06, 3.6310E-06,
     &   3.3102E-06, 3.0042E-06, 2.7131E-06, 2.4367E-06, 2.1752E-06,
     &   1.9285E-06, 1.6966E-06, 1.4796E-06, 1.2774E-06, 1.0900E-06,
     &   9.1746E-07, 9.0102E-07, 8.8473E-07, 8.6859E-07, 8.5260E-07,
     &   8.3676E-07, 8.2107E-07, 8.0552E-07, 7.9012E-07, 7.7487E-07,
     &   7.5977E-07, 7.4482E-07, 7.3002E-07, 7.1537E-07, 7.0086E-07,
     &   6.8650E-07, 6.7229E-07, 6.5823E-07, 6.4432E-07, 6.3056E-07/

         DATA ((DAT  (I,J), I= 501, 600), J=1,1) /
     &   6.1694E-07, 6.0348E-07, 5.9016E-07, 5.7699E-07, 5.6397E-07,
     &   5.5109E-07, 5.3837E-07, 5.2580E-07, 5.1337E-07, 5.0109E-07,
     &   4.8896E-07, 4.7698E-07, 4.6515E-07, 4.5346E-07, 4.4193E-07,
     &   4.3054E-07, 4.1930E-07, 4.0821E-07, 3.9727E-07, 3.8648E-07,
     &   3.7584E-07, 3.6534E-07, 3.5500E-07, 3.4480E-07, 3.3475E-07,
     &   3.2485E-07, 3.1510E-07, 3.0549E-07, 2.9604E-07, 2.8673E-07,
     &   2.7757E-07, 2.6856E-07, 2.5970E-07, 2.5099E-07, 2.4243E-07,
     &   2.3401E-07, 2.2575E-07, 2.1763E-07, 2.0966E-07, 2.0184E-07,
     &   1.9417E-07, 1.8664E-07, 1.7927E-07, 1.7204E-07, 1.6497E-07,
     &   1.5804E-07, 1.5126E-07, 1.4463E-07, 1.3814E-07, 1.3181E-07,
     &   1.2562E-07, 1.1959E-07, 1.1370E-07, 1.0796E-07, 1.0237E-07,
     &   9.6925E-08, 9.1631E-08, 8.6486E-08, 8.1490E-08, 7.6642E-08,
     &   7.1943E-08, 6.7392E-08, 6.2990E-08, 5.8737E-08, 5.4633E-08,
     &   5.0677E-08, 4.6869E-08, 4.3211E-08, 3.9701E-08, 3.6340E-08,
     &   3.3127E-08, 3.0063E-08, 2.7148E-08, 2.4381E-08, 2.1763E-08,
     &   1.9294E-08, 1.6973E-08, 1.4801E-08, 1.2777E-08, 1.0903E-08,
     &   9.1765E-09, 9.0121E-09, 8.8491E-09, 8.6877E-09, 8.5277E-09,
     &   8.3692E-09, 8.2122E-09, 8.0567E-09, 7.9027E-09, 7.7501E-09,
     &   7.5991E-09, 7.4495E-09, 7.3014E-09, 7.1548E-09, 7.0097E-09,
     &   6.8661E-09, 6.7240E-09, 6.5833E-09, 6.4441E-09, 6.3065E-09/

         DATA ((DAT  (I,J), I= 601, 700), J=1,1) /
     &   6.1703E-09, 6.0356E-09, 5.9024E-09, 5.7706E-09, 5.6404E-09,
     &   5.5116E-09, 5.3844E-09, 5.2586E-09, 5.1343E-09, 5.0115E-09,
     &   4.8902E-09, 4.7703E-09, 4.6520E-09, 4.5351E-09, 4.4197E-09,
     &   4.3058E-09, 4.1934E-09, 4.0825E-09, 3.9731E-09, 3.8652E-09,
     &   3.7587E-09, 3.6537E-09, 3.5502E-09, 3.4482E-09, 3.3477E-09,
     &   3.2487E-09, 3.1512E-09, 3.0551E-09, 2.9606E-09, 2.8675E-09,
     &   2.7759E-09, 2.6858E-09, 2.5972E-09, 2.5100E-09, 2.4244E-09,
     &   2.3402E-09, 2.2576E-09, 2.1764E-09, 2.0967E-09, 2.0185E-09,
     &   1.9417E-09, 1.8665E-09, 1.7928E-09, 1.7205E-09, 1.6497E-09,
     &   1.5804E-09, 1.5126E-09, 1.4463E-09, 1.3815E-09, 1.3181E-09,
     &   1.2563E-09, 1.1959E-09, 1.1370E-09, 1.0796E-09, 1.0237E-09,
     &   9.6927E-10, 9.1633E-10, 8.6488E-10, 8.1491E-10, 7.6643E-10,
     &   7.1944E-10, 6.7393E-10, 6.2991E-10, 5.8738E-10, 5.4633E-10,
     &   5.0677E-10, 4.6870E-10, 4.3211E-10, 3.9701E-10, 3.6340E-10,
     &   3.3127E-10, 3.0063E-10, 2.7148E-10, 2.4381E-10, 2.1763E-10,
     &   1.9294E-10, 1.6973E-10, 1.4801E-10, 1.2777E-10, 1.0903E-10,
     &   9.1765E-11, 9.0121E-11, 8.8491E-11, 8.6877E-11, 8.5277E-11,
     &   8.3692E-11, 8.2122E-11, 8.0567E-11, 7.9027E-11, 7.7501E-11,
     &   7.5991E-11, 7.4495E-11, 7.3014E-11, 7.1548E-11, 7.0097E-11,
     &   6.8661E-11, 6.7240E-11, 6.5833E-11, 6.4442E-11, 6.3065E-11/

         DATA ((DAT  (I,J), I= 701, 800), J=1,1) /
     &   6.1703E-11, 6.0356E-11, 5.9024E-11, 5.7706E-11, 5.6404E-11,
     &   5.5116E-11, 5.3844E-11, 5.2586E-11, 5.1343E-11, 5.0115E-11,
     &   4.8902E-11, 4.7703E-11, 4.6520E-11, 4.5351E-11, 4.4197E-11,
     &   4.3058E-11, 4.1934E-11, 4.0825E-11, 3.9731E-11, 3.8652E-11,
     &   3.7587E-11, 3.6537E-11, 3.5502E-11, 3.4482E-11, 3.3477E-11,
     &   3.2487E-11, 3.1512E-11, 3.0551E-11, 2.9606E-11, 2.8675E-11,
     &   2.7759E-11, 2.6858E-11, 2.5972E-11, 2.5100E-11, 2.4244E-11,
     &   2.3402E-11, 2.2576E-11, 2.1764E-11, 2.0967E-11, 2.0185E-11,
     &   1.9418E-11, 1.8665E-11, 1.7928E-11, 1.7205E-11, 1.6497E-11,
     &   1.5804E-11, 1.5126E-11, 1.4463E-11, 1.3815E-11, 1.3181E-11,
     &   1.2563E-11, 1.1959E-11, 1.1370E-11, 1.0796E-11, 1.0237E-11,
     &   9.6927E-12, 9.1633E-12, 8.6488E-12, 8.1491E-12, 7.6643E-12,
     &   7.1944E-12, 6.7393E-12, 6.2991E-12, 5.8738E-12, 5.4633E-12,
     &   5.0677E-12, 4.6870E-12, 4.3211E-12, 3.9701E-12, 3.6340E-12,
     &   3.3127E-12, 3.0063E-12, 2.7148E-12, 2.4381E-12, 2.1763E-12,
     &   1.9294E-12, 1.6973E-12, 1.4801E-12, 1.2777E-12, 1.0903E-12,
     &   9.1765E-13, 9.0121E-13, 8.8491E-13, 8.6877E-13, 8.5277E-13,
     &   8.3692E-13, 8.2122E-13, 8.0567E-13, 7.9027E-13, 7.7501E-13,
     &   7.5991E-13, 7.4495E-13, 7.3014E-13, 7.1548E-13, 7.0097E-13,
     &   6.8661E-13, 6.7240E-13, 6.5833E-13, 6.4442E-13, 6.3065E-13/

         DATA ((DAT  (I,J), I= 801, 900), J=1,1) /
     &   6.1703E-13, 6.0356E-13, 5.9024E-13, 5.7706E-13, 5.6404E-13,
     &   5.5116E-13, 5.3844E-13, 5.2586E-13, 5.1343E-13, 5.0115E-13,
     &   4.8902E-13, 4.7703E-13, 4.6520E-13, 4.5351E-13, 4.4197E-13,
     &   4.3058E-13, 4.1934E-13, 4.0825E-13, 3.9731E-13, 3.8652E-13,
     &   3.7587E-13, 3.6537E-13, 3.5502E-13, 3.4482E-13, 3.3477E-13,
     &   3.2487E-13, 3.1512E-13, 3.0551E-13, 2.9606E-13, 2.8675E-13,
     &   2.7759E-13, 2.6858E-13, 2.5972E-13, 2.5100E-13, 2.4244E-13,
     &   2.3402E-13, 2.2576E-13, 2.1764E-13, 2.0967E-13, 2.0185E-13,
     &   1.9418E-13, 1.8665E-13, 1.7928E-13, 1.7205E-13, 1.6497E-13,
     &   1.5804E-13, 1.5126E-13, 1.4463E-13, 1.3815E-13, 1.3181E-13,
     &   1.2563E-13, 1.1959E-13, 1.1370E-13, 1.0796E-13, 1.0237E-13,
     &   9.6927E-14, 9.1633E-14, 8.6488E-14, 8.1491E-14, 7.6643E-14,
     &   7.1944E-14, 6.7393E-14, 6.2991E-14, 5.8738E-14, 5.4633E-14,
     &   5.0677E-14, 4.6870E-14, 4.3211E-14, 3.9701E-14, 3.6340E-14,
     &   3.3127E-14, 3.0063E-14, 2.7148E-14, 2.4381E-14, 2.1763E-14,
     &   1.9294E-14, 1.6973E-14, 1.4801E-14, 1.2777E-14, 1.0903E-14,
     &   9.1765E-15, 0.0000E+00,-9.1765E-15,-8.2645E-08,-3.3049E-07,
     &  -7.4359E-07,-1.3218E-06,-2.0657E-06,-2.9752E-06,-4.0506E-06,
     &  -5.2921E-06,-6.7000E-06,-8.2746E-06,-1.0016E-05,-1.1925E-05,
     &  -1.4002E-05,-1.6248E-05,-1.8662E-05,-2.1246E-05,-2.4000E-05/

         DATA ((DAT  (I,J), I= 901, 979), J=1,1) /
     &  -2.6925E-05,-3.0021E-05,-3.3289E-05,-3.6730E-05,-4.0345E-05,
     &  -4.4134E-05,-4.8099E-05,-5.2241E-05,-5.6560E-05,-6.1058E-05,
     &  -6.5735E-05,-7.0593E-05,-7.5633E-05,-8.0857E-05,-8.6265E-05,
     &  -9.1860E-05,-9.7641E-05,-1.0361E-04,-1.0977E-04,-1.1613E-04,
     &  -1.2268E-04,-1.2942E-04,-1.3636E-04,-1.4350E-04,-1.5084E-04,
     &  -1.5839E-04,-1.6614E-04,-1.7410E-04,-1.8227E-04,-1.9065E-04,
     &  -1.9925E-04,-2.0806E-04,-2.1710E-04,-2.2636E-04,-2.3585E-04,
     &  -2.4557E-04,-2.5552E-04,-2.6571E-04,-2.7614E-04,-2.8681E-04,
     &  -2.9762E-04,-3.0878E-04,-3.2021E-04,-3.3189E-04,-3.4384E-04,
     &  -3.5606E-04,-3.6856E-04,-3.8133E-04,-3.9440E-04,-4.0776E-04,
     &  -4.2141E-04,-4.3538E-04,-4.4965E-04,-4.6425E-04,-4.7918E-04,
     &  -4.9446E-04,-5.1006E-04,-5.2604E-04,-5.4240E-04,-5.5915E-04,
     &  -5.7629E-04,-5.9384E-04,-6.1182E-04,-6.3026E-04,-6.4916E-04,
     &  -6.6856E-04,-6.8849E-04,-7.0898E-04,-7.3007E-04,-7.5183E-04,
     &  -7.7429E-04,-7.9758E-04,-8.2183E-04,-8.3015E-04,-8.4724E-04,
     &  -8.53372657E-04,-8.74228401E-04,-8.77053061E-04,-8.83763141E-04/

         DATA ((DAT  (I,J), I=   1, 100), J=2,2) /
     &   5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05,
     &   5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05,
     &   5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05,
     &   5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05,
     &   5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05,
     &   5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3366E-05,
     &   5.3366E-05, 5.3366E-05, 5.3366E-05, 5.3366E-05, 5.3366E-05,
     &   5.3366E-05, 5.3366E-05, 5.3366E-05, 5.3366E-05, 5.3366E-05,
     &   5.3366E-05, 5.3366E-05, 5.3366E-05, 5.3366E-05, 5.3366E-05,
     &   5.3366E-05, 5.3366E-05, 5.3365E-05, 5.3365E-05, 5.3365E-05,
     &   5.3365E-05, 5.3365E-05, 5.3365E-05, 5.3365E-05, 5.3365E-05,
     &   5.3365E-05, 5.3364E-05, 5.3364E-05, 5.3364E-05, 5.3364E-05,
     &   5.3364E-05, 5.3363E-05, 5.3363E-05, 5.3363E-05, 5.3363E-05,
     &   5.3362E-05, 5.3362E-05, 5.3362E-05, 5.3361E-05, 5.3361E-05,
     &   5.3360E-05, 5.3360E-05, 5.3359E-05, 5.3359E-05, 5.3358E-05,
     &   5.3357E-05, 5.3356E-05, 5.3355E-05, 5.3354E-05, 5.3353E-05,
     &   5.3351E-05, 5.3349E-05, 5.3347E-05, 5.3345E-05, 5.3342E-05,
     &   5.3338E-05, 5.3334E-05, 5.3329E-05, 5.3322E-05, 5.3313E-05,
     &   5.3302E-05, 5.3300E-05, 5.3299E-05, 5.3297E-05, 5.3296E-05,
     &   5.3294E-05, 5.3293E-05, 5.3291E-05, 5.3290E-05, 5.3288E-05/

         DATA ((DAT  (I,J), I= 101, 200), J=2,2) /
     &   5.3286E-05, 5.3284E-05, 5.3282E-05, 5.3280E-05, 5.3278E-05,
     &   5.3276E-05, 5.3274E-05, 5.3272E-05, 5.3269E-05, 5.3267E-05,
     &   5.3264E-05, 5.3262E-05, 5.3259E-05, 5.3256E-05, 5.3253E-05,
     &   5.3250E-05, 5.3247E-05, 5.3244E-05, 5.3240E-05, 5.3237E-05,
     &   5.3233E-05, 5.3229E-05, 5.3225E-05, 5.3221E-05, 5.3216E-05,
     &   5.3211E-05, 5.3207E-05, 5.3201E-05, 5.3196E-05, 5.3190E-05,
     &   5.3185E-05, 5.3178E-05, 5.3172E-05, 5.3165E-05, 5.3158E-05,
     &   5.3150E-05, 5.3142E-05, 5.3133E-05, 5.3124E-05, 5.3115E-05,
     &   5.3105E-05, 5.3094E-05, 5.3083E-05, 5.3070E-05, 5.3057E-05,
     &   5.3044E-05, 5.3029E-05, 5.3013E-05, 5.2996E-05, 5.2978E-05,
     &   5.2959E-05, 5.2938E-05, 5.2915E-05, 5.2891E-05, 5.2865E-05,
     &   5.2836E-05, 5.2805E-05, 5.2771E-05, 5.2734E-05, 5.2694E-05,
     &   5.2649E-05, 5.2600E-05, 5.2546E-05, 5.2485E-05, 5.2419E-05,
     &   5.2344E-05, 5.2260E-05, 5.2165E-05, 5.2058E-05, 5.1936E-05,
     &   5.1797E-05, 5.1636E-05, 5.1450E-05, 5.1233E-05, 5.0977E-05,
     &   5.0673E-05, 5.0309E-05, 4.9866E-05, 4.9323E-05, 4.8647E-05,
     &   4.7791E-05, 4.7704E-05, 4.7614E-05, 4.7522E-05, 4.7428E-05,
     &   4.7331E-05, 4.7233E-05, 4.7132E-05, 4.7028E-05, 4.6922E-05,
     &   4.6814E-05, 4.6703E-05, 4.6589E-05, 4.6472E-05, 4.6353E-05,
     &   4.6230E-05, 4.6104E-05, 4.5975E-05, 4.5843E-05, 4.5708E-05/

         DATA ((DAT  (I,J), I= 201, 300), J=2,2) /
     &   4.5568E-05, 4.5426E-05, 4.5279E-05, 4.5128E-05, 4.4974E-05,
     &   4.4815E-05, 4.4652E-05, 4.4485E-05, 4.4313E-05, 4.4136E-05,
     &   4.3954E-05, 4.3767E-05, 4.3575E-05, 4.3378E-05, 4.3174E-05,
     &   4.2965E-05, 4.2750E-05, 4.2529E-05, 4.2301E-05, 4.2067E-05,
     &   4.1825E-05, 4.1576E-05, 4.1320E-05, 4.1056E-05, 4.0785E-05,
     &   4.0505E-05, 4.0216E-05, 3.9918E-05, 3.9612E-05, 3.9296E-05,
     &   3.8970E-05, 3.8633E-05, 3.8287E-05, 3.7929E-05, 3.7560E-05,
     &   3.7180E-05, 3.6787E-05, 3.6382E-05, 3.5964E-05, 3.5533E-05,
     &   3.5088E-05, 3.4630E-05, 3.4156E-05, 3.3668E-05, 3.3164E-05,
     &   3.2645E-05, 3.2110E-05, 3.1557E-05, 3.0989E-05, 3.0402E-05,
     &   2.9798E-05, 2.9177E-05, 2.8537E-05, 2.7878E-05, 2.7202E-05,
     &   2.6506E-05, 2.5792E-05, 2.5060E-05, 2.4309E-05, 2.3540E-05,
     &   2.2753E-05, 2.1949E-05, 2.1129E-05, 2.0293E-05, 1.9442E-05,
     &   1.8579E-05, 1.7704E-05, 1.6819E-05, 1.5926E-05, 1.5027E-05,
     &   1.4125E-05, 1.3223E-05, 1.2323E-05, 1.1430E-05, 1.0545E-05,
     &   9.6737E-06, 8.8186E-06, 7.9837E-06, 7.1722E-06, 6.3873E-06,
     &   5.6314E-06, 5.5575E-06, 5.4839E-06, 5.4106E-06, 5.3376E-06,
     &   5.2649E-06, 5.1926E-06, 5.1206E-06, 5.0489E-06, 4.9775E-06,
     &   4.9064E-06, 4.8357E-06, 4.7652E-06, 4.6951E-06, 4.6253E-06,
     &   4.5559E-06, 4.4868E-06, 4.4179E-06, 4.3495E-06, 4.2813E-06/

         DATA ((DAT  (I,J), I= 301, 400), J=2,2) /
     &   4.2135E-06, 4.1459E-06, 4.0787E-06, 4.0119E-06, 3.9453E-06,
     &   3.8791E-06, 3.8132E-06, 3.7476E-06, 3.6824E-06, 3.6174E-06,
     &   3.5528E-06, 3.4886E-06, 3.4246E-06, 3.3609E-06, 3.2976E-06,
     &   3.2346E-06, 3.1719E-06, 3.1096E-06, 3.0475E-06, 2.9858E-06,
     &   2.9244E-06, 2.8633E-06, 2.8026E-06, 2.7421E-06, 2.6820E-06,
     &   2.6222E-06, 2.5628E-06, 2.5036E-06, 2.4448E-06, 2.3864E-06,
     &   2.3282E-06, 2.2704E-06, 2.2130E-06, 2.1559E-06, 2.0991E-06,
     &   2.0427E-06, 1.9866E-06, 1.9309E-06, 1.8756E-06, 1.8207E-06,
     &   1.7661E-06, 1.7120E-06, 1.6582E-06, 1.6049E-06, 1.5519E-06,
     &   1.4995E-06, 1.4474E-06, 1.3958E-06, 1.3447E-06, 1.2941E-06,
     &   1.2440E-06, 1.1945E-06, 1.1455E-06, 1.0970E-06, 1.0491E-06,
     &   1.0019E-06, 9.5524E-07, 9.0928E-07, 8.6401E-07, 8.1945E-07,
     &   7.7564E-07, 7.3260E-07, 6.9037E-07, 6.4899E-07, 6.0849E-07,
     &   5.6890E-07, 5.3026E-07, 4.9262E-07, 4.5602E-07, 4.2048E-07,
     &   3.8607E-07, 3.5282E-07, 3.2078E-07, 2.9000E-07, 2.6051E-07,
     &   2.3237E-07, 2.0562E-07, 1.8031E-07, 1.5649E-07, 1.3420E-07,
     &   1.1348E-07, 1.1150E-07, 1.0953E-07, 1.0758E-07, 1.0564E-07,
     &   1.0373E-07, 1.0182E-07, 9.9938E-08, 9.8069E-08, 9.6217E-08,
     &   9.4381E-08, 9.2562E-08, 9.0759E-08, 8.8973E-08, 8.7204E-08,
     &   8.5451E-08, 8.3715E-08, 8.1996E-08, 8.0294E-08, 7.8609E-08/

         DATA ((DAT  (I,J), I= 401, 500), J=2,2) /
     &   7.6941E-08, 7.5290E-08, 7.3656E-08, 7.2039E-08, 7.0438E-08,
     &   6.8856E-08, 6.7290E-08, 6.5741E-08, 6.4210E-08, 6.2696E-08,
     &   6.1199E-08, 5.9720E-08, 5.8258E-08, 5.6813E-08, 5.5386E-08,
     &   5.3976E-08, 5.2584E-08, 5.1210E-08, 4.9853E-08, 4.8513E-08,
     &   4.7191E-08, 4.5887E-08, 4.4601E-08, 4.3332E-08, 4.2081E-08,
     &   4.0848E-08, 3.9633E-08, 3.8436E-08, 3.7256E-08, 3.6095E-08,
     &   3.4951E-08, 3.3825E-08, 3.2718E-08, 3.1628E-08, 3.0556E-08,
     &   2.9503E-08, 2.8468E-08, 2.7450E-08, 2.6451E-08, 2.5470E-08,
     &   2.4508E-08, 2.3563E-08, 2.2637E-08, 2.1729E-08, 2.0840E-08,
     &   1.9968E-08, 1.9116E-08, 1.8281E-08, 1.7465E-08, 1.6667E-08,
     &   1.5888E-08, 1.5127E-08, 1.4385E-08, 1.3661E-08, 1.2955E-08,
     &   1.2269E-08, 1.1600E-08, 1.0951E-08, 1.0320E-08, 9.7070E-09,
     &   9.1131E-09, 8.5378E-09, 7.9812E-09, 7.4432E-09, 6.9239E-09,
     &   6.4233E-09, 5.9414E-09, 5.4782E-09, 5.0337E-09, 4.6080E-09,
     &   4.2010E-09, 3.8128E-09, 3.4434E-09, 3.0927E-09, 2.7608E-09,
     &   2.4477E-09, 2.1535E-09, 1.8780E-09, 1.6213E-09, 1.3835E-09,
     &   1.1646E-09, 1.1437E-09, 1.1230E-09, 1.1025E-09, 1.0822E-09,
     &   1.0621E-09, 1.0422E-09, 1.0225E-09, 1.0029E-09, 9.8358E-10,
     &   9.6441E-10, 9.4543E-10, 9.2664E-10, 9.0804E-10, 8.8963E-10,
     &   8.7141E-10, 8.5337E-10, 8.3552E-10, 8.1787E-10, 8.0040E-10/

         DATA ((DAT  (I,J), I= 501, 600), J=2,2) /
     &   7.8311E-10, 7.6602E-10, 7.4912E-10, 7.3240E-10, 7.1587E-10,
     &   6.9954E-10, 6.8338E-10, 6.6742E-10, 6.5165E-10, 6.3607E-10,
     &   6.2067E-10, 6.0546E-10, 5.9044E-10, 5.7561E-10, 5.6097E-10,
     &   5.4652E-10, 5.3225E-10, 5.1817E-10, 5.0429E-10, 4.9059E-10,
     &   4.7708E-10, 4.6376E-10, 4.5062E-10, 4.3768E-10, 4.2492E-10,
     &   4.1235E-10, 3.9997E-10, 3.8778E-10, 3.7578E-10, 3.6397E-10,
     &   3.5234E-10, 3.4091E-10, 3.2966E-10, 3.1860E-10, 3.0773E-10,
     &   2.9705E-10, 2.8656E-10, 2.7625E-10, 2.6614E-10, 2.5621E-10,
     &   2.4647E-10, 2.3692E-10, 2.2756E-10, 2.1839E-10, 2.0941E-10,
     &   2.0061E-10, 1.9200E-10, 1.8359E-10, 1.7536E-10, 1.6732E-10,
     &   1.5946E-10, 1.5180E-10, 1.4433E-10, 1.3704E-10, 1.2994E-10,
     &   1.2304E-10, 1.1632E-10, 1.0979E-10, 1.0344E-10, 9.7289E-11,
     &   9.1324E-11, 8.5548E-11, 7.9960E-11, 7.4561E-11, 6.9351E-11,
     &   6.4329E-11, 5.9496E-11, 5.4852E-11, 5.0396E-11, 4.6130E-11,
     &   4.2051E-11, 3.8162E-11, 3.4461E-11, 3.0949E-11, 2.7626E-11,
     &   2.4491E-11, 2.1545E-11, 1.8788E-11, 1.6220E-11, 1.3840E-11,
     &   1.1649E-11, 1.1440E-11, 1.1233E-11, 1.1028E-11, 1.0825E-11,
     &   1.0624E-11, 1.0425E-11, 1.0227E-11, 1.0032E-11, 9.8380E-12,
     &   9.6463E-12, 9.4564E-12, 9.2684E-12, 9.0823E-12, 8.8981E-12,
     &   8.7158E-12, 8.5354E-12, 8.3569E-12, 8.1802E-12, 8.0055E-12/

         DATA ((DAT  (I,J), I= 601, 700), J=2,2) /
     &   7.8326E-12, 7.6616E-12, 7.4925E-12, 7.3253E-12, 7.1599E-12,
     &   6.9965E-12, 6.8349E-12, 6.6753E-12, 6.5175E-12, 6.3616E-12,
     &   6.2076E-12, 6.0555E-12, 5.9052E-12, 5.7569E-12, 5.6104E-12,
     &   5.4659E-12, 5.3232E-12, 5.1824E-12, 5.0435E-12, 4.9064E-12,
     &   4.7713E-12, 4.6381E-12, 4.5067E-12, 4.3772E-12, 4.2496E-12,
     &   4.1239E-12, 4.0001E-12, 3.8782E-12, 3.7581E-12, 3.6400E-12,
     &   3.5237E-12, 3.4094E-12, 3.2969E-12, 3.1863E-12, 3.0775E-12,
     &   2.9707E-12, 2.8658E-12, 2.7627E-12, 2.6615E-12, 2.5623E-12,
     &   2.4649E-12, 2.3694E-12, 2.2757E-12, 2.1840E-12, 2.0942E-12,
     &   2.0062E-12, 1.9201E-12, 1.8359E-12, 1.7536E-12, 1.6732E-12,
     &   1.5947E-12, 1.5181E-12, 1.4433E-12, 1.3705E-12, 1.2995E-12,
     &   1.2304E-12, 1.1632E-12, 1.0979E-12, 1.0345E-12, 9.7291E-13,
     &   9.1326E-13, 8.5549E-13, 7.9961E-13, 7.4562E-13, 6.9352E-13,
     &   6.4330E-13, 5.9497E-13, 5.4853E-13, 5.0397E-13, 4.6130E-13,
     &   4.2052E-13, 3.8162E-13, 3.4462E-13, 3.0949E-13, 2.7626E-13,
     &   2.4491E-13, 2.1545E-13, 1.8788E-13, 1.6220E-13, 1.3840E-13,
     &   1.1649E-13, 1.1440E-13, 1.1233E-13, 1.1028E-13, 1.0825E-13,
     &   1.0624E-13, 1.0425E-13, 1.0227E-13, 1.0032E-13, 9.8380E-14,
     &   9.6463E-14, 9.4564E-14, 9.2685E-14, 9.0824E-14, 8.8982E-14,
     &   8.7158E-14, 8.5354E-14, 8.3569E-14, 8.1802E-14, 8.0055E-14/

         DATA ((DAT  (I,J), I= 701, 800), J=2,2) /
     &   7.8326E-14, 7.6616E-14, 7.4925E-14, 7.3253E-14, 7.1599E-14,
     &   6.9965E-14, 6.8349E-14, 6.6753E-14, 6.5175E-14, 6.3616E-14,
     &   6.2076E-14, 6.0555E-14, 5.9052E-14, 5.7569E-14, 5.6104E-14,
     &   5.4659E-14, 5.3232E-14, 5.1824E-14, 5.0435E-14, 4.9064E-14,
     &   4.7713E-14, 4.6381E-14, 4.5067E-14, 4.3772E-14, 4.2496E-14,
     &   4.1239E-14, 4.0001E-14, 3.8782E-14, 3.7582E-14, 3.6400E-14,
     &   3.5237E-14, 3.4094E-14, 3.2969E-14, 3.1863E-14, 3.0775E-14,
     &   2.9707E-14, 2.8658E-14, 2.7627E-14, 2.6615E-14, 2.5623E-14,
     &   2.4649E-14, 2.3694E-14, 2.2757E-14, 2.1840E-14, 2.0942E-14,
     &   2.0062E-14, 1.9201E-14, 1.8359E-14, 1.7536E-14, 1.6732E-14,
     &   1.5947E-14, 1.5181E-14, 1.4433E-14, 1.3705E-14, 1.2995E-14,
     &   1.2304E-14, 1.1632E-14, 1.0979E-14, 1.0345E-14, 9.7291E-15,
     &   9.1326E-15, 8.5549E-15, 7.9961E-15, 7.4562E-15, 6.9352E-15,
     &   6.4330E-15, 5.9497E-15, 5.4853E-15, 5.0397E-15, 4.6130E-15,
     &   4.2052E-15, 3.8162E-15, 3.4462E-15, 3.0949E-15, 2.7626E-15,
     &   2.4491E-15, 2.1545E-15, 1.8788E-15, 1.6220E-15, 1.3840E-15,
     &   1.1649E-15, 1.1440E-15, 1.1233E-15, 1.1028E-15, 1.0825E-15,
     &   1.0624E-15, 1.0425E-15, 1.0227E-15, 1.0032E-15, 9.8380E-16,
     &   9.6463E-16, 9.4564E-16, 9.2685E-16, 9.0824E-16, 8.8982E-16,
     &   8.7159E-16, 8.5354E-16, 8.3569E-16, 8.1802E-16, 8.0055E-16/

         DATA ((DAT  (I,J), I= 801, 900), J=2,2) /
     &   7.8326E-16, 7.6616E-16, 7.4925E-16, 7.3253E-16, 7.1599E-16,
     &   6.9965E-16, 6.8349E-16, 6.6753E-16, 6.5175E-16, 6.3616E-16,
     &   6.2076E-16, 6.0555E-16, 5.9052E-16, 5.7569E-16, 5.6104E-16,
     &   5.4659E-16, 5.3232E-16, 5.1824E-16, 5.0435E-16, 4.9064E-16,
     &   4.7713E-16, 4.6381E-16, 4.5067E-16, 4.3772E-16, 4.2496E-16,
     &   4.1239E-16, 4.0001E-16, 3.8782E-16, 3.7582E-16, 3.6400E-16,
     &   3.5237E-16, 3.4094E-16, 3.2969E-16, 3.1863E-16, 3.0775E-16,
     &   2.9707E-16, 2.8658E-16, 2.7627E-16, 2.6615E-16, 2.5623E-16,
     &   2.4649E-16, 2.3694E-16, 2.2757E-16, 2.1840E-16, 2.0942E-16,
     &   2.0062E-16, 1.9201E-16, 1.8359E-16, 1.7536E-16, 1.6732E-16,
     &   1.5947E-16, 1.5181E-16, 1.4433E-16, 1.3705E-16, 1.2995E-16,
     &   1.2304E-16, 1.1632E-16, 1.0979E-16, 1.0345E-16, 9.7291E-17,
     &   9.1326E-17, 8.5549E-17, 7.9961E-17, 7.4562E-17, 6.9352E-17,
     &   6.4330E-17, 5.9497E-17, 5.4853E-17, 5.0397E-17, 4.6130E-17,
     &   4.2052E-17, 3.8162E-17, 3.4462E-17, 3.0949E-17, 2.7626E-17,
     &   2.4491E-17, 2.1545E-17, 1.8788E-17, 1.6220E-17, 1.3840E-17,
     &   1.1649E-17, 0.0000E+00, 1.1649E-17, 1.0491E-10, 4.1953E-10,
     &   9.4396E-10, 1.6781E-09, 2.6226E-09, 3.7775E-09, 5.1433E-09,
     &   6.7203E-09, 8.5089E-09, 1.0510E-08, 1.2723E-08, 1.5151E-08,
     &   1.7792E-08, 2.0649E-08, 2.3721E-08, 2.7010E-08, 3.0517E-08/

         DATA ((DAT  (I,J), I= 901, 979), J=2,2) /
     &   3.4244E-08, 3.8190E-08, 4.2358E-08, 4.6748E-08, 5.1363E-08,
     &   5.6204E-08, 6.1272E-08, 6.6569E-08, 7.2096E-08, 7.7857E-08,
     &   8.3852E-08, 9.0085E-08, 9.6556E-08, 1.0327E-07, 1.1023E-07,
     &   1.1743E-07, 1.2488E-07, 1.3259E-07, 1.4055E-07, 1.4877E-07,
     &   1.5725E-07, 1.6600E-07, 1.7501E-07, 1.8430E-07, 1.9387E-07,
     &   2.0372E-07, 2.1385E-07, 2.2428E-07, 2.3500E-07, 2.4603E-07,
     &   2.5736E-07, 2.6901E-07, 2.8098E-07, 2.9328E-07, 3.0592E-07,
     &   3.1890E-07, 3.3223E-07, 3.4592E-07, 3.5998E-07, 3.7443E-07,
     &   3.8926E-07, 4.0451E-07, 4.2017E-07, 4.3627E-07, 4.5281E-07,
     &   4.6982E-07, 4.8731E-07, 5.0531E-07, 5.2383E-07, 5.4290E-07,
     &   5.6255E-07, 5.8281E-07, 6.0370E-07, 6.2528E-07, 6.4758E-07,
     &   6.7064E-07, 6.9453E-07, 7.1930E-07, 7.4503E-07, 7.7181E-07,
     &   7.9972E-07, 8.2889E-07, 8.5945E-07, 8.9159E-07, 9.2549E-07,
     &   9.6142E-07, 9.9971E-07, 1.0408E-06, 1.0852E-06, 1.1337E-06,
     &   1.1873E-06, 1.2478E-06, 1.3175E-06, 1.3435E-06, 1.4012E-06,
     &   1.42363451E-06, 1.50912693E-06, 1.52211932E-06, 1.55487350E-06/

         DATA ((DAT  (I,J), I=   1, 100), J=3,3) /
     &   9.4556E-05, 9.4556E-05, 9.4555E-05, 9.4555E-05, 9.4555E-05,
     &   9.4555E-05, 9.4555E-05, 9.4554E-05, 9.4554E-05, 9.4554E-05,
     &   9.4554E-05, 9.4553E-05, 9.4553E-05, 9.4553E-05, 9.4552E-05,
     &   9.4552E-05, 9.4552E-05, 9.4552E-05, 9.4551E-05, 9.4551E-05,
     &   9.4551E-05, 9.4550E-05, 9.4550E-05, 9.4550E-05, 9.4549E-05,
     &   9.4549E-05, 9.4549E-05, 9.4548E-05, 9.4548E-05, 9.4548E-05,
     &   9.4547E-05, 9.4547E-05, 9.4546E-05, 9.4546E-05, 9.4546E-05,
     &   9.4545E-05, 9.4545E-05, 9.4544E-05, 9.4544E-05, 9.4543E-05,
     &   9.4543E-05, 9.4542E-05, 9.4542E-05, 9.4541E-05, 9.4541E-05,
     &   9.4540E-05, 9.4540E-05, 9.4539E-05, 9.4539E-05, 9.4538E-05,
     &   9.4537E-05, 9.4537E-05, 9.4536E-05, 9.4535E-05, 9.4535E-05,
     &   9.4534E-05, 9.4533E-05, 9.4532E-05, 9.4531E-05, 9.4530E-05,
     &   9.4529E-05, 9.4528E-05, 9.4527E-05, 9.4526E-05, 9.4525E-05,
     &   9.4524E-05, 9.4523E-05, 9.4521E-05, 9.4520E-05, 9.4519E-05,
     &   9.4517E-05, 9.4515E-05, 9.4513E-05, 9.4511E-05, 9.4509E-05,
     &   9.4507E-05, 9.4504E-05, 9.4502E-05, 9.4499E-05, 9.4495E-05,
     &   9.4491E-05, 9.4487E-05, 9.4482E-05, 9.4477E-05, 9.4470E-05,
     &   9.4463E-05, 9.4454E-05, 9.4443E-05, 9.4430E-05, 9.4413E-05,
     &   9.4392E-05, 9.4390E-05, 9.4387E-05, 9.4385E-05, 9.4382E-05,
     &   9.4379E-05, 9.4377E-05, 9.4374E-05, 9.4371E-05, 9.4368E-05/

         DATA ((DAT  (I,J), I= 101, 200), J=3,3) /
     &   9.4364E-05, 9.4361E-05, 9.4358E-05, 9.4354E-05, 9.4351E-05,
     &   9.4347E-05, 9.4343E-05, 9.4339E-05, 9.4335E-05, 9.4331E-05,
     &   9.4327E-05, 9.4322E-05, 9.4317E-05, 9.4312E-05, 9.4307E-05,
     &   9.4302E-05, 9.4297E-05, 9.4291E-05, 9.4285E-05, 9.4279E-05,
     &   9.4273E-05, 9.4266E-05, 9.4259E-05, 9.4252E-05, 9.4244E-05,
     &   9.4237E-05, 9.4228E-05, 9.4220E-05, 9.4211E-05, 9.4201E-05,
     &   9.4192E-05, 9.4181E-05, 9.4171E-05, 9.4159E-05, 9.4147E-05,
     &   9.4135E-05, 9.4122E-05, 9.4108E-05, 9.4093E-05, 9.4078E-05,
     &   9.4061E-05, 9.4044E-05, 9.4026E-05, 9.4006E-05, 9.3986E-05,
     &   9.3964E-05, 9.3940E-05, 9.3915E-05, 9.3889E-05, 9.3861E-05,
     &   9.3830E-05, 9.3798E-05, 9.3762E-05, 9.3725E-05, 9.3684E-05,
     &   9.3640E-05, 9.3593E-05, 9.3541E-05, 9.3486E-05, 9.3425E-05,
     &   9.3358E-05, 9.3285E-05, 9.3206E-05, 9.3118E-05, 9.3021E-05,
     &   9.2913E-05, 9.2794E-05, 9.2660E-05, 9.2511E-05, 9.2343E-05,
     &   9.2154E-05, 9.1938E-05, 9.1692E-05, 9.1410E-05, 9.1083E-05,
     &   9.0701E-05, 9.0253E-05, 8.9721E-05, 8.9082E-05, 8.8305E-05,
     &   8.7344E-05, 8.7247E-05, 8.7147E-05, 8.7046E-05, 8.6942E-05,
     &   8.6836E-05, 8.6728E-05, 8.6617E-05, 8.6504E-05, 8.6388E-05,
     &   8.6270E-05, 8.6149E-05, 8.6026E-05, 8.5900E-05, 8.5770E-05,
     &   8.5638E-05, 8.5503E-05, 8.5364E-05, 8.5222E-05, 8.5077E-05/

         DATA ((DAT  (I,J), I= 201, 300), J=3,3) /
     &   8.4928E-05, 8.4776E-05, 8.4620E-05, 8.4460E-05, 8.4296E-05,
     &   8.4128E-05, 8.3955E-05, 8.3778E-05, 8.3597E-05, 8.3411E-05,
     &   8.3219E-05, 8.3023E-05, 8.2822E-05, 8.2615E-05, 8.2403E-05,
     &   8.2184E-05, 8.1960E-05, 8.1729E-05, 8.1492E-05, 8.1248E-05,
     &   8.0997E-05, 8.0739E-05, 8.0473E-05, 8.0200E-05, 7.9918E-05,
     &   7.9628E-05, 7.9329E-05, 7.9021E-05, 7.8703E-05, 7.8376E-05,
     &   7.8038E-05, 7.7690E-05, 7.7330E-05, 7.6959E-05, 7.6576E-05,
     &   7.6180E-05, 7.5771E-05, 7.5348E-05, 7.4911E-05, 7.4459E-05,
     &   7.3991E-05, 7.3507E-05, 7.3006E-05, 7.2486E-05, 7.1949E-05,
     &   7.1391E-05, 7.0813E-05, 7.0213E-05, 6.9590E-05, 6.8943E-05,
     &   6.8271E-05, 6.7573E-05, 6.6847E-05, 6.6091E-05, 6.5303E-05,
     &   6.4483E-05, 6.3628E-05, 6.2735E-05, 6.1803E-05, 6.0830E-05,
     &   5.9812E-05, 5.8746E-05, 5.7631E-05, 5.6461E-05, 5.5234E-05,
     &   5.3946E-05, 5.2593E-05, 5.1171E-05, 4.9675E-05, 4.8101E-05,
     &   4.6443E-05, 4.4698E-05, 4.2860E-05, 4.0925E-05, 3.8889E-05,
     &   3.6750E-05, 3.4506E-05, 3.2155E-05, 2.9699E-05, 2.7143E-05,
     &   2.4495E-05, 2.4225E-05, 2.3955E-05, 2.3684E-05, 2.3412E-05,
     &   2.3139E-05, 2.2866E-05, 2.2592E-05, 2.2317E-05, 2.2042E-05,
     &   2.1766E-05, 2.1489E-05, 2.1212E-05, 2.0934E-05, 2.0656E-05,
     &   2.0377E-05, 2.0098E-05, 1.9818E-05, 1.9537E-05, 1.9257E-05/

         DATA ((DAT  (I,J), I= 301, 400), J=3,3) /
     &   1.8975E-05, 1.8694E-05, 1.8412E-05, 1.8130E-05, 1.7847E-05,
     &   1.7565E-05, 1.7282E-05, 1.6999E-05, 1.6716E-05, 1.6432E-05,
     &   1.6149E-05, 1.5866E-05, 1.5582E-05, 1.5299E-05, 1.5016E-05,
     &   1.4733E-05, 1.4450E-05, 1.4168E-05, 1.3886E-05, 1.3604E-05,
     &   1.3323E-05, 1.3042E-05, 1.2762E-05, 1.2482E-05, 1.2203E-05,
     &   1.1925E-05, 1.1647E-05, 1.1371E-05, 1.1095E-05, 1.0820E-05,
     &   1.0547E-05, 1.0274E-05, 1.0003E-05, 9.7332E-06, 9.4647E-06,
     &   9.1977E-06, 8.9322E-06, 8.6684E-06, 8.4063E-06, 8.1461E-06,
     &   7.8877E-06, 7.6314E-06, 7.3772E-06, 7.1253E-06, 6.8756E-06,
     &   6.6284E-06, 6.3837E-06, 6.1417E-06, 5.9025E-06, 5.6661E-06,
     &   5.4327E-06, 5.2024E-06, 4.9754E-06, 4.7517E-06, 4.5315E-06,
     &   4.3148E-06, 4.1020E-06, 3.8929E-06, 3.6879E-06, 3.4870E-06,
     &   3.2903E-06, 3.0980E-06, 2.9102E-06, 2.7271E-06, 2.5487E-06,
     &   2.3753E-06, 2.2069E-06, 2.0437E-06, 1.8859E-06, 1.7335E-06,
     &   1.5866E-06, 1.4456E-06, 1.3103E-06, 1.1811E-06, 1.0579E-06,
     &   9.4098E-07, 8.3041E-07, 7.2630E-07, 6.2876E-07, 5.3792E-07,
     &   4.5386E-07, 4.4583E-07, 4.3787E-07, 4.2998E-07, 4.2216E-07,
     &   4.1441E-07, 4.0672E-07, 3.9911E-07, 3.9156E-07, 3.8409E-07,
     &   3.7669E-07, 3.6935E-07, 3.6209E-07, 3.5489E-07, 3.4776E-07,
     &   3.4071E-07, 3.3372E-07, 3.2681E-07, 3.1997E-07, 3.1319E-07/

         DATA ((DAT  (I,J), I= 401, 500), J=3,3) /
     &   3.0649E-07, 2.9986E-07, 2.9329E-07, 2.8680E-07, 2.8038E-07,
     &   2.7403E-07, 2.6775E-07, 2.6155E-07, 2.5541E-07, 2.4935E-07,
     &   2.4335E-07, 2.3743E-07, 2.3158E-07, 2.2580E-07, 2.2009E-07,
     &   2.1446E-07, 2.0889E-07, 2.0340E-07, 1.9798E-07, 1.9263E-07,
     &   1.8735E-07, 1.8215E-07, 1.7702E-07, 1.7196E-07, 1.6697E-07,
     &   1.6206E-07, 1.5721E-07, 1.5244E-07, 1.4774E-07, 1.4312E-07,
     &   1.3857E-07, 1.3409E-07, 1.2968E-07, 1.2534E-07, 1.2108E-07,
     &   1.1689E-07, 1.1278E-07, 1.0874E-07, 1.0477E-07, 1.0087E-07,
     &   9.7046E-08, 9.3296E-08, 8.9620E-08, 8.6016E-08, 8.2486E-08,
     &   7.9030E-08, 7.5647E-08, 7.2337E-08, 6.9101E-08, 6.5939E-08,
     &   6.2850E-08, 5.9835E-08, 5.6894E-08, 5.4026E-08, 5.1232E-08,
     &   4.8513E-08, 4.5866E-08, 4.3294E-08, 4.0796E-08, 3.8372E-08,
     &   3.6022E-08, 3.3746E-08, 3.1543E-08, 2.9415E-08, 2.7361E-08,
     &   2.5382E-08, 2.3476E-08, 2.1645E-08, 1.9888E-08, 1.8205E-08,
     &   1.6596E-08, 1.5062E-08, 1.3602E-08, 1.2216E-08, 1.0905E-08,
     &   9.6677E-09, 8.5051E-09, 7.4169E-09, 6.4032E-09, 5.4638E-09,
     &   4.5989E-09, 4.5165E-09, 4.4348E-09, 4.3539E-09, 4.2738E-09,
     &   4.1944E-09, 4.1157E-09, 4.0378E-09, 3.9606E-09, 3.8841E-09,
     &   3.8084E-09, 3.7335E-09, 3.6593E-09, 3.5858E-09, 3.5131E-09,
     &   3.4411E-09, 3.3699E-09, 3.2994E-09, 3.2297E-09, 3.1607E-09/

         DATA ((DAT  (I,J), I= 501, 600), J=3,3) /
     &   3.0924E-09, 3.0249E-09, 2.9582E-09, 2.8922E-09, 2.8269E-09,
     &   2.7624E-09, 2.6986E-09, 2.6355E-09, 2.5733E-09, 2.5117E-09,
     &   2.4509E-09, 2.3909E-09, 2.3315E-09, 2.2730E-09, 2.2152E-09,
     &   2.1581E-09, 2.1017E-09, 2.0462E-09, 1.9913E-09, 1.9372E-09,
     &   1.8839E-09, 1.8313E-09, 1.7794E-09, 1.7283E-09, 1.6779E-09,
     &   1.6283E-09, 1.5794E-09, 1.5312E-09, 1.4839E-09, 1.4372E-09,
     &   1.3913E-09, 1.3461E-09, 1.3017E-09, 1.2581E-09, 1.2151E-09,
     &   1.1730E-09, 1.1315E-09, 1.0908E-09, 1.0509E-09, 1.0117E-09,
     &   9.7323E-10, 9.3552E-10, 8.9856E-10, 8.6234E-10, 8.2686E-10,
     &   7.9213E-10, 7.5815E-10, 7.2491E-10, 6.9242E-10, 6.6067E-10,
     &   6.2966E-10, 5.9940E-10, 5.6989E-10, 5.4112E-10, 5.1310E-10,
     &   4.8582E-10, 4.5928E-10, 4.3350E-10, 4.0845E-10, 3.8415E-10,
     &   3.6060E-10, 3.3779E-10, 3.1573E-10, 2.9441E-10, 2.7384E-10,
     &   2.5401E-10, 2.3492E-10, 2.1659E-10, 1.9899E-10, 1.8214E-10,
     &   1.6604E-10, 1.5068E-10, 1.3607E-10, 1.2220E-10, 1.0908E-10,
     &   9.6705E-11, 8.5073E-11, 7.4186E-11, 6.4044E-11, 5.4647E-11,
     &   4.5995E-11, 4.5171E-11, 4.4354E-11, 4.3545E-11, 4.2743E-11,
     &   4.1949E-11, 4.1162E-11, 4.0382E-11, 3.9610E-11, 3.8846E-11,
     &   3.8089E-11, 3.7339E-11, 3.6597E-11, 3.5862E-11, 3.5135E-11,
     &   3.4415E-11, 3.3702E-11, 3.2997E-11, 3.2300E-11, 3.1610E-11/

         DATA ((DAT  (I,J), I= 601, 700), J=3,3) /
     &   3.0927E-11, 3.0252E-11, 2.9584E-11, 2.8924E-11, 2.8271E-11,
     &   2.7626E-11, 2.6988E-11, 2.6358E-11, 2.5735E-11, 2.5119E-11,
     &   2.4511E-11, 2.3910E-11, 2.3317E-11, 2.2731E-11, 2.2153E-11,
     &   2.1582E-11, 2.1019E-11, 2.0463E-11, 1.9914E-11, 1.9373E-11,
     &   1.8840E-11, 1.8314E-11, 1.7795E-11, 1.7284E-11, 1.6780E-11,
     &   1.6283E-11, 1.5795E-11, 1.5313E-11, 1.4839E-11, 1.4373E-11,
     &   1.3914E-11, 1.3462E-11, 1.3018E-11, 1.2581E-11, 1.2152E-11,
     &   1.1730E-11, 1.1316E-11, 1.0909E-11, 1.0509E-11, 1.0117E-11,
     &   9.7326E-12, 9.3555E-12, 8.9858E-12, 8.6236E-12, 8.2688E-12,
     &   7.9215E-12, 7.5817E-12, 7.2493E-12, 6.9243E-12, 6.6068E-12,
     &   6.2967E-12, 5.9941E-12, 5.6990E-12, 5.4113E-12, 5.1310E-12,
     &   4.8582E-12, 4.5929E-12, 4.3350E-12, 4.0846E-12, 3.8416E-12,
     &   3.6060E-12, 3.3779E-12, 3.1573E-12, 2.9441E-12, 2.7384E-12,
     &   2.5401E-12, 2.3493E-12, 2.1659E-12, 1.9899E-12, 1.8215E-12,
     &   1.6604E-12, 1.5069E-12, 1.3607E-12, 1.2220E-12, 1.0908E-12,
     &   9.6705E-13, 8.5073E-13, 7.4186E-13, 6.4044E-13, 5.4647E-13,
     &   4.5995E-13, 4.5171E-13, 4.4354E-13, 4.3545E-13, 4.2743E-13,
     &   4.1949E-13, 4.1162E-13, 4.0382E-13, 3.9610E-13, 3.8846E-13,
     &   3.8089E-13, 3.7339E-13, 3.6597E-13, 3.5862E-13, 3.5135E-13,
     &   3.4415E-13, 3.3702E-13, 3.2997E-13, 3.2300E-13, 3.1610E-13/

         DATA ((DAT  (I,J), I= 701, 800), J=3,3) /
     &   3.0927E-13, 3.0252E-13, 2.9584E-13, 2.8924E-13, 2.8271E-13,
     &   2.7626E-13, 2.6988E-13, 2.6358E-13, 2.5735E-13, 2.5119E-13,
     &   2.4511E-13, 2.3910E-13, 2.3317E-13, 2.2731E-13, 2.2153E-13,
     &   2.1582E-13, 2.1019E-13, 2.0463E-13, 1.9914E-13, 1.9373E-13,
     &   1.8840E-13, 1.8314E-13, 1.7795E-13, 1.7284E-13, 1.6780E-13,
     &   1.6283E-13, 1.5795E-13, 1.5313E-13, 1.4839E-13, 1.4373E-13,
     &   1.3914E-13, 1.3462E-13, 1.3018E-13, 1.2581E-13, 1.2152E-13,
     &   1.1730E-13, 1.1316E-13, 1.0909E-13, 1.0509E-13, 1.0117E-13,
     &   9.7326E-14, 9.3555E-14, 8.9858E-14, 8.6236E-14, 8.2688E-14,
     &   7.9215E-14, 7.5817E-14, 7.2493E-14, 6.9243E-14, 6.6068E-14,
     &   6.2967E-14, 5.9941E-14, 5.6990E-14, 5.4113E-14, 5.1310E-14,
     &   4.8582E-14, 4.5929E-14, 4.3350E-14, 4.0846E-14, 3.8416E-14,
     &   3.6060E-14, 3.3779E-14, 3.1573E-14, 2.9441E-14, 2.7384E-14,
     &   2.5401E-14, 2.3493E-14, 2.1659E-14, 1.9899E-14, 1.8215E-14,
     &   1.6604E-14, 1.5069E-14, 1.3607E-14, 1.2220E-14, 1.0908E-14,
     &   9.6705E-15, 8.5073E-15, 7.4186E-15, 6.4044E-15, 5.4647E-15,
     &   4.5995E-15, 4.5171E-15, 4.4354E-15, 4.3545E-15, 4.2743E-15,
     &   4.1949E-15, 4.1162E-15, 4.0382E-15, 3.9610E-15, 3.8846E-15,
     &   3.8089E-15, 3.7339E-15, 3.6597E-15, 3.5862E-15, 3.5135E-15,
     &   3.4415E-15, 3.3702E-15, 3.2997E-15, 3.2300E-15, 3.1610E-15/

         DATA ((DAT  (I,J), I= 801, 900), J=3,3) /
     &   3.0927E-15, 3.0252E-15, 2.9584E-15, 2.8924E-15, 2.8271E-15,
     &   2.7626E-15, 2.6988E-15, 2.6358E-15, 2.5735E-15, 2.5119E-15,
     &   2.4511E-15, 2.3910E-15, 2.3317E-15, 2.2731E-15, 2.2153E-15,
     &   2.1582E-15, 2.1019E-15, 2.0463E-15, 1.9914E-15, 1.9373E-15,
     &   1.8840E-15, 1.8314E-15, 1.7795E-15, 1.7284E-15, 1.6780E-15,
     &   1.6283E-15, 1.5795E-15, 1.5313E-15, 1.4839E-15, 1.4373E-15,
     &   1.3914E-15, 1.3462E-15, 1.3018E-15, 1.2581E-15, 1.2152E-15,
     &   1.1730E-15, 1.1316E-15, 1.0909E-15, 1.0509E-15, 1.0117E-15,
     &   9.7326E-16, 9.3555E-16, 8.9858E-16, 8.6236E-16, 8.2688E-16,
     &   7.9215E-16, 7.5817E-16, 7.2493E-16, 6.9243E-16, 6.6068E-16,
     &   6.2967E-16, 5.9941E-16, 5.6990E-16, 5.4113E-16, 5.1310E-16,
     &   4.8582E-16, 4.5929E-16, 4.3350E-16, 4.0846E-16, 3.8416E-16,
     &   3.6060E-16, 3.3779E-16, 3.1573E-16, 2.9441E-16, 2.7384E-16,
     &   2.5401E-16, 2.3493E-16, 2.1659E-16, 1.9899E-16, 1.8215E-16,
     &   1.6604E-16, 1.5069E-16, 1.3607E-16, 1.2220E-16, 1.0908E-16,
     &   9.6705E-17, 8.5073E-17, 7.4186E-17, 6.4044E-17, 5.4647E-17,
     &   4.5995E-17, 0.0000E+00, 4.5995E-17, 4.1424E-10, 1.6565E-09,
     &   3.7269E-09, 6.6246E-09, 1.0352E-08, 1.4909E-08, 2.0296E-08,
     &   2.6514E-08, 3.3564E-08, 4.1447E-08, 5.0164E-08, 5.9716E-08,
     &   7.0105E-08, 8.1333E-08, 9.3400E-08, 1.0631E-07, 1.2006E-07/

         DATA ((DAT  (I,J), I= 901, 979), J=3,3) /
     &   1.3466E-07, 1.5011E-07, 1.6641E-07, 1.8356E-07, 2.0157E-07,
     &   2.2043E-07, 2.4016E-07, 2.6075E-07, 2.8221E-07, 3.0455E-07,
     &   3.2775E-07, 3.5184E-07, 3.7681E-07, 4.0267E-07, 4.2941E-07,
     &   4.5706E-07, 4.8560E-07, 5.1505E-07, 5.4541E-07, 5.7669E-07,
     &   6.0889E-07, 6.4202E-07, 6.7608E-07, 7.1108E-07, 7.4702E-07,
     &   7.8392E-07, 8.2178E-07, 8.6061E-07, 9.0042E-07, 9.4121E-07,
     &   9.8299E-07, 1.0258E-06, 1.0696E-06, 1.1144E-06, 1.1602E-06,
     &   1.2071E-06, 1.2550E-06, 1.3040E-06, 1.3541E-06, 1.4053E-06,
     &   1.4575E-06, 1.5109E-06, 1.5654E-06, 1.6211E-06, 1.6779E-06,
     &   1.7359E-06, 1.7951E-06, 1.8555E-06, 1.9171E-06, 1.9800E-06,
     &   2.0442E-06, 2.1097E-06, 2.1765E-06, 2.2447E-06, 2.3143E-06,
     &   2.3853E-06, 2.4577E-06, 2.5317E-06, 2.6071E-06, 2.6842E-06,
     &   2.7629E-06, 2.8433E-06, 2.9255E-06, 3.0094E-06, 3.0953E-06,
     &   3.1831E-06, 3.2730E-06, 3.3651E-06, 3.4595E-06, 3.5564E-06,
     &   3.6559E-06, 3.7583E-06, 3.8638E-06, 3.8997E-06, 3.9729E-06,
     &   3.99885118E-06, 4.08592866E-06, 4.09748748E-06, 4.12465134E-06/


c      ++++ deghadt19 ++++

c update 12/29/19

c******************************************************************************
c*                                                                            *
c*    --- intRdatx ---                              Sun Dec 29 11:48:50 2019  *
c*                                                                            *
c***************************# FJ@HU Berlin #***********************************

c  "Delta alpha"               Update version (24/02/2019)

c ISCAN: -1, Ismooth: 0, IGS: 0, DSET: WAVE, iplot: 0 itestflavorspl: 0
c IKE,Emin,Emax,NP,N,EPSREL:  1, 6.0000E-01, 1.4000E+00, 100, 10, 1.00E-02,
c ALS(sta)(sys):  0.1184( 0.0007)( 0.0000), MZINP:   91.1876, MTOP:    173.20
c NF,IOR,ICHK: 5,4,2,ISU2: 2,NFL: 5,INAG: 1,inotail: 0, sin2W_ref: 0.23153
c CHPTCUT,EC,ECUT: 0.3180,   5.20,  11.50  sin2W_eff: 0.23153 sin2W_ref/sin2W_eff: 1.00000


         DATA ((DGT  (I,J), I=   1, 100), J=1,1) /
     &   8.4016E-02, 8.3910E-02, 8.3803E-02, 8.3694E-02, 8.3584E-02,
     &   8.3473E-02, 8.3361E-02, 8.3247E-02, 8.3132E-02, 8.3016E-02,
     &   8.2898E-02, 8.2779E-02, 8.2658E-02, 8.2536E-02, 8.2412E-02,
     &   8.2286E-02, 8.2159E-02, 8.2030E-02, 8.1900E-02, 8.1767E-02,
     &   8.1633E-02, 8.1497E-02, 8.1359E-02, 8.1219E-02, 8.1077E-02,
     &   8.0933E-02, 8.0786E-02, 8.0638E-02, 8.0487E-02, 8.0334E-02,
     &   8.0178E-02, 8.0020E-02, 7.9859E-02, 7.9696E-02, 7.9530E-02,
     &   7.9361E-02, 7.9189E-02, 7.9014E-02, 7.8836E-02, 7.8655E-02,
     &   7.8470E-02, 7.8282E-02, 7.8090E-02, 7.7895E-02, 7.7696E-02,
     &   7.7493E-02, 7.7286E-02, 7.7074E-02, 7.6858E-02, 7.6638E-02,
     &   7.6413E-02, 7.6182E-02, 7.5947E-02, 7.5706E-02, 7.5459E-02,
     &   7.5207E-02, 7.4948E-02, 7.4683E-02, 7.4412E-02, 7.4133E-02,
     &   7.3847E-02, 7.3553E-02, 7.3250E-02, 7.2940E-02, 7.2620E-02,
     &   7.2290E-02, 7.1951E-02, 7.1600E-02, 7.1238E-02, 7.0864E-02,
     &   7.0478E-02, 7.0077E-02, 6.9662E-02, 6.9230E-02, 6.8782E-02,
     &   6.8316E-02, 6.7830E-02, 6.7322E-02, 6.6791E-02, 6.6235E-02,
     &   6.5651E-02, 6.5035E-02, 6.4386E-02, 6.3699E-02, 6.2969E-02,
     &   6.2191E-02, 6.1359E-02, 6.0464E-02, 5.9496E-02, 5.8442E-02,
     &   5.7287E-02, 5.7165E-02, 5.7042E-02, 5.6918E-02, 5.6792E-02,
     &   5.6665E-02, 5.6537E-02, 5.6407E-02, 5.6276E-02, 5.6143E-02/

         DATA ((DGT  (I,J), I= 101, 200), J=1,1) /
     &   5.6009E-02, 5.5873E-02, 5.5736E-02, 5.5598E-02, 5.5457E-02,
     &   5.5315E-02, 5.5171E-02, 5.5026E-02, 5.4879E-02, 5.4730E-02,
     &   5.4579E-02, 5.4426E-02, 5.4271E-02, 5.4114E-02, 5.3956E-02,
     &   5.3795E-02, 5.3631E-02, 5.3466E-02, 5.3298E-02, 5.3128E-02,
     &   5.2956E-02, 5.2781E-02, 5.2603E-02, 5.2423E-02, 5.2240E-02,
     &   5.2055E-02, 5.1866E-02, 5.1675E-02, 5.1480E-02, 5.1282E-02,
     &   5.1081E-02, 5.0876E-02, 5.0668E-02, 5.0457E-02, 5.0241E-02,
     &   5.0022E-02, 4.9799E-02, 4.9571E-02, 4.9339E-02, 4.9103E-02,
     &   4.8862E-02, 4.8616E-02, 4.8365E-02, 4.8109E-02, 4.7847E-02,
     &   4.7579E-02, 4.7306E-02, 4.7026E-02, 4.6739E-02, 4.6446E-02,
     &   4.6145E-02, 4.5837E-02, 4.5521E-02, 4.5197E-02, 4.4863E-02,
     &   4.4521E-02, 4.4168E-02, 4.3805E-02, 4.3431E-02, 4.3046E-02,
     &   4.2647E-02, 4.2236E-02, 4.1810E-02, 4.1369E-02, 4.0911E-02,
     &   4.0436E-02, 3.9942E-02, 3.9428E-02, 3.8891E-02, 3.8330E-02,
     &   3.7742E-02, 3.7126E-02, 3.6477E-02, 3.5793E-02, 3.5069E-02,
     &   3.4301E-02, 3.3487E-02, 3.2613E-02, 3.1674E-02, 3.0660E-02,
     &   2.9558E-02, 2.9454E-02, 2.9349E-02, 2.9243E-02, 2.9136E-02,
     &   2.9029E-02, 2.8920E-02, 2.8811E-02, 2.8701E-02, 2.8589E-02,
     &   2.8477E-02, 2.8364E-02, 2.8250E-02, 2.8134E-02, 2.8018E-02,
     &   2.7900E-02, 2.7782E-02, 2.7662E-02, 2.7541E-02, 2.7419E-02/

         DATA ((DGT  (I,J), I= 201, 300), J=1,1) /
     &   2.7296E-02, 2.7172E-02, 2.7046E-02, 2.6920E-02, 2.6791E-02,
     &   2.6662E-02, 2.6531E-02, 2.6399E-02, 2.6266E-02, 2.6131E-02,
     &   2.5995E-02, 2.5857E-02, 2.5717E-02, 2.5577E-02, 2.5434E-02,
     &   2.5290E-02, 2.5140E-02, 2.4993E-02, 2.4844E-02, 2.4693E-02,
     &   2.4540E-02, 2.4385E-02, 2.4229E-02, 2.4070E-02, 2.3910E-02,
     &   2.3747E-02, 2.3583E-02, 2.3416E-02, 2.3247E-02, 2.3075E-02,
     &   2.2902E-02, 2.2725E-02, 2.2547E-02, 2.2366E-02, 2.2182E-02,
     &   2.1995E-02, 2.1806E-02, 2.1614E-02, 2.1418E-02, 2.1220E-02,
     &   2.1019E-02, 2.0814E-02, 2.0606E-02, 2.0394E-02, 2.0179E-02,
     &   1.9960E-02, 1.9737E-02, 1.9510E-02, 1.9279E-02, 1.9043E-02,
     &   1.8803E-02, 1.8559E-02, 1.8309E-02, 1.8054E-02, 1.7794E-02,
     &   1.7529E-02, 1.7257E-02, 1.6980E-02, 1.6696E-02, 1.6405E-02,
     &   1.6107E-02, 1.5802E-02, 1.5489E-02, 1.5167E-02, 1.4837E-02,
     &   1.4496E-02, 1.4146E-02, 1.3785E-02, 1.3412E-02, 1.3026E-02,
     &   1.2627E-02, 1.2213E-02, 1.1782E-02, 1.1334E-02, 1.0865E-02,
     &   1.0375E-02, 9.8612E-03, 9.3205E-03, 8.7501E-03, 8.1469E-03,
     &   7.5080E-03, 7.4421E-03, 7.3758E-03, 7.3091E-03, 7.2419E-03,
     &   7.1744E-03, 7.1064E-03, 7.0381E-03, 6.9693E-03, 6.9001E-03,
     &   6.8306E-03, 6.7606E-03, 6.6902E-03, 6.6193E-03, 6.5481E-03,
     &   6.4765E-03, 6.4044E-03, 6.3319E-03, 6.2590E-03, 6.1857E-03/

         DATA ((DGT  (I,J), I= 301, 400), J=1,1) /
     &   6.1120E-03, 6.0378E-03, 5.9632E-03, 5.8882E-03, 5.8128E-03,
     &   5.7370E-03, 5.6608E-03, 5.5842E-03, 5.5071E-03, 5.4297E-03,
     &   5.3519E-03, 5.2736E-03, 5.1950E-03, 5.1160E-03, 5.0366E-03,
     &   4.9569E-03, 4.8768E-03, 4.7963E-03, 4.7155E-03, 4.6344E-03,
     &   4.5529E-03, 4.4711E-03, 4.3890E-03, 4.3066E-03, 4.2240E-03,
     &   4.1410E-03, 4.0579E-03, 3.9745E-03, 3.8908E-03, 3.8070E-03,
     &   3.7230E-03, 3.6389E-03, 3.5546E-03, 3.4703E-03, 3.3858E-03,
     &   3.3013E-03, 3.2168E-03, 3.1322E-03, 3.0477E-03, 2.9633E-03,
     &   2.8789E-03, 2.7947E-03, 2.7107E-03, 2.6269E-03, 2.5433E-03,
     &   2.4600E-03, 2.3770E-03, 2.2945E-03, 2.2123E-03, 2.1307E-03,
     &   2.0496E-03, 1.9690E-03, 1.8891E-03, 1.8099E-03, 1.7315E-03,
     &   1.6539E-03, 1.5771E-03, 1.5013E-03, 1.4265E-03, 1.3528E-03,
     &   1.2802E-03, 1.2088E-03, 1.1388E-03, 1.0701E-03, 1.0028E-03,
     &   9.3703E-04, 8.7286E-04, 8.1035E-04, 7.4959E-04, 6.9065E-04,
     &   6.3360E-04, 5.7854E-04, 5.2553E-04, 4.7466E-04, 4.2657E-04,
     &   3.8013E-04, 3.3606E-04, 2.9442E-04, 2.5528E-04, 2.1872E-04,
     &   1.8479E-04, 1.8154E-04, 1.7832E-04, 1.7513E-04, 1.7197E-04,
     &   1.6883E-04, 1.6572E-04, 1.6264E-04, 1.5959E-04, 1.5656E-04,
     &   1.5356E-04, 1.5059E-04, 1.4764E-04, 1.4473E-04, 1.4184E-04,
     &   1.3897E-04, 1.3614E-04, 1.3333E-04, 1.3056E-04, 1.2781E-04/

         DATA ((DGT  (I,J), I= 401, 500), J=1,1) /
     &   1.2509E-04, 1.2239E-04, 1.1973E-04, 1.1709E-04, 1.1448E-04,
     &   1.1190E-04, 1.0935E-04, 1.0682E-04, 1.0433E-04, 1.0186E-04,
     &   9.9420E-05, 9.7010E-05, 9.4628E-05, 9.2275E-05, 8.9951E-05,
     &   8.7656E-05, 8.5389E-05, 8.3152E-05, 8.0943E-05, 7.8763E-05,
     &   7.6612E-05, 7.4490E-05, 7.2398E-05, 7.0334E-05, 6.8300E-05,
     &   6.6294E-05, 6.4318E-05, 6.2371E-05, 6.0454E-05, 5.8565E-05,
     &   5.6707E-05, 5.4877E-05, 5.3077E-05, 5.1307E-05, 4.9566E-05,
     &   4.7855E-05, 4.6173E-05, 4.4521E-05, 4.2898E-05, 4.1305E-05,
     &   3.9742E-05, 3.8209E-05, 3.6705E-05, 3.5232E-05, 3.3788E-05,
     &   3.2374E-05, 3.0990E-05, 2.9635E-05, 2.8311E-05, 2.7017E-05,
     &   2.5753E-05, 2.4518E-05, 2.3314E-05, 2.2140E-05, 2.0996E-05,
     &   1.9883E-05, 1.8799E-05, 1.7745E-05, 1.6722E-05, 1.5729E-05,
     &   1.4766E-05, 1.3834E-05, 1.2931E-05, 1.2059E-05, 1.1218E-05,
     &   1.0406E-05, 9.6255E-06, 8.8749E-06, 8.1546E-06, 7.4648E-06,
     &   6.8054E-06, 6.1763E-06, 5.5778E-06, 5.0096E-06, 4.4720E-06,
     &   3.9648E-06, 3.4881E-06, 3.0418E-06, 2.6261E-06, 2.2409E-06,
     &   1.8862E-06, 1.8524E-06, 1.8189E-06, 1.7857E-06, 1.7528E-06,
     &   1.7203E-06, 1.6880E-06, 1.6561E-06, 1.6244E-06, 1.5930E-06,
     &   1.5620E-06, 1.5313E-06, 1.5008E-06, 1.4707E-06, 1.4409E-06,
     &   1.4114E-06, 1.3822E-06, 1.3532E-06, 1.3246E-06, 1.2963E-06/

         DATA ((DGT  (I,J), I= 501, 600), J=1,1) /
     &   1.2684E-06, 1.2407E-06, 1.2133E-06, 1.1862E-06, 1.1594E-06,
     &   1.1330E-06, 1.1068E-06, 1.0810E-06, 1.0554E-06, 1.0302E-06,
     &   1.0052E-06, 9.8061E-07, 9.5629E-07, 9.3227E-07, 9.0855E-07,
     &   8.8514E-07, 8.6204E-07, 8.3924E-07, 8.1675E-07, 7.9456E-07,
     &   7.7268E-07, 7.5110E-07, 7.2983E-07, 7.0886E-07, 6.8820E-07,
     &   6.6785E-07, 6.4780E-07, 6.2805E-07, 6.0861E-07, 5.8948E-07,
     &   5.7065E-07, 5.5213E-07, 5.3392E-07, 5.1600E-07, 4.9840E-07,
     &   4.8110E-07, 4.6411E-07, 4.4742E-07, 4.3103E-07, 4.1496E-07,
     &   3.9918E-07, 3.8372E-07, 3.6855E-07, 3.5370E-07, 3.3915E-07,
     &   3.2490E-07, 3.1097E-07, 2.9733E-07, 2.8400E-07, 2.7098E-07,
     &   2.5827E-07, 2.4585E-07, 2.3375E-07, 2.2195E-07, 2.1045E-07,
     &   1.9927E-07, 1.8838E-07, 1.7780E-07, 1.6753E-07, 1.5757E-07,
     &   1.4791E-07, 1.3855E-07, 1.2950E-07, 1.2076E-07, 1.1232E-07,
     &   1.0419E-07, 9.6358E-08, 8.8836E-08, 8.1620E-08, 7.4710E-08,
     &   6.8105E-08, 6.1806E-08, 5.5812E-08, 5.0124E-08, 4.4742E-08,
     &   3.9665E-08, 3.4894E-08, 3.0429E-08, 2.6269E-08, 2.2414E-08,
     &   1.8866E-08, 1.8528E-08, 1.8193E-08, 1.7861E-08, 1.7532E-08,
     &   1.7206E-08, 1.6883E-08, 1.6564E-08, 1.6247E-08, 1.5933E-08,
     &   1.5623E-08, 1.5315E-08, 1.5011E-08, 1.4709E-08, 1.4411E-08,
     &   1.4116E-08, 1.3824E-08, 1.3534E-08, 1.3248E-08, 1.2965E-08/

         DATA ((DGT  (I,J), I= 601, 700), J=1,1) /
     &   1.2685E-08, 1.2408E-08, 1.2135E-08, 1.1864E-08, 1.1596E-08,
     &   1.1331E-08, 1.1070E-08, 1.0811E-08, 1.0555E-08, 1.0303E-08,
     &   1.0054E-08, 9.8072E-09, 9.5639E-09, 9.3236E-09, 9.0864E-09,
     &   8.8523E-09, 8.6212E-09, 8.3932E-09, 8.1682E-09, 7.9463E-09,
     &   7.7274E-09, 7.5116E-09, 7.2989E-09, 7.0892E-09, 6.8825E-09,
     &   6.6790E-09, 6.4784E-09, 6.2810E-09, 6.0866E-09, 5.8952E-09,
     &   5.7069E-09, 5.5217E-09, 5.3395E-09, 5.1603E-09, 4.9843E-09,
     &   4.8113E-09, 4.6413E-09, 4.4744E-09, 4.3105E-09, 4.1497E-09,
     &   3.9920E-09, 3.8373E-09, 3.6857E-09, 3.5371E-09, 3.3916E-09,
     &   3.2492E-09, 3.1098E-09, 2.9734E-09, 2.8401E-09, 2.7099E-09,
     &   2.5827E-09, 2.4586E-09, 2.3376E-09, 2.2195E-09, 2.1046E-09,
     &   1.9927E-09, 1.8839E-09, 1.7781E-09, 1.6754E-09, 1.5757E-09,
     &   1.4791E-09, 1.3855E-09, 1.2950E-09, 1.2076E-09, 1.1232E-09,
     &   1.0419E-09, 9.6359E-10, 8.8837E-10, 8.1621E-10, 7.4711E-10,
     &   6.8106E-10, 6.1806E-10, 5.5813E-10, 5.0125E-10, 4.4742E-10,
     &   3.9665E-10, 3.4894E-10, 3.0429E-10, 2.6269E-10, 2.2414E-10,
     &   1.8866E-10, 1.8528E-10, 1.8193E-10, 1.7861E-10, 1.7532E-10,
     &   1.7206E-10, 1.6883E-10, 1.6564E-10, 1.6247E-10, 1.5933E-10,
     &   1.5623E-10, 1.5315E-10, 1.5011E-10, 1.4709E-10, 1.4411E-10,
     &   1.4116E-10, 1.3824E-10, 1.3535E-10, 1.3248E-10, 1.2965E-10/

         DATA ((DGT  (I,J), I= 701, 800), J=1,1) /
     &   1.2685E-10, 1.2408E-10, 1.2135E-10, 1.1864E-10, 1.1596E-10,
     &   1.1331E-10, 1.1070E-10, 1.0811E-10, 1.0556E-10, 1.0303E-10,
     &   1.0054E-10, 9.8072E-11, 9.5639E-11, 9.3237E-11, 9.0865E-11,
     &   8.8523E-11, 8.6212E-11, 8.3932E-11, 8.1682E-11, 7.9463E-11,
     &   7.7274E-11, 7.5116E-11, 7.2989E-11, 7.0892E-11, 6.8826E-11,
     &   6.6790E-11, 6.4784E-11, 6.2810E-11, 6.0866E-11, 5.8952E-11,
     &   5.7069E-11, 5.5217E-11, 5.3395E-11, 5.1603E-11, 4.9843E-11,
     &   4.8113E-11, 4.6413E-11, 4.4744E-11, 4.3105E-11, 4.1497E-11,
     &   3.9920E-11, 3.8373E-11, 3.6857E-11, 3.5371E-11, 3.3916E-11,
     &   3.2492E-11, 3.1098E-11, 2.9734E-11, 2.8401E-11, 2.7099E-11,
     &   2.5827E-11, 2.4586E-11, 2.3376E-11, 2.2195E-11, 2.1046E-11,
     &   1.9927E-11, 1.8839E-11, 1.7781E-11, 1.6754E-11, 1.5757E-11,
     &   1.4791E-11, 1.3855E-11, 1.2950E-11, 1.2076E-11, 1.1232E-11,
     &   1.0419E-11, 9.6359E-12, 8.8837E-12, 8.1621E-12, 7.4711E-12,
     &   6.8106E-12, 6.1806E-12, 5.5813E-12, 5.0125E-12, 4.4742E-12,
     &   3.9665E-12, 3.4894E-12, 3.0429E-12, 2.6269E-12, 2.2414E-12,
     &   1.8866E-12, 1.8528E-12, 1.8193E-12, 1.7861E-12, 1.7532E-12,
     &   1.7206E-12, 1.6883E-12, 1.6564E-12, 1.6247E-12, 1.5933E-12,
     &   1.5623E-12, 1.5315E-12, 1.5011E-12, 1.4709E-12, 1.4411E-12,
     &   1.4116E-12, 1.3824E-12, 1.3535E-12, 1.3248E-12, 1.2965E-12/

         DATA ((DGT  (I,J), I= 801, 900), J=1,1) /
     &   1.2685E-12, 1.2408E-12, 1.2135E-12, 1.1864E-12, 1.1596E-12,
     &   1.1331E-12, 1.1070E-12, 1.0811E-12, 1.0556E-12, 1.0303E-12,
     &   1.0054E-12, 9.8072E-13, 9.5639E-13, 9.3237E-13, 9.0865E-13,
     &   8.8523E-13, 8.6212E-13, 8.3932E-13, 8.1682E-13, 7.9463E-13,
     &   7.7274E-13, 7.5116E-13, 7.2989E-13, 7.0892E-13, 6.8826E-13,
     &   6.6790E-13, 6.4784E-13, 6.2810E-13, 6.0866E-13, 5.8952E-13,
     &   5.7069E-13, 5.5217E-13, 5.3395E-13, 5.1603E-13, 4.9843E-13,
     &   4.8113E-13, 4.6413E-13, 4.4744E-13, 4.3105E-13, 4.1497E-13,
     &   3.9920E-13, 3.8373E-13, 3.6857E-13, 3.5371E-13, 3.3916E-13,
     &   3.2492E-13, 3.1098E-13, 2.9734E-13, 2.8401E-13, 2.7099E-13,
     &   2.5827E-13, 2.4586E-13, 2.3376E-13, 2.2195E-13, 2.1046E-13,
     &   1.9927E-13, 1.8839E-13, 1.7781E-13, 1.6754E-13, 1.5757E-13,
     &   1.4791E-13, 1.3855E-13, 1.2950E-13, 1.2076E-13, 1.1232E-13,
     &   1.0419E-13, 9.6359E-14, 8.8837E-14, 8.1621E-14, 7.4711E-14,
     &   6.8106E-14, 6.1806E-14, 5.5813E-14, 5.0125E-14, 4.4742E-14,
     &   3.9665E-14, 3.4894E-14, 3.0429E-14, 2.6269E-14, 2.2414E-14,
     &   1.8866E-14, 0.0000E+00,-1.8866E-14,-1.6991E-07,-6.7945E-07,
     &  -1.5287E-06,-2.7175E-06,-4.2468E-06,-6.1167E-06,-8.3276E-06,
     &  -1.0880E-05,-1.3775E-05,-1.7012E-05,-2.0593E-05,-2.4518E-05,
     &  -2.8788E-05,-3.3405E-05,-3.8369E-05,-4.3681E-05,-4.9343E-05/

         DATA ((DGT  (I,J), I= 901, 979), J=1,1) /
     &  -5.5357E-05,-6.1722E-05,-6.8442E-05,-7.5518E-05,-8.2951E-05,
     &  -9.0743E-05,-9.8896E-05,-1.0741E-04,-1.1629E-04,-1.2554E-04,
     &  -1.3516E-04,-1.4515E-04,-1.5552E-04,-1.6626E-04,-1.7738E-04,
     &  -1.8889E-04,-2.0078E-04,-2.1306E-04,-2.2573E-04,-2.3880E-04,
     &  -2.5227E-04,-2.6614E-04,-2.8042E-04,-2.9511E-04,-3.1021E-04,
     &  -3.2574E-04,-3.4168E-04,-3.5806E-04,-3.7487E-04,-3.9212E-04,
     &  -4.0981E-04,-4.2796E-04,-4.4655E-04,-4.6561E-04,-4.8514E-04,
     &  -5.0514E-04,-5.2563E-04,-5.4660E-04,-5.6807E-04,-5.9004E-04,
     &  -6.1253E-04,-6.3553E-04,-6.5907E-04,-6.8314E-04,-7.0776E-04,
     &  -7.3295E-04,-7.5870E-04,-7.8504E-04,-8.1197E-04,-8.3951E-04,
     &  -8.6768E-04,-8.9648E-04,-9.2593E-04,-9.5605E-04,-9.8685E-04,
     &  -1.0184E-03,-1.0506E-03,-1.0836E-03,-1.1174E-03,-1.1519E-03,
     &  -1.1873E-03,-1.2236E-03,-1.2608E-03,-1.2989E-03,-1.3380E-03,
     &  -1.3781E-03,-1.4194E-03,-1.4618E-03,-1.5055E-03,-1.5506E-03,
     &  -1.5972E-03,-1.6455E-03,-1.6959E-03,-1.7132E-03,-1.7488E-03,
     &  -1.76154462E-03,-1.80506369E-03,-1.81095705E-03,-1.82496442E-03/

         DATA ((DGT  (I,J), I=   1, 100), J=2,2) /
     &   8.9064E-05, 8.9064E-05, 8.9064E-05, 8.9064E-05, 8.9063E-05,
     &   8.9063E-05, 8.9063E-05, 8.9063E-05, 8.9063E-05, 8.9063E-05,
     &   8.9063E-05, 8.9063E-05, 8.9063E-05, 8.9063E-05, 8.9063E-05,
     &   8.9063E-05, 8.9063E-05, 8.9063E-05, 8.9063E-05, 8.9063E-05,
     &   8.9063E-05, 8.9063E-05, 8.9063E-05, 8.9063E-05, 8.9063E-05,
     &   8.9063E-05, 8.9062E-05, 8.9062E-05, 8.9062E-05, 8.9062E-05,
     &   8.9062E-05, 8.9062E-05, 8.9062E-05, 8.9062E-05, 8.9062E-05,
     &   8.9062E-05, 8.9062E-05, 8.9062E-05, 8.9061E-05, 8.9061E-05,
     &   8.9061E-05, 8.9061E-05, 8.9061E-05, 8.9061E-05, 8.9061E-05,
     &   8.9060E-05, 8.9060E-05, 8.9060E-05, 8.9060E-05, 8.9060E-05,
     &   8.9060E-05, 8.9059E-05, 8.9059E-05, 8.9059E-05, 8.9059E-05,
     &   8.9058E-05, 8.9058E-05, 8.9058E-05, 8.9057E-05, 8.9057E-05,
     &   8.9057E-05, 8.9056E-05, 8.9056E-05, 8.9055E-05, 8.9055E-05,
     &   8.9054E-05, 8.9053E-05, 8.9053E-05, 8.9052E-05, 8.9051E-05,
     &   8.9050E-05, 8.9049E-05, 8.9048E-05, 8.9046E-05, 8.9045E-05,
     &   8.9043E-05, 8.9042E-05, 8.9039E-05, 8.9037E-05, 8.9034E-05,
     &   8.9031E-05, 8.9028E-05, 8.9023E-05, 8.9018E-05, 8.9012E-05,
     &   8.9005E-05, 8.8996E-05, 8.8985E-05, 8.8972E-05, 8.8954E-05,
     &   8.8931E-05, 8.8928E-05, 8.8925E-05, 8.8922E-05, 8.8919E-05,
     &   8.8916E-05, 8.8913E-05, 8.8910E-05, 8.8906E-05, 8.8903E-05/

         DATA ((DGT  (I,J), I= 101, 200), J=2,2) /
     &   8.8899E-05, 8.8896E-05, 8.8892E-05, 8.8888E-05, 8.8884E-05,
     &   8.8879E-05, 8.8875E-05, 8.8870E-05, 8.8866E-05, 8.8861E-05,
     &   8.8856E-05, 8.8850E-05, 8.8845E-05, 8.8839E-05, 8.8833E-05,
     &   8.8827E-05, 8.8821E-05, 8.8814E-05, 8.8807E-05, 8.8800E-05,
     &   8.8792E-05, 8.8784E-05, 8.8776E-05, 8.8768E-05, 8.8759E-05,
     &   8.8749E-05, 8.8739E-05, 8.8729E-05, 8.8718E-05, 8.8707E-05,
     &   8.8695E-05, 8.8683E-05, 8.8670E-05, 8.8656E-05, 8.8641E-05,
     &   8.8626E-05, 8.8610E-05, 8.8593E-05, 8.8575E-05, 8.8556E-05,
     &   8.8536E-05, 8.8514E-05, 8.8492E-05, 8.8467E-05, 8.8442E-05,
     &   8.8414E-05, 8.8385E-05, 8.8354E-05, 8.8321E-05, 8.8285E-05,
     &   8.8247E-05, 8.8205E-05, 8.8161E-05, 8.8113E-05, 8.8062E-05,
     &   8.8006E-05, 8.7945E-05, 8.7879E-05, 8.7807E-05, 8.7728E-05,
     &   8.7642E-05, 8.7547E-05, 8.7442E-05, 8.7327E-05, 8.7199E-05,
     &   8.7057E-05, 8.6898E-05, 8.6719E-05, 8.6518E-05, 8.6291E-05,
     &   8.6032E-05, 8.5736E-05, 8.5396E-05, 8.5001E-05, 8.4540E-05,
     &   8.3998E-05, 8.3354E-05, 8.2580E-05, 8.1640E-05, 8.0483E-05,
     &   7.9036E-05, 7.8888E-05, 7.8737E-05, 7.8583E-05, 7.8425E-05,
     &   7.8263E-05, 7.8098E-05, 7.7929E-05, 7.7757E-05, 7.7580E-05,
     &   7.7399E-05, 7.7214E-05, 7.7024E-05, 7.6830E-05, 7.6632E-05,
     &   7.6428E-05, 7.6219E-05, 7.6006E-05, 7.5787E-05, 7.5562E-05/

         DATA ((DGT  (I,J), I= 201, 300), J=2,2) /
     &   7.5332E-05, 7.5096E-05, 7.4855E-05, 7.4606E-05, 7.4352E-05,
     &   7.4091E-05, 7.3822E-05, 7.3547E-05, 7.3265E-05, 7.2974E-05,
     &   7.2676E-05, 7.2370E-05, 7.2056E-05, 7.1733E-05, 7.1400E-05,
     &   7.1059E-05, 7.0708E-05, 7.0346E-05, 6.9975E-05, 6.9593E-05,
     &   6.9199E-05, 6.8795E-05, 6.8378E-05, 6.7949E-05, 6.7508E-05,
     &   6.7053E-05, 6.6584E-05, 6.6102E-05, 6.5605E-05, 6.5092E-05,
     &   6.4564E-05, 6.4020E-05, 6.3459E-05, 6.2881E-05, 6.2284E-05,
     &   6.1670E-05, 6.1036E-05, 6.0382E-05, 5.9707E-05, 5.9012E-05,
     &   5.8295E-05, 5.7555E-05, 5.6793E-05, 5.6006E-05, 5.5196E-05,
     &   5.4360E-05, 5.3498E-05, 5.2610E-05, 5.1696E-05, 5.0754E-05,
     &   4.9783E-05, 4.8785E-05, 4.7757E-05, 4.6701E-05, 4.5615E-05,
     &   4.4499E-05, 4.3355E-05, 4.2180E-05, 4.0977E-05, 3.9745E-05,
     &   3.8485E-05, 3.7197E-05, 3.5884E-05, 3.4546E-05, 3.3186E-05,
     &   3.1804E-05, 3.0403E-05, 2.8986E-05, 2.7556E-05, 2.6116E-05,
     &   2.4670E-05, 2.3221E-05, 2.1773E-05, 2.0330E-05, 1.8898E-05,
     &   1.7479E-05, 1.6078E-05, 1.4697E-05, 1.3341E-05, 1.2011E-05,
     &   1.0708E-05, 1.0579E-05, 1.0451E-05, 1.0323E-05, 1.0195E-05,
     &   1.0067E-05, 9.9399E-06, 9.8128E-06, 9.6861E-06, 9.5596E-06,
     &   9.4334E-06, 9.3076E-06, 9.1819E-06, 9.0566E-06, 8.9316E-06,
     &   8.8069E-06, 8.6824E-06, 8.5582E-06, 8.4343E-06, 8.3108E-06/

         DATA ((DGT  (I,J), I= 301, 400), J=2,2) /
     &   8.1875E-06, 8.0645E-06, 7.9418E-06, 7.8193E-06, 7.6972E-06,
     &   7.5754E-06, 7.4539E-06, 7.3326E-06, 7.2117E-06, 7.0911E-06,
     &   6.9708E-06, 6.8508E-06, 6.7311E-06, 6.6118E-06, 6.4927E-06,
     &   6.3740E-06, 6.2556E-06, 6.1376E-06, 6.0199E-06, 5.9025E-06,
     &   5.7855E-06, 5.6688E-06, 5.5525E-06, 5.4366E-06, 5.3211E-06,
     &   5.2059E-06, 5.0912E-06, 4.9768E-06, 4.8629E-06, 4.7494E-06,
     &   4.6364E-06, 4.5238E-06, 4.4116E-06, 4.3000E-06, 4.1889E-06,
     &   4.0783E-06, 3.9682E-06, 3.8587E-06, 3.7497E-06, 3.6414E-06,
     &   3.5337E-06, 3.4266E-06, 3.3202E-06, 3.2145E-06, 3.1095E-06,
     &   3.0053E-06, 2.9018E-06, 2.7992E-06, 2.6975E-06, 2.5966E-06,
     &   2.4967E-06, 2.3977E-06, 2.2998E-06, 2.2030E-06, 2.1073E-06,
     &   2.0127E-06, 1.9194E-06, 1.8273E-06, 1.7366E-06, 1.6473E-06,
     &   1.5594E-06, 1.4731E-06, 1.3883E-06, 1.3053E-06, 1.2239E-06,
     &   1.1444E-06, 1.0668E-06, 9.9118E-07, 9.1760E-07, 8.4618E-07,
     &   7.7699E-07, 7.1013E-07, 6.4569E-07, 5.8376E-07, 5.2444E-07,
     &   4.6782E-07, 4.1400E-07, 3.6306E-07, 3.1511E-07, 2.7024E-07,
     &   2.2853E-07, 2.2454E-07, 2.2058E-07, 2.1665E-07, 2.1275E-07,
     &   2.0889E-07, 2.0506E-07, 2.0127E-07, 1.9750E-07, 1.9377E-07,
     &   1.9008E-07, 1.8641E-07, 1.8278E-07, 1.7919E-07, 1.7563E-07,
     &   1.7210E-07, 1.6860E-07, 1.6514E-07, 1.6171E-07, 1.5832E-07/

         DATA ((DGT  (I,J), I= 401, 500), J=2,2) /
     &   1.5496E-07, 1.5164E-07, 1.4835E-07, 1.4509E-07, 1.4187E-07,
     &   1.3868E-07, 1.3553E-07, 1.3241E-07, 1.2932E-07, 1.2628E-07,
     &   1.2326E-07, 1.2028E-07, 1.1734E-07, 1.1443E-07, 1.1155E-07,
     &   1.0872E-07, 1.0591E-07, 1.0314E-07, 1.0041E-07, 9.7714E-08,
     &   9.5052E-08, 9.2426E-08, 8.9835E-08, 8.7280E-08, 8.4761E-08,
     &   8.2277E-08, 7.9830E-08, 7.7418E-08, 7.5043E-08, 7.2703E-08,
     &   7.0400E-08, 6.8133E-08, 6.5902E-08, 6.3707E-08, 6.1549E-08,
     &   5.9427E-08, 5.7342E-08, 5.5293E-08, 5.3281E-08, 5.1305E-08,
     &   4.9366E-08, 4.7464E-08, 4.5598E-08, 4.3770E-08, 4.1978E-08,
     &   4.0223E-08, 3.8505E-08, 3.6824E-08, 3.5180E-08, 3.3573E-08,
     &   3.2004E-08, 3.0471E-08, 2.8976E-08, 2.7518E-08, 2.6097E-08,
     &   2.4713E-08, 2.3367E-08, 2.2059E-08, 2.0787E-08, 1.9554E-08,
     &   1.8357E-08, 1.7198E-08, 1.6077E-08, 1.4994E-08, 1.3948E-08,
     &   1.2939E-08, 1.1968E-08, 1.1035E-08, 1.0140E-08, 9.2824E-09,
     &   8.4626E-09, 7.6806E-09, 6.9364E-09, 6.2300E-09, 5.5614E-09,
     &   4.9308E-09, 4.3380E-09, 3.7831E-09, 3.2661E-09, 2.7870E-09,
     &   2.3459E-09, 2.3039E-09, 2.2622E-09, 2.2210E-09, 2.1801E-09,
     &   2.1396E-09, 2.0994E-09, 2.0597E-09, 2.0203E-09, 1.9813E-09,
     &   1.9427E-09, 1.9045E-09, 1.8667E-09, 1.8292E-09, 1.7921E-09,
     &   1.7554E-09, 1.7191E-09, 1.6831E-09, 1.6475E-09, 1.6123E-09/

         DATA ((DGT  (I,J), I= 501, 600), J=2,2) /
     &   1.5775E-09, 1.5431E-09, 1.5090E-09, 1.4754E-09, 1.4421E-09,
     &   1.4092E-09, 1.3766E-09, 1.3445E-09, 1.3127E-09, 1.2813E-09,
     &   1.2503E-09, 1.2197E-09, 1.1894E-09, 1.1595E-09, 1.1300E-09,
     &   1.1009E-09, 1.0722E-09, 1.0438E-09, 1.0159E-09, 9.8826E-10,
     &   9.6104E-10, 9.3420E-10, 9.0775E-10, 8.8167E-10, 8.5597E-10,
     &   8.3066E-10, 8.0572E-10, 7.8116E-10, 7.5699E-10, 7.3319E-10,
     &   7.0977E-10, 6.8674E-10, 6.6408E-10, 6.4180E-10, 6.1990E-10,
     &   5.9839E-10, 5.7725E-10, 5.5649E-10, 5.3612E-10, 5.1612E-10,
     &   4.9650E-10, 4.7726E-10, 4.5841E-10, 4.3993E-10, 4.2183E-10,
     &   4.0412E-10, 3.8678E-10, 3.6982E-10, 3.5324E-10, 3.3705E-10,
     &   3.2123E-10, 3.0579E-10, 2.9074E-10, 2.7606E-10, 2.6176E-10,
     &   2.4785E-10, 2.3431E-10, 2.2115E-10, 2.0838E-10, 1.9598E-10,
     &   1.8397E-10, 1.7233E-10, 1.6107E-10, 1.5020E-10, 1.3970E-10,
     &   1.2959E-10, 1.1985E-10, 1.1050E-10, 1.0152E-10, 9.2925E-11,
     &   8.4710E-11, 7.6875E-11, 6.9420E-11, 6.2345E-11, 5.5650E-11,
     &   4.9336E-11, 4.3402E-11, 3.7847E-11, 3.2673E-11, 2.7879E-11,
     &   2.3465E-11, 2.3045E-11, 2.2628E-11, 2.2215E-11, 2.1806E-11,
     &   2.1401E-11, 2.1000E-11, 2.0602E-11, 2.0208E-11, 1.9818E-11,
     &   1.9432E-11, 1.9049E-11, 1.8671E-11, 1.8296E-11, 1.7925E-11,
     &   1.7557E-11, 1.7194E-11, 1.6834E-11, 1.6478E-11, 1.6126E-11/

         DATA ((DGT  (I,J), I= 601, 700), J=2,2) /
     &   1.5778E-11, 1.5434E-11, 1.5093E-11, 1.4756E-11, 1.4423E-11,
     &   1.4094E-11, 1.3769E-11, 1.3447E-11, 1.3129E-11, 1.2815E-11,
     &   1.2505E-11, 1.2198E-11, 1.1896E-11, 1.1597E-11, 1.1302E-11,
     &   1.1011E-11, 1.0723E-11, 1.0440E-11, 1.0160E-11, 9.8837E-12,
     &   9.6115E-12, 9.3430E-12, 9.0784E-12, 8.8176E-12, 8.5606E-12,
     &   8.3074E-12, 8.0580E-12, 7.8124E-12, 7.5705E-12, 7.3325E-12,
     &   7.0983E-12, 6.8679E-12, 6.6413E-12, 6.4185E-12, 6.1995E-12,
     &   5.9843E-12, 5.7729E-12, 5.5653E-12, 5.3615E-12, 5.1615E-12,
     &   4.9653E-12, 4.7729E-12, 4.5843E-12, 4.3995E-12, 4.2185E-12,
     &   4.0413E-12, 3.8680E-12, 3.6984E-12, 3.5326E-12, 3.3706E-12,
     &   3.2124E-12, 3.0581E-12, 2.9075E-12, 2.7607E-12, 2.6177E-12,
     &   2.4785E-12, 2.3432E-12, 2.2116E-12, 2.0838E-12, 1.9599E-12,
     &   1.8397E-12, 1.7233E-12, 1.6108E-12, 1.5020E-12, 1.3970E-12,
     &   1.2959E-12, 1.1985E-12, 1.1050E-12, 1.0152E-12, 9.2926E-13,
     &   8.4711E-13, 7.6875E-13, 6.9420E-13, 6.2346E-13, 5.5651E-13,
     &   4.9336E-13, 4.3402E-13, 3.7848E-13, 3.2673E-13, 2.7879E-13,
     &   2.3466E-13, 2.3045E-13, 2.2628E-13, 2.2216E-13, 2.1806E-13,
     &   2.1401E-13, 2.1000E-13, 2.0602E-13, 2.0208E-13, 1.9818E-13,
     &   1.9432E-13, 1.9049E-13, 1.8671E-13, 1.8296E-13, 1.7925E-13,
     &   1.7557E-13, 1.7194E-13, 1.6834E-13, 1.6479E-13, 1.6126E-13/

         DATA ((DGT  (I,J), I= 701, 800), J=2,2) /
     &   1.5778E-13, 1.5434E-13, 1.5093E-13, 1.4756E-13, 1.4423E-13,
     &   1.4094E-13, 1.3769E-13, 1.3447E-13, 1.3129E-13, 1.2815E-13,
     &   1.2505E-13, 1.2198E-13, 1.1896E-13, 1.1597E-13, 1.1302E-13,
     &   1.1011E-13, 1.0723E-13, 1.0440E-13, 1.0160E-13, 9.8837E-14,
     &   9.6115E-14, 9.3431E-14, 9.0784E-14, 8.8176E-14, 8.5606E-14,
     &   8.3074E-14, 8.0580E-14, 7.8124E-14, 7.5705E-14, 7.3325E-14,
     &   7.0983E-14, 6.8679E-14, 6.6413E-14, 6.4185E-14, 6.1995E-14,
     &   5.9843E-14, 5.7729E-14, 5.5653E-14, 5.3615E-14, 5.1615E-14,
     &   4.9653E-14, 4.7729E-14, 4.5843E-14, 4.3995E-14, 4.2185E-14,
     &   4.0414E-14, 3.8680E-14, 3.6984E-14, 3.5326E-14, 3.3706E-14,
     &   3.2124E-14, 3.0581E-14, 2.9075E-14, 2.7607E-14, 2.6177E-14,
     &   2.4785E-14, 2.3432E-14, 2.2116E-14, 2.0838E-14, 1.9599E-14,
     &   1.8397E-14, 1.7233E-14, 1.6108E-14, 1.5020E-14, 1.3970E-14,
     &   1.2959E-14, 1.1985E-14, 1.1050E-14, 1.0152E-14, 9.2926E-15,
     &   8.4711E-15, 7.6875E-15, 6.9420E-15, 6.2346E-15, 5.5651E-15,
     &   4.9336E-15, 4.3402E-15, 3.7848E-15, 3.2673E-15, 2.7879E-15,
     &   2.3466E-15, 2.3045E-15, 2.2628E-15, 2.2216E-15, 2.1806E-15,
     &   2.1401E-15, 2.1000E-15, 2.0602E-15, 2.0208E-15, 1.9818E-15,
     &   1.9432E-15, 1.9049E-15, 1.8671E-15, 1.8296E-15, 1.7925E-15,
     &   1.7558E-15, 1.7194E-15, 1.6834E-15, 1.6479E-15, 1.6126E-15/

         DATA ((DGT  (I,J), I= 801, 900), J=2,2) /
     &   1.5778E-15, 1.5434E-15, 1.5093E-15, 1.4756E-15, 1.4423E-15,
     &   1.4094E-15, 1.3769E-15, 1.3447E-15, 1.3129E-15, 1.2815E-15,
     &   1.2505E-15, 1.2198E-15, 1.1896E-15, 1.1597E-15, 1.1302E-15,
     &   1.1011E-15, 1.0723E-15, 1.0440E-15, 1.0160E-15, 9.8837E-16,
     &   9.6115E-16, 9.3431E-16, 9.0784E-16, 8.8176E-16, 8.5606E-16,
     &   8.3074E-16, 8.0580E-16, 7.8124E-16, 7.5705E-16, 7.3325E-16,
     &   7.0983E-16, 6.8679E-16, 6.6413E-16, 6.4185E-16, 6.1995E-16,
     &   5.9843E-16, 5.7729E-16, 5.5653E-16, 5.3615E-16, 5.1615E-16,
     &   4.9653E-16, 4.7729E-16, 4.5843E-16, 4.3995E-16, 4.2185E-16,
     &   4.0414E-16, 3.8680E-16, 3.6984E-16, 3.5326E-16, 3.3706E-16,
     &   3.2124E-16, 3.0581E-16, 2.9075E-16, 2.7607E-16, 2.6177E-16,
     &   2.4785E-16, 2.3432E-16, 2.2116E-16, 2.0838E-16, 1.9599E-16,
     &   1.8397E-16, 1.7233E-16, 1.6108E-16, 1.5020E-16, 1.3970E-16,
     &   1.2959E-16, 1.1985E-16, 1.1050E-16, 1.0152E-16, 9.2926E-17,
     &   8.4711E-17, 7.6875E-17, 6.9420E-17, 6.2346E-17, 5.5651E-17,
     &   4.9336E-17, 4.3402E-17, 3.7848E-17, 3.2673E-17, 2.7879E-17,
     &   2.3466E-17, 0.0000E+00, 2.3466E-17, 2.1134E-10, 8.4513E-10,
     &   1.9016E-09, 3.3804E-09, 5.2830E-09, 7.6097E-09, 1.0361E-08,
     &   1.3538E-08, 1.7141E-08, 2.1172E-08, 2.5632E-08, 3.0521E-08,
     &   3.5843E-08, 4.1598E-08, 4.7787E-08, 5.4414E-08, 6.1480E-08/

         DATA ((DGT  (I,J), I= 901, 979), J=2,2) /
     &   6.8988E-08, 7.6939E-08, 8.5337E-08, 9.4184E-08, 1.0348E-07,
     &   1.1324E-07, 1.2345E-07, 1.3412E-07, 1.4526E-07, 1.5687E-07,
     &   1.6896E-07, 1.8152E-07, 1.9456E-07, 2.0809E-07, 2.2212E-07,
     &   2.3664E-07, 2.5166E-07, 2.6720E-07, 2.8325E-07, 2.9982E-07,
     &   3.1692E-07, 3.3456E-07, 3.5274E-07, 3.7148E-07, 3.9077E-07,
     &   4.1064E-07, 4.3109E-07, 4.5212E-07, 4.7376E-07, 4.9601E-07,
     &   5.1889E-07, 5.4240E-07, 5.6657E-07, 5.9140E-07, 6.1691E-07,
     &   6.4312E-07, 6.7005E-07, 6.9771E-07, 7.2613E-07, 7.5533E-07,
     &   7.8532E-07, 8.1615E-07, 8.4782E-07, 8.8039E-07, 9.1387E-07,
     &   9.4830E-07, 9.8373E-07, 1.0202E-06, 1.0577E-06, 1.0964E-06,
     &   1.1363E-06, 1.1774E-06, 1.2198E-06, 1.2637E-06, 1.3090E-06,
     &   1.3560E-06, 1.4047E-06, 1.4552E-06, 1.5077E-06, 1.5625E-06,
     &   1.6196E-06, 1.6794E-06, 1.7422E-06, 1.8084E-06, 1.8783E-06,
     &   1.9526E-06, 2.0321E-06, 2.1175E-06, 2.2103E-06, 2.3120E-06,
     &   2.4251E-06, 2.5531E-06, 2.7019E-06, 2.7577E-06, 2.8817E-06,
     &   2.93007507E-06, 3.11547513E-06, 3.14376905E-06, 3.21523598E-06/

         DATA ((DGT  (I,J), I=   1, 100), J=3,3) /
     &   1.6453E-04, 1.6453E-04, 1.6453E-04, 1.6453E-04, 1.6452E-04,
     &   1.6452E-04, 1.6452E-04, 1.6452E-04, 1.6452E-04, 1.6452E-04,
     &   1.6452E-04, 1.6452E-04, 1.6452E-04, 1.6452E-04, 1.6452E-04,
     &   1.6452E-04, 1.6452E-04, 1.6452E-04, 1.6451E-04, 1.6451E-04,
     &   1.6451E-04, 1.6451E-04, 1.6451E-04, 1.6451E-04, 1.6451E-04,
     &   1.6451E-04, 1.6451E-04, 1.6451E-04, 1.6451E-04, 1.6451E-04,
     &   1.6450E-04, 1.6450E-04, 1.6450E-04, 1.6450E-04, 1.6450E-04,
     &   1.6450E-04, 1.6450E-04, 1.6450E-04, 1.6450E-04, 1.6449E-04,
     &   1.6449E-04, 1.6449E-04, 1.6449E-04, 1.6449E-04, 1.6449E-04,
     &   1.6449E-04, 1.6449E-04, 1.6448E-04, 1.6448E-04, 1.6448E-04,
     &   1.6448E-04, 1.6448E-04, 1.6448E-04, 1.6447E-04, 1.6447E-04,
     &   1.6447E-04, 1.6447E-04, 1.6446E-04, 1.6446E-04, 1.6446E-04,
     &   1.6446E-04, 1.6445E-04, 1.6445E-04, 1.6445E-04, 1.6445E-04,
     &   1.6444E-04, 1.6444E-04, 1.6444E-04, 1.6443E-04, 1.6443E-04,
     &   1.6442E-04, 1.6442E-04, 1.6441E-04, 1.6441E-04, 1.6440E-04,
     &   1.6439E-04, 1.6439E-04, 1.6438E-04, 1.6437E-04, 1.6436E-04,
     &   1.6435E-04, 1.6433E-04, 1.6432E-04, 1.6430E-04, 1.6428E-04,
     &   1.6426E-04, 1.6423E-04, 1.6419E-04, 1.6415E-04, 1.6410E-04,
     &   1.6403E-04, 1.6402E-04, 1.6401E-04, 1.6400E-04, 1.6400E-04,
     &   1.6399E-04, 1.6398E-04, 1.6397E-04, 1.6396E-04, 1.6395E-04/

         DATA ((DGT  (I,J), I= 101, 200), J=3,3) /
     &   1.6394E-04, 1.6393E-04, 1.6392E-04, 1.6390E-04, 1.6389E-04,
     &   1.6388E-04, 1.6387E-04, 1.6386E-04, 1.6384E-04, 1.6383E-04,
     &   1.6381E-04, 1.6380E-04, 1.6378E-04, 1.6377E-04, 1.6375E-04,
     &   1.6373E-04, 1.6372E-04, 1.6370E-04, 1.6368E-04, 1.6366E-04,
     &   1.6364E-04, 1.6361E-04, 1.6359E-04, 1.6357E-04, 1.6354E-04,
     &   1.6352E-04, 1.6349E-04, 1.6346E-04, 1.6343E-04, 1.6340E-04,
     &   1.6337E-04, 1.6334E-04, 1.6330E-04, 1.6326E-04, 1.6323E-04,
     &   1.6318E-04, 1.6314E-04, 1.6310E-04, 1.6305E-04, 1.6300E-04,
     &   1.6294E-04, 1.6289E-04, 1.6283E-04, 1.6277E-04, 1.6270E-04,
     &   1.6263E-04, 1.6255E-04, 1.6247E-04, 1.6239E-04, 1.6230E-04,
     &   1.6220E-04, 1.6209E-04, 1.6198E-04, 1.6186E-04, 1.6173E-04,
     &   1.6159E-04, 1.6144E-04, 1.6128E-04, 1.6111E-04, 1.6092E-04,
     &   1.6071E-04, 1.6049E-04, 1.6025E-04, 1.5998E-04, 1.5969E-04,
     &   1.5937E-04, 1.5901E-04, 1.5863E-04, 1.5819E-04, 1.5772E-04,
     &   1.5719E-04, 1.5659E-04, 1.5593E-04, 1.5518E-04, 1.5434E-04,
     &   1.5339E-04, 1.5230E-04, 1.5105E-04, 1.4962E-04, 1.4794E-04,
     &   1.4596E-04, 1.4576E-04, 1.4556E-04, 1.4536E-04, 1.4516E-04,
     &   1.4495E-04, 1.4473E-04, 1.4452E-04, 1.4430E-04, 1.4407E-04,
     &   1.4384E-04, 1.4361E-04, 1.4337E-04, 1.4313E-04, 1.4288E-04,
     &   1.4263E-04, 1.4238E-04, 1.4212E-04, 1.4185E-04, 1.4158E-04/

         DATA ((DGT  (I,J), I= 201, 300), J=3,3) /
     &   1.4130E-04, 1.4102E-04, 1.4073E-04, 1.4044E-04, 1.4014E-04,
     &   1.3983E-04, 1.3952E-04, 1.3920E-04, 1.3888E-04, 1.3854E-04,
     &   1.3820E-04, 1.3785E-04, 1.3749E-04, 1.3713E-04, 1.3675E-04,
     &   1.3637E-04, 1.3597E-04, 1.3557E-04, 1.3516E-04, 1.3474E-04,
     &   1.3430E-04, 1.3386E-04, 1.3340E-04, 1.3293E-04, 1.3245E-04,
     &   1.3195E-04, 1.3145E-04, 1.3092E-04, 1.3039E-04, 1.2983E-04,
     &   1.2926E-04, 1.2868E-04, 1.2808E-04, 1.2746E-04, 1.2682E-04,
     &   1.2616E-04, 1.2548E-04, 1.2478E-04, 1.2405E-04, 1.2331E-04,
     &   1.2253E-04, 1.2174E-04, 1.2091E-04, 1.2006E-04, 1.1918E-04,
     &   1.1827E-04, 1.1732E-04, 1.1635E-04, 1.1533E-04, 1.1428E-04,
     &   1.1319E-04, 1.1206E-04, 1.1088E-04, 1.0966E-04, 1.0839E-04,
     &   1.0706E-04, 1.0569E-04, 1.0425E-04, 1.0275E-04, 1.0119E-04,
     &   9.9559E-05, 9.7852E-05, 9.6067E-05, 9.4198E-05, 9.2238E-05,
     &   9.0183E-05, 8.8025E-05, 8.5757E-05, 8.3372E-05, 8.0862E-05,
     &   7.8219E-05, 7.5433E-05, 7.2496E-05, 6.9400E-05, 6.6135E-05,
     &   6.2693E-05, 5.9068E-05, 5.5254E-05, 5.1247E-05, 4.7049E-05,
     &   4.2664E-05, 4.2216E-05, 4.1766E-05, 4.1315E-05, 4.0861E-05,
     &   4.0406E-05, 3.9950E-05, 3.9491E-05, 3.9032E-05, 3.8570E-05,
     &   3.8107E-05, 3.7643E-05, 3.7177E-05, 3.6710E-05, 3.6241E-05,
     &   3.5771E-05, 3.5299E-05, 3.4826E-05, 3.4352E-05, 3.3877E-05/

         DATA ((DGT  (I,J), I= 301, 400), J=3,3) /
     &   3.3401E-05, 3.2923E-05, 3.2444E-05, 3.1965E-05, 3.1484E-05,
     &   3.1003E-05, 3.0520E-05, 3.0037E-05, 2.9553E-05, 2.9068E-05,
     &   2.8583E-05, 2.8097E-05, 2.7611E-05, 2.7124E-05, 2.6637E-05,
     &   2.6149E-05, 2.5662E-05, 2.5174E-05, 2.4687E-05, 2.4199E-05,
     &   2.3712E-05, 2.3225E-05, 2.2738E-05, 2.2252E-05, 2.1767E-05,
     &   2.1282E-05, 2.0798E-05, 2.0315E-05, 1.9833E-05, 1.9353E-05,
     &   1.8873E-05, 1.8396E-05, 1.7919E-05, 1.7445E-05, 1.6973E-05,
     &   1.6502E-05, 1.6034E-05, 1.5569E-05, 1.5105E-05, 1.4645E-05,
     &   1.4188E-05, 1.3734E-05, 1.3283E-05, 1.2835E-05, 1.2391E-05,
     &   1.1951E-05, 1.1516E-05, 1.1084E-05, 1.0657E-05, 1.0235E-05,
     &   9.8179E-06, 9.4058E-06, 8.9992E-06, 8.5982E-06, 8.2032E-06,
     &   7.8142E-06, 7.4317E-06, 7.0557E-06, 6.6867E-06, 6.3247E-06,
     &   5.9702E-06, 5.6233E-06, 5.2843E-06, 4.9535E-06, 4.6310E-06,
     &   4.3173E-06, 4.0125E-06, 3.7170E-06, 3.4309E-06, 3.1545E-06,
     &   2.8881E-06, 2.6320E-06, 2.3863E-06, 2.1514E-06, 1.9275E-06,
     &   1.7149E-06, 1.5137E-06, 1.3242E-06, 1.1465E-06, 9.8105E-07,
     &   8.2787E-07, 8.1324E-07, 7.9873E-07, 7.8435E-07, 7.7009E-07,
     &   7.5596E-07, 7.4195E-07, 7.2807E-07, 7.1432E-07, 7.0070E-07,
     &   6.8720E-07, 6.7382E-07, 6.6058E-07, 6.4746E-07, 6.3447E-07,
     &   6.2161E-07, 6.0887E-07, 5.9626E-07, 5.8378E-07, 5.7143E-07/

         DATA ((DGT  (I,J), I= 401, 500), J=3,3) /
     &   5.5921E-07, 5.4711E-07, 5.3514E-07, 5.2331E-07, 5.1160E-07,
     &   5.0002E-07, 4.8857E-07, 4.7725E-07, 4.6606E-07, 4.5499E-07,
     &   4.4406E-07, 4.3326E-07, 4.2259E-07, 4.1205E-07, 4.0164E-07,
     &   3.9135E-07, 3.8120E-07, 3.7119E-07, 3.6130E-07, 3.5154E-07,
     &   3.4191E-07, 3.3242E-07, 3.2306E-07, 3.1382E-07, 3.0472E-07,
     &   2.9576E-07, 2.8692E-07, 2.7822E-07, 2.6964E-07, 2.6121E-07,
     &   2.5290E-07, 2.4472E-07, 2.3668E-07, 2.2877E-07, 2.2100E-07,
     &   2.1335E-07, 2.0584E-07, 1.9847E-07, 1.9122E-07, 1.8411E-07,
     &   1.7713E-07, 1.7029E-07, 1.6358E-07, 1.5700E-07, 1.5056E-07,
     &   1.4425E-07, 1.3808E-07, 1.3204E-07, 1.2613E-07, 1.2036E-07,
     &   1.1472E-07, 1.0922E-07, 1.0385E-07, 9.8619E-08, 9.3520E-08,
     &   8.8555E-08, 8.3726E-08, 7.9031E-08, 7.4471E-08, 7.0046E-08,
     &   6.5756E-08, 6.1601E-08, 5.7582E-08, 5.3697E-08, 4.9948E-08,
     &   4.6334E-08, 4.2856E-08, 3.9513E-08, 3.6305E-08, 3.3233E-08,
     &   3.0297E-08, 2.7496E-08, 2.4830E-08, 2.2301E-08, 1.9907E-08,
     &   1.7649E-08, 1.5527E-08, 1.3540E-08, 1.1689E-08, 9.9745E-09,
     &   8.3956E-09, 8.2451E-09, 8.0961E-09, 7.9484E-09, 7.8020E-09,
     &   7.6571E-09, 7.5134E-09, 7.3712E-09, 7.2303E-09, 7.0907E-09,
     &   6.9525E-09, 6.8157E-09, 6.6802E-09, 6.5461E-09, 6.4134E-09,
     &   6.2820E-09, 6.1520E-09, 6.0233E-09, 5.8960E-09, 5.7700E-09/

         DATA ((DGT  (I,J), I= 501, 600), J=3,3) /
     &   5.6454E-09, 5.5222E-09, 5.4003E-09, 5.2798E-09, 5.1607E-09,
     &   5.0429E-09, 4.9264E-09, 4.8114E-09, 4.6977E-09, 4.5853E-09,
     &   4.4743E-09, 4.3647E-09, 4.2564E-09, 4.1495E-09, 4.0439E-09,
     &   3.9397E-09, 3.8369E-09, 3.7354E-09, 3.6353E-09, 3.5365E-09,
     &   3.4391E-09, 3.3431E-09, 3.2484E-09, 3.1551E-09, 3.0631E-09,
     &   2.9725E-09, 2.8833E-09, 2.7954E-09, 2.7089E-09, 2.6237E-09,
     &   2.5399E-09, 2.4575E-09, 2.3764E-09, 2.2967E-09, 2.2183E-09,
     &   2.1413E-09, 2.0657E-09, 1.9914E-09, 1.9185E-09, 1.8469E-09,
     &   1.7767E-09, 1.7079E-09, 1.6404E-09, 1.5743E-09, 1.5095E-09,
     &   1.4461E-09, 1.3841E-09, 1.3234E-09, 1.2641E-09, 1.2061E-09,
     &   1.1495E-09, 1.0943E-09, 1.0404E-09, 9.8786E-10, 9.3670E-10,
     &   8.8690E-10, 8.3846E-10, 7.9138E-10, 7.4566E-10, 7.0130E-10,
     &   6.5830E-10, 6.1666E-10, 5.7638E-10, 5.3746E-10, 4.9991E-10,
     &   4.6371E-10, 4.2887E-10, 3.9539E-10, 3.6328E-10, 3.3252E-10,
     &   3.0312E-10, 2.7509E-10, 2.4841E-10, 2.2309E-10, 1.9914E-10,
     &   1.7654E-10, 1.5531E-10, 1.3543E-10, 1.1692E-10, 9.9762E-11,
     &   8.3968E-11, 8.2463E-11, 8.0972E-11, 7.9495E-11, 7.8031E-11,
     &   7.6581E-11, 7.5144E-11, 7.3721E-11, 7.2312E-11, 7.0916E-11,
     &   6.9534E-11, 6.8165E-11, 6.6810E-11, 6.5469E-11, 6.4141E-11,
     &   6.2827E-11, 6.1526E-11, 6.0239E-11, 5.8966E-11, 5.7706E-11/

         DATA ((DGT  (I,J), I= 601, 700), J=3,3) /
     &   5.6460E-11, 5.5227E-11, 5.4008E-11, 5.2803E-11, 5.1611E-11,
     &   5.0433E-11, 4.9269E-11, 4.8118E-11, 4.6980E-11, 4.5857E-11,
     &   4.4746E-11, 4.3650E-11, 4.2567E-11, 4.1498E-11, 4.0442E-11,
     &   3.9400E-11, 3.8371E-11, 3.7356E-11, 3.6355E-11, 3.5367E-11,
     &   3.4393E-11, 3.3433E-11, 3.2486E-11, 3.1552E-11, 3.0633E-11,
     &   2.9727E-11, 2.8834E-11, 2.7955E-11, 2.7090E-11, 2.6238E-11,
     &   2.5400E-11, 2.4576E-11, 2.3765E-11, 2.2968E-11, 2.2184E-11,
     &   2.1414E-11, 2.0657E-11, 1.9915E-11, 1.9185E-11, 1.8470E-11,
     &   1.7768E-11, 1.7079E-11, 1.6404E-11, 1.5743E-11, 1.5095E-11,
     &   1.4461E-11, 1.3841E-11, 1.3234E-11, 1.2641E-11, 1.2061E-11,
     &   1.1495E-11, 1.0943E-11, 1.0404E-11, 9.8787E-12, 9.3671E-12,
     &   8.8691E-12, 8.3847E-12, 7.9139E-12, 7.4567E-12, 7.0131E-12,
     &   6.5831E-12, 6.1667E-12, 5.7639E-12, 5.3747E-12, 4.9991E-12,
     &   4.6371E-12, 4.2887E-12, 3.9540E-12, 3.6328E-12, 3.3252E-12,
     &   3.0312E-12, 2.7509E-12, 2.4841E-12, 2.2309E-12, 1.9914E-12,
     &   1.7654E-12, 1.5531E-12, 1.3543E-12, 1.1692E-12, 9.9762E-13,
     &   8.3968E-13, 8.2463E-13, 8.0972E-13, 7.9495E-13, 7.8031E-13,
     &   7.6581E-13, 7.5144E-13, 7.3721E-13, 7.2312E-13, 7.0916E-13,
     &   6.9534E-13, 6.8165E-13, 6.6810E-13, 6.5469E-13, 6.4141E-13,
     &   6.2827E-13, 6.1526E-13, 6.0239E-13, 5.8966E-13, 5.7706E-13/

         DATA ((DGT  (I,J), I= 701, 800), J=3,3) /
     &   5.6460E-13, 5.5227E-13, 5.4008E-13, 5.2803E-13, 5.1611E-13,
     &   5.0433E-13, 4.9269E-13, 4.8118E-13, 4.6980E-13, 4.5857E-13,
     &   4.4746E-13, 4.3650E-13, 4.2567E-13, 4.1498E-13, 4.0442E-13,
     &   3.9400E-13, 3.8371E-13, 3.7356E-13, 3.6355E-13, 3.5367E-13,
     &   3.4393E-13, 3.3433E-13, 3.2486E-13, 3.1553E-13, 3.0633E-13,
     &   2.9727E-13, 2.8834E-13, 2.7955E-13, 2.7090E-13, 2.6238E-13,
     &   2.5400E-13, 2.4576E-13, 2.3765E-13, 2.2968E-13, 2.2184E-13,
     &   2.1414E-13, 2.0657E-13, 1.9915E-13, 1.9185E-13, 1.8470E-13,
     &   1.7768E-13, 1.7079E-13, 1.6404E-13, 1.5743E-13, 1.5095E-13,
     &   1.4461E-13, 1.3841E-13, 1.3234E-13, 1.2641E-13, 1.2061E-13,
     &   1.1495E-13, 1.0943E-13, 1.0404E-13, 9.8787E-14, 9.3671E-14,
     &   8.8691E-14, 8.3847E-14, 7.9139E-14, 7.4567E-14, 7.0131E-14,
     &   6.5831E-14, 6.1667E-14, 5.7639E-14, 5.3747E-14, 4.9991E-14,
     &   4.6371E-14, 4.2887E-14, 3.9540E-14, 3.6328E-14, 3.3252E-14,
     &   3.0312E-14, 2.7509E-14, 2.4841E-14, 2.2309E-14, 1.9914E-14,
     &   1.7654E-14, 1.5531E-14, 1.3543E-14, 1.1692E-14, 9.9762E-15,
     &   8.3968E-15, 8.2463E-15, 8.0972E-15, 7.9495E-15, 7.8031E-15,
     &   7.6581E-15, 7.5144E-15, 7.3721E-15, 7.2312E-15, 7.0916E-15,
     &   6.9534E-15, 6.8165E-15, 6.6810E-15, 6.5469E-15, 6.4141E-15,
     &   6.2827E-15, 6.1526E-15, 6.0239E-15, 5.8966E-15, 5.7706E-15/

         DATA ((DGT  (I,J), I= 801, 900), J=3,3) /
     &   5.6460E-15, 5.5227E-15, 5.4008E-15, 5.2803E-15, 5.1611E-15,
     &   5.0433E-15, 4.9269E-15, 4.8118E-15, 4.6980E-15, 4.5857E-15,
     &   4.4746E-15, 4.3650E-15, 4.2567E-15, 4.1498E-15, 4.0442E-15,
     &   3.9400E-15, 3.8371E-15, 3.7356E-15, 3.6355E-15, 3.5367E-15,
     &   3.4393E-15, 3.3433E-15, 3.2486E-15, 3.1553E-15, 3.0633E-15,
     &   2.9727E-15, 2.8834E-15, 2.7955E-15, 2.7090E-15, 2.6238E-15,
     &   2.5400E-15, 2.4576E-15, 2.3765E-15, 2.2968E-15, 2.2184E-15,
     &   2.1414E-15, 2.0657E-15, 1.9915E-15, 1.9185E-15, 1.8470E-15,
     &   1.7768E-15, 1.7079E-15, 1.6404E-15, 1.5743E-15, 1.5095E-15,
     &   1.4461E-15, 1.3841E-15, 1.3234E-15, 1.2641E-15, 1.2061E-15,
     &   1.1495E-15, 1.0943E-15, 1.0404E-15, 9.8787E-16, 9.3671E-16,
     &   8.8691E-16, 8.3847E-16, 7.9139E-16, 7.4567E-16, 7.0131E-16,
     &   6.5831E-16, 6.1667E-16, 5.7639E-16, 5.3747E-16, 4.9991E-16,
     &   4.6371E-16, 4.2887E-16, 3.9540E-16, 3.6328E-16, 3.3252E-16,
     &   3.0312E-16, 2.7509E-16, 2.4841E-16, 2.2309E-16, 1.9914E-16,
     &   1.7654E-16, 1.5531E-16, 1.3543E-16, 1.1692E-16, 9.9762E-17,
     &   8.3968E-17, 0.0000E+00, 8.3968E-17, 7.5622E-10, 3.0240E-09,
     &   6.8037E-09, 1.2094E-08, 1.8899E-08, 2.7218E-08, 3.7054E-08,
     &   4.8406E-08, 6.1277E-08, 7.5670E-08, 9.1586E-08, 1.0903E-07,
     &   1.2800E-07, 1.4850E-07, 1.7054E-07, 1.9411E-07, 2.1923E-07/

         DATA ((DGT  (I,J), I= 901, 979), J=3,3) /
     &   2.4590E-07, 2.7411E-07, 3.0388E-07, 3.3521E-07, 3.6811E-07,
     &   4.0258E-07, 4.3862E-07, 4.7625E-07, 5.1547E-07, 5.5628E-07,
     &   5.9869E-07, 6.4272E-07, 6.8836E-07, 7.3563E-07, 7.8454E-07,
     &   8.3508E-07, 8.8728E-07, 9.4114E-07, 9.9667E-07, 1.0539E-06,
     &   1.1128E-06, 1.1734E-06, 1.2357E-06, 1.2998E-06, 1.3656E-06,
     &   1.4331E-06, 1.5025E-06, 1.5736E-06, 1.6465E-06, 1.7212E-06,
     &   1.7977E-06, 1.8761E-06, 1.9563E-06, 2.0385E-06, 2.1225E-06,
     &   2.2084E-06, 2.2963E-06, 2.3862E-06, 2.4780E-06, 2.5719E-06,
     &   2.6678E-06, 2.7658E-06, 2.8658E-06, 2.9680E-06, 3.0723E-06,
     &   3.1788E-06, 3.2875E-06, 3.3985E-06, 3.5118E-06, 3.6274E-06,
     &   3.7454E-06, 3.8658E-06, 3.9887E-06, 4.1142E-06, 4.2422E-06,
     &   4.3728E-06, 4.5062E-06, 4.6424E-06, 4.7814E-06, 4.9234E-06,
     &   5.0684E-06, 5.2166E-06, 5.3680E-06, 5.5229E-06, 5.6813E-06,
     &   5.8434E-06, 6.0093E-06, 6.1794E-06, 6.3539E-06, 6.5330E-06,
     &   6.7171E-06, 6.9066E-06, 7.1022E-06, 7.1689E-06, 7.3046E-06,
     &   7.35292550E-06, 7.51494345E-06, 7.53647879E-06, 7.58712056E-06/


c      ++++ dalhadthigh19 ++++

c update 12/24/19

c******************************************************************************
c*                                                                            *
c*    --- intRdatx ---                              Tue Dec 24 18:14:43 2019  *
c*                                                                            *
c***************************# FJ@HU Berlin #***********************************

c  "Delta alpha"               Update version (24/02/2019)

c ISCAN: -2, Ismooth: 0, IGS: 0, DSET: WAVE, iplot: 0 itestflavorspl: 0
c IKE,Emin,Emax,NP,N,EPSREL:  1,-2.0000E+04,-1.0000E+03, 100, 10, 1.00E-02,
c ALS(sta)(sys):  0.1184( 0.0007)( 0.0000), MZINP:   91.1876, MTOP:    173.20
c NF,IOR,ICHK: 5,4,2,ISU2: 0,NFL: 5,INAG: 1,inotail:  0, sin2W:   0.23153
c CHPTCUT,EC,ECUT: 0.3180,     5.20,    11.50


       DATA  (XXX  (I), I=   1, 100) /
     &  -2.0000E+04,-1.9701E+04,-1.9407E+04,-1.9117E+04,-1.8831E+04,
     &  -1.8550E+04,-1.8273E+04,-1.8000E+04,-1.7731E+04,-1.7466E+04,
     &  -1.7205E+04,-1.6948E+04,-1.6695E+04,-1.6445E+04,-1.6199E+04,
     &  -1.5957E+04,-1.5719E+04,-1.5484E+04,-1.5253E+04,-1.5025E+04,
     &  -1.4800E+04,-1.4579E+04,-1.4361E+04,-1.4147E+04,-1.3935E+04,
     &  -1.3727E+04,-1.3522E+04,-1.3320E+04,-1.3121E+04,-1.2925E+04,
     &  -1.2732E+04,-1.2542E+04,-1.2354E+04,-1.2170E+04,-1.1988E+04,
     &  -1.1809E+04,-1.1632E+04,-1.1459E+04,-1.1287E+04,-1.1119E+04,
     &  -1.0953E+04,-1.0789E+04,-1.0628E+04,-1.0469E+04,-1.0313E+04,
     &  -1.0158E+04,-1.0007E+04,-9.8572E+03,-9.7099E+03,-9.5648E+03,
     &  -9.4219E+03,-9.2811E+03,-9.1424E+03,-9.0058E+03,-8.8713E+03,
     &  -8.7387E+03,-8.6082E+03,-8.4796E+03,-8.3529E+03,-8.2281E+03,
     &  -8.1051E+03,-7.9840E+03,-7.8647E+03,-7.7472E+03,-7.6315E+03,
     &  -7.5174E+03,-7.4051E+03,-7.2945E+03,-7.1855E+03,-7.0781E+03,
     &  -6.9724E+03,-6.8682E+03,-6.7656E+03,-6.6645E+03,-6.5649E+03,
     &  -6.4668E+03,-6.3702E+03,-6.2750E+03,-6.1813E+03,-6.0889E+03,
     &  -5.9980E+03,-5.9083E+03,-5.8201E+03,-5.7331E+03,-5.6474E+03,
     &  -5.5631E+03,-5.4799E+03,-5.3981E+03,-5.3174E+03,-5.2380E+03,
     &  -5.1597E+03,-5.0826E+03,-5.0067E+03,-4.9319E+03,-4.8582E+03,
     &  -4.7856E+03,-4.7141E+03,-4.6437E+03,-4.5743E+03,-4.5059E+03/

       DATA  (XXX  (I), I= 101, 200) /
     &  -4.4386E+03,-4.3723E+03,-4.3070E+03,-4.2426E+03,-4.1792E+03,
     &  -4.1168E+03,-4.0553E+03,-3.9947E+03,-3.9350E+03,-3.8762E+03,
     &  -3.8183E+03,-3.7612E+03,-3.7050E+03,-3.6497E+03,-3.5951E+03,
     &  -3.5414E+03,-3.4885E+03,-3.4364E+03,-3.3850E+03,-3.3345E+03,
     &  -3.2847E+03,-3.2356E+03,-3.1872E+03,-3.1396E+03,-3.0927E+03,
     &  -3.0465E+03,-3.0010E+03,-2.9561E+03,-2.9120E+03,-2.8685E+03,
     &  -2.8256E+03,-2.7834E+03,-2.7418E+03,-2.7008E+03,-2.6605E+03,
     &  -2.6207E+03,-2.5816E+03,-2.5430E+03,-2.5050E+03,-2.4676E+03,
     &  -2.4307E+03,-2.3944E+03,-2.3586E+03,-2.3234E+03,-2.2887E+03,
     &  -2.2545E+03,-2.2208E+03,-2.1876E+03,-2.1549E+03,-2.1227E+03,
     &  -2.0910E+03,-2.0598E+03,-2.0290E+03,-1.9987E+03,-1.9688E+03,
     &  -1.9394E+03,-1.9104E+03,-1.8819E+03,-1.8538E+03,-1.8261E+03,
     &  -1.7988E+03,-1.7719E+03,-1.7454E+03,-1.7193E+03,-1.6937E+03,
     &  -1.6683E+03,-1.6434E+03,-1.6189E+03,-1.5947E+03,-1.5709E+03,
     &  -1.5474E+03,-1.5243E+03,-1.5015E+03,-1.4791E+03,-1.4570E+03,
     &  -1.4352E+03,-1.4137E+03,-1.3926E+03,-1.3718E+03,-1.3513E+03,
     &  -1.3311E+03,-1.3112E+03,-1.2916E+03,-1.2723E+03,-1.2533E+03,
     &  -1.2346E+03,-1.2162E+03,-1.1980E+03,-1.1801E+03,-1.1625E+03,
     &  -1.1451E+03,-1.1280E+03,-1.1111E+03,-1.0945E+03,-1.0782E+03,
     &  -1.0621E+03,-1.0462E+03,-1.0306E+03,-1.0152E+03,-1.0000E+03/


         DATA ((YGG  (I,J), I=   1, 100), J=1,1) /
     &   5.7103E-02, 5.7017E-02, 5.6930E-02, 5.6844E-02, 5.6757E-02,
     &   5.6671E-02, 5.6584E-02, 5.6498E-02, 5.6412E-02, 5.6325E-02,
     &   5.6239E-02, 5.6153E-02, 5.6067E-02, 5.5980E-02, 5.5894E-02,
     &   5.5808E-02, 5.5722E-02, 5.5636E-02, 5.5550E-02, 5.5464E-02,
     &   5.5378E-02, 5.5292E-02, 5.5206E-02, 5.5120E-02, 5.5034E-02,
     &   5.4948E-02, 5.4862E-02, 5.4776E-02, 5.4691E-02, 5.4605E-02,
     &   5.4519E-02, 5.4434E-02, 5.4348E-02, 5.4262E-02, 5.4177E-02,
     &   5.4091E-02, 5.4006E-02, 5.3921E-02, 5.3835E-02, 5.3750E-02,
     &   5.3665E-02, 5.3580E-02, 5.3494E-02, 5.3409E-02, 5.3324E-02,
     &   5.3239E-02, 5.3154E-02, 5.3069E-02, 5.2985E-02, 5.2900E-02,
     &   5.2815E-02, 5.2730E-02, 5.2646E-02, 5.2561E-02, 5.2477E-02,
     &   5.2392E-02, 5.2308E-02, 5.2223E-02, 5.2139E-02, 5.2055E-02,
     &   5.1971E-02, 5.1887E-02, 5.1803E-02, 5.1719E-02, 5.1635E-02,
     &   5.1551E-02, 5.1467E-02, 5.1384E-02, 5.1300E-02, 5.1217E-02,
     &   5.1133E-02, 5.1050E-02, 5.0967E-02, 5.0883E-02, 5.0800E-02,
     &   5.0717E-02, 5.0634E-02, 5.0551E-02, 5.0469E-02, 5.0386E-02,
     &   5.0303E-02, 5.0221E-02, 5.0138E-02, 5.0056E-02, 4.9974E-02,
     &   4.9892E-02, 4.9810E-02, 4.9728E-02, 4.9646E-02, 4.9564E-02,
     &   4.9482E-02, 4.9401E-02, 4.9319E-02, 4.9238E-02, 4.9157E-02,
     &   4.9075E-02, 4.8994E-02, 4.8913E-02, 4.8832E-02, 4.8752E-02/

         DATA ((YGG  (I,J), I= 101, 200), J=1,1) /
     &   4.8671E-02, 4.8590E-02, 4.8510E-02, 4.8430E-02, 4.8349E-02,
     &   4.8269E-02, 4.8189E-02, 4.8109E-02, 4.8030E-02, 4.7950E-02,
     &   4.7870E-02, 4.7791E-02, 4.7711E-02, 4.7632E-02, 4.7553E-02,
     &   4.7474E-02, 4.7395E-02, 4.7316E-02, 4.7237E-02, 4.7159E-02,
     &   4.7080E-02, 4.7002E-02, 4.6924E-02, 4.6846E-02, 4.6767E-02,
     &   4.6690E-02, 4.6612E-02, 4.6534E-02, 4.6456E-02, 4.6379E-02,
     &   4.6301E-02, 4.6224E-02, 4.6147E-02, 4.6070E-02, 4.5993E-02,
     &   4.5916E-02, 4.5839E-02, 4.5762E-02, 4.5686E-02, 4.5609E-02,
     &   4.5532E-02, 4.5456E-02, 4.5380E-02, 4.5304E-02, 4.5227E-02,
     &   4.5151E-02, 4.5075E-02, 4.4999E-02, 4.4924E-02, 4.4848E-02,
     &   4.4772E-02, 4.4696E-02, 4.4621E-02, 4.4545E-02, 4.4470E-02,
     &   4.4394E-02, 4.4319E-02, 4.4244E-02, 4.4168E-02, 4.4093E-02,
     &   4.4018E-02, 4.3943E-02, 4.3867E-02, 4.3792E-02, 4.3717E-02,
     &   4.3642E-02, 4.3567E-02, 4.3492E-02, 4.3416E-02, 4.3341E-02,
     &   4.3266E-02, 4.3191E-02, 4.3116E-02, 4.3041E-02, 4.2965E-02,
     &   4.2890E-02, 4.2815E-02, 4.2740E-02, 4.2664E-02, 4.2589E-02,
     &   4.2514E-02, 4.2438E-02, 4.2363E-02, 4.2287E-02, 4.2211E-02,
     &   4.2136E-02, 4.2060E-02, 4.1984E-02, 4.1908E-02, 4.1832E-02,
     &   4.1756E-02, 4.1680E-02, 4.1604E-02, 4.1527E-02, 4.1451E-02,
     &   4.1374E-02, 4.1297E-02, 4.1221E-02, 4.1144E-02, 4.1067E-02/


         DATA ((YGG  (I,J), I=   1, 100), J=2,2) /
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05/

         DATA ((YGG  (I,J), I= 101, 200), J=2,2) /
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05, 5.3368E-05,
     &   5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05,
     &   5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05,
     &   5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05,
     &   5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05,
     &   5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05, 5.3367E-05/


         DATA ((YGG  (I,J), I=   1, 100), J=3,3) /
     &   9.4795E-05, 9.4794E-05, 9.4793E-05, 9.4793E-05, 9.4792E-05,
     &   9.4792E-05, 9.4791E-05, 9.4791E-05, 9.4790E-05, 9.4790E-05,
     &   9.4789E-05, 9.4789E-05, 9.4788E-05, 9.4787E-05, 9.4787E-05,
     &   9.4786E-05, 9.4786E-05, 9.4785E-05, 9.4785E-05, 9.4784E-05,
     &   9.4784E-05, 9.4783E-05, 9.4782E-05, 9.4782E-05, 9.4781E-05,
     &   9.4781E-05, 9.4780E-05, 9.4780E-05, 9.4779E-05, 9.4779E-05,
     &   9.4778E-05, 9.4777E-05, 9.4777E-05, 9.4776E-05, 9.4776E-05,
     &   9.4775E-05, 9.4775E-05, 9.4774E-05, 9.4773E-05, 9.4773E-05,
     &   9.4772E-05, 9.4772E-05, 9.4771E-05, 9.4771E-05, 9.4770E-05,
     &   9.4769E-05, 9.4769E-05, 9.4768E-05, 9.4768E-05, 9.4767E-05,
     &   9.4767E-05, 9.4766E-05, 9.4765E-05, 9.4765E-05, 9.4764E-05,
     &   9.4764E-05, 9.4763E-05, 9.4763E-05, 9.4762E-05, 9.4761E-05,
     &   9.4761E-05, 9.4760E-05, 9.4760E-05, 9.4759E-05, 9.4758E-05,
     &   9.4758E-05, 9.4757E-05, 9.4757E-05, 9.4756E-05, 9.4756E-05,
     &   9.4755E-05, 9.4754E-05, 9.4754E-05, 9.4753E-05, 9.4753E-05,
     &   9.4752E-05, 9.4751E-05, 9.4751E-05, 9.4750E-05, 9.4750E-05,
     &   9.4749E-05, 9.4749E-05, 9.4748E-05, 9.4747E-05, 9.4747E-05,
     &   9.4746E-05, 9.4746E-05, 9.4745E-05, 9.4744E-05, 9.4744E-05,
     &   9.4743E-05, 9.4743E-05, 9.4742E-05, 9.4741E-05, 9.4741E-05,
     &   9.4740E-05, 9.4740E-05, 9.4739E-05, 9.4738E-05, 9.4738E-05/

         DATA ((YGG  (I,J), I= 101, 200), J=3,3) /
     &   9.4737E-05, 9.4737E-05, 9.4736E-05, 9.4736E-05, 9.4735E-05,
     &   9.4734E-05, 9.4734E-05, 9.4733E-05, 9.4733E-05, 9.4732E-05,
     &   9.4731E-05, 9.4731E-05, 9.4730E-05, 9.4730E-05, 9.4729E-05,
     &   9.4728E-05, 9.4728E-05, 9.4727E-05, 9.4727E-05, 9.4726E-05,
     &   9.4725E-05, 9.4725E-05, 9.4724E-05, 9.4724E-05, 9.4723E-05,
     &   9.4722E-05, 9.4722E-05, 9.4721E-05, 9.4721E-05, 9.4720E-05,
     &   9.4719E-05, 9.4719E-05, 9.4718E-05, 9.4718E-05, 9.4717E-05,
     &   9.4716E-05, 9.4716E-05, 9.4715E-05, 9.4715E-05, 9.4714E-05,
     &   9.4713E-05, 9.4713E-05, 9.4712E-05, 9.4712E-05, 9.4711E-05,
     &   9.4710E-05, 9.4710E-05, 9.4709E-05, 9.4708E-05, 9.4708E-05,
     &   9.4707E-05, 9.4707E-05, 9.4706E-05, 9.4705E-05, 9.4705E-05,
     &   9.4704E-05, 9.4704E-05, 9.4703E-05, 9.4702E-05, 9.4702E-05,
     &   9.4701E-05, 9.4700E-05, 9.4700E-05, 9.4699E-05, 9.4698E-05,
     &   9.4698E-05, 9.4697E-05, 9.4697E-05, 9.4696E-05, 9.4695E-05,
     &   9.4695E-05, 9.4694E-05, 9.4693E-05, 9.4693E-05, 9.4692E-05,
     &   9.4691E-05, 9.4691E-05, 9.4690E-05, 9.4689E-05, 9.4689E-05,
     &   9.4688E-05, 9.4687E-05, 9.4687E-05, 9.4686E-05, 9.4685E-05,
     &   9.4685E-05, 9.4684E-05, 9.4683E-05, 9.4683E-05, 9.4682E-05,
     &   9.4681E-05, 9.4681E-05, 9.4680E-05, 9.4679E-05, 9.4679E-05,
     &   9.4678E-05, 9.4677E-05, 9.4676E-05, 9.4676E-05, 9.4675E-05/


c      ++++ deghadthigh19 ++++

c update 12/28/19

c******************************************************************************
c*                                                                            *
c*    --- intRdatx ---                              Sat Dec 28 10:32:26 2019  *
c*                                                                            *
c***************************# FJ@HU Berlin #***********************************

c  "Delta alpha"               Update version (24/02/2019)

c ISCAN: -2, Ismooth: 0, IGS: 0, DSET: WAVE, iplot: 0 itestflavorspl: 0
c IKE,Emin,Emax,NP,N,EPSREL:  1,-2.0000E+04,-1.0000E+03, 100, 10, 1.00E-02,
c ALS(sta)(sys):  0.1184( 0.0007)( 0.0000), MZINP:   91.1876, MTOP:    173.20
c NF,IOR,ICHK: 5,4,2,ISU2: 2,NFL: 5,INAG: 1,inotail:  0, sin2W:   0.23153
c CHPTCUT,EC,ECUT: 0.3180,     5.20,    11.50


         DATA ((Y3G  (I,J), I=   1, 100), J=1,1) /
     &   1.1707E-01, 1.1690E-01, 1.1672E-01, 1.1654E-01, 1.1636E-01,
     &   1.1618E-01, 1.1600E-01, 1.1583E-01, 1.1565E-01, 1.1547E-01,
     &   1.1529E-01, 1.1512E-01, 1.1494E-01, 1.1476E-01, 1.1458E-01,
     &   1.1440E-01, 1.1423E-01, 1.1405E-01, 1.1387E-01, 1.1369E-01,
     &   1.1352E-01, 1.1334E-01, 1.1316E-01, 1.1299E-01, 1.1281E-01,
     &   1.1263E-01, 1.1245E-01, 1.1228E-01, 1.1210E-01, 1.1192E-01,
     &   1.1175E-01, 1.1157E-01, 1.1139E-01, 1.1122E-01, 1.1104E-01,
     &   1.1087E-01, 1.1069E-01, 1.1051E-01, 1.1034E-01, 1.1016E-01,
     &   1.0999E-01, 1.0981E-01, 1.0964E-01, 1.0946E-01, 1.0928E-01,
     &   1.0911E-01, 1.0893E-01, 1.0876E-01, 1.0858E-01, 1.0841E-01,
     &   1.0823E-01, 1.0806E-01, 1.0789E-01, 1.0771E-01, 1.0754E-01,
     &   1.0736E-01, 1.0719E-01, 1.0701E-01, 1.0684E-01, 1.0667E-01,
     &   1.0649E-01, 1.0632E-01, 1.0615E-01, 1.0597E-01, 1.0580E-01,
     &   1.0563E-01, 1.0546E-01, 1.0528E-01, 1.0511E-01, 1.0494E-01,
     &   1.0477E-01, 1.0460E-01, 1.0442E-01, 1.0425E-01, 1.0408E-01,
     &   1.0391E-01, 1.0374E-01, 1.0357E-01, 1.0340E-01, 1.0323E-01,
     &   1.0306E-01, 1.0289E-01, 1.0272E-01, 1.0255E-01, 1.0238E-01,
     &   1.0221E-01, 1.0204E-01, 1.0187E-01, 1.0170E-01, 1.0153E-01,
     &   1.0136E-01, 1.0120E-01, 1.0103E-01, 1.0086E-01, 1.0069E-01,
     &   1.0053E-01, 1.0036E-01, 1.0019E-01, 1.0002E-01, 9.9858E-02/

         DATA ((Y3G  (I,J), I= 101, 200), J=1,1) /
     &   9.9692E-02, 9.9526E-02, 9.9360E-02, 9.9194E-02, 9.9029E-02,
     &   9.8864E-02, 9.8699E-02, 9.8534E-02, 9.8369E-02, 9.8205E-02,
     &   9.8041E-02, 9.7877E-02, 9.7714E-02, 9.7550E-02, 9.7387E-02,
     &   9.7224E-02, 9.7061E-02, 9.6899E-02, 9.6737E-02, 9.6575E-02,
     &   9.6413E-02, 9.6251E-02, 9.6090E-02, 9.5929E-02, 9.5768E-02,
     &   9.5607E-02, 9.5447E-02, 9.5286E-02, 9.5126E-02, 9.4967E-02,
     &   9.4807E-02, 9.4648E-02, 9.4488E-02, 9.4329E-02, 9.4171E-02,
     &   9.4012E-02, 9.3854E-02, 9.3695E-02, 9.3537E-02, 9.3380E-02,
     &   9.3222E-02, 9.3064E-02, 9.2907E-02, 9.2750E-02, 9.2593E-02,
     &   9.2436E-02, 9.2280E-02, 9.2123E-02, 9.1967E-02, 9.1810E-02,
     &   9.1654E-02, 9.1498E-02, 9.1343E-02, 9.1187E-02, 9.1031E-02,
     &   9.0876E-02, 9.0720E-02, 9.0565E-02, 9.0410E-02, 9.0254E-02,
     &   9.0099E-02, 8.9944E-02, 8.9789E-02, 8.9634E-02, 8.9479E-02,
     &   8.9324E-02, 8.9169E-02, 8.9015E-02, 8.8860E-02, 8.8705E-02,
     &   8.8550E-02, 8.8395E-02, 8.8240E-02, 8.8085E-02, 8.7930E-02,
     &   8.7775E-02, 8.7620E-02, 8.7465E-02, 8.7309E-02, 8.7154E-02,
     &   8.6998E-02, 8.6843E-02, 8.6687E-02, 8.6531E-02, 8.6375E-02,
     &   8.6219E-02, 8.6063E-02, 8.5907E-02, 8.5750E-02, 8.5594E-02,
     &   8.5437E-02, 8.5280E-02, 8.5122E-02, 8.4965E-02, 8.4807E-02,
     &   8.4649E-02, 8.4491E-02, 8.4333E-02, 8.4175E-02, 8.4016E-02/


         DATA ((Y3G  (I,J), I=   1, 100), J=2,2) /
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05/

         DATA ((Y3G  (I,J), I= 101, 200), J=2,2) /
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05,
     &   8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9065E-05, 8.9064E-05,
     &   8.9064E-05, 8.9064E-05, 8.9064E-05, 8.9064E-05, 8.9064E-05,
     &   8.9064E-05, 8.9064E-05, 8.9064E-05, 8.9064E-05, 8.9064E-05,
     &   8.9064E-05, 8.9064E-05, 8.9064E-05, 8.9064E-05, 8.9064E-05,
     &   8.9064E-05, 8.9064E-05, 8.9064E-05, 8.9064E-05, 8.9064E-05,
     &   8.9064E-05, 8.9064E-05, 8.9064E-05, 8.9064E-05, 8.9064E-05,
     &   8.9064E-05, 8.9064E-05, 8.9064E-05, 8.9064E-05, 8.9064E-05,
     &   8.9064E-05, 8.9064E-05, 8.9064E-05, 8.9064E-05, 8.9064E-05/


         DATA ((Y3G  (I,J), I=   1, 100), J=3,3) /
     &   1.6526E-04, 1.6526E-04, 1.6526E-04, 1.6526E-04, 1.6526E-04,
     &   1.6525E-04, 1.6525E-04, 1.6525E-04, 1.6525E-04, 1.6525E-04,
     &   1.6525E-04, 1.6525E-04, 1.6524E-04, 1.6524E-04, 1.6524E-04,
     &   1.6524E-04, 1.6524E-04, 1.6524E-04, 1.6524E-04, 1.6523E-04,
     &   1.6523E-04, 1.6523E-04, 1.6523E-04, 1.6523E-04, 1.6523E-04,
     &   1.6522E-04, 1.6522E-04, 1.6522E-04, 1.6522E-04, 1.6522E-04,
     &   1.6522E-04, 1.6522E-04, 1.6521E-04, 1.6521E-04, 1.6521E-04,
     &   1.6521E-04, 1.6521E-04, 1.6521E-04, 1.6521E-04, 1.6520E-04,
     &   1.6520E-04, 1.6520E-04, 1.6520E-04, 1.6520E-04, 1.6520E-04,
     &   1.6519E-04, 1.6519E-04, 1.6519E-04, 1.6519E-04, 1.6519E-04,
     &   1.6519E-04, 1.6519E-04, 1.6518E-04, 1.6518E-04, 1.6518E-04,
     &   1.6518E-04, 1.6518E-04, 1.6518E-04, 1.6517E-04, 1.6517E-04,
     &   1.6517E-04, 1.6517E-04, 1.6517E-04, 1.6517E-04, 1.6517E-04,
     &   1.6516E-04, 1.6516E-04, 1.6516E-04, 1.6516E-04, 1.6516E-04,
     &   1.6516E-04, 1.6515E-04, 1.6515E-04, 1.6515E-04, 1.6515E-04,
     &   1.6515E-04, 1.6515E-04, 1.6514E-04, 1.6514E-04, 1.6514E-04,
     &   1.6514E-04, 1.6514E-04, 1.6514E-04, 1.6514E-04, 1.6513E-04,
     &   1.6513E-04, 1.6513E-04, 1.6513E-04, 1.6513E-04, 1.6513E-04,
     &   1.6512E-04, 1.6512E-04, 1.6512E-04, 1.6512E-04, 1.6512E-04,
     &   1.6512E-04, 1.6512E-04, 1.6511E-04, 1.6511E-04, 1.6511E-04/

         DATA ((Y3G  (I,J), I= 101, 200), J=3,3) /
     &   1.6511E-04, 1.6511E-04, 1.6511E-04, 1.6510E-04, 1.6510E-04,
     &   1.6510E-04, 1.6510E-04, 1.6510E-04, 1.6510E-04, 1.6509E-04,
     &   1.6509E-04, 1.6509E-04, 1.6509E-04, 1.6509E-04, 1.6509E-04,
     &   1.6508E-04, 1.6508E-04, 1.6508E-04, 1.6508E-04, 1.6508E-04,
     &   1.6508E-04, 1.6508E-04, 1.6507E-04, 1.6507E-04, 1.6507E-04,
     &   1.6507E-04, 1.6507E-04, 1.6507E-04, 1.6506E-04, 1.6506E-04,
     &   1.6506E-04, 1.6506E-04, 1.6506E-04, 1.6506E-04, 1.6505E-04,
     &   1.6505E-04, 1.6505E-04, 1.6505E-04, 1.6505E-04, 1.6505E-04,
     &   1.6504E-04, 1.6504E-04, 1.6504E-04, 1.6504E-04, 1.6504E-04,
     &   1.6504E-04, 1.6503E-04, 1.6503E-04, 1.6503E-04, 1.6503E-04,
     &   1.6503E-04, 1.6503E-04, 1.6502E-04, 1.6502E-04, 1.6502E-04,
     &   1.6502E-04, 1.6502E-04, 1.6502E-04, 1.6501E-04, 1.6501E-04,
     &   1.6501E-04, 1.6501E-04, 1.6501E-04, 1.6501E-04, 1.6500E-04,
     &   1.6500E-04, 1.6500E-04, 1.6500E-04, 1.6500E-04, 1.6500E-04,
     &   1.6499E-04, 1.6499E-04, 1.6499E-04, 1.6499E-04, 1.6499E-04,
     &   1.6499E-04, 1.6498E-04, 1.6498E-04, 1.6498E-04, 1.6498E-04,
     &   1.6498E-04, 1.6497E-04, 1.6497E-04, 1.6497E-04, 1.6497E-04,
     &   1.6497E-04, 1.6497E-04, 1.6496E-04, 1.6496E-04, 1.6496E-04,
     &   1.6496E-04, 1.6496E-04, 1.6495E-04, 1.6495E-04, 1.6495E-04,
     &   1.6495E-04, 1.6495E-04, 1.6494E-04, 1.6494E-04, 1.6494E-04/

       save
       RETURN
       END
